"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var transforms_1 = require("../../mol-plugin/state/transforms");
var representation_1 = require("../../mol-plugin/state/transforms/representation");
var builder_1 = require("../../mol-script/language/builder");
var StateHelper;
(function (StateHelper) {
    function download(b, url, ref) {
        return b.apply(transforms_1.StateTransforms.Data.Download, { url: url, isBinary: false }, { ref: ref });
    }
    StateHelper.download = download;
    function getModel(b, format, modelIndex) {
        if (modelIndex === void 0) { modelIndex = 0; }
        var parsed = format === 'cif'
            ? b.apply(transforms_1.StateTransforms.Data.ParseCif).apply(transforms_1.StateTransforms.Model.TrajectoryFromMmCif)
            : b.apply(transforms_1.StateTransforms.Model.TrajectoryFromPDB);
        return parsed.apply(transforms_1.StateTransforms.Model.ModelFromTrajectory, { modelIndex: modelIndex });
    }
    StateHelper.getModel = getModel;
    function structure(b) {
        return b.apply(transforms_1.StateTransforms.Model.StructureFromModel, { tags: 'structure' });
    }
    StateHelper.structure = structure;
    ;
    function selectChain(b, auth_asym_id) {
        var query = builder_1.MolScriptBuilder.struct.generator.atomGroups({
            'chain-test': builder_1.MolScriptBuilder.core.rel.eq([builder_1.MolScriptBuilder.struct.atomProperty.macromolecular.auth_asym_id(), auth_asym_id])
        });
        return b.apply(transforms_1.StateTransforms.Model.StructureSelection, { query: query, label: "Chain " + auth_asym_id });
    }
    StateHelper.selectChain = selectChain;
    function select(b, query) {
        return b.apply(transforms_1.StateTransforms.Model.StructureSelection, { query: query });
    }
    StateHelper.select = select;
    function selectSurroundingsOfFirstResidue(b, comp_id, radius) {
        var query = builder_1.MolScriptBuilder.struct.modifier.includeSurroundings({
            0: builder_1.MolScriptBuilder.struct.filter.first([
                builder_1.MolScriptBuilder.struct.generator.atomGroups({
                    'residue-test': builder_1.MolScriptBuilder.core.rel.eq([builder_1.MolScriptBuilder.struct.atomProperty.macromolecular.label_comp_id(), comp_id]),
                    'group-by': builder_1.MolScriptBuilder.struct.atomProperty.macromolecular.residueKey()
                })
            ]),
            radius: radius
        });
        return b.apply(transforms_1.StateTransforms.Model.StructureSelection, { query: query, label: "Surr. " + comp_id + " (" + radius + " ang)" });
    }
    StateHelper.selectSurroundingsOfFirstResidue = selectSurroundingsOfFirstResidue;
    function identityTransform(b, m) {
        return b.apply(transforms_1.StateTransforms.Model.TransformStructureConformation, { axis: linear_algebra_1.Vec3.create(1, 0, 0), angle: 0, translation: linear_algebra_1.Vec3.zero() }, { tags: 'transform' });
    }
    StateHelper.identityTransform = identityTransform;
    function transform(b, matrix) {
        return b.apply(transforms_1.StateTransforms.Model.TransformStructureConformationByMatrix, { matrix: matrix }, { tags: 'transform' });
    }
    StateHelper.transform = transform;
    function assemble(b, id) {
        return b.apply(transforms_1.StateTransforms.Model.StructureAssemblyFromModel, { id: id || 'deposited' }, { tags: 'asm' });
    }
    StateHelper.assemble = assemble;
    function visual(ctx, visualRoot) {
        visualRoot.apply(transforms_1.StateTransforms.Model.StructureComplexElement, { type: 'atomic-sequence' })
            .apply(transforms_1.StateTransforms.Representation.StructureRepresentation3D, representation_1.StructureRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'cartoon'), { tags: 'seq-visual' });
        visualRoot.apply(transforms_1.StateTransforms.Model.StructureComplexElement, { type: 'atomic-het' })
            .apply(transforms_1.StateTransforms.Representation.StructureRepresentation3D, representation_1.StructureRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'ball-and-stick'), { tags: 'het-visual' });
        // visualRoot.apply(StateTransforms.Model.StructureComplexElement, { type: 'water' })
        //     .apply(StateTransforms.Representation.StructureRepresentation3D,
        //         StructureRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'ball-and-stick', { alpha: 0.51 }), { tags: 'water-visual' });
        return visualRoot;
    }
    StateHelper.visual = visual;
    function ballsAndSticks(ctx, visualRoot, query, coloring) {
        visualRoot
            .apply(transforms_1.StateTransforms.Model.StructureSelection, { query: query })
            .apply(transforms_1.StateTransforms.Representation.StructureRepresentation3D, representation_1.StructureRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'ball-and-stick', void 0, coloring), { tags: 'het-visual' });
        return visualRoot;
    }
    StateHelper.ballsAndSticks = ballsAndSticks;
})(StateHelper = exports.StateHelper || (exports.StateHelper = {}));
