"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var argparse = require("argparse");
var util = require("util");
var path = require("path");
var fs = require("fs");
var zlib = require("zlib");
var node_fetch_1 = require("node-fetch");
require('util.promisify').shim();
var readFile = util.promisify(fs.readFile);
var writeFile = util.promisify(fs.writeFile);
var mol_task_1 = require("../../mol-task");
var db_1 = require("../../mol-data/db");
var cif_1 = require("../../mol-io/reader/cif");
var cif_2 = require("../../mol-io/writer/cif");
var ccd_1 = require("../../mol-io/reader/cif/schema/ccd");
var set_1 = require("../../mol-util/set");
var map_1 = require("../../mol-util/map");
var mmcif_extras_1 = require("../../mol-io/reader/cif/schema/mmcif-extras");
function ensureAvailable(path, url) {
    return __awaiter(this, void 0, void 0, function () {
        var data, _a, _b, _c, _d, _e, _f;
        return __generator(this, function (_g) {
            switch (_g.label) {
                case 0:
                    if (!(FORCE_DOWNLOAD || !fs.existsSync(path))) return [3 /*break*/, 8];
                    console.log("downloading " + url + "...");
                    return [4 /*yield*/, node_fetch_1.default(url)];
                case 1:
                    data = _g.sent();
                    if (!fs.existsSync(DATA_DIR)) {
                        fs.mkdirSync(DATA_DIR);
                    }
                    if (!url.endsWith('.gz')) return [3 /*break*/, 4];
                    _a = writeFile;
                    _b = [path];
                    _d = (_c = zlib).gunzipSync;
                    return [4 /*yield*/, data.buffer()];
                case 2: return [4 /*yield*/, _a.apply(void 0, _b.concat([_d.apply(_c, [_g.sent()])]))];
                case 3:
                    _g.sent();
                    return [3 /*break*/, 7];
                case 4:
                    _e = writeFile;
                    _f = [path];
                    return [4 /*yield*/, data.text()];
                case 5: return [4 /*yield*/, _e.apply(void 0, _f.concat([_g.sent()]))];
                case 6:
                    _g.sent();
                    _g.label = 7;
                case 7:
                    console.log("done downloading " + url);
                    _g.label = 8;
                case 8: return [2 /*return*/];
            }
        });
    });
}
exports.ensureAvailable = ensureAvailable;
function ensureDataAvailable() {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, ensureAvailable(CCD_PATH, CCD_URL)];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, ensureAvailable(PVCD_PATH, PVCD_URL)];
                case 2:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
}
exports.ensureDataAvailable = ensureDataAvailable;
function readFileAsCollection(path, schema) {
    return __awaiter(this, void 0, void 0, function () {
        var parsed, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = parseCif;
                    return [4 /*yield*/, readFile(path, 'utf8')];
                case 1: return [4 /*yield*/, _a.apply(void 0, [_b.sent()])];
                case 2:
                    parsed = _b.sent();
                    return [2 /*return*/, cif_1.CIF.toDatabaseCollection(schema, parsed.result)];
            }
        });
    });
}
exports.readFileAsCollection = readFileAsCollection;
function readCCD() {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, readFileAsCollection(CCD_PATH, ccd_1.CCD_Schema)];
        });
    });
}
exports.readCCD = readCCD;
function readPVCD() {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, readFileAsCollection(PVCD_PATH, ccd_1.CCD_Schema)];
        });
    });
}
exports.readPVCD = readPVCD;
function parseCif(data) {
    return __awaiter(this, void 0, void 0, function () {
        var comp, parsed;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    comp = cif_1.CIF.parse(data);
                    console.time('parse cif');
                    return [4 /*yield*/, comp.run(function (p) { return console.log(mol_task_1.Progress.format(p)); }, 250)];
                case 1:
                    parsed = _a.sent();
                    console.timeEnd('parse cif');
                    if (parsed.isError)
                        throw parsed;
                    return [2 /*return*/, parsed];
            }
        });
    });
}
function getEncodedCif(name, database, binary) {
    if (binary === void 0) { binary = false; }
    var encoder = cif_2.CifWriter.createEncoder({ binary: binary, encoderName: 'mol*' });
    cif_2.CifWriter.Encoder.writeDatabase(encoder, name, database);
    return encoder.getData();
}
exports.getEncodedCif = getEncodedCif;
function ccbKey(compId, atomId1, atomId2) {
    return atomId1 < atomId2 ? compId + ":" + atomId1 + "-" + atomId2 : compId + ":" + atomId2 + "-" + atomId1;
}
function addChemCompBondToSet(set, ccb) {
    for (var i = 0, il = ccb._rowCount; i < il; ++i) {
        set.add(ccbKey(ccb.comp_id.value(i), ccb.atom_id_1.value(i), ccb.atom_id_2.value(i)));
    }
    return set;
}
function addChemCompAtomToSet(set, cca) {
    for (var i = 0, il = cca._rowCount; i < il; ++i) {
        set.add(cca.atom_id.value(i));
    }
    return set;
}
function checkAddingBondsFromPVCD(pvcd) {
    var ccbSetByParent = map_1.DefaultMap(function () { return new Set(); });
    for (var k in pvcd) {
        var _a = pvcd[k], chem_comp = _a.chem_comp, chem_comp_bond = _a.chem_comp_bond;
        if (chem_comp_bond._rowCount) {
            var parentIds = chem_comp.mon_nstd_parent_comp_id.value(0);
            if (parentIds.length === 0) {
                var set = ccbSetByParent.getDefault(chem_comp.id.value(0));
                addChemCompBondToSet(set, chem_comp_bond);
            }
            else {
                for (var i = 0, il = parentIds.length; i < il; ++i) {
                    var parentId = parentIds[i];
                    var set = ccbSetByParent.getDefault(parentId);
                    addChemCompBondToSet(set, chem_comp_bond);
                }
            }
        }
    }
    var _loop_1 = function (k) {
        var _a = pvcd[k], chem_comp = _a.chem_comp, chem_comp_atom = _a.chem_comp_atom, chem_comp_bond = _a.chem_comp_bond;
        if (chem_comp_bond._rowCount) {
            var parentIds = chem_comp.mon_nstd_parent_comp_id.value(0);
            if (parentIds.length > 0) {
                var _loop_2 = function (i, il) {
                    var entryBonds = addChemCompBondToSet(new Set(), chem_comp_bond);
                    var entryAtoms = addChemCompAtomToSet(new Set(), chem_comp_atom);
                    var extraBonds = set_1.SetUtils.difference(ccbSetByParent.get(parentIds[i]), entryBonds);
                    extraBonds.forEach(function (bk) {
                        var _a = bk.split('|'), a1 = _a[0], a2 = _a[1];
                        if (entryAtoms.has(a1) && entryAtoms.has(a2)) {
                            console.error("Adding all PVCD bonds would wrongly add bond " + bk + " for " + k);
                        }
                    });
                };
                for (var i = 0, il = parentIds.length; i < il; ++i) {
                    _loop_2(i, il);
                }
            }
        }
    };
    for (var k in pvcd) {
        _loop_1(k);
    }
}
function createBonds() {
    return __awaiter(this, void 0, void 0, function () {
        function addBonds(compId, ccb, protonationVariant) {
            for (var i = 0, il = ccb._rowCount; i < il; ++i) {
                var atomId1 = ccb.atom_id_1.value(i);
                var atomId2 = ccb.atom_id_2.value(i);
                var k = ccbKey(compId, atomId1, atomId2);
                if (!ccbSet.has(k)) {
                    atom_id_1.push(atomId1);
                    atom_id_2.push(atomId2);
                    comp_id.push(compId);
                    value_order.push(ccb.value_order.value(i));
                    pdbx_aromatic_flag.push(ccb.pdbx_aromatic_flag.value(i));
                    pdbx_stereo_config.push(ccb.pdbx_stereo_config.value(i));
                    molstar_protonation_variant.push(protonationVariant ? 'Y' : 'N');
                    ccbSet.add(k);
                }
            }
        }
        var ccd, pvcd, ccbSet, comp_id, atom_id_1, atom_id_2, value_order, pdbx_aromatic_flag, pdbx_stereo_config, molstar_protonation_variant, k, _a, chem_comp, chem_comp_bond, parentIds, i, il, k, _b, chem_comp, chem_comp_bond, bondTable, bondDatabase;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0: return [4 /*yield*/, ensureDataAvailable()];
                case 1:
                    _c.sent();
                    return [4 /*yield*/, readCCD()];
                case 2:
                    ccd = _c.sent();
                    return [4 /*yield*/, readPVCD()];
                case 3:
                    pvcd = _c.sent();
                    ccbSet = new Set();
                    comp_id = [];
                    atom_id_1 = [];
                    atom_id_2 = [];
                    value_order = [];
                    pdbx_aromatic_flag = [];
                    pdbx_stereo_config = [];
                    molstar_protonation_variant = [];
                    // check adding bonds from PVCD
                    checkAddingBondsFromPVCD(pvcd);
                    // add bonds from PVCD
                    for (k in pvcd) {
                        _a = pvcd[k], chem_comp = _a.chem_comp, chem_comp_bond = _a.chem_comp_bond;
                        if (chem_comp_bond._rowCount) {
                            parentIds = chem_comp.mon_nstd_parent_comp_id.value(0);
                            if (parentIds.length === 0) {
                                addBonds(chem_comp.id.value(0), chem_comp_bond, false);
                            }
                            else {
                                for (i = 0, il = parentIds.length; i < il; ++i) {
                                    addBonds(parentIds[i], chem_comp_bond, true);
                                }
                            }
                        }
                    }
                    // add bonds from CCD
                    for (k in ccd) {
                        _b = ccd[k], chem_comp = _b.chem_comp, chem_comp_bond = _b.chem_comp_bond;
                        if (chem_comp_bond._rowCount) {
                            addBonds(chem_comp.id.value(0), chem_comp_bond, false);
                        }
                    }
                    bondTable = db_1.Table.ofArrays(mmcif_extras_1.mmCIF_chemCompBond_schema, {
                        comp_id: comp_id, atom_id_1: atom_id_1, atom_id_2: atom_id_2, value_order: value_order,
                        pdbx_aromatic_flag: pdbx_aromatic_flag, pdbx_stereo_config: pdbx_stereo_config, molstar_protonation_variant: molstar_protonation_variant
                    });
                    bondDatabase = db_1.Database.ofTables(TABLE_NAME, { chem_comp_bond: mmcif_extras_1.mmCIF_chemCompBond_schema }, { chem_comp_bond: bondTable });
                    return [2 /*return*/, bondDatabase];
            }
        });
    });
}
function run(out, binary) {
    if (binary === void 0) { binary = false; }
    return __awaiter(this, void 0, void 0, function () {
        var bonds, cif;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, createBonds()];
                case 1:
                    bonds = _a.sent();
                    cif = getEncodedCif(TABLE_NAME, bonds, binary);
                    if (!fs.existsSync(path.dirname(out))) {
                        fs.mkdirSync(path.dirname(out));
                    }
                    writeFile(out, cif);
                    return [2 /*return*/];
            }
        });
    });
}
var TABLE_NAME = 'CHEM_COMP_BONDS';
var DATA_DIR = path.join(__dirname, '..', '..', '..', 'build/data');
var CCD_PATH = path.join(DATA_DIR, 'components.cif');
var PVCD_PATH = path.join(DATA_DIR, 'aa-variants-v1.cif');
var CCD_URL = 'http://ftp.wwpdb.org/pub/pdb/data/monomers/components.cif';
var PVCD_URL = 'http://ftp.wwpdb.org/pub/pdb/data/monomers/aa-variants-v1.cif';
var parser = new argparse.ArgumentParser({
    addHelp: true,
    description: 'Create a cif file with one big table of all chem_comp_bond entries from the CCD and PVCD.'
});
parser.addArgument('out', {
    help: 'Generated file output path.'
});
parser.addArgument(['--forceDownload', '-f'], {
    action: 'storeTrue',
    help: 'Force download of CCD and PVCD.'
});
parser.addArgument(['--binary', '-b'], {
    action: 'storeTrue',
    help: 'Output as BinaryCIF.'
});
var args = parser.parseArgs();
var FORCE_DOWNLOAD = args.forceDownload;
run(args.out, args.binary);
