"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var argparse = require("argparse");
require('util.promisify').shim();
var structure_1 = require("../../mol-model/structure");
// import { Run, Progress } from '../../mol-task'
var int_1 = require("../../mol-data/int");
var helpers_1 = require("./helpers");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var mmcif_1 = require("../../mol-model-formats/structure/mmcif");
function downloadFromPdb(pdb) {
    return __awaiter(this, void 0, void 0, function () {
        var parsed;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, helpers_1.downloadCif("http://www.ebi.ac.uk/pdbe/static/entry/" + pdb + "_updated.cif", false)];
                case 1:
                    parsed = _a.sent();
                    return [2 /*return*/, parsed.blocks[0]];
            }
        });
    });
}
function readCifFile(path) {
    return __awaiter(this, void 0, void 0, function () {
        var parsed;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, helpers_1.openCif(path)];
                case 1:
                    parsed = _a.sent();
                    return [2 /*return*/, parsed.blocks[0]];
            }
        });
    });
}
exports.readCifFile = readCifFile;
function atomLabel(model, aI) {
    var _a = model.atomicHierarchy, atoms = _a.atoms, residues = _a.residues, chains = _a.chains, residueAtomSegments = _a.residueAtomSegments, chainAtomSegments = _a.chainAtomSegments;
    var label_atom_id = atoms.label_atom_id;
    var label_comp_id = residues.label_comp_id, label_seq_id = residues.label_seq_id;
    var label_asym_id = chains.label_asym_id;
    var rI = residueAtomSegments.index[aI];
    var cI = chainAtomSegments.index[aI];
    return label_asym_id.value(cI) + " " + label_comp_id.value(rI) + " " + label_seq_id.value(rI) + " " + label_atom_id.value(aI);
}
exports.atomLabel = atomLabel;
function residueLabel(model, rI) {
    var _a = model.atomicHierarchy, residues = _a.residues, chains = _a.chains, residueAtomSegments = _a.residueAtomSegments, chainAtomSegments = _a.chainAtomSegments;
    var label_comp_id = residues.label_comp_id, label_seq_id = residues.label_seq_id;
    var label_asym_id = chains.label_asym_id;
    var cI = chainAtomSegments.index[residueAtomSegments.offsets[rI]];
    return label_asym_id.value(cI) + " " + label_comp_id.value(rI) + " " + label_seq_id.value(rI);
}
exports.residueLabel = residueLabel;
function printSecStructure(model) {
    console.log('\nSecondary Structure\n=============');
    var residues = model.atomicHierarchy.residues;
    var _a = model.properties.secondaryStructure, key = _a.key, elements = _a.elements;
    var count = residues._rowCount;
    var rI = 0;
    while (rI < count) {
        var start = rI;
        while (rI < count && key[start] === key[rI])
            rI++;
        rI--;
        var e = elements[key[start]];
        if (e.kind !== 'none')
            console.log(e.kind + ": " + residueLabel(model, start) + " - " + residueLabel(model, rI));
        rI++;
    }
}
exports.printSecStructure = printSecStructure;
function printLinks(structure, showIntra, showInter) {
    if (showIntra) {
        console.log('\nIntra Unit Links\n=============');
        for (var _a = 0, _b = structure.units; _a < _b.length; _a++) {
            var unit = _b[_a];
            if (!structure_1.Unit.isAtomic(unit))
                continue;
            var elements = unit.elements;
            var _c = unit.links, a = _c.a, b = _c.b, edgeCount = _c.edgeCount;
            var model = unit.model;
            if (!edgeCount)
                continue;
            for (var bI = 0, _bI = edgeCount * 2; bI < _bI; bI++) {
                var x = a[bI], y = b[bI];
                if (x >= y)
                    continue;
                console.log(atomLabel(model, elements[x]) + " -- " + atomLabel(model, elements[y]));
            }
        }
    }
    if (showInter) {
        console.log('\nInter Unit Links\n=============');
        var links = structure.links;
        for (var _d = 0, _e = structure.units; _d < _e.length; _d++) {
            var unit = _e[_d];
            if (!structure_1.Unit.isAtomic(unit))
                continue;
            for (var _f = 0, _g = links.getLinkedUnits(unit); _f < _g.length; _f++) {
                var pairLinks = _g[_f];
                if (!pairLinks.areUnitsOrdered || pairLinks.bondCount === 0)
                    continue;
                var unitA = pairLinks.unitA, unitB = pairLinks.unitB;
                console.log(pairLinks.unitA.id + " - " + pairLinks.unitB.id + ": " + pairLinks.bondCount + " bond(s)");
                for (var _h = 0, _k = pairLinks.linkedElementIndices; _h < _k.length; _h++) {
                    var aI = _k[_h];
                    for (var _l = 0, _m = pairLinks.getBonds(aI); _l < _m.length; _l++) {
                        var link = _m[_l];
                        console.log(atomLabel(unitA.model, unitA.elements[aI]) + " -- " + atomLabel(unitB.model, unitB.elements[link.indexB]));
                    }
                }
            }
        }
    }
}
exports.printLinks = printLinks;
function printSequence(model) {
    console.log('\nSequence\n=============');
    var byEntityKey = model.sequence.byEntityKey;
    for (var _a = 0, _b = Object.keys(byEntityKey); _a < _b.length; _a++) {
        var key = _b[_a];
        var seq = byEntityKey[+key];
        console.log(seq.entityId + " (" + seq.sequence.kind + " " + seq.num.value(0) + " (offset " + seq.sequence.offset + "), " + seq.num.value(seq.num.rowCount - 1) + ") (" + seq.compId.value(0) + ", " + seq.compId.value(seq.compId.rowCount - 1) + ")");
        console.log("" + seq.sequence.sequence);
    }
    console.log();
}
exports.printSequence = printSequence;
function printModRes(model) {
    console.log('\nModified Residues\n=============');
    var map = model.properties.modifiedResidues.parentId;
    var _a = model.atomicHierarchy.residues, label_comp_id = _a.label_comp_id, _rowCount = _a._rowCount;
    for (var i = 0; i < _rowCount; i++) {
        var comp_id = label_comp_id.value(i);
        if (!map.has(comp_id))
            continue;
        console.log("[" + i + "] " + map.get(comp_id) + " -> " + comp_id);
    }
    console.log();
}
exports.printModRes = printModRes;
function printRings(structure) {
    console.log('\nRings\n=============');
    for (var _a = 0, _b = structure.units; _a < _b.length; _a++) {
        var unit = _b[_a];
        if (!structure_1.Unit.isAtomic(unit))
            continue;
        var _c = unit.rings, all = _c.all, byFingerprint = _c.byFingerprint;
        var fps = [];
        for (var i = 0, _i = Math.min(5, all.length); i < _i; i++) {
            fps[fps.length] = structure_1.UnitRing.fingerprint(unit, all[i]);
        }
        if (all.length > 5)
            fps.push('...');
        console.log("Unit " + unit.id + ", " + all.length + " ring(s), " + byFingerprint.size + " different fingerprint(s).\n  " + fps.join(', '));
    }
    console.log();
}
exports.printRings = printRings;
function printUnits(structure) {
    console.log('\nUnits\n=============');
    var l = structure_1.StructureElement.create();
    for (var _a = 0, _b = structure.units; _a < _b.length; _a++) {
        var unit = _b[_a];
        l.unit = unit;
        var elements = unit.elements;
        var size = int_1.OrderedSet.size(elements);
        if (structure_1.Unit.isAtomic(l.unit)) {
            console.log("Atomic unit " + unit.id + " " + unit.conformation.operator.name + ": " + size + " elements");
        }
        else if (structure_1.Unit.isCoarse(l.unit)) {
            console.log("Coarse unit " + unit.id + " " + unit.conformation.operator.name + " (" + (structure_1.Unit.isSpheres(l.unit) ? 'spheres' : 'gaussians') + "): " + size + " elements.");
            var props = structure_1.StructureProperties.coarse;
            var seq = l.unit.model.sequence;
            for (var j = 0, _j = Math.min(size, 3); j < _j; j++) {
                l.element = int_1.OrderedSet.getAt(elements, j);
                var residues = [];
                var start = props.seq_id_begin(l), end = props.seq_id_end(l);
                var compId = seq.byEntityKey[props.entityKey(l)].compId.value;
                for (var e = start; e <= end; e++)
                    residues.push(compId(e));
                console.log(props.asym_id(l) + ":" + start + "-" + end + " (" + residues.join('-') + ") " + props.asym_id(l) + " [" + props.x(l).toFixed(2) + ", " + props.y(l).toFixed(2) + ", " + props.z(l).toFixed(2) + "]");
            }
            if (size > 3)
                console.log("...");
        }
    }
}
exports.printUnits = printUnits;
function printSymmetryInfo(model) {
    console.log('\nSymmetry Info\n=============');
    var symmetry = model.symmetry;
    var _a = symmetry.spacegroup.cell, size = _a.size, anglesInRadians = _a.anglesInRadians;
    console.log("Spacegroup: " + symmetry.spacegroup.name + " size: " + linear_algebra_1.Vec3.toString(size) + " angles: " + linear_algebra_1.Vec3.toString(anglesInRadians));
    console.log("Assembly names: " + symmetry.assemblies.map(function (a) { return a.id; }).join(', '));
    // NCS example: 1auy
    console.log("NCS operators: " + (symmetry.ncsOperators && symmetry.ncsOperators.map(function (a) { return a.name; }).join(', ')));
}
exports.printSymmetryInfo = printSymmetryInfo;
function printModelStats(models) {
    console.log('\nModels\n=============');
    for (var _a = 0, models_1 = models; _a < models_1.length; _a++) {
        var m = models_1[_a];
        if (m.coarseHierarchy.isDefined) {
            console.log(m.label + " " + m.modelNum + ": " + m.atomicHierarchy.atoms._rowCount + " atom(s), " + m.coarseHierarchy.spheres.count + " sphere(s), " + m.coarseHierarchy.gaussians.count + " gaussian(s)");
        }
        else {
            console.log(m.label + " " + m.modelNum + ": " + m.atomicHierarchy.atoms._rowCount + " atom(s)");
        }
    }
    console.log();
}
exports.printModelStats = printModelStats;
function getModelsAndStructure(frame) {
    return __awaiter(this, void 0, void 0, function () {
        var models, structure;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, mmcif_1.trajectoryFromMmCIF(frame).run()];
                case 1:
                    models = _a.sent();
                    structure = structure_1.Structure.ofModel(models[0]);
                    return [2 /*return*/, { models: models, structure: structure }];
            }
        });
    });
}
exports.getModelsAndStructure = getModelsAndStructure;
function run(frame, args) {
    return __awaiter(this, void 0, void 0, function () {
        var _a, models, structure;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, getModelsAndStructure(frame)];
                case 1:
                    _a = _b.sent(), models = _a.models, structure = _a.structure;
                    if (args.models)
                        printModelStats(models);
                    if (args.seq)
                        printSequence(models[0]);
                    if (args.units)
                        printUnits(structure);
                    if (args.sym)
                        printSymmetryInfo(models[0]);
                    if (args.rings)
                        printRings(structure);
                    if (args.intraLinks)
                        printLinks(structure, true, false);
                    if (args.interLinks)
                        printLinks(structure, false, true);
                    if (args.mod)
                        printModRes(models[0]);
                    if (args.sec)
                        printSecStructure(models[0]);
                    return [2 /*return*/];
            }
        });
    });
}
function runDL(pdb, args) {
    return __awaiter(this, void 0, void 0, function () {
        var mmcif;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, downloadFromPdb(pdb)];
                case 1:
                    mmcif = _a.sent();
                    run(mmcif, args);
                    return [2 /*return*/];
            }
        });
    });
}
function runFile(filename, args) {
    return __awaiter(this, void 0, void 0, function () {
        var mmcif;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, readCifFile(filename)];
                case 1:
                    mmcif = _a.sent();
                    run(mmcif, args);
                    return [2 /*return*/];
            }
        });
    });
}
var parser = new argparse.ArgumentParser({
    addHelp: true,
    description: 'Print info about a structure, mainly to test and showcase the mol-model module'
});
parser.addArgument(['--download', '-d'], { help: 'Pdb entry id' });
parser.addArgument(['--file', '-f'], { help: 'filename' });
parser.addArgument(['--models'], { help: 'print models info', action: 'storeTrue' });
parser.addArgument(['--seq'], { help: 'print sequence', action: 'storeTrue' });
parser.addArgument(['--units'], { help: 'print units', action: 'storeTrue' });
parser.addArgument(['--sym'], { help: 'print symmetry', action: 'storeTrue' });
parser.addArgument(['--rings'], { help: 'print rings', action: 'storeTrue' });
parser.addArgument(['--intraLinks'], { help: 'print intra unit links', action: 'storeTrue' });
parser.addArgument(['--interLinks'], { help: 'print inter unit links', action: 'storeTrue' });
parser.addArgument(['--mod'], { help: 'print modified residues', action: 'storeTrue' });
parser.addArgument(['--sec'], { help: 'print secoundary structure', action: 'storeTrue' });
var args = parser.parseArgs();
if (args.download)
    runDL(args.download, args);
else if (args.file)
    runFile(args.file, args);
