"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var argparse = require("argparse");
var util = require("util");
var volume_1 = require("../../mol-model/volume");
var helpers_1 = require("./helpers");
var cif_1 = require("../../mol-io/reader/cif");
var db_1 = require("../../mol-data/db");
var mol_util_1 = require("../../mol-util");
var mol_task_1 = require("../../mol-task");
var isosurface_1 = require("../../mol-repr/volume/isosurface");
var theme_1 = require("../../mol-theme/theme");
var density_server_1 = require("../../mol-model-formats/volume/density-server");
require('util.promisify').shim();
var writeFileAsync = util.promisify(fs.writeFile);
function getVolume(url) {
    return __awaiter(this, void 0, void 0, function () {
        var cif, data, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, helpers_1.downloadCif(url, true)];
                case 1:
                    cif = _b.sent();
                    data = cif_1.CIF.schema.densityServer(cif.blocks[1]);
                    _a = { source: data };
                    return [4 /*yield*/, density_server_1.volumeFromDensityServerData(data).run()];
                case 2: return [2 /*return*/, (_a.volume = _b.sent(), _a)];
            }
        });
    });
}
function print(data) {
    var volume_data_3d_info = data.source.volume_data_3d_info;
    var row = db_1.Table.getRow(volume_data_3d_info, 0);
    console.log(row);
    console.log(data.volume.cell);
    console.log(data.volume.dataStats);
    console.log(data.volume.fractionalBox);
}
function doMesh(data, filename) {
    return __awaiter(this, void 0, void 0, function () {
        var mesh, vertexCount, triangleCount, vs, ts, obj, i, i;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, mol_task_1.Task.create('', function (runtime) { return isosurface_1.createVolumeIsosurfaceMesh({ runtime: runtime }, data.volume, theme_1.createEmptyTheme(), { isoValue: volume_1.VolumeIsoValue.absolute(1.5) }); }).run()];
                case 1:
                    mesh = _a.sent();
                    console.log({ vc: mesh.vertexCount, tc: mesh.triangleCount });
                    vertexCount = mesh.vertexCount, triangleCount = mesh.triangleCount;
                    vs = mesh.vertexBuffer.ref.value;
                    ts = mesh.indexBuffer.ref.value;
                    obj = mol_util_1.StringBuilder.create();
                    for (i = 0; i < vertexCount; i++) {
                        mol_util_1.StringBuilder.write(obj, 'v ');
                        mol_util_1.StringBuilder.writeFloat(obj, vs[3 * i + 0], 100);
                        mol_util_1.StringBuilder.whitespace1(obj);
                        mol_util_1.StringBuilder.writeFloat(obj, vs[3 * i + 1], 100);
                        mol_util_1.StringBuilder.whitespace1(obj);
                        mol_util_1.StringBuilder.writeFloat(obj, vs[3 * i + 2], 100);
                        mol_util_1.StringBuilder.newline(obj);
                    }
                    for (i = 0; i < triangleCount; i++) {
                        mol_util_1.StringBuilder.write(obj, 'f ');
                        mol_util_1.StringBuilder.writeIntegerAndSpace(obj, ts[3 * i + 0] + 1);
                        mol_util_1.StringBuilder.writeIntegerAndSpace(obj, ts[3 * i + 1] + 1);
                        mol_util_1.StringBuilder.writeInteger(obj, ts[3 * i + 2] + 1);
                        mol_util_1.StringBuilder.newline(obj);
                    }
                    return [4 /*yield*/, writeFileAsync(filename, mol_util_1.StringBuilder.getString(obj))];
                case 2:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
}
function run(url, meshFilename) {
    return __awaiter(this, void 0, void 0, function () {
        var volume;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, getVolume(url)];
                case 1:
                    volume = _a.sent();
                    print(volume);
                    return [4 /*yield*/, doMesh(volume, meshFilename)];
                case 2:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
}
var parser = new argparse.ArgumentParser({
    addHelp: true,
    description: 'Info about VolumeData from mol-model module'
});
parser.addArgument(['--emdb', '-e'], {
    help: 'EMDB id, for example 8116',
});
parser.addArgument(['--mesh'], {
    help: 'Mesh filename',
    required: true
});
var args = parser.parseArgs();
run("https://ds.litemol.org/em/emd-" + args.emdb + "/cell?detail=4", args.mesh);
