"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var custom_element_property_1 = require("../../mol-model-props/common/custom-element-property");
var color_1 = require("../../mol-util/color");
var EvolutionaryConservationPalette = [
    [255, 255, 129],
    [160, 37, 96],
    [240, 125, 171],
    [250, 201, 222],
    [252, 237, 244],
    [255, 255, 255],
    [234, 255, 255],
    [215, 255, 255],
    [140, 255, 255],
    [16, 200, 209] // 1
].reverse().map(function (_a) {
    var r = _a[0], g = _a[1], b = _a[2];
    return color_1.Color.fromRgb(r, g, b);
});
var EvolutionaryConservationDefaultColor = color_1.Color(0x999999);
exports.EvolutionaryConservation = custom_element_property_1.CustomElementProperty.create({
    isStatic: true,
    name: 'proteopedia-wrapper-evolutionary-conservation',
    display: 'Evolutionary Conservation',
    getData: function (model) {
        return __awaiter(this, void 0, void 0, function () {
            var id, req, json, annotations, conservationMap, _i, annotations_1, e, _a, _b, r, map, residueCount, residueOffsets, chainIndex, rI, cI, key, ann, aI;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        id = model.label.toLowerCase();
                        return [4 /*yield*/, fetch("https://proteopedia.org/cgi-bin/cnsrf?" + id)];
                    case 1:
                        req = _c.sent();
                        return [4 /*yield*/, req.json()];
                    case 2:
                        json = _c.sent();
                        annotations = (json && json.residueAnnotations) || [];
                        conservationMap = new Map();
                        for (_i = 0, annotations_1 = annotations; _i < annotations_1.length; _i++) {
                            e = annotations_1[_i];
                            for (_a = 0, _b = e.ids; _a < _b.length; _a++) {
                                r = _b[_a];
                                conservationMap.set(r, e.annotation);
                            }
                        }
                        map = new Map();
                        residueCount = model.atomicHierarchy.residues._rowCount;
                        residueOffsets = model.atomicHierarchy.residueAtomSegments.offsets;
                        chainIndex = model.atomicHierarchy.chainAtomSegments.index;
                        for (rI = 0; rI < residueCount; rI++) {
                            cI = chainIndex[residueOffsets[rI]];
                            key = model.atomicHierarchy.chains.auth_asym_id.value(cI) + " " + model.atomicHierarchy.residues.auth_seq_id.value(rI);
                            if (!conservationMap.has(key))
                                continue;
                            ann = conservationMap.get(key);
                            for (aI = residueOffsets[rI]; aI < residueOffsets[rI + 1]; aI++) {
                                map.set(aI, ann);
                            }
                        }
                        return [2 /*return*/, map];
                }
            });
        });
    },
    coloring: {
        getColor: function (e) {
            if (e < 1 || e > 10)
                return EvolutionaryConservationDefaultColor;
            return EvolutionaryConservationPalette[e - 1];
        },
        defaultColor: EvolutionaryConservationDefaultColor
    },
    format: function (e) {
        if (e === 10)
            return "Evolutionary Conservation: InsufficientData";
        return e ? "Evolutionary Conservation: " + e : void 0;
    }
});
