"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("../../mol-model/structure/model/types");
var ModelInfo;
(function (ModelInfo) {
    function getPreferredAssembly(ctx, model) {
        return __awaiter(this, void 0, void 0, function () {
            var id, src, json, data, assemblies, _i, assemblies_1, asm, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (model.label.length <= 3)
                            return [2 /*return*/, void 0];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        id = model.label.toLowerCase();
                        return [4 /*yield*/, ctx.runTask(ctx.fetch({ url: "https://www.ebi.ac.uk/pdbe/api/pdb/entry/summary/" + id }))];
                    case 2:
                        src = _a.sent();
                        json = JSON.parse(src);
                        data = json && json[id];
                        assemblies = data[0] && data[0].assemblies;
                        if (!assemblies || !assemblies.length)
                            return [2 /*return*/, void 0];
                        for (_i = 0, assemblies_1 = assemblies; _i < assemblies_1.length; _i++) {
                            asm = assemblies_1[_i];
                            if (asm.preferred) {
                                return [2 /*return*/, asm.assembly_id];
                            }
                        }
                        return [2 /*return*/, void 0];
                    case 3:
                        e_1 = _a.sent();
                        console.warn('getPreferredAssembly', e_1);
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    }
    function get(ctx, model, checkPreferred) {
        return __awaiter(this, void 0, void 0, function () {
            var residueCount, residueOffsets, chainIndex, pref, hetResidues, hetMap, rI, comp_id, mod_parent, cI, eI, lig, preferredAssemblyId;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        residueCount = model.atomicHierarchy.residues._rowCount;
                        residueOffsets = model.atomicHierarchy.residueAtomSegments.offsets;
                        chainIndex = model.atomicHierarchy.chainAtomSegments.index;
                        pref = checkPreferred
                            ? getPreferredAssembly(ctx, model)
                            : void 0;
                        hetResidues = [];
                        hetMap = new Map();
                        for (rI = 0; rI < residueCount; rI++) {
                            comp_id = model.atomicHierarchy.residues.label_comp_id.value(rI);
                            if (types_1.AminoAcidNames.has(comp_id))
                                continue;
                            mod_parent = model.properties.modifiedResidues.parentId.get(comp_id);
                            if (mod_parent && types_1.AminoAcidNames.has(mod_parent))
                                continue;
                            cI = chainIndex[residueOffsets[rI]];
                            eI = model.atomicHierarchy.index.getEntityFromChain(cI);
                            if (model.entities.data.type.value(eI) === 'water')
                                continue;
                            lig = hetMap.get(comp_id);
                            if (!lig) {
                                lig = { name: comp_id, indices: [] };
                                hetResidues.push(lig);
                                hetMap.set(comp_id, lig);
                            }
                            lig.indices.push(rI);
                        }
                        return [4 /*yield*/, pref];
                    case 1:
                        preferredAssemblyId = _a.sent();
                        return [2 /*return*/, {
                                hetResidues: hetResidues,
                                assemblies: model.symmetry.assemblies.map(function (a) { return ({ id: a.id, details: a.details, isPreferred: a.id === preferredAssemblyId }); }),
                                preferredAssemblyId: preferredAssemblyId
                            }];
                }
            });
        });
    }
    ModelInfo.get = get;
})(ModelInfo = exports.ModelInfo || (exports.ModelInfo = {}));
var StateElements;
(function (StateElements) {
    StateElements["Model"] = "model";
    StateElements["ModelProps"] = "model-props";
    StateElements["Assembly"] = "assembly";
    StateElements["VolumeStreaming"] = "volume-streaming";
    StateElements["Sequence"] = "sequence";
    StateElements["SequenceVisual"] = "sequence-visual";
    StateElements["Het"] = "het";
    StateElements["HetVisual"] = "het-visual";
    StateElements["Het3DSNFG"] = "het-3dsnfg";
    StateElements["Water"] = "water";
    StateElements["WaterVisual"] = "water-visual";
    StateElements["HetGroupFocus"] = "het-group-focus";
    StateElements["HetGroupFocusGroup"] = "het-group-focus-group";
})(StateElements = exports.StateElements || (exports.StateElements = {}));
