/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Mat4, Vec3, Vec4 } from '../mol-math/linear-algebra';
import { Viewport } from './camera/util';
import { Object3D } from '../mol-gl/object3d';
import { BehaviorSubject } from 'rxjs';
import { CameraTransitionManager } from './camera/transition';
export { Camera };
declare class Camera implements Object3D {
    readonly updatedViewProjection: BehaviorSubject<Camera>;
    readonly view: Mat4;
    readonly projection: Mat4;
    readonly projectionView: Mat4;
    readonly inverseProjectionView: Mat4;
    readonly viewport: Viewport;
    readonly state: Readonly<Camera.Snapshot>;
    readonly viewOffset: Camera.ViewOffset;
    readonly transition: CameraTransitionManager;
    position: Vec3;
    direction: Vec3;
    up: Vec3;
    target: Vec3;
    private prevProjection;
    private prevView;
    private deltaDirection;
    private newPosition;
    updateMatrices(): boolean;
    setState(snapshot: Partial<Camera.Snapshot>): void;
    getSnapshot(): Camera.Snapshot;
    getFocus(target: Vec3, radius: number): Partial<Camera.Snapshot>;
    focus(target: Vec3, radius: number): void;
    project(out: Vec4, point: Vec3): Vec4;
    unproject(out: Vec3, point: Vec3): Vec3;
    dispose(): void;
    constructor(state?: Partial<Camera.Snapshot>, viewport?: Viewport);
}
declare namespace Camera {
    type Mode = 'perspective' | 'orthographic';
    interface ClippingInfo {
        near: number;
        far: number;
        fogNear: number;
        fogFar: number;
    }
    /**
     * Sets an offseted view in a larger frustum. This is useful for
     * - multi-window or multi-monitor/multi-machine setups
     * - jittering the camera position for
     */
    interface ViewOffset {
        enabled: boolean;
        fullWidth: number;
        fullHeight: number;
        offsetX: number;
        offsetY: number;
        width: number;
        height: number;
    }
    function setViewOffset(out: ViewOffset, fullWidth: number, fullHeight: number, offsetX: number, offsetY: number, width: number, height: number): void;
    function createDefaultSnapshot(): Snapshot;
    interface Snapshot {
        mode: Mode;
        position: Vec3;
        direction: Vec3;
        up: Vec3;
        target: Vec3;
        near: number;
        far: number;
        fogNear: number;
        fogFar: number;
        fov: number;
        zoom: number;
    }
    function copySnapshot(out: Snapshot, source?: Partial<Snapshot>): void;
}
