/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { BehaviorSubject } from 'rxjs';
import { now } from '../mol-util/now';
import InputObserver, { ModifiersKeys, ButtonsType } from '../mol-util/input/input-observer';
import { RendererStats } from '../mol-gl/renderer';
import { WebGLContext } from '../mol-gl/webgl/context';
import { Representation } from '../mol-repr/representation';
import { GraphicsRenderVariant } from '../mol-gl/webgl/render-item';
import { PickingId } from '../mol-geo/geometry/picking';
import { MarkerAction } from '../mol-util/marker-action';
import { Camera } from './camera';
import { ParamDefinition as PD } from '../mol-util/param-definition';
import { Canvas3dInteractionHelper } from './helper/interaction-events';
import { GLRenderingContext } from '../mol-gl/webgl/compat';
import { PixelData } from '../mol-util/image';
export declare const Canvas3DParams: {
    cameraMode: PD.Select<Camera.Mode>;
    cameraClipDistance: PD.Numeric;
    clip: PD.Interval;
    fog: PD.Interval;
    multiSample: PD.Group<PD.Normalize<{
        mode: "on" | "off" | "temporal";
        sampleLevel: number;
    }>>;
    postprocessing: PD.Group<PD.Normalize<{
        occlusionEnable: boolean;
        occlusionKernelSize: number;
        occlusionBias: number;
        occlusionRadius: number;
        outlineEnable: boolean;
        outlineScale: number;
        outlineThreshold: number;
        useFog: boolean;
    }>>;
    renderer: PD.Group<PD.Normalize<{
        backgroundColor: import("../mol-util/color").Color;
        pickingAlphaThreshold: number;
        lightIntensity: number;
        ambientIntensity: number;
        metalness: number;
        roughness: number;
        reflectivity: number;
    }>>;
    trackball: PD.Group<PD.Normalize<{
        noScroll: boolean;
        rotateSpeed: number;
        zoomSpeed: number;
        panSpeed: number;
        spin: boolean;
        spinSpeed: number;
        staticMoving: boolean;
        dynamicDampingFactor: number;
        minDistance: number;
        maxDistance: number;
    }>>;
    debug: PD.Group<PD.Normalize<{
        sceneBoundingSpheres: boolean;
        objectBoundingSpheres: boolean;
        instanceBoundingSpheres: boolean;
    }>>;
};
export declare type Canvas3DProps = PD.Values<typeof Canvas3DParams>;
export { Canvas3D };
interface Canvas3D {
    readonly webgl: WebGLContext;
    add: (repr: Representation.Any) => void;
    remove: (repr: Representation.Any) => void;
    update: (repr?: Representation.Any, keepBoundingSphere?: boolean) => void;
    clear: () => void;
    requestDraw: (force?: boolean) => void;
    animate: () => void;
    identify: (x: number, y: number) => PickingId | undefined;
    mark: (loci: Representation.Loci, action: MarkerAction) => void;
    getLoci: (pickingId: PickingId) => Representation.Loci;
    readonly didDraw: BehaviorSubject<now.Timestamp>;
    handleResize: () => void;
    /** Focuses camera on scene's bounding sphere, centered and zoomed. */
    resetCamera: () => void;
    readonly camera: Camera;
    downloadScreenshot: () => void;
    getPixelData: (variant: GraphicsRenderVariant) => PixelData;
    setProps: (props: Partial<Canvas3DProps>) => void;
    /** Returns a copy of the current Canvas3D instance props */
    readonly props: Readonly<Canvas3DProps>;
    readonly input: InputObserver;
    readonly stats: RendererStats;
    readonly interaction: Canvas3dInteractionHelper['events'];
    dispose: () => void;
}
declare namespace Canvas3D {
    interface HighlightEvent {
        current: Representation.Loci;
        modifiers?: ModifiersKeys;
    }
    interface ClickEvent {
        current: Representation.Loci;
        buttons: ButtonsType;
        modifiers: ModifiersKeys;
    }
    function fromCanvas(canvas: HTMLCanvasElement, props?: Partial<Canvas3DProps>): Canvas3D;
    function create(gl: GLRenderingContext, input: InputObserver, props?: Partial<Canvas3DProps>): Canvas3D;
}
