/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 *
 * This code has been modified from https://github.com/mrdoob/three.js/,
 * copyright (c) 2010-2018 three.js authors. MIT License
 */
import { Vec3 } from '../../mol-math/linear-algebra';
import { Viewport } from '../camera/util';
import InputObserver from '../../mol-util/input/input-observer';
import { Object3D } from '../../mol-gl/object3d';
import { ParamDefinition as PD } from '../../mol-util/param-definition';
export declare const TrackballControlsParams: {
    noScroll: PD.BooleanParam;
    rotateSpeed: PD.Numeric;
    zoomSpeed: PD.Numeric;
    panSpeed: PD.Numeric;
    spin: PD.BooleanParam;
    spinSpeed: PD.Numeric;
    staticMoving: PD.BooleanParam;
    dynamicDampingFactor: PD.Numeric;
    minDistance: PD.Numeric;
    maxDistance: PD.Numeric;
};
export declare type TrackballControlsProps = PD.Values<typeof TrackballControlsParams>;
export { TrackballControls };
interface TrackballControls {
    viewport: Viewport;
    readonly props: Readonly<TrackballControlsProps>;
    setProps: (props: Partial<TrackballControlsProps>) => void;
    update: (t: number) => void;
    reset: () => void;
    dispose: () => void;
}
declare namespace TrackballControls {
    function create(input: InputObserver, object: Object3D & {
        target: Vec3;
    }, props?: Partial<TrackballControlsProps>): TrackballControls;
}
