"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var loci_1 = require("../../mol-model/loci");
var representation_1 = require("../../mol-repr/representation");
var input_observer_1 = require("../../mol-util/input/input-observer");
var rx_event_helper_1 = require("../../mol-util/rx-event-helper");
var Canvas3dInteractionHelper = /** @class */ (function () {
    function Canvas3dInteractionHelper(canvasIdentify, getLoci, input, maxFps) {
        var _this = this;
        if (maxFps === void 0) { maxFps = 15; }
        this.canvasIdentify = canvasIdentify;
        this.getLoci = getLoci;
        this.maxFps = maxFps;
        this.ev = rx_event_helper_1.RxEventHelper.create();
        this.events = {
            highlight: this.ev(),
            click: this.ev(),
        };
        this.cX = -1;
        this.cY = -1;
        this.lastX = -1;
        this.lastY = -1;
        this.id = void 0;
        this.currentIdentifyT = 0;
        this.prevLoci = representation_1.Representation.Loci.Empty;
        this.prevT = 0;
        this.inside = false;
        this.buttons = input_observer_1.ButtonsType.create(0);
        this.modifiers = input_observer_1.ModifiersKeys.None;
        input.move.subscribe(function (_a) {
            var x = _a.x, y = _a.y, inside = _a.inside, buttons = _a.buttons, modifiers = _a.modifiers;
            if (!inside || buttons) {
                return;
            }
            _this.move(x, y, modifiers);
        });
        input.leave.subscribe(function () {
            _this.leave();
        });
        input.click.subscribe(function (_a) {
            var x = _a.x, y = _a.y, buttons = _a.buttons, modifiers = _a.modifiers;
            _this.select(x, y, buttons, modifiers);
        });
        input.modifiers.subscribe(function (modifiers) { return _this.modify(modifiers); });
    }
    Canvas3dInteractionHelper.prototype.identify = function (isClick, t) {
        if (this.lastX !== this.cX || this.lastY !== this.cY) {
            this.id = this.canvasIdentify(this.cX, this.cY);
            this.lastX = this.cX;
            this.lastY = this.cY;
        }
        if (!this.id)
            return;
        if (isClick) {
            this.events.click.next({ current: this.getLoci(this.id), buttons: this.buttons, modifiers: this.modifiers });
            return;
        }
        // only highlight the latest
        if (!this.inside || this.currentIdentifyT !== t) {
            return;
        }
        var loci = this.getLoci(this.id);
        if (!representation_1.Representation.Loci.areEqual(this.prevLoci, loci)) {
            this.events.highlight.next({ current: loci, modifiers: this.modifiers });
            this.prevLoci = loci;
        }
    };
    Canvas3dInteractionHelper.prototype.tick = function (t) {
        if (this.inside && t - this.prevT > 1000 / this.maxFps) {
            this.prevT = t;
            this.currentIdentifyT = t;
            this.identify(false, t);
        }
    };
    Canvas3dInteractionHelper.prototype.leave = function () {
        this.inside = false;
        if (this.prevLoci.loci !== loci_1.EmptyLoci) {
            this.prevLoci = representation_1.Representation.Loci.Empty;
            this.events.highlight.next({ current: this.prevLoci });
        }
    };
    Canvas3dInteractionHelper.prototype.move = function (x, y, modifiers) {
        this.inside = true;
        this.modifiers = modifiers;
        this.cX = x;
        this.cY = y;
    };
    Canvas3dInteractionHelper.prototype.select = function (x, y, buttons, modifiers) {
        this.cX = x;
        this.cY = y;
        this.buttons = buttons;
        this.modifiers = modifiers;
        this.identify(true, 0);
    };
    Canvas3dInteractionHelper.prototype.modify = function (modifiers) {
        if (this.prevLoci.loci === loci_1.EmptyLoci || input_observer_1.ModifiersKeys.areEqual(modifiers, this.modifiers))
            return;
        this.modifiers = modifiers;
        this.events.highlight.next({ current: this.prevLoci, modifiers: this.modifiers });
    };
    Canvas3dInteractionHelper.prototype.dispose = function () {
        this.ev.dispose();
    };
    return Canvas3dInteractionHelper;
}());
exports.Canvas3dInteractionHelper = Canvas3dInteractionHelper;
