"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var render_target_1 = require("../../mol-gl/webgl/render-target");
var texture_1 = require("../../mol-gl/webgl/texture");
var DrawPass = /** @class */ (function () {
    function DrawPass(webgl, renderer, scene, debugHelper) {
        this.webgl = webgl;
        this.renderer = renderer;
        this.scene = scene;
        this.debugHelper = debugHelper;
        var gl = webgl.gl, extensions = webgl.extensions;
        var width = gl.drawingBufferWidth;
        var height = gl.drawingBufferHeight;
        this.colorTarget = render_target_1.createRenderTarget(webgl, gl.drawingBufferWidth, gl.drawingBufferHeight);
        this.packedDepth = !extensions.depthTexture;
        this.depthTarget = this.packedDepth ? render_target_1.createRenderTarget(webgl, width, height) : null;
        this.depthTexture = this.depthTarget ? this.depthTarget.texture : texture_1.createTexture(webgl, 'image-depth', 'depth', 'ushort', 'nearest');
        if (!this.packedDepth) {
            this.depthTexture.define(width, height);
            this.depthTexture.attachFramebuffer(this.colorTarget.framebuffer, 'depth');
        }
    }
    DrawPass.prototype.setSize = function (width, height) {
        this.colorTarget.setSize(width, height);
        if (this.depthTarget) {
            this.depthTarget.setSize(width, height);
        }
        else {
            this.depthTexture.define(width, height);
        }
    };
    DrawPass.prototype.render = function (toDrawingBuffer) {
        var _a = this, webgl = _a.webgl, renderer = _a.renderer, scene = _a.scene, debugHelper = _a.debugHelper, colorTarget = _a.colorTarget, depthTarget = _a.depthTarget;
        var gl = webgl.gl;
        if (toDrawingBuffer) {
            webgl.unbindFramebuffer();
            gl.viewport(0, 0, gl.drawingBufferWidth, gl.drawingBufferHeight);
        }
        else {
            colorTarget.bind();
        }
        renderer.setViewport(0, 0, gl.drawingBufferWidth, gl.drawingBufferHeight);
        renderer.render(scene, 'color', true);
        if (debugHelper.isEnabled) {
            debugHelper.syncVisibility();
            renderer.render(debugHelper.scene, 'color', false);
        }
        // do a depth pass if not rendering to drawing buffer and
        // extensions.depthTexture is unsupported (i.e. depthTarget is set)
        if (!toDrawingBuffer && depthTarget) {
            depthTarget.bind();
            renderer.render(scene, 'depth', true);
            if (debugHelper.isEnabled) {
                debugHelper.syncVisibility();
                renderer.render(debugHelper.scene, 'depth', false);
            }
        }
    };
    return DrawPass;
}());
exports.DrawPass = DrawPass;
