"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var render_target_1 = require("../../mol-gl/webgl/render-target");
var float_packing_1 = require("../../mol-util/float-packing");
var readBuffer = new Uint8Array(4);
var PickPass = /** @class */ (function () {
    function PickPass(webgl, renderer, scene, pickBaseScale) {
        this.webgl = webgl;
        this.renderer = renderer;
        this.scene = scene;
        this.pickBaseScale = pickBaseScale;
        this.pickDirty = true;
        var gl = webgl.gl;
        var width = gl.drawingBufferWidth;
        var height = gl.drawingBufferHeight;
        this.pickScale = pickBaseScale / webgl.pixelRatio;
        this.pickWidth = Math.round(width * this.pickScale);
        this.pickHeight = Math.round(height * this.pickScale);
        this.objectPickTarget = render_target_1.createRenderTarget(webgl, this.pickWidth, this.pickHeight);
        this.instancePickTarget = render_target_1.createRenderTarget(webgl, this.pickWidth, this.pickHeight);
        this.groupPickTarget = render_target_1.createRenderTarget(webgl, this.pickWidth, this.pickHeight);
    }
    PickPass.prototype.setSize = function (width, height) {
        this.pickScale = this.pickBaseScale / this.webgl.pixelRatio;
        this.pickWidth = Math.round(width * this.pickScale);
        this.pickHeight = Math.round(height * this.pickScale);
        this.objectPickTarget.setSize(this.pickWidth, this.pickHeight);
        this.instancePickTarget.setSize(this.pickWidth, this.pickHeight);
        this.groupPickTarget.setSize(this.pickWidth, this.pickHeight);
    };
    PickPass.prototype.render = function () {
        var _a = this, renderer = _a.renderer, scene = _a.scene;
        renderer.setViewport(0, 0, this.pickWidth, this.pickHeight);
        this.objectPickTarget.bind();
        renderer.render(scene, 'pickObject', true);
        this.instancePickTarget.bind();
        renderer.render(scene, 'pickInstance', true);
        this.groupPickTarget.bind();
        renderer.render(scene, 'pickGroup', true);
    };
    PickPass.prototype.identify = function (x, y) {
        var _a = this, webgl = _a.webgl, pickScale = _a.pickScale;
        var gl = webgl.gl;
        if (this.pickDirty)
            this.render();
        x *= webgl.pixelRatio;
        y *= webgl.pixelRatio;
        y = gl.drawingBufferHeight - y; // flip y
        var xp = Math.round(x * pickScale);
        var yp = Math.round(y * pickScale);
        this.objectPickTarget.bind();
        webgl.readPixels(xp, yp, 1, 1, readBuffer);
        var objectId = float_packing_1.decodeFloatRGB(readBuffer[0], readBuffer[1], readBuffer[2]);
        if (objectId === -1)
            return;
        this.instancePickTarget.bind();
        webgl.readPixels(xp, yp, 1, 1, readBuffer);
        var instanceId = float_packing_1.decodeFloatRGB(readBuffer[0], readBuffer[1], readBuffer[2]);
        if (instanceId === -1)
            return;
        this.groupPickTarget.bind();
        webgl.readPixels(xp, yp, 1, 1, readBuffer);
        var groupId = float_packing_1.decodeFloatRGB(readBuffer[0], readBuffer[1], readBuffer[2]);
        if (groupId === -1)
            return;
        return { objectId: objectId, instanceId: instanceId, groupId: groupId };
    };
    return PickPass;
}());
exports.PickPass = PickPass;
