/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { TextureSpec, Values, UniformSpec, DefineSpec } from '../../mol-gl/renderable/schema';
import { WebGLContext } from '../../mol-gl/webgl/context';
import { ComputeRenderable } from '../../mol-gl/renderable';
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { RenderTarget } from '../../mol-gl/webgl/render-target';
import { DrawPass } from './draw';
import { Camera } from '../../mol-canvas3d/camera';
declare const PostprocessingSchema: {
    tColor: TextureSpec<"texture">;
    tDepth: TextureSpec<"texture">;
    uTexSize: UniformSpec<"v2">;
    dUseFog: DefineSpec<"boolean">;
    dOrthographic: DefineSpec<"number">;
    uNear: UniformSpec<"f">;
    uFar: UniformSpec<"f">;
    uFogNear: UniformSpec<"f">;
    uFogFar: UniformSpec<"f">;
    uFogColor: UniformSpec<"v3">;
    dOcclusionEnable: DefineSpec<"boolean">;
    dOcclusionKernelSize: DefineSpec<"number">;
    uOcclusionBias: UniformSpec<"f">;
    uOcclusionRadius: UniformSpec<"f">;
    dOutlineEnable: DefineSpec<"boolean">;
    uOutlineScale: UniformSpec<"f">;
    uOutlineThreshold: UniformSpec<"f">;
    dPackedDepth: DefineSpec<"boolean">;
    drawCount: import("../../mol-gl/renderable/schema").ValueSpec<"number">;
    instanceCount: import("../../mol-gl/renderable/schema").ValueSpec<"number">;
    aPosition: import("../../mol-gl/renderable/schema").AttributeSpec<"float32">;
    uQuadScale: UniformSpec<"v2">;
};
export declare const PostprocessingParams: {
    occlusionEnable: PD.BooleanParam;
    occlusionKernelSize: PD.Numeric;
    occlusionBias: PD.Numeric;
    occlusionRadius: PD.Numeric;
    outlineEnable: PD.BooleanParam;
    outlineScale: PD.Numeric;
    outlineThreshold: PD.Numeric;
    useFog: PD.BooleanParam;
};
export declare type PostprocessingProps = PD.Values<typeof PostprocessingParams>;
declare type PostprocessingRenderable = ComputeRenderable<Values<typeof PostprocessingSchema>>;
export declare class PostprocessingPass {
    private webgl;
    private camera;
    target: RenderTarget;
    props: PostprocessingProps;
    renderable: PostprocessingRenderable;
    constructor(webgl: WebGLContext, camera: Camera, drawPass: DrawPass, props: Partial<PostprocessingProps>);
    readonly enabled: boolean;
    setSize(width: number, height: number): void;
    setProps(props: Partial<PostprocessingProps>): void;
    render(toDrawingBuffer: boolean): void;
}
export {};
