"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../../mol-gl/compute/util");
var schema_1 = require("../../mol-gl/renderable/schema");
var shader_code_1 = require("../../mol-gl/shader-code");
var mol_util_1 = require("../../mol-util");
var render_item_1 = require("../../mol-gl/webgl/render-item");
var renderable_1 = require("../../mol-gl/renderable");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var param_definition_1 = require("../../mol-util/param-definition");
var render_target_1 = require("../../mol-gl/webgl/render-target");
var quad_vert_1 = require("../../mol-gl/shader/quad.vert");
var postprocessing_frag_1 = require("../../mol-gl/shader/postprocessing.frag");
var PostprocessingSchema = __assign({}, util_1.QuadSchema, { tColor: schema_1.TextureSpec('texture', 'rgba', 'ubyte', 'nearest'), tDepth: schema_1.TextureSpec('texture', 'rgba', 'ubyte', 'nearest'), uTexSize: schema_1.UniformSpec('v2'), dUseFog: schema_1.DefineSpec('boolean'), dOrthographic: schema_1.DefineSpec('number'), uNear: schema_1.UniformSpec('f'), uFar: schema_1.UniformSpec('f'), uFogNear: schema_1.UniformSpec('f'), uFogFar: schema_1.UniformSpec('f'), uFogColor: schema_1.UniformSpec('v3'), dOcclusionEnable: schema_1.DefineSpec('boolean'), dOcclusionKernelSize: schema_1.DefineSpec('number'), uOcclusionBias: schema_1.UniformSpec('f'), uOcclusionRadius: schema_1.UniformSpec('f'), dOutlineEnable: schema_1.DefineSpec('boolean'), uOutlineScale: schema_1.UniformSpec('f'), uOutlineThreshold: schema_1.UniformSpec('f'), dPackedDepth: schema_1.DefineSpec('boolean') });
exports.PostprocessingParams = {
    occlusionEnable: param_definition_1.ParamDefinition.Boolean(false),
    occlusionKernelSize: param_definition_1.ParamDefinition.Numeric(4, { min: 1, max: 32, step: 1 }),
    occlusionBias: param_definition_1.ParamDefinition.Numeric(0.5, { min: 0, max: 1, step: 0.01 }),
    occlusionRadius: param_definition_1.ParamDefinition.Numeric(64, { min: 0, max: 256, step: 1 }),
    outlineEnable: param_definition_1.ParamDefinition.Boolean(false),
    outlineScale: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 10, step: 1 }),
    outlineThreshold: param_definition_1.ParamDefinition.Numeric(0.8, { min: 0, max: 1, step: 0.01 }),
    useFog: param_definition_1.ParamDefinition.Boolean(true),
};
function getPostprocessingRenderable(ctx, colorTexture, depthTexture, packedDepth, props) {
    var p = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(exports.PostprocessingParams), props);
    var values = __assign({}, util_1.QuadValues, { tColor: mol_util_1.ValueCell.create(colorTexture), tDepth: mol_util_1.ValueCell.create(depthTexture), uTexSize: mol_util_1.ValueCell.create(linear_algebra_1.Vec2.create(colorTexture.width, colorTexture.height)), dUseFog: mol_util_1.ValueCell.create(p.useFog), dOrthographic: mol_util_1.ValueCell.create(0), uNear: mol_util_1.ValueCell.create(1), uFar: mol_util_1.ValueCell.create(10000), uFogNear: mol_util_1.ValueCell.create(10000), uFogFar: mol_util_1.ValueCell.create(10000), uFogColor: mol_util_1.ValueCell.create(linear_algebra_1.Vec3.create(1, 1, 1)), dOcclusionEnable: mol_util_1.ValueCell.create(p.occlusionEnable), dOcclusionKernelSize: mol_util_1.ValueCell.create(p.occlusionKernelSize), uOcclusionBias: mol_util_1.ValueCell.create(p.occlusionBias), uOcclusionRadius: mol_util_1.ValueCell.create(p.occlusionRadius), dOutlineEnable: mol_util_1.ValueCell.create(p.outlineEnable), uOutlineScale: mol_util_1.ValueCell.create(p.outlineScale * ctx.pixelRatio), uOutlineThreshold: mol_util_1.ValueCell.create(p.outlineThreshold), dPackedDepth: mol_util_1.ValueCell.create(packedDepth) });
    var schema = __assign({}, PostprocessingSchema);
    var shaderCode = shader_code_1.ShaderCode(quad_vert_1.default, postprocessing_frag_1.default);
    var renderItem = render_item_1.createComputeRenderItem(ctx, 'triangles', shaderCode, schema, values);
    return renderable_1.createComputeRenderable(renderItem, values);
}
var PostprocessingPass = /** @class */ (function () {
    function PostprocessingPass(webgl, camera, drawPass, props) {
        this.webgl = webgl;
        this.camera = camera;
        var gl = webgl.gl;
        this.target = render_target_1.createRenderTarget(webgl, gl.drawingBufferWidth, gl.drawingBufferHeight);
        this.props = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(exports.PostprocessingParams), props);
        var colorTarget = drawPass.colorTarget, depthTexture = drawPass.depthTexture, packedDepth = drawPass.packedDepth;
        this.renderable = getPostprocessingRenderable(webgl, colorTarget.texture, depthTexture, packedDepth, this.props);
    }
    Object.defineProperty(PostprocessingPass.prototype, "enabled", {
        get: function () {
            return this.props.occlusionEnable || this.props.outlineEnable;
        },
        enumerable: true,
        configurable: true
    });
    PostprocessingPass.prototype.setSize = function (width, height) {
        this.target.setSize(width, height);
        mol_util_1.ValueCell.update(this.renderable.values.uTexSize, linear_algebra_1.Vec2.set(this.renderable.values.uTexSize.ref.value, width, height));
    };
    PostprocessingPass.prototype.setProps = function (props) {
        if (props.occlusionEnable !== undefined) {
            this.props.occlusionEnable = props.occlusionEnable;
            mol_util_1.ValueCell.update(this.renderable.values.dOcclusionEnable, props.occlusionEnable);
        }
        if (props.occlusionKernelSize !== undefined) {
            this.props.occlusionKernelSize = props.occlusionKernelSize;
            mol_util_1.ValueCell.update(this.renderable.values.dOcclusionKernelSize, props.occlusionKernelSize);
        }
        if (props.occlusionBias !== undefined) {
            this.props.occlusionBias = props.occlusionBias;
            mol_util_1.ValueCell.update(this.renderable.values.uOcclusionBias, props.occlusionBias);
        }
        if (props.occlusionRadius !== undefined) {
            this.props.occlusionRadius = props.occlusionRadius;
            mol_util_1.ValueCell.update(this.renderable.values.uOcclusionRadius, props.occlusionRadius);
        }
        if (props.outlineEnable !== undefined) {
            this.props.outlineEnable = props.outlineEnable;
            mol_util_1.ValueCell.update(this.renderable.values.dOutlineEnable, props.outlineEnable);
        }
        if (props.outlineScale !== undefined) {
            this.props.outlineScale = props.outlineScale;
            mol_util_1.ValueCell.update(this.renderable.values.uOutlineScale, props.outlineScale * this.webgl.pixelRatio);
        }
        if (props.outlineThreshold !== undefined) {
            this.props.outlineThreshold = props.outlineThreshold;
            mol_util_1.ValueCell.update(this.renderable.values.uOutlineThreshold, props.outlineThreshold);
        }
        if (props.useFog !== undefined) {
            this.props.useFog = props.useFog;
            mol_util_1.ValueCell.update(this.renderable.values.dUseFog, props.useFog);
        }
        this.renderable.update();
    };
    PostprocessingPass.prototype.render = function (toDrawingBuffer) {
        mol_util_1.ValueCell.update(this.renderable.values.uFar, this.camera.state.far);
        mol_util_1.ValueCell.update(this.renderable.values.uNear, this.camera.state.near);
        mol_util_1.ValueCell.update(this.renderable.values.uFogFar, this.camera.state.fogFar);
        mol_util_1.ValueCell.update(this.renderable.values.uFogNear, this.camera.state.fogNear);
        mol_util_1.ValueCell.update(this.renderable.values.dOrthographic, this.camera.state.mode === 'orthographic' ? 1 : 0);
        var _a = this.webgl, gl = _a.gl, state = _a.state;
        if (toDrawingBuffer) {
            this.webgl.unbindFramebuffer();
            gl.viewport(0, 0, gl.drawingBufferWidth, gl.drawingBufferHeight);
        }
        else {
            this.target.bind();
        }
        state.disable(gl.SCISSOR_TEST);
        state.disable(gl.BLEND);
        state.disable(gl.DEPTH_TEST);
        state.depthMask(false);
        this.renderable.render();
    };
    return PostprocessingPass;
}());
exports.PostprocessingPass = PostprocessingPass;
