"use strict";
/**
 * Copyright (c) 2017 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var interval_1 = require("../interval");
var sorted_array_1 = require("../sorted-array");
describe('sortedArray', function () {
    function testI(name, a, b) {
        it(name, function () { return expect(interval_1.default.areEqual(a, b)).toBe(true); });
    }
    function test(name, a, b) {
        it(name, function () { return expect(a).toEqual(b); });
    }
    function compareArrays(a, b) {
        expect(a.length).toBe(b.length);
        for (var i = 0; i < a.length; i++)
            expect(a[i]).toBe(b[i]);
    }
    var a1234 = sorted_array_1.default.ofSortedArray([1, 2, 3, 4]);
    var a2468 = sorted_array_1.default.ofSortedArray([2, 4, 6, 8]);
    test('size', sorted_array_1.default.size(a1234), 4);
    test('min/max', [sorted_array_1.default.min(a1234), sorted_array_1.default.max(a1234)], [1, 4]);
    test('start/end', [sorted_array_1.default.start(a1234), sorted_array_1.default.end(a1234)], [1, 5]);
    test('has', sorted_array_1.default.has(a1234, 5), false);
    test('has', sorted_array_1.default.has(a1234, 4), true);
    it('has-all', function () {
        for (var i = 1; i <= 4; i++)
            expect(sorted_array_1.default.has(a1234, i)).toBe(true);
    });
    test('indexOf', sorted_array_1.default.indexOf(a2468, 5), -1);
    test('indexOf', sorted_array_1.default.indexOf(a2468, 2), 0);
    test('getAt', a2468[1], 4);
    test('areEqual', sorted_array_1.default.areEqual(a2468, a2468), true);
    test('areEqual1', sorted_array_1.default.areEqual(a2468, sorted_array_1.default.ofUnsortedArray([4, 2, 8, 6])), true);
    test('areEqual2', sorted_array_1.default.areEqual(a1234, a2468), false);
    test('predIndex1', sorted_array_1.default.findPredecessorIndex(a1234, 5), 4);
    test('predIndex2', sorted_array_1.default.findPredecessorIndex(a1234, 2), 1);
    test('predIndex3', sorted_array_1.default.findPredecessorIndex(a2468, 4), 1);
    test('predIndex4', sorted_array_1.default.findPredecessorIndex(a2468, 3), 1);
    test('predIndexInt', sorted_array_1.default.findPredecessorIndexInInterval(a1234, 0, interval_1.default.ofRange(2, 3)), 2);
    testI('findRange', sorted_array_1.default.findRange(a2468, 2, 4), interval_1.default.ofRange(0, 1));
    it('deduplicate', function () {
        compareArrays(sorted_array_1.default.deduplicate(sorted_array_1.default.ofSortedArray([1, 1, 1, 1])), [1]);
        compareArrays(sorted_array_1.default.deduplicate(sorted_array_1.default.ofSortedArray([1, 1, 2, 2, 3, 4])), [1, 2, 3, 4]);
        compareArrays(sorted_array_1.default.deduplicate(sorted_array_1.default.ofSortedArray([1, 2, 3])), [1, 2, 3]);
    });
    it('indicesOf', function () {
        compareArrays(sorted_array_1.default.indicesOf(sorted_array_1.default.ofSortedArray([10, 11, 12]), sorted_array_1.default.ofSortedArray([10, 12, 14])), [0, 2]);
    });
    test('intersectionSize', sorted_array_1.default.intersectionSize(a1234, a2468), 2);
    // console.log(Interval.findPredecessorIndexInInterval(Interval.ofBounds(0, 3), 2, Interval.ofBounds(0, 3)))
    // console.log(SortedArray.findPredecessorIndexInInterval(SortedArray.ofSortedArray([0, 1, 2]), 2, Interval.ofBounds(0, 3)))
});
