"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var int_1 = require("../int");
var SortedRanges;
(function (SortedRanges) {
    function ofSortedRanges(array) { return int_1.SortedArray.ofSortedArray(array); }
    SortedRanges.ofSortedRanges = ofSortedRanges;
    function start(ranges) { return ranges[0]; }
    SortedRanges.start = start;
    function end(ranges) { return ranges[ranges.length - 1] + 1; }
    SortedRanges.end = end;
    function min(ranges) { return ranges[0]; }
    SortedRanges.min = min;
    function max(ranges) { return ranges[ranges.length - 1]; }
    SortedRanges.max = max;
    function size(ranges) {
        var size = 0;
        for (var i = 0, il = ranges.length; i < il; i += 2) {
            size += ranges[i + 1] - ranges[i] + 1;
        }
        return size;
    }
    SortedRanges.size = size;
    function transientSegments(ranges, set) {
        return new Iterator(ranges, set);
    }
    SortedRanges.transientSegments = transientSegments;
    var Iterator = /** @class */ (function () {
        function Iterator(ranges, set) {
            this.ranges = ranges;
            this.set = set;
            this.value = { index: 0, start: 0, end: 0 };
            this.curIndex = 0;
            this.maxIndex = 0;
            this.curMin = 0;
            this.hasNext = false;
            // TODO cleanup, refactor to make it clearer
            var min = int_1.SortedArray.findPredecessorIndex(this.ranges, int_1.OrderedSet.min(set));
            var max = int_1.SortedArray.findPredecessorIndex(this.ranges, int_1.OrderedSet.max(set) + 1);
            if (ranges.length && min !== max) {
                this.curIndex = this.getRangeIndex(int_1.OrderedSet.min(set));
                this.maxIndex = Math.min(ranges.length - 2, this.getRangeIndex(int_1.OrderedSet.max(set)));
                this.curMin = this.ranges[this.curIndex];
                this.updateInterval();
            }
            this.hasNext = ranges.length > 0 && min !== max && this.curIndex <= this.maxIndex;
        }
        Iterator.prototype.updateInterval = function () {
            this.interval = int_1.Interval.ofRange(this.ranges[this.curIndex], this.ranges[this.curIndex + 1]);
        };
        Iterator.prototype.updateValue = function () {
            this.value.index = this.curIndex / 2;
            this.value.start = int_1.OrderedSet.findPredecessorIndex(this.set, this.ranges[this.curIndex]);
            this.value.end = int_1.OrderedSet.findPredecessorIndex(this.set, this.ranges[this.curIndex + 1]) + 1;
        };
        Iterator.prototype.move = function () {
            if (this.hasNext) {
                this.updateValue();
                while (this.curIndex <= this.maxIndex) {
                    this.curIndex += 2;
                    this.curMin = int_1.Interval.end(this.interval);
                    this.updateInterval();
                    if (int_1.Interval.min(this.interval) >= this.curMin && int_1.OrderedSet.areIntersecting(this.interval, this.set))
                        break;
                }
                this.hasNext = this.curIndex <= this.maxIndex;
            }
            return this.value;
        };
        Iterator.prototype.getRangeIndex = function (value) {
            var index = int_1.SortedArray.findPredecessorIndex(this.ranges, value);
            return (index % 2 === 1) ? index - 1 : index;
        };
        return Iterator;
    }());
    SortedRanges.Iterator = Iterator;
})(SortedRanges || (SortedRanges = {}));
exports.default = SortedRanges;
