/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { RenderableState } from '../../mol-gl/renderable';
import { BaseValues } from '../../mol-gl/renderable/schema';
import { LocationIterator } from '../util/location-iterator';
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { Color } from '../../mol-util/color';
import { Vec3 } from '../../mol-math/linear-algebra';
import { TransformData } from './transform-data';
import { Theme } from '../../mol-theme/theme';
export declare const VisualQualityInfo: {
    'custom': {};
    'auto': {};
    'highest': {};
    'higher': {};
    'high': {};
    'medium': {};
    'low': {};
    'lower': {};
    'lowest': {};
};
export declare type VisualQuality = keyof typeof VisualQualityInfo;
export declare const VisualQualityNames: string[];
export declare const VisualQualityOptions: ["auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest", string][];
export declare namespace BaseGeometry {
    const Params: {
        alpha: PD.Numeric;
        useFog: PD.BooleanParam;
        highlightColor: PD.Color;
        selectColor: PD.Color;
        quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    };
    type Params = typeof Params;
    type Counts = {
        drawCount: number;
        groupCount: number;
        instanceCount: number;
    };
    function createSimple(colorValue?: Color, sizeValue?: number, transform?: TransformData): {
        transform: TransformData;
        locationIterator: LocationIterator;
        theme: Theme;
    };
    function createValues(props: PD.Values<Params>, counts: Counts): {
        alpha: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<number, never>>;
        uAlpha: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<number, never>>;
        uHighlightColor: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<Vec3, never>>;
        uSelectColor: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<Vec3, never>>;
        dUseFog: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<boolean, never>>;
        uGroupCount: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<number, never>>;
        drawCount: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<number, never>>;
    };
    function updateValues(values: BaseValues, props: PD.Values<Params>): void;
    function createRenderableState(props?: Partial<PD.Values<Params>>): RenderableState;
    function updateRenderableState(state: RenderableState, props: PD.Values<Params>): void;
}
