"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../mol-util");
var location_iterator_1 = require("../util/location-iterator");
var param_definition_1 = require("../../mol-util/param-definition");
var color_1 = require("../../mol-util/color");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var transform_data_1 = require("./transform-data");
var tables_1 = require("../../mol-util/color/tables");
var location_1 = require("../../mol-model/location");
var uniform_1 = require("../../mol-theme/color/uniform");
var uniform_2 = require("../../mol-theme/size/uniform");
exports.VisualQualityInfo = {
    'custom': {},
    'auto': {},
    'highest': {},
    'higher': {},
    'high': {},
    'medium': {},
    'low': {},
    'lower': {},
    'lowest': {},
};
exports.VisualQualityNames = Object.keys(exports.VisualQualityInfo);
exports.VisualQualityOptions = exports.VisualQualityNames.map(function (n) { return [n, n]; });
//
var BaseGeometry;
(function (BaseGeometry) {
    BaseGeometry.Params = {
        alpha: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 1, step: 0.01 }, { label: 'Opacity' }),
        useFog: param_definition_1.ParamDefinition.Boolean(true),
        highlightColor: param_definition_1.ParamDefinition.Color(color_1.Color.fromNormalizedRgb(1.0, 0.4, 0.6)),
        selectColor: param_definition_1.ParamDefinition.Color(color_1.Color.fromNormalizedRgb(0.2, 1.0, 0.1)),
        quality: param_definition_1.ParamDefinition.Select('auto', exports.VisualQualityOptions),
    };
    function createSimple(colorValue, sizeValue, transform) {
        if (colorValue === void 0) { colorValue = tables_1.ColorNames.grey; }
        if (sizeValue === void 0) { sizeValue = 1; }
        if (!transform)
            transform = transform_data_1.createIdentityTransform();
        var locationIterator = location_iterator_1.LocationIterator(1, transform.instanceCount.ref.value, function () { return location_1.NullLocation; }, false, function () { return false; });
        var theme = {
            color: uniform_1.UniformColorTheme({}, { value: colorValue }),
            size: uniform_2.UniformSizeTheme({}, { value: sizeValue })
        };
        return { transform: transform, locationIterator: locationIterator, theme: theme };
    }
    BaseGeometry.createSimple = createSimple;
    function createValues(props, counts) {
        return {
            alpha: mol_util_1.ValueCell.create(props.alpha),
            uAlpha: mol_util_1.ValueCell.create(props.alpha),
            uHighlightColor: mol_util_1.ValueCell.create(color_1.Color.toArrayNormalized(props.highlightColor, linear_algebra_1.Vec3.zero(), 0)),
            uSelectColor: mol_util_1.ValueCell.create(color_1.Color.toArrayNormalized(props.selectColor, linear_algebra_1.Vec3.zero(), 0)),
            dUseFog: mol_util_1.ValueCell.create(props.useFog),
            uGroupCount: mol_util_1.ValueCell.create(counts.groupCount),
            drawCount: mol_util_1.ValueCell.create(counts.drawCount),
        };
    }
    BaseGeometry.createValues = createValues;
    function updateValues(values, props) {
        if (color_1.Color.fromNormalizedArray(values.uHighlightColor.ref.value, 0) !== props.highlightColor) {
            mol_util_1.ValueCell.update(values.uHighlightColor, color_1.Color.toArrayNormalized(props.highlightColor, values.uHighlightColor.ref.value, 0));
        }
        if (color_1.Color.fromNormalizedArray(values.uSelectColor.ref.value, 0) !== props.selectColor) {
            mol_util_1.ValueCell.update(values.uSelectColor, color_1.Color.toArrayNormalized(props.selectColor, values.uSelectColor.ref.value, 0));
        }
        mol_util_1.ValueCell.updateIfChanged(values.alpha, props.alpha); // `uAlpha` is set in renderable.render
        mol_util_1.ValueCell.updateIfChanged(values.dUseFog, props.useFog);
    }
    BaseGeometry.updateValues = updateValues;
    function createRenderableState(props) {
        if (props === void 0) { props = {}; }
        return {
            visible: true,
            alphaFactor: 1,
            pickable: true,
            opaque: props.alpha === undefined ? true : props.alpha === 1
        };
    }
    BaseGeometry.createRenderableState = createRenderableState;
    function updateRenderableState(state, props) {
        state.opaque = props.alpha * state.alphaFactor >= 1;
    }
    BaseGeometry.updateRenderableState = updateRenderableState;
})(BaseGeometry = exports.BaseGeometry || (exports.BaseGeometry = {}));
