/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ValueCell } from '../../../mol-util';
import { Sphere3D, Box3D } from '../../../mol-math/geometry';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { Vec3, Mat4 } from '../../../mol-math/linear-algebra';
import { Texture } from '../../../mol-gl/webgl/texture';
import { GeometryUtils } from '../geometry';
export interface DirectVolume {
    readonly kind: 'direct-volume';
    readonly gridTexture: ValueCell<Texture>;
    readonly gridTextureDim: ValueCell<Vec3>;
    readonly gridDimension: ValueCell<Vec3>;
    readonly bboxSize: ValueCell<Vec3>;
    readonly bboxMin: ValueCell<Vec3>;
    readonly bboxMax: ValueCell<Vec3>;
    readonly transform: ValueCell<Mat4>;
    /** Bounding sphere of the volume */
    boundingSphere?: Sphere3D;
}
export declare namespace DirectVolume {
    function create(bbox: Box3D, gridDimension: Vec3, transform: Mat4, texture: Texture, directVolume?: DirectVolume): DirectVolume;
    function createEmpty(directVolume?: DirectVolume): DirectVolume;
    const Params: {
        isoValueNorm: PD.Numeric;
        renderMode: PD.Select<string>;
        controlPoints: PD.LineGraph;
        list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
        alpha: PD.Numeric;
        useFog: PD.BooleanParam;
        highlightColor: PD.Color;
        selectColor: PD.Color;
        quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    };
    type Params = typeof Params;
    const Utils: GeometryUtils<DirectVolume, Params>;
}
