"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../../mol-util");
var geometry_1 = require("../../../mol-math/geometry");
var param_definition_1 = require("../../../mol-util/param-definition");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var box_1 = require("../../primitive/box");
var transfer_function_1 = require("./transfer-function");
var color_data_1 = require("../color-data");
var marker_data_1 = require("../marker-data");
var util_1 = require("../../../mol-geo/util");
var util_2 = require("../../../mol-gl/renderable/util");
var scale_1 = require("../../../mol-util/color/scale");
var base_1 = require("../base");
var overpaint_data_1 = require("../overpaint-data");
var transparency_data_1 = require("../transparency-data");
var VolumeBox = box_1.Box();
var RenderModeOptions = [['isosurface', 'Isosurface'], ['volume', 'Volume']];
var DirectVolume;
(function (DirectVolume) {
    function create(bbox, gridDimension, transform, texture, directVolume) {
        var width = texture.width, height = texture.height, depth = texture.depth;
        if (directVolume) {
            mol_util_1.ValueCell.update(directVolume.gridDimension, gridDimension);
            mol_util_1.ValueCell.update(directVolume.gridTextureDim, linear_algebra_1.Vec3.set(directVolume.gridTextureDim.ref.value, width, height, depth));
            mol_util_1.ValueCell.update(directVolume.bboxMin, bbox.min);
            mol_util_1.ValueCell.update(directVolume.bboxMax, bbox.max);
            mol_util_1.ValueCell.update(directVolume.bboxSize, linear_algebra_1.Vec3.sub(directVolume.bboxSize.ref.value, bbox.max, bbox.min));
            mol_util_1.ValueCell.update(directVolume.transform, transform);
            return directVolume;
        }
        else {
            return {
                kind: 'direct-volume',
                gridDimension: mol_util_1.ValueCell.create(gridDimension),
                gridTexture: mol_util_1.ValueCell.create(texture),
                gridTextureDim: mol_util_1.ValueCell.create(linear_algebra_1.Vec3.create(width, height, depth)),
                bboxMin: mol_util_1.ValueCell.create(bbox.min),
                bboxMax: mol_util_1.ValueCell.create(bbox.max),
                bboxSize: mol_util_1.ValueCell.create(linear_algebra_1.Vec3.sub(linear_algebra_1.Vec3.zero(), bbox.max, bbox.min)),
                transform: mol_util_1.ValueCell.create(transform),
            };
        }
    }
    DirectVolume.create = create;
    function createEmpty(directVolume) {
        return {}; // TODO
    }
    DirectVolume.createEmpty = createEmpty;
    DirectVolume.Params = __assign({}, base_1.BaseGeometry.Params, { isoValueNorm: param_definition_1.ParamDefinition.Numeric(0.22, { min: 0, max: 1, step: 0.01 }, { description: 'Normalized Isolevel Value' }), renderMode: param_definition_1.ParamDefinition.Select('volume', RenderModeOptions), controlPoints: param_definition_1.ParamDefinition.LineGraph([
            linear_algebra_1.Vec2.create(0.19, 0.0), linear_algebra_1.Vec2.create(0.2, 0.05), linear_algebra_1.Vec2.create(0.25, 0.05), linear_algebra_1.Vec2.create(0.26, 0.0),
            linear_algebra_1.Vec2.create(0.79, 0.0), linear_algebra_1.Vec2.create(0.8, 0.05), linear_algebra_1.Vec2.create(0.85, 0.05), linear_algebra_1.Vec2.create(0.86, 0.0),
        ]), list: param_definition_1.ParamDefinition.ColorScale('RedYellowBlue', scale_1.ColorListOptions) });
    DirectVolume.Utils = {
        Params: DirectVolume.Params,
        createEmpty: createEmpty,
        createValues: createValues,
        createValuesSimple: createValuesSimple,
        updateValues: updateValues,
        updateBoundingSphere: updateBoundingSphere,
        createRenderableState: createRenderableState,
        updateRenderableState: updateRenderableState
    };
    function createValues(directVolume, transform, locationIt, theme, props) {
        var gridTexture = directVolume.gridTexture, gridTextureDim = directVolume.gridTextureDim;
        var bboxSize = directVolume.bboxSize, bboxMin = directVolume.bboxMin, bboxMax = directVolume.bboxMax, gridDimension = directVolume.gridDimension, gridTransform = directVolume.transform;
        var instanceCount = locationIt.instanceCount, groupCount = locationIt.groupCount;
        var color = color_data_1.createColors(locationIt, theme.color);
        var marker = marker_data_1.createMarkers(instanceCount * groupCount);
        var overpaint = overpaint_data_1.createEmptyOverpaint();
        var transparency = transparency_data_1.createEmptyTransparency();
        var counts = { drawCount: VolumeBox.indices.length, groupCount: groupCount, instanceCount: instanceCount };
        var _a = getBoundingSphere(gridDimension.ref.value, gridTransform.ref.value, transform.aTransform.ref.value, transform.instanceCount.ref.value), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        var controlPoints = transfer_function_1.getControlPointsFromVec2Array(props.controlPoints);
        var transferTex = transfer_function_1.createTransferFunctionTexture(controlPoints, props.list);
        var maxSteps = Math.ceil(linear_algebra_1.Vec3.magnitude(gridDimension.ref.value)) * 2 * 5;
        return __assign({}, color, marker, overpaint, transparency, transform, base_1.BaseGeometry.createValues(props, counts), { aPosition: mol_util_1.ValueCell.create(VolumeBox.vertices), elements: mol_util_1.ValueCell.create(VolumeBox.indices), boundingSphere: mol_util_1.ValueCell.create(boundingSphere), invariantBoundingSphere: mol_util_1.ValueCell.create(invariantBoundingSphere), uIsoValue: mol_util_1.ValueCell.create(props.isoValueNorm), uBboxMin: bboxMin, uBboxMax: bboxMax, uBboxSize: bboxSize, dMaxSteps: mol_util_1.ValueCell.create(maxSteps), uTransform: gridTransform, uGridDim: gridDimension, dRenderMode: mol_util_1.ValueCell.create(props.renderMode), tTransferTex: transferTex, dGridTexType: mol_util_1.ValueCell.create(gridTexture.ref.value.depth > 0 ? '3d' : '2d'), uGridTexDim: gridTextureDim, tGridTex: gridTexture });
    }
    function createValuesSimple(directVolume, props, colorValue, sizeValue, transform) {
        var s = base_1.BaseGeometry.createSimple(colorValue, sizeValue, transform);
        var p = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(DirectVolume.Params), props);
        return createValues(directVolume, s.transform, s.locationIterator, s.theme, p);
    }
    function updateValues(values, props) {
        mol_util_1.ValueCell.updateIfChanged(values.uIsoValue, props.isoValueNorm);
        mol_util_1.ValueCell.updateIfChanged(values.uAlpha, props.alpha);
        mol_util_1.ValueCell.updateIfChanged(values.dUseFog, props.useFog);
        mol_util_1.ValueCell.updateIfChanged(values.dRenderMode, props.renderMode);
        var controlPoints = transfer_function_1.getControlPointsFromVec2Array(props.controlPoints);
        transfer_function_1.createTransferFunctionTexture(controlPoints, props.list, values.tTransferTex);
    }
    function updateBoundingSphere(values, directVolume) {
        var _a = getBoundingSphere(values.uGridDim.ref.value, values.uTransform.ref.value, values.aTransform.ref.value, values.instanceCount.ref.value), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        if (!geometry_1.Sphere3D.equals(boundingSphere, values.boundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.boundingSphere, boundingSphere);
        }
        if (!geometry_1.Sphere3D.equals(invariantBoundingSphere, values.invariantBoundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.invariantBoundingSphere, invariantBoundingSphere);
        }
    }
    function createRenderableState(props) {
        var state = base_1.BaseGeometry.createRenderableState(props);
        state.opaque = false;
        return state;
    }
    function updateRenderableState(state, props) {
        base_1.BaseGeometry.updateRenderableState(state, props);
        state.opaque = false;
    }
})(DirectVolume = exports.DirectVolume || (exports.DirectVolume = {}));
//
var mTmp = linear_algebra_1.Mat4.identity();
var mTmp2 = linear_algebra_1.Mat4.identity();
var vHalfUnit = linear_algebra_1.Vec3.create(0.5, 0.5, 0.5);
var tmpVertices = new Float32Array(VolumeBox.vertices.length);
function getBoundingSphere(gridDimension, gridTransform, transform, transformCount) {
    tmpVertices.set(VolumeBox.vertices);
    linear_algebra_1.Mat4.fromTranslation(mTmp, vHalfUnit);
    linear_algebra_1.Mat4.mul(mTmp, linear_algebra_1.Mat4.fromScaling(mTmp2, gridDimension), mTmp);
    linear_algebra_1.Mat4.mul(mTmp, gridTransform, mTmp);
    util_1.transformPositionArray(mTmp, tmpVertices, 0, tmpVertices.length / 3);
    return util_2.calculateBoundingSphere(tmpVertices, tmpVertices.length / 3, transform, transformCount);
}
