/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Mesh } from './mesh/mesh';
import { Points } from './points/points';
import { Text } from './text/text';
import { RenderableState } from '../../mol-gl/renderable';
import { LocationIterator } from '../util/location-iterator';
import { ColorType } from './color-data';
import { SizeType } from './size-data';
import { Lines } from './lines/lines';
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { DirectVolume } from './direct-volume/direct-volume';
import { Color } from '../../mol-util/color';
import { Spheres } from './spheres/spheres';
import { TransformData } from './transform-data';
import { Theme } from '../../mol-theme/theme';
import { RenderObjectValuesType } from '../../mol-gl/render-object';
import { ValueOf } from '../../mol-util/type-helpers';
import { TextureMesh } from './texture-mesh/texture-mesh';
export declare type GeometryKindType = {
    'mesh': Mesh;
    'points': Points;
    'spheres': Spheres;
    'text': Text;
    'lines': Lines;
    'direct-volume': DirectVolume;
    'texture-mesh': TextureMesh;
};
export declare type GeometryKindParams = {
    'mesh': Mesh.Params;
    'points': Points.Params;
    'spheres': Spheres.Params;
    'text': Text.Params;
    'lines': Lines.Params;
    'direct-volume': DirectVolume.Params;
    'texture-mesh': TextureMesh.Params;
};
export declare type GeometryKind = keyof GeometryKindType;
export declare type Geometry = ValueOf<GeometryKindType>;
export interface GeometryUtils<G extends Geometry, P extends PD.Params = GeometryKindParams[G['kind']], V = RenderObjectValuesType[G['kind']]> {
    Params: P;
    createEmpty(geometry?: G): G;
    createValues(geometry: G, transform: TransformData, locationIt: LocationIterator, theme: Theme, props: PD.Values<P>): V;
    createValuesSimple(geometry: G, props: Partial<PD.Values<P>>, colorValue: Color, sizeValue: number, transform?: TransformData): V;
    updateValues(values: V, props: PD.Values<P>): void;
    updateBoundingSphere(values: V, geometry: G): void;
    createRenderableState(props: Partial<PD.Values<P>>): RenderableState;
    updateRenderableState(state: RenderableState, props: PD.Values<P>): void;
}
export declare namespace Geometry {
    type Params<G extends Geometry> = GeometryKindParams[G['kind']];
    function getDrawCount(geometry: Geometry): number;
    function getGroupCount(geometry: Geometry): number;
    function getUtils<G extends Geometry>(geometry: G): GeometryUtils<G>;
    function getGranularity(locationIt: LocationIterator, granularity: ColorType | SizeType): ColorType;
}
