"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var mesh_1 = require("./mesh/mesh");
var points_1 = require("./points/points");
var text_1 = require("./text/text");
var lines_1 = require("./lines/lines");
var direct_volume_1 = require("./direct-volume/direct-volume");
var spheres_1 = require("./spheres/spheres");
var array_1 = require("../../mol-util/array");
var texture_mesh_1 = require("./texture-mesh/texture-mesh");
var Geometry;
(function (Geometry) {
    function getDrawCount(geometry) {
        switch (geometry.kind) {
            case 'mesh': return geometry.triangleCount * 3;
            case 'points': return geometry.pointCount;
            case 'spheres': return geometry.sphereCount * 2 * 3;
            case 'text': return geometry.charCount * 2 * 3;
            case 'lines': return geometry.lineCount * 2 * 3;
            case 'direct-volume': return 12 * 3;
            case 'texture-mesh': return geometry.vertexCount.ref.value;
        }
    }
    Geometry.getDrawCount = getDrawCount;
    function getGroupCount(geometry) {
        switch (geometry.kind) {
            case 'mesh':
            case 'points':
            case 'spheres':
            case 'text':
            case 'lines':
                return getDrawCount(geometry) === 0 ? 0 : (array_1.arrayMax(geometry.groupBuffer.ref.value) + 1);
            case 'direct-volume':
                return 1;
            case 'texture-mesh':
                return geometry.groupCount.ref.value;
        }
    }
    Geometry.getGroupCount = getGroupCount;
    function getUtils(geometry) {
        // TODO avoid casting
        switch (geometry.kind) {
            case 'mesh': return mesh_1.Mesh.Utils;
            case 'points': return points_1.Points.Utils;
            case 'spheres': return spheres_1.Spheres.Utils;
            case 'text': return text_1.Text.Utils;
            case 'lines': return lines_1.Lines.Utils;
            case 'direct-volume': return direct_volume_1.DirectVolume.Utils;
            case 'texture-mesh': return texture_mesh_1.TextureMesh.Utils;
        }
        throw new Error('unknown geometry kind');
    }
    Geometry.getUtils = getUtils;
    function getGranularity(locationIt, granularity) {
        // Always use 'group' granularity for 'complex' location iterators,
        // i.e. for which an instance may include multiple units
        return granularity === 'instance' && locationIt.isComplex ? 'group' : granularity;
    }
    Geometry.getGranularity = getGranularity;
})(Geometry = exports.Geometry || (exports.Geometry = {}));
