"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var value_cell_1 = require("../../../mol-util/value-cell");
var util_1 = require("../../../mol-data/util");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var tmpVecA = linear_algebra_1.Vec3.zero();
var tmpVecB = linear_algebra_1.Vec3.zero();
var LinesBuilder;
(function (LinesBuilder) {
    function create(initialCount, chunkSize, lines) {
        if (initialCount === void 0) { initialCount = 2048; }
        if (chunkSize === void 0) { chunkSize = 1024; }
        var mappings = util_1.ChunkedArray.create(Float32Array, 2, chunkSize, lines ? lines.mappingBuffer.ref.value : initialCount);
        var groups = util_1.ChunkedArray.create(Float32Array, 1, chunkSize, lines ? lines.groupBuffer.ref.value : initialCount);
        var indices = util_1.ChunkedArray.create(Uint32Array, 3, chunkSize * 3, lines ? lines.indexBuffer.ref.value : initialCount * 3);
        var starts = util_1.ChunkedArray.create(Float32Array, 3, chunkSize, lines ? lines.startBuffer.ref.value : initialCount);
        var ends = util_1.ChunkedArray.create(Float32Array, 3, chunkSize, lines ? lines.endBuffer.ref.value : initialCount);
        var add = function (startX, startY, startZ, endX, endY, endZ, group) {
            var offset = mappings.elementCount;
            for (var i = 0; i < 4; ++i) {
                util_1.ChunkedArray.add3(starts, startX, startY, startZ);
                util_1.ChunkedArray.add3(ends, endX, endY, endZ);
                util_1.ChunkedArray.add(groups, group);
            }
            util_1.ChunkedArray.add2(mappings, -1, 1);
            util_1.ChunkedArray.add2(mappings, -1, -1);
            util_1.ChunkedArray.add2(mappings, 1, 1);
            util_1.ChunkedArray.add2(mappings, 1, -1);
            util_1.ChunkedArray.add3(indices, offset, offset + 1, offset + 2);
            util_1.ChunkedArray.add3(indices, offset + 1, offset + 3, offset + 2);
        };
        return {
            add: add,
            addCage: function (t, cage, group) {
                var vertices = cage.vertices, edges = cage.edges;
                for (var i = 0, il = edges.length; i < il; i += 2) {
                    linear_algebra_1.Vec3.fromArray(tmpVecA, vertices, edges[i] * 3);
                    linear_algebra_1.Vec3.fromArray(tmpVecB, vertices, edges[i + 1] * 3);
                    linear_algebra_1.Vec3.transformMat4(tmpVecA, tmpVecA, t);
                    linear_algebra_1.Vec3.transformMat4(tmpVecB, tmpVecB, t);
                    add(tmpVecA[0], tmpVecA[1], tmpVecA[2], tmpVecB[0], tmpVecB[1], tmpVecB[2], group);
                }
            },
            getLines: function () {
                var mb = util_1.ChunkedArray.compact(mappings, true);
                var ib = util_1.ChunkedArray.compact(indices, true);
                var gb = util_1.ChunkedArray.compact(groups, true);
                var sb = util_1.ChunkedArray.compact(starts, true);
                var eb = util_1.ChunkedArray.compact(ends, true);
                return {
                    kind: 'lines',
                    lineCount: indices.elementCount / 2,
                    mappingBuffer: lines ? value_cell_1.ValueCell.update(lines.mappingBuffer, mb) : value_cell_1.ValueCell.create(mb),
                    indexBuffer: lines ? value_cell_1.ValueCell.update(lines.indexBuffer, ib) : value_cell_1.ValueCell.create(ib),
                    groupBuffer: lines ? value_cell_1.ValueCell.update(lines.groupBuffer, gb) : value_cell_1.ValueCell.create(gb),
                    startBuffer: lines ? value_cell_1.ValueCell.update(lines.startBuffer, sb) : value_cell_1.ValueCell.create(sb),
                    endBuffer: lines ? value_cell_1.ValueCell.update(lines.endBuffer, eb) : value_cell_1.ValueCell.create(eb),
                };
            }
        };
    }
    LinesBuilder.create = create;
})(LinesBuilder = exports.LinesBuilder || (exports.LinesBuilder = {}));
