/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ValueCell } from '../../../mol-util';
import { Mat4 } from '../../../mol-math/linear-algebra';
import { GeometryUtils } from '../geometry';
import { Mesh } from '../mesh/mesh';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
/** Wide line */
export interface Lines {
    readonly kind: 'lines';
    /** Number of lines */
    lineCount: number;
    /** Mapping buffer as array of xy values wrapped in a value cell */
    readonly mappingBuffer: ValueCell<Float32Array>;
    /** Index buffer as array of vertex index triplets wrapped in a value cell */
    readonly indexBuffer: ValueCell<Uint32Array>;
    /** Group buffer as array of group ids for each vertex wrapped in a value cell */
    readonly groupBuffer: ValueCell<Float32Array>;
    /** Line start buffer as array of xyz values wrapped in a value cell */
    readonly startBuffer: ValueCell<Float32Array>;
    /** Line end buffer as array of xyz values wrapped in a value cell */
    readonly endBuffer: ValueCell<Float32Array>;
}
export declare namespace Lines {
    function createEmpty(lines?: Lines): Lines;
    function fromMesh(mesh: Mesh, lines?: Lines): Lines;
    function transformImmediate(line: Lines, t: Mat4): void;
    function transformRangeImmediate(lines: Lines, t: Mat4, offset: number, count: number): void;
    const Params: {
        sizeFactor: PD.Numeric;
        lineSizeAttenuation: PD.BooleanParam;
        alpha: PD.Numeric;
        useFog: PD.BooleanParam; /** Index buffer as array of vertex index triplets wrapped in a value cell */
        highlightColor: PD.Color;
        selectColor: PD.Color;
        quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    };
    type Params = typeof Params;
    const Utils: GeometryUtils<Lines, Params>;
}
