"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../../mol-util");
var util_1 = require("../../util");
var color_data_1 = require("../color-data");
var marker_data_1 = require("../marker-data");
var size_data_1 = require("../size-data");
var lines_builder_1 = require("./lines-builder");
var param_definition_1 = require("../../../mol-util/param-definition");
var util_2 = require("../../../mol-gl/renderable/util");
var geometry_1 = require("../../../mol-math/geometry");
var base_1 = require("../base");
var overpaint_data_1 = require("../overpaint-data");
var transparency_data_1 = require("../transparency-data");
var Lines;
(function (Lines) {
    function createEmpty(lines) {
        var mb = lines ? lines.mappingBuffer.ref.value : new Float32Array(0);
        var ib = lines ? lines.indexBuffer.ref.value : new Uint32Array(0);
        var gb = lines ? lines.groupBuffer.ref.value : new Float32Array(0);
        var sb = lines ? lines.startBuffer.ref.value : new Float32Array(0);
        var eb = lines ? lines.endBuffer.ref.value : new Float32Array(0);
        return {
            kind: 'lines',
            lineCount: 0,
            mappingBuffer: lines ? mol_util_1.ValueCell.update(lines.mappingBuffer, mb) : mol_util_1.ValueCell.create(mb),
            indexBuffer: lines ? mol_util_1.ValueCell.update(lines.indexBuffer, ib) : mol_util_1.ValueCell.create(ib),
            groupBuffer: lines ? mol_util_1.ValueCell.update(lines.groupBuffer, gb) : mol_util_1.ValueCell.create(gb),
            startBuffer: lines ? mol_util_1.ValueCell.update(lines.startBuffer, sb) : mol_util_1.ValueCell.create(sb),
            endBuffer: lines ? mol_util_1.ValueCell.update(lines.endBuffer, eb) : mol_util_1.ValueCell.create(eb),
        };
    }
    Lines.createEmpty = createEmpty;
    function fromMesh(mesh, lines) {
        var vb = mesh.vertexBuffer.ref.value;
        var ib = mesh.indexBuffer.ref.value;
        var gb = mesh.groupBuffer.ref.value;
        var builder = lines_builder_1.LinesBuilder.create(mesh.triangleCount * 3, mesh.triangleCount / 10, lines);
        // TODO avoid duplicate lines
        for (var i = 0, il = mesh.triangleCount * 3; i < il; i += 3) {
            var i0 = ib[i], i1 = ib[i + 1], i2 = ib[i + 2];
            var x0 = vb[i0 * 3], y0 = vb[i0 * 3 + 1], z0 = vb[i0 * 3 + 2];
            var x1 = vb[i1 * 3], y1 = vb[i1 * 3 + 1], z1 = vb[i1 * 3 + 2];
            var x2 = vb[i2 * 3], y2 = vb[i2 * 3 + 1], z2 = vb[i2 * 3 + 2];
            builder.add(x0, y0, z0, x1, y1, z1, gb[i0]);
            builder.add(x0, y0, z0, x2, y2, z2, gb[i0]);
            builder.add(x1, y1, z1, x2, y2, z2, gb[i1]);
        }
        return builder.getLines();
    }
    Lines.fromMesh = fromMesh;
    function transformImmediate(line, t) {
        transformRangeImmediate(line, t, 0, line.lineCount);
    }
    Lines.transformImmediate = transformImmediate;
    function transformRangeImmediate(lines, t, offset, count) {
        var start = lines.startBuffer.ref.value;
        util_1.transformPositionArray(t, start, offset, count * 4);
        mol_util_1.ValueCell.update(lines.startBuffer, start);
        var end = lines.endBuffer.ref.value;
        util_1.transformPositionArray(t, end, offset, count * 4);
        mol_util_1.ValueCell.update(lines.endBuffer, end);
    }
    Lines.transformRangeImmediate = transformRangeImmediate;
    //
    Lines.Params = __assign({}, base_1.BaseGeometry.Params, { sizeFactor: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 10, step: 0.1 }), lineSizeAttenuation: param_definition_1.ParamDefinition.Boolean(false) });
    Lines.Utils = {
        Params: Lines.Params,
        createEmpty: createEmpty,
        createValues: createValues,
        createValuesSimple: createValuesSimple,
        updateValues: updateValues,
        updateBoundingSphere: updateBoundingSphere,
        createRenderableState: base_1.BaseGeometry.createRenderableState,
        updateRenderableState: base_1.BaseGeometry.updateRenderableState
    };
    function createValues(lines, transform, locationIt, theme, props) {
        var instanceCount = locationIt.instanceCount, groupCount = locationIt.groupCount;
        var color = color_data_1.createColors(locationIt, theme.color);
        var size = size_data_1.createSizes(locationIt, theme.size);
        var marker = marker_data_1.createMarkers(instanceCount * groupCount);
        var overpaint = overpaint_data_1.createEmptyOverpaint();
        var transparency = transparency_data_1.createEmptyTransparency();
        var counts = { drawCount: lines.lineCount * 2 * 3, groupCount: groupCount, instanceCount: instanceCount };
        var _a = getBoundingSphere(lines.startBuffer.ref.value, lines.endBuffer.ref.value, lines.lineCount, transform.aTransform.ref.value, transform.instanceCount.ref.value), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        return __assign({ aMapping: lines.mappingBuffer, aGroup: lines.groupBuffer, aStart: lines.startBuffer, aEnd: lines.endBuffer, elements: lines.indexBuffer, boundingSphere: mol_util_1.ValueCell.create(boundingSphere), invariantBoundingSphere: mol_util_1.ValueCell.create(invariantBoundingSphere) }, color, size, marker, overpaint, transparency, transform, base_1.BaseGeometry.createValues(props, counts), { uSizeFactor: mol_util_1.ValueCell.create(props.sizeFactor), dLineSizeAttenuation: mol_util_1.ValueCell.create(props.lineSizeAttenuation), dDoubleSided: mol_util_1.ValueCell.create(true), dFlipSided: mol_util_1.ValueCell.create(false) });
    }
    function createValuesSimple(lines, props, colorValue, sizeValue, transform) {
        var s = base_1.BaseGeometry.createSimple(colorValue, sizeValue, transform);
        var p = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(Lines.Params), props);
        return createValues(lines, s.transform, s.locationIterator, s.theme, p);
    }
    function updateValues(values, props) {
        base_1.BaseGeometry.updateValues(values, props);
        mol_util_1.ValueCell.updateIfChanged(values.uSizeFactor, props.sizeFactor);
        mol_util_1.ValueCell.updateIfChanged(values.dLineSizeAttenuation, props.lineSizeAttenuation);
    }
    function updateBoundingSphere(values, lines) {
        var _a = getBoundingSphere(values.aStart.ref.value, values.aEnd.ref.value, lines.lineCount, values.aTransform.ref.value, values.instanceCount.ref.value), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        if (!geometry_1.Sphere3D.equals(boundingSphere, values.boundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.boundingSphere, boundingSphere);
        }
        if (!geometry_1.Sphere3D.equals(invariantBoundingSphere, values.invariantBoundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.invariantBoundingSphere, invariantBoundingSphere);
        }
    }
})(Lines = exports.Lines || (exports.Lines = {}));
function getBoundingSphere(lineStart, lineEnd, lineCount, transform, transformCount) {
    var start = util_2.calculateBoundingSphere(lineStart, lineCount * 4, transform, transformCount);
    var end = util_2.calculateBoundingSphere(lineEnd, lineCount * 4, transform, transformCount);
    return {
        boundingSphere: geometry_1.Sphere3D.expandBySphere(start.boundingSphere, end.boundingSphere),
        invariantBoundingSphere: geometry_1.Sphere3D.expandBySphere(start.invariantBoundingSphere, end.invariantBoundingSphere)
    };
}
