"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../../../mol-math/linear-algebra");
var mesh_builder_1 = require("../mesh-builder");
var cylinder_1 = require("../../../primitive/cylinder");
var cylinderMap = new Map();
var up = linear_algebra_1.Vec3.create(0, 1, 0);
var tmpCylinderDir = linear_algebra_1.Vec3.zero();
var tmpCylinderMatDir = linear_algebra_1.Vec3.zero();
var tmpCylinderCenter = linear_algebra_1.Vec3.zero();
var tmpCylinderMat = linear_algebra_1.Mat4.zero();
var tmpCylinderStart = linear_algebra_1.Vec3.zero();
var tmpUp = linear_algebra_1.Vec3.zero();
function setCylinderMat(m, start, dir, length) {
    linear_algebra_1.Vec3.setMagnitude(tmpCylinderMatDir, dir, length / 2);
    linear_algebra_1.Vec3.add(tmpCylinderCenter, start, tmpCylinderMatDir);
    // ensure the direction used to create the rotation is always pointing in the same
    // direction so the triangles of adjacent cylinder will line up
    linear_algebra_1.Vec3.copy(tmpUp, up);
    if (linear_algebra_1.Vec3.dot(tmpCylinderMatDir, tmpUp) < 0)
        linear_algebra_1.Vec3.scale(tmpUp, tmpUp, -1);
    linear_algebra_1.Vec3.makeRotation(m, tmpUp, tmpCylinderMatDir);
    return linear_algebra_1.Mat4.setTranslation(m, tmpCylinderCenter);
}
function getCylinder(props) {
    var key = JSON.stringify(props);
    var cylinder = cylinderMap.get(key);
    if (cylinder === undefined) {
        cylinder = cylinder_1.Cylinder(props);
        cylinderMap.set(key, cylinder);
    }
    return cylinder;
}
function addCylinder(state, start, end, lengthScale, props) {
    var d = linear_algebra_1.Vec3.distance(start, end) * lengthScale;
    props.height = d;
    linear_algebra_1.Vec3.sub(tmpCylinderDir, end, start);
    setCylinderMat(tmpCylinderMat, start, tmpCylinderDir, d);
    mesh_builder_1.MeshBuilder.addPrimitive(state, tmpCylinderMat, getCylinder(props));
}
exports.addCylinder = addCylinder;
function addDoubleCylinder(state, start, end, lengthScale, shift, props) {
    var d = linear_algebra_1.Vec3.distance(start, end) * lengthScale;
    props.height = d;
    var cylinder = getCylinder(props);
    linear_algebra_1.Vec3.sub(tmpCylinderDir, end, start);
    // positivly shifted cylinder
    linear_algebra_1.Vec3.add(tmpCylinderStart, start, shift);
    setCylinderMat(tmpCylinderMat, tmpCylinderStart, tmpCylinderDir, d);
    mesh_builder_1.MeshBuilder.addPrimitive(state, tmpCylinderMat, cylinder);
    // negativly shifted cylinder
    linear_algebra_1.Vec3.sub(tmpCylinderStart, start, shift);
    setCylinderMat(tmpCylinderMat, tmpCylinderStart, tmpCylinderDir, d);
    mesh_builder_1.MeshBuilder.addPrimitive(state, tmpCylinderMat, cylinder);
}
exports.addDoubleCylinder = addDoubleCylinder;
function addFixedCountDashedCylinder(state, start, end, lengthScale, segmentCount, props) {
    var s = Math.floor(segmentCount / 2);
    var step = 1 / segmentCount;
    // automatically adjust length so links/bonds that are rendered as two half cylinders
    // have evenly spaced dashed cylinders
    if (lengthScale < 1) {
        var bias = lengthScale / 2 / segmentCount;
        lengthScale += segmentCount % 2 === 1 ? bias : -bias;
    }
    var d = linear_algebra_1.Vec3.distance(start, end) * lengthScale;
    props.height = d * step;
    var cylinder = getCylinder(props);
    linear_algebra_1.Vec3.sub(tmpCylinderDir, end, start);
    for (var j = 0; j < s; ++j) {
        var f = step * (j * 2 + 1);
        linear_algebra_1.Vec3.setMagnitude(tmpCylinderDir, tmpCylinderDir, d * f);
        linear_algebra_1.Vec3.add(tmpCylinderStart, start, tmpCylinderDir);
        setCylinderMat(tmpCylinderMat, tmpCylinderStart, tmpCylinderDir, d * step);
        mesh_builder_1.MeshBuilder.addPrimitive(state, tmpCylinderMat, cylinder);
    }
}
exports.addFixedCountDashedCylinder = addFixedCountDashedCylinder;
