"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../../../mol-math/linear-algebra");
var util_1 = require("../../../../mol-data/util");
var tA = linear_algebra_1.Vec3.zero();
var tB = linear_algebra_1.Vec3.zero();
var tV = linear_algebra_1.Vec3.zero();
var horizontalVector = linear_algebra_1.Vec3.zero();
var verticalVector = linear_algebra_1.Vec3.zero();
var verticalRightVector = linear_algebra_1.Vec3.zero();
var verticalLeftVector = linear_algebra_1.Vec3.zero();
var normalOffset = linear_algebra_1.Vec3.zero();
var positionVector = linear_algebra_1.Vec3.zero();
var normalVector = linear_algebra_1.Vec3.zero();
var torsionVector = linear_algebra_1.Vec3.zero();
var p1 = linear_algebra_1.Vec3.zero();
var p2 = linear_algebra_1.Vec3.zero();
var p3 = linear_algebra_1.Vec3.zero();
var p4 = linear_algebra_1.Vec3.zero();
function addCap(offset, state, controlPoints, normalVectors, binormalVectors, width, leftHeight, rightHeight) {
    var vertices = state.vertices, normals = state.normals, indices = state.indices;
    var vertexCount = vertices.elementCount;
    linear_algebra_1.Vec3.fromArray(verticalLeftVector, normalVectors, offset);
    linear_algebra_1.Vec3.scale(verticalLeftVector, verticalLeftVector, leftHeight);
    linear_algebra_1.Vec3.fromArray(verticalRightVector, normalVectors, offset);
    linear_algebra_1.Vec3.scale(verticalRightVector, verticalRightVector, rightHeight);
    linear_algebra_1.Vec3.fromArray(horizontalVector, binormalVectors, offset);
    linear_algebra_1.Vec3.scale(horizontalVector, horizontalVector, width);
    linear_algebra_1.Vec3.fromArray(positionVector, controlPoints, offset);
    linear_algebra_1.Vec3.add(p1, linear_algebra_1.Vec3.add(p1, positionVector, horizontalVector), verticalRightVector);
    linear_algebra_1.Vec3.sub(p2, linear_algebra_1.Vec3.add(p2, positionVector, horizontalVector), verticalLeftVector);
    linear_algebra_1.Vec3.sub(p3, linear_algebra_1.Vec3.sub(p3, positionVector, horizontalVector), verticalLeftVector);
    linear_algebra_1.Vec3.add(p4, linear_algebra_1.Vec3.sub(p4, positionVector, horizontalVector), verticalRightVector);
    if (leftHeight < rightHeight) {
        util_1.ChunkedArray.add3(vertices, p4[0], p4[1], p4[2]);
        util_1.ChunkedArray.add3(vertices, p3[0], p3[1], p3[2]);
        util_1.ChunkedArray.add3(vertices, p2[0], p2[1], p2[2]);
        util_1.ChunkedArray.add3(vertices, p1[0], p1[1], p1[2]);
        linear_algebra_1.Vec3.copy(verticalVector, verticalRightVector);
    }
    else {
        util_1.ChunkedArray.add3(vertices, p1[0], p1[1], p1[2]);
        util_1.ChunkedArray.add3(vertices, p2[0], p2[1], p2[2]);
        util_1.ChunkedArray.add3(vertices, p3[0], p3[1], p3[2]);
        util_1.ChunkedArray.add3(vertices, p4[0], p4[1], p4[2]);
        linear_algebra_1.Vec3.copy(verticalVector, verticalLeftVector);
    }
    linear_algebra_1.Vec3.cross(normalVector, horizontalVector, verticalVector);
    for (var i = 0; i < 4; ++i) {
        util_1.ChunkedArray.add3(normals, normalVector[0], normalVector[1], normalVector[2]);
    }
    util_1.ChunkedArray.add3(indices, vertexCount + 2, vertexCount + 1, vertexCount);
    util_1.ChunkedArray.add3(indices, vertexCount, vertexCount + 3, vertexCount + 2);
}
/** set arrowHeight = 0 for no arrow */
function addSheet(state, controlPoints, normalVectors, binormalVectors, linearSegments, width, height, arrowHeight, startCap, endCap) {
    var currentGroup = state.currentGroup, vertices = state.vertices, normals = state.normals, indices = state.indices, groups = state.groups;
    var vertexCount = vertices.elementCount;
    var offsetLength = 0;
    if (arrowHeight > 0) {
        linear_algebra_1.Vec3.fromArray(tA, controlPoints, 0);
        linear_algebra_1.Vec3.fromArray(tB, controlPoints, linearSegments * 3);
        offsetLength = arrowHeight / linear_algebra_1.Vec3.magnitude(linear_algebra_1.Vec3.sub(tV, tB, tA));
    }
    for (var i = 0; i <= linearSegments; ++i) {
        var actualHeight = arrowHeight === 0 ? height : arrowHeight * (1 - i / linearSegments);
        var i3 = i * 3;
        linear_algebra_1.Vec3.fromArray(verticalVector, normalVectors, i3);
        linear_algebra_1.Vec3.scale(verticalVector, verticalVector, actualHeight);
        linear_algebra_1.Vec3.fromArray(horizontalVector, binormalVectors, i3);
        linear_algebra_1.Vec3.scale(horizontalVector, horizontalVector, width);
        if (arrowHeight > 0) {
            linear_algebra_1.Vec3.fromArray(tA, normalVectors, i3);
            linear_algebra_1.Vec3.fromArray(tB, binormalVectors, i3);
            linear_algebra_1.Vec3.scale(normalOffset, linear_algebra_1.Vec3.cross(normalOffset, tA, tB), offsetLength);
        }
        linear_algebra_1.Vec3.fromArray(positionVector, controlPoints, i3);
        linear_algebra_1.Vec3.fromArray(normalVector, normalVectors, i3);
        linear_algebra_1.Vec3.fromArray(torsionVector, binormalVectors, i3);
        linear_algebra_1.Vec3.add(tA, linear_algebra_1.Vec3.add(tA, positionVector, horizontalVector), verticalVector);
        linear_algebra_1.Vec3.copy(tB, normalVector);
        util_1.ChunkedArray.add3(vertices, tA[0], tA[1], tA[2]);
        util_1.ChunkedArray.add3(normals, tB[0], tB[1], tB[2]);
        linear_algebra_1.Vec3.add(tA, linear_algebra_1.Vec3.sub(tA, positionVector, horizontalVector), verticalVector);
        util_1.ChunkedArray.add3(vertices, tA[0], tA[1], tA[2]);
        util_1.ChunkedArray.add3(normals, tB[0], tB[1], tB[2]);
        // Vec3.add(tA, Vec3.sub(tA, positionVector, horizontalVector), verticalVector) // reuse tA
        linear_algebra_1.Vec3.add(tB, linear_algebra_1.Vec3.negate(tB, torsionVector), normalOffset);
        util_1.ChunkedArray.add3(vertices, tA[0], tA[1], tA[2]);
        util_1.ChunkedArray.add3(normals, tB[0], tB[1], tB[2]);
        linear_algebra_1.Vec3.sub(tA, linear_algebra_1.Vec3.sub(tA, positionVector, horizontalVector), verticalVector);
        util_1.ChunkedArray.add3(vertices, tA[0], tA[1], tA[2]);
        util_1.ChunkedArray.add3(normals, tB[0], tB[1], tB[2]);
        // Vec3.sub(tA, Vec3.sub(tA, positionVector, horizontalVector), verticalVector) // reuse tA
        linear_algebra_1.Vec3.negate(tB, normalVector);
        util_1.ChunkedArray.add3(vertices, tA[0], tA[1], tA[2]);
        util_1.ChunkedArray.add3(normals, tB[0], tB[1], tB[2]);
        linear_algebra_1.Vec3.sub(tA, linear_algebra_1.Vec3.add(tA, positionVector, horizontalVector), verticalVector);
        util_1.ChunkedArray.add3(vertices, tA[0], tA[1], tA[2]);
        util_1.ChunkedArray.add3(normals, tB[0], tB[1], tB[2]);
        // Vec3.sub(tA, Vec3.add(tA, positionVector, horizontalVector), verticalVector) // reuse tA
        linear_algebra_1.Vec3.add(tB, torsionVector, normalOffset);
        util_1.ChunkedArray.add3(vertices, tA[0], tA[1], tA[2]);
        util_1.ChunkedArray.add3(normals, tB[0], tB[1], tB[2]);
        linear_algebra_1.Vec3.add(tA, linear_algebra_1.Vec3.add(tA, positionVector, horizontalVector), verticalVector);
        util_1.ChunkedArray.add3(vertices, tA[0], tA[1], tA[2]);
        util_1.ChunkedArray.add3(normals, tB[0], tB[1], tB[2]);
    }
    for (var i = 0; i < linearSegments; ++i) {
        for (var j = 0; j < 4; j++) {
            util_1.ChunkedArray.add3(indices, vertexCount + i * 8 + 2 * j, vertexCount + (i + 1) * 8 + 2 * j + 1, vertexCount + i * 8 + 2 * j + 1);
            util_1.ChunkedArray.add3(indices, vertexCount + i * 8 + 2 * j, vertexCount + (i + 1) * 8 + 2 * j, vertexCount + (i + 1) * 8 + 2 * j + 1);
        }
    }
    if (startCap) {
        var h = arrowHeight === 0 ? height : arrowHeight;
        addCap(0, state, controlPoints, normalVectors, binormalVectors, width, h, h);
    }
    else if (arrowHeight > 0) {
        addCap(0, state, controlPoints, normalVectors, binormalVectors, width, arrowHeight, -height);
        addCap(0, state, controlPoints, normalVectors, binormalVectors, width, -arrowHeight, height);
    }
    if (endCap && arrowHeight === 0) {
        addCap(linearSegments * 3, state, controlPoints, normalVectors, binormalVectors, width, height, height);
    }
    var addedVertexCount = (linearSegments + 1) * 8 +
        (startCap ? 4 : (arrowHeight > 0 ? 8 : 0)) +
        (endCap && arrowHeight === 0 ? 4 : 0);
    for (var i = 0, il = addedVertexCount; i < il; ++i)
        util_1.ChunkedArray.add(groups, currentGroup);
}
exports.addSheet = addSheet;
