"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../../../mol-math/linear-algebra");
var util_1 = require("../../../../mol-data/util");
var normalVector = linear_algebra_1.Vec3.zero();
var surfacePoint = linear_algebra_1.Vec3.zero();
var controlPoint = linear_algebra_1.Vec3.zero();
var u = linear_algebra_1.Vec3.zero();
var v = linear_algebra_1.Vec3.zero();
function add2AndScale2(out, a, b, sa, sb) {
    out[0] = (a[0] * sa) + (b[0] * sb);
    out[1] = (a[1] * sa) + (b[1] * sb);
    out[2] = (a[2] * sa) + (b[2] * sb);
}
function add3AndScale2(out, a, b, c, sa, sb) {
    out[0] = (a[0] * sa) + (b[0] * sb) + c[0];
    out[1] = (a[1] * sa) + (b[1] * sb) + c[1];
    out[2] = (a[2] * sa) + (b[2] * sb) + c[2];
}
function addTube(state, controlPoints, normalVectors, binormalVectors, linearSegments, radialSegments, widthValues, heightValues, waveFactor, startCap, endCap) {
    var currentGroup = state.currentGroup, vertices = state.vertices, normals = state.normals, indices = state.indices, groups = state.groups;
    var vertexCount = vertices.elementCount;
    var di = 1 / linearSegments;
    for (var i = 0; i <= linearSegments; ++i) {
        var i3 = i * 3;
        linear_algebra_1.Vec3.fromArray(u, normalVectors, i3);
        linear_algebra_1.Vec3.fromArray(v, binormalVectors, i3);
        linear_algebra_1.Vec3.fromArray(controlPoint, controlPoints, i3);
        var width = widthValues[i];
        var height = heightValues[i];
        var tt = di * i - 0.5;
        var ff = 1 + (waveFactor - 1) * (Math.cos(2 * Math.PI * tt) + 1);
        var w = ff * width, h = ff * height;
        for (var j = 0; j < radialSegments; ++j) {
            var t = 2 * Math.PI * j / radialSegments;
            add3AndScale2(surfacePoint, u, v, controlPoint, h * Math.cos(t), w * Math.sin(t));
            add2AndScale2(normalVector, u, v, w * Math.cos(t), h * Math.sin(t));
            linear_algebra_1.Vec3.normalize(normalVector, normalVector);
            util_1.ChunkedArray.add3(vertices, surfacePoint[0], surfacePoint[1], surfacePoint[2]);
            util_1.ChunkedArray.add3(normals, normalVector[0], normalVector[1], normalVector[2]);
        }
    }
    for (var i = 0; i < linearSegments; ++i) {
        for (var j = 0; j < radialSegments; ++j) {
            util_1.ChunkedArray.add3(indices, vertexCount + i * radialSegments + (j + 1) % radialSegments, vertexCount + (i + 1) * radialSegments + (j + 1) % radialSegments, vertexCount + i * radialSegments + j);
            util_1.ChunkedArray.add3(indices, vertexCount + (i + 1) * radialSegments + (j + 1) % radialSegments, vertexCount + (i + 1) * radialSegments + j, vertexCount + i * radialSegments + j);
        }
    }
    if (startCap) {
        var offset = 0;
        var centerVertex = vertices.elementCount;
        linear_algebra_1.Vec3.fromArray(u, normalVectors, offset);
        linear_algebra_1.Vec3.fromArray(v, binormalVectors, offset);
        linear_algebra_1.Vec3.fromArray(controlPoint, controlPoints, offset);
        linear_algebra_1.Vec3.cross(normalVector, v, u);
        util_1.ChunkedArray.add3(vertices, controlPoint[0], controlPoint[1], controlPoint[2]);
        util_1.ChunkedArray.add3(normals, normalVector[0], normalVector[1], normalVector[2]);
        var width = widthValues[0];
        var height = heightValues[0];
        vertexCount = vertices.elementCount;
        for (var i = 0; i < radialSegments; ++i) {
            var t = 2 * Math.PI * i / radialSegments;
            add3AndScale2(surfacePoint, u, v, controlPoint, height * Math.cos(t), width * Math.sin(t));
            util_1.ChunkedArray.add3(vertices, surfacePoint[0], surfacePoint[1], surfacePoint[2]);
            util_1.ChunkedArray.add3(normals, normalVector[0], normalVector[1], normalVector[2]);
            util_1.ChunkedArray.add3(indices, vertexCount + (i + 1) % radialSegments, vertexCount + i, centerVertex);
        }
    }
    if (endCap) {
        var offset = linearSegments * 3;
        var centerVertex = vertices.elementCount;
        linear_algebra_1.Vec3.fromArray(u, normalVectors, offset);
        linear_algebra_1.Vec3.fromArray(v, binormalVectors, offset);
        linear_algebra_1.Vec3.fromArray(controlPoint, controlPoints, offset);
        linear_algebra_1.Vec3.cross(normalVector, u, v);
        util_1.ChunkedArray.add3(vertices, controlPoint[0], controlPoint[1], controlPoint[2]);
        util_1.ChunkedArray.add3(normals, normalVector[0], normalVector[1], normalVector[2]);
        var width = widthValues[linearSegments];
        var height = heightValues[linearSegments];
        vertexCount = vertices.elementCount;
        for (var i = 0; i < radialSegments; ++i) {
            var t = 2 * Math.PI * i / radialSegments;
            add3AndScale2(surfacePoint, u, v, controlPoint, height * Math.cos(t), width * Math.sin(t));
            util_1.ChunkedArray.add3(vertices, surfacePoint[0], surfacePoint[1], surfacePoint[2]);
            util_1.ChunkedArray.add3(normals, normalVector[0], normalVector[1], normalVector[2]);
            util_1.ChunkedArray.add3(indices, vertexCount + i, vertexCount + (i + 1) % radialSegments, centerVertex);
        }
    }
    var addedVertexCount = (linearSegments + 1) * radialSegments + (startCap ? radialSegments + 1 : 0) + (endCap ? radialSegments + 1 : 0);
    util_1.ChunkedArray.addRepeat(groups, addedVertexCount, currentGroup);
}
exports.addTube = addTube;
