"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var value_cell_1 = require("../../../mol-util/value-cell");
var util_1 = require("../../../mol-data/util");
var PointsBuilder;
(function (PointsBuilder) {
    function create(initialCount, chunkSize, points) {
        if (initialCount === void 0) { initialCount = 2048; }
        if (chunkSize === void 0) { chunkSize = 1024; }
        var centers = util_1.ChunkedArray.create(Float32Array, 3, chunkSize, points ? points.centerBuffer.ref.value : initialCount);
        var groups = util_1.ChunkedArray.create(Float32Array, 1, chunkSize, points ? points.groupBuffer.ref.value : initialCount);
        return {
            add: function (x, y, z, group) {
                util_1.ChunkedArray.add3(centers, x, y, z);
                util_1.ChunkedArray.add(groups, group);
            },
            getPoints: function () {
                var cb = util_1.ChunkedArray.compact(centers, true);
                var gb = util_1.ChunkedArray.compact(groups, true);
                return {
                    kind: 'points',
                    pointCount: centers.elementCount,
                    centerBuffer: points ? value_cell_1.ValueCell.update(points.centerBuffer, cb) : value_cell_1.ValueCell.create(cb),
                    groupBuffer: points ? value_cell_1.ValueCell.update(points.groupBuffer, gb) : value_cell_1.ValueCell.create(gb),
                };
            }
        };
    }
    PointsBuilder.create = create;
})(PointsBuilder = exports.PointsBuilder || (exports.PointsBuilder = {}));
