/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ValueCell } from '../../../mol-util';
import { Mat4 } from '../../../mol-math/linear-algebra';
import { GeometryUtils } from '../geometry';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
/** Point cloud */
export interface Points {
    readonly kind: 'points';
    /** Number of vertices in the point cloud */
    pointCount: number;
    /** Center buffer as array of xyz values wrapped in a value cell */
    readonly centerBuffer: ValueCell<Float32Array>;
    /** Group buffer as array of group ids for each vertex wrapped in a value cell */
    readonly groupBuffer: ValueCell<Float32Array>;
}
export declare namespace Points {
    function createEmpty(points?: Points): Points;
    function transformImmediate(points: Points, t: Mat4): void;
    function transformRangeImmediate(points: Points, t: Mat4, offset: number, count: number): void;
    const Params: {
        sizeFactor: PD.Numeric;
        pointSizeAttenuation: PD.BooleanParam;
        pointFilledCircle: PD.BooleanParam;
        pointEdgeBleach: PD.Numeric;
        alpha: PD.Numeric;
        useFog: PD.BooleanParam;
        highlightColor: PD.Color;
        selectColor: PD.Color;
        quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    };
    type Params = typeof Params;
    const Utils: GeometryUtils<Points, Params>;
}
