"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../../mol-util");
var util_1 = require("../../util");
var color_data_1 = require("../color-data");
var marker_data_1 = require("../marker-data");
var size_data_1 = require("../size-data");
var param_definition_1 = require("../../../mol-util/param-definition");
var util_2 = require("../../../mol-gl/renderable/util");
var geometry_1 = require("../../../mol-math/geometry");
var base_1 = require("../base");
var overpaint_data_1 = require("../overpaint-data");
var transparency_data_1 = require("../transparency-data");
var Points;
(function (Points) {
    function createEmpty(points) {
        var cb = points ? points.centerBuffer.ref.value : new Float32Array(0);
        var gb = points ? points.groupBuffer.ref.value : new Float32Array(0);
        return {
            kind: 'points',
            pointCount: 0,
            centerBuffer: points ? mol_util_1.ValueCell.update(points.centerBuffer, cb) : mol_util_1.ValueCell.create(cb),
            groupBuffer: points ? mol_util_1.ValueCell.update(points.groupBuffer, gb) : mol_util_1.ValueCell.create(gb),
        };
    }
    Points.createEmpty = createEmpty;
    function transformImmediate(points, t) {
        transformRangeImmediate(points, t, 0, points.pointCount);
    }
    Points.transformImmediate = transformImmediate;
    function transformRangeImmediate(points, t, offset, count) {
        var c = points.centerBuffer.ref.value;
        util_1.transformPositionArray(t, c, offset, count);
        mol_util_1.ValueCell.update(points.centerBuffer, c);
    }
    Points.transformRangeImmediate = transformRangeImmediate;
    //
    Points.Params = __assign({}, base_1.BaseGeometry.Params, { sizeFactor: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 10, step: 0.1 }), pointSizeAttenuation: param_definition_1.ParamDefinition.Boolean(false), pointFilledCircle: param_definition_1.ParamDefinition.Boolean(false), pointEdgeBleach: param_definition_1.ParamDefinition.Numeric(0.2, { min: 0, max: 1, step: 0.05 }) });
    Points.Utils = {
        Params: Points.Params,
        createEmpty: createEmpty,
        createValues: createValues,
        createValuesSimple: createValuesSimple,
        updateValues: updateValues,
        updateBoundingSphere: updateBoundingSphere,
        createRenderableState: createRenderableState,
        updateRenderableState: updateRenderableState
    };
    function createValues(points, transform, locationIt, theme, props) {
        var instanceCount = locationIt.instanceCount, groupCount = locationIt.groupCount;
        var color = color_data_1.createColors(locationIt, theme.color);
        var size = size_data_1.createSizes(locationIt, theme.size);
        var marker = marker_data_1.createMarkers(instanceCount * groupCount);
        var overpaint = overpaint_data_1.createEmptyOverpaint();
        var transparency = transparency_data_1.createEmptyTransparency();
        var counts = { drawCount: points.pointCount, groupCount: groupCount, instanceCount: instanceCount };
        var _a = util_2.calculateBoundingSphere(points.centerBuffer.ref.value, points.pointCount, transform.aTransform.ref.value, transform.instanceCount.ref.value), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        return __assign({ aPosition: points.centerBuffer, aGroup: points.groupBuffer, boundingSphere: mol_util_1.ValueCell.create(boundingSphere), invariantBoundingSphere: mol_util_1.ValueCell.create(invariantBoundingSphere) }, color, size, marker, overpaint, transparency, transform, base_1.BaseGeometry.createValues(props, counts), { uSizeFactor: mol_util_1.ValueCell.create(props.sizeFactor), dPointSizeAttenuation: mol_util_1.ValueCell.create(props.pointSizeAttenuation), dPointFilledCircle: mol_util_1.ValueCell.create(props.pointFilledCircle), uPointEdgeBleach: mol_util_1.ValueCell.create(props.pointEdgeBleach) });
    }
    function createValuesSimple(points, props, colorValue, sizeValue, transform) {
        var s = base_1.BaseGeometry.createSimple(colorValue, sizeValue, transform);
        var p = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(Points.Params), props);
        return createValues(points, s.transform, s.locationIterator, s.theme, p);
    }
    function updateValues(values, props) {
        base_1.BaseGeometry.updateValues(values, props);
        mol_util_1.ValueCell.updateIfChanged(values.uSizeFactor, props.sizeFactor);
        mol_util_1.ValueCell.updateIfChanged(values.dPointSizeAttenuation, props.pointSizeAttenuation);
        mol_util_1.ValueCell.updateIfChanged(values.dPointFilledCircle, props.pointFilledCircle);
        mol_util_1.ValueCell.updateIfChanged(values.uPointEdgeBleach, props.pointEdgeBleach);
    }
    function updateBoundingSphere(values, points) {
        var _a = util_2.calculateBoundingSphere(values.aPosition.ref.value, points.pointCount, values.aTransform.ref.value, values.instanceCount.ref.value), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        if (!geometry_1.Sphere3D.equals(boundingSphere, values.boundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.boundingSphere, boundingSphere);
        }
        if (!geometry_1.Sphere3D.equals(invariantBoundingSphere, values.invariantBoundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.invariantBoundingSphere, invariantBoundingSphere);
        }
    }
    function createRenderableState(props) {
        var state = base_1.BaseGeometry.createRenderableState(props);
        updateRenderableState(state, props);
        return state;
    }
    function updateRenderableState(state, props) {
        base_1.BaseGeometry.updateRenderableState(state, props);
        state.opaque = state.opaque && (!props.pointFilledCircle ||
            (props.pointFilledCircle && props.pointEdgeBleach === 0));
    }
})(Points = exports.Points || (exports.Points = {}));
