"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var value_cell_1 = require("../../../mol-util/value-cell");
var util_1 = require("../../../mol-data/util");
var quadMapping = new Float32Array([
    -1.0, 1.0,
    -1.0, -1.0,
    1.0, 1.0,
    1.0, -1.0
]);
var quadIndices = new Uint16Array([
    0, 1, 2,
    1, 3, 2
]);
var SpheresBuilder;
(function (SpheresBuilder) {
    function create(initialCount, chunkSize, spheres) {
        if (initialCount === void 0) { initialCount = 2048; }
        if (chunkSize === void 0) { chunkSize = 1024; }
        initialCount *= 4;
        chunkSize *= 4;
        var centers = util_1.ChunkedArray.create(Float32Array, 3, chunkSize, spheres ? spheres.centerBuffer.ref.value : initialCount);
        var mappings = util_1.ChunkedArray.create(Float32Array, 2, chunkSize, spheres ? spheres.mappingBuffer.ref.value : initialCount);
        var indices = util_1.ChunkedArray.create(Uint32Array, 3, chunkSize / 2, spheres ? spheres.indexBuffer.ref.value : initialCount / 2);
        var groups = util_1.ChunkedArray.create(Float32Array, 1, chunkSize, spheres ? spheres.groupBuffer.ref.value : initialCount);
        return {
            add: function (x, y, z, group) {
                var offset = centers.elementCount;
                for (var i = 0; i < 4; ++i) {
                    util_1.ChunkedArray.add3(centers, x, y, z);
                    util_1.ChunkedArray.add2(mappings, quadMapping[i * 2], quadMapping[i * 2 + 1]);
                    util_1.ChunkedArray.add(groups, group);
                }
                util_1.ChunkedArray.add3(indices, offset + quadIndices[0], offset + quadIndices[1], offset + quadIndices[2]);
                util_1.ChunkedArray.add3(indices, offset + quadIndices[3], offset + quadIndices[4], offset + quadIndices[5]);
            },
            getSpheres: function () {
                var cb = util_1.ChunkedArray.compact(centers, true);
                var mb = util_1.ChunkedArray.compact(mappings, true);
                var ib = util_1.ChunkedArray.compact(indices, true);
                var gb = util_1.ChunkedArray.compact(groups, true);
                return {
                    kind: 'spheres',
                    sphereCount: centers.elementCount / 4,
                    centerBuffer: spheres ? value_cell_1.ValueCell.update(spheres.centerBuffer, cb) : value_cell_1.ValueCell.create(cb),
                    mappingBuffer: spheres ? value_cell_1.ValueCell.update(spheres.mappingBuffer, mb) : value_cell_1.ValueCell.create(mb),
                    indexBuffer: spheres ? value_cell_1.ValueCell.update(spheres.indexBuffer, ib) : value_cell_1.ValueCell.create(ib),
                    groupBuffer: spheres ? value_cell_1.ValueCell.update(spheres.groupBuffer, gb) : value_cell_1.ValueCell.create(gb),
                };
            }
        };
    }
    SpheresBuilder.create = create;
})(SpheresBuilder = exports.SpheresBuilder || (exports.SpheresBuilder = {}));
