/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ValueCell } from '../../../mol-util';
import { GeometryUtils } from '../geometry';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
/** Spheres */
export interface Spheres {
    readonly kind: 'spheres';
    /** Number of spheres */
    sphereCount: number;
    /** Center buffer as array of xyz values wrapped in a value cell */
    readonly centerBuffer: ValueCell<Float32Array>;
    /** Mapping buffer as array of xy values wrapped in a value cell */
    readonly mappingBuffer: ValueCell<Float32Array>;
    /** Index buffer as array of center index triplets wrapped in a value cell */
    readonly indexBuffer: ValueCell<Uint32Array>;
    /** Group buffer as array of group ids for each vertex wrapped in a value cell */
    readonly groupBuffer: ValueCell<Float32Array>;
}
export declare namespace Spheres {
    function createEmpty(spheres?: Spheres): Spheres;
    const Params: {
        sizeFactor: PD.Numeric;
        doubleSided: PD.BooleanParam;
        alpha: PD.Numeric;
        useFog: PD.BooleanParam;
        highlightColor: PD.Color;
        selectColor: PD.Color;
        quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    };
    type Params = typeof Params;
    const Utils: GeometryUtils<Spheres, Params>;
}
