"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../../mol-util");
var param_definition_1 = require("../../../mol-util/param-definition");
var color_data_1 = require("../color-data");
var marker_data_1 = require("../marker-data");
var util_1 = require("../../../mol-gl/renderable/util");
var geometry_1 = require("../../../mol-math/geometry");
var size_data_1 = require("../size-data");
var base_1 = require("../base");
var overpaint_data_1 = require("../overpaint-data");
var transparency_data_1 = require("../transparency-data");
var Spheres;
(function (Spheres) {
    function createEmpty(spheres) {
        var cb = spheres ? spheres.centerBuffer.ref.value : new Float32Array(0);
        var mb = spheres ? spheres.mappingBuffer.ref.value : new Float32Array(0);
        var ib = spheres ? spheres.indexBuffer.ref.value : new Uint32Array(0);
        var gb = spheres ? spheres.groupBuffer.ref.value : new Float32Array(0);
        return {
            kind: 'spheres',
            sphereCount: 0,
            centerBuffer: spheres ? mol_util_1.ValueCell.update(spheres.centerBuffer, cb) : mol_util_1.ValueCell.create(cb),
            mappingBuffer: spheres ? mol_util_1.ValueCell.update(spheres.mappingBuffer, mb) : mol_util_1.ValueCell.create(mb),
            indexBuffer: spheres ? mol_util_1.ValueCell.update(spheres.indexBuffer, ib) : mol_util_1.ValueCell.create(ib),
            groupBuffer: spheres ? mol_util_1.ValueCell.update(spheres.groupBuffer, gb) : mol_util_1.ValueCell.create(gb)
        };
    }
    Spheres.createEmpty = createEmpty;
    Spheres.Params = __assign({}, base_1.BaseGeometry.Params, { sizeFactor: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 10, step: 0.1 }), doubleSided: param_definition_1.ParamDefinition.Boolean(false) });
    Spheres.Utils = {
        Params: Spheres.Params,
        createEmpty: createEmpty,
        createValues: createValues,
        createValuesSimple: createValuesSimple,
        updateValues: updateValues,
        updateBoundingSphere: updateBoundingSphere,
        createRenderableState: base_1.BaseGeometry.createRenderableState,
        updateRenderableState: base_1.BaseGeometry.updateRenderableState
    };
    function createValues(spheres, transform, locationIt, theme, props) {
        var instanceCount = locationIt.instanceCount, groupCount = locationIt.groupCount;
        if (instanceCount !== transform.instanceCount.ref.value) {
            throw new Error('instanceCount values in TransformData and LocationIterator differ');
        }
        var color = color_data_1.createColors(locationIt, theme.color);
        var size = size_data_1.createSizes(locationIt, theme.size);
        var marker = marker_data_1.createMarkers(instanceCount * groupCount);
        var overpaint = overpaint_data_1.createEmptyOverpaint();
        var transparency = transparency_data_1.createEmptyTransparency();
        var counts = { drawCount: spheres.sphereCount * 2 * 3, groupCount: groupCount, instanceCount: instanceCount };
        var padding = size_data_1.getMaxSize(size);
        var _a = util_1.calculateBoundingSphere(spheres.centerBuffer.ref.value, spheres.sphereCount * 4, transform.aTransform.ref.value, instanceCount, padding), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        return __assign({ aPosition: spheres.centerBuffer, aMapping: spheres.mappingBuffer, aGroup: spheres.groupBuffer, elements: spheres.indexBuffer, boundingSphere: mol_util_1.ValueCell.create(boundingSphere), invariantBoundingSphere: mol_util_1.ValueCell.create(invariantBoundingSphere) }, color, size, marker, overpaint, transparency, transform, { padding: mol_util_1.ValueCell.create(padding) }, base_1.BaseGeometry.createValues(props, counts), { uSizeFactor: mol_util_1.ValueCell.create(props.sizeFactor), dDoubleSided: mol_util_1.ValueCell.create(props.doubleSided) });
    }
    function createValuesSimple(spheres, props, colorValue, sizeValue, transform) {
        var s = base_1.BaseGeometry.createSimple(colorValue, sizeValue, transform);
        var p = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(Spheres.Params), props);
        return createValues(spheres, s.transform, s.locationIterator, s.theme, p);
    }
    function updateValues(values, props) {
        base_1.BaseGeometry.updateValues(values, props);
        mol_util_1.ValueCell.updateIfChanged(values.uSizeFactor, props.sizeFactor);
        mol_util_1.ValueCell.updateIfChanged(values.dDoubleSided, props.doubleSided);
    }
    function updateBoundingSphere(values, spheres) {
        var padding = size_data_1.getMaxSize(values);
        var _a = util_1.calculateBoundingSphere(values.aPosition.ref.value, spheres.sphereCount * 4, values.aTransform.ref.value, values.instanceCount.ref.value, padding), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        if (!geometry_1.Sphere3D.equals(boundingSphere, values.boundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.boundingSphere, boundingSphere);
        }
        if (!geometry_1.Sphere3D.equals(invariantBoundingSphere, values.invariantBoundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.invariantBoundingSphere, invariantBoundingSphere);
        }
        mol_util_1.ValueCell.update(values.padding, padding);
    }
})(Spheres = exports.Spheres || (exports.Spheres = {}));
