"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var value_cell_1 = require("../../../mol-util/value-cell");
var util_1 = require("../../../mol-data/util");
var text_1 = require("./text");
var font_atlas_1 = require("./font-atlas");
var quadIndices = new Uint16Array([
    0, 1, 2,
    1, 3, 2
]);
var TextBuilder;
(function (TextBuilder) {
    function create(props, initialCount, chunkSize, text) {
        if (props === void 0) { props = {}; }
        if (initialCount === void 0) { initialCount = 2048; }
        if (chunkSize === void 0) { chunkSize = 1024; }
        initialCount *= 2;
        chunkSize *= 2;
        var centers = util_1.ChunkedArray.create(Float32Array, 3, chunkSize, text ? text.centerBuffer.ref.value : initialCount);
        var mappings = util_1.ChunkedArray.create(Float32Array, 2, chunkSize, text ? text.mappingBuffer.ref.value : initialCount);
        var depths = util_1.ChunkedArray.create(Float32Array, 1, chunkSize, text ? text.depthBuffer.ref.value : initialCount);
        var indices = util_1.ChunkedArray.create(Uint32Array, 3, chunkSize, text ? text.indexBuffer.ref.value : initialCount);
        var groups = util_1.ChunkedArray.create(Float32Array, 1, chunkSize, text ? text.groupBuffer.ref.value : initialCount);
        var tcoords = util_1.ChunkedArray.create(Float32Array, 2, chunkSize, text ? text.tcoordBuffer.ref.value : initialCount);
        var p = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(text_1.Text.Params), props);
        var attachment = p.attachment, background = p.background, backgroundMargin = p.backgroundMargin, tether = p.tether, tetherLength = p.tetherLength, tetherBaseWidth = p.tetherBaseWidth;
        var fontAtlas = font_atlas_1.getFontAtlas(p);
        var margin = (1 / 2.5) * backgroundMargin;
        var outline = fontAtlas.buffer / fontAtlas.lineHeight;
        var add = function (x, y, z, depth, group) {
            util_1.ChunkedArray.add3(centers, x, y, z);
            util_1.ChunkedArray.add(depths, depth);
            util_1.ChunkedArray.add(groups, group);
        };
        return {
            add: function (str, x, y, z, depth, group) {
                var bWidth = 0;
                var nChar = str.length;
                // calculate width
                for (var iChar = 0; iChar < nChar; ++iChar) {
                    var c = fontAtlas.get(str[iChar]);
                    bWidth += c.nw - 2 * outline;
                }
                var bHeight = 1 / 1.25;
                // attachment
                var yShift, xShift;
                // vertical
                if (attachment.startsWith('top')) {
                    yShift = bHeight;
                }
                else if (attachment.startsWith('middle')) {
                    yShift = bHeight / 2;
                }
                else {
                    yShift = 0; // "bottom"
                }
                // horizontal
                if (attachment.endsWith('right')) {
                    xShift = bWidth;
                }
                else if (attachment.endsWith('center')) {
                    xShift = bWidth / 2;
                }
                else {
                    xShift = 0; // "left"
                }
                if (tether) {
                    switch (attachment) {
                        case 'bottom-left':
                            xShift -= tetherLength / 2 + margin + 0.1;
                            yShift -= tetherLength / 2 + margin;
                            break;
                        case 'bottom-center':
                            yShift -= tetherLength + margin;
                            break;
                        case 'bottom-right':
                            xShift += tetherLength / 2 + margin + 0.1;
                            yShift -= tetherLength / 2 + margin;
                            break;
                        case 'middle-left':
                            xShift -= tetherLength + margin + 0.1;
                            break;
                        case 'middle-center':
                            break;
                        case 'middle-right':
                            xShift += tetherLength + margin + 0.1;
                            break;
                        case 'top-left':
                            xShift -= tetherLength / 2 + margin + 0.1;
                            yShift += tetherLength / 2 + margin;
                            break;
                        case 'top-center':
                            yShift += tetherLength + margin;
                            break;
                        case 'top-right':
                            xShift += tetherLength / 2 + margin + 0.1;
                            yShift += tetherLength / 2 + margin;
                            break;
                    }
                }
                var xLeft = -xShift - margin - 0.1;
                var xRight = bWidth - xShift + margin + 0.1;
                var yTop = bHeight - yShift + margin;
                var yBottom = -yShift - margin;
                // background
                if (background) {
                    util_1.ChunkedArray.add2(mappings, xLeft, yTop); // top left
                    util_1.ChunkedArray.add2(mappings, xLeft, yBottom); // bottom left
                    util_1.ChunkedArray.add2(mappings, xRight, yTop); // top right
                    util_1.ChunkedArray.add2(mappings, xRight, yBottom); // bottom right
                    var offset = centers.elementCount;
                    for (var i = 0; i < 4; ++i) {
                        util_1.ChunkedArray.add2(tcoords, 10, 10);
                        add(x, y, z, depth, group);
                    }
                    util_1.ChunkedArray.add3(indices, offset + quadIndices[0], offset + quadIndices[1], offset + quadIndices[2]);
                    util_1.ChunkedArray.add3(indices, offset + quadIndices[3], offset + quadIndices[4], offset + quadIndices[5]);
                }
                if (tether) {
                    var xTip = void 0, yTip = void 0;
                    var xBaseA = void 0, yBaseA = void 0;
                    var xBaseB = void 0, yBaseB = void 0;
                    var xBaseCenter = void 0, yBaseCenter = void 0;
                    switch (attachment) {
                        case 'bottom-left':
                            xTip = xLeft - tetherLength / 2;
                            xBaseA = xLeft + tetherBaseWidth / 2;
                            xBaseB = xLeft;
                            xBaseCenter = xLeft;
                            yTip = yBottom - tetherLength / 2;
                            yBaseA = yBottom;
                            yBaseB = yBottom + tetherBaseWidth / 2;
                            yBaseCenter = yBottom;
                            break;
                        case 'bottom-center':
                            xTip = 0;
                            xBaseA = tetherBaseWidth / 2;
                            xBaseB = -tetherBaseWidth / 2;
                            xBaseCenter = 0;
                            yTip = yBottom - tetherLength;
                            yBaseA = yBottom;
                            yBaseB = yBottom;
                            yBaseCenter = yBottom;
                            break;
                        case 'bottom-right':
                            xTip = xRight + tetherLength / 2;
                            xBaseA = xRight;
                            xBaseB = xRight - tetherBaseWidth / 2;
                            xBaseCenter = xRight;
                            yTip = yBottom - tetherLength / 2;
                            yBaseA = yBottom + tetherBaseWidth / 2;
                            yBaseB = yBottom;
                            yBaseCenter = yBottom;
                            break;
                        case 'middle-left':
                            xTip = xLeft - tetherLength;
                            xBaseA = xLeft;
                            xBaseB = xLeft;
                            xBaseCenter = xLeft;
                            yTip = 0;
                            yBaseA = -tetherBaseWidth / 2;
                            yBaseB = tetherBaseWidth / 2;
                            yBaseCenter = 0;
                            break;
                        case 'middle-center':
                            xTip = 0;
                            xBaseA = 0;
                            xBaseB = 0;
                            xBaseCenter = 0;
                            yTip = 0;
                            yBaseA = 0;
                            yBaseB = 0;
                            yBaseCenter = 0;
                            break;
                        case 'middle-right':
                            xTip = xRight + tetherLength;
                            xBaseA = xRight;
                            xBaseB = xRight;
                            xBaseCenter = xRight;
                            yTip = 0;
                            yBaseA = tetherBaseWidth / 2;
                            yBaseB = -tetherBaseWidth / 2;
                            yBaseCenter = 0;
                            break;
                        case 'top-left':
                            xTip = xLeft - tetherLength / 2;
                            xBaseA = xLeft + tetherBaseWidth / 2;
                            xBaseB = xLeft;
                            xBaseCenter = xLeft;
                            yTip = yTop + tetherLength / 2;
                            yBaseA = yTop;
                            yBaseB = yTop - tetherBaseWidth / 2;
                            yBaseCenter = yTop;
                            break;
                        case 'top-center':
                            xTip = 0;
                            xBaseA = tetherBaseWidth / 2;
                            xBaseB = -tetherBaseWidth / 2;
                            xBaseCenter = 0;
                            yTip = yTop + tetherLength;
                            yBaseA = yTop;
                            yBaseB = yTop;
                            yBaseCenter = yTop;
                            break;
                        case 'top-right':
                            xTip = xRight + tetherLength / 2;
                            xBaseA = xRight;
                            xBaseB = xRight - tetherBaseWidth / 2;
                            xBaseCenter = xRight;
                            yTip = yTop + tetherLength / 2;
                            yBaseA = yTop - tetherBaseWidth / 2;
                            yBaseB = yTop;
                            yBaseCenter = yTop;
                            break;
                        default:
                            throw new Error('unsupported attachment');
                    }
                    util_1.ChunkedArray.add2(mappings, xTip, yTip); // tip
                    util_1.ChunkedArray.add2(mappings, xBaseA, yBaseA); // base A
                    util_1.ChunkedArray.add2(mappings, xBaseB, yBaseB); // base B
                    util_1.ChunkedArray.add2(mappings, xBaseCenter, yBaseCenter); // base center
                    var offset = centers.elementCount;
                    for (var i = 0; i < 4; ++i) {
                        util_1.ChunkedArray.add2(tcoords, 10, 10);
                        add(x, y, z, depth, group);
                    }
                    util_1.ChunkedArray.add3(indices, offset, offset + 1, offset + 3);
                    util_1.ChunkedArray.add3(indices, offset, offset + 3, offset + 2);
                }
                xShift += outline;
                yShift += outline;
                var xadvance = 0;
                for (var iChar = 0; iChar < nChar; ++iChar) {
                    var c = fontAtlas.get(str[iChar]);
                    util_1.ChunkedArray.add2(mappings, xadvance - xShift, c.nh - yShift); // top left
                    util_1.ChunkedArray.add2(mappings, xadvance - xShift, -yShift); // bottom left
                    util_1.ChunkedArray.add2(mappings, xadvance + c.nw - xShift, c.nh - yShift); // top right
                    util_1.ChunkedArray.add2(mappings, xadvance + c.nw - xShift, -yShift); // bottom right
                    var texWidth = fontAtlas.texture.width;
                    var texHeight = fontAtlas.texture.height;
                    util_1.ChunkedArray.add2(tcoords, c.x / texWidth, c.y / texHeight); // top left
                    util_1.ChunkedArray.add2(tcoords, c.x / texWidth, (c.y + c.h) / texHeight); // bottom left
                    util_1.ChunkedArray.add2(tcoords, (c.x + c.w) / texWidth, c.y / texHeight); // top right
                    util_1.ChunkedArray.add2(tcoords, (c.x + c.w) / texWidth, (c.y + c.h) / texHeight); // bottom right
                    xadvance += c.nw - 2 * outline;
                    var offset = centers.elementCount;
                    for (var i = 0; i < 4; ++i)
                        add(x, y, z, depth, group);
                    util_1.ChunkedArray.add3(indices, offset + quadIndices[0], offset + quadIndices[1], offset + quadIndices[2]);
                    util_1.ChunkedArray.add3(indices, offset + quadIndices[3], offset + quadIndices[4], offset + quadIndices[5]);
                }
            },
            getText: function () {
                var ft = fontAtlas.texture;
                var cb = util_1.ChunkedArray.compact(centers, true);
                var mb = util_1.ChunkedArray.compact(mappings, true);
                var db = util_1.ChunkedArray.compact(depths, true);
                var ib = util_1.ChunkedArray.compact(indices, true);
                var gb = util_1.ChunkedArray.compact(groups, true);
                var tb = util_1.ChunkedArray.compact(tcoords, true);
                return {
                    kind: 'text',
                    charCount: indices.elementCount / 2,
                    fontTexture: text ? value_cell_1.ValueCell.update(text.fontTexture, ft) : value_cell_1.ValueCell.create(ft),
                    centerBuffer: text ? value_cell_1.ValueCell.update(text.centerBuffer, cb) : value_cell_1.ValueCell.create(cb),
                    mappingBuffer: text ? value_cell_1.ValueCell.update(text.mappingBuffer, mb) : value_cell_1.ValueCell.create(mb),
                    depthBuffer: text ? value_cell_1.ValueCell.update(text.depthBuffer, db) : value_cell_1.ValueCell.create(db),
                    indexBuffer: text ? value_cell_1.ValueCell.update(text.indexBuffer, ib) : value_cell_1.ValueCell.create(ib),
                    groupBuffer: text ? value_cell_1.ValueCell.update(text.groupBuffer, gb) : value_cell_1.ValueCell.create(gb),
                    tcoordBuffer: text ? value_cell_1.ValueCell.update(text.tcoordBuffer, tb) : value_cell_1.ValueCell.create(tb),
                };
            }
        };
    }
    TextBuilder.create = create;
})(TextBuilder = exports.TextBuilder || (exports.TextBuilder = {}));
