/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { ValueCell } from '../../../mol-util';
import { GeometryUtils } from '../geometry';
import { TextureImage } from '../../../mol-gl/renderable/util';
/** Text */
export interface Text {
    readonly kind: 'text';
    /** Number of characters in the text */
    readonly charCount: number;
    /** Font Atlas */
    readonly fontTexture: ValueCell<TextureImage<Uint8Array>>;
    /** Center buffer as array of xyz values wrapped in a value cell */
    readonly centerBuffer: ValueCell<Float32Array>;
    /** Mapping buffer as array of xy values wrapped in a value cell */
    readonly mappingBuffer: ValueCell<Float32Array>;
    /** Depth buffer as array of z values wrapped in a value cell */
    readonly depthBuffer: ValueCell<Float32Array>;
    /** Index buffer as array of center index triplets wrapped in a value cell */
    readonly indexBuffer: ValueCell<Uint32Array>;
    /** Group buffer as array of group ids for each vertex wrapped in a value cell */
    readonly groupBuffer: ValueCell<Float32Array>;
    /** Texture coordinates buffer as array of uv values wrapped in a value cell */
    readonly tcoordBuffer: ValueCell<Float32Array>;
}
export declare namespace Text {
    function createEmpty(text?: Text): Text;
    const Params: {
        sizeFactor: PD.Numeric;
        borderWidth: PD.Numeric;
        borderColor: PD.Color;
        offsetX: PD.Numeric;
        offsetY: PD.Numeric;
        offsetZ: PD.Numeric;
        background: PD.BooleanParam;
        backgroundMargin: PD.Numeric;
        backgroundColor: PD.Color;
        backgroundOpacity: PD.Numeric;
        tether: PD.BooleanParam;
        tetherLength: PD.Numeric;
        tetherBaseWidth: PD.Numeric;
        attachment: PD.Select<"middle-center" | "bottom-left" | "bottom-center" | "bottom-right" | "middle-left" | "middle-right" | "top-left" | "top-center" | "top-right">;
        fontFamily: PD.Select<import("./font-atlas").FontFamily>;
        fontQuality: PD.Select<0 | 1 | 2 | 4 | 3>;
        fontStyle: PD.Select<import("./font-atlas").FontStyle>;
        fontVariant: PD.Select<import("./font-atlas").FontVariant>;
        fontWeight: PD.Select<import("./font-atlas").FontWeight>;
        alpha: PD.Numeric;
        useFog: PD.BooleanParam;
        highlightColor: PD.Color;
        selectColor: PD.Color;
        quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    };
    type Params = typeof Params;
    const Utils: GeometryUtils<Text, Params>;
}
