"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var mol_util_1 = require("../../../mol-util");
var color_data_1 = require("../color-data");
var size_data_1 = require("../size-data");
var marker_data_1 = require("../marker-data");
var tables_1 = require("../../../mol-util/color/tables");
var geometry_1 = require("../../../mol-math/geometry");
var util_1 = require("../../../mol-gl/renderable/util");
var color_1 = require("../../../mol-util/color");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var font_atlas_1 = require("./font-atlas");
var interpolate_1 = require("../../../mol-math/interpolate");
var base_1 = require("../base");
var overpaint_data_1 = require("../overpaint-data");
var transparency_data_1 = require("../transparency-data");
var Text;
(function (Text) {
    function createEmpty(text) {
        var ft = text ? text.fontTexture.ref.value : util_1.createTextureImage(0, 1, Uint8Array);
        var cb = text ? text.centerBuffer.ref.value : new Float32Array(0);
        var mb = text ? text.mappingBuffer.ref.value : new Float32Array(0);
        var db = text ? text.depthBuffer.ref.value : new Float32Array(0);
        var ib = text ? text.indexBuffer.ref.value : new Uint32Array(0);
        var gb = text ? text.groupBuffer.ref.value : new Float32Array(0);
        var tb = text ? text.tcoordBuffer.ref.value : new Float32Array(0);
        return {
            kind: 'text',
            charCount: 0,
            fontTexture: text ? mol_util_1.ValueCell.update(text.fontTexture, ft) : mol_util_1.ValueCell.create(ft),
            centerBuffer: text ? mol_util_1.ValueCell.update(text.centerBuffer, cb) : mol_util_1.ValueCell.create(cb),
            mappingBuffer: text ? mol_util_1.ValueCell.update(text.mappingBuffer, mb) : mol_util_1.ValueCell.create(mb),
            depthBuffer: text ? mol_util_1.ValueCell.update(text.depthBuffer, db) : mol_util_1.ValueCell.create(db),
            indexBuffer: text ? mol_util_1.ValueCell.update(text.indexBuffer, ib) : mol_util_1.ValueCell.create(ib),
            groupBuffer: text ? mol_util_1.ValueCell.update(text.groupBuffer, gb) : mol_util_1.ValueCell.create(gb),
            tcoordBuffer: text ? mol_util_1.ValueCell.update(text.tcoordBuffer, tb) : mol_util_1.ValueCell.create(tb)
        };
    }
    Text.createEmpty = createEmpty;
    Text.Params = __assign({}, base_1.BaseGeometry.Params, font_atlas_1.FontAtlasParams, { sizeFactor: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 10, step: 0.1 }), borderWidth: param_definition_1.ParamDefinition.Numeric(0, { min: 0, max: 0.5, step: 0.01 }), borderColor: param_definition_1.ParamDefinition.Color(tables_1.ColorNames.grey), offsetX: param_definition_1.ParamDefinition.Numeric(0, { min: 0, max: 10, step: 0.1 }), offsetY: param_definition_1.ParamDefinition.Numeric(0, { min: 0, max: 10, step: 0.1 }), offsetZ: param_definition_1.ParamDefinition.Numeric(0, { min: 0, max: 10, step: 0.1 }), background: param_definition_1.ParamDefinition.Boolean(false), backgroundMargin: param_definition_1.ParamDefinition.Numeric(0.2, { min: 0, max: 1, step: 0.01 }), backgroundColor: param_definition_1.ParamDefinition.Color(tables_1.ColorNames.grey), backgroundOpacity: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 1, step: 0.01 }), tether: param_definition_1.ParamDefinition.Boolean(false), tetherLength: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 5, step: 0.1 }), tetherBaseWidth: param_definition_1.ParamDefinition.Numeric(0.3, { min: 0, max: 1, step: 0.01 }), attachment: param_definition_1.ParamDefinition.Select('middle-center', [
            ['bottom-left', 'bottom-left'], ['bottom-center', 'bottom-center'], ['bottom-right', 'bottom-right'],
            ['middle-left', 'middle-left'], ['middle-center', 'middle-center'], ['middle-right', 'middle-right'],
            ['top-left', 'top-left'], ['top-center', 'top-center'], ['top-right', 'top-right'],
        ]) });
    Text.Utils = {
        Params: Text.Params,
        createEmpty: createEmpty,
        createValues: createValues,
        createValuesSimple: createValuesSimple,
        updateValues: updateValues,
        updateBoundingSphere: updateBoundingSphere,
        createRenderableState: createRenderableState,
        updateRenderableState: updateRenderableState,
    };
    function createValues(text, transform, locationIt, theme, props) {
        var instanceCount = locationIt.instanceCount, groupCount = locationIt.groupCount;
        if (instanceCount !== transform.instanceCount.ref.value) {
            throw new Error('instanceCount values in TransformData and LocationIterator differ');
        }
        var color = color_data_1.createColors(locationIt, theme.color);
        var size = size_data_1.createSizes(locationIt, theme.size);
        var marker = marker_data_1.createMarkers(instanceCount * groupCount);
        var overpaint = overpaint_data_1.createEmptyOverpaint();
        var transparency = transparency_data_1.createEmptyTransparency();
        var counts = { drawCount: text.charCount * 2 * 3, groupCount: groupCount, instanceCount: instanceCount };
        var padding = getPadding(text.mappingBuffer.ref.value, text.depthBuffer.ref.value, text.charCount, size_data_1.getMaxSize(size));
        var _a = util_1.calculateBoundingSphere(text.centerBuffer.ref.value, text.charCount * 4, transform.aTransform.ref.value, instanceCount, padding), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        return __assign({ aPosition: text.centerBuffer, aMapping: text.mappingBuffer, aDepth: text.depthBuffer, aGroup: text.groupBuffer, elements: text.indexBuffer, boundingSphere: mol_util_1.ValueCell.create(boundingSphere), invariantBoundingSphere: mol_util_1.ValueCell.create(invariantBoundingSphere) }, color, size, marker, overpaint, transparency, transform, { aTexCoord: text.tcoordBuffer, tFont: text.fontTexture, padding: mol_util_1.ValueCell.create(padding) }, base_1.BaseGeometry.createValues(props, counts), { uSizeFactor: mol_util_1.ValueCell.create(props.sizeFactor), uBorderWidth: mol_util_1.ValueCell.create(interpolate_1.clamp(props.borderWidth / 2, 0, 0.5)), uBorderColor: mol_util_1.ValueCell.create(color_1.Color.toArrayNormalized(props.borderColor, linear_algebra_1.Vec3.zero(), 0)), uOffsetX: mol_util_1.ValueCell.create(props.offsetX), uOffsetY: mol_util_1.ValueCell.create(props.offsetY), uOffsetZ: mol_util_1.ValueCell.create(props.offsetZ), uBackgroundColor: mol_util_1.ValueCell.create(color_1.Color.toArrayNormalized(props.backgroundColor, linear_algebra_1.Vec3.zero(), 0)), uBackgroundOpacity: mol_util_1.ValueCell.create(props.backgroundOpacity) });
    }
    function createValuesSimple(text, props, colorValue, sizeValue, transform) {
        var s = base_1.BaseGeometry.createSimple(colorValue, sizeValue, transform);
        var p = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(Text.Params), props);
        return createValues(text, s.transform, s.locationIterator, s.theme, p);
    }
    function updateValues(values, props) {
        base_1.BaseGeometry.updateValues(values, props);
        mol_util_1.ValueCell.updateIfChanged(values.uSizeFactor, props.sizeFactor);
        mol_util_1.ValueCell.updateIfChanged(values.uBorderWidth, props.borderWidth);
        if (color_1.Color.fromNormalizedArray(values.uBorderColor.ref.value, 0) !== props.borderColor) {
            color_1.Color.toArrayNormalized(props.borderColor, values.uBorderColor.ref.value, 0);
            mol_util_1.ValueCell.update(values.uBorderColor, values.uBorderColor.ref.value);
        }
        mol_util_1.ValueCell.updateIfChanged(values.uOffsetX, props.offsetX);
        mol_util_1.ValueCell.updateIfChanged(values.uOffsetY, props.offsetY);
        mol_util_1.ValueCell.updateIfChanged(values.uOffsetZ, props.offsetZ);
        if (color_1.Color.fromNormalizedArray(values.uBackgroundColor.ref.value, 0) !== props.backgroundColor) {
            color_1.Color.toArrayNormalized(props.backgroundColor, values.uBackgroundColor.ref.value, 0);
            mol_util_1.ValueCell.update(values.uBackgroundColor, values.uBackgroundColor.ref.value);
        }
        mol_util_1.ValueCell.updateIfChanged(values.uBackgroundOpacity, props.backgroundOpacity);
    }
    function updateBoundingSphere(values, text) {
        var padding = getPadding(values.aMapping.ref.value, values.aDepth.ref.value, text.charCount, size_data_1.getMaxSize(values));
        var _a = util_1.calculateBoundingSphere(values.aPosition.ref.value, text.charCount * 4, values.aTransform.ref.value, values.instanceCount.ref.value, padding), boundingSphere = _a.boundingSphere, invariantBoundingSphere = _a.invariantBoundingSphere;
        if (!geometry_1.Sphere3D.equals(boundingSphere, values.boundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.boundingSphere, boundingSphere);
        }
        if (!geometry_1.Sphere3D.equals(invariantBoundingSphere, values.invariantBoundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.invariantBoundingSphere, invariantBoundingSphere);
        }
        mol_util_1.ValueCell.update(values.padding, padding);
    }
    function createRenderableState(props) {
        var state = base_1.BaseGeometry.createRenderableState(props);
        updateRenderableState(state, props);
        return state;
    }
    function updateRenderableState(state, props) {
        base_1.BaseGeometry.updateRenderableState(state, props);
        state.opaque = false;
    }
})(Text = exports.Text || (exports.Text = {}));
function getPadding(mappings, depths, charCount, maxSize) {
    var maxOffset = 0;
    var maxDepth = 0;
    for (var i = 0, il = charCount * 4; i < il; ++i) {
        var i2 = 2 * i;
        var ox = Math.abs(mappings[i2]);
        if (ox > maxOffset)
            maxOffset = ox;
        var oy = Math.abs(mappings[i2 + 1]);
        if (oy > maxOffset)
            maxOffset = oy;
        var d = Math.abs(depths[i]);
        if (d > maxDepth)
            maxDepth = d;
    }
    // console.log(maxDepth + maxSize, maxDepth, maxSize, maxSize + maxSize * maxOffset, depths)
    return Math.max(maxDepth, maxSize + maxSize * maxOffset);
    // return maxSize + maxSize * maxOffset + maxDepth
}
