/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ValueCell } from '../../../mol-util';
import { Sphere3D } from '../../../mol-math/geometry';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { GeometryUtils } from '../geometry';
import { Texture } from '../../../mol-gl/webgl/texture';
import { Vec2 } from '../../../mol-math/linear-algebra';
export interface TextureMesh {
    readonly kind: 'texture-mesh';
    /** Number of vertices in the texture-mesh */
    readonly vertexCount: ValueCell<number>;
    /** Number of groups in the texture-mesh */
    readonly groupCount: ValueCell<number>;
    readonly geoTextureDim: ValueCell<Vec2>;
    /** texture has vertex positions in XYZ and group id in W */
    readonly vertexGroupTexture: ValueCell<Texture>;
    readonly normalTexture: ValueCell<Texture>;
    readonly boundingSphere: ValueCell<Sphere3D>;
}
export declare namespace TextureMesh {
    function create(vertexCount: number, groupCount: number, vertexGroupTexture: Texture, normalTexture: Texture, boundingSphere: Sphere3D, textureMesh?: TextureMesh): TextureMesh;
    function createEmpty(textureMesh?: TextureMesh): TextureMesh;
    const Params: {
        doubleSided: PD.BooleanParam;
        flipSided: PD.BooleanParam;
        flatShaded: PD.BooleanParam;
        alpha: PD.Numeric;
        useFog: PD.BooleanParam;
        highlightColor: PD.Color;
        selectColor: PD.Color;
        quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    };
    type Params = typeof Params;
    const Utils: GeometryUtils<TextureMesh, Params>;
}
