"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../../mol-util");
var geometry_1 = require("../../../mol-math/geometry");
var param_definition_1 = require("../../../mol-util/param-definition");
var color_data_1 = require("../color-data");
var marker_data_1 = require("../marker-data");
var color_1 = require("../../../mol-util/color");
var base_1 = require("../base");
var overpaint_data_1 = require("../overpaint-data");
var transparency_data_1 = require("../transparency-data");
var util_1 = require("../../../mol-gl/renderable/util");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var array_1 = require("../../../mol-util/array");
var TextureMesh;
(function (TextureMesh) {
    function create(vertexCount, groupCount, vertexGroupTexture, normalTexture, boundingSphere, textureMesh) {
        var width = vertexGroupTexture.width, height = vertexGroupTexture.height;
        if (textureMesh) {
            mol_util_1.ValueCell.update(textureMesh.vertexCount, vertexCount);
            mol_util_1.ValueCell.update(textureMesh.groupCount, groupCount);
            mol_util_1.ValueCell.update(textureMesh.geoTextureDim, linear_algebra_1.Vec2.set(textureMesh.geoTextureDim.ref.value, width, height));
            mol_util_1.ValueCell.update(textureMesh.vertexGroupTexture, vertexGroupTexture);
            mol_util_1.ValueCell.update(textureMesh.normalTexture, normalTexture);
            mol_util_1.ValueCell.update(textureMesh.boundingSphere, boundingSphere);
            return textureMesh;
        }
        else {
            return {
                kind: 'texture-mesh',
                vertexCount: mol_util_1.ValueCell.create(vertexCount),
                groupCount: mol_util_1.ValueCell.create(groupCount),
                geoTextureDim: mol_util_1.ValueCell.create(linear_algebra_1.Vec2.create(width, height)),
                vertexGroupTexture: mol_util_1.ValueCell.create(vertexGroupTexture),
                normalTexture: mol_util_1.ValueCell.create(normalTexture),
                boundingSphere: mol_util_1.ValueCell.create(boundingSphere),
            };
        }
    }
    TextureMesh.create = create;
    function createEmpty(textureMesh) {
        return {}; // TODO
    }
    TextureMesh.createEmpty = createEmpty;
    TextureMesh.Params = __assign({}, base_1.BaseGeometry.Params, { doubleSided: param_definition_1.ParamDefinition.Boolean(false), flipSided: param_definition_1.ParamDefinition.Boolean(false), flatShaded: param_definition_1.ParamDefinition.Boolean(false) });
    TextureMesh.Utils = {
        Params: TextureMesh.Params,
        createEmpty: createEmpty,
        createValues: createValues,
        createValuesSimple: createValuesSimple,
        updateValues: updateValues,
        updateBoundingSphere: updateBoundingSphere,
        createRenderableState: base_1.BaseGeometry.createRenderableState,
        updateRenderableState: base_1.BaseGeometry.updateRenderableState
    };
    function createValues(textureMesh, transform, locationIt, theme, props) {
        var instanceCount = locationIt.instanceCount, groupCount = locationIt.groupCount;
        var color = color_data_1.createColors(locationIt, theme.color);
        var marker = marker_data_1.createMarkers(instanceCount * groupCount);
        var overpaint = overpaint_data_1.createEmptyOverpaint();
        var transparency = transparency_data_1.createEmptyTransparency();
        var counts = { drawCount: textureMesh.vertexCount.ref.value, groupCount: groupCount, instanceCount: instanceCount };
        var transformBoundingSphere = util_1.calculateTransformBoundingSphere(textureMesh.boundingSphere.ref.value, transform.aTransform.ref.value, transform.instanceCount.ref.value);
        return __assign({ uGeoTexDim: textureMesh.geoTextureDim, tPositionGroup: textureMesh.vertexGroupTexture, tNormal: textureMesh.normalTexture, 
            // aGroup is used as a vertex index here and the group id is retirieved from tPositionGroup
            aGroup: mol_util_1.ValueCell.create(array_1.fillSerial(new Float32Array(textureMesh.vertexCount.ref.value))), boundingSphere: mol_util_1.ValueCell.create(transformBoundingSphere), invariantBoundingSphere: textureMesh.boundingSphere }, color, marker, overpaint, transparency, transform, base_1.BaseGeometry.createValues(props, counts), { dDoubleSided: mol_util_1.ValueCell.create(props.doubleSided), dFlatShaded: mol_util_1.ValueCell.create(props.flatShaded), dFlipSided: mol_util_1.ValueCell.create(props.flipSided), dGeoTexture: mol_util_1.ValueCell.create(true) });
    }
    function createValuesSimple(textureMesh, props, colorValue, sizeValue, transform) {
        var s = base_1.BaseGeometry.createSimple(colorValue, sizeValue, transform);
        var p = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(TextureMesh.Params), props);
        return createValues(textureMesh, s.transform, s.locationIterator, s.theme, p);
    }
    function updateValues(values, props) {
        if (color_1.Color.fromNormalizedArray(values.uHighlightColor.ref.value, 0) !== props.highlightColor) {
            mol_util_1.ValueCell.update(values.uHighlightColor, color_1.Color.toArrayNormalized(props.highlightColor, values.uHighlightColor.ref.value, 0));
        }
        if (color_1.Color.fromNormalizedArray(values.uSelectColor.ref.value, 0) !== props.selectColor) {
            mol_util_1.ValueCell.update(values.uSelectColor, color_1.Color.toArrayNormalized(props.selectColor, values.uSelectColor.ref.value, 0));
        }
        mol_util_1.ValueCell.updateIfChanged(values.alpha, props.alpha); // `uAlpha` is set in renderable.render
        mol_util_1.ValueCell.updateIfChanged(values.dUseFog, props.useFog);
        mol_util_1.ValueCell.updateIfChanged(values.dDoubleSided, props.doubleSided);
        mol_util_1.ValueCell.updateIfChanged(values.dFlatShaded, props.flatShaded);
        mol_util_1.ValueCell.updateIfChanged(values.dFlipSided, props.flipSided);
        if (values.drawCount.ref.value > values.aGroup.ref.value.length) {
            // console.log('updating vertex ids in aGroup to handle larger drawCount')
            mol_util_1.ValueCell.update(values.aGroup, array_1.fillSerial(new Float32Array(values.drawCount.ref.value)));
        }
    }
    function updateBoundingSphere(values, textureMesh) {
        var invariantBoundingSphere = textureMesh.boundingSphere.ref.value;
        var boundingSphere = util_1.calculateTransformBoundingSphere(invariantBoundingSphere, values.aTransform.ref.value, values.instanceCount.ref.value);
        if (!geometry_1.Sphere3D.equals(boundingSphere, values.boundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.boundingSphere, boundingSphere);
        }
        if (!geometry_1.Sphere3D.equals(invariantBoundingSphere, values.invariantBoundingSphere.ref.value)) {
            mol_util_1.ValueCell.update(values.invariantBoundingSphere, invariantBoundingSphere);
        }
    }
})(TextureMesh = exports.TextureMesh || (exports.TextureMesh = {}));
