"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var value_cell_1 = require("../../mol-util/value-cell");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var util_1 = require("../../mol-gl/renderable/util");
function applyTransparencyValue(array, start, end, value) {
    for (var i = start; i < end; ++i) {
        array[i] = value * 255;
    }
    return true;
}
exports.applyTransparencyValue = applyTransparencyValue;
function clearTransparency(array, start, end) {
    array.fill(0, start, end);
}
exports.clearTransparency = clearTransparency;
function createTransparency(count, variant, transparencyData) {
    var transparency = util_1.createTextureImage(Math.max(1, count), 1, Uint8Array, transparencyData && transparencyData.tTransparency.ref.value.array);
    if (transparencyData) {
        value_cell_1.ValueCell.update(transparencyData.tTransparency, transparency);
        value_cell_1.ValueCell.update(transparencyData.uTransparencyTexDim, linear_algebra_1.Vec2.create(transparency.width, transparency.height));
        value_cell_1.ValueCell.update(transparencyData.dTransparency, count > 0);
        value_cell_1.ValueCell.update(transparencyData.dTransparencyVariant, variant);
        return transparencyData;
    }
    else {
        return {
            tTransparency: value_cell_1.ValueCell.create(transparency),
            uTransparencyTexDim: value_cell_1.ValueCell.create(linear_algebra_1.Vec2.create(transparency.width, transparency.height)),
            dTransparency: value_cell_1.ValueCell.create(count > 0),
            dTransparencyVariant: value_cell_1.ValueCell.create(variant),
        };
    }
}
exports.createTransparency = createTransparency;
var emptyTransparencyTexture = { array: new Uint8Array(1), width: 1, height: 1 };
function createEmptyTransparency(transparencyData) {
    if (transparencyData) {
        value_cell_1.ValueCell.update(transparencyData.tTransparency, emptyTransparencyTexture);
        value_cell_1.ValueCell.update(transparencyData.uTransparencyTexDim, linear_algebra_1.Vec2.create(1, 1));
        return transparencyData;
    }
    else {
        return {
            tTransparency: value_cell_1.ValueCell.create(emptyTransparencyTexture),
            uTransparencyTexDim: value_cell_1.ValueCell.create(linear_algebra_1.Vec2.create(1, 1)),
            dTransparency: value_cell_1.ValueCell.create(false),
            dTransparencyVariant: value_cell_1.ValueCell.create('single'),
        };
    }
}
exports.createEmptyTransparency = createEmptyTransparency;
