"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var a = linear_algebra_1.Vec3.zero(), b = linear_algebra_1.Vec3.zero(), c = linear_algebra_1.Vec3.zero();
/** Create primitive with face normals from vertices and indices */
function createPrimitive(vertices, indices) {
    var count = indices.length;
    var builder = PrimitiveBuilder(count / 3);
    for (var i = 0; i < count; i += 3) {
        linear_algebra_1.Vec3.fromArray(a, vertices, indices[i] * 3);
        linear_algebra_1.Vec3.fromArray(b, vertices, indices[i + 1] * 3);
        linear_algebra_1.Vec3.fromArray(c, vertices, indices[i + 2] * 3);
        builder.add(a, b, c);
    }
    return builder.getPrimitive();
}
exports.createPrimitive = createPrimitive;
var vn = linear_algebra_1.Vec3.zero();
/** Builder to create primitive with face normals */
function PrimitiveBuilder(triangleCount) {
    var vertices = new Float32Array(triangleCount * 3 * 3);
    var normals = new Float32Array(triangleCount * 3 * 3);
    var indices = new Uint32Array(triangleCount * 3);
    var offset = 0;
    return {
        add: function (a, b, c) {
            linear_algebra_1.Vec3.toArray(a, vertices, offset);
            linear_algebra_1.Vec3.toArray(b, vertices, offset + 3);
            linear_algebra_1.Vec3.toArray(c, vertices, offset + 6);
            linear_algebra_1.Vec3.triangleNormal(vn, a, b, c);
            for (var j = 0; j < 3; ++j) {
                linear_algebra_1.Vec3.toArray(vn, normals, offset + 3 * j);
                indices[offset / 3 + j] = offset / 3 + j;
            }
            offset += 9;
        },
        getPrimitive: function () { return ({ vertices: vertices, normals: normals, indices: indices }); }
    };
}
exports.PrimitiveBuilder = PrimitiveBuilder;
