"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var primitive_1 = require("./primitive");
var polygon_1 = require("./polygon");
var on = linear_algebra_1.Vec3.create(0, 0, -0.5), op = linear_algebra_1.Vec3.create(0, 0, 0.5);
var a = linear_algebra_1.Vec3.zero(), b = linear_algebra_1.Vec3.zero(), c = linear_algebra_1.Vec3.zero(), d = linear_algebra_1.Vec3.zero();
/**
 * Create a prism with a base of 4 or more points
 */
function Prism(points) {
    var sideCount = points.length / 3;
    if (sideCount < 4)
        throw new Error('need at least 4 points to build a prism');
    var count = 4 * sideCount;
    var builder = primitive_1.PrimitiveBuilder(count);
    // create sides
    for (var i = 0; i < sideCount; ++i) {
        var ni = (i + 1) % sideCount;
        linear_algebra_1.Vec3.set(a, points[i * 3], points[i * 3 + 1], -0.5);
        linear_algebra_1.Vec3.set(b, points[ni * 3], points[ni * 3 + 1], -0.5);
        linear_algebra_1.Vec3.set(c, points[ni * 3], points[ni * 3 + 1], 0.5);
        linear_algebra_1.Vec3.set(d, points[i * 3], points[i * 3 + 1], 0.5);
        builder.add(a, b, c);
        builder.add(c, d, a);
    }
    // create bases
    for (var i = 0; i < sideCount; ++i) {
        var ni = (i + 1) % sideCount;
        linear_algebra_1.Vec3.set(a, points[i * 3], points[i * 3 + 1], -0.5);
        linear_algebra_1.Vec3.set(b, points[ni * 3], points[ni * 3 + 1], -0.5);
        builder.add(on, b, a);
        linear_algebra_1.Vec3.set(a, points[i * 3], points[i * 3 + 1], 0.5);
        linear_algebra_1.Vec3.set(b, points[ni * 3], points[ni * 3 + 1], 0.5);
        builder.add(a, b, op);
    }
    return builder.getPrimitive();
}
exports.Prism = Prism;
var diamond;
function DiamondPrism() {
    if (!diamond)
        diamond = Prism(polygon_1.polygon(4, false));
    return diamond;
}
exports.DiamondPrism = DiamondPrism;
var pentagonalPrism;
function PentagonalPrism() {
    if (!pentagonalPrism)
        pentagonalPrism = Prism(polygon_1.polygon(5, false));
    return pentagonalPrism;
}
exports.PentagonalPrism = PentagonalPrism;
var hexagonalPrism;
function HexagonalPrism() {
    if (!hexagonalPrism)
        hexagonalPrism = Prism(polygon_1.polygon(6, true));
    return hexagonalPrism;
}
exports.HexagonalPrism = HexagonalPrism;
//
/**
 * Create a prism cage
 */
function PrismCage(points) {
    var sideCount = points.length / 3;
    // const count = 4 * sideCount
    var vertices = [];
    var edges = [];
    var offset = 0;
    // vertices and side edges
    for (var i = 0; i < sideCount; ++i) {
        vertices.push(points[i * 3], points[i * 3 + 1], -0.5, points[i * 3], points[i * 3 + 1], 0.5);
        edges.push(offset, offset + 1);
        offset += 2;
    }
    // bases edges
    for (var i = 0; i < sideCount; ++i) {
        var ni = (i + 1) % sideCount;
        edges.push(i * 2, ni * 2, i * 2 + 1, ni * 2 + 1);
    }
    return { vertices: vertices, edges: edges };
}
exports.PrismCage = PrismCage;
var diamondCage;
function DiamondPrismCage() {
    if (!diamondCage)
        diamondCage = PrismCage(polygon_1.polygon(4, false));
    return diamondCage;
}
exports.DiamondPrismCage = DiamondPrismCage;
var pentagonalPrismCage;
function PentagonalPrismCage() {
    if (!pentagonalPrismCage)
        pentagonalPrismCage = PrismCage(polygon_1.polygon(5, false));
    return pentagonalPrismCage;
}
exports.PentagonalPrismCage = PentagonalPrismCage;
var hexagonalPrismCage;
function HexagonalPrismCage() {
    if (!hexagonalPrismCage)
        hexagonalPrismCage = PrismCage(polygon_1.polygon(6, true));
    return hexagonalPrismCage;
}
exports.HexagonalPrismCage = HexagonalPrismCage;
