"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../mol-math/linear-algebra");
function normalizeVec3Array(a) {
    var n = a.length;
    for (var i = 0; i < n; i += 3) {
        var x = a[i];
        var y = a[i + 1];
        var z = a[i + 2];
        var s = 1 / Math.sqrt(x * x + y * y + z * z);
        a[i] = x * s;
        a[i + 1] = y * s;
        a[i + 2] = z * s;
    }
}
exports.normalizeVec3Array = normalizeVec3Array;
function getNormalMatrix(out, t) {
    linear_algebra_1.Mat3.fromMat4(out, t);
    linear_algebra_1.Mat3.invert(out, out);
    linear_algebra_1.Mat3.transpose(out, out);
    return out;
}
exports.getNormalMatrix = getNormalMatrix;
var tmpV3 = linear_algebra_1.Vec3.zero();
function transformPositionArray(t, array, offset, count) {
    for (var i = 0, il = count * 3; i < il; i += 3) {
        linear_algebra_1.Vec3.fromArray(tmpV3, array, offset + i);
        linear_algebra_1.Vec3.transformMat4(tmpV3, tmpV3, t);
        linear_algebra_1.Vec3.toArray(tmpV3, array, offset + i);
    }
}
exports.transformPositionArray = transformPositionArray;
function transformDirectionArray(n, array, offset, count) {
    for (var i = 0, il = count * 3; i < il; i += 3) {
        linear_algebra_1.Vec3.fromArray(tmpV3, array, offset + i);
        linear_algebra_1.Vec3.transformMat3(tmpV3, tmpV3, n);
        linear_algebra_1.Vec3.toArray(tmpV3, array, offset + i);
    }
}
exports.transformDirectionArray = transformDirectionArray;
function setArrayZero(array) {
    var n = array.length;
    for (var i = 0; i < n; ++i)
        array[i] = 0;
}
exports.setArrayZero = setArrayZero;
/** iterate over the entire buffer and apply the radius to each vertex */
function appplyRadius(vertices, radius) {
    var v = linear_algebra_1.Vec3.zero();
    var n = vertices.length;
    for (var i = 0; i < n; i += 3) {
        linear_algebra_1.Vec3.fromArray(v, vertices, i);
        linear_algebra_1.Vec3.normalize(v, v);
        linear_algebra_1.Vec3.scale(v, v, radius);
        linear_algebra_1.Vec3.toArray(v, vertices, i);
    }
}
exports.appplyRadius = appplyRadius;
/**
 * indexed vertex normals weighted by triangle areas http://www.iquilezles.org/www/articles/normals/normals.htm
 * normal array must contain only zeros
 */
function computeIndexedVertexNormals(vertices, indices, normals) {
    var a = linear_algebra_1.Vec3.zero();
    var b = linear_algebra_1.Vec3.zero();
    var c = linear_algebra_1.Vec3.zero();
    var cb = linear_algebra_1.Vec3.zero();
    var ab = linear_algebra_1.Vec3.zero();
    for (var i = 0, il = indices.length; i < il; i += 3) {
        var ai = indices[i] * 3;
        var bi = indices[i + 1] * 3;
        var ci = indices[i + 2] * 3;
        linear_algebra_1.Vec3.fromArray(a, vertices, ai);
        linear_algebra_1.Vec3.fromArray(b, vertices, bi);
        linear_algebra_1.Vec3.fromArray(c, vertices, ci);
        linear_algebra_1.Vec3.sub(cb, c, b);
        linear_algebra_1.Vec3.sub(ab, a, b);
        linear_algebra_1.Vec3.cross(cb, cb, ab);
        normals[ai] += cb[0];
        normals[ai + 1] += cb[1];
        normals[ai + 2] += cb[2];
        normals[bi] += cb[0];
        normals[bi + 1] += cb[1];
        normals[bi + 2] += cb[2];
        normals[ci] += cb[0];
        normals[ci + 1] += cb[1];
        normals[ci + 2] += cb[2];
    }
    normalizeVec3Array(normals);
    return normals;
}
exports.computeIndexedVertexNormals = computeIndexedVertexNormals;
/** vertex normals for unindexed triangle soup, normal array must contain only zeros */
function computeVertexNormals(vertices, normals) {
    setArrayZero(normals);
    var a = linear_algebra_1.Vec3.zero();
    var b = linear_algebra_1.Vec3.zero();
    var c = linear_algebra_1.Vec3.zero();
    var cb = linear_algebra_1.Vec3.zero();
    var ab = linear_algebra_1.Vec3.zero();
    for (var i = 0, il = vertices.length; i < il; i += 9) {
        linear_algebra_1.Vec3.fromArray(a, vertices, i);
        linear_algebra_1.Vec3.fromArray(b, vertices, i + 3);
        linear_algebra_1.Vec3.fromArray(c, vertices, i + 6);
        linear_algebra_1.Vec3.sub(cb, c, b);
        linear_algebra_1.Vec3.sub(ab, a, b);
        linear_algebra_1.Vec3.cross(cb, cb, ab);
        normals[i] = cb[0];
        normals[i + 1] = cb[1];
        normals[i + 2] = cb[2];
        normals[i + 3] = cb[0];
        normals[i + 4] = cb[1];
        normals[i + 5] = cb[2];
        normals[i + 6] = cb[0];
        normals[i + 7] = cb[1];
        normals[i + 8] = cb[2];
    }
    normalizeVec3Array(normals);
    return normals;
}
exports.computeVertexNormals = computeVertexNormals;
