"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var gl_shim_1 = require("./gl.shim");
var camera_1 = require("../../mol-canvas3d/camera");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var mol_util_1 = require("../../mol-util");
var renderer_1 = require("../renderer");
var color_data_1 = require("../../mol-geo/geometry/color-data");
var size_data_1 = require("../../mol-geo/geometry/size-data");
var context_1 = require("../webgl/context");
var render_object_1 = require("../render-object");
var scene_1 = require("../scene");
var marker_data_1 = require("../../mol-geo/geometry/marker-data");
var array_1 = require("../../mol-util/array");
var color_1 = require("../../mol-util/color");
var geometry_1 = require("../../mol-math/geometry");
var overpaint_data_1 = require("../../mol-geo/geometry/overpaint-data");
var transparency_data_1 = require("../../mol-geo/geometry/transparency-data");
function createRenderer(gl) {
    var ctx = context_1.createContext(gl);
    var camera = new camera_1.Camera({
        near: 0.01,
        far: 10000,
        position: linear_algebra_1.Vec3.create(0, 0, 50)
    });
    var renderer = renderer_1.default.create(ctx, camera);
    return { ctx: ctx, camera: camera, renderer: renderer };
}
function createPoints() {
    var aPosition = mol_util_1.ValueCell.create(new Float32Array([0, -1, 0, -1, 0, 0, 1, 1, 0]));
    var aGroup = mol_util_1.ValueCell.create(array_1.fillSerial(new Float32Array(3)));
    var aInstance = mol_util_1.ValueCell.create(array_1.fillSerial(new Float32Array(1)));
    var color = color_data_1.createValueColor(color_1.Color(0xFF0000));
    var size = size_data_1.createValueSize(1);
    var marker = marker_data_1.createEmptyMarkers();
    var overpaint = overpaint_data_1.createEmptyOverpaint();
    var transparency = transparency_data_1.createEmptyTransparency();
    var aTransform = mol_util_1.ValueCell.create(new Float32Array(16));
    var m4 = linear_algebra_1.Mat4.identity();
    linear_algebra_1.Mat4.toArray(m4, aTransform.ref.value, 0);
    var transform = mol_util_1.ValueCell.create(new Float32Array(aTransform.ref.value));
    var extraTransform = mol_util_1.ValueCell.create(new Float32Array(aTransform.ref.value));
    var boundingSphere = mol_util_1.ValueCell.create(geometry_1.Sphere3D.create(linear_algebra_1.Vec3.zero(), 2));
    var invariantBoundingSphere = mol_util_1.ValueCell.create(geometry_1.Sphere3D.create(linear_algebra_1.Vec3.zero(), 2));
    var values = __assign({ aPosition: aPosition,
        aGroup: aGroup,
        aTransform: aTransform,
        aInstance: aInstance }, color, marker, size, overpaint, transparency, { uAlpha: mol_util_1.ValueCell.create(1.0), uHighlightColor: mol_util_1.ValueCell.create(linear_algebra_1.Vec3.create(1.0, 0.4, 0.6)), uSelectColor: mol_util_1.ValueCell.create(linear_algebra_1.Vec3.create(0.2, 1.0, 0.1)), uInstanceCount: mol_util_1.ValueCell.create(1), uGroupCount: mol_util_1.ValueCell.create(3), alpha: mol_util_1.ValueCell.create(1.0), drawCount: mol_util_1.ValueCell.create(3), instanceCount: mol_util_1.ValueCell.create(1), matrix: mol_util_1.ValueCell.create(m4), transform: transform,
        extraTransform: extraTransform,
        boundingSphere: boundingSphere,
        invariantBoundingSphere: invariantBoundingSphere, uSizeFactor: mol_util_1.ValueCell.create(1), dPointSizeAttenuation: mol_util_1.ValueCell.create(true), dPointFilledCircle: mol_util_1.ValueCell.create(false), uPointEdgeBleach: mol_util_1.ValueCell.create(0.5), dUseFog: mol_util_1.ValueCell.create(true) });
    var state = {
        visible: true,
        alphaFactor: 1,
        pickable: true,
        opaque: true
    };
    return render_object_1.createRenderObject('points', values, state, -1);
}
describe('renderer', function () {
    it('basic', function () {
        var _a = [32, 32], width = _a[0], height = _a[1];
        var gl = gl_shim_1.createGl(width, height, { preserveDrawingBuffer: true });
        var _b = createRenderer(gl), ctx = _b.ctx, renderer = _b.renderer;
        expect(ctx.gl.canvas.width).toBe(32);
        expect(ctx.gl.canvas.height).toBe(32);
        expect(ctx.stats.bufferCount).toBe(0);
        expect(ctx.stats.textureCount).toBe(0);
        expect(ctx.stats.vaoCount).toBe(0);
        expect(ctx.programCache.count).toBe(0);
        expect(ctx.shaderCache.count).toBe(0);
        renderer.setViewport(0, 0, 64, 48);
        expect(ctx.gl.getParameter(ctx.gl.VIEWPORT)[2]).toBe(64);
        expect(ctx.gl.getParameter(ctx.gl.VIEWPORT)[3]).toBe(48);
    });
    it('points', function () {
        var _a = [32, 32], width = _a[0], height = _a[1];
        var gl = gl_shim_1.createGl(width, height, { preserveDrawingBuffer: true });
        var ctx = createRenderer(gl).ctx;
        var scene = scene_1.default.create(ctx);
        var points = createPoints();
        scene.add(points);
        expect(ctx.stats.bufferCount).toBe(4);
        expect(ctx.stats.textureCount).toBe(5);
        expect(ctx.stats.vaoCount).toBe(5);
        expect(ctx.programCache.count).toBe(5);
        expect(ctx.shaderCache.count).toBe(10);
        scene.remove(points);
        expect(ctx.stats.bufferCount).toBe(0);
        expect(ctx.stats.textureCount).toBe(0);
        expect(ctx.stats.vaoCount).toBe(0);
        expect(ctx.programCache.count).toBe(5);
        expect(ctx.shaderCache.count).toBe(10);
        ctx.programCache.dispose();
        expect(ctx.programCache.count).toBe(0);
        ctx.shaderCache.clear();
        expect(ctx.shaderCache.count).toBe(0);
    });
});
