"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var renderable_1 = require("../../renderable");
var render_item_1 = require("../../webgl/render-item");
var schema_1 = require("../../renderable/schema");
var texture_1 = require("../../../mol-gl/webgl/texture");
var shader_code_1 = require("../../../mol-gl/shader-code");
var mol_util_1 = require("../../../mol-util");
var util_1 = require("../util");
var sum_1 = require("./sum");
var framebuffer_1 = require("../../../mol-gl/webgl/framebuffer");
var misc_1 = require("../../../mol-math/misc");
var quad_vert_1 = require("../../../mol-gl/shader/quad.vert");
var reduction_frag_1 = require("../../../mol-gl/shader/histogram-pyramid/reduction.frag");
var HistopyramidReductionSchema = __assign({}, util_1.QuadSchema, { tPreviousLevel: schema_1.TextureSpec('texture', 'rgba', 'float', 'nearest'), uSize: schema_1.UniformSpec('f'), uTexSize: schema_1.UniformSpec('f') });
var HistopyramidReductionRenderable;
function getHistopyramidReductionRenderable(ctx, initialTexture) {
    if (HistopyramidReductionRenderable) {
        mol_util_1.ValueCell.update(HistopyramidReductionRenderable.values.tPreviousLevel, initialTexture);
        HistopyramidReductionRenderable.update();
        return HistopyramidReductionRenderable;
    }
    else {
        var values = __assign({}, util_1.QuadValues, { tPreviousLevel: mol_util_1.ValueCell.create(initialTexture), uSize: mol_util_1.ValueCell.create(0), uTexSize: mol_util_1.ValueCell.create(0) });
        var schema = __assign({}, HistopyramidReductionSchema);
        var shaderCode = shader_code_1.ShaderCode(quad_vert_1.default, reduction_frag_1.default);
        var renderItem = render_item_1.createComputeRenderItem(ctx, 'triangles', shaderCode, schema, values);
        HistopyramidReductionRenderable = renderable_1.createComputeRenderable(renderItem, values);
        return HistopyramidReductionRenderable;
    }
}
var LevelTexturesFramebuffers = [];
function getLevelTextureFramebuffer(ctx, level) {
    var textureFramebuffer = LevelTexturesFramebuffers[level];
    var size = Math.pow(2, level);
    if (textureFramebuffer === undefined) {
        var texture = texture_1.createTexture(ctx, 'image-float32', 'rgba', 'float', 'nearest');
        var framebuffer = framebuffer_1.createFramebuffer(ctx.gl, ctx.stats);
        texture.attachFramebuffer(framebuffer, 0);
        textureFramebuffer = { texture: texture, framebuffer: framebuffer };
        textureFramebuffer.texture.define(size, size);
        LevelTexturesFramebuffers[level] = textureFramebuffer;
    }
    return textureFramebuffer;
}
function setRenderingDefaults(ctx) {
    var gl = ctx.gl, state = ctx.state;
    state.disable(gl.CULL_FACE);
    state.disable(gl.BLEND);
    state.disable(gl.DEPTH_TEST);
    state.disable(gl.SCISSOR_TEST);
    state.depthMask(false);
    state.colorMask(true, true, true, true);
    state.clearColor(0, 0, 0, 0);
}
function createHistogramPyramid(ctx, inputTexture, scale) {
    var gl = ctx.gl, framebufferCache = ctx.framebufferCache;
    // printTexture(ctx, inputTexture, 2)
    if (inputTexture.width !== inputTexture.height || !misc_1.isPowerOfTwo(inputTexture.width)) {
        throw new Error('inputTexture must be of square power-of-two size');
    }
    // This part set the levels
    var levels = Math.ceil(Math.log(inputTexture.width) / Math.log(2));
    var maxSize = Math.pow(2, levels);
    // console.log('levels', levels, 'maxSize', maxSize)
    var pyramidTexture = texture_1.createTexture(ctx, 'image-float32', 'rgba', 'float', 'nearest');
    pyramidTexture.define(maxSize, maxSize);
    var framebuffer = framebufferCache.get('reduction').value;
    pyramidTexture.attachFramebuffer(framebuffer, 0);
    gl.clear(gl.COLOR_BUFFER_BIT);
    var levelTexturesFramebuffers = [];
    for (var i = 0; i < levels; ++i)
        levelTexturesFramebuffers.push(getLevelTextureFramebuffer(ctx, i));
    var renderable = getHistopyramidReductionRenderable(ctx, inputTexture);
    ctx.state.currentRenderItemId = -1;
    setRenderingDefaults(ctx);
    var offset = 0;
    for (var i = 0; i < levels; i++) {
        var currLevel = levels - 1 - i;
        var tf = levelTexturesFramebuffers[currLevel];
        tf.framebuffer.bind();
        // levelTextures[currLevel].attachFramebuffer(framebuffer, 0)
        var size = Math.pow(2, currLevel);
        // console.log('size', size, 'draw-level', currLevel, 'read-level', levels - i)
        gl.clear(gl.COLOR_BUFFER_BIT);
        gl.viewport(0, 0, size, size);
        mol_util_1.ValueCell.update(renderable.values.uSize, Math.pow(2, i + 1) / maxSize);
        mol_util_1.ValueCell.update(renderable.values.uTexSize, size);
        if (i > 0) {
            mol_util_1.ValueCell.update(renderable.values.tPreviousLevel, levelTexturesFramebuffers[levels - i].texture);
            renderable.update();
        }
        ctx.state.currentRenderItemId = -1;
        renderable.render();
        pyramidTexture.bind(0);
        gl.copyTexSubImage2D(gl.TEXTURE_2D, 0, offset, 0, 0, 0, size, size);
        pyramidTexture.unbind(0);
        offset += size;
    }
    gl.finish();
    // printTexture(ctx, pyramidTexture, 2)
    //
    var finalCount = sum_1.getHistopyramidSum(ctx, levelTexturesFramebuffers[0].texture);
    var height = Math.ceil(finalCount / Math.pow(2, levels));
    // const scale = Vec2.create(maxSize / inputTexture.width, maxSize / inputTexture.height)
    // console.log('height', height, 'finalCount', finalCount, 'scale', scale)
    return {
        pyramidTex: pyramidTexture,
        count: finalCount,
        height: height,
        levels: levels,
        scale: scale
    };
}
exports.createHistogramPyramid = createHistogramPyramid;
