"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var renderable_1 = require("../../renderable");
var render_item_1 = require("../../webgl/render-item");
var schema_1 = require("../../renderable/schema");
var texture_1 = require("../../../mol-gl/webgl/texture");
var shader_code_1 = require("../../../mol-gl/shader-code");
var mol_util_1 = require("../../../mol-util");
var float_packing_1 = require("../../../mol-util/float-packing");
var util_1 = require("../util");
var quad_vert_1 = require("../../../mol-gl/shader/quad.vert");
var sum_frag_1 = require("../../../mol-gl/shader/histogram-pyramid/sum.frag");
var HistopyramidSumSchema = __assign({}, util_1.QuadSchema, { tTexture: schema_1.TextureSpec('texture', 'rgba', 'float', 'nearest') });
var HistopyramidSumRenderable;
function getHistopyramidSumRenderable(ctx, texture) {
    if (HistopyramidSumRenderable) {
        mol_util_1.ValueCell.update(HistopyramidSumRenderable.values.tTexture, texture);
        HistopyramidSumRenderable.update();
        return HistopyramidSumRenderable;
    }
    else {
        var values = __assign({}, util_1.QuadValues, { tTexture: mol_util_1.ValueCell.create(texture) });
        var schema = __assign({}, HistopyramidSumSchema);
        var shaderCode = shader_code_1.ShaderCode(quad_vert_1.default, sum_frag_1.default);
        var renderItem = render_item_1.createComputeRenderItem(ctx, 'triangles', shaderCode, schema, values);
        HistopyramidSumRenderable = renderable_1.createComputeRenderable(renderItem, values);
        return HistopyramidSumRenderable;
    }
}
var SumTexture;
function getSumTexture(ctx) {
    if (SumTexture)
        return SumTexture;
    SumTexture = texture_1.createTexture(ctx, 'image-uint8', 'rgba', 'ubyte', 'nearest');
    SumTexture.define(1, 1);
    return SumTexture;
}
/** name for shared framebuffer used for histogram-pyramid operations */
var FramebufferName = 'histogram-pyramid-sum';
function setRenderingDefaults(ctx) {
    var gl = ctx.gl, state = ctx.state;
    state.disable(gl.CULL_FACE);
    state.disable(gl.BLEND);
    state.disable(gl.DEPTH_TEST);
    state.disable(gl.SCISSOR_TEST);
    state.depthMask(false);
    state.colorMask(true, true, true, true);
    state.clearColor(0, 0, 0, 0);
}
var sumArray = new Uint8Array(4);
function getHistopyramidSum(ctx, pyramidTopTexture) {
    var gl = ctx.gl, framebufferCache = ctx.framebufferCache;
    var renderable = getHistopyramidSumRenderable(ctx, pyramidTopTexture);
    ctx.state.currentRenderItemId = -1;
    var framebuffer = framebufferCache.get(FramebufferName).value;
    var sumTexture = getSumTexture(ctx);
    sumTexture.attachFramebuffer(framebuffer, 0);
    setRenderingDefaults(ctx);
    gl.viewport(0, 0, 1, 1);
    renderable.render();
    gl.finish();
    ctx.readPixels(0, 0, 1, 1, sumArray);
    ctx.unbindFramebuffer();
    return float_packing_1.decodeFloatRGB(sumArray[0], sumArray[1], sumArray[2]);
}
exports.getHistopyramidSum = getHistopyramidSum;
