"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var renderable_1 = require("../../renderable");
var render_item_1 = require("../../webgl/render-item");
var schema_1 = require("../../renderable/schema");
var texture_1 = require("../../../mol-gl/webgl/texture");
var shader_code_1 = require("../../../mol-gl/shader-code");
var mol_util_1 = require("../../../mol-util");
var util_1 = require("../util");
var tables_1 = require("./tables");
var quad_vert_1 = require("../../../mol-gl/shader/quad.vert");
var active_voxels_frag_1 = require("../../../mol-gl/shader/marching-cubes/active-voxels.frag");
/** name for shared framebuffer used for gpu marching cubes operations */
var FramebufferName = 'marching-cubes-active-voxels';
var ActiveVoxelsSchema = __assign({}, util_1.QuadSchema, { tTriCount: schema_1.TextureSpec('image-uint8', 'alpha', 'ubyte', 'nearest'), tVolumeData: schema_1.TextureSpec('texture', 'rgba', 'ubyte', 'nearest'), uIsoValue: schema_1.UniformSpec('f'), uGridDim: schema_1.UniformSpec('v3'), uGridTexDim: schema_1.UniformSpec('v3'), uScale: schema_1.UniformSpec('v2') });
function getActiveVoxelsRenderable(ctx, volumeData, gridDim, gridTexDim, isoValue, scale) {
    var values = __assign({}, util_1.QuadValues, { uQuadScale: mol_util_1.ValueCell.create(scale), tTriCount: mol_util_1.ValueCell.create(tables_1.getTriCount()), tVolumeData: mol_util_1.ValueCell.create(volumeData), uIsoValue: mol_util_1.ValueCell.create(isoValue), uGridDim: mol_util_1.ValueCell.create(gridDim), uGridTexDim: mol_util_1.ValueCell.create(gridTexDim), uScale: mol_util_1.ValueCell.create(scale) });
    var schema = __assign({}, ActiveVoxelsSchema);
    var shaderCode = shader_code_1.ShaderCode(quad_vert_1.default, active_voxels_frag_1.default);
    var renderItem = render_item_1.createComputeRenderItem(ctx, 'triangles', shaderCode, schema, values);
    return renderable_1.createComputeRenderable(renderItem, values);
}
function setRenderingDefaults(ctx) {
    var gl = ctx.gl, state = ctx.state;
    state.disable(gl.CULL_FACE);
    state.disable(gl.BLEND);
    state.disable(gl.DEPTH_TEST);
    state.disable(gl.SCISSOR_TEST);
    state.depthMask(false);
    state.colorMask(true, true, true, true);
    state.clearColor(0, 0, 0, 0);
}
function calcActiveVoxels(ctx, volumeData, gridDim, gridTexDim, isoValue, gridScale) {
    var gl = ctx.gl, framebufferCache = ctx.framebufferCache;
    var width = volumeData.width, height = volumeData.height;
    var framebuffer = framebufferCache.get(FramebufferName).value;
    framebuffer.bind();
    var activeVoxelsTex = texture_1.createTexture(ctx, 'image-float32', 'rgba', 'float', 'nearest');
    activeVoxelsTex.define(width, height);
    var renderable = getActiveVoxelsRenderable(ctx, volumeData, gridDim, gridTexDim, isoValue, gridScale);
    ctx.state.currentRenderItemId = -1;
    activeVoxelsTex.attachFramebuffer(framebuffer, 0);
    setRenderingDefaults(ctx);
    gl.viewport(0, 0, width, height);
    renderable.render();
    // console.log('gridScale', gridScale, 'gridTexDim', gridTexDim, 'gridDim', gridDim)
    // console.log('volumeData', volumeData)
    // console.log('at', readTexture(ctx, activeVoxelsTex))
    gl.finish();
    return activeVoxelsTex;
}
exports.calcActiveVoxels = calcActiveVoxels;
