"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../../mol-gl/renderable/util");
var mol_util_1 = require("../../mol-util");
var schema_1 = require("../../mol-gl/renderable/schema");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
exports.QuadPositions = new Float32Array([
    1.0, 1.0, -1.0, 1.0, -1.0, -1.0,
    -1.0, -1.0, 1.0, -1.0, 1.0, 1.0 // Second triangle
]);
exports.QuadSchema = {
    drawCount: schema_1.ValueSpec('number'),
    instanceCount: schema_1.ValueSpec('number'),
    aPosition: schema_1.AttributeSpec('float32', 2, 0),
    uQuadScale: schema_1.UniformSpec('v2'),
};
exports.QuadValues = {
    drawCount: mol_util_1.ValueCell.create(6),
    instanceCount: mol_util_1.ValueCell.create(1),
    aPosition: mol_util_1.ValueCell.create(exports.QuadPositions),
    uQuadScale: mol_util_1.ValueCell.create(linear_algebra_1.Vec2.create(1, 1)),
};
//
function getArrayForTexture(gl, texture, size) {
    switch (texture.type) {
        case gl.UNSIGNED_BYTE: return new Uint8Array(size);
        case gl.FLOAT: return new Float32Array(size);
    }
    throw new Error('unknown/unsupported texture type');
}
function readTexture(ctx, texture, width, height) {
    var gl = ctx.gl, framebufferCache = ctx.framebufferCache;
    width = mol_util_1.defaults(width, texture.width);
    height = mol_util_1.defaults(height, texture.height);
    var size = width * height * 4;
    var framebuffer = framebufferCache.get('read-texture').value;
    var array = getArrayForTexture(gl, texture, size);
    framebuffer.bind();
    texture.attachFramebuffer(framebuffer, 0);
    ctx.readPixels(0, 0, width, height, array);
    return { array: array, width: width, height: height };
}
exports.readTexture = readTexture;
function printTexture(ctx, texture, scale) {
    util_1.printTextureImage(readTexture(ctx, texture), scale);
}
exports.printTexture = printTexture;
