/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { RenderableState, Renderable } from './renderable';
import { RenderableValues } from './renderable/schema';
import { WebGLContext } from './webgl/context';
import { DirectVolumeValues } from './renderable/direct-volume';
import { MeshValues } from './renderable/mesh';
import { PointsValues } from './renderable/points';
import { LinesValues } from './renderable/lines';
import { SpheresValues } from './renderable/spheres';
import { TextValues } from './renderable/text';
import { TextureMeshValues } from './renderable/texture-mesh';
export declare const getNextMaterialId: () => number;
export interface BaseRenderObject<T extends RenderableValues> {
    id: number;
    type: string;
    values: T;
    state: RenderableState;
    materialId: number;
}
export interface MeshRenderObject extends BaseRenderObject<MeshValues> {
    type: 'mesh';
}
export interface PointsRenderObject extends BaseRenderObject<PointsValues> {
    type: 'points';
}
export interface SpheresRenderObject extends BaseRenderObject<SpheresValues> {
    type: 'spheres';
}
export interface TextRenderObject extends BaseRenderObject<TextValues> {
    type: 'text';
}
export interface LinesRenderObject extends BaseRenderObject<LinesValues> {
    type: 'lines';
}
export interface DirectVolumeRenderObject extends BaseRenderObject<DirectVolumeValues> {
    type: 'direct-volume';
}
export interface TextureMeshRenderObject extends BaseRenderObject<TextureMeshValues> {
    type: 'texture-mesh';
}
export declare type GraphicsRenderObject = MeshRenderObject | PointsRenderObject | SpheresRenderObject | TextRenderObject | LinesRenderObject | DirectVolumeRenderObject | TextureMeshRenderObject;
export declare type RenderObjectKindType = {
    'mesh': MeshRenderObject;
    'points': PointsRenderObject;
    'spheres': SpheresRenderObject;
    'text': TextRenderObject;
    'lines': LinesRenderObject;
    'direct-volume': DirectVolumeRenderObject;
    'texture-mesh': TextureMeshRenderObject;
};
export declare type RenderObjectValuesType = {
    'mesh': MeshValues;
    'points': PointsValues;
    'spheres': SpheresValues;
    'text': TextValues;
    'lines': LinesValues;
    'direct-volume': DirectVolumeValues;
    'texture-mesh': TextureMeshValues;
};
export declare type RenderObjectType = keyof RenderObjectKindType;
export declare function createRenderObject<T extends RenderObjectType>(type: T, values: RenderObjectValuesType[T], state: RenderableState, materialId: number): RenderObjectKindType[T];
export declare function createRenderable(ctx: WebGLContext, o: GraphicsRenderObject): Renderable<any>;
