/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Renderable, RenderableState } from '../renderable';
import { WebGLContext } from '../webgl/context';
import { AttributeSpec, Values, UniformSpec, TextureSpec, ValueSpec, ElementsSpec, DefineSpec } from './schema';
export declare const DirectVolumeSchema: {
    uColor: UniformSpec<"v3">;
    uColorTexDim: UniformSpec<"v2">;
    tColor: TextureSpec<"image-uint8">;
    dColorType: DefineSpec<"string">;
    uMarkerTexDim: UniformSpec<"v2">;
    tMarker: TextureSpec<"image-uint8">;
    uOverpaintTexDim: UniformSpec<"v2">;
    tOverpaint: TextureSpec<"image-uint8">;
    dOverpaint: DefineSpec<"boolean">;
    uTransparencyTexDim: UniformSpec<"v2">;
    tTransparency: TextureSpec<"image-uint8">;
    dTransparency: DefineSpec<"boolean">;
    dTransparencyVariant: DefineSpec<"string">;
    uInstanceCount: UniformSpec<"i">;
    uGroupCount: UniformSpec<"i">;
    aInstance: AttributeSpec<"float32">;
    aTransform: AttributeSpec<"float32">;
    drawCount: ValueSpec<"number">;
    instanceCount: ValueSpec<"number">;
    alpha: ValueSpec<"number">;
    matrix: ValueSpec<"m4">;
    transform: ValueSpec<"float32">;
    extraTransform: ValueSpec<"float32">;
    boundingSphere: ValueSpec<"sphere">;
    invariantBoundingSphere: ValueSpec<"sphere">;
    aPosition: AttributeSpec<"float32">;
    elements: ElementsSpec<"uint32">;
    uAlpha: UniformSpec<"f">;
    uHighlightColor: UniformSpec<"v3">;
    uSelectColor: UniformSpec<"v3">;
    dUseFog: DefineSpec<"boolean">;
    uIsoValue: UniformSpec<"f">;
    uBboxMin: UniformSpec<"v3">;
    uBboxMax: UniformSpec<"v3">;
    uBboxSize: UniformSpec<"v3">;
    dMaxSteps: DefineSpec<"number">;
    uTransform: UniformSpec<"m4">;
    uGridDim: UniformSpec<"v3">;
    dRenderMode: DefineSpec<"string">;
    tTransferTex: TextureSpec<"image-uint8">;
    dGridTexType: DefineSpec<"string">;
    uGridTexDim: UniformSpec<"v3">;
    tGridTex: TextureSpec<"texture">;
};
export declare type DirectVolumeSchema = typeof DirectVolumeSchema;
export declare type DirectVolumeValues = Values<DirectVolumeSchema>;
export declare function DirectVolumeRenderable(ctx: WebGLContext, id: number, values: DirectVolumeValues, state: RenderableState, materialId: number): Renderable<DirectVolumeValues>;
