"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var renderable_1 = require("../renderable");
var render_item_1 = require("../webgl/render-item");
var schema_1 = require("./schema");
var shader_code_1 = require("../shader-code");
var mol_util_1 = require("../../mol-util");
exports.DirectVolumeSchema = {
    uColor: schema_1.UniformSpec('v3'),
    uColorTexDim: schema_1.UniformSpec('v2'),
    tColor: schema_1.TextureSpec('image-uint8', 'rgb', 'ubyte', 'nearest'),
    dColorType: schema_1.DefineSpec('string', ['uniform', 'instance', 'group', 'group_instance']),
    uMarkerTexDim: schema_1.UniformSpec('v2'),
    tMarker: schema_1.TextureSpec('image-uint8', 'alpha', 'ubyte', 'nearest'),
    uOverpaintTexDim: schema_1.UniformSpec('v2'),
    tOverpaint: schema_1.TextureSpec('image-uint8', 'rgba', 'ubyte', 'nearest'),
    dOverpaint: schema_1.DefineSpec('boolean'),
    uTransparencyTexDim: schema_1.UniformSpec('v2'),
    tTransparency: schema_1.TextureSpec('image-uint8', 'alpha', 'ubyte', 'nearest'),
    dTransparency: schema_1.DefineSpec('boolean'),
    dTransparencyVariant: schema_1.DefineSpec('string', ['single', 'multi']),
    uInstanceCount: schema_1.UniformSpec('i'),
    uGroupCount: schema_1.UniformSpec('i'),
    aInstance: schema_1.AttributeSpec('float32', 1, 1),
    aTransform: schema_1.AttributeSpec('float32', 16, 1),
    drawCount: schema_1.ValueSpec('number'),
    instanceCount: schema_1.ValueSpec('number'),
    alpha: schema_1.ValueSpec('number'),
    matrix: schema_1.ValueSpec('m4'),
    transform: schema_1.ValueSpec('float32'),
    extraTransform: schema_1.ValueSpec('float32'),
    boundingSphere: schema_1.ValueSpec('sphere'),
    invariantBoundingSphere: schema_1.ValueSpec('sphere'),
    aPosition: schema_1.AttributeSpec('float32', 3, 0),
    elements: schema_1.ElementsSpec('uint32'),
    uAlpha: schema_1.UniformSpec('f'),
    uHighlightColor: schema_1.UniformSpec('v3'),
    uSelectColor: schema_1.UniformSpec('v3'),
    dUseFog: schema_1.DefineSpec('boolean'),
    uIsoValue: schema_1.UniformSpec('f'),
    uBboxMin: schema_1.UniformSpec('v3'),
    uBboxMax: schema_1.UniformSpec('v3'),
    uBboxSize: schema_1.UniformSpec('v3'),
    dMaxSteps: schema_1.DefineSpec('number'),
    uTransform: schema_1.UniformSpec('m4'),
    uGridDim: schema_1.UniformSpec('v3'),
    dRenderMode: schema_1.DefineSpec('string', ['isosurface', 'volume']),
    tTransferTex: schema_1.TextureSpec('image-uint8', 'rgba', 'ubyte', 'linear'),
    dGridTexType: schema_1.DefineSpec('string', ['2d', '3d']),
    uGridTexDim: schema_1.UniformSpec('v3'),
    tGridTex: schema_1.TextureSpec('texture', 'rgba', 'float', 'nearest'),
};
function DirectVolumeRenderable(ctx, id, values, state, materialId) {
    var schema = __assign({}, schema_1.GlobalUniformSchema, schema_1.InternalSchema, exports.DirectVolumeSchema);
    var internalValues = {
        uObjectId: mol_util_1.ValueCell.create(id),
        uPickable: mol_util_1.ValueCell.create(state.pickable ? 1 : 0),
    };
    var shaderCode = shader_code_1.DirectVolumeShaderCode;
    var renderItem = render_item_1.createGraphicsRenderItem(ctx, 'triangles', shaderCode, schema, __assign({}, values, internalValues), materialId);
    return renderable_1.createRenderable(renderItem, values, state);
}
exports.DirectVolumeRenderable = DirectVolumeRenderable;
