/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Renderable, RenderableState } from '../renderable';
import { WebGLContext } from '../webgl/context';
import { AttributeSpec, ElementsSpec, DefineSpec, Values } from './schema';
export declare const MeshSchema: {
    aPosition: AttributeSpec<"float32">;
    aNormal: AttributeSpec<"float32">;
    elements: ElementsSpec<"uint32">;
    dFlatShaded: DefineSpec<"boolean">;
    dDoubleSided: DefineSpec<"boolean">;
    dFlipSided: DefineSpec<"boolean">;
    aInstance: AttributeSpec<"float32">;
    aGroup: AttributeSpec<"float32">;
    aTransform: AttributeSpec<"float32">;
    uAlpha: import("./schema").UniformSpec<"f">;
    uInstanceCount: import("./schema").UniformSpec<"i">;
    uGroupCount: import("./schema").UniformSpec<"i">;
    uHighlightColor: import("./schema").UniformSpec<"v3">;
    uSelectColor: import("./schema").UniformSpec<"v3">;
    drawCount: import("./schema").ValueSpec<"number">;
    instanceCount: import("./schema").ValueSpec<"number">;
    alpha: import("./schema").ValueSpec<"number">;
    matrix: import("./schema").ValueSpec<"m4">;
    transform: import("./schema").ValueSpec<"float32">;
    extraTransform: import("./schema").ValueSpec<"float32">;
    boundingSphere: import("./schema").ValueSpec<"sphere">;
    invariantBoundingSphere: import("./schema").ValueSpec<"sphere">;
    dUseFog: DefineSpec<"boolean">;
    uTransparencyTexDim: import("./schema").UniformSpec<"v2">;
    tTransparency: import("./schema").TextureSpec<"image-uint8">;
    dTransparency: DefineSpec<"boolean">;
    dTransparencyVariant: DefineSpec<"string">;
    uOverpaintTexDim: import("./schema").UniformSpec<"v2">;
    tOverpaint: import("./schema").TextureSpec<"image-uint8">;
    dOverpaint: DefineSpec<"boolean">;
    uMarkerTexDim: import("./schema").UniformSpec<"v2">;
    tMarker: import("./schema").TextureSpec<"image-uint8">;
    uColor: import("./schema").UniformSpec<"v3">;
    uColorTexDim: import("./schema").UniformSpec<"v2">;
    tColor: import("./schema").TextureSpec<"image-uint8">;
    dColorType: DefineSpec<"string">;
};
export declare type MeshSchema = typeof MeshSchema;
export declare type MeshValues = Values<MeshSchema>;
export declare function MeshRenderable(ctx: WebGLContext, id: number, values: MeshValues, state: RenderableState, materialId: number): Renderable<MeshValues>;
