/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Renderable, RenderableState } from '../renderable';
import { WebGLContext } from '../webgl/context';
import { AttributeSpec, UniformSpec, DefineSpec, Values } from './schema';
export declare const PointsSchema: {
    aPosition: AttributeSpec<"float32">;
    dPointSizeAttenuation: DefineSpec<"boolean">;
    dPointFilledCircle: DefineSpec<"boolean">;
    uPointEdgeBleach: UniformSpec<"f">;
    uSize: UniformSpec<"f">;
    uSizeTexDim: UniformSpec<"v2">;
    tSize: import("./schema").TextureSpec<"image-uint8">;
    dSizeType: DefineSpec<"string">;
    uSizeFactor: UniformSpec<"f">;
    aInstance: AttributeSpec<"float32">;
    aGroup: AttributeSpec<"float32">;
    aTransform: AttributeSpec<"float32">;
    uAlpha: UniformSpec<"f">;
    uInstanceCount: UniformSpec<"i">;
    uGroupCount: UniformSpec<"i">;
    uHighlightColor: UniformSpec<"v3">;
    uSelectColor: UniformSpec<"v3">;
    drawCount: import("./schema").ValueSpec<"number">;
    instanceCount: import("./schema").ValueSpec<"number">;
    alpha: import("./schema").ValueSpec<"number">;
    matrix: import("./schema").ValueSpec<"m4">;
    transform: import("./schema").ValueSpec<"float32">;
    extraTransform: import("./schema").ValueSpec<"float32">;
    boundingSphere: import("./schema").ValueSpec<"sphere">;
    invariantBoundingSphere: import("./schema").ValueSpec<"sphere">;
    dUseFog: DefineSpec<"boolean">;
    uTransparencyTexDim: UniformSpec<"v2">;
    tTransparency: import("./schema").TextureSpec<"image-uint8">;
    dTransparency: DefineSpec<"boolean">;
    dTransparencyVariant: DefineSpec<"string">;
    uOverpaintTexDim: UniformSpec<"v2">;
    tOverpaint: import("./schema").TextureSpec<"image-uint8">;
    dOverpaint: DefineSpec<"boolean">;
    uMarkerTexDim: UniformSpec<"v2">;
    tMarker: import("./schema").TextureSpec<"image-uint8">;
    uColor: UniformSpec<"v3">;
    uColorTexDim: UniformSpec<"v2">;
    tColor: import("./schema").TextureSpec<"image-uint8">;
    dColorType: DefineSpec<"string">;
};
export declare type PointsSchema = typeof PointsSchema;
export declare type PointsValues = Values<PointsSchema>;
export declare function PointsRenderable(ctx: WebGLContext, id: number, values: PointsValues, state: RenderableState, materialId: number): Renderable<PointsValues>;
