/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Renderable, RenderableState } from '../renderable';
import { WebGLContext } from '../webgl/context';
import { AttributeSpec, Values, ElementsSpec, ValueSpec, DefineSpec } from './schema';
export declare const SpheresSchema: {
    aPosition: AttributeSpec<"float32">;
    aMapping: AttributeSpec<"float32">;
    elements: ElementsSpec<"uint32">;
    padding: ValueSpec<"number">;
    dDoubleSided: DefineSpec<"boolean">;
    uSize: import("./schema").UniformSpec<"f">;
    uSizeTexDim: import("./schema").UniformSpec<"v2">;
    tSize: import("./schema").TextureSpec<"image-uint8">;
    dSizeType: DefineSpec<"string">;
    uSizeFactor: import("./schema").UniformSpec<"f">;
    aInstance: AttributeSpec<"float32">;
    aGroup: AttributeSpec<"float32">;
    aTransform: AttributeSpec<"float32">;
    uAlpha: import("./schema").UniformSpec<"f">;
    uInstanceCount: import("./schema").UniformSpec<"i">;
    uGroupCount: import("./schema").UniformSpec<"i">;
    uHighlightColor: import("./schema").UniformSpec<"v3">;
    uSelectColor: import("./schema").UniformSpec<"v3">;
    drawCount: ValueSpec<"number">;
    instanceCount: ValueSpec<"number">;
    alpha: ValueSpec<"number">;
    matrix: ValueSpec<"m4">;
    transform: ValueSpec<"float32">;
    extraTransform: ValueSpec<"float32">;
    boundingSphere: ValueSpec<"sphere">;
    invariantBoundingSphere: ValueSpec<"sphere">;
    dUseFog: DefineSpec<"boolean">;
    uTransparencyTexDim: import("./schema").UniformSpec<"v2">;
    tTransparency: import("./schema").TextureSpec<"image-uint8">;
    dTransparency: DefineSpec<"boolean">;
    dTransparencyVariant: DefineSpec<"string">;
    uOverpaintTexDim: import("./schema").UniformSpec<"v2">;
    tOverpaint: import("./schema").TextureSpec<"image-uint8">;
    dOverpaint: DefineSpec<"boolean">;
    uMarkerTexDim: import("./schema").UniformSpec<"v2">;
    tMarker: import("./schema").TextureSpec<"image-uint8">;
    uColor: import("./schema").UniformSpec<"v3">;
    uColorTexDim: import("./schema").UniformSpec<"v2">;
    tColor: import("./schema").TextureSpec<"image-uint8">;
    dColorType: DefineSpec<"string">;
};
export declare type SpheresSchema = typeof SpheresSchema;
export declare type SpheresValues = Values<SpheresSchema>;
export declare function SpheresRenderable(ctx: WebGLContext, id: number, values: SpheresValues, state: RenderableState, materialId: number): Renderable<SpheresValues>;
