/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Renderable, RenderableState } from '../renderable';
import { WebGLContext } from '../webgl/context';
import { DefineSpec, Values, UniformSpec, TextureSpec } from './schema';
export declare const TextureMeshSchema: {
    uGeoTexDim: UniformSpec<"v2">;
    /** texture has vertex positions in XYZ and group id in W */
    tPositionGroup: TextureSpec<"texture">;
    tNormal: TextureSpec<"texture">;
    dFlatShaded: DefineSpec<"boolean">;
    dDoubleSided: DefineSpec<"boolean">;
    dFlipSided: DefineSpec<"boolean">;
    dGeoTexture: DefineSpec<"boolean">;
    aInstance: import("./schema").AttributeSpec<"float32">;
    aGroup: import("./schema").AttributeSpec<"float32">;
    aTransform: import("./schema").AttributeSpec<"float32">;
    uAlpha: UniformSpec<"f">;
    uInstanceCount: UniformSpec<"i">;
    uGroupCount: UniformSpec<"i">;
    uHighlightColor: UniformSpec<"v3">;
    uSelectColor: UniformSpec<"v3">;
    drawCount: import("./schema").ValueSpec<"number">;
    instanceCount: import("./schema").ValueSpec<"number">;
    alpha: import("./schema").ValueSpec<"number">;
    matrix: import("./schema").ValueSpec<"m4">;
    transform: import("./schema").ValueSpec<"float32">;
    extraTransform: import("./schema").ValueSpec<"float32">;
    boundingSphere: import("./schema").ValueSpec<"sphere">;
    invariantBoundingSphere: import("./schema").ValueSpec<"sphere">;
    dUseFog: DefineSpec<"boolean">;
    uTransparencyTexDim: UniformSpec<"v2">;
    tTransparency: TextureSpec<"image-uint8">;
    dTransparency: DefineSpec<"boolean">;
    dTransparencyVariant: DefineSpec<"string">;
    uOverpaintTexDim: UniformSpec<"v2">;
    tOverpaint: TextureSpec<"image-uint8">;
    dOverpaint: DefineSpec<"boolean">;
    uMarkerTexDim: UniformSpec<"v2">;
    tMarker: TextureSpec<"image-uint8">;
    uColor: UniformSpec<"v3">;
    uColorTexDim: UniformSpec<"v2">;
    tColor: TextureSpec<"image-uint8">;
    dColorType: DefineSpec<"string">;
};
export declare type TextureMeshSchema = typeof TextureMeshSchema;
export declare type TextureMeshValues = Values<TextureMeshSchema>;
export declare function TextureMeshRenderable(ctx: WebGLContext, id: number, values: TextureMeshValues, state: RenderableState, materialId: number): Renderable<TextureMeshValues>;
