/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Camera } from '../mol-canvas3d/camera';
import Scene from './scene';
import { WebGLContext } from './webgl/context';
import { GraphicsRenderVariant } from './webgl/render-item';
import { ParamDefinition as PD } from '../mol-util/param-definition';
export interface RendererStats {
    programCount: number;
    shaderCount: number;
    bufferCount: number;
    framebufferCount: number;
    renderbufferCount: number;
    textureCount: number;
    vaoCount: number;
    drawCount: number;
    instanceCount: number;
    instancedDrawCount: number;
}
interface Renderer {
    readonly stats: RendererStats;
    readonly props: Readonly<RendererProps>;
    clear: () => void;
    render: (scene: Scene, variant: GraphicsRenderVariant, clear: boolean) => void;
    setProps: (props: Partial<RendererProps>) => void;
    setViewport: (x: number, y: number, width: number, height: number) => void;
    dispose: () => void;
}
export declare const RendererParams: {
    backgroundColor: PD.Color;
    pickingAlphaThreshold: PD.Numeric;
    lightIntensity: PD.Numeric;
    ambientIntensity: PD.Numeric;
    metalness: PD.Numeric;
    roughness: PD.Numeric;
    reflectivity: PD.Numeric;
};
export declare type RendererProps = PD.Values<typeof RendererParams>;
declare namespace Renderer {
    function create(ctx: WebGLContext, camera: Camera, props?: Partial<RendererProps>): Renderer;
}
export default Renderer;
