/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ValueCell } from '../mol-util';
import { WebGLExtensions } from './webgl/extensions';
import { GLRenderingContext } from './webgl/compat';
export declare type DefineKind = 'boolean' | 'string' | 'number';
export declare type DefineType = boolean | string;
export declare type DefineValues = {
    [k: string]: ValueCell<DefineType>;
};
export interface ShaderExtensions {
    readonly standardDerivatives?: boolean;
    readonly fragDepth?: boolean;
    readonly drawBuffers?: boolean;
    readonly shaderTextureLod?: boolean;
}
export interface ShaderCode {
    readonly id: number;
    readonly vert: string;
    readonly frag: string;
    readonly extensions: ShaderExtensions;
}
export declare function ShaderCode(vert: string, frag: string, extensions?: ShaderExtensions): ShaderCode;
export declare const PointsShaderCode: ShaderCode;
export declare const SpheresShaderCode: ShaderCode;
export declare const TextShaderCode: ShaderCode;
export declare const LinesShaderCode: ShaderCode;
export declare const MeshShaderCode: ShaderCode;
export declare const DirectVolumeShaderCode: ShaderCode;
export declare type ShaderDefines = {
    [k: string]: ValueCell<DefineType>;
};
export declare function addShaderDefines(gl: GLRenderingContext, extensions: WebGLExtensions, defines: ShaderDefines, shaders: ShaderCode): ShaderCode;
