/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { WebGLContext } from './context';
import { ValueCell } from '../../mol-util';
import { RenderableSchema } from '../renderable/schema';
import { ValueOf } from '../../mol-util/type-helpers';
import { GLRenderingContext } from './compat';
export declare type UsageHint = 'static' | 'dynamic' | 'stream';
export declare type DataType = 'uint8' | 'int8' | 'uint16' | 'int16' | 'uint32' | 'int32' | 'float32';
export declare type BufferType = 'attribute' | 'elements' | 'uniform';
export declare type DataTypeArrayType = {
    'uint8': Uint8Array;
    'int8': Int8Array;
    'uint16': Uint16Array;
    'int16': Int16Array;
    'uint32': Uint32Array;
    'int32': Int32Array;
    'float32': Float32Array;
};
export declare type ArrayType = ValueOf<DataTypeArrayType>;
export declare type ArrayKind = keyof DataTypeArrayType;
export declare function getUsageHint(ctx: WebGLContext, usageHint: UsageHint): number;
export declare function getDataType(ctx: WebGLContext, dataType: DataType): number;
export declare function getBufferType(ctx: WebGLContext, bufferType: BufferType): number;
export interface Buffer {
    readonly id: number;
    readonly _buffer: WebGLBuffer;
    readonly _usageHint: number;
    readonly _bufferType: number;
    readonly _dataType: number;
    readonly _bpe: number;
    readonly length: number;
    updateData: (array: ArrayType) => void;
    updateSubData: (array: ArrayType, offset: number, count: number) => void;
    destroy: () => void;
}
export declare function createBuffer(ctx: WebGLContext, array: ArrayType, usageHint: UsageHint, bufferType: BufferType): Buffer;
export declare type AttributeItemSize = 1 | 2 | 3 | 4 | 16;
export declare type AttributeKind = 'float32' | 'int32';
export declare function getAttribType(gl: GLRenderingContext, kind: AttributeKind, itemSize: AttributeItemSize): number;
export declare type AttributeDefs = {
    [k: string]: {
        kind: AttributeKind;
        itemSize: AttributeItemSize;
        divisor: number;
    };
};
export declare type AttributeValues = {
    [k: string]: ValueCell<ArrayType>;
};
export declare type AttributeBuffers = [string, AttributeBuffer][];
export interface AttributeBuffer extends Buffer {
    bind: (location: number) => void;
}
export declare function createAttributeBuffer<T extends ArrayType, S extends AttributeItemSize>(ctx: WebGLContext, array: T, itemSize: S, divisor: number, usageHint?: UsageHint): AttributeBuffer;
export declare function createAttributeBuffers(ctx: WebGLContext, schema: RenderableSchema, values: AttributeValues): [string, AttributeBuffer][];
export declare type ElementsType = Uint16Array | Uint32Array;
export declare type ElementsKind = 'uint16' | 'uint32';
export interface ElementsBuffer extends Buffer {
    bind: () => void;
}
export declare function createElementsBuffer(ctx: WebGLContext, array: ElementsType, usageHint?: UsageHint): ElementsBuffer;
