/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ProgramCache } from './program';
import { ShaderCache } from './shader';
import { GLRenderingContext } from './compat';
import { FramebufferCache } from './framebuffer';
import { WebGLExtensions } from './extensions';
import { WebGLState } from './state';
import { PixelData } from '../../mol-util/image';
export declare function getGLContext(canvas: HTMLCanvasElement, contextAttributes?: WebGLContextAttributes): GLRenderingContext | null;
export declare function checkError(gl: GLRenderingContext): void;
export declare type WebGLStats = {
    bufferCount: number;
    framebufferCount: number;
    renderbufferCount: number;
    textureCount: number;
    vaoCount: number;
    drawCount: number;
    instanceCount: number;
    instancedDrawCount: number;
};
/** A WebGL context object, including the rendering context, resource caches and counts */
export interface WebGLContext {
    readonly gl: GLRenderingContext;
    readonly isWebGL2: boolean;
    readonly pixelRatio: number;
    readonly extensions: WebGLExtensions;
    readonly state: WebGLState;
    readonly stats: WebGLStats;
    readonly shaderCache: ShaderCache;
    readonly programCache: ProgramCache;
    readonly framebufferCache: FramebufferCache;
    readonly maxTextureSize: number;
    readonly maxDrawBuffers: number;
    unbindFramebuffer: () => void;
    readPixels: (x: number, y: number, width: number, height: number, buffer: Uint8Array | Float32Array) => void;
    readPixelsAsync: (x: number, y: number, width: number, height: number, buffer: Uint8Array) => Promise<void>;
    waitForGpuCommandsComplete: () => Promise<void>;
    waitForGpuCommandsCompleteSync: () => void;
    getDrawingBufferPixelData: () => PixelData;
    destroy: () => void;
}
export declare function createContext(gl: GLRenderingContext): WebGLContext;
