"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var program_1 = require("./program");
var shader_1 = require("./shader");
var compat_1 = require("./compat");
var framebuffer_1 = require("./framebuffer");
var mol_task_1 = require("../../mol-task");
var debug_1 = require("../../mol-util/debug");
var extensions_1 = require("./extensions");
var state_1 = require("./state");
var image_1 = require("../../mol-util/image");
function getGLContext(canvas, contextAttributes) {
    function getContext(contextId) {
        try {
            return canvas.getContext(contextId, contextAttributes);
        }
        catch (e) {
            return null;
        }
    }
    return getContext('webgl2') || getContext('webgl') || getContext('experimental-webgl');
}
exports.getGLContext = getGLContext;
function getPixelRatio() {
    return (typeof window !== 'undefined') ? window.devicePixelRatio : 1;
}
function getErrorDescription(gl, error) {
    switch (error) {
        case gl.NO_ERROR: return 'no error';
        case gl.INVALID_ENUM: return 'invalid enum';
        case gl.INVALID_VALUE: return 'invalid value';
        case gl.INVALID_OPERATION: return 'invalid operation';
        case gl.INVALID_FRAMEBUFFER_OPERATION: return 'invalid framebuffer operation';
        case gl.OUT_OF_MEMORY: return 'out of memory';
        case gl.CONTEXT_LOST_WEBGL: return 'context lost';
    }
    return 'unknown error';
}
function checkError(gl) {
    var error = gl.getError();
    if (error)
        throw new Error("WebGL error: '" + getErrorDescription(gl, error) + "'");
}
exports.checkError = checkError;
function unbindResources(gl) {
    // bind null to all texture units
    var maxTextureImageUnits = gl.getParameter(gl.MAX_TEXTURE_IMAGE_UNITS);
    for (var i = 0; i < maxTextureImageUnits; ++i) {
        gl.activeTexture(gl.TEXTURE0 + i);
        gl.bindTexture(gl.TEXTURE_2D, null);
        gl.bindTexture(gl.TEXTURE_CUBE_MAP, null);
        if (compat_1.isWebGL2(gl)) {
            gl.bindTexture(gl.TEXTURE_2D_ARRAY, null);
            gl.bindTexture(gl.TEXTURE_3D, null);
        }
    }
    // assign the smallest possible buffer to all attributes
    var buf = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, buf);
    var maxVertexAttribs = gl.getParameter(gl.MAX_VERTEX_ATTRIBS);
    for (var i = 0; i < maxVertexAttribs; ++i) {
        gl.vertexAttribPointer(i, 1, gl.FLOAT, false, 0, 0);
    }
    // bind null to all buffers
    gl.bindBuffer(gl.ARRAY_BUFFER, null);
    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, null);
    gl.bindRenderbuffer(gl.RENDERBUFFER, null);
    unbindFramebuffer(gl);
}
function unbindFramebuffer(gl) {
    gl.bindFramebuffer(gl.FRAMEBUFFER, null);
}
var tmpPixel = new Uint8Array(1 * 4);
function checkSync(gl, sync, resolve) {
    if (gl.getSyncParameter(sync, gl.SYNC_STATUS) === gl.SIGNALED) {
        gl.deleteSync(sync);
        resolve();
    }
    else {
        mol_task_1.Scheduler.setImmediate(checkSync, gl, sync, resolve);
    }
}
function fence(gl, resolve) {
    var sync = gl.fenceSync(gl.SYNC_GPU_COMMANDS_COMPLETE, 0);
    if (!sync) {
        console.warn('Could not create a WebGLSync object');
        gl.readPixels(0, 0, 1, 1, gl.RGBA, gl.UNSIGNED_BYTE, tmpPixel);
        resolve();
    }
    else {
        mol_task_1.Scheduler.setImmediate(checkSync, gl, sync, resolve);
    }
}
var SentWebglSyncObjectNotSupportedInWebglMessage = false;
function waitForGpuCommandsComplete(gl) {
    return new Promise(function (resolve) {
        if (compat_1.isWebGL2(gl)) {
            // TODO seems quite slow
            fence(gl, resolve);
        }
        else {
            if (!SentWebglSyncObjectNotSupportedInWebglMessage) {
                console.info('Sync object not supported in WebGL');
                SentWebglSyncObjectNotSupportedInWebglMessage = true;
            }
            waitForGpuCommandsCompleteSync(gl);
            resolve();
        }
    });
}
function waitForGpuCommandsCompleteSync(gl) {
    gl.bindFramebuffer(gl.FRAMEBUFFER, null);
    gl.readPixels(0, 0, 1, 1, gl.RGBA, gl.UNSIGNED_BYTE, tmpPixel);
}
function readPixels(gl, x, y, width, height, buffer) {
    if (debug_1.isDebugMode)
        framebuffer_1.checkFramebufferStatus(gl);
    if (buffer instanceof Uint8Array) {
        gl.readPixels(x, y, width, height, gl.RGBA, gl.UNSIGNED_BYTE, buffer);
    }
    else if (buffer instanceof Float32Array) {
        gl.readPixels(x, y, width, height, gl.RGBA, gl.FLOAT, buffer);
    }
    else {
        throw new Error('unsupported readPixels buffer type');
    }
    if (debug_1.isDebugMode)
        checkError(gl);
}
function getDrawingBufferPixelData(gl) {
    var w = gl.drawingBufferWidth;
    var h = gl.drawingBufferHeight;
    var buffer = new Uint8Array(w * h * 4);
    unbindFramebuffer(gl);
    gl.viewport(0, 0, w, h);
    readPixels(gl, 0, 0, w, h, buffer);
    return image_1.PixelData.flipY(image_1.PixelData.create(buffer, w, h));
}
function createStats() {
    return {
        bufferCount: 0,
        framebufferCount: 0,
        renderbufferCount: 0,
        textureCount: 0,
        vaoCount: 0,
        drawCount: 0,
        instanceCount: 0,
        instancedDrawCount: 0,
    };
}
function createContext(gl) {
    var _this = this;
    var extensions = extensions_1.createExtensions(gl);
    var state = state_1.createState(gl);
    var stats = createStats();
    var shaderCache = shader_1.createShaderCache(gl);
    var programCache = program_1.createProgramCache(gl, state, extensions, shaderCache);
    var framebufferCache = framebuffer_1.createFramebufferCache(gl, stats);
    var parameters = {
        maxTextureSize: gl.getParameter(gl.MAX_TEXTURE_SIZE),
        maxDrawBuffers: compat_1.isWebGL2(gl) ? gl.getParameter(gl.MAX_DRAW_BUFFERS) : 0,
        maxVertexTextureImageUnits: gl.getParameter(gl.MAX_VERTEX_TEXTURE_IMAGE_UNITS),
    };
    if (parameters.maxVertexTextureImageUnits < 8) {
        throw new Error('Need "MAX_VERTEX_TEXTURE_IMAGE_UNITS" >= 8');
    }
    var readPixelsAsync;
    if (compat_1.isWebGL2(gl)) {
        var pbo_1 = gl.createBuffer();
        var _buffer_1 = void 0;
        var _resolve_1 = void 0;
        var _reading_1 = false;
        var bindPBO_1 = function () {
            gl.bindBuffer(gl.PIXEL_PACK_BUFFER, pbo_1);
            gl.getBufferSubData(gl.PIXEL_PACK_BUFFER, 0, _buffer_1);
            gl.bindBuffer(gl.PIXEL_PACK_BUFFER, null);
            _reading_1 = false;
            _resolve_1();
            _resolve_1 = void 0;
            _buffer_1 = void 0;
        };
        readPixelsAsync = function (x, y, width, height, buffer) { return new Promise(function (resolve, reject) {
            if (_reading_1) {
                reject('Can not call multiple readPixelsAsync at the same time');
                return;
            }
            _reading_1 = true;
            gl.bindBuffer(gl.PIXEL_PACK_BUFFER, pbo_1);
            gl.bufferData(gl.PIXEL_PACK_BUFFER, width * height * 4, gl.STREAM_READ);
            gl.readPixels(x, y, width, height, gl.RGBA, gl.UNSIGNED_BYTE, 0);
            gl.bindBuffer(gl.PIXEL_PACK_BUFFER, null);
            // need to unbind/bind PBO before/after async awaiting the fence
            _resolve_1 = resolve;
            _buffer_1 = buffer;
            fence(gl, bindPBO_1);
        }); };
    }
    else {
        readPixelsAsync = function (x, y, width, height, buffer) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                readPixels(gl, x, y, width, height, buffer);
                return [2 /*return*/];
            });
        }); };
    }
    return {
        gl: gl,
        isWebGL2: compat_1.isWebGL2(gl),
        get pixelRatio() {
            // this can change during the lifetime of a rendering context, so need to re-obtain on access
            return getPixelRatio();
        },
        extensions: extensions,
        state: state,
        stats: stats,
        shaderCache: shaderCache,
        programCache: programCache,
        framebufferCache: framebufferCache,
        get maxTextureSize() { return parameters.maxTextureSize; },
        get maxDrawBuffers() { return parameters.maxDrawBuffers; },
        unbindFramebuffer: function () { return unbindFramebuffer(gl); },
        readPixels: function (x, y, width, height, buffer) {
            readPixels(gl, x, y, width, height, buffer);
        },
        readPixelsAsync: readPixelsAsync,
        waitForGpuCommandsComplete: function () { return waitForGpuCommandsComplete(gl); },
        waitForGpuCommandsCompleteSync: function () { return waitForGpuCommandsCompleteSync(gl); },
        getDrawingBufferPixelData: function () { return getDrawingBufferPixelData(gl); },
        destroy: function () {
            unbindResources(gl);
            programCache.dispose();
            shaderCache.dispose();
            framebufferCache.dispose();
            // TODO destroy buffers and textures
        }
    };
}
exports.createContext = createContext;
