"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var compat_1 = require("./compat");
function createExtensions(gl) {
    var instancedArrays = compat_1.getInstancedArrays(gl);
    if (instancedArrays === null) {
        throw new Error('Could not find support for "instanced_arrays"');
    }
    var textureFloat = compat_1.getTextureFloat(gl);
    if (textureFloat === null) {
        throw new Error('Could not find support for "texture_float"');
    }
    var elementIndexUint = compat_1.getElementIndexUint(gl);
    if (elementIndexUint === null) {
        throw new Error('Could not find support for "element_index_uint"');
    }
    var standardDerivatives = compat_1.getStandardDerivatives(gl);
    if (standardDerivatives === null) {
        // TODO handle non-support downstream (e.g. no flat shading)
        // throw new Error('Could not find support for "standard_derivatives"')
        console.log('Could not find support for "standard_derivatives"');
    }
    var textureFloatLinear = compat_1.getTextureFloatLinear(gl);
    if (textureFloatLinear === null) {
        // TODO handle non-support downstream (no gpu gaussian calc, no gpu mc???)
        // throw new Error('Could not find support for "texture_float_linear"')
        console.log('Could not find support for "texture_float_linear"');
    }
    var depthTexture = compat_1.getDepthTexture(gl);
    if (depthTexture === null) {
        console.log('Could not find support for "depth_texture"');
    }
    var blendMinMax = compat_1.getBlendMinMax(gl);
    if (blendMinMax === null) {
        console.log('Could not find support for "blend_minmax"');
    }
    var vertexArrayObject = compat_1.getVertexArrayObject(gl);
    if (vertexArrayObject === null) {
        console.log('Could not find support for "vertex_array_object"');
    }
    var fragDepth = compat_1.getFragDepth(gl);
    if (fragDepth === null) {
        console.log('Could not find support for "frag_depth"');
    }
    var colorBufferFloat = compat_1.getColorBufferFloat(gl);
    if (colorBufferFloat === null) {
        console.log('Could not find support for "color_buffer_float"');
    }
    var drawBuffers = compat_1.getDrawBuffers(gl);
    if (drawBuffers === null) {
        console.log('Could not find support for "draw_buffers"');
    }
    var shaderTextureLod = compat_1.getShaderTextureLod(gl);
    if (shaderTextureLod === null) {
        console.log('Could not find support for "shader_texture_lod"');
    }
    return {
        instancedArrays: instancedArrays,
        standardDerivatives: standardDerivatives,
        textureFloat: textureFloat,
        textureFloatLinear: textureFloatLinear,
        elementIndexUint: elementIndexUint,
        depthTexture: depthTexture,
        blendMinMax: blendMinMax,
        vertexArrayObject: vertexArrayObject,
        fragDepth: fragDepth,
        colorBufferFloat: colorBufferFloat,
        drawBuffers: drawBuffers,
        shaderTextureLod: shaderTextureLod,
    };
}
exports.createExtensions = createExtensions;
