"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var id_factory_1 = require("../../mol-util/id-factory");
var reference_cache_1 = require("../../mol-util/reference-cache");
var compat_1 = require("./compat");
var getNextFramebufferId = id_factory_1.idFactory();
function getFramebufferStatusDescription(gl, status) {
    switch (status) {
        case gl.FRAMEBUFFER_COMPLETE: return 'complete';
        case gl.FRAMEBUFFER_INCOMPLETE_ATTACHMENT: return 'incomplete attachment';
        case gl.FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: return 'incomplete missing attachment';
        case gl.FRAMEBUFFER_INCOMPLETE_DIMENSIONS: return 'incomplete dimensions';
        case gl.FRAMEBUFFER_UNSUPPORTED: return 'unsupported';
    }
    if (compat_1.isWebGL2(gl)) {
        switch (status) {
            case gl.FRAMEBUFFER_INCOMPLETE_MULTISAMPLE: return 'incomplete multisample';
            case gl.RENDERBUFFER_SAMPLES: return 'renderbuffer samples';
        }
    }
    return 'unknown error';
}
function checkFramebufferStatus(gl) {
    var status = gl.checkFramebufferStatus(gl.FRAMEBUFFER);
    if (status !== gl.FRAMEBUFFER_COMPLETE) {
        var description = getFramebufferStatusDescription(gl, status);
        throw new Error("Framebuffer status: " + description);
    }
}
exports.checkFramebufferStatus = checkFramebufferStatus;
function createFramebuffer(gl, stats) {
    var _framebuffer = gl.createFramebuffer();
    if (_framebuffer === null) {
        throw new Error('Could not create WebGL framebuffer');
    }
    var destroyed = false;
    stats.framebufferCount += 1;
    return {
        id: getNextFramebufferId(),
        bind: function () { return gl.bindFramebuffer(gl.FRAMEBUFFER, _framebuffer); },
        destroy: function () {
            if (destroyed)
                return;
            gl.deleteFramebuffer(_framebuffer);
            destroyed = true;
            stats.framebufferCount -= 1;
        }
    };
}
exports.createFramebuffer = createFramebuffer;
function createFramebufferCache(gl, stats) {
    return reference_cache_1.createReferenceCache(function (name) { return name; }, function () { return createFramebuffer(gl, stats); }, function (framebuffer) { framebuffer.destroy(); });
}
exports.createFramebufferCache = createFramebufferCache;
