/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ShaderCode, DefineValues } from '../shader-code';
import { WebGLState } from './state';
import { WebGLExtensions } from './extensions';
import { UniformsList } from './uniform';
import { AttributeBuffers } from './buffer';
import { Textures } from './texture';
import { ReferenceCache } from '../../mol-util/reference-cache';
import { RenderableSchema } from '../renderable/schema';
import { GLRenderingContext } from './compat';
import { ShaderCache } from './shader';
export interface Program {
    readonly id: number;
    use: () => void;
    setUniforms: (uniformValues: UniformsList) => void;
    bindAttributes: (attribueBuffers: AttributeBuffers) => void;
    bindTextures: (textures: Textures) => void;
    destroy: () => void;
}
export interface ProgramProps {
    defineValues: DefineValues;
    shaderCode: ShaderCode;
    schema: RenderableSchema;
}
export declare function createProgram(gl: GLRenderingContext, state: WebGLState, extensions: WebGLExtensions, shaderCache: ShaderCache, props: ProgramProps): Program;
export declare type ProgramCache = ReferenceCache<Program, ProgramProps>;
export declare function createProgramCache(gl: GLRenderingContext, state: WebGLState, extensions: WebGLExtensions, shaderCache: ShaderCache): ProgramCache;
