/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { WebGLContext } from './context';
import { ShaderCode } from '../shader-code';
import { Program } from './program';
import { RenderableSchema, RenderableValues } from '../renderable/schema';
export declare type DrawMode = 'points' | 'lines' | 'line-strip' | 'line-loop' | 'triangles' | 'triangle-strip' | 'triangle-fan';
export declare function getDrawMode(ctx: WebGLContext, drawMode: DrawMode): number;
export interface RenderItem<T extends string> {
    readonly id: number;
    readonly materialId: number;
    getProgram: (variant: T) => Program;
    render: (variant: T) => void;
    update: () => Readonly<ValueChanges>;
    destroy: () => void;
}
declare const GraphicsRenderVariantDefines: {
    'color': {};
    'pickObject': {
        dColorType: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<string, never>>;
    };
    'pickInstance': {
        dColorType: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<string, never>>;
    };
    'pickGroup': {
        dColorType: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<string, never>>;
    };
    'depth': {
        dColorType: import("../../mol-util/value-cell").ValueRef<import("../../mol-util/value-cell").ValueBox<string, never>>;
    };
};
export declare type GraphicsRenderVariant = keyof typeof GraphicsRenderVariantDefines;
declare const ComputeRenderVariantDefines: {
    'compute': {};
};
export declare type ComputeRenderVariant = keyof typeof ComputeRenderVariantDefines;
declare type RenderVariantDefines = typeof GraphicsRenderVariantDefines | typeof ComputeRenderVariantDefines;
interface ValueChanges {
    attributes: boolean;
    defines: boolean;
    elements: boolean;
    textures: boolean;
}
export declare type GraphicsRenderItem = RenderItem<keyof typeof GraphicsRenderVariantDefines & string>;
export declare function createGraphicsRenderItem(ctx: WebGLContext, drawMode: DrawMode, shaderCode: ShaderCode, schema: RenderableSchema, values: RenderableValues, materialId: number): RenderItem<"color" | "depth" | "pickObject" | "pickInstance" | "pickGroup">;
export declare type ComputeRenderItem = RenderItem<keyof typeof ComputeRenderVariantDefines & string>;
export declare function createComputeRenderItem(ctx: WebGLContext, drawMode: DrawMode, shaderCode: ShaderCode, schema: RenderableSchema, values: RenderableValues, materialId?: number): RenderItem<"compute">;
/**
 * Creates a render item
 *
 * - assumes that `values.drawCount` and `values.instanceCount` exist
 */
export declare function createRenderItem<T extends RenderVariantDefines, S extends keyof T & string>(ctx: WebGLContext, drawMode: DrawMode, shaderCode: ShaderCode, schema: RenderableSchema, values: RenderableValues, materialId: number, renderVariantDefines: T): RenderItem<S>;
export {};
