"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var id_factory_1 = require("../../mol-util/id-factory");
var texture_1 = require("./texture");
var framebuffer_1 = require("./framebuffer");
var renderbuffer_1 = require("./renderbuffer");
var image_1 = require("../../mol-util/image");
var getNextRenderTargetId = id_factory_1.idFactory();
function createRenderTarget(ctx, _width, _height) {
    var gl = ctx.gl, stats = ctx.stats;
    var image = {
        array: new Uint8Array(_width * _height * 4),
        width: _width,
        height: _height
    };
    var targetTexture = texture_1.createTexture(ctx, 'image-uint8', 'rgba', 'ubyte', 'linear');
    targetTexture.load(image);
    var framebuffer = framebuffer_1.createFramebuffer(gl, stats);
    // attach the texture as the first color attachment
    targetTexture.attachFramebuffer(framebuffer, 'color0');
    // make a depth renderbuffer of the same size as the targetTexture
    var depthRenderbuffer = renderbuffer_1.createRenderbuffer(ctx, 'depth16', 'depth', _width, _height);
    var destroyed = false;
    function readBuffer(x, y, width, height, dst) {
        framebuffer.bind();
        gl.viewport(0, 0, _width, _height);
        ctx.readPixels(x, y, width, height, dst);
    }
    function getBuffer() {
        readBuffer(0, 0, _width, _height, image.array);
        return image.array;
    }
    return {
        id: getNextRenderTargetId(),
        get width() { return _width; },
        get height() { return _height; },
        image: image,
        texture: targetTexture,
        framebuffer: framebuffer,
        bind: function () {
            framebuffer.bind();
            gl.viewport(0, 0, _width, _height);
        },
        setSize: function (width, height) {
            _width = width;
            _height = height;
            image.array = new Uint8Array(_width * _height * 4);
            image.width = _width;
            image.height = _height;
            targetTexture.load(image);
            depthRenderbuffer.setSize(_width, _height);
        },
        readBuffer: readBuffer,
        getBuffer: getBuffer,
        getPixelData: function () { return image_1.PixelData.flipY(image_1.PixelData.create(getBuffer(), _width, _height)); },
        destroy: function () {
            if (destroyed)
                return;
            targetTexture.destroy();
            framebuffer.destroy();
            depthRenderbuffer.destroy();
            destroyed = true;
        }
    };
}
exports.createRenderTarget = createRenderTarget;
