"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var id_factory_1 = require("../../mol-util/id-factory");
var getNextRenderbufferId = id_factory_1.idFactory();
function getFormat(ctx, format) {
    var gl = ctx.gl;
    switch (format) {
        case 'depth16': return gl.DEPTH_COMPONENT16;
        case 'stencil8': return gl.STENCIL_INDEX8;
        case 'rgba4': return gl.RGBA4;
        case 'depth-stencil': return gl.DEPTH_STENCIL;
    }
}
exports.getFormat = getFormat;
function getAttachment(ctx, attachment) {
    var gl = ctx.gl;
    switch (attachment) {
        case 'depth': return gl.DEPTH_ATTACHMENT;
        case 'stencil': return gl.STENCIL_ATTACHMENT;
        case 'depth-stencil': return gl.DEPTH_STENCIL_ATTACHMENT;
        case 'color0': return gl.COLOR_ATTACHMENT0;
    }
}
exports.getAttachment = getAttachment;
function createRenderbuffer(ctx, format, attachment, _width, _height) {
    var gl = ctx.gl, stats = ctx.stats;
    var _renderbuffer = gl.createRenderbuffer();
    if (_renderbuffer === null) {
        throw new Error('Could not create WebGL renderbuffer');
    }
    var bind = function () { return gl.bindRenderbuffer(gl.RENDERBUFFER, _renderbuffer); };
    var _format = getFormat(ctx, format);
    var _attachment = getAttachment(ctx, attachment);
    bind();
    gl.renderbufferStorage(gl.RENDERBUFFER, _format, _width, _height);
    gl.framebufferRenderbuffer(gl.FRAMEBUFFER, _attachment, gl.RENDERBUFFER, _renderbuffer);
    var destroyed = false;
    stats.renderbufferCount += 1;
    return {
        id: getNextRenderbufferId(),
        bind: bind,
        setSize: function (_width, _height) {
            bind();
            gl.renderbufferStorage(gl.RENDERBUFFER, _format, _width, _height);
        },
        destroy: function () {
            if (destroyed)
                return;
            gl.deleteRenderbuffer(_renderbuffer);
            destroyed = true;
            stats.framebufferCount -= 1;
        }
    };
}
exports.createRenderbuffer = createRenderbuffer;
