"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var reference_cache_1 = require("../../mol-util/reference-cache");
var id_factory_1 = require("../../mol-util/id-factory");
var debug_1 = require("../../mol-util/debug");
var getNextShaderId = id_factory_1.idFactory();
function addLineNumbers(source) {
    var lines = source.split('\n');
    for (var i = 0; i < lines.length; ++i) {
        lines[i] = (i + 1) + ': ' + lines[i];
    }
    return lines.join('\n');
}
function createShader(gl, props) {
    var type = props.type, source = props.source;
    var shader = gl.createShader(type === 'vert' ? gl.VERTEX_SHADER : gl.FRAGMENT_SHADER);
    if (shader === null) {
        throw new Error("Error creating " + type + " shader");
    }
    gl.shaderSource(shader, source);
    gl.compileShader(shader);
    if (debug_1.isDebugMode && gl.getShaderParameter(shader, gl.COMPILE_STATUS) === false) {
        console.warn("'" + type + "' shader info log '" + gl.getShaderInfoLog(shader) + "'\n" + addLineNumbers(source));
        throw new Error("Error compiling " + type + " shader");
    }
    return {
        id: getNextShaderId(),
        attach: function (program) {
            gl.attachShader(program, shader);
        },
        destroy: function () {
            gl.deleteShader(shader);
        }
    };
}
function createShaderCache(gl) {
    return reference_cache_1.createReferenceCache(function (props) { return JSON.stringify(props); }, function (props) { return createShader(gl, props); }, function (shader) { shader.destroy(); });
}
exports.createShaderCache = createShaderCache;
