"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
function getUniformType(gl, kind) {
    switch (kind) {
        case 'f': return gl.FLOAT;
        case 'i': return gl.INT;
        case 'v2': return gl.FLOAT_VEC2;
        case 'v3': return gl.FLOAT_VEC3;
        case 'v4': return gl.FLOAT_VEC4;
        case 'm3': return gl.FLOAT_MAT3;
        case 'm4': return gl.FLOAT_MAT4;
        default: console.error("unknown uniform kind '" + kind + "'");
    }
}
exports.getUniformType = getUniformType;
function setUniform(gl, location, kind, value) {
    switch (kind) {
        case 'f':
            gl.uniform1f(location, value);
            break;
        case 'i':
        case 't':
            gl.uniform1i(location, value);
            break;
        case 'v2':
            gl.uniform2fv(location, value);
            break;
        case 'v3':
            gl.uniform3fv(location, value);
            break;
        case 'v4':
            gl.uniform4fv(location, value);
            break;
        case 'm3':
            gl.uniformMatrix3fv(location, false, value);
            break;
        case 'm4':
            gl.uniformMatrix4fv(location, false, value);
            break;
        default: console.error("unknown uniform kind '" + kind + "'");
    }
}
exports.setUniform = setUniform;
function uniform1f(gl, location, value) { gl.uniform1f(location, value); }
function uniform1i(gl, location, value) { gl.uniform1i(location, value); }
function uniform2fv(gl, location, value) { gl.uniform2fv(location, value); }
function uniform3fv(gl, location, value) { gl.uniform3fv(location, value); }
function uniform4fv(gl, location, value) { gl.uniform4fv(location, value); }
function uniformMatrix3fv(gl, location, value) { gl.uniformMatrix3fv(location, false, value); }
function uniformMatrix4fv(gl, location, value) { gl.uniformMatrix4fv(location, false, value); }
function getUniformSetter(kind) {
    switch (kind) {
        case 'f': return uniform1f;
        case 'i':
        case 't': return uniform1i;
        case 'v2': return uniform2fv;
        case 'v3': return uniform3fv;
        case 'v4': return uniform4fv;
        case 'm3': return uniformMatrix3fv;
        case 'm4': return uniformMatrix4fv;
    }
    throw new Error("unknown uniform kind '" + kind + "'");
}
function getUniformSetters(schema) {
    var setters = {};
    Object.keys(schema).forEach(function (k) {
        var spec = schema[k];
        if (spec.type === 'uniform') {
            setters[k] = getUniformSetter(spec.kind);
        }
        else if (spec.type === 'texture') {
            setters[k] = getUniformSetter('t');
        }
    });
    return setters;
}
exports.getUniformSetters = getUniformSetters;
