"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
function createVertexArray(ctx, program, attributeBuffers, elementsBuffer) {
    var vertexArrayObject = ctx.extensions.vertexArrayObject;
    var vertexArray = null;
    if (vertexArrayObject) {
        vertexArray = vertexArrayObject.createVertexArray();
        if (vertexArray) {
            updateVertexArray(ctx, vertexArray, program, attributeBuffers, elementsBuffer);
            ctx.stats.vaoCount += 1;
        }
        else {
            console.warn('Could not create WebGL vertex array');
        }
    }
    return vertexArray;
}
exports.createVertexArray = createVertexArray;
function updateVertexArray(ctx, vertexArray, program, attributeBuffers, elementsBuffer) {
    var vertexArrayObject = ctx.extensions.vertexArrayObject;
    if (vertexArrayObject && vertexArray) {
        vertexArrayObject.bindVertexArray(vertexArray);
        if (elementsBuffer)
            elementsBuffer.bind();
        program.bindAttributes(attributeBuffers);
        vertexArrayObject.bindVertexArray(null);
    }
}
exports.updateVertexArray = updateVertexArray;
function deleteVertexArray(ctx, vertexArray) {
    var vertexArrayObject = ctx.extensions.vertexArrayObject;
    if (vertexArrayObject && vertexArray) {
        vertexArrayObject.deleteVertexArray(vertexArray);
        ctx.stats.vaoCount -= 1;
    }
}
exports.deleteVertexArray = deleteVertexArray;
