"use strict";
/**
 * Copyright (c) 2017 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var parser_1 = require("../gro/parser");
var groString = "MD of 2 waters, t= 4.2\n    6\n    1WATER  OW1    1   0.126   1.624   1.679  0.1227 -0.0580  0.0434\n    1WATER  HW2    2   0.190   1.661   1.747  0.8085  0.3191 -0.7791\n    1WATER  HW3    3   0.177   1.568   1.613 -0.9045 -2.6469  1.3180\n    2WATER  OW1    4   1.275   0.053   0.622  0.2519  0.3140 -0.1734\n    2WATER  HW2    5   1.337   0.002   0.680 -1.0641 -1.1349  0.0257\n    2WATER  HW3    6   1.326   0.120   0.568  1.9427 -0.8216 -0.0244\n   1.82060   2.82060   3.82060";
var groStringHighPrecision = "Generated by trjconv : 2168 system t=  15.00000\n    3\n    1ETH     C1    1   2.735383   2.672010   1.450194  0.2345 -0.1622 0.2097\n    1ETH    H11    2   0.015804   2.716597   1.460588  0.8528 -0.7984 0.6605\n    1ETH    H12    3   2.744822   2.565544   1.409227 -2.3812  2.8618 1.8101\n   1.82060   2.82060   3.82060";
describe('gro reader', function () {
    it('basic', function () { return __awaiter(_this, void 0, void 0, function () {
        var parsed, groFile, data, header, atoms;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, parser_1.parseGRO(groString).run()];
                case 1:
                    parsed = _a.sent();
                    if (parsed.isError) {
                        console.log(parsed);
                        return [2 /*return*/];
                    }
                    groFile = parsed.result;
                    data = groFile.structures[0];
                    header = data.header, atoms = data.atoms;
                    expect(header.title).toBe('MD of 2 waters');
                    expect(header.timeInPs).toBeCloseTo(4.2);
                    expect(header.hasVelocities).toBe(true);
                    expect(header.precision.position).toBe(3);
                    expect(header.precision.velocity).toBe(4);
                    expect(header.box[0]).toBeCloseTo(1.82060, 0.00001);
                    expect(header.box[1]).toBeCloseTo(2.82060, 0.00001);
                    expect(header.box[2]).toBeCloseTo(3.82060, 0.00001);
                    expect(atoms.count).toBe(6);
                    expect(atoms.x.value(0)).toBeCloseTo(0.126, 0.001);
                    expect(atoms.y.value(0)).toBeCloseTo(1.624, 0.001);
                    expect(atoms.z.value(0)).toBeCloseTo(1.679, 0.001);
                    expect(atoms.vx.value(5)).toBeCloseTo(1.9427, 0.0001);
                    expect(atoms.vy.value(5)).toBeCloseTo(-0.8216, 0.0001);
                    expect(atoms.vz.value(5)).toBeCloseTo(-0.0244, 0.0001);
                    return [2 /*return*/];
            }
        });
    }); });
    it('high precision', function () { return __awaiter(_this, void 0, void 0, function () {
        var parsed, groFile, data, header, atoms;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, parser_1.parseGRO(groStringHighPrecision).run()];
                case 1:
                    parsed = _a.sent();
                    if (parsed.isError) {
                        console.log(parsed);
                        return [2 /*return*/];
                    }
                    groFile = parsed.result;
                    data = groFile.structures[0];
                    header = data.header, atoms = data.atoms;
                    expect(header.title).toBe('Generated by trjconv : 2168 system');
                    expect(header.timeInPs).toBeCloseTo(15);
                    expect(header.hasVelocities).toBe(true);
                    expect(header.precision.position).toBe(6);
                    expect(header.precision.velocity).toBe(4);
                    expect(header.box[0]).toBeCloseTo(1.82060, 0.00001);
                    expect(header.box[1]).toBeCloseTo(2.82060, 0.00001);
                    expect(header.box[2]).toBeCloseTo(3.82060, 0.00001);
                    expect(atoms.count).toBe(3);
                    expect(atoms.x.value(1)).toBeCloseTo(0.015804, 0.000001);
                    expect(atoms.y.value(1)).toBeCloseTo(2.716597, 0.000001);
                    expect(atoms.z.value(1)).toBeCloseTo(1.460588, 0.000001);
                    expect(atoms.vx.value(0)).toBeCloseTo(0.2345, 0.0001);
                    expect(atoms.vy.value(0)).toBeCloseTo(-0.1622, 0.0001);
                    expect(atoms.vz.value(0)).toBeCloseTo(0.2097, 0.0001);
                    return [2 /*return*/];
            }
        });
    }); });
});
