"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var _3d_1 = require("../../mol-math/linear-algebra/3d");
var CentroidHelper = /** @class */ (function () {
    function CentroidHelper() {
        this.count = 0;
        this.center = _3d_1.Vec3.zero();
        this.radiusSq = 0;
    }
    CentroidHelper.prototype.reset = function () {
        _3d_1.Vec3.set(this.center, 0, 0, 0);
        this.radiusSq = 0;
        this.count = 0;
    };
    CentroidHelper.prototype.includeStep = function (p) {
        _3d_1.Vec3.add(this.center, this.center, p);
        this.count++;
    };
    CentroidHelper.prototype.finishedIncludeStep = function () {
        if (this.count === 0)
            return;
        _3d_1.Vec3.scale(this.center, this.center, 1 / this.count);
    };
    CentroidHelper.prototype.radiusStep = function (p) {
        var d = _3d_1.Vec3.squaredDistance(p, this.center);
        if (d > this.radiusSq)
            this.radiusSq = d;
    };
    return CentroidHelper;
}());
exports.CentroidHelper = CentroidHelper;
(function (CentroidHelper) {
    var helper = new CentroidHelper(), p = _3d_1.Vec3.zero();
    function compute(_a, to) {
        var x = _a.x, y = _a.y, z = _a.z;
        helper.reset();
        var n = x.length;
        for (var i = 0; i < n; i++) {
            _3d_1.Vec3.set(p, x[i], y[i], z[i]);
            helper.includeStep(p);
        }
        helper.finishedIncludeStep();
        for (var i = 0; i < n; i++) {
            _3d_1.Vec3.set(p, x[i], y[i], z[i]);
            helper.radiusStep(p);
        }
        _3d_1.Vec3.copy(to, helper.center);
        return to;
    }
    CentroidHelper.compute = compute;
})(CentroidHelper || (CentroidHelper = {}));
exports.CentroidHelper = CentroidHelper;
