/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Vec3, Mat4 } from '../../linear-algebra';
import { PositionData } from '../common';
interface Box3D {
    min: Vec3;
    max: Vec3;
}
declare function Box3D(): Box3D;
declare namespace Box3D {
    function create(min: Vec3, max: Vec3): Box3D;
    function empty(): Box3D;
    function clone(a: Box3D): Box3D;
    function computeBounding(data: PositionData): Box3D;
    /** Get size/extent of the box */
    function size(size: Vec3, box: Box3D): Vec3;
    /** Get size of the box */
    function volume(box: Box3D): number;
    function setEmpty(box: Box3D): Box3D;
    /** Add point to box */
    function add(box: Box3D, point: Vec3): Box3D;
    /** Expand box by delta */
    function expand(out: Box3D, box: Box3D, delta: Vec3): Box3D;
    function scale(out: Box3D, box: Box3D, scale: number): Box3D;
    /** Transform box with a Mat4 */
    function transform(out: Box3D, box: Box3D, m: Mat4): Box3D;
}
export { Box3D };
