/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Vec3, Mat4 } from '../../linear-algebra';
import { PositionData } from '../common';
import { NumberArray } from '../../../mol-util/type-helpers';
import { Box3D } from './box3d';
interface Sphere3D {
    center: Vec3;
    radius: number;
}
declare function Sphere3D(): Sphere3D;
declare namespace Sphere3D {
    function create(center: Vec3, radius: number): Sphere3D;
    function zero(): Sphere3D;
    function clone(a: Sphere3D): Sphere3D;
    function copy(out: Sphere3D, a: Sphere3D): Sphere3D;
    function computeBounding(data: PositionData): Sphere3D;
    /** Transform sphere with a Mat4 */
    function transform(out: Sphere3D, sphere: Sphere3D, m: Mat4): Sphere3D;
    function toArray(s: Sphere3D, out: NumberArray, offset: number): void;
    function fromArray(out: Sphere3D, array: NumberArray, offset: number): Sphere3D;
    function fromBox3D(out: Sphere3D, box: Box3D): Sphere3D;
    function addVec3(out: Sphere3D, s: Sphere3D, v: Vec3): Sphere3D;
    /** Expand sphere radius by another sphere */
    function expandBySphere(out: Sphere3D, sphere: Sphere3D): Sphere3D;
    /** Expand sphere radius by delta */
    function expand(out: Sphere3D, sphere: Sphere3D, delta: number): Sphere3D;
    /**
     * Returns whether or not the spheres have exactly the same center and radius (when compared with ===)
     */
    function exactEquals(a: Sphere3D, b: Sphere3D): boolean;
    /**
     * Returns whether or not the spheres have approximately the same center and radius.
     */
    function equals(a: Sphere3D, b: Sphere3D): boolean;
}
export { Sphere3D };
