/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { Vec3, Mat4 } from '../../linear-algebra';
import { SpacegroupName } from './tables';
import { SymmetryOperator } from '../../geometry';
interface SpacegroupCell {
    readonly index: number;
    readonly size: Vec3;
    readonly anglesInRadians: Vec3;
    /** Transfrom cartesian -> fractional coordinates within the cell */
    readonly toFractional: Mat4;
    /** Transfrom fractional coordinates within the cell -> cartesian */
    readonly fromFractional: Mat4;
}
interface Spacegroup {
    readonly name: string;
    readonly cell: SpacegroupCell;
    readonly operators: ReadonlyArray<Mat4>;
}
declare namespace SpacegroupCell {
    /** Create a 'P 1' with cellsize [1, 1, 1] */
    const Zero: SpacegroupCell;
    /** True if 'P 1' with cellsize [1, 1, 1] */
    function isZero(cell: SpacegroupCell): boolean;
    function create(nameOrNumber: number | string | SpacegroupName, size: Vec3, anglesInRadians: Vec3): SpacegroupCell;
}
declare namespace Spacegroup {
    const ZeroP1: Spacegroup;
    function create(cell: SpacegroupCell): Spacegroup;
    function updateOperatorMatrix(spacegroup: Spacegroup, index: number, i: number, j: number, k: number, target: Mat4): Mat4;
    function getSymmetryOperator(spacegroup: Spacegroup, index: number, i: number, j: number, k: number): SymmetryOperator;
}
export { Spacegroup, SpacegroupCell };
