/**
 * Copyright (c) 2017-2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Mat4 } from '../3d';
import { NumberArray } from '../../../mol-util/type-helpers';
interface Mat3 extends Array<number> {
    [d: number]: number;
    '@type': 'mat3';
    length: 9;
}
declare function Mat3(): Mat3;
declare namespace Mat3 {
    function zero(): Mat3;
    function identity(): Mat3;
    function setIdentity(mat: Mat3): Mat3;
    function toArray(a: Mat3, out: NumberArray, offset: number): void;
    function fromArray(a: Mat3, array: NumberArray, offset: number): Mat3;
    /**
     * Copies the upper-left 3x3 values into the given mat3.
     */
    function fromMat4(out: Mat3, a: Mat4): Mat3;
    function hasNaN(m: Mat3): boolean;
    /**
     * Creates a new Mat3 initialized with values from an existing matrix
     */
    function clone(a: Mat3): Mat3;
    function setValue(a: Mat3, i: number, j: number, value: number): void;
    function getValue(a: Mat3, i: number, j: number): number;
    /**
     * Copy the values from one Mat3 to another
     */
    function copy(out: Mat3, a: Mat3): Mat3;
    /**
     * Transpose the values of a Mat3
     */
    function transpose(out: Mat3, a: Mat3): Mat3;
    /**
     * Inverts a Mat3
     */
    function invert(out: Mat3, a: Mat3): Mat3;
    function determinant(a: Mat3): number;
}
export default Mat3;
