/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { NumberArray } from '../../../mol-util/type-helpers';
interface Matrix {
    data: NumberArray;
    size: number;
    cols: number;
    rows: number;
}
declare namespace Matrix {
    function create(cols: number, rows: number, ctor?: {
        new (size: number): NumberArray;
    }): Matrix;
    /** Get element assuming data are stored in column-major order */
    function get(m: Matrix, i: number, j: number): number;
    /** Set element assuming data are stored in column-major order */
    function set(m: Matrix, i: number, j: number, value: number): void;
    /** Add to element assuming data are stored in column-major order */
    function add(m: Matrix, i: number, j: number, value: number): void;
    /** Zero out the matrix */
    function makeZero(m: Matrix): void;
    function fromArray(data: NumberArray, cols: number, rows: number): Matrix;
    function transpose(out: Matrix, mat: Matrix): Matrix;
    /** out = matA * matB' */
    function multiplyABt(out: Matrix, matA: Matrix, matB: Matrix): Matrix;
    /** Get the mean of rows in `mat` */
    function meanRows(mat: Matrix): any[];
    /** Subtract `row` from all rows in `mat` */
    function subRows(mat: Matrix, row: NumberArray): Matrix;
}
export default Matrix;
