"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var matrix_1 = require("./matrix");
var _3d_1 = require("../3d");
// import { Vec3, Mat4 } from '../3d.js';
var svd_1 = require("./svd");
// const negateVector = Vec3.create(-1, -1, -1)
// const tmpMatrix = Mat4.identity()
/**
 * Principal axes
 */
var PrincipalAxes = /** @class */ (function () {
    /**
     * points is a 3xN matrix
     */
    function PrincipalAxes(points) {
        var n = points.rows;
        var n3 = n / 3;
        var pointsT = matrix_1.default.create(n, 3);
        var A = matrix_1.default.create(3, 3);
        var W = matrix_1.default.create(1, 3);
        var U = matrix_1.default.create(3, 3);
        var V = matrix_1.default.create(3, 3);
        // calculate
        var mean = matrix_1.default.meanRows(points);
        matrix_1.default.subRows(points, mean);
        matrix_1.default.transpose(pointsT, points);
        matrix_1.default.multiplyABt(A, pointsT, pointsT);
        svd_1.svd(A, W, U, V);
        // center
        var vm = _3d_1.Vec3.create(mean[0], mean[1], mean[2]);
        // normalized
        var van = _3d_1.Vec3.create(U.data[0], U.data[3], U.data[6]);
        var vbn = _3d_1.Vec3.create(U.data[1], U.data[4], U.data[7]);
        var vcn = _3d_1.Vec3.create(U.data[2], U.data[5], U.data[8]);
        // scaled
        var va = _3d_1.Vec3.scale(_3d_1.Vec3.zero(), van, Math.sqrt(W.data[0] / n3));
        var vb = _3d_1.Vec3.scale(_3d_1.Vec3.zero(), vbn, Math.sqrt(W.data[1] / n3));
        var vc = _3d_1.Vec3.scale(_3d_1.Vec3.zero(), vcn, Math.sqrt(W.data[2] / n3));
        // const va = van.clone().multiplyScalar(Math.sqrt(W.data[0] / n3))
        // const vb = vbn.clone().multiplyScalar(Math.sqrt(W.data[1] / n3))
        // const vc = vcn.clone().multiplyScalar(Math.sqrt(W.data[2] / n3))
        // points
        this.begA = _3d_1.Vec3.sub(_3d_1.Vec3.clone(vm), vm, va);
        this.endA = _3d_1.Vec3.add(_3d_1.Vec3.clone(vm), vm, va);
        this.begB = _3d_1.Vec3.sub(_3d_1.Vec3.clone(vm), vm, vb);
        this.endB = _3d_1.Vec3.add(_3d_1.Vec3.clone(vm), vm, vb);
        this.begC = _3d_1.Vec3.sub(_3d_1.Vec3.clone(vm), vm, vc);
        this.endC = _3d_1.Vec3.add(_3d_1.Vec3.clone(vm), vm, vc);
        // this.begA = vm.clone().sub(va)
        // this.endA = vm.clone().add(va)
        // this.begB = vm.clone().sub(vb)
        // this.endB = vm.clone().add(vb)
        // this.begC = vm.clone().sub(vc)
        // this.endC = vm.clone().add(vc)
        //
        this.center = vm;
        this.vecA = va;
        this.vecB = vb;
        this.vecC = vc;
        this.normVecA = van;
        this.normVecB = vbn;
        this.normVecC = vcn;
    }
    return PrincipalAxes;
}());
exports.default = PrincipalAxes;
