"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_task_1 = require("../../mol-task");
var format_1 = require("./format");
var parser_1 = require("./mmcif/parser");
var cif_1 = require("../../mol-io/reader/cif");
var db_1 = require("../../mol-data/db");
var util_1 = require("./util");
// TODO multi model files
// TODO seperate chains
// TODO better entity handling
// TODO improve performance
function _entity() {
    return {
        id: cif_1.CifField.ofStrings(['1', '2', '3']),
        type: cif_1.CifField.ofStrings(['polymer', 'non-polymer', 'water'])
    };
}
function _atom_site(atoms) {
    var auth_asym_id = cif_1.CifField.ofColumn(db_1.Column.Undefined(atoms.count, db_1.Column.Schema.str));
    var auth_atom_id = cif_1.CifField.ofColumn(atoms.atomName);
    var auth_comp_id = cif_1.CifField.ofColumn(atoms.residueName);
    var auth_seq_id = cif_1.CifField.ofColumn(atoms.residueNumber);
    return {
        auth_asym_id: auth_asym_id,
        auth_atom_id: auth_atom_id,
        auth_comp_id: auth_comp_id,
        auth_seq_id: auth_seq_id,
        B_iso_or_equiv: cif_1.CifField.ofColumn(db_1.Column.Undefined(atoms.count, db_1.Column.Schema.float)),
        Cartn_x: cif_1.CifField.ofNumbers(db_1.Column.mapToArray(atoms.x, function (x) { return x * 10; }, Float32Array)),
        Cartn_y: cif_1.CifField.ofNumbers(db_1.Column.mapToArray(atoms.y, function (y) { return y * 10; }, Float32Array)),
        Cartn_z: cif_1.CifField.ofNumbers(db_1.Column.mapToArray(atoms.z, function (z) { return z * 10; }, Float32Array)),
        group_PDB: cif_1.CifField.ofColumn(db_1.Column.Undefined(atoms.count, db_1.Column.Schema.str)),
        id: cif_1.CifField.ofColumn(atoms.atomNumber),
        label_alt_id: cif_1.CifField.ofColumn(db_1.Column.Undefined(atoms.count, db_1.Column.Schema.str)),
        label_asym_id: auth_asym_id,
        label_atom_id: auth_atom_id,
        label_comp_id: auth_comp_id,
        label_seq_id: auth_seq_id,
        label_entity_id: cif_1.CifField.ofColumn(db_1.Column.ofConst('1', atoms.count, db_1.Column.Schema.str)),
        occupancy: cif_1.CifField.ofColumn(db_1.Column.ofConst(1, atoms.count, db_1.Column.Schema.float)),
        type_symbol: cif_1.CifField.ofStrings(db_1.Column.mapToArray(atoms.atomName, function (s) { return util_1.guessElementSymbolString(s); })),
        // type_symbol: CifField.ofColumn(Column.Undefined(atoms.count, Column.Schema.str)),
        pdbx_PDB_ins_code: cif_1.CifField.ofColumn(db_1.Column.Undefined(atoms.count, db_1.Column.Schema.str)),
        pdbx_PDB_model_num: cif_1.CifField.ofColumn(db_1.Column.ofConst('1', atoms.count, db_1.Column.Schema.str)),
    };
}
function groToMmCif(gro) {
    return __awaiter(this, void 0, void 0, function () {
        var categories;
        return __generator(this, function (_a) {
            categories = {
                entity: cif_1.CifCategory.ofFields('entity', _entity()),
                atom_site: cif_1.CifCategory.ofFields('atom_site', _atom_site(gro.structures[0].atoms))
            };
            return [2 /*return*/, {
                    header: 'GRO',
                    categoryNames: Object.keys(categories),
                    categories: categories
                }];
        });
    });
}
function trajectoryFromGRO(gro) {
    var _this = this;
    return mol_task_1.Task.create('Parse GRO', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
        var cif, format;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, ctx.update('Converting to mmCIF')];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, groToMmCif(gro)];
                case 2:
                    cif = _a.sent();
                    format = format_1.ModelFormat.mmCIF(cif);
                    return [2 /*return*/, parser_1._parse_mmCif(format, ctx)];
            }
        });
    }); });
}
exports.trajectoryFromGRO = trajectoryFromGRO;
