"use strict";
/**
 * Copyright (c) 2018 Mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../util");
function findAtomIndex(model, entityId, asymId, seqId, atomId) {
    if (!model.atomicHierarchy.atoms.auth_atom_id.isDefined)
        return -1;
    var residueIndex = model.atomicHierarchy.index.findResidue(entityId, asymId, seqId);
    if (residueIndex < 0)
        return -1;
    return util_1.findAtomIndexByLabelName(model, residueIndex, atomId, '');
}
var IHMCrossLinkRestraint;
(function (IHMCrossLinkRestraint) {
    IHMCrossLinkRestraint.PropName = '__CrossLinkRestraint__';
    function fromModel(model) {
        if (model._staticPropertyData[IHMCrossLinkRestraint.PropName])
            return model._staticPropertyData[IHMCrossLinkRestraint.PropName];
        if (model.sourceData.kind !== 'mmCIF')
            return;
        var ihm_cross_link_restraint = model.sourceData.data.ihm_cross_link_restraint;
        if (!ihm_cross_link_restraint._rowCount)
            return;
        var p1 = {
            entity_id: ihm_cross_link_restraint.entity_id_1,
            asym_id: ihm_cross_link_restraint.asym_id_1,
            seq_id: ihm_cross_link_restraint.seq_id_1,
            atom_id: ihm_cross_link_restraint.atom_id_1,
        };
        var p2 = {
            entity_id: ihm_cross_link_restraint.entity_id_2,
            asym_id: ihm_cross_link_restraint.asym_id_2,
            seq_id: ihm_cross_link_restraint.seq_id_2,
            atom_id: ihm_cross_link_restraint.atom_id_2,
        };
        function _add(map, element, row) {
            var indices = map.get(element);
            if (indices)
                indices.push(row);
            else
                map.set(element, [row]);
        }
        function add(row, ps) {
            var entityId = ps.entity_id.value(row);
            var asymId = ps.asym_id.value(row);
            var seqId = ps.seq_id.value(row);
            if (ihm_cross_link_restraint.model_granularity.value(row) === 'by-atom') {
                var atomicElement = findAtomIndex(model, entityId, asymId, seqId, ps.atom_id.value(row));
                if (atomicElement >= 0)
                    _add(atomicElementMap, atomicElement, row);
            }
            else if (model.coarseHierarchy.isDefined) {
                var sphereElement = model.coarseHierarchy.spheres.findSequenceKey(entityId, asymId, seqId);
                if (sphereElement >= 0) {
                    _add(sphereElementMap, sphereElement, row);
                }
                else {
                    var gaussianElement = model.coarseHierarchy.gaussians.findSequenceKey(entityId, asymId, seqId);
                    if (gaussianElement >= 0)
                        _add(gaussianElementMap, gaussianElement, row);
                }
            }
        }
        function getMapByKind(kind) {
            switch (kind) {
                case 0 /* Atomic */: return atomicElementMap;
                case 1 /* Spheres */: return sphereElementMap;
                case 2 /* Gaussians */: return gaussianElementMap;
            }
        }
        /** map from atomic element to cross link indices */
        var atomicElementMap = new Map();
        /** map from sphere element to cross link indices */
        var sphereElementMap = new Map();
        /** map from gaussian element to cross link indices */
        var gaussianElementMap = new Map();
        var emptyIndexArray = [];
        for (var i = 0; i < ihm_cross_link_restraint._rowCount; ++i) {
            add(i, p1);
            add(i, p2);
        }
        var crossLinkRestraint = {
            getIndicesByElement: function (element, kind) {
                var map = getMapByKind(kind);
                var idx = map.get(element);
                return idx !== undefined ? idx : emptyIndexArray;
            },
            data: ihm_cross_link_restraint
        };
        model._staticPropertyData[IHMCrossLinkRestraint.PropName] = crossLinkRestraint;
        return crossLinkRestraint;
    }
    IHMCrossLinkRestraint.fromModel = fromModel;
})(IHMCrossLinkRestraint = exports.IHMCrossLinkRestraint || (exports.IHMCrossLinkRestraint = {}));
