"use strict";
/**
 * Copyright (c) 2017-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var db_1 = require("../../../mol-data/db");
var mmcif_1 = require("../../../mol-io/reader/cif/schema/mmcif");
var geometry_1 = require("../../../mol-math/geometry");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var uuid_1 = require("../../../mol-util/uuid");
var structure_1 = require("../../../mol-model/structure");
var assembly_1 = require("./assembly");
var atomic_1 = require("./atomic");
var bonds_1 = require("./bonds");
var ihm_1 = require("./ihm");
var secondary_structure_1 = require("./secondary-structure");
var sequence_1 = require("./sequence");
var sort_1 = require("./sort");
var struct_conn_1 = require("./bonds/struct_conn");
var types_1 = require("../../../mol-model/structure/model/types");
var constants_1 = require("../../../mol-model/structure/structure/carbohydrates/constants");
var memoize_1 = require("../../../mol-util/memoize");
function _parse_mmCif(format, ctx) {
    return __awaiter(this, void 0, void 0, function () {
        var formatData, isIHM, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    formatData = getFormatData(format);
                    isIHM = format.data.ihm_model_list._rowCount > 0;
                    if (!isIHM) return [3 /*break*/, 2];
                    return [4 /*yield*/, readIHM(ctx, format, formatData)];
                case 1:
                    _a = _b.sent();
                    return [3 /*break*/, 4];
                case 2: return [4 /*yield*/, readStandard(ctx, format, formatData)];
                case 3:
                    _a = _b.sent();
                    _b.label = 4;
                case 4: return [2 /*return*/, _a];
            }
        });
    });
}
exports._parse_mmCif = _parse_mmCif;
function getSymmetry(format) {
    var assemblies = assembly_1.createAssemblies(format);
    var spacegroup = getSpacegroup(format);
    var isNonStandardCrytalFrame = checkNonStandardCrystalFrame(format, spacegroup);
    return { assemblies: assemblies, spacegroup: spacegroup, isNonStandardCrytalFrame: isNonStandardCrytalFrame, ncsOperators: getNcsOperators(format) };
}
function checkNonStandardCrystalFrame(format, spacegroup) {
    var atom_sites = format.data.atom_sites;
    if (atom_sites._rowCount === 0)
        return false;
    // TODO: parse atom_sites transform and check if it corresponds to the toFractional matrix
    return false;
}
function getSpacegroup(format) {
    var _a = format.data, symmetry = _a.symmetry, cell = _a.cell;
    if (symmetry._rowCount === 0 || cell._rowCount === 0)
        return geometry_1.Spacegroup.ZeroP1;
    var groupName = symmetry['space_group_name_H-M'].value(0);
    var spaceCell = geometry_1.SpacegroupCell.create(groupName, linear_algebra_1.Vec3.create(cell.length_a.value(0), cell.length_b.value(0), cell.length_c.value(0)), linear_algebra_1.Vec3.scale(linear_algebra_1.Vec3.zero(), linear_algebra_1.Vec3.create(cell.angle_alpha.value(0), cell.angle_beta.value(0), cell.angle_gamma.value(0)), Math.PI / 180));
    return geometry_1.Spacegroup.create(spaceCell);
}
function getNcsOperators(format) {
    var struct_ncs_oper = format.data.struct_ncs_oper;
    if (struct_ncs_oper._rowCount === 0)
        return void 0;
    var id = struct_ncs_oper.id, matrix = struct_ncs_oper.matrix, vector = struct_ncs_oper.vector;
    var matrixSpace = mmcif_1.mmCIF_Schema.struct_ncs_oper.matrix.space, vectorSpace = mmcif_1.mmCIF_Schema.struct_ncs_oper.vector.space;
    var opers = [];
    for (var i = 0; i < struct_ncs_oper._rowCount; i++) {
        var m = linear_algebra_1.Tensor.toMat3(matrixSpace, matrix.value(i));
        var v = linear_algebra_1.Tensor.toVec3(vectorSpace, vector.value(i));
        if (!geometry_1.SymmetryOperator.checkIfRotationAndTranslation(m, v))
            continue;
        var ncsId = id.value(i);
        opers[opers.length] = geometry_1.SymmetryOperator.ofRotationAndOffset("ncs_" + ncsId, m, v, ncsId);
    }
    return opers;
}
function getModifiedResidueNameMap(format) {
    var data = format.data.pdbx_struct_mod_residue;
    var parentId = new Map();
    var details = new Map();
    var comp_id = data.label_comp_id.isDefined ? data.label_comp_id : data.auth_comp_id;
    var parent_id = data.parent_comp_id, details_data = data.details;
    for (var i = 0; i < data._rowCount; i++) {
        var id = comp_id.value(i);
        parentId.set(id, parent_id.value(i));
        details.set(id, details_data.value(i));
    }
    return { parentId: parentId, details: details };
}
function getMissingResidues(format) {
    var map = new Map();
    var c = format.data.pdbx_unobs_or_zero_occ_residues;
    var getKey = function (model_num, asym_id, seq_id) {
        return model_num + "|" + asym_id + "|" + seq_id;
    };
    for (var i = 0, il = c._rowCount; i < il; ++i) {
        var key = getKey(c.PDB_model_num.value(i), c.label_asym_id.value(i), c.label_seq_id.value(i));
        map.set(key, { polymer_flag: c.polymer_flag.value(i), occupancy_flag: c.occupancy_flag.value(i) });
    }
    return {
        has: function (model_num, asym_id, seq_id) {
            return map.has(getKey(model_num, asym_id, seq_id));
        },
        get: function (model_num, asym_id, seq_id) {
            return map.get(getKey(model_num, asym_id, seq_id));
        },
        size: map.size
    };
}
function getChemicalComponentMap(format) {
    var map = new Map();
    var chem_comp = format.data.chem_comp;
    var id = chem_comp.id;
    for (var i = 0, il = id.rowCount; i < il; ++i) {
        map.set(id.value(i), db_1.Table.getRow(chem_comp, i));
    }
    return map;
}
function getSaccharideComponentMap(format) {
    var map = new Map();
    var pdbx_chem_comp_identifier = format.data.pdbx_chem_comp_identifier;
    if (pdbx_chem_comp_identifier._rowCount > 0) {
        var comp_id = pdbx_chem_comp_identifier.comp_id, type = pdbx_chem_comp_identifier.type, identifier = pdbx_chem_comp_identifier.identifier;
        for (var i = 0, il = pdbx_chem_comp_identifier._rowCount; i < il; ++i) {
            if (type.value(i) === 'SNFG CARB SYMBOL') {
                var snfgName = identifier.value(i);
                var saccharideComp = constants_1.SaccharidesSnfgMap.get(snfgName);
                if (saccharideComp) {
                    map.set(comp_id.value(i), saccharideComp);
                }
                else {
                    console.warn("Unknown SNFG name '" + snfgName + "'");
                }
            }
        }
    }
    else if (format.data.chem_comp._rowCount > 0) {
        var _a = format.data.chem_comp, id = _a.id, type = _a.type;
        for (var i = 0, il = id.rowCount; i < il; ++i) {
            var _id = id.value(i);
            var _type = type.value(i);
            if (constants_1.SaccharideCompIdMap.has(_id)) {
                map.set(_id, constants_1.SaccharideCompIdMap.get(_id));
            }
            else if (!map.has(_id) && types_1.getMoleculeType(_type, _id) === 8 /* saccharide */) {
                map.set(_id, constants_1.UnknownSaccharideComponent);
            }
        }
    }
    else {
        var uniqueNames_1 = getUniqueComponentNames(format);
        constants_1.SaccharideCompIdMap.forEach(function (v, k) {
            if (uniqueNames_1.has(k))
                map.set(k, v);
        });
    }
    return map;
}
var getUniqueComponentNames = memoize_1.memoize1(function (format) {
    var uniqueNames = new Set();
    var data = format.data.atom_site;
    var comp_id = data.label_comp_id.isDefined ? data.label_comp_id : data.auth_comp_id;
    for (var i = 0, il = comp_id.rowCount; i < il; ++i) {
        uniqueNames.add(comp_id.value(i));
    }
    return uniqueNames;
});
function getFormatData(format) {
    return {
        modifiedResidues: getModifiedResidueNameMap(format),
        missingResidues: getMissingResidues(format),
        chemicalComponentMap: getChemicalComponentMap(format),
        saccharideComponentMap: getSaccharideComponentMap(format)
    };
}
function createStandardModel(format, atom_site, sourceIndex, entities, formatData, previous) {
    var atomic = atomic_1.getAtomicHierarchyAndConformation(atom_site, sourceIndex, entities, formatData, previous);
    var modelNum = atom_site.pdbx_PDB_model_num.value(0);
    if (previous && atomic.sameAsPrevious) {
        return __assign({}, previous, { id: uuid_1.default.create22(), modelNum: modelNum, atomicConformation: atomic.conformation, _dynamicPropertyData: Object.create(null) });
    }
    var coarse = ihm_1.EmptyIHMCoarse;
    var label = format.data.entry.id.valueKind(0) === 0 /* Present */
        ? format.data.entry.id.value(0)
        : format.data._name;
    return {
        id: uuid_1.default.create22(),
        label: label,
        entry: label,
        sourceData: format,
        modelNum: modelNum,
        entities: entities,
        symmetry: getSymmetry(format),
        sequence: sequence_1.getSequence(format.data, entities, atomic.hierarchy, formatData.modifiedResidues.parentId),
        atomicHierarchy: atomic.hierarchy,
        atomicConformation: atomic.conformation,
        coarseHierarchy: coarse.hierarchy,
        coarseConformation: coarse.conformation,
        properties: __assign({ secondaryStructure: secondary_structure_1.getSecondaryStructure(format.data, atomic.hierarchy) }, formatData),
        customProperties: new structure_1.CustomProperties(),
        _staticPropertyData: Object.create(null),
        _dynamicPropertyData: Object.create(null)
    };
}
function createModelIHM(format, data, formatData) {
    var atomic = atomic_1.getAtomicHierarchyAndConformation(data.atom_site, data.atom_site_sourceIndex, data.entities, formatData);
    var coarse = ihm_1.getIHMCoarse(data, formatData);
    var entry = format.data.entry.id.valueKind(0) === 0 /* Present */
        ? format.data.entry.id.value(0)
        : format.data._name;
    var label = data.model_group_name ? data.model_name + ": " + data.model_group_name : data.model_name;
    return {
        id: uuid_1.default.create22(),
        label: label,
        entry: entry,
        sourceData: format,
        modelNum: data.model_id,
        entities: data.entities,
        symmetry: getSymmetry(format),
        sequence: sequence_1.getSequence(format.data, data.entities, atomic.hierarchy, formatData.modifiedResidues.parentId),
        atomicHierarchy: atomic.hierarchy,
        atomicConformation: atomic.conformation,
        coarseHierarchy: coarse.hierarchy,
        coarseConformation: coarse.conformation,
        properties: __assign({ secondaryStructure: secondary_structure_1.getSecondaryStructure(format.data, atomic.hierarchy) }, formatData),
        customProperties: new structure_1.CustomProperties(),
        _staticPropertyData: Object.create(null),
        _dynamicPropertyData: Object.create(null)
    };
}
function attachProps(model) {
    bonds_1.ComponentBond.attachFromMmCif(model);
    struct_conn_1.StructConn.attachFromMmCif(model);
}
function findModelEnd(num, startIndex) {
    var rowCount = num.rowCount;
    if (!num.isDefined)
        return rowCount;
    var endIndex = startIndex + 1;
    while (endIndex < rowCount && num.areValuesEqual(startIndex, endIndex))
        endIndex++;
    return endIndex;
}
function getEntities(format) {
    var entityData;
    if (!format.data.entity.id.isDefined) {
        var entityIds = new Set();
        var entityList = [];
        var _a = format.data.atom_site, label_entity_id = _a.label_entity_id, label_comp_id = _a.label_comp_id;
        for (var i = 0, il = format.data.atom_site._rowCount; i < il; i++) {
            var entityId = label_entity_id.value(i);
            if (!entityIds.has(entityId)) {
                entityList.push({ id: entityId, type: types_1.getEntityType(label_comp_id.value(i)) });
                entityIds.add(entityId);
            }
        }
        var sphere_entity_id = format.data.ihm_sphere_obj_site.entity_id;
        for (var i = 0, il = format.data.ihm_sphere_obj_site._rowCount; i < il; i++) {
            var entityId = sphere_entity_id.value(i);
            if (!entityIds.has(entityId)) {
                entityList.push({ id: entityId, type: 'polymer' });
                entityIds.add(entityId);
            }
        }
        var gaussian_entity_id = format.data.ihm_gaussian_obj_site.entity_id;
        for (var i = 0, il = format.data.ihm_gaussian_obj_site._rowCount; i < il; i++) {
            var entityId = gaussian_entity_id.value(i);
            if (!entityIds.has(entityId)) {
                entityList.push({ id: entityId, type: 'polymer' });
                entityIds.add(entityId);
            }
        }
        entityData = db_1.Table.ofRows(mmcif_1.mmCIF_Schema.entity, entityList);
    }
    else {
        entityData = format.data.entity;
    }
    return { data: entityData, getEntityIndex: db_1.Column.createIndexer(entityData.id) };
}
function readStandard(ctx, format, formatData) {
    return __awaiter(this, void 0, void 0, function () {
        var atomCount, entities, models, modelStart, modelEnd, _a, atom_site, sourceIndex, model;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    atomCount = format.data.atom_site._rowCount;
                    entities = getEntities(format);
                    models = [];
                    modelStart = 0;
                    _b.label = 1;
                case 1:
                    if (!(modelStart < atomCount)) return [3 /*break*/, 3];
                    modelEnd = findModelEnd(format.data.atom_site.pdbx_PDB_model_num, modelStart);
                    return [4 /*yield*/, sort_1.sortAtomSite(ctx, format.data.atom_site, modelStart, modelEnd)];
                case 2:
                    _a = _b.sent(), atom_site = _a.atom_site, sourceIndex = _a.sourceIndex;
                    model = createStandardModel(format, atom_site, sourceIndex, entities, formatData, models.length > 0 ? models[models.length - 1] : void 0);
                    attachProps(model);
                    models.push(model);
                    modelStart = modelEnd;
                    return [3 /*break*/, 1];
                case 3: return [2 /*return*/, models];
            }
        });
    });
}
function splitTable(table, col) {
    var ret = new Map();
    var rowCount = table._rowCount;
    var modelStart = 0;
    while (modelStart < rowCount) {
        var modelEnd = findModelEnd(col, modelStart);
        var id = col.value(modelStart);
        ret.set(id, {
            table: db_1.Table.window(table, table._schema, modelStart, modelEnd),
            start: modelStart,
            end: modelEnd
        });
        modelStart = modelEnd;
    }
    return ret;
}
function getModelGroupName(model_id, format) {
    var _a = format.data, ihm_model_group = _a.ihm_model_group, ihm_model_group_link = _a.ihm_model_group_link;
    var link = db_1.Table.pickRow(ihm_model_group_link, function (i) { return ihm_model_group_link.model_id.value(i) === model_id; });
    if (link) {
        var group = db_1.Table.pickRow(ihm_model_group, function (i) { return ihm_model_group.id.value(i) === link.group_id; });
        if (group)
            return group.name;
    }
    return '';
}
function readIHM(ctx, format, formatData) {
    return __awaiter(this, void 0, void 0, function () {
        var atom_sites_modelColumn, ihm_model_list, entities, atom_sites, sphere_sites, gauss_sites, models, model_id, model_name, i, id, atom_site, atom_site_sourceIndex, e, _a, sorted, sourceIndex, data, model;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    atom_sites_modelColumn = format.data.atom_site.ihm_model_id.isDefined ? format.data.atom_site.ihm_model_id : format.data.atom_site.pdbx_PDB_model_num;
                    ihm_model_list = format.data.ihm_model_list;
                    entities = getEntities(format);
                    atom_sites = splitTable(format.data.atom_site, atom_sites_modelColumn);
                    sphere_sites = splitTable(format.data.ihm_sphere_obj_site, format.data.ihm_sphere_obj_site.model_id);
                    gauss_sites = splitTable(format.data.ihm_gaussian_obj_site, format.data.ihm_gaussian_obj_site.model_id);
                    models = [];
                    model_id = ihm_model_list.model_id, model_name = ihm_model_list.model_name;
                    i = 0;
                    _b.label = 1;
                case 1:
                    if (!(i < ihm_model_list._rowCount)) return [3 /*break*/, 6];
                    id = model_id.value(i);
                    atom_site = void 0, atom_site_sourceIndex = void 0;
                    if (!atom_sites.has(id)) return [3 /*break*/, 3];
                    e = atom_sites.get(id);
                    return [4 /*yield*/, sort_1.sortAtomSite(ctx, format.data.atom_site, e.start, e.end)];
                case 2:
                    _a = _b.sent(), sorted = _a.atom_site, sourceIndex = _a.sourceIndex;
                    atom_site = sorted;
                    atom_site_sourceIndex = sourceIndex;
                    return [3 /*break*/, 4];
                case 3:
                    atom_site = db_1.Table.window(format.data.atom_site, format.data.atom_site._schema, 0, 0);
                    atom_site_sourceIndex = db_1.Column.ofIntArray([]);
                    _b.label = 4;
                case 4:
                    data = {
                        model_id: id,
                        model_name: model_name.value(i),
                        model_group_name: getModelGroupName(id, format),
                        entities: entities,
                        atom_site: atom_site,
                        atom_site_sourceIndex: atom_site_sourceIndex,
                        ihm_sphere_obj_site: sphere_sites.has(id) ? sphere_sites.get(id).table : db_1.Table.window(format.data.ihm_sphere_obj_site, format.data.ihm_sphere_obj_site._schema, 0, 0),
                        ihm_gaussian_obj_site: gauss_sites.has(id) ? gauss_sites.get(id).table : db_1.Table.window(format.data.ihm_gaussian_obj_site, format.data.ihm_gaussian_obj_site._schema, 0, 0)
                    };
                    model = createModelIHM(format, data, formatData);
                    attachProps(model);
                    models.push(model);
                    _b.label = 5;
                case 5:
                    i++;
                    return [3 /*break*/, 1];
                case 6: return [2 /*return*/, models];
            }
        });
    });
}
