"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var sequence_1 = require("../../../mol-model/structure/model/properties/sequence");
var db_1 = require("../../../mol-data/db");
var sequence_2 = require("../../../mol-model/sequence");
// TODO how to handle microheterogeneity
//    see http://mmcif.wwpdb.org/dictionaries/mmcif_pdbx_v50.dic/Categories/entity_poly_seq.html
//
// Data items in the ENTITY_POLY_SEQ category specify the sequence
// of monomers in a polymer. Allowance is made for the possibility
// of microheterogeneity in a sample by allowing a given sequence
// number to be correlated with more than one monomer ID. The
// corresponding ATOM_SITE entries should reflect this
// heterogeneity.
function getSequence(cif, entities, hierarchy, modResMap) {
    if (!cif.entity_poly_seq._rowCount)
        return sequence_1.default.fromAtomicHierarchy(entities, hierarchy, modResMap);
    var _a = cif.entity_poly_seq, entity_id = _a.entity_id, num = _a.num, mon_id = _a.mon_id;
    var byEntityKey = {};
    var sequences = [];
    var count = entity_id.rowCount;
    var i = 0;
    while (i < count) {
        var start = i;
        while (i < count - 1 && entity_id.areValuesEqual(i, i + 1))
            i++;
        i++;
        var id = entity_id.value(start);
        var _compId = db_1.Column.window(mon_id, start, i);
        var _num = db_1.Column.window(num, start, i);
        var entityKey = entities.getEntityIndex(id);
        byEntityKey[entityKey] = {
            entityId: id,
            compId: _compId,
            num: _num,
            sequence: sequence_2.Sequence.ofResidueNames(_compId, _num, modResMap)
        };
        sequences.push(byEntityKey[entityKey]);
    }
    return { byEntityKey: byEntityKey, sequences: sequences };
}
exports.getSequence = getSequence;
