"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var string_1 = require("../../../mol-util/string");
var cif_1 = require("../../../mol-io/reader/cif");
var tokenizer_1 = require("../../../mol-io/reader/common/text/tokenizer");
var assembly_1 = require("./assembly");
var types_1 = require("../../../mol-model/structure/model/types");
var secondary_structure_1 = require("./secondary-structure");
var util_1 = require("../util");
function _entity() {
    return {
        id: cif_1.CifField.ofStrings(['1', '2', '3']),
        type: cif_1.CifField.ofStrings(['polymer', 'non-polymer', 'water'])
    };
}
function atom_site_template(data, count) {
    var str = function () { return []; };
    var ts = function () { return tokenizer_1.TokenBuilder.create(data, 2 * count); };
    return {
        index: 0,
        count: count,
        group_PDB: ts(),
        id: str(),
        auth_atom_id: ts(),
        label_alt_id: ts(),
        auth_comp_id: ts(),
        auth_asym_id: ts(),
        auth_seq_id: ts(),
        pdbx_PDB_ins_code: ts(),
        Cartn_x: ts(),
        Cartn_y: ts(),
        Cartn_z: ts(),
        occupancy: ts(),
        B_iso_or_equiv: ts(),
        type_symbol: ts(),
        pdbx_PDB_model_num: str(),
        label_entity_id: str()
    };
}
function _atom_site(sites) {
    var auth_asym_id = cif_1.CifField.ofTokens(sites.auth_asym_id);
    var auth_atom_id = cif_1.CifField.ofTokens(sites.auth_atom_id);
    var auth_comp_id = cif_1.CifField.ofTokens(sites.auth_comp_id);
    var auth_seq_id = cif_1.CifField.ofTokens(sites.auth_seq_id);
    return {
        auth_asym_id: auth_asym_id,
        auth_atom_id: auth_atom_id,
        auth_comp_id: auth_comp_id,
        auth_seq_id: auth_seq_id,
        B_iso_or_equiv: cif_1.CifField.ofTokens(sites.B_iso_or_equiv),
        Cartn_x: cif_1.CifField.ofTokens(sites.Cartn_x),
        Cartn_y: cif_1.CifField.ofTokens(sites.Cartn_y),
        Cartn_z: cif_1.CifField.ofTokens(sites.Cartn_z),
        group_PDB: cif_1.CifField.ofTokens(sites.group_PDB),
        id: cif_1.CifField.ofStrings(sites.id),
        label_alt_id: cif_1.CifField.ofTokens(sites.label_alt_id),
        label_asym_id: auth_asym_id,
        label_atom_id: auth_atom_id,
        label_comp_id: auth_comp_id,
        label_seq_id: auth_seq_id,
        label_entity_id: cif_1.CifField.ofStrings(sites.label_entity_id),
        occupancy: cif_1.CifField.ofTokens(sites.occupancy),
        type_symbol: cif_1.CifField.ofTokens(sites.type_symbol),
        pdbx_PDB_ins_code: cif_1.CifField.ofTokens(sites.pdbx_PDB_ins_code),
        pdbx_PDB_model_num: cif_1.CifField.ofStrings(sites.pdbx_PDB_model_num)
    };
}
function getEntityId(residueName, isHet) {
    if (isHet) {
        if (types_1.WaterNames.has(residueName))
            return '3';
        return '2';
    }
    return '1';
}
function addAtom(sites, model, data, s, e, isHet) {
    var str = data.data;
    var length = e - s;
    // TODO: filter invalid atoms
    // COLUMNS        DATA TYPE       CONTENTS
    // --------------------------------------------------------------------------------
    // 1 -  6        Record name     "ATOM  "
    tokenizer_1.TokenBuilder.addToken(sites.group_PDB, tokenizer_1.Tokenizer.trim(data, s, s + 6));
    // 7 - 11        Integer         Atom serial number.
    // TODO: support HEX
    tokenizer_1.Tokenizer.trim(data, s + 6, s + 11);
    sites.id[sites.index] = data.data.substring(data.tokenStart, data.tokenEnd);
    // 13 - 16        Atom            Atom name.
    tokenizer_1.TokenBuilder.addToken(sites.auth_atom_id, tokenizer_1.Tokenizer.trim(data, s + 12, s + 16));
    // 17             Character       Alternate location indicator.
    if (str.charCodeAt(s + 16) === 32) { // ' '
        tokenizer_1.TokenBuilder.add(sites.label_alt_id, 0, 0);
    }
    else {
        tokenizer_1.TokenBuilder.add(sites.label_alt_id, s + 16, s + 17);
    }
    // 18 - 20        Residue name    Residue name.
    tokenizer_1.TokenBuilder.addToken(sites.auth_comp_id, tokenizer_1.Tokenizer.trim(data, s + 17, s + 20));
    var residueName = str.substring(data.tokenStart, data.tokenEnd);
    // 22             Character       Chain identifier.
    tokenizer_1.TokenBuilder.add(sites.auth_asym_id, s + 21, s + 22);
    // 23 - 26        Integer         Residue sequence number.
    // TODO: support HEX
    tokenizer_1.TokenBuilder.addToken(sites.auth_seq_id, tokenizer_1.Tokenizer.trim(data, s + 22, s + 26));
    // 27             AChar           Code for insertion of residues.
    if (str.charCodeAt(s + 26) === 32) { // ' '
        tokenizer_1.TokenBuilder.add(sites.label_alt_id, 0, 0);
    }
    else {
        tokenizer_1.TokenBuilder.add(sites.label_alt_id, s + 26, s + 27);
    }
    // 31 - 38        Real(8.3)       Orthogonal coordinates for X in Angstroms.
    tokenizer_1.TokenBuilder.addToken(sites.Cartn_x, tokenizer_1.Tokenizer.trim(data, s + 30, s + 38));
    // 39 - 46        Real(8.3)       Orthogonal coordinates for Y in Angstroms.
    tokenizer_1.TokenBuilder.addToken(sites.Cartn_y, tokenizer_1.Tokenizer.trim(data, s + 38, s + 46));
    // 47 - 54        Real(8.3)       Orthogonal coordinates for Z in Angstroms.
    tokenizer_1.TokenBuilder.addToken(sites.Cartn_z, tokenizer_1.Tokenizer.trim(data, s + 46, s + 54));
    // 55 - 60        Real(6.2)       Occupancy.
    tokenizer_1.TokenBuilder.addToken(sites.occupancy, tokenizer_1.Tokenizer.trim(data, s + 54, s + 60));
    // 61 - 66        Real(6.2)       Temperature factor (Default = 0.0).
    if (length >= 66) {
        tokenizer_1.TokenBuilder.addToken(sites.B_iso_or_equiv, tokenizer_1.Tokenizer.trim(data, s + 60, s + 66));
    }
    else {
        tokenizer_1.TokenBuilder.add(sites.label_alt_id, 0, 0);
    }
    // 73 - 76        LString(4)      Segment identifier, left-justified.
    // ignored
    // 77 - 78        LString(2)      Element symbol, right-justified.
    if (length >= 78) {
        tokenizer_1.Tokenizer.trim(data, s + 76, s + 78);
        if (data.tokenStart < data.tokenEnd) {
            tokenizer_1.TokenBuilder.addToken(sites.type_symbol, data);
        }
        else {
            util_1.guessElementSymbolTokens(sites.type_symbol, str, s + 12, s + 16);
        }
    }
    else {
        util_1.guessElementSymbolTokens(sites.type_symbol, str, s + 12, s + 16);
    }
    sites.label_entity_id[sites.index] = getEntityId(residueName, isHet);
    sites.pdbx_PDB_model_num[sites.index] = model;
    sites.index++;
}
function pdbToMmCif(pdb) {
    return __awaiter(this, void 0, void 0, function () {
        var lines, data, indices, tokenizer, atomCount, i, _i, s, e, atom_site, helperCategories, modelNum, modelStr, i, _i, s, e, j, j, j, j, categories, _a, helperCategories_1, c;
        return __generator(this, function (_b) {
            lines = pdb.lines;
            data = lines.data, indices = lines.indices;
            tokenizer = tokenizer_1.Tokenizer(data);
            atomCount = 0;
            for (i = 0, _i = lines.count; i < _i; i++) {
                s = indices[2 * i], e = indices[2 * i + 1];
                switch (data[s]) {
                    case 'A':
                        if (string_1.substringStartsWith(data, s, e, 'ATOM  '))
                            atomCount++;
                        break;
                    case 'H':
                        if (string_1.substringStartsWith(data, s, e, 'HETATM'))
                            atomCount++;
                        break;
                }
            }
            atom_site = atom_site_template(data, atomCount);
            helperCategories = [];
            modelNum = 0, modelStr = '';
            for (i = 0, _i = lines.count; i < _i; i++) {
                s = indices[2 * i], e = indices[2 * i + 1];
                switch (data[s]) {
                    case 'A':
                        if (!string_1.substringStartsWith(data, s, e, 'ATOM  '))
                            continue;
                        if (!modelNum) {
                            modelNum++;
                            modelStr = '' + modelNum;
                        }
                        addAtom(atom_site, modelStr, tokenizer, s, e, false);
                        break;
                    case 'C':
                        if (string_1.substringStartsWith(data, s, e, 'CRYST1')) {
                            helperCategories.push.apply(helperCategories, assembly_1.parseCryst1(pdb.id || '?', data.substring(s, e)));
                        }
                        // TODO CONNECT records => struct_conn
                        // TODO COMPND records => entity
                        break;
                    case 'H':
                        if (string_1.substringStartsWith(data, s, e, 'HETATM')) {
                            if (!modelNum) {
                                modelNum++;
                                modelStr = '' + modelNum;
                            }
                            addAtom(atom_site, modelStr, tokenizer, s, e, true);
                        }
                        else if (string_1.substringStartsWith(data, s, e, 'HELIX')) {
                            j = i + 1;
                            while (true) {
                                s = indices[2 * j];
                                e = indices[2 * j + 1];
                                if (!string_1.substringStartsWith(data, s, e, 'HELIX'))
                                    break;
                                j++;
                            }
                            helperCategories.push(secondary_structure_1.parseHelix(lines, i, j));
                            i = j - 1;
                        }
                        // TODO HETNAM records => chem_comp (at least partially, needs to be completed with common bases and amino acids)
                        break;
                    case 'M':
                        if (string_1.substringStartsWith(data, s, e, 'MODEL ')) {
                            modelNum++;
                            modelStr = '' + modelNum;
                        }
                        if (string_1.substringStartsWith(data, s, e, 'MTRIX')) {
                            j = i + 1;
                            while (true) {
                                s = indices[2 * j];
                                e = indices[2 * j + 1];
                                if (!string_1.substringStartsWith(data, s, e, 'MTRIX'))
                                    break;
                                j++;
                            }
                            helperCategories.push.apply(helperCategories, assembly_1.parseMtrix(lines, i, j));
                            i = j - 1;
                        }
                        // TODO MODRES records => pdbx_struct_mod_residue
                        break;
                    case 'O':
                        // TODO ORIGX record => cif.database_PDB_matrix.origx, cif.database_PDB_matrix.origx_vector
                        break;
                    case 'R':
                        if (string_1.substringStartsWith(data, s, e, 'REMARK 350')) {
                            j = i + 1;
                            while (true) {
                                s = indices[2 * j];
                                e = indices[2 * j + 1];
                                if (!string_1.substringStartsWith(data, s, e, 'REMARK 350'))
                                    break;
                                j++;
                            }
                            helperCategories.push.apply(helperCategories, assembly_1.parseRemark350(lines, i, j));
                            i = j - 1;
                        }
                        break;
                    case 'S':
                        if (string_1.substringStartsWith(data, s, e, 'SHEET')) {
                            j = i + 1;
                            while (true) {
                                s = indices[2 * j];
                                e = indices[2 * j + 1];
                                if (!string_1.substringStartsWith(data, s, e, 'SHEET'))
                                    break;
                                j++;
                            }
                            helperCategories.push(secondary_structure_1.parseSheet(lines, i, j));
                            i = j - 1;
                        }
                        // TODO SCALE record => cif.atom_sites.fract_transf_matrix, cif.atom_sites.fract_transf_vector
                        break;
                }
            }
            categories = {
                entity: cif_1.CifCategory.ofFields('entity', _entity()),
                atom_site: cif_1.CifCategory.ofFields('atom_site', _atom_site(atom_site))
            };
            for (_a = 0, helperCategories_1 = helperCategories; _a < helperCategories_1.length; _a++) {
                c = helperCategories_1[_a];
                categories[c.name] = c;
            }
            return [2 /*return*/, {
                    header: pdb.id || 'PDB',
                    categoryNames: Object.keys(categories),
                    categories: categories
                }];
        });
    });
}
exports.pdbToMmCif = pdbToMmCif;
