"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_task_1 = require("../../mol-task");
var geometry_1 = require("../../mol-math/geometry");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var misc_1 = require("../../mol-math/misc");
var parser_1 = require("../../mol-io/reader/ccp4/parser");
var typed_array_1 = require("../../mol-io/common/typed-array");
/** When available (e.g. in MRC files) use ORIGIN records instead of N[CRS]START */
function getCcp4Origin(header) {
    var gridOrigin;
    if (header.originX === 0.0 && header.originY === 0.0 && header.originZ === 0.0) {
        gridOrigin = [header.NCSTART, header.NRSTART, header.NSSTART];
    }
    else {
        gridOrigin = [header.originX, header.originY, header.originZ];
    }
    return gridOrigin;
}
exports.getCcp4Origin = getCcp4Origin;
function getTypedArrayCtor(header) {
    var valueType = parser_1.getCcp4ValueType(header);
    switch (valueType) {
        case typed_array_1.TypedArrayValueType.Float32: return Float32Array;
        case typed_array_1.TypedArrayValueType.Int8: return Int8Array;
        case typed_array_1.TypedArrayValueType.Int16: return Int16Array;
    }
    throw Error(valueType + " is not a supported value format.");
}
function volumeFromCcp4(source, params) {
    var _this = this;
    return mol_task_1.Task.create('Create Volume Data', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
        var header, values, size, angles, cell, axis_order_fast_to_slow, normalizeOrder, grid, extent, gridOrigin, origin_frac, dimensions_frac, space, data;
        return __generator(this, function (_a) {
            header = source.header, values = source.values;
            console.log({ header: header, values: values });
            size = linear_algebra_1.Vec3.create(header.xLength, header.yLength, header.zLength);
            if (params && params.voxelSize)
                linear_algebra_1.Vec3.mul(size, size, params.voxelSize);
            angles = linear_algebra_1.Vec3.create(misc_1.degToRad(header.alpha), misc_1.degToRad(header.beta), misc_1.degToRad(header.gamma));
            cell = geometry_1.SpacegroupCell.create(header.ISPG || 'P 1', size, angles);
            axis_order_fast_to_slow = linear_algebra_1.Vec3.create(header.MAPC - 1, header.MAPR - 1, header.MAPS - 1);
            normalizeOrder = linear_algebra_1.Tensor.convertToCanonicalAxisIndicesFastToSlow(axis_order_fast_to_slow);
            grid = [header.NX, header.NY, header.NZ];
            extent = normalizeOrder([header.NC, header.NR, header.NS]);
            gridOrigin = normalizeOrder(getCcp4Origin(header));
            origin_frac = linear_algebra_1.Vec3.create(gridOrigin[0] / grid[0], gridOrigin[1] / grid[1], gridOrigin[2] / grid[2]);
            dimensions_frac = linear_algebra_1.Vec3.create(extent[0] / grid[0], extent[1] / grid[1], extent[2] / grid[2]);
            space = linear_algebra_1.Tensor.Space(extent, linear_algebra_1.Tensor.invertAxisOrder(axis_order_fast_to_slow), getTypedArrayCtor(header));
            data = linear_algebra_1.Tensor.create(space, linear_algebra_1.Tensor.Data1(values));
            // TODO Calculate stats? When to trust header data?
            // Min/max/mean are reliable (based on LiteMol/DensityServer usage)
            // These, however, calculate sigma, so no data on that.
            return [2 /*return*/, {
                    cell: cell,
                    fractionalBox: geometry_1.Box3D.create(origin_frac, linear_algebra_1.Vec3.add(linear_algebra_1.Vec3.zero(), origin_frac, dimensions_frac)),
                    data: data,
                    dataStats: {
                        min: header.AMIN,
                        max: header.AMAX,
                        mean: header.AMEAN,
                        sigma: header.ARMS
                    }
                }];
        });
    }); });
}
exports.volumeFromCcp4 = volumeFromCcp4;
