"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../mol-model/structure");
var structure_2 = require("../../mol-model/structure/structure");
var mol_task_1 = require("../../mol-task");
var tables_1 = require("../../mol-util/color/tables");
var int_1 = require("../../mol-data/int");
var CustomElementProperty;
(function (CustomElementProperty) {
    function create(params) {
        var name = params.name;
        var Descriptor = structure_1.CustomPropertyDescriptor({
            isStatic: params.isStatic,
            name: params.name,
        });
        function attach(model) {
            var _this = this;
            return mol_task_1.Task.create("Attach " + params.display, function () { return __awaiter(_this, void 0, void 0, function () {
                var data, e_1;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            _a.trys.push([0, 2, , 3]);
                            if (model.customProperties.has(Descriptor))
                                return [2 /*return*/, true];
                            return [4 /*yield*/, params.getData(model)];
                        case 1:
                            data = _a.sent();
                            if (params.isStatic) {
                                model._staticPropertyData[name] = data;
                            }
                            else {
                                model._dynamicPropertyData[name] = data;
                            }
                            model.customProperties.add(Descriptor);
                            return [2 /*return*/, true];
                        case 2:
                            e_1 = _a.sent();
                            console.warn('Attach Property', e_1);
                            return [2 /*return*/, false];
                        case 3: return [2 /*return*/];
                    }
                });
            }); });
        }
        function getStatic(e) { return e.unit.model._staticPropertyData[name].get(e.element); }
        function getDynamic(e) { return e.unit.model._staticPropertyData[name].get(e.element); }
        var propertyProvider = {
            option: [name, params.display],
            descriptor: Descriptor,
            defaultSelected: !!params.autoAttach,
            attachableTo: params.attachableTo || (function () { return true; }),
            attach: attach
        };
        var get = params.isStatic ? getStatic : getDynamic;
        function has(model) { return model.customProperties.has(Descriptor); }
        function Coloring(ctx, props) {
            var color;
            var getColor = params.coloring.getColor;
            var defaultColor = params.coloring.defaultColor;
            if (ctx.structure && !ctx.structure.isEmpty && has(ctx.structure.models[0])) {
                color = function (location) {
                    if (structure_2.StructureElement.isLocation(location)) {
                        var e = get(location);
                        if (typeof e !== 'undefined')
                            return getColor(e);
                    }
                    return defaultColor;
                };
            }
            else {
                color = function () { return defaultColor; };
            }
            return {
                factory: Coloring,
                granularity: 'group',
                color: color,
                props: props,
                description: 'Assign element colors based on the provided data.',
                legend: tables_1.TableLegend([])
            };
        }
        var colorTheme = {
            label: params.display,
            factory: Coloring,
            getParams: function () { return ({}); },
            defaultValues: {},
            isApplicable: function (ctx) { return !!ctx.structure && !ctx.structure.isEmpty && has(ctx.structure.models[0]); }
        };
        function LabelProvider(loci) {
            if (loci.kind === 'element-loci') {
                var e = loci.elements[0];
                if (!has(e.unit.model))
                    return void 0;
                return params.format(get(structure_2.StructureElement.create(e.unit, e.unit.elements[int_1.OrderedSet.getAt(e.indices, 0)])));
            }
            return void 0;
        }
        return {
            Descriptor: Descriptor,
            attach: attach,
            get: get,
            propertyProvider: propertyProvider,
            colorTheme: params.coloring ? colorTheme : void 0,
            labelProvider: params.format ? LabelProvider : (function (loci) { return void 0; })
        };
    }
    CustomElementProperty.create = create;
})(CustomElementProperty || (CustomElementProperty = {}));
exports.CustomElementProperty = CustomElementProperty;
