"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var param_definition_1 = require("../../mol-util/param-definition");
var CustomPropertyRegistry = /** @class */ (function () {
    function CustomPropertyRegistry() {
        this.providers = immutable_1.OrderedMap().asMutable();
    }
    CustomPropertyRegistry.prototype.getSelect = function (object) {
        var values = this.providers.values();
        var options = [], selected = [];
        while (true) {
            var v = values.next();
            if (v.done)
                break;
            if (!v.value.attachableTo(object))
                continue;
            options.push(v.value.option);
            if (v.value.defaultSelected)
                selected.push(v.value.option[0]);
        }
        return param_definition_1.ParamDefinition.MultiSelect(selected, options);
    };
    CustomPropertyRegistry.prototype.getDefault = function (object) {
        var values = this.providers.values();
        var selected = [];
        while (true) {
            var v = values.next();
            if (v.done)
                break;
            if (!v.value.attachableTo(object))
                continue;
            if (v.value.defaultSelected)
                selected.push(v.value.option[0]);
        }
        return selected;
    };
    CustomPropertyRegistry.prototype.get = function (name) {
        var prop = this.providers.get(name);
        if (!prop)
            throw new Error("Custom prop '" + name + "' is not registered.");
        return this.providers.get(name);
    };
    CustomPropertyRegistry.prototype.register = function (provider) {
        this.providers.set(provider.descriptor.name, provider);
    };
    CustomPropertyRegistry.prototype.unregister = function (name) {
        this.providers.delete(name);
    };
    return CustomPropertyRegistry;
}());
exports.CustomPropertyRegistry = CustomPropertyRegistry;
