"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author Sebastian Bittrich <sebastian.bittrich@rcsb.org>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var seconday_structure_1 = require("../../../mol-model/structure/model/properties/seconday-structure");
var types_1 = require("../../../mol-model/structure/model/types");
var param_definition_1 = require("../../../mol-util/param-definition");
var bends_1 = require("./dssp/bends");
var backbone_hbonds_1 = require("./dssp/backbone-hbonds");
var common_1 = require("./dssp/common");
var turns_1 = require("./dssp/turns");
var helices_1 = require("./dssp/helices");
var ladders_1 = require("./dssp/ladders");
var bridges_1 = require("./dssp/bridges");
var sheets_1 = require("./dssp/sheets");
var dihedral_angles_1 = require("./dssp/dihedral-angles");
var trace_lookup_1 = require("./dssp/trace-lookup");
var protein_info_1 = require("./dssp/protein-info");
var int_1 = require("../../../mol-data/int");
/**
 * TODO bugs to fix:
 * - some turns are not detected correctly: see e.g. pdb:1acj - maybe more than 2 hbonds require some residue to donate electrons
 * - some sheets are not extended correctly: see e.g. pdb:1acj
 * - validate new helix definition
 * - validate new ordering of secondary structure elements
 */
exports.DSSPComputationParams = {
    oldDefinition: param_definition_1.ParamDefinition.Boolean(true, { description: 'Whether to use the old DSSP convention for the annotation of turns and helices, causes them to be two residues shorter' }),
    oldOrdering: param_definition_1.ParamDefinition.Boolean(true, { description: 'Alpha-helices are preferred over 3-10 helices' })
};
function computeUnitDSSP(unit, params) {
    return __awaiter(this, void 0, void 0, function () {
        var proteinInfo, residueIndices, lookup3d, hbonds, residueCount, flags, torsionAngles, ladders, bridges, getResidueFlag, getFlagName, ctx, assignment, type, keys, elements, getIndex, i, il, assign, flag;
        return __generator(this, function (_a) {
            proteinInfo = protein_info_1.getUnitProteinInfo(unit);
            residueIndices = proteinInfo.residueIndices;
            lookup3d = trace_lookup_1.calcUnitProteinTraceLookup3D(unit, residueIndices);
            hbonds = backbone_hbonds_1.calcUnitBackboneHbonds(unit, proteinInfo, lookup3d);
            residueCount = residueIndices.length;
            flags = new Uint32Array(residueCount);
            torsionAngles = dihedral_angles_1.calculateUnitDihedralAngles(unit, proteinInfo);
            ladders = [];
            bridges = [];
            getResidueFlag = params.oldDefinition ? getOriginalResidueFlag : getUpdatedResidueFlag;
            getFlagName = params.oldOrdering ? getOriginalFlagName : getUpdatedFlagName;
            ctx = {
                params: params,
                getResidueFlag: getResidueFlag,
                getFlagName: getFlagName,
                unit: unit,
                proteinInfo: proteinInfo,
                flags: flags,
                hbonds: hbonds,
                torsionAngles: torsionAngles,
                ladders: ladders,
                bridges: bridges
            };
            turns_1.assignTurns(ctx);
            helices_1.assignHelices(ctx);
            bends_1.assignBends(ctx);
            bridges_1.assignBridges(ctx);
            ladders_1.assignLadders(ctx);
            sheets_1.assignSheets(ctx);
            assignment = getDSSPAssignment(flags, getResidueFlag);
            type = new Uint32Array(residueCount);
            keys = [];
            elements = [];
            getIndex = function (rI) { return int_1.SortedArray.indexOf(residueIndices, rI); };
            for (i = 0, il = residueIndices.length; i < il; ++i) {
                assign = assignment[i];
                type[i] = assign;
                flag = getResidueFlag(flags[i]);
                // console.log(i, SortedArray.indexOf(residueIndices, i), getFlagName(flags[i]))
                // TODO is this expected behavior? elements will be strictly split depending on 'winning' flag
                if (elements.length === 0 /* would fail at very start */ || flag !== elements[elements.length - 1].flags /* flag changed */) {
                    elements[elements.length] = createElement(mapToKind(assign), flags[i], getResidueFlag);
                }
                keys[i] = elements.length - 1;
            }
            return [2 /*return*/, seconday_structure_1.SecondaryStructure(type, keys, elements, getIndex)];
        });
    });
}
exports.computeUnitDSSP = computeUnitDSSP;
function createElement(kind, flag, getResidueFlag) {
    // TODO would be nice to add more detailed information
    if (kind === 'helix') {
        return {
            kind: 'helix',
            flags: getResidueFlag(flag)
        };
    }
    else if (kind === 'sheet') {
        return {
            kind: 'sheet',
            flags: getResidueFlag(flag)
        };
    }
    else if (kind === 'turn' || kind === 'bend') {
        return {
            kind: 'turn',
            flags: getResidueFlag(flag)
        };
    }
    else {
        return {
            kind: 'none'
        };
    }
}
function mapToKind(assignment) {
    if (assignment === types_1.SecondaryStructureType.SecondaryStructureDssp.H || assignment === types_1.SecondaryStructureType.SecondaryStructureDssp.G || assignment === types_1.SecondaryStructureType.SecondaryStructureDssp.I) {
        return 'helix';
    }
    else if (assignment === types_1.SecondaryStructureType.SecondaryStructureDssp.B || assignment === types_1.SecondaryStructureType.SecondaryStructureDssp.E) {
        return 'sheet';
    }
    else if (assignment === types_1.SecondaryStructureType.SecondaryStructureDssp.T) {
        return 'turn';
    }
    else if (assignment === types_1.SecondaryStructureType.SecondaryStructureDssp.S) {
        return 'bend';
    }
    else {
        return 'none';
    }
}
/** Original priority: H,B,E,G,I,T,S */
function getOriginalResidueFlag(f) {
    if (common_1.DSSPType.is(f, 1 /* H */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.H;
    if (common_1.DSSPType.is(f, 4 /* E */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.E;
    if (common_1.DSSPType.is(f, 2 /* B */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.B;
    if (common_1.DSSPType.is(f, 8 /* G */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.G;
    if (common_1.DSSPType.is(f, 16 /* I */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.I;
    if (common_1.DSSPType.is(f, 64 /* T */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.T;
    if (common_1.DSSPType.is(f, 32 /* S */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.S;
    return 0 /* None */;
}
function getOriginalFlagName(f) {
    if (common_1.DSSPType.is(f, 1 /* H */))
        return 'H';
    if (common_1.DSSPType.is(f, 4 /* E */))
        return 'E';
    if (common_1.DSSPType.is(f, 2 /* B */))
        return 'B';
    if (common_1.DSSPType.is(f, 8 /* G */))
        return 'G';
    if (common_1.DSSPType.is(f, 16 /* I */))
        return 'I';
    if (common_1.DSSPType.is(f, 64 /* T */))
        return 'T';
    if (common_1.DSSPType.is(f, 32 /* S */))
        return 'S';
    return '-';
}
/** Version 2.1.0 priority: I,H,B,E,G,T,S */
function getUpdatedResidueFlag(f) {
    if (common_1.DSSPType.is(f, 16 /* I */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.I;
    if (common_1.DSSPType.is(f, 1 /* H */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.H;
    if (common_1.DSSPType.is(f, 4 /* E */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.E;
    if (common_1.DSSPType.is(f, 2 /* B */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.B;
    if (common_1.DSSPType.is(f, 8 /* G */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.G;
    if (common_1.DSSPType.is(f, 64 /* T */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.T;
    if (common_1.DSSPType.is(f, 32 /* S */))
        return types_1.SecondaryStructureType.SecondaryStructureDssp.S;
    return 0 /* None */;
}
function getUpdatedFlagName(f) {
    if (common_1.DSSPType.is(f, 16 /* I */))
        return 'I';
    if (common_1.DSSPType.is(f, 1 /* H */))
        return 'H';
    if (common_1.DSSPType.is(f, 4 /* E */))
        return 'E';
    if (common_1.DSSPType.is(f, 2 /* B */))
        return 'B';
    if (common_1.DSSPType.is(f, 8 /* G */))
        return 'G';
    if (common_1.DSSPType.is(f, 64 /* T */))
        return 'T';
    if (common_1.DSSPType.is(f, 32 /* S */))
        return 'S';
    return '-';
}
function getDSSPAssignment(flags, getResidueFlag) {
    var type = new Uint32Array(flags.length);
    for (var i = 0, il = flags.length; i < il; ++i) {
        var f = common_1.DSSPType.create(flags[i]);
        type[i] = getResidueFlag(f);
    }
    return type;
}
