/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { Column, Table } from '../../mol-data/db';
import { CifWriter } from '../../mol-io/writer/cif';
import { Model, Unit, IndexedCustomProperty } from '../../mol-model/structure';
import { StructureElement, CifExportContext } from '../../mol-model/structure/structure';
import { QuerySymbolRuntime } from '../../mol-script/runtime/query/compiler';
import { PropertyWrapper } from '../common/wrapper';
import { Task } from '../../mol-task';
export declare namespace StructureQualityReport {
    type IssueMap = IndexedCustomProperty.Residue<string[]>;
    type Property = PropertyWrapper<IssueMap | undefined>;
    function get(model: Model): Property | undefined;
    const Schema: {
        pdbe_structure_quality_report: {
            updated_datetime_utc: Column.Schema.Str;
        };
        pdbe_structure_quality_report_issues: {
            pdbx_PDB_model_num: Column.Schema.Int;
            issue_type_group_id: Column.Schema.Int;
            label_comp_id: Column.Schema.Str;
            label_seq_id: Column.Schema.Int;
            pdbx_PDB_ins_code: Column.Schema.Str;
            label_asym_id: Column.Schema.Str;
            label_entity_id: Column.Schema.Str;
            auth_comp_id: Column.Schema.Str;
            auth_seq_id: Column.Schema.Int;
            auth_asym_id: Column.Schema.Str;
            id: Column.Schema.Int;
        };
        pdbe_structure_quality_report_issue_types: {
            group_id: Column.Schema.Int;
            issue_type: Column.Schema.Str;
        };
    };
    type Schema = typeof Schema;
    const Descriptor: {
        isStatic: false;
        name: string;
        cifExport: {
            prefix: string;
            context(ctx: CifExportContext): ReportExportContext;
            categories: (CifWriter.Category<ReportExportContext> | {
                name: string;
                instance(ctx: ReportExportContext): {
                    fields: CifWriter.Field<number, {
                        elements: StructureElement<Unit>[];
                        groupId: number[];
                    }>[];
                    source: {
                        data: {
                            elements: StructureElement<Unit>[];
                            groupId: number[];
                        };
                        rowCount: number;
                    }[];
                };
            })[];
        };
        symbols: {
            issueCount: QuerySymbolRuntime;
        };
    };
    function createAttachTask(mapUrl: (model: Model) => string, fetch: import('../../mol-util/data-source').AjaxTask): (model: Model) => Task<boolean>;
    function attachFromCifOrApi(model: Model, params: {
        PDBe_apiSourceJson?: (model: Model) => Promise<any>;
    }): Promise<boolean>;
    function getIssueMap(model: Model): IssueMap | undefined;
    function getIssues(e: StructureElement): string[];
}
interface ReportExportContext {
    models: {
        elements: StructureElement[];
        groupId: number[];
    }[];
    info: PropertyWrapper.Info;
    issueTypes: Table<StructureQualityReport.Schema['pdbe_structure_quality_report_issue_types']>;
}
export {};
