"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var db_1 = require("../../mol-data/db");
var schema_1 = require("../../mol-io/reader/cif/schema");
var mmcif_extras_1 = require("../../mol-io/reader/cif/schema/mmcif-extras");
var cif_1 = require("../../mol-io/writer/cif");
var structure_1 = require("../../mol-model/structure");
var atom_site_1 = require("../../mol-model/structure/export/categories/atom_site");
var symbol_1 = require("../../mol-script/language/symbol");
var type_1 = require("../../mol-script/language/type");
var compiler_1 = require("../../mol-script/runtime/query/compiler");
var wrapper_1 = require("../common/wrapper");
var mol_task_1 = require("../../mol-task");
var StructureQualityReport;
(function (StructureQualityReport) {
    function get(model) {
        // must be defined before the descriptor so it's not undefined.
        return model._dynamicPropertyData.__PDBeStructureQualityReport__;
    }
    StructureQualityReport.get = get;
    StructureQualityReport.Schema = {
        pdbe_structure_quality_report: {
            updated_datetime_utc: db_1.Column.Schema.str
        },
        pdbe_structure_quality_report_issues: __assign({ id: db_1.Column.Schema.int }, mmcif_extras_1.mmCIF_residueId_schema, { pdbx_PDB_model_num: db_1.Column.Schema.int, issue_type_group_id: db_1.Column.Schema.int }),
        pdbe_structure_quality_report_issue_types: {
            group_id: db_1.Column.Schema.int,
            issue_type: db_1.Column.Schema.str
        }
    };
    StructureQualityReport.Descriptor = structure_1.CustomPropertyDescriptor({
        isStatic: false,
        name: 'pdbe_structure_quality_report',
        cifExport: {
            prefix: 'pdbe',
            context: function (ctx) {
                return createExportContext(ctx);
            },
            categories: [
                wrapper_1.PropertyWrapper.defaultInfoCategory('pdbe_structure_quality_report', function (ctx) { return ctx.info; }),
                {
                    name: 'pdbe_structure_quality_report_issues',
                    instance: function (ctx) {
                        return {
                            fields: _structure_quality_report_issues_fields,
                            source: ctx.models.map(function (data) { return ({ data: data, rowCount: data.elements.length }); })
                        };
                    }
                }, {
                    name: 'pdbe_structure_quality_report_issue_types',
                    instance: function (ctx) {
                        return cif_1.CifWriter.Category.ofTable(ctx.issueTypes);
                    }
                }
            ]
        },
        symbols: {
            issueCount: compiler_1.QuerySymbolRuntime.Dynamic(symbol_1.CustomPropSymbol('pdbe', 'structure-quality.issue-count', type_1.default.Num), function (ctx) { return StructureQualityReport.getIssues(ctx.element).length; }),
        }
    });
    function getCifData(model) {
        if (model.sourceData.kind !== 'mmCIF')
            throw new Error('Data format must be mmCIF.');
        return {
            residues: schema_1.toTable(StructureQualityReport.Schema.pdbe_structure_quality_report_issues, model.sourceData.frame.categories.pdbe_structure_quality_report_issues),
            groups: schema_1.toTable(StructureQualityReport.Schema.pdbe_structure_quality_report_issue_types, model.sourceData.frame.categories.pdbe_structure_quality_report_issue_types),
        };
    }
    function createAttachTask(mapUrl, fetch) {
        var _this = this;
        return function (model) { return mol_task_1.Task.create('PDBe Structure Quality Report', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var issueMap, info, url, dataStr, data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (get(model))
                            return [2 /*return*/, true];
                        url = mapUrl(model);
                        return [4 /*yield*/, fetch({ url: url }).runInContext(ctx)];
                    case 1:
                        dataStr = _a.sent();
                        data = JSON.parse(dataStr)[model.label.toLowerCase()];
                        if (!data)
                            return [2 /*return*/, false];
                        info = wrapper_1.PropertyWrapper.createInfo();
                        issueMap = createIssueMapFromJson(model, data);
                        model.customProperties.add(StructureQualityReport.Descriptor);
                        set(model, { info: info, data: issueMap });
                        return [2 /*return*/, false];
                }
            });
        }); }); };
    }
    StructureQualityReport.createAttachTask = createAttachTask;
    function attachFromCifOrApi(model, params) {
        return __awaiter(this, void 0, void 0, function () {
            var issueMap, info, data, data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (get(model))
                            return [2 /*return*/, true];
                        info = wrapper_1.PropertyWrapper.tryGetInfoFromCif('pdbe_structure_quality_report', model);
                        if (!info) return [3 /*break*/, 1];
                        data = getCifData(model);
                        issueMap = createIssueMapFromCif(model, data.residues, data.groups);
                        return [3 /*break*/, 4];
                    case 1:
                        if (!params.PDBe_apiSourceJson) return [3 /*break*/, 3];
                        return [4 /*yield*/, params.PDBe_apiSourceJson(model)];
                    case 2:
                        data = _a.sent();
                        if (!data)
                            return [2 /*return*/, false];
                        info = wrapper_1.PropertyWrapper.createInfo();
                        issueMap = createIssueMapFromJson(model, data);
                        return [3 /*break*/, 4];
                    case 3: return [2 /*return*/, false];
                    case 4:
                        model.customProperties.add(StructureQualityReport.Descriptor);
                        set(model, { info: info, data: issueMap });
                        return [2 /*return*/, true];
                }
            });
        });
    }
    StructureQualityReport.attachFromCifOrApi = attachFromCifOrApi;
    function set(model, prop) {
        model._dynamicPropertyData.__PDBeStructureQualityReport__ = prop;
    }
    function getIssueMap(model) {
        var prop = get(model);
        return prop && prop.data;
    }
    StructureQualityReport.getIssueMap = getIssueMap;
    var _emptyArray = [];
    function getIssues(e) {
        if (!structure_1.Unit.isAtomic(e.unit))
            return _emptyArray;
        var prop = StructureQualityReport.get(e.unit.model);
        if (!prop || !prop.data)
            return _emptyArray;
        var rI = e.unit.residueIndex[e.element];
        return prop.data.has(rI) ? prop.data.get(rI) : _emptyArray;
    }
    StructureQualityReport.getIssues = getIssues;
})(StructureQualityReport = exports.StructureQualityReport || (exports.StructureQualityReport = {}));
var _structure_quality_report_issues_fields = cif_1.CifWriter.fields()
    .index('id')
    .many(atom_site_1.residueIdFields(function (i, d) { return d.elements[i]; }, { includeModelNum: true }))
    .int('issue_type_group_id', function (i, d) { return d.groupId[i]; })
    .getFields();
function createExportContext(ctx) {
    var groupMap = new Map();
    var models = [];
    var group_id = [], issue_type = [];
    var info = wrapper_1.PropertyWrapper.createInfo();
    for (var _i = 0, _a = ctx.structures; _i < _a.length; _i++) {
        var s = _a[_i];
        var prop = StructureQualityReport.get(s.model);
        if (prop)
            info = prop.info;
        if (!prop || !prop.data)
            continue;
        var _b = prop.data.getElements(s), elements = _b.elements, property = _b.property;
        if (elements.length === 0)
            continue;
        var elementGroupId = [];
        for (var i = 0; i < elements.length; i++) {
            var issues = property(i);
            var key = issues.join(',');
            if (!groupMap.has(key)) {
                var idx = groupMap.size + 1;
                groupMap.set(key, idx);
                for (var _c = 0, issues_1 = issues; _c < issues_1.length; _c++) {
                    var issue = issues_1[_c];
                    group_id.push(idx);
                    issue_type.push(issue);
                }
            }
            elementGroupId[i] = groupMap.get(key);
        }
        models.push({ elements: elements, groupId: elementGroupId });
    }
    return {
        info: info,
        models: models,
        issueTypes: db_1.Table.ofArrays(StructureQualityReport.Schema.pdbe_structure_quality_report_issue_types, { group_id: group_id, issue_type: issue_type })
    };
}
function createIssueMapFromJson(modelData, data) {
    var ret = new Map();
    if (!data.molecules)
        return;
    for (var _i = 0, _a = data.molecules; _i < _a.length; _i++) {
        var entity = _a[_i];
        var entity_id = entity.entity_id.toString();
        for (var _b = 0, _c = entity.chains; _b < _c.length; _b++) {
            var chain = _c[_b];
            var asym_id = chain.struct_asym_id.toString();
            for (var _d = 0, _e = chain.models; _d < _e.length; _d++) {
                var model = _e[_d];
                var model_id = model.model_id.toString();
                if (+model_id !== modelData.modelNum)
                    continue;
                for (var _f = 0, _g = model.residues; _f < _g.length; _f++) {
                    var residue = _g[_f];
                    var auth_seq_id = residue.author_residue_number, ins_code = residue.author_insertion_code || '';
                    var idx = modelData.atomicHierarchy.index.findResidue(entity_id, asym_id, auth_seq_id, ins_code);
                    ret.set(idx, residue.outlier_types);
                }
            }
        }
    }
    return structure_1.IndexedCustomProperty.fromResidueMap(ret);
}
function createIssueMapFromCif(modelData, residueData, groupData) {
    var ret = new Map();
    var label_entity_id = residueData.label_entity_id, label_asym_id = residueData.label_asym_id, auth_seq_id = residueData.auth_seq_id, pdbx_PDB_ins_code = residueData.pdbx_PDB_ins_code, issue_type_group_id = residueData.issue_type_group_id, pdbx_PDB_model_num = residueData.pdbx_PDB_model_num, _rowCount = residueData._rowCount;
    var groups = parseIssueTypes(groupData);
    for (var i = 0; i < _rowCount; i++) {
        if (pdbx_PDB_model_num.value(i) !== modelData.modelNum)
            continue;
        var idx = modelData.atomicHierarchy.index.findResidue(label_entity_id.value(i), label_asym_id.value(i), auth_seq_id.value(i), pdbx_PDB_ins_code.value(i));
        ret.set(idx, groups.get(issue_type_group_id.value(i)));
    }
    return structure_1.IndexedCustomProperty.fromResidueMap(ret);
}
function parseIssueTypes(groupData) {
    var ret = new Map();
    var group_id = groupData.group_id, issue_type = groupData.issue_type;
    for (var i = 0; i < groupData._rowCount; i++) {
        var group = void 0;
        var id = group_id.value(i);
        if (ret.has(id))
            group = ret.get(id);
        else {
            group = [];
            ret.set(id, group);
        }
        group.push(issue_type.value(i));
    }
    return ret;
}
