"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var structure_quality_report_1 = require("../../../mol-model-props/pdbe/structure-quality-report");
var structure_1 = require("../../../mol-model/structure");
var color_1 = require("../../../mol-util/color");
var tables_1 = require("../../../mol-util/color/tables");
var ValidationColors = [
    color_1.Color.fromRgb(170, 170, 170),
    color_1.Color.fromRgb(0, 255, 0),
    color_1.Color.fromRgb(255, 255, 0),
    color_1.Color.fromRgb(255, 128, 0),
    color_1.Color.fromRgb(255, 0, 0),
];
var ValidationColorTable = [
    ['No Issues', ValidationColors[1]],
    ['One Issue', ValidationColors[2]],
    ['Two Issues', ValidationColors[3]],
    ['Three Or More Issues', ValidationColors[4]],
    ['Not Applicable', ValidationColors[9]]
];
function StructureQualityReportColorTheme(ctx, props) {
    var color;
    if (ctx.structure && !ctx.structure.isEmpty && ctx.structure.models[0].customProperties.has(structure_quality_report_1.StructureQualityReport.Descriptor)) {
        var getIssues_1 = structure_quality_report_1.StructureQualityReport.getIssues;
        color = function (location) {
            if (structure_1.StructureElement.isLocation(location)) {
                return ValidationColors[Math.min(3, getIssues_1(location).length) + 1];
            }
            return ValidationColors[0];
        };
    }
    else {
        color = function () { return ValidationColors[0]; };
    }
    return {
        factory: StructureQualityReportColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: 'Assigns residue colors according to the number of issues in the PDBe Validation Report.',
        legend: tables_1.TableLegend(ValidationColorTable)
    };
}
exports.StructureQualityReportColorTheme = StructureQualityReportColorTheme;
