"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var symmetry_gql_1 = require("./graphql/symmetry.gql");
var cif_1 = require("../../mol-io/writer/cif");
var db_1 = require("../../mol-data/db");
var encoder_1 = require("../../mol-io/writer/cif/encoder");
var schema_1 = require("../../mol-io/reader/cif/schema");
var cif_2 = require("../../mol-io/reader/cif");
var wrapper_1 = require("../../mol-model-props/common/wrapper");
var mol_task_1 = require("../../mol-task");
var graphql_client_1 = require("../../mol-util/graphql-client");
var _a = db_1.Column.Schema, str = _a.str, int = _a.int, float = _a.float, Aliased = _a.Aliased, Vector = _a.Vector, List = _a.List;
function getInstance(name) {
    return function (ctx) {
        var assemblySymmetry = AssemblySymmetry.get(ctx.structures[0].model);
        return assemblySymmetry ? encoder_1.Category.ofTable(assemblySymmetry.db[name]) : cif_1.CifWriter.Category.Empty;
    };
}
function getCategory(name) {
    return { name: name, instance: getInstance(name) };
}
function createDatabaseFromJson(assemblies) {
    var Schema = AssemblySymmetry.Schema;
    var featureRows = [];
    var clusterRows = [];
    var clusterMemberRows = [];
    var axisRows = [];
    var id = 1; // start feature ids at 1
    var clusterCount = 0;
    for (var i = 0, il = assemblies.length; i < il; ++i) {
        var _a = assemblies[i], pdbx_struct_assembly = _a.pdbx_struct_assembly, rcsb_struct_symmetry = _a.rcsb_struct_symmetry;
        if (!pdbx_struct_assembly || !rcsb_struct_symmetry)
            continue;
        var assembly_id = pdbx_struct_assembly.id;
        for (var j = 0, jl = rcsb_struct_symmetry.length; j < jl; ++j) {
            var rss = rcsb_struct_symmetry[j]; // TODO upstream, array members should not be nullable
            featureRows.push({
                id: id,
                assembly_id: assembly_id,
                type: rss.type,
                stoichiometry: rss.stoichiometry,
                kind: rss.kind,
                symbol: rss.symbol,
                oligomeric_state: rss.oligomeric_state
            });
            var clusters = rss.clusters;
            if (clusters) {
                for (var k = 0, kl = clusters.length; k < kl; ++k) {
                    var c = clusters[k]; // TODO upstream, array members should not be nullable
                    var cluster_id = clusterCount + k;
                    clusterRows.push({
                        id: cluster_id,
                        symmetry_id: id,
                        avg_rmsd: c.avg_rmsd || 0,
                    });
                    for (var l = 0, ll = c.members.length; l < ll; ++l) {
                        var m = c.members[l]; // TODO upstream, array members should not be nullable
                        clusterMemberRows.push({
                            cluster_id: cluster_id,
                            asym_id: m.asym_id,
                            pdbx_struct_oper_list_ids: (m.pdbx_struct_oper_list_ids || [])
                        });
                    }
                }
                clusterCount += clusters.length;
            }
            var axes = rss.rotation_axes;
            if (axes) {
                for (var k = 0, kl = axes.length; k < kl; ++k) {
                    var a = axes[k];
                    axisRows.push({
                        symmetry_id: id,
                        order: a.order,
                        start: a.start,
                        end: a.end // TODO upstream, array members should not be nullable
                    });
                }
            }
            id += 1;
        }
    }
    return db_1.Database.ofTables('assembly_symmetry', Schema, {
        rcsb_assembly_symmetry: db_1.Table.ofRows(Schema.rcsb_assembly_symmetry, featureRows),
        rcsb_assembly_symmetry_cluster: db_1.Table.ofRows(Schema.rcsb_assembly_symmetry_cluster, clusterRows),
        rcsb_assembly_symmetry_cluster_member: db_1.Table.ofRows(Schema.rcsb_assembly_symmetry_cluster_member, clusterMemberRows),
        rcsb_assembly_symmetry_axis: db_1.Table.ofRows(Schema.rcsb_assembly_symmetry_axis, axisRows)
    });
}
function createDatabaseFromCif(model) {
    var Schema = AssemblySymmetry.Schema;
    var rcsb_assembly_symmetry = schema_1.toTable(Schema.rcsb_assembly_symmetry, model.sourceData.frame.categories.rcsb_assembly_symmetry_feature);
    var rcsb_assembly_symmetry_cluster;
    if (model.sourceData.frame.categoryNames.includes('rcsb_assembly_symmetry_cluster')) {
        rcsb_assembly_symmetry_cluster = schema_1.toTable(Schema.rcsb_assembly_symmetry_cluster, model.sourceData.frame.categories.rcsb_assembly_symmetry_cluster);
    }
    else {
        rcsb_assembly_symmetry_cluster = schema_1.toTable(Schema.rcsb_assembly_symmetry_cluster, cif_2.CifCategory.empty);
    }
    var rcsb_assembly_symmetry_cluster_member;
    if (model.sourceData.frame.categoryNames.includes('rcsb_assembly_symmetry_cluster_member')) {
        rcsb_assembly_symmetry_cluster_member = schema_1.toTable(Schema.rcsb_assembly_symmetry_cluster_member, model.sourceData.frame.categories.rcsb_assembly_symmetry_cluster_member);
    }
    else {
        rcsb_assembly_symmetry_cluster_member = schema_1.toTable(Schema.rcsb_assembly_symmetry_cluster_member, cif_2.CifCategory.empty);
    }
    var rcsb_assembly_symmetry_axis;
    if (model.sourceData.frame.categoryNames.includes('rcsb_assembly_symmetry_axis')) {
        rcsb_assembly_symmetry_axis = schema_1.toTable(Schema.rcsb_assembly_symmetry_axis, model.sourceData.frame.categories.rcsb_assembly_symmetry_axis);
    }
    else {
        rcsb_assembly_symmetry_axis = schema_1.toTable(Schema.rcsb_assembly_symmetry_axis, cif_2.CifCategory.empty);
    }
    return db_1.Database.ofTables('rcsb_assembly_symmetry', Schema, {
        rcsb_assembly_symmetry: rcsb_assembly_symmetry,
        rcsb_assembly_symmetry_cluster: rcsb_assembly_symmetry_cluster,
        rcsb_assembly_symmetry_cluster_member: rcsb_assembly_symmetry_cluster_member,
        rcsb_assembly_symmetry_axis: rcsb_assembly_symmetry_axis
    });
}
var _Descriptor = {
    isStatic: true,
    name: 'rcsb_assembly_symmetry',
    cifExport: {
        prefix: 'rcsb',
        categories: [
            wrapper_1.PropertyWrapper.defaultInfoCategory('rcsb_assembly_symmetry_info', function (ctx) { return wrapper_1.PropertyWrapper.createInfo(); }),
            getCategory('rcsb_assembly_symmetry'),
            getCategory('rcsb_assembly_symmetry_cluster'),
            getCategory('rcsb_assembly_symmetry_cluster_member'),
            getCategory('rcsb_assembly_symmetry_axis')
        ]
    }
};
function AssemblySymmetry(db) {
    var f = db.rcsb_assembly_symmetry;
    var c = db.rcsb_assembly_symmetry_cluster;
    var cm = db.rcsb_assembly_symmetry_cluster_member;
    var a = db.rcsb_assembly_symmetry_axis;
    return {
        '@type': 'rcsb_assembly_symmetry',
        db: db,
        getSymmetries: function (assemblyIds) { return db_1.Table.pick(f, f._schema, function (i) { return assemblyIds.includes(f.assembly_id.value(i)); }); },
        getClusters: function (symmetryId) { return db_1.Table.pick(c, c._schema, function (i) { return c.symmetry_id.value(i) === symmetryId; }); },
        getClusterMembers: function (clusterId) { return db_1.Table.pick(cm, cm._schema, function (i) { return cm.cluster_id.value(i) === clusterId; }); },
        getAxes: function (symmetryId) { return db_1.Table.pick(a, a._schema, function (i) { return a.symmetry_id.value(i) === symmetryId; }); },
    };
}
exports.AssemblySymmetry = AssemblySymmetry;
(function (AssemblySymmetry) {
    function is(x) {
        return x['@type'] === 'rcsb_assembly_symmetry';
    }
    AssemblySymmetry.is = is;
    AssemblySymmetry.GraphQLEndpointURL = '//rest-staging.rcsb.org/graphql';
    AssemblySymmetry.Schema = {
        rcsb_assembly_symmetry_info: {
            updated_datetime_utc: db_1.Column.Schema.str
        },
        rcsb_assembly_symmetry: {
            /** Uniquely identifies a record in `rcsb_assembly_symmetry` */
            id: int,
            /**
             * A pointer to `pdbx_struct_assembly.id`.
             * The value 'deposited' refers to the coordinates as given in the file.
             * */
            assembly_id: str,
            /** Type of protein symmetry */
            kind: Aliased(str),
            /** Quantitative description of every individual subunit in a given protein */
            stoichiometry: List(',', function (x) { return x; }),
            /**
             * Symmetry symbol refers to point group or helical symmetry of identical subunits.
             * Contains point group symbol (e.g., C2, C5, D2, T, O, I) or H for helical symmetry.
             */
            symbol: str,
            /** Point group or helical symmetry */
            type: Aliased(str),
            /**
             * Oligomeric state refers to a composition of subunits in quaternary structure.
             * Quaternary structure may be composed either exclusively of several copies of identical
             * subunits, in which case they are termed homo-oligomers, or alternatively by at least
             * one copy of different subunits (hetero-oligomers). Quaternary structure composed of
             * a single subunit is demoted as 'Monomer'.
             */
            oligomeric_state: str,
        },
        rcsb_assembly_symmetry_cluster: {
            /** Uniquely identifies a record in `rcsb_assembly_symmetry_cluster` */
            id: int,
            /** A pointer to `rcsb_assembly_symmetry.id` */
            symmetry_id: int,
            /** Average RMSD between members of a given cluster */
            avg_rmsd: float
        },
        rcsb_assembly_symmetry_cluster_member: {
            /** A pointer to `rcsb_assembly_symmetry_cluster.id` */
            cluster_id: int,
            /** The `label_asym_id` value of the member */
            asym_id: str,
            /** List of `pdbx_struct_oper_list_id` values of the member */
            pdbx_struct_oper_list_ids: List(',', function (x) { return x; })
        },
        rcsb_assembly_symmetry_axis: {
            /** A pointer to `rcsb_assembly_symmetry.id` */
            symmetry_id: int,
            /**
             * The number of times (order of rotation) that a subunit can be repeated by a rotation
             * operation, being transformed into a new state indistinguishable from its starting state.
             */
            order: int,
            /** The x,y,z coordinate of the start point of a symmetry axis */
            start: Vector(3),
            /** The x,y,z coordinate of the end point of a symmetry axis */
            end: Vector(3)
        }
    };
    AssemblySymmetry.Descriptor = _Descriptor;
    function attachFromCifOrAPI(model, client, ctx) {
        return __awaiter(this, void 0, void 0, function () {
            var db, info, result, variables, e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (model.customProperties.has(AssemblySymmetry.Descriptor))
                            return [2 /*return*/, true];
                        info = wrapper_1.PropertyWrapper.tryGetInfoFromCif('rcsb_assembly_symmetry_info', model);
                        if (!info) return [3 /*break*/, 1];
                        db = createDatabaseFromCif(model);
                        return [3 /*break*/, 6];
                    case 1:
                        result = void 0;
                        console.log('model.label.toLowerCase()', model.label.toLowerCase());
                        variables = { pdbId: model.label.toLowerCase() };
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, 4, , 5]);
                        console.log('foo', client);
                        return [4 /*yield*/, client.request(ctx || mol_task_1.RuntimeContext.Synchronous, symmetry_gql_1.default, variables)];
                    case 3:
                        result = _a.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        e_1 = _a.sent();
                        console.error(e_1);
                        return [2 /*return*/, false];
                    case 5:
                        if (!result || !result.entry || !result.entry.assemblies)
                            return [2 /*return*/, false];
                        db = createDatabaseFromJson(result.entry.assemblies);
                        _a.label = 6;
                    case 6:
                        model.customProperties.add(AssemblySymmetry.Descriptor);
                        model._staticPropertyData.__RCSBAssemblySymmetry__ = AssemblySymmetry(db);
                        return [2 /*return*/, true];
                }
            });
        });
    }
    AssemblySymmetry.attachFromCifOrAPI = attachFromCifOrAPI;
    function createAttachTask(fetch) {
        var _this = this;
        return function (model) { return mol_task_1.Task.create('RCSB Assembly Symmetry', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (get(model))
                            return [2 /*return*/, true];
                        return [4 /*yield*/, attachFromCifOrAPI(model, new graphql_client_1.GraphQLClient(AssemblySymmetry.GraphQLEndpointURL, fetch), ctx)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        }); }); };
    }
    AssemblySymmetry.createAttachTask = createAttachTask;
    function get(model) {
        return model._staticPropertyData.__RCSBAssemblySymmetry__;
    }
    AssemblySymmetry.get = get;
})(AssemblySymmetry = exports.AssemblySymmetry || (exports.AssemblySymmetry = {}));
