"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var representation_1 = require("../../../mol-repr/structure/representation");
var assembly_symmetry_1 = require("../assembly-symmetry");
var db_1 = require("../../../mol-data/db");
var mesh_builder_1 = require("../../../mol-geo/geometry/mesh/mesh-builder");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var sphere_1 = require("../../../mol-geo/geometry/mesh/builder/sphere");
var cylinder_1 = require("../../../mol-geo/geometry/mesh/builder/cylinder");
var complex_visual_1 = require("../../../mol-repr/structure/complex-visual");
var mesh_1 = require("../../../mol-geo/geometry/mesh/mesh");
var loci_1 = require("../../../mol-model/loci");
var location_iterator_1 = require("../../../mol-geo/util/location-iterator");
var location_1 = require("../../../mol-model/location");
var int_1 = require("../../../mol-data/int");
var util_1 = require("../util");
exports.AssemblySymmetryAxesParams = __assign({ symmetryId: util_1.getSymmetrySelectParam(), sizeFactor: param_definition_1.ParamDefinition.Numeric(0.4, { min: 0, max: 3, step: 0.01 }) }, complex_visual_1.ComplexMeshParams, { radialSegments: param_definition_1.ParamDefinition.Numeric(16, { min: 3, max: 56, step: 1 }), detail: param_definition_1.ParamDefinition.Numeric(0, { min: 0, max: 3, step: 1 }) });
function getAssemblySymmetryAxesParams(ctx, structure) {
    var params = param_definition_1.ParamDefinition.clone(exports.AssemblySymmetryAxesParams);
    params.symmetryId = util_1.getSymmetrySelectParam(structure);
    params.unitKinds.isHidden = true;
    return params;
}
exports.getAssemblySymmetryAxesParams = getAssemblySymmetryAxesParams;
function AssemblySymmetryAxesRepresentation(ctx, getParams) {
    return representation_1.ComplexRepresentation('RCSB Assembly Symmetry Axes', ctx, getParams, AssemblySymmetryAxesVisual);
}
exports.AssemblySymmetryAxesRepresentation = AssemblySymmetryAxesRepresentation;
exports.AssemblySymmetryAxesRepresentationProvider = {
    label: 'RCSB Assembly Symmetry Axes',
    description: 'Displays assembly symmetry axes.',
    factory: AssemblySymmetryAxesRepresentation,
    getParams: getAssemblySymmetryAxesParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.AssemblySymmetryAxesParams),
    defaultColorTheme: 'uniform',
    defaultSizeTheme: 'uniform'
};
//
function AssemblySymmetryAxesVisual(materialId) {
    return complex_visual_1.ComplexMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.AssemblySymmetryAxesParams),
        createGeometry: createAssemblySymmetryAxesMesh,
        createLocationIterator: createLocationIterator,
        getLoci: getLoci,
        eachLocation: eachAxisLocation,
        setUpdateState: function (state, newProps, currentProps) {
            state.createGeometry = (newProps.sizeFactor !== currentProps.sizeFactor ||
                newProps.detail !== currentProps.detail ||
                newProps.symmetryId !== currentProps.symmetryId);
        }
    }, materialId);
}
exports.AssemblySymmetryAxesVisual = AssemblySymmetryAxesVisual;
function createLocationIterator(structure) {
    var assemblySymmetry = assembly_symmetry_1.AssemblySymmetry.get(structure.models[0]);
    var groupCount = assemblySymmetry ? assemblySymmetry.db.rcsb_assembly_symmetry_axis._rowCount : 0;
    return location_iterator_1.LocationIterator(groupCount, 1, function () { return location_1.NullLocation; });
}
function getLoci(pickingId, structure, id) {
    var objectId = pickingId.objectId, groupId = pickingId.groupId;
    if (id === objectId) {
        var assemblySymmetry = assembly_symmetry_1.AssemblySymmetry.get(structure.models[0]);
        if (assemblySymmetry) {
            return loci_1.createDataLoci(assemblySymmetry, 'axes', int_1.OrderedSet.ofSingleton(groupId));
        }
    }
    return loci_1.EmptyLoci;
}
function eachAxisLocation(loci, structure, apply) {
    var changed = false;
    if (!loci_1.isDataLoci(loci) || loci.tag !== 'axes')
        return false;
    var assemblySymmetry = assembly_symmetry_1.AssemblySymmetry.get(structure.models[0]);
    if (!assemblySymmetry || loci.data !== assemblySymmetry)
        return false;
    int_1.OrderedSet.forEach(loci.indices, function (v) { if (apply(int_1.Interval.ofSingleton(v)))
        changed = true; });
    return changed;
}
function createAssemblySymmetryAxesMesh(ctx, structure, theme, props, mesh) {
    var symmetryId = props.symmetryId, sizeFactor = props.sizeFactor;
    var assemblySymmetry = assembly_symmetry_1.AssemblySymmetry.get(structure.models[0]);
    if (!assemblySymmetry)
        return mesh_1.Mesh.createEmpty(mesh);
    var s = assemblySymmetry.db.rcsb_assembly_symmetry;
    var symmetry = db_1.Table.pickRow(s, function (i) { return s.id.value(i) === symmetryId; });
    if (!symmetry)
        return mesh_1.Mesh.createEmpty(mesh);
    // check if structure.units operators have symmetry.assembly_id
    if (!util_1.getAssemblyIds(structure.units).includes(symmetry.assembly_id))
        return mesh_1.Mesh.createEmpty(mesh);
    var axes = assemblySymmetry.db.rcsb_assembly_symmetry_axis;
    var vectorSpace = assembly_symmetry_1.AssemblySymmetry.Schema.rcsb_assembly_symmetry_axis.start.space;
    var radius = 1 * sizeFactor;
    var cylinderProps = { radiusTop: radius, radiusBottom: radius };
    var builderState = mesh_builder_1.MeshBuilder.createState(256, 128, mesh);
    for (var i = 0, il = axes._rowCount; i < il; ++i) {
        if (axes.symmetry_id.value(i) !== symmetryId)
            continue;
        var start = linear_algebra_1.Tensor.toVec3(vectorSpace, axes.start.value(i));
        var end = linear_algebra_1.Tensor.toVec3(vectorSpace, axes.end.value(i));
        builderState.currentGroup = i;
        sphere_1.addSphere(builderState, start, radius, 2);
        sphere_1.addSphere(builderState, end, radius, 2);
        cylinder_1.addCylinder(builderState, start, end, 1, cylinderProps);
    }
    return mesh_builder_1.MeshBuilder.getMesh(builderState);
}
exports.createAssemblySymmetryAxesMesh = createAssemblySymmetryAxesMesh;
