"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var db_1 = require("../../../mol-data/db");
var assembly_symmetry_1 = require("../assembly-symmetry");
var color_1 = require("../../../mol-util/color");
var structure_1 = require("../../../mol-model/structure");
var util_1 = require("../util");
var util_2 = require("../../../mol-theme/color/util");
var DefaultColor = color_1.Color(0xCCCCCC);
function getAsymId(unit) {
    switch (unit.kind) {
        case 0 /* Atomic */:
            return structure_1.StructureProperties.chain.label_asym_id;
        case 1 /* Spheres */:
        case 2 /* Gaussians */:
            return structure_1.StructureProperties.coarse.asym_id;
    }
}
function clusterMemberKey(assemblyId, asymId, operList) {
    return assemblyId + "-" + asymId + "-" + operList.join('|');
}
exports.AssemblySymmetryClusterColorThemeParams = __assign({}, util_2.getPaletteParams({ scaleList: 'RedYellowBlue' }), { symmetryId: util_1.getSymmetrySelectParam() });
function getAssemblySymmetryClusterColorThemeParams(ctx) {
    var params = param_definition_1.ParamDefinition.clone(exports.AssemblySymmetryClusterColorThemeParams);
    params.symmetryId = util_1.getSymmetrySelectParam(ctx.structure);
    return params;
}
exports.getAssemblySymmetryClusterColorThemeParams = getAssemblySymmetryClusterColorThemeParams;
function AssemblySymmetryClusterColorTheme(ctx, props) {
    var color = function () { return DefaultColor; };
    var legend;
    var symmetryId = props.symmetryId;
    if (ctx.structure && !ctx.structure.isEmpty && ctx.structure.models[0].customProperties.has(assembly_symmetry_1.AssemblySymmetry.Descriptor)) {
        var assemblySymmetry = assembly_symmetry_1.AssemblySymmetry.get(ctx.structure.models[0]);
        var s_1 = assemblySymmetry.db.rcsb_assembly_symmetry;
        var symmetry_1 = db_1.Table.pickRow(s_1, function (i) { return s_1.id.value(i) === symmetryId; });
        if (symmetry_1) {
            var clusters = assemblySymmetry.getClusters(symmetryId);
            if (clusters._rowCount) {
                var clusterByMember_1 = new Map();
                for (var i = 0, il = clusters._rowCount; i < il; ++i) {
                    var clusterMembers = assemblySymmetry.getClusterMembers(clusters.id.value(i));
                    for (var j = 0, jl = clusterMembers._rowCount; j < jl; ++j) {
                        var asymId = clusterMembers.asym_id.value(j);
                        var operList = clusterMembers.pdbx_struct_oper_list_ids.value(j);
                        if (operList.length === 0)
                            operList.push('1'); // TODO hack assuming '1' is the id of the identity operator
                        clusterByMember_1.set(clusterMemberKey(symmetry_1.assembly_id, asymId, operList), i);
                    }
                }
                var palette_1 = util_2.getPalette(clusters._rowCount, props);
                legend = palette_1.legend;
                color = function (location) {
                    if (structure_1.StructureElement.isLocation(location)) {
                        var assembly = location.unit.conformation.operator.assembly;
                        if (assembly && assembly.id === symmetry_1.assembly_id) {
                            var asymId = getAsymId(location.unit)(location);
                            var cluster = clusterByMember_1.get(clusterMemberKey(assembly.id, asymId, assembly.operList));
                            return cluster !== undefined ? palette_1.color(cluster) : DefaultColor;
                        }
                    }
                    return DefaultColor;
                };
            }
        }
    }
    return {
        factory: AssemblySymmetryClusterColorTheme,
        granularity: 'instance',
        color: color,
        props: props,
        description: 'Assigns chain colors according to assembly symmetry cluster membership.',
        legend: legend
    };
}
exports.AssemblySymmetryClusterColorTheme = AssemblySymmetryClusterColorTheme;
exports.AssemblySymmetryClusterColorThemeProvider = {
    label: 'RCSB Assembly Symmetry Cluster',
    factory: AssemblySymmetryClusterColorTheme,
    getParams: getAssemblySymmetryClusterColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.AssemblySymmetryClusterColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure && !ctx.structure.isEmpty && ctx.structure.models[0].customProperties.has(assembly_symmetry_1.AssemblySymmetry.Descriptor); }
};
