"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../mol-util/param-definition");
var assembly_symmetry_1 = require("./assembly-symmetry");
function getAssemblyIds(units) {
    var ids = new Set();
    units.forEach(function (u) {
        if (u.conformation.operator.assembly)
            ids.add(u.conformation.operator.assembly.id);
    });
    return Array.from(ids);
}
exports.getAssemblyIds = getAssemblyIds;
function getSymmetrySelectParam(structure) {
    var param = param_definition_1.ParamDefinition.Select(-1, [[-1, 'No Symmetries']]);
    if (structure && structure.models[0].customProperties.has(assembly_symmetry_1.AssemblySymmetry.Descriptor)) {
        var assemblySymmetry = assembly_symmetry_1.AssemblySymmetry.get(structure.models[0]);
        var assemblyIds = getAssemblyIds(structure.units);
        var s = assemblySymmetry.getSymmetries(assemblyIds);
        if (s._rowCount) {
            var options = [];
            for (var i = 0, il = s._rowCount; i < il; ++i) {
                options.push([s.id.value(i), s.assembly_id.value(i) + ": " + s.symbol.value(i) + " " + s.kind.value(i)]);
            }
            if (options.length) {
                param.options = options;
                param.defaultValue = options[0][0];
            }
        }
    }
    return param;
}
exports.getSymmetrySelectParam = getSymmetrySelectParam;
