/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Column, Table } from '../../mol-data/db';
import { Model } from '../../mol-model/structure';
import { CifWriter } from '../../mol-io/writer/cif';
export declare namespace ChemCompBond {
    type Property = Table<Schema['chem_comp_bond']>;
    function getFromModel(model: Model): Property;
    function get(model: Model): Property;
    const Schema: {
        chem_comp_bond: {
            molstar_protonation_variant: Column.Schema.Str;
            atom_id_1: Column.Schema.Str;
            atom_id_2: Column.Schema.Str;
            comp_id: Column.Schema.Str;
            value_order: Column.Schema.Aliased<"sing" | "doub" | "trip" | "quad" | "arom" | "poly" | "delo" | "pi">;
            pdbx_ordinal: Column.Schema.Int;
            pdbx_stereo_config: Column.Schema.Aliased<"E" | "Z" | "N">;
            pdbx_aromatic_flag: Column.Schema.Aliased<"N" | "Y">;
        };
    };
    type Schema = typeof Schema;
    const Descriptor: {
        isStatic: true;
        name: string;
        cifExport: {
            prefix: string;
            context(ctx: import("../../mol-model/structure").CifExportContext): Table<{
                molstar_protonation_variant: Column.Schema.Str;
                atom_id_1: Column.Schema.Str;
                atom_id_2: Column.Schema.Str;
                comp_id: Column.Schema.Str;
                value_order: Column.Schema.Aliased<"sing" | "doub" | "trip" | "quad" | "arom" | "poly" | "delo" | "pi">;
                pdbx_ordinal: Column.Schema.Int;
                pdbx_stereo_config: Column.Schema.Aliased<"E" | "Z" | "N">;
                pdbx_aromatic_flag: Column.Schema.Aliased<"N" | "Y">;
            }>;
            categories: {
                name: string;
                instance(ctx: Table<{
                    molstar_protonation_variant: Column.Schema.Str;
                    atom_id_1: Column.Schema.Str;
                    atom_id_2: Column.Schema.Str;
                    comp_id: Column.Schema.Str;
                    value_order: Column.Schema.Aliased<"sing" | "doub" | "trip" | "quad" | "arom" | "poly" | "delo" | "pi">;
                    pdbx_ordinal: Column.Schema.Int;
                    pdbx_stereo_config: Column.Schema.Aliased<"E" | "Z" | "N">;
                    pdbx_aromatic_flag: Column.Schema.Aliased<"N" | "Y">;
                }>): CifWriter.Category.Instance<any, any>;
            }[];
        };
    };
    function attachFromCifOrTable(model: Model, params: {
        wwPDB_apiSourceTable?: (model: Model) => Promise<Table<Schema['chem_comp_bond']>>;
    }): Promise<boolean>;
}
