/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { StructureElement } from './structure';
import { Link } from './structure/structure/unit/links';
import { Shape, ShapeGroup } from './shape';
import { Sphere3D } from '../mol-math/geometry';
import { Vec3 } from '../mol-math/linear-algebra';
import { OrderedSet } from '../mol-data/int';
import { Structure } from './structure/structure';
/** A Loci that includes every loci */
export declare const EveryLoci: {
    kind: "every-loci";
};
export declare type EveryLoci = typeof EveryLoci;
export declare function isEveryLoci(x: any): x is EveryLoci;
/** A Loci that is empty */
export declare const EmptyLoci: {
    kind: "empty-loci";
};
export declare type EmptyLoci = typeof EmptyLoci;
export declare function isEmptyLoci(x: any): x is EmptyLoci;
/** A generic data loci */
export interface DataLoci {
    readonly kind: 'data-loci';
    readonly data: any;
    readonly tag: string;
    readonly indices: OrderedSet<number>;
}
export declare function isDataLoci(x: any): x is DataLoci;
export declare function areDataLociEqual(a: DataLoci, b: DataLoci): boolean;
export declare function createDataLoci(data: any, tag: string, indices: OrderedSet<number>): DataLoci;
export { Loci };
declare type Loci = StructureElement.Loci | Structure.Loci | Link.Loci | EveryLoci | EmptyLoci | DataLoci | Shape.Loci | ShapeGroup.Loci;
declare namespace Loci {
    function areEqual(lociA: Loci, lociB: Loci): boolean;
    function getBoundingSphere(loci: Loci, boundingSphere?: Sphere3D): Sphere3D | undefined;
    function getCenter(loci: Loci, center?: Vec3): Vec3 | undefined;
}
