/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { AminoAlphabet, NuclecicAlphabet } from './constants';
import { Column } from '../../mol-data/db';
declare type Sequence = Sequence.Protein | Sequence.DNA | Sequence.RNA | Sequence.Generic;
declare namespace Sequence {
    const enum Kind {
        Protein = "protein",
        RNA = "RNA",
        DNA = "DNA",
        Generic = "generic"
    }
    interface Base<K extends Kind, Alphabet extends string> {
        readonly kind: K;
        readonly offset: number;
        readonly sequence: ArrayLike<Alphabet>;
    }
    interface Protein extends Base<Kind.Protein, AminoAlphabet> {
    }
    interface RNA extends Base<Kind.RNA, NuclecicAlphabet> {
    }
    interface DNA extends Base<Kind.DNA, NuclecicAlphabet> {
    }
    interface Generic extends Base<Kind.Generic, 'X' | '-'> {
    }
    function create(kind: Kind, sequence: string, offset?: number): Sequence;
    function getSequenceString(seq: Sequence): string;
    function ofResidueNames(residueName: Column<string>, seqId: Column<number>, modifiedMap?: ReadonlyMap<string, string>): Sequence;
}
export { Sequence };
