"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("./constants");
var Sequence;
(function (Sequence) {
    function create(kind, sequence, offset) {
        if (offset === void 0) { offset = 0; }
        return { kind: kind, sequence: sequence, offset: offset };
    }
    Sequence.create = create;
    function getSequenceString(seq) {
        return seq.sequence;
    }
    Sequence.getSequenceString = getSequenceString;
    function determineKind(names) {
        for (var i = 0, _i = Math.min(names.rowCount, 10); i < _i; i++) {
            var name_1 = names.value(i) || '';
            if (constants_1.getProteinOneLetterCode(name_1) !== 'X')
                return { kind: "protein" /* Protein */, code: constants_1.getProteinOneLetterCode };
            if (constants_1.getRnaOneLetterCode(name_1) !== 'X')
                return { kind: "RNA" /* RNA */, code: constants_1.getRnaOneLetterCode };
            if (constants_1.getDnaOneLetterCode(name_1) !== 'X')
                return { kind: "DNA" /* DNA */, code: constants_1.getDnaOneLetterCode };
        }
        return { kind: "generic" /* Generic */, code: function (v) { return 'X'; } };
    }
    function modCode(code, map) {
        return function (n) {
            var ret = code(n);
            if (ret !== 'X' || !map.has(n))
                return ret;
            return code(map.get(n));
        };
    }
    function ofResidueNames(residueName, seqId, modifiedMap) {
        if (seqId.rowCount === 0)
            throw new Error('cannot be empty');
        var _a = determineKind(residueName), kind = _a.kind, code = _a.code;
        if (!modifiedMap || modifiedMap.size === 0)
            return new Impl(kind, residueName, seqId, code);
        return new Impl(kind, residueName, seqId, modCode(code, modifiedMap));
    }
    Sequence.ofResidueNames = ofResidueNames;
    var Impl = /** @class */ (function () {
        function Impl(kind, residueName, seqId, code) {
            this.kind = kind;
            this.residueName = residueName;
            this.seqId = seqId;
            this.code = code;
            this._offset = 0;
            this._seq = void 0;
        }
        Object.defineProperty(Impl.prototype, "offset", {
            get: function () {
                if (this._seq !== void 0)
                    return this._offset;
                this.create();
                return this._offset;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Impl.prototype, "sequence", {
            get: function () {
                if (this._seq !== void 0)
                    return this._seq;
                this.create();
                return this._seq;
            },
            enumerable: true,
            configurable: true
        });
        Impl.prototype.create = function () {
            var maxSeqId = 0, minSeqId = Number.MAX_SAFE_INTEGER;
            for (var i = 0, _i = this.seqId.rowCount; i < _i; i++) {
                var id = this.seqId.value(i);
                if (maxSeqId < id)
                    maxSeqId = id;
                if (id < minSeqId)
                    minSeqId = id;
            }
            var count = maxSeqId - minSeqId + 1;
            var sequenceArray = new Array(maxSeqId + 1);
            for (var i = 0; i < count; i++) {
                sequenceArray[i] = '-';
            }
            for (var i = 0, _i = this.seqId.rowCount; i < _i; i++) {
                sequenceArray[this.seqId.value(i) - minSeqId] = this.code(this.residueName.value(i) || '');
            }
            this._seq = sequenceArray.join('');
            this._offset = minSeqId - 1;
        };
        return Impl;
    }());
})(Sequence || (Sequence = {}));
exports.Sequence = Sequence;
