"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../mol-util");
var int_1 = require("../../mol-data/int");
var geometry_1 = require("../../mol-geo/geometry/geometry");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var Shape;
(function (Shape) {
    function create(name, sourceData, geometry, getColor, getSize, getLabel, transforms) {
        return {
            id: mol_util_1.UUID.create22(),
            name: name,
            sourceData: sourceData,
            geometry: geometry,
            transforms: transforms || [linear_algebra_1.Mat4.identity()],
            get groupCount() { return geometry_1.Geometry.getGroupCount(geometry); },
            getColor: getColor,
            getSize: getSize,
            getLabel: getLabel
        };
    }
    Shape.create = create;
    function Loci(shape) { return { kind: 'shape-loci', shape: shape }; }
    Shape.Loci = Loci;
    function isLoci(x) { return !!x && x.kind === 'shape-loci'; }
    Shape.isLoci = isLoci;
    function areLociEqual(a, b) { return a.shape === b.shape; }
    Shape.areLociEqual = areLociEqual;
})(Shape = exports.Shape || (exports.Shape = {}));
var ShapeGroup;
(function (ShapeGroup) {
    function Location(shape, group, instance) {
        return { kind: 'group-location', shape: shape, group: group || 0, instance: instance || 0 };
    }
    ShapeGroup.Location = Location;
    function isLocation(x) {
        return !!x && x.kind === 'group-location';
    }
    ShapeGroup.isLocation = isLocation;
    function Loci(shape, groups, instance) {
        return { kind: 'group-loci', shape: shape, groups: groups, instance: instance };
    }
    ShapeGroup.Loci = Loci;
    function isLoci(x) {
        return !!x && x.kind === 'group-loci';
    }
    ShapeGroup.isLoci = isLoci;
    function areLociEqual(a, b) {
        if (a.shape !== b.shape)
            return false;
        if (a.groups.length !== b.groups.length)
            return false;
        if (a.instance !== b.instance)
            return false;
        for (var i = 0, il = a.groups.length; i < il; ++i) {
            var groupA = a.groups[i];
            var groupB = b.groups[i];
            if (!int_1.OrderedSet.areEqual(groupA.ids, groupB.ids))
                return false;
        }
        return true;
    }
    ShapeGroup.areLociEqual = areLociEqual;
})(ShapeGroup = exports.ShapeGroup || (exports.ShapeGroup = {}));
