"use strict";
/**
 * Copyright (c) 2017-2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var cif_1 = require("../../../../mol-io/writer/cif");
var structure_1 = require("../../structure");
var CifField = cif_1.CifWriter.Field;
var E = cif_1.CifWriter.Encodings;
var atom_site_fields = cif_1.CifWriter.fields()
    .str('group_PDB', structure_1.StructureProperties.residue.group_PDB)
    .index('id')
    .str('type_symbol', structure_1.StructureProperties.atom.type_symbol)
    .str('label_atom_id', structure_1.StructureProperties.atom.label_atom_id)
    .str('label_comp_id', structure_1.StructureProperties.residue.label_comp_id)
    .int('label_seq_id', structure_1.StructureProperties.residue.label_seq_id, {
    encoder: E.deltaRLE,
    valueKind: function (k, d) {
        var m = k.unit.model;
        return m.atomicHierarchy.residues.label_seq_id.valueKind(m.atomicHierarchy.residueAtomSegments.index[k.element]);
    }
})
    .str('label_alt_id', structure_1.StructureProperties.atom.label_alt_id)
    .str('pdbx_PDB_ins_code', structure_1.StructureProperties.residue.pdbx_PDB_ins_code)
    .str('label_asym_id', structure_1.StructureProperties.chain.label_asym_id)
    .str('label_entity_id', structure_1.StructureProperties.chain.label_entity_id)
    .float('Cartn_x', structure_1.StructureProperties.atom.x, { digitCount: 3, encoder: E.fixedPoint3 })
    .float('Cartn_y', structure_1.StructureProperties.atom.y, { digitCount: 3, encoder: E.fixedPoint3 })
    .float('Cartn_z', structure_1.StructureProperties.atom.z, { digitCount: 3, encoder: E.fixedPoint3 })
    .float('occupancy', structure_1.StructureProperties.atom.occupancy, { digitCount: 2, encoder: E.fixedPoint2 })
    .int('pdbx_formal_charge', structure_1.StructureProperties.atom.pdbx_formal_charge, { encoder: E.deltaRLE })
    .str('auth_atom_id', structure_1.StructureProperties.atom.auth_atom_id)
    .str('auth_comp_id', structure_1.StructureProperties.residue.auth_comp_id)
    .int('auth_seq_id', structure_1.StructureProperties.residue.auth_seq_id, { encoder: E.deltaRLE })
    .str('auth_asym_id', structure_1.StructureProperties.chain.auth_asym_id)
    .int('pdbx_PDB_model_num', structure_1.StructureProperties.unit.model_num, { encoder: E.deltaRLE })
    .str('operator_name', structure_1.StructureProperties.unit.operator_name, {
    shouldInclude: function (structure) { return structure.units.some(function (u) { return !u.conformation.operator.isIdentity; }); }
})
    .getFields();
exports._atom_site = {
    name: 'atom_site',
    instance: function (_a) {
        var structures = _a.structures;
        return {
            fields: atom_site_fields,
            source: structures.map(function (s) { return ({
                data: s,
                rowCount: s.elementCount,
                keys: function () { return s.elementLocations(); }
            }); })
        };
    }
};
function prepostfixed(prefix, postfix, name) {
    if (prefix && postfix)
        return prefix + "_" + name + "_" + postfix;
    if (prefix)
        return prefix + "_" + name;
    if (postfix)
        return name + "_" + postfix;
    return name;
}
function mappedProp(loc, prop) {
    return function (k, d) { return prop(loc(k, d)); };
}
function addModelNum(fields, getLocation, options) {
    if (options && options.includeModelNum) {
        fields.int('pdbx_PDB_model_num', mappedProp(getLocation, structure_1.StructureProperties.unit.model_num));
    }
}
function residueIdFields(getLocation, options) {
    var prefix = options && options.prefix, postfix = options && options.postfix;
    var ret = cif_1.CifWriter.fields()
        .str(prepostfixed(prefix, postfix, "label_comp_id"), mappedProp(getLocation, structure_1.StructureProperties.residue.label_comp_id))
        .int(prepostfixed(prefix, postfix, "label_seq_id"), mappedProp(getLocation, structure_1.StructureProperties.residue.label_seq_id), {
        encoder: E.deltaRLE,
        valueKind: function (k, d) {
            var e = getLocation(k, d);
            var m = e.unit.model;
            return m.atomicHierarchy.residues.label_seq_id.valueKind(m.atomicHierarchy.residueAtomSegments.index[e.element]);
        }
    })
        .str(prepostfixed(prefix, postfix, "pdbx_PDB_ins_code"), mappedProp(getLocation, structure_1.StructureProperties.residue.pdbx_PDB_ins_code))
        .str(prepostfixed(prefix, postfix, "label_asym_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.label_asym_id))
        .str(prepostfixed(prefix, postfix, "label_entity_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.label_entity_id))
        .str(prepostfixed(prefix, postfix, "auth_comp_id"), mappedProp(getLocation, structure_1.StructureProperties.residue.auth_comp_id))
        .int(prepostfixed(prefix, postfix, "auth_seq_id"), mappedProp(getLocation, structure_1.StructureProperties.residue.auth_seq_id), { encoder: E.deltaRLE })
        .str(prepostfixed(prefix, postfix, "auth_asym_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.auth_asym_id));
    addModelNum(ret, getLocation, options);
    return ret.getFields();
}
exports.residueIdFields = residueIdFields;
function chainIdFields(getLocation, options) {
    var prefix = options && options.prefix, postfix = options && options.postfix;
    var ret = CifField.build()
        .str(prepostfixed(prefix, postfix, "label_asym_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.label_asym_id))
        .str(prepostfixed(prefix, postfix, "label_entity_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.label_entity_id))
        .str(prepostfixed(prefix, postfix, "auth_asym_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.auth_asym_id));
    addModelNum(ret, getLocation, options);
    return ret.getFields();
}
exports.chainIdFields = chainIdFields;
function entityIdFields(getLocation, options) {
    var prefix = options && options.prefix, postfix = options && options.postfix;
    var ret = CifField.build()
        .str(prepostfixed(prefix, postfix, "label_entity_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.label_entity_id));
    addModelNum(ret, getLocation, options);
    return ret.getFields();
}
exports.entityIdFields = entityIdFields;
function atomIdFields(getLocation, options) {
    var prefix = options && options.prefix, postfix = options && options.postfix;
    var ret = cif_1.CifWriter.fields()
        .str(prepostfixed(prefix, postfix, "label_atom_id"), mappedProp(getLocation, structure_1.StructureProperties.atom.label_atom_id))
        .str(prepostfixed(prefix, postfix, "label_comp_id"), mappedProp(getLocation, structure_1.StructureProperties.residue.label_comp_id))
        .int(prepostfixed(prefix, postfix, "label_seq_id"), mappedProp(getLocation, structure_1.StructureProperties.residue.label_seq_id), {
        encoder: E.deltaRLE,
        valueKind: function (k, d) {
            var e = getLocation(k, d);
            var m = e.unit.model;
            return m.atomicHierarchy.residues.label_seq_id.valueKind(m.atomicHierarchy.residueAtomSegments.index[e.element]);
        }
    })
        .str(prepostfixed(prefix, postfix, "label_alt_id"), mappedProp(getLocation, structure_1.StructureProperties.atom.label_alt_id))
        .str(prepostfixed(prefix, postfix, "pdbx_PDB_ins_code"), mappedProp(getLocation, structure_1.StructureProperties.residue.pdbx_PDB_ins_code))
        .str(prepostfixed(prefix, postfix, "label_asym_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.label_asym_id))
        .str(prepostfixed(prefix, postfix, "label_entity_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.label_entity_id))
        .str(prepostfixed(prefix, postfix, "auth_atom_id"), mappedProp(getLocation, structure_1.StructureProperties.atom.auth_atom_id))
        .str(prepostfixed(prefix, postfix, "auth_comp_id"), mappedProp(getLocation, structure_1.StructureProperties.residue.auth_comp_id))
        .int(prepostfixed(prefix, postfix, "auth_seq_id"), mappedProp(getLocation, structure_1.StructureProperties.residue.auth_seq_id), { encoder: E.deltaRLE })
        .str(prepostfixed(prefix, postfix, "auth_asym_id"), mappedProp(getLocation, structure_1.StructureProperties.chain.auth_asym_id));
    addModelNum(ret, getLocation, options);
    return ret.getFields();
}
exports.atomIdFields = atomIdFields;
