"use strict";
/**
 * Copyright (c) 2017-2018 Mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var int_1 = require("../../../../mol-data/int");
var cif_1 = require("../../../../mol-io/writer/cif");
var structure_1 = require("../../../structure");
var CifField = cif_1.CifWriter.Field;
var pdbx_struct_mod_residue_fields = [
    CifField.index('id'),
    CifField.str("label_comp_id", function (i, xs) { return structure_1.StructureProperties.residue.label_comp_id(xs[i]); }),
    CifField.int("label_seq_id", function (i, xs) { return structure_1.StructureProperties.residue.label_seq_id(xs[i]); }),
    CifField.str("pdbx_PDB_ins_code", function (i, xs) { return structure_1.StructureProperties.residue.pdbx_PDB_ins_code(xs[i]); }),
    CifField.str("label_asym_id", function (i, xs) { return structure_1.StructureProperties.chain.label_asym_id(xs[i]); }),
    CifField.str("label_entity_id", function (i, xs) { return structure_1.StructureProperties.chain.label_entity_id(xs[i]); }),
    CifField.str("auth_comp_id", function (i, xs) { return structure_1.StructureProperties.residue.auth_comp_id(xs[i]); }),
    CifField.int("auth_seq_id", function (i, xs) { return structure_1.StructureProperties.residue.auth_seq_id(xs[i]); }),
    CifField.str("auth_asym_id", function (i, xs) { return structure_1.StructureProperties.chain.auth_asym_id(xs[i]); }),
    CifField.str('parent_comp_id', function (i, xs) { return xs[i].unit.model.properties.modifiedResidues.parentId.get(structure_1.StructureProperties.residue.label_comp_id(xs[i])); }),
    CifField.str('details', function (i, xs) { return xs[i].unit.model.properties.modifiedResidues.details.get(structure_1.StructureProperties.residue.label_comp_id(xs[i])); })
];
function getModifiedResidues(_a) {
    var structures = _a.structures;
    // TODO: can different models have differnt modified residues?
    var structure = structures[0], model = structure.model;
    var map = model.properties.modifiedResidues.parentId;
    if (!map.size)
        return [];
    var ret = [];
    var prop = structure_1.StructureProperties.residue.label_comp_id;
    var loc = structure_1.StructureElement.create();
    for (var _i = 0, _b = structure.units; _i < _b.length; _i++) {
        var unit = _b[_i];
        if (!structure_1.Unit.isAtomic(unit) || !unit.conformation.operator.isIdentity)
            continue;
        var residues = int_1.Segmentation.transientSegments(unit.model.atomicHierarchy.residueAtomSegments, unit.elements);
        loc.unit = unit;
        while (residues.hasNext) {
            var seg = residues.move();
            loc.element = unit.elements[seg.start];
            var name_1 = prop(loc);
            if (map.has(name_1)) {
                ret[ret.length] = structure_1.StructureElement.create(loc.unit, loc.element);
            }
        }
    }
    return ret;
}
exports._pdbx_struct_mod_residue = {
    name: 'pdbx_struct_mod_residue',
    instance: function (ctx) {
        var residues = getModifiedResidues(ctx);
        return {
            fields: pdbx_struct_mod_residue_fields,
            source: [{ data: residues, rowCount: residues.length }]
        };
    }
};
