"use strict";
/**
 * Copyright (c) 2017-2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var set_1 = require("../../../../mol-util/set");
var generic_1 = require("../../../../mol-data/generic");
var util_1 = require("../../../../mol-data/util");
var cif_1 = require("../../../../mol-io/writer/cif");
function getModelMmCifCategory(model, name) {
    if (model.sourceData.kind !== 'mmCIF')
        return;
    return model.sourceData.data[name];
}
exports.getModelMmCifCategory = getModelMmCifCategory;
function getUniqueResidueNamesFromStructures(structures) {
    return set_1.SetUtils.unionMany.apply(set_1.SetUtils, structures.map(function (s) { return s.uniqueResidueNames; }));
}
exports.getUniqueResidueNamesFromStructures = getUniqueResidueNamesFromStructures;
function getUniqueEntityIdsFromStructures(structures) {
    if (structures.length === 0)
        return new Set();
    var names = structures[0].model.entities.data.id;
    return new Set(getUniqueEntityIndicesFromStructures(structures).map(function (i) { return names.value(i); }));
}
exports.getUniqueEntityIdsFromStructures = getUniqueEntityIdsFromStructures;
function getUniqueEntityIndicesFromStructures(structures) {
    if (structures.length === 0)
        return [];
    if (structures.length === 1)
        return structures[0].entityIndices;
    var ret = generic_1.UniqueArray.create();
    for (var _i = 0, structures_1 = structures; _i < structures_1.length; _i++) {
        var s = structures_1[_i];
        for (var _a = 0, _b = s.entityIndices; _a < _b.length; _a++) {
            var e = _b[_a];
            generic_1.UniqueArray.add(ret, e, e);
        }
    }
    util_1.sortArray(ret.array);
    return ret.array;
}
exports.getUniqueEntityIndicesFromStructures = getUniqueEntityIndicesFromStructures;
function copy_mmCif_category(name, condition) {
    return {
        name: name,
        instance: function (_a) {
            var structures = _a.structures;
            if (condition && !condition(structures[0]))
                return cif_1.CifWriter.Category.Empty;
            var model = structures[0].model;
            if (model.sourceData.kind !== 'mmCIF')
                return cif_1.CifWriter.Category.Empty;
            var table = model.sourceData.data[name];
            if (!table || !table._rowCount)
                return cif_1.CifWriter.Category.Empty;
            return cif_1.CifWriter.Category.ofTable(table);
        }
    };
}
exports.copy_mmCif_category = copy_mmCif_category;
