"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var db_1 = require("../../../../mol-data/db");
var sequence_1 = require("../../../sequence");
var StructureSequence;
(function (StructureSequence) {
    function fromAtomicHierarchy(entities, hierarchy, modResMap) {
        var _a = hierarchy.residues, label_comp_id = _a.label_comp_id, label_seq_id = _a.label_seq_id;
        var chainAtomSegments = hierarchy.chainAtomSegments, residueAtomSegments = hierarchy.residueAtomSegments;
        var byEntityKey = {};
        var sequences = [];
        for (var cI = 0, _cI = hierarchy.chains._rowCount; cI < _cI; cI++) {
            var entityKey = hierarchy.index.getEntityFromChain(cI);
            // Only for polymers, trying to mirror _entity_poly_seq
            if (byEntityKey[entityKey] !== void 0 || entities.data.type.value(entityKey) !== 'polymer')
                continue;
            var start = cI;
            cI++;
            while (cI < _cI && entityKey === hierarchy.index.getEntityFromChain(cI) && entities.data.type.value(entityKey) !== 'polymer') {
                cI++;
            }
            cI--;
            var rStart = residueAtomSegments.index[chainAtomSegments.offsets[start]];
            var rEnd = residueAtomSegments.index[chainAtomSegments.offsets[cI + 1]];
            var compId = db_1.Column.window(label_comp_id, rStart, rEnd);
            var num = db_1.Column.window(label_seq_id, rStart, rEnd);
            byEntityKey[entityKey] = {
                entityId: entities.data.id.value(entityKey),
                compId: compId,
                num: num,
                sequence: sequence_1.Sequence.ofResidueNames(compId, num, modResMap)
            };
            sequences.push(byEntityKey[entityKey]);
        }
        return { byEntityKey: byEntityKey, sequences: sequences };
    }
    StructureSequence.fromAtomicHierarchy = fromAtomicHierarchy;
})(StructureSequence || (StructureSequence = {}));
exports.default = StructureSequence;
