/**
 * Copyright (c) 2017 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { SymmetryOperator } from '../../../../mol-math/geometry/symmetry-operator';
import { StructureQuery } from '../../query';
import { Model } from '../../model';
import { Spacegroup } from '../../../../mol-math/geometry';
/** Determine an atom set and a list of operators that should be applied to that set  */
export interface OperatorGroup {
    readonly selector: StructureQuery;
    readonly operators: ReadonlyArray<SymmetryOperator>;
}
export declare type OperatorGroups = ReadonlyArray<OperatorGroup>;
export declare class Assembly {
    private operatorsProvider;
    readonly id: string;
    readonly details: string;
    private _operators;
    readonly operatorGroups: OperatorGroups;
    constructor(id: string, details: string, operatorsProvider: () => OperatorGroups);
}
export declare namespace Assembly {
    function create(id: string, details: string, operatorsProvider: () => OperatorGroups): Assembly;
}
interface ModelSymmetry {
    readonly assemblies: ReadonlyArray<Assembly>;
    readonly spacegroup: Spacegroup;
    readonly isNonStandardCrytalFrame: boolean;
    readonly ncsOperators?: ReadonlyArray<SymmetryOperator>;
    _operators_333?: SymmetryOperator[];
}
declare namespace ModelSymmetry {
    const Default: ModelSymmetry;
    function findAssembly(model: Model, id: string): Assembly | undefined;
}
export { ModelSymmetry };
