"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("../../types");
var util_1 = require("../../../../../mol-model/structure/util");
function getAtomicDerivedData(data, index, chemicalComponentMap) {
    var _a = data.residues, label_comp_id = _a.label_comp_id, n = _a._rowCount;
    var traceElementIndex = new Int32Array(n);
    var directionFromElementIndex = new Int32Array(n);
    var directionToElementIndex = new Int32Array(n);
    var moleculeType = new Uint8Array(n);
    var moleculeTypeMap = new Map();
    for (var i = 0; i < n; ++i) {
        var compId = label_comp_id.value(i);
        var chemCompMap = chemicalComponentMap;
        var molType = void 0;
        if (moleculeTypeMap.has(compId)) {
            molType = moleculeTypeMap.get(compId);
        }
        else if (chemCompMap.has(compId)) {
            molType = types_1.getMoleculeType(chemCompMap.get(compId).type, compId);
            moleculeTypeMap.set(compId, molType);
        }
        else {
            molType = types_1.getMoleculeType(types_1.getComponentType(compId), compId);
            // TODO if unknown molecule type, use atom names to guess molecule type
            moleculeTypeMap.set(compId, molType);
        }
        moleculeType[i] = molType;
        var traceAtomId = util_1.getAtomIdForAtomRole(molType, 'trace');
        var traceIndex = index.findAtomsOnResidue(i, traceAtomId);
        if (traceIndex === -1) {
            var coarseAtomId = util_1.getAtomIdForAtomRole(molType, 'coarseBackbone');
            traceIndex = index.findAtomsOnResidue(i, coarseAtomId);
        }
        traceElementIndex[i] = traceIndex;
        var directionFromAtomId = util_1.getAtomIdForAtomRole(molType, 'directionFrom');
        directionFromElementIndex[i] = index.findAtomsOnResidue(i, directionFromAtomId);
        var directionToAtomId = util_1.getAtomIdForAtomRole(molType, 'directionTo');
        directionToElementIndex[i] = index.findAtomsOnResidue(i, directionToAtomId);
    }
    return {
        residue: {
            traceElementIndex: traceElementIndex,
            directionFromElementIndex: directionFromElementIndex,
            directionToElementIndex: directionToElementIndex,
            moleculeType: moleculeType,
        }
    };
}
exports.getAtomicDerivedData = getAtomicDerivedData;
