"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var int_1 = require("../../../../../mol-data/int");
var sorted_ranges_1 = require("../../../../../mol-data/int/sorted-ranges");
var types_1 = require("../../types");
var util_1 = require("../../../util");
var linear_algebra_1 = require("../../../../../mol-math/linear-algebra");
// TODO add gaps at the ends of the chains by comparing to the polymer sequence data
function areBackboneConnected(riStart, riEnd, data, segments, conformation, index, moleculeType) {
    var mtStart = moleculeType[riStart];
    var mtEnd = moleculeType[riEnd];
    if (!types_1.isPolymer(mtStart) || !types_1.isPolymer(mtEnd))
        return false;
    var eiStart = index.findAtomsOnResidue(riStart, util_1.getAtomIdForAtomRole(mtStart, 'backboneStart'));
    var eiEnd = index.findAtomsOnResidue(riEnd, util_1.getAtomIdForAtomRole(mtEnd, 'backboneEnd'));
    if (eiStart === -1 || eiEnd === -1) {
        eiStart = index.findAtomsOnResidue(riStart, util_1.getAtomIdForAtomRole(mtStart, 'coarseBackbone'));
        eiEnd = index.findAtomsOnResidue(riEnd, util_1.getAtomIdForAtomRole(mtEnd, 'coarseBackbone'));
    }
    var x = conformation.x, y = conformation.y, z = conformation.z;
    var pStart = linear_algebra_1.Vec3.create(x[eiStart], y[eiStart], z[eiStart]);
    var pEnd = linear_algebra_1.Vec3.create(x[eiEnd], y[eiEnd], z[eiEnd]);
    return linear_algebra_1.Vec3.distance(pStart, pEnd) < 10; // TODO better distance check, take into account if protein/nucleic and if coarse
}
function getAtomicRanges(data, segments, conformation, index, moleculeType) {
    var polymerRanges = [];
    var gapRanges = [];
    var cyclicPolymerMap = new Map();
    var chainIt = int_1.Segmentation.transientSegments(segments.chainAtomSegments, int_1.Interval.ofBounds(0, data.atoms._rowCount));
    var residueIt = int_1.Segmentation.transientSegments(segments.residueAtomSegments, int_1.Interval.ofBounds(0, data.atoms._rowCount));
    var label_seq_id = data.residues.label_seq_id;
    var prevSeqId;
    var prevStart;
    var prevEnd;
    var startIndex;
    while (chainIt.hasNext) {
        var chainSegment = chainIt.move();
        residueIt.setSegment(chainSegment);
        prevSeqId = -1;
        prevStart = -1;
        prevEnd = -1;
        startIndex = -1;
        var riStart = segments.residueAtomSegments.index[chainSegment.start];
        var riEnd = segments.residueAtomSegments.index[chainSegment.end - 1];
        if (areBackboneConnected(riStart, riEnd, data, segments, conformation, index, moleculeType)) {
            cyclicPolymerMap.set(riStart, riEnd);
            cyclicPolymerMap.set(riEnd, riStart);
        }
        while (residueIt.hasNext) {
            var residueSegment = residueIt.move();
            var residueIndex = residueSegment.index;
            var seqId = label_seq_id.value(residueIndex);
            if (types_1.isPolymer(moleculeType[residueIndex])) {
                if (startIndex !== -1) {
                    if (seqId !== prevSeqId + 1) {
                        polymerRanges.push(startIndex, prevEnd - 1);
                        gapRanges.push(prevStart, residueSegment.end - 1);
                        startIndex = residueSegment.start;
                    }
                    else if (!residueIt.hasNext) {
                        polymerRanges.push(startIndex, residueSegment.end - 1);
                    }
                    else {
                        var riStart_1 = segments.residueAtomSegments.index[residueSegment.start];
                        var riEnd_1 = segments.residueAtomSegments.index[prevEnd - 1];
                        if (!areBackboneConnected(riStart_1, riEnd_1, data, segments, conformation, index, moleculeType)) {
                            polymerRanges.push(startIndex, prevEnd - 1);
                            startIndex = residueSegment.start;
                        }
                    }
                }
                else {
                    startIndex = residueSegment.start; // start polymer
                }
            }
            else {
                if (startIndex !== -1) {
                    polymerRanges.push(startIndex, prevEnd - 1);
                    startIndex = -1;
                }
            }
            prevStart = residueSegment.start;
            prevEnd = residueSegment.end;
            prevSeqId = seqId;
        }
    }
    return {
        polymerRanges: sorted_ranges_1.default.ofSortedRanges(polymerRanges),
        gapRanges: sorted_ranges_1.default.ofSortedRanges(gapRanges),
        cyclicPolymerMap: cyclicPolymerMap
    };
}
exports.getAtomicRanges = getAtomicRanges;
