/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { Structure, StructureElement, Unit } from '../structure';
import { ElementIndex } from '../model';
import { Link } from '../structure/unit/links';
export interface QueryContextView {
    readonly element: StructureElement;
    readonly currentStructure: Structure;
}
export declare class QueryContext implements QueryContextView {
    private currentElementStack;
    private currentAtomicLinkStack;
    private currentStructureStack;
    private inputStructureStack;
    private timeCreated;
    readonly timeoutMs: number;
    readonly inputStructure: Structure;
    /** Current element */
    readonly element: StructureElement;
    currentStructure: Structure;
    /** Current link between atoms */
    readonly atomicLink: Link.Location<Unit.Atomic>;
    setElement(unit: Unit, e: ElementIndex): void;
    pushCurrentElement(): StructureElement;
    popCurrentElement(): void;
    pushCurrentLink(): Link.Location<Unit.Atomic>;
    popCurrentLink(): void;
    pushCurrentStructure(): void;
    popCurrentStructure(): void;
    pushInputStructure(structure: Structure): void;
    popInputStructure(): void;
    throwIfTimedOut(): void;
    constructor(structure: Structure, timeoutMs?: number);
}
export interface QueryPredicate {
    (ctx: QueryContext): boolean;
}
export interface QueryFn<T = any> {
    (ctx: QueryContext): T;
}
