"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../structure");
var now_1 = require("../../../mol-util/now");
var links_1 = require("../structure/unit/links");
var QueryContext = /** @class */ (function () {
    function QueryContext(structure, timeoutMs) {
        if (timeoutMs === void 0) { timeoutMs = 0; }
        this.currentElementStack = [];
        this.currentAtomicLinkStack = [];
        this.currentStructureStack = [];
        this.inputStructureStack = [];
        this.timeCreated = now_1.now();
        /** Current element */
        this.element = structure_1.StructureElement.create();
        this.currentStructure = void 0;
        /** Current link between atoms */
        this.atomicLink = links_1.Link.Location();
        this.inputStructure = structure;
        this.timeoutMs = timeoutMs;
    }
    QueryContext.prototype.setElement = function (unit, e) {
        this.element.unit = unit;
        this.element.element = e;
    };
    QueryContext.prototype.pushCurrentElement = function () {
        this.currentElementStack[this.currentElementStack.length] = this.element;
        this.element = structure_1.StructureElement.create();
        return this.element;
    };
    QueryContext.prototype.popCurrentElement = function () {
        this.element = this.currentElementStack.pop();
    };
    QueryContext.prototype.pushCurrentLink = function () {
        if (this.atomicLink)
            this.currentAtomicLinkStack.push(this.atomicLink);
        this.atomicLink = links_1.Link.Location();
        return this.atomicLink;
    };
    QueryContext.prototype.popCurrentLink = function () {
        if (this.currentAtomicLinkStack.length > 0) {
            this.atomicLink = this.currentAtomicLinkStack.pop();
        }
        else {
            this.atomicLink = void 0;
        }
    };
    QueryContext.prototype.pushCurrentStructure = function () {
        if (this.currentStructure)
            this.currentStructureStack.push(this.currentStructure);
    };
    QueryContext.prototype.popCurrentStructure = function () {
        if (this.currentStructureStack.length)
            this.currentStructure = this.currentStructureStack.pop();
        else
            this.currentStructure = void 0;
    };
    QueryContext.prototype.pushInputStructure = function (structure) {
        this.inputStructureStack.push(this.inputStructure);
        this.inputStructure = structure;
    };
    QueryContext.prototype.popInputStructure = function () {
        if (this.inputStructureStack.length === 0)
            throw new Error('Must push before pop.');
        this.inputStructure = this.inputStructureStack.pop();
    };
    QueryContext.prototype.throwIfTimedOut = function () {
        if (this.timeoutMs === 0)
            return;
        if (now_1.now() - this.timeCreated > this.timeoutMs) {
            throw new Error("The query took too long to execute (> " + this.timeoutMs / 1000 + "s).");
        }
    };
    return QueryContext;
}());
exports.QueryContext = QueryContext;
