"use strict";
/**
 * Copyright (c) 2017 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var int_1 = require("../../../../mol-data/int");
var element_1 = require("../../../../mol-model/structure/structure/element");
var structure_1 = require("../../structure");
var structure_2 = require("../../structure/structure");
var selection_1 = require("../selection");
function atomicSequence() {
    return function (ctx) {
        var inputStructure = ctx.inputStructure;
        var l = element_1.default.create();
        var units = [];
        for (var _i = 0, _a = inputStructure.units; _i < _a.length; _i++) {
            var unit = _a[_i];
            if (unit.kind !== 0 /* Atomic */)
                continue;
            l.unit = unit;
            var elements = unit.elements;
            l.element = elements[0];
            if (structure_1.StructureProperties.entity.type(l) !== 'polymer')
                continue;
            var residuesIt = int_1.Segmentation.transientSegments(unit.model.atomicHierarchy.residueAtomSegments, elements);
            var residueCount = 0;
            while (residuesIt.hasNext) {
                residueCount++;
                residuesIt.move();
            }
            if (residueCount < 8)
                continue;
            units.push(unit);
        }
        return selection_1.StructureSelection.Singletons(inputStructure, new structure_2.default(units, inputStructure));
    };
}
exports.atomicSequence = atomicSequence;
function water() {
    return function (ctx) {
        var inputStructure = ctx.inputStructure;
        var l = element_1.default.create();
        var units = [];
        for (var _i = 0, _a = inputStructure.units; _i < _a.length; _i++) {
            var unit = _a[_i];
            if (unit.kind !== 0 /* Atomic */)
                continue;
            l.unit = unit;
            var elements = unit.elements;
            l.element = elements[0];
            if (structure_1.StructureProperties.entity.type(l) !== 'water')
                continue;
            units.push(unit);
        }
        return selection_1.StructureSelection.Singletons(inputStructure, new structure_2.default(units, inputStructure));
    };
}
exports.water = water;
function atomicHet() {
    return function (ctx) {
        var inputStructure = ctx.inputStructure;
        var l = element_1.default.create();
        var units = [];
        for (var _i = 0, _a = inputStructure.units; _i < _a.length; _i++) {
            var unit = _a[_i];
            if (unit.kind !== 0 /* Atomic */)
                continue;
            l.unit = unit;
            var elements = unit.elements;
            l.element = elements[0];
            if (structure_1.StructureProperties.entity.type(l) === 'water')
                continue;
            if (structure_1.StructureProperties.entity.type(l) === 'polymer') {
                var residuesIt = int_1.Segmentation.transientSegments(unit.model.atomicHierarchy.residueAtomSegments, elements);
                var residueCount = 0;
                while (residuesIt.hasNext) {
                    residueCount++;
                    residuesIt.move();
                }
                if (residueCount >= 8)
                    continue;
            }
            units.push(unit);
        }
        return selection_1.StructureSelection.Singletons(inputStructure, new structure_2.default(units, inputStructure));
    };
}
exports.atomicHet = atomicHet;
function spheres() {
    return function (ctx) {
        var inputStructure = ctx.inputStructure;
        var units = [];
        for (var _i = 0, _a = inputStructure.units; _i < _a.length; _i++) {
            var unit = _a[_i];
            if (unit.kind !== 1 /* Spheres */)
                continue;
            units.push(unit);
        }
        return selection_1.StructureSelection.Singletons(inputStructure, new structure_2.default(units, inputStructure));
    };
}
exports.spheres = spheres;
