/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Sebastian Bittrich <sebastian.bittrich@rcsb.org>
 */
import Structure from './structure';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
declare namespace AccessibleSurfaceArea {
    const VdWLookup: number[];
    /**
     * Adapts the BioJava implementation by Jose Duarte. That implementation is based on the publication by Shrake, A., and
     * J. A. Rupley. "Environment and Exposure to Solvent of Protein Atoms. Lysozyme and Insulin." JMB (1973).
     */
    function compute(structure: Structure, params?: Partial<PD.Values<AccessibleSurfaceAreaComputationParams>>): Promise<AccessibleSurfaceArea>;
    /** Maximum accessible surface area observed for amino acids. Taken from: http://dx.doi.org/10.1371/journal.pone.0080635 */
    const MaxAsa: {
        'ALA': number;
        'ARG': number;
        'ASN': number;
        'ASP': number;
        'CYS': number;
        'GLU': number;
        'GLN': number;
        'GLY': number;
        'HIS': number;
        'ILE': number;
        'LEU': number;
        'LYS': number;
        'MET': number;
        'PHE': number;
        'PRO': number;
        'SER': number;
        'THR': number;
        'TRP': number;
        'TYR': number;
        'VAL': number;
    };
    const DefaultMaxAsa = 121;
    const AccessibleSurfaceAreaComputationParams: {
        numberOfSpherePoints: PD.Numeric;
        probeSize: PD.Numeric;
        buriedRasaThreshold: PD.Numeric;
        nonPolymer: PD.BooleanParam;
    };
    type AccessibleSurfaceAreaComputationParams = typeof AccessibleSurfaceAreaComputationParams;
    namespace SolventAccessibility {
        const is: (t: number, f: Flag) => boolean;
        const create: (f: Flag) => number;
        const enum Flag {
            _ = 0,
            BURIED = 1,
            ACCESSIBLE = 2
        }
    }
}
interface AccessibleSurfaceArea {
    readonly accessibleSurfaceArea?: ArrayLike<number>;
    readonly relativeAccessibleSurfaceArea?: ArrayLike<number>;
    buried(index: number): boolean;
}
export { AccessibleSurfaceArea };
