"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Sebastian Bittrich <sebastian.bittrich@rcsb.org>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_task_1 = require("../../../mol-task");
var mol_util_1 = require("../../../mol-util");
var param_definition_1 = require("../../../mol-util/param-definition");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var types_1 = require("../model/types");
var atomic_1 = require("../model/properties/atomic");
var common_1 = require("./unit/links/common");
var AccessibleSurfaceArea;
(function (AccessibleSurfaceArea) {
    // Chothia's amino acid and nucleotide atom vdw radii
    AccessibleSurfaceArea.VdWLookup = [
        -1.0,
        1.76,
        1.87,
        1.65,
        1.50,
        1.40,
        1.85,
        1.80,
        1.60,
        1.40 // 9: P (nucleic)
    ]; // can still be appended on-the-fly for rare elements like selenium
    /**
     * Adapts the BioJava implementation by Jose Duarte. That implementation is based on the publication by Shrake, A., and
     * J. A. Rupley. "Environment and Exposure to Solvent of Protein Atoms. Lysozyme and Insulin." JMB (1973).
     */
    function compute(structure, params) {
        var _this = this;
        if (params === void 0) { params = {}; }
        params = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(AccessibleSurfaceArea.AccessibleSurfaceAreaComputationParams), params);
        return mol_task_1.Task.create('Compute Accessible Surface Area', function (rtctx) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _compute(rtctx, structure, params)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        }); }).run();
    }
    AccessibleSurfaceArea.compute = compute;
    function _compute(rtctx, structure, params) {
        if (params === void 0) { params = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var ctx;
            return __generator(this, function (_a) {
                ctx = initialize(rtctx, structure, params);
                assignRadiusForHeavyAtoms(ctx);
                computePerResidue(ctx);
                normalizeAccessibleSurfaceArea(ctx);
                return [2 /*return*/, {
                        accessibleSurfaceArea: ctx.accessibleSurfaceArea,
                        relativeAccessibleSurfaceArea: ctx.relativeAccessibleSurfaceArea,
                        buried: function (index) { return ctx.relativeAccessibleSurfaceArea[index] < 0.16; } // TODO this doesnt seem super elegant
                    }];
            });
        });
    }
    function normalizeAccessibleSurfaceArea(ctx) {
        var accessibleSurfaceArea = ctx.accessibleSurfaceArea, relativeAccessibleSurfaceArea = ctx.relativeAccessibleSurfaceArea, structure = ctx.structure;
        var _a = structure.model.atomicHierarchy, residues = _a.residues, derived = _a.derived;
        for (var i = 0; i < residues.label_comp_id.rowCount; ++i) {
            // skip entities not part of a polymer chain
            if (!ctx.params.nonPolymer) {
                if (!types_1.isPolymer(derived.residue.moleculeType[i]))
                    continue;
            }
            var maxAsa = AccessibleSurfaceArea.MaxAsa[residues.label_comp_id.value(i)];
            var rasa = accessibleSurfaceArea[i] / (maxAsa === undefined ? AccessibleSurfaceArea.DefaultMaxAsa : maxAsa);
            relativeAccessibleSurfaceArea[i] = rasa;
        }
    }
    function computePerResidue(ctx) {
        return __awaiter(this, void 0, void 0, function () {
            var structure, atomRadius, accessibleSurfaceArea, spherePoints, cons, params, maxLookupRadius, probeSize, model, atomCount, _a, x, y, z, residueAtomSegments, lookup3d, position, aPos, bPos, testPoint, aI, radius1, _b, count, units, indices, squaredDistances, cutoff1, neighbors, iI, bI, radius2, cutoff2, scalar, accessiblePointCount, sI, spherePoint, accessible, _nI, nI, radius3, cutoff3;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        structure = ctx.structure, atomRadius = ctx.atomRadius, accessibleSurfaceArea = ctx.accessibleSurfaceArea, spherePoints = ctx.spherePoints, cons = ctx.cons, params = ctx.params, maxLookupRadius = ctx.maxLookupRadius;
                        probeSize = params.probeSize;
                        model = structure.model, atomCount = structure.elementCount;
                        _a = model.atomicConformation, x = _a.x, y = _a.y, z = _a.z;
                        residueAtomSegments = model.atomicHierarchy.residueAtomSegments;
                        lookup3d = structure.lookup3d;
                        position = function (i, v) { return linear_algebra_1.Vec3.set(v, x[i], y[i], z[i]); };
                        aPos = linear_algebra_1.Vec3.zero();
                        bPos = linear_algebra_1.Vec3.zero();
                        testPoint = linear_algebra_1.Vec3.zero();
                        aI = 0;
                        _c.label = 1;
                    case 1:
                        if (!(aI < atomCount)) return [3 /*break*/, 5];
                        if (!(aI % 10000 === 0)) return [3 /*break*/, 3];
                        if (!ctx.rtctx.shouldUpdate) return [3 /*break*/, 3];
                        return [4 /*yield*/, ctx.rtctx.update({ message: 'calculating accessible surface area', current: aI, max: atomCount })];
                    case 2:
                        _c.sent();
                        _c.label = 3;
                    case 3:
                        radius1 = AccessibleSurfaceArea.VdWLookup[atomRadius[aI]];
                        if (radius1 === AccessibleSurfaceArea.VdWLookup[0])
                            return [3 /*break*/, 4];
                        _b = lookup3d.find(x[aI], y[aI], z[aI], maxLookupRadius), count = _b.count, units = _b.units, indices = _b.indices, squaredDistances = _b.squaredDistances;
                        cutoff1 = probeSize + probeSize + radius1;
                        neighbors = [];
                        for (iI = 0; iI < count; ++iI) {
                            bI = units[iI].elements[indices[iI]];
                            radius2 = AccessibleSurfaceArea.VdWLookup[atomRadius[bI]];
                            if (aI === bI || radius2 === AccessibleSurfaceArea.VdWLookup[0])
                                continue;
                            cutoff2 = (cutoff1 + radius2) * (cutoff1 + radius2);
                            if (squaredDistances[iI] < cutoff2) {
                                neighbors[neighbors.length] = bI;
                            }
                        }
                        // for all neighbors: test all sphere points
                        position(aI, aPos);
                        scalar = probeSize + radius1;
                        accessiblePointCount = 0;
                        for (sI = 0; sI < spherePoints.length; ++sI) {
                            spherePoint = spherePoints[sI];
                            testPoint = [spherePoint[0] * scalar + aPos[0], spherePoint[1] * scalar + aPos[1], spherePoint[2] * scalar + aPos[2]];
                            accessible = true;
                            for (_nI = 0; _nI < neighbors.length; ++_nI) {
                                nI = neighbors[_nI];
                                position(nI, bPos);
                                radius3 = AccessibleSurfaceArea.VdWLookup[atomRadius[nI]];
                                cutoff3 = (radius3 + probeSize) * (radius3 + probeSize);
                                if (linear_algebra_1.Vec3.squaredDistance(testPoint, bPos) < cutoff3) {
                                    accessible = false;
                                    break;
                                }
                            }
                            if (accessible)
                                ++accessiblePointCount;
                        }
                        accessibleSurfaceArea[residueAtomSegments.index[aI]] += cons * accessiblePointCount * scalar * scalar;
                        _c.label = 4;
                    case 4:
                        ++aI;
                        return [3 /*break*/, 1];
                    case 5: return [2 /*return*/];
                }
            });
        });
    }
    function assignRadiusForHeavyAtoms(ctx) {
        var structure = ctx.structure;
        var model = structure.model, atomCount = structure.elementCount;
        var _a = model.atomicHierarchy, atomInfo = _a.atoms, derived = _a.derived, residues = _a.residues, residueAtomSegments = _a.residueAtomSegments;
        var label_comp_id = residues.label_comp_id;
        var moleculeType = derived.residue.moleculeType;
        var type_symbol = atomInfo.type_symbol, label_atom_id = atomInfo.label_atom_id;
        var residueCount = moleculeType.length;
        // with atom and residue count at hand: initialize arrays
        ctx.atomRadius = new Int8Array(atomCount - 1);
        ctx.accessibleSurfaceArea = new Float32Array(residueCount - 1);
        ctx.relativeAccessibleSurfaceArea = new Float32Array(residueCount - 1);
        for (var aI = 0; aI < atomCount; ++aI) {
            var rI = residueAtomSegments.index[aI];
            var element = type_symbol.value(aI);
            var elementIdx = common_1.getElementIdx(element);
            // skip hydrogen atoms
            if (common_1.isHydrogen(elementIdx)) {
                ctx.atomRadius[aI] = AccessibleSurfaceArea.VdWLookup[0];
                continue;
            }
            var residueType = moleculeType[rI];
            // skip non-polymer groups
            if (!ctx.params.nonPolymer) {
                if (!types_1.isPolymer(residueType)) {
                    ctx.atomRadius[aI] = AccessibleSurfaceArea.VdWLookup[0];
                    continue;
                }
            }
            var atomId = label_atom_id.value(aI);
            var compId = label_comp_id.value(rI);
            // handle modified residues
            var parentId = model.properties.modifiedResidues.parentId.get(compId);
            if (parentId !== void 0)
                compId = parentId;
            if (types_1.isNucleic(residueType)) {
                ctx.atomRadius[aI] = determineRadiusNucl(atomId, element, compId);
            }
            else if (residueType === 4 /* protein */) {
                ctx.atomRadius[aI] = determineRadiusAmino(atomId, element, compId);
            }
            else {
                ctx.atomRadius[aI] = handleNonStandardCase(element);
            }
        }
    }
    /**
     * Gets the van der Waals radius of the given atom following the values defined by Chothia (1976)
     * J.Mol.Biol.105,1-14. NOTE: the vdw values defined by the paper assume no Hydrogens and thus "inflates" slightly
     * the heavy atoms to account for Hydrogens.
     */
    function determineRadiusAmino(atomId, element, compId) {
        switch (element) {
            case 'O':
                return 5;
            case 'S':
                return 6;
            case 'N':
                return atomId === 'NZ' ? 4 : 3;
            case 'C':
                switch (atomId) {
                    case 'C':
                    case 'CE1':
                    case 'CE2':
                    case 'CE3':
                    case 'CH2':
                    case 'CZ':
                    case 'CZ2':
                    case 'CZ3':
                        return 1;
                    case 'CA':
                    case 'CB':
                    case 'CE':
                    case 'CG1':
                    case 'CG2':
                        return 2;
                    default:
                        switch (compId) {
                            case 'PHE':
                            case 'TRP':
                            case 'TYR':
                            case 'HIS':
                            case 'ASP':
                            case 'ASN':
                                return 1;
                            case 'PRO':
                            case 'LYS':
                            case 'ARG':
                            case 'MET':
                            case 'ILE':
                            case 'LEU':
                                return 2;
                            case 'GLU':
                            case 'GLN':
                                return atomId === 'CD' ? 1 : 2;
                        }
                }
        }
        return handleNonStandardCase(element);
    }
    function determineRadiusNucl(atomId, element, compId) {
        switch (element) {
            case 'C':
                return 7;
            case 'N':
                return 8;
            case 'P':
                return 9;
            case 'O':
                return 5;
        }
        return handleNonStandardCase(element);
    }
    function handleNonStandardCase(element) {
        var radius = atomic_1.VdwRadius(element);
        var index = AccessibleSurfaceArea.VdWLookup.indexOf(radius);
        if (index === -1) {
            // add novel value to lookup array
            index = AccessibleSurfaceArea.VdWLookup.length;
            AccessibleSurfaceArea.VdWLookup[index] = radius;
        }
        return index;
    }
    function initialize(rtctx, structure, params) {
        return {
            rtctx: rtctx,
            structure: structure,
            params: params,
            spherePoints: generateSpherePoints(params.numberOfSpherePoints),
            cons: 4.0 * Math.PI / params.numberOfSpherePoints,
            maxLookupRadius: 2 * params.probeSize + 2 * AccessibleSurfaceArea.VdWLookup[2] // 2x probe size + 2x largest VdW
        };
    }
    /** Creates a collection of points on a sphere by the Golden Section Spiral algorithm. */
    function generateSpherePoints(numberOfSpherePoints) {
        var points = [];
        var inc = Math.PI * (3.0 - Math.sqrt(5.0));
        var offset = 2.0 / numberOfSpherePoints;
        for (var k = 0; k < numberOfSpherePoints; ++k) {
            var y = k * offset - 1.0 + (offset / 2.0);
            var r = Math.sqrt(1.0 - y * y);
            var phi = k * inc;
            points[points.length] = [Math.cos(phi) * r, y, Math.sin(phi) * r];
        }
        return points;
    }
    /** Maximum accessible surface area observed for amino acids. Taken from: http://dx.doi.org/10.1371/journal.pone.0080635 */
    AccessibleSurfaceArea.MaxAsa = {
        'ALA': 121.0,
        'ARG': 265.0,
        'ASN': 187.0,
        'ASP': 187.0,
        'CYS': 148.0,
        'GLU': 214.0,
        'GLN': 214.0,
        'GLY': 97.0,
        'HIS': 216.0,
        'ILE': 195.0,
        'LEU': 191.0,
        'LYS': 230.0,
        'MET': 203.0,
        'PHE': 228.0,
        'PRO': 154.0,
        'SER': 143.0,
        'THR': 163.0,
        'TRP': 264.0,
        'TYR': 255.0,
        'VAL': 165.0
    };
    AccessibleSurfaceArea.DefaultMaxAsa = 121.0;
    AccessibleSurfaceArea.AccessibleSurfaceAreaComputationParams = {
        numberOfSpherePoints: param_definition_1.ParamDefinition.Numeric(92, {}, { description: 'number of sphere points to sample per atom: 92 (original paper), 960 (BioJava), 3000 (EPPIC) - see Shrake A, Rupley JA: Environment and exposure to solvent of protein atoms. Lysozyme and insulin. J Mol Biol 1973.' }),
        probeSize: param_definition_1.ParamDefinition.Numeric(1.4, {}, { description: 'corresponds to the size of a water molecule: 1.4 (original paper), 1.5 (occassionally used)' }),
        buriedRasaThreshold: param_definition_1.ParamDefinition.Numeric(0.16, { min: 0.0, max: 1.0 }, { description: 'below this cutoff of relative accessible surface area a residue will be considered buried - see: Rost B, Sander C: Conservation and prediction of solvent accessibility in protein families. Proteins 1994.' }),
        nonPolymer: param_definition_1.ParamDefinition.Boolean(false, { description: 'Include non-polymer atoms in computation.' })
    };
    var SolventAccessibility;
    (function (SolventAccessibility) {
        SolventAccessibility.is = mol_util_1.BitFlags.has;
        SolventAccessibility.create = mol_util_1.BitFlags.create;
    })(SolventAccessibility = AccessibleSurfaceArea.SolventAccessibility || (AccessibleSurfaceArea.SolventAccessibility = {}));
})(AccessibleSurfaceArea || (AccessibleSurfaceArea = {}));
exports.AccessibleSurfaceArea = AccessibleSurfaceArea;
