"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = require("../../../../mol-util/color");
exports.SaccharideColors = color_1.ColorMap({
    Blue: 0x0090bc,
    Green: 0x00a651,
    Yellow: 0xffd400,
    Orange: 0xf47920,
    Pink: 0xf69ea1,
    Purple: 0xa54399,
    LightBlue: 0x8fcce9,
    Brown: 0xa17a4d,
    Red: 0xed1c24,
    Secondary: 0xf1ece1
});
var SaccharideTypeNameMap = (_a = {},
    _a[0 /* Hexose */] = 'Hexose',
    _a[1 /* HexNAc */] = 'HexNAc',
    _a[2 /* Hexosamine */] = 'Hexosamine',
    _a[3 /* Hexuronate */] = 'Hexuronate',
    _a[4 /* Deoxyhexose */] = 'Deoxyhexose',
    _a[5 /* DeoxyhexNAc */] = 'DeoxyhexNAc',
    _a[6 /* DiDeoxyhexose */] = 'Di-deoxyhexose',
    _a[7 /* Pentose */] = 'Pentose',
    _a[8 /* Deoxynonulosonate */] = 'Deoxynonulosonate',
    _a[9 /* DiDeoxynonulosonate */] = 'Di-deoxynonulosonate',
    _a[10 /* Unknown */] = 'Unknown',
    _a[11 /* Assigned */] = 'Assigned',
    _a);
function getSaccharideName(type) {
    return SaccharideTypeNameMap[type];
}
exports.getSaccharideName = getSaccharideName;
var SaccharideTypeShapeMap = (_b = {},
    _b[0 /* Hexose */] = 0 /* FilledSphere */,
    _b[1 /* HexNAc */] = 1 /* FilledCube */,
    _b[2 /* Hexosamine */] = 2 /* CrossedCube */,
    _b[3 /* Hexuronate */] = 3 /* DividedDiamond */,
    _b[4 /* Deoxyhexose */] = 4 /* FilledCone */,
    _b[5 /* DeoxyhexNAc */] = 5 /* DevidedCone */,
    _b[6 /* DiDeoxyhexose */] = 6 /* FlatBox */,
    _b[7 /* Pentose */] = 7 /* FilledStar */,
    _b[8 /* Deoxynonulosonate */] = 8 /* FilledDiamond */,
    _b[9 /* DiDeoxynonulosonate */] = 9 /* FlatDiamond */,
    _b[10 /* Unknown */] = 10 /* FlatHexagon */,
    _b[11 /* Assigned */] = 11 /* Pentagon */,
    _b);
function getSaccharideShape(type) {
    return SaccharideTypeShapeMap[type];
}
exports.getSaccharideShape = getSaccharideShape;
exports.UnknownSaccharideComponent = {
    abbr: 'Unk',
    name: 'Unknown',
    color: exports.SaccharideColors.Secondary,
    type: 10 /* Unknown */
};
var Monosaccharides = [
    { abbr: 'Glc', name: 'Glucose', color: exports.SaccharideColors.Blue, type: 0 /* Hexose */ },
    { abbr: 'Man', name: 'Mannose', color: exports.SaccharideColors.Green, type: 0 /* Hexose */ },
    { abbr: 'Gal', name: 'Galactose', color: exports.SaccharideColors.Yellow, type: 0 /* Hexose */ },
    { abbr: 'Gul', name: 'Gulose', color: exports.SaccharideColors.Orange, type: 0 /* Hexose */ },
    { abbr: 'Alt', name: 'Altrose', color: exports.SaccharideColors.Pink, type: 0 /* Hexose */ },
    { abbr: 'All', name: 'Allose', color: exports.SaccharideColors.Purple, type: 0 /* Hexose */ },
    { abbr: 'Tal', name: 'Talose', color: exports.SaccharideColors.LightBlue, type: 0 /* Hexose */ },
    { abbr: 'Ido', name: 'Idose', color: exports.SaccharideColors.Brown, type: 0 /* Hexose */ },
    { abbr: 'GlcNAc', name: 'N-Acetyl Glucosamine', color: exports.SaccharideColors.Blue, type: 1 /* HexNAc */ },
    { abbr: 'ManNAc', name: 'N-Acetyl Mannosamine', color: exports.SaccharideColors.Green, type: 1 /* HexNAc */ },
    { abbr: 'GalNAc', name: 'N-Acetyl Galactosamine', color: exports.SaccharideColors.Yellow, type: 1 /* HexNAc */ },
    { abbr: 'GulNAc', name: 'N-Acetyl Gulosamine', color: exports.SaccharideColors.Orange, type: 1 /* HexNAc */ },
    { abbr: 'AltNAc', name: 'N-Acetyl Altrosamine', color: exports.SaccharideColors.Pink, type: 1 /* HexNAc */ },
    { abbr: 'AllNAc', name: 'N-Acetyl Allosamine', color: exports.SaccharideColors.Purple, type: 1 /* HexNAc */ },
    { abbr: 'TalNAc', name: 'N-Acetyl Talosamine', color: exports.SaccharideColors.LightBlue, type: 1 /* HexNAc */ },
    { abbr: 'IdoNAc', name: 'N-Acetyl Idosamine', color: exports.SaccharideColors.Brown, type: 1 /* HexNAc */ },
    { abbr: 'GlcN', name: 'Glucosamine', color: exports.SaccharideColors.Blue, type: 2 /* Hexosamine */ },
    { abbr: 'ManN', name: 'Mannosamine', color: exports.SaccharideColors.Green, type: 2 /* Hexosamine */ },
    { abbr: 'GalN', name: 'Galactosamine', color: exports.SaccharideColors.Yellow, type: 2 /* Hexosamine */ },
    { abbr: 'GulN', name: 'Gulosamine', color: exports.SaccharideColors.Orange, type: 2 /* Hexosamine */ },
    { abbr: 'AltN', name: 'Altrosamine', color: exports.SaccharideColors.Pink, type: 2 /* Hexosamine */ },
    { abbr: 'AllN', name: 'Allosamine', color: exports.SaccharideColors.Purple, type: 2 /* Hexosamine */ },
    { abbr: 'TalN', name: 'Talosamine', color: exports.SaccharideColors.LightBlue, type: 2 /* Hexosamine */ },
    { abbr: 'IdoN', name: 'Idosamine', color: exports.SaccharideColors.Brown, type: 2 /* Hexosamine */ },
    { abbr: 'GlcA', name: 'Glucuronic Acid', color: exports.SaccharideColors.Blue, type: 3 /* Hexuronate */ },
    { abbr: 'ManA', name: 'Mannuronic Acid', color: exports.SaccharideColors.Green, type: 3 /* Hexuronate */ },
    { abbr: 'GalA', name: 'Galacturonic Acid', color: exports.SaccharideColors.Yellow, type: 3 /* Hexuronate */ },
    { abbr: 'GulA', name: 'Guluronic Acid', color: exports.SaccharideColors.Orange, type: 3 /* Hexuronate */ },
    { abbr: 'AltA', name: 'Altruronic Acid', color: exports.SaccharideColors.Pink, type: 3 /* Hexuronate */ },
    { abbr: 'AllA', name: 'Alluronic Acid', color: exports.SaccharideColors.Purple, type: 3 /* Hexuronate */ },
    { abbr: 'TalA', name: 'Taluronic Acid', color: exports.SaccharideColors.LightBlue, type: 3 /* Hexuronate */ },
    { abbr: 'IdoA', name: 'Iduronic Acid', color: exports.SaccharideColors.Brown, type: 3 /* Hexuronate */ },
    { abbr: 'Qui', name: 'Quinovose', color: exports.SaccharideColors.Blue, type: 4 /* Deoxyhexose */ },
    { abbr: 'Rha', name: 'Rhamnose', color: exports.SaccharideColors.Green, type: 4 /* Deoxyhexose */ },
    { abbr: '6dGul', name: '6-Deoxy Gulose', color: exports.SaccharideColors.Orange, type: 4 /* Deoxyhexose */ },
    { abbr: '6dAlt', name: '6-Deoxy Altrose', color: exports.SaccharideColors.Pink, type: 4 /* Deoxyhexose */ },
    { abbr: '6dTal', name: '6-Deoxy Talose', color: exports.SaccharideColors.LightBlue, type: 4 /* Deoxyhexose */ },
    { abbr: 'Fuc', name: 'Fucose', color: exports.SaccharideColors.Red, type: 4 /* Deoxyhexose */ },
    { abbr: 'QuiNAc', name: 'N-Acetyl Quinovosamine', color: exports.SaccharideColors.Blue, type: 5 /* DeoxyhexNAc */ },
    { abbr: 'RhaNAc', name: 'N-Acetyl Rhamnosamine', color: exports.SaccharideColors.Green, type: 5 /* DeoxyhexNAc */ },
    { abbr: '6dAltNAc', name: 'N-Acetyl 6-Deoxy Altrosamine', color: exports.SaccharideColors.Pink, type: 5 /* DeoxyhexNAc */ },
    { abbr: '6dTalNAc', name: 'N-Acetyl 6-Deoxy Talosamine', color: exports.SaccharideColors.LightBlue, type: 5 /* DeoxyhexNAc */ },
    { abbr: 'FucNAc', name: 'N-Acetyl Fucosamine', color: exports.SaccharideColors.Red, type: 5 /* DeoxyhexNAc */ },
    { abbr: 'Oli', name: 'Olivose', color: exports.SaccharideColors.Blue, type: 6 /* DiDeoxyhexose */ },
    { abbr: 'Tyv', name: 'Tyvelose', color: exports.SaccharideColors.Green, type: 6 /* DiDeoxyhexose */ },
    { abbr: 'Abe', name: 'Abequose', color: exports.SaccharideColors.Orange, type: 6 /* DiDeoxyhexose */ },
    { abbr: 'Par', name: 'Paratose', color: exports.SaccharideColors.Pink, type: 6 /* DiDeoxyhexose */ },
    { abbr: 'Dig', name: 'Digitoxose', color: exports.SaccharideColors.Purple, type: 6 /* DiDeoxyhexose */ },
    { abbr: 'Col', name: 'Colitose', color: exports.SaccharideColors.LightBlue, type: 6 /* DiDeoxyhexose */ },
    { abbr: 'Ara', name: 'Arabinose', color: exports.SaccharideColors.Green, type: 7 /* Pentose */ },
    { abbr: 'Lyx', name: 'Lyxose', color: exports.SaccharideColors.Yellow, type: 7 /* Pentose */ },
    { abbr: 'Xyl', name: 'Xylose', color: exports.SaccharideColors.Orange, type: 7 /* Pentose */ },
    { abbr: 'Rib', name: 'Ribose', color: exports.SaccharideColors.Pink, type: 7 /* Pentose */ },
    { abbr: 'Kdn', name: 'Keto-Deoxy Nonulonic Acid', color: exports.SaccharideColors.Green, type: 8 /* Deoxynonulosonate */ },
    { abbr: 'Neu5Ac', name: 'N-Acetyl Neuraminic Acid', color: exports.SaccharideColors.Purple, type: 8 /* Deoxynonulosonate */ },
    { abbr: 'Neu5Gc', name: 'N-Glycolyl Neuraminic Acid', color: exports.SaccharideColors.LightBlue, type: 8 /* Deoxynonulosonate */ },
    { abbr: 'Neu', name: 'Neuraminic Acid', color: exports.SaccharideColors.Brown, type: 8 /* Deoxynonulosonate */ },
    { abbr: 'Sia', name: 'Sialic acid', color: exports.SaccharideColors.Red, type: 8 /* Deoxynonulosonate */ },
    { abbr: 'Pse', name: 'Pseudaminic Acid', color: exports.SaccharideColors.Green, type: 9 /* DiDeoxynonulosonate */ },
    { abbr: 'Leg', name: 'Legionaminic Acid', color: exports.SaccharideColors.Yellow, type: 9 /* DiDeoxynonulosonate */ },
    { abbr: 'Aci', name: 'Acinetaminic Acid', color: exports.SaccharideColors.Pink, type: 9 /* DiDeoxynonulosonate */ },
    { abbr: '4eLeg', name: '4-Epilegionaminic Acid', color: exports.SaccharideColors.LightBlue, type: 9 /* DiDeoxynonulosonate */ },
    { abbr: 'Bac', name: 'Bacillosamine', color: exports.SaccharideColors.Blue, type: 10 /* Unknown */ },
    { abbr: 'LDManHep', name: 'L-Glycero-D-Manno Heptose', color: exports.SaccharideColors.Green, type: 10 /* Unknown */ },
    { abbr: 'Kdo', name: 'Keto-Deoxy Octulonic Acid', color: exports.SaccharideColors.Yellow, type: 10 /* Unknown */ },
    { abbr: 'Dha', name: '3-Deoxy Lyxo-Heptulosaric Acid', color: exports.SaccharideColors.Orange, type: 10 /* Unknown */ },
    { abbr: 'DDManHep', name: 'D-Glycero-D-Manno-Heptose', color: exports.SaccharideColors.Pink, type: 10 /* Unknown */ },
    { abbr: 'MurNAc', name: 'N-Acetyl Muramic Acid', color: exports.SaccharideColors.Purple, type: 10 /* Unknown */ },
    { abbr: 'MurNGc', name: 'N-Glycolyl Muramic Acid', color: exports.SaccharideColors.LightBlue, type: 10 /* Unknown */ },
    { abbr: 'Mur', name: 'Muramic Acid', color: exports.SaccharideColors.Brown, type: 10 /* Unknown */ },
    { abbr: 'Api', name: 'Apicose', color: exports.SaccharideColors.Green, type: 11 /* Assigned */ },
    { abbr: 'Fru', name: 'Fructose', color: exports.SaccharideColors.Green, type: 11 /* Assigned */ },
    { abbr: 'Tag', name: 'Tagatose', color: exports.SaccharideColors.Yellow, type: 11 /* Assigned */ },
    { abbr: 'Sor', name: 'Sorbose', color: exports.SaccharideColors.Orange, type: 11 /* Assigned */ },
    { abbr: 'Psi', name: 'Psicose', color: exports.SaccharideColors.Pink, type: 11 /* Assigned */ },
];
exports.SaccharidesSnfgMap = (function () {
    var map = new Map();
    for (var i = 0, il = Monosaccharides.length; i < il; ++i) {
        var saccharide = Monosaccharides[i];
        map.set(saccharide.abbr, saccharide);
    }
    return map;
})();
exports.MonosaccharidesColorTable = [
    ['Glc-family', exports.SaccharideColors.Blue],
    ['Man-family', exports.SaccharideColors.Green],
    ['Gal-family', exports.SaccharideColors.Yellow],
    ['Gul-family', exports.SaccharideColors.Orange],
    ['Alt-family', exports.SaccharideColors.Pink],
    ['All-family', exports.SaccharideColors.Purple],
    ['Tal-family', exports.SaccharideColors.LightBlue],
    ['Ido-family', exports.SaccharideColors.Brown],
    ['Fuc-family', exports.SaccharideColors.Red],
    ['Generic/Unknown/Secondary', exports.SaccharideColors.Secondary],
];
var CommonSaccharideNames = {
    // Hexose
    Glc: [
        'GLC', 'BGC',
        'BOG',
        'TRE',
        'MLR',
    ],
    Man: ['MAN', 'BMA'],
    Gal: [
        'GAL', 'GLA',
        'GXL' // via PubChem
    ],
    Gul: ['GUP', 'GL0'],
    Alt: ['ALT'],
    All: ['ALL', 'AFD'],
    Tal: ['TAL'],
    Ido: ['4N2'],
    // HexNAc
    GlcNAc: ['NAG', 'NDG'],
    ManNAc: ['NGA', 'A2G'],
    GulNAc: [],
    AltNAc: [],
    AllNAc: ['NAA'],
    TalNAc: [],
    IdoNAc: ['HSQ'],
    // Hexosamine
    GlcN: [
        'GCS', 'PA1',
        'IDU', 'SGN', 'SUS',
    ],
    ManN: ['95Z'],
    GalN: ['X6X', '1GN'],
    GulN: [],
    AltN: [],
    AllN: [],
    TalN: [],
    IdoN: [],
    // Hexuronate
    GlcA: ['GCU', 'BDP'],
    ManA: ['MAV', 'BEM'],
    GalA: ['ADA', 'GTR'],
    GulA: ['LGU'],
    AltA: [],
    AllA: [],
    TalA: ['X0X', 'X1X'],
    IdoA: [
        'IDR',
        'IDS',
    ],
    // Deoxyhexose
    Qui: ['G6D'],
    Rha: ['RAM', 'RM4'],
    '6dGul': [],
    '6dAlt': [],
    '6dTal': [],
    Fuc: ['FUC', 'FUL'],
    // DeoxyhexNAc
    QuiNAc: [],
    RhaNAc: [],
    '6dAltNAc': [],
    '6dTalNAc': [],
    FucNAc: [],
    // Di-deoxyhexose
    Oli: ['DDA'],
    Tyv: ['TYV'],
    Abe: ['ABE'],
    Par: ['PZU'],
    Dig: [],
    Col: [],
    // Pentose
    Ara: ['ARA', 'ARB'],
    Lyx: ['LDY'],
    Xyl: ['XYS', 'XYP'],
    Rib: ['RIP', '0MK'],
    // Deoxynonulosonate
    Kdn: ['KDN', 'KDM'],
    Neu5Ac: ['SIA', 'SLB'],
    Neu5Gc: ['NGC', 'NGE'],
    Neu: [],
    Sia: [],
    // Di-deoxynonulosonate
    Pse: ['6PZ'],
    Leg: [],
    Aci: [],
    '4eLeg': [],
    // Unknown
    Bac: ['B6D'],
    LDManHep: ['GMH'],
    Kdo: ['KDO'],
    Dha: [],
    DDManHep: [],
    MurNAc: ['AMU'],
    MurNGc: [],
    Mur: ['MUR'],
    // Assigned
    Api: ['XXM'],
    Fru: ['BDF'],
    Tag: ['T6T'],
    Sor: ['SOE'],
    Psi: [],
};
var UnknownSaccharideNames = [
    'NGZ',
    'LAT',
];
exports.SaccharideCompIdMap = (function () {
    var map = new Map();
    for (var i = 0, il = Monosaccharides.length; i < il; ++i) {
        var saccharide = Monosaccharides[i];
        var names = CommonSaccharideNames[saccharide.abbr];
        if (names) {
            for (var j = 0, jl = names.length; j < jl; ++j) {
                map.set(names[j], saccharide);
            }
        }
    }
    for (var i = 0, il = UnknownSaccharideNames.length; i < il; ++i) {
        map.set(UnknownSaccharideNames[i], exports.UnknownSaccharideComponent);
    }
    return map;
})();
