/**
 * Copyright (c) 2017-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { OrderedSet, SortedArray } from '../../../mol-data/int';
import { ElementIndex } from '../model';
import { ChainIndex, ResidueIndex } from '../model/indexing';
import Structure from './structure';
import Unit from './unit';
import { Boundary } from './util/boundary';
import Expression from '../../../mol-script/language/expression';
interface StructureElement<U = Unit> {
    readonly kind: 'element-location';
    unit: U;
    /** Index into element (atomic/coarse) properties of unit.model */
    element: ElementIndex;
}
declare namespace StructureElement {
    function create(unit?: Unit, element?: ElementIndex): StructureElement;
    function set(a: StructureElement, unit?: Unit, element?: ElementIndex): StructureElement;
    type Set = SortedArray<ElementIndex>;
    /** Index into Unit.elements */
    type UnitIndex = {
        readonly '@type': 'structure-element-index';
    } & number;
    interface Property<T> {
        (location: StructureElement): T;
    }
    interface Predicate extends Property<boolean> {
    }
    function property<T>(p: Property<T>): Property<T>;
    function atomicProperty<T>(p: (location: StructureElement<Unit.Atomic>) => T): Property<void | T>;
    function coarseProperty<T>(p: (location: StructureElement<Unit.Spheres | Unit.Gaussians>) => T): Property<void | T>;
    /** Represents multiple element index locations */
    interface Loci {
        readonly kind: 'element-loci';
        readonly structure: Structure;
        /** Access i-th element as unit.elements[indices[i]] */
        readonly elements: ReadonlyArray<{
            unit: Unit;
            /**
             * Indices into the unit.elements array.
             * Can use OrderedSet.forEach to iterate (or OrderedSet.size + OrderedSet.getAt)
             */
            indices: OrderedSet<UnitIndex>;
        }>;
    }
    function Loci(structure: Structure, elements: ArrayLike<{
        unit: Unit;
        indices: OrderedSet<UnitIndex>;
    }>): Loci;
    function isLoci(x: any): x is Loci;
    function areLociEqual(a: Loci, b: Loci): boolean;
    function isLocation(x: any): x is StructureElement;
    function residueIndex(e: StructureElement): ResidueIndex;
    function chainIndex(e: StructureElement): ChainIndex;
    function entityIndex(l: StructureElement): import("../model").EntityIndex;
    namespace Loci {
        function size(loci: Loci): number;
        function all(structure: Structure): Loci;
        function remap(loci: Loci, structure: Structure): Loci;
        function union(xs: Loci, ys: Loci): Loci;
        function subtract(xs: Loci, ys: Loci): Loci;
        function areIntersecting(xs: Loci, ys: Loci): boolean;
        function extendToWholeResidues(loci: Loci): Loci;
        function extendToWholeChains(loci: Loci): Loci;
        function getBoundary(loci: Loci): Boundary;
        function toScriptExpression(loci: Loci): Expression;
    }
}
export default StructureElement;
