"use strict";
/**
 * Copyright (c) 2017-2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var element_1 = require("./element");
var unit_1 = require("./unit");
var atomic_1 = require("../model/properties/atomic");
var constant = {
    true: element_1.default.property(function (l) { return true; }),
    false: element_1.default.property(function (l) { return false; }),
    zero: element_1.default.property(function (l) { return 0; })
};
function notAtomic() {
    throw 'Property only available for atomic models.';
}
function notCoarse(kind) {
    if (!!kind)
        throw "Property only available for coarse models (" + kind + ").";
    throw "Property only available for coarse models.";
}
// TODO: remove the type checks?
var atom = {
    key: element_1.default.property(function (l) { return l.element; }),
    // Conformation
    x: element_1.default.property(function (l) { return l.unit.conformation.x(l.element); }),
    y: element_1.default.property(function (l) { return l.unit.conformation.y(l.element); }),
    z: element_1.default.property(function (l) { return l.unit.conformation.z(l.element); }),
    id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicConformation.atomId.value(l.element); }),
    occupancy: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicConformation.occupancy.value(l.element); }),
    B_iso_or_equiv: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicConformation.B_iso_or_equiv.value(l.element); }),
    sourceIndex: element_1.default.property(function (l) { return unit_1.default.isAtomic(l.unit)
        ? l.unit.model.atomicHierarchy.atoms.sourceIndex.value(l.element)
        // TODO: when implemented, this should map to the source index.
        : l.element; }),
    // Hierarchy
    type_symbol: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.atoms.type_symbol.value(l.element); }),
    label_atom_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.atoms.label_atom_id.value(l.element); }),
    auth_atom_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.atoms.auth_atom_id.value(l.element); }),
    label_alt_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.atoms.label_alt_id.value(l.element); }),
    pdbx_formal_charge: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.atoms.pdbx_formal_charge.value(l.element); }),
    // Derived
    vdw_radius: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : atomic_1.VdwRadius(l.unit.model.atomicHierarchy.atoms.type_symbol.value(l.element)); }),
};
var residue = {
    key: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.residueIndex[l.element]; }),
    group_PDB: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.residues.group_PDB.value(l.unit.residueIndex[l.element]); }),
    label_comp_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.residues.label_comp_id.value(l.unit.residueIndex[l.element]); }),
    auth_comp_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.residues.auth_comp_id.value(l.unit.residueIndex[l.element]); }),
    label_seq_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.residues.label_seq_id.value(l.unit.residueIndex[l.element]); }),
    auth_seq_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.residues.auth_seq_id.value(l.unit.residueIndex[l.element]); }),
    pdbx_PDB_ins_code: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.residues.pdbx_PDB_ins_code.value(l.unit.residueIndex[l.element]); }),
    // Properties
    secondary_structure_type: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.properties.secondaryStructure.type[l.unit.residueIndex[l.element]]; }),
    secondary_structure_key: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.properties.secondaryStructure.key[l.unit.residueIndex[l.element]]; }),
};
var chain = {
    key: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.chainIndex[l.element]; }),
    label_asym_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.chains.label_asym_id.value(l.unit.chainIndex[l.element]); }),
    auth_asym_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.chains.auth_asym_id.value(l.unit.chainIndex[l.element]); }),
    label_entity_id: element_1.default.property(function (l) { return !unit_1.default.isAtomic(l.unit) ? notAtomic() : l.unit.model.atomicHierarchy.chains.label_entity_id.value(l.unit.chainIndex[l.element]); })
};
var coarse = {
    key: atom.key,
    entityKey: element_1.default.property(function (l) { return !unit_1.default.isCoarse(l.unit) ? notCoarse() : l.unit.coarseElements.entityKey[l.element]; }),
    x: atom.x,
    y: atom.y,
    z: atom.z,
    asym_id: element_1.default.property(function (l) { return !unit_1.default.isCoarse(l.unit) ? notCoarse() : l.unit.coarseElements.asym_id.value(l.element); }),
    seq_id_begin: element_1.default.property(function (l) { return !unit_1.default.isCoarse(l.unit) ? notCoarse() : l.unit.coarseElements.seq_id_begin.value(l.element); }),
    seq_id_end: element_1.default.property(function (l) { return !unit_1.default.isCoarse(l.unit) ? notCoarse() : l.unit.coarseElements.seq_id_end.value(l.element); }),
    sphere_radius: element_1.default.property(function (l) { return !unit_1.default.isSpheres(l.unit) ? notCoarse('spheres') : l.unit.coarseConformation.radius[l.element]; }),
    sphere_rmsf: element_1.default.property(function (l) { return !unit_1.default.isSpheres(l.unit) ? notCoarse('spheres') : l.unit.coarseConformation.rmsf[l.element]; }),
    gaussian_weight: element_1.default.property(function (l) { return !unit_1.default.isGaussians(l.unit) ? notCoarse('gaussians') : l.unit.coarseConformation.weight[l.element]; }),
    gaussian_covariance_matrix: element_1.default.property(function (l) { return !unit_1.default.isGaussians(l.unit) ? notCoarse('gaussians') : l.unit.coarseConformation.covariance_matrix[l.element]; })
};
var eK = element_1.default.entityIndex;
var entity = {
    key: eK,
    id: element_1.default.property(function (l) { return l.unit.model.entities.data.id.value(eK(l)); }),
    type: element_1.default.property(function (l) { return l.unit.model.entities.data.type.value(eK(l)); }),
    src_method: element_1.default.property(function (l) { return l.unit.model.entities.data.src_method.value(eK(l)); }),
    pdbx_description: element_1.default.property(function (l) { return l.unit.model.entities.data.pdbx_description.value(eK(l)); }),
    formula_weight: element_1.default.property(function (l) { return l.unit.model.entities.data.formula_weight.value(eK(l)); }),
    pdbx_number_of_molecules: element_1.default.property(function (l) { return l.unit.model.entities.data.pdbx_number_of_molecules.value(eK(l)); }),
    details: element_1.default.property(function (l) { return l.unit.model.entities.data.details.value(eK(l)); }),
    pdbx_mutation: element_1.default.property(function (l) { return l.unit.model.entities.data.pdbx_mutation.value(eK(l)); }),
    pdbx_fragment: element_1.default.property(function (l) { return l.unit.model.entities.data.pdbx_fragment.value(eK(l)); }),
    pdbx_ec: element_1.default.property(function (l) { return l.unit.model.entities.data.pdbx_ec.value(eK(l)); })
};
var unit = {
    id: element_1.default.property(function (l) { return l.unit.id; }),
    operator_name: element_1.default.property(function (l) { return l.unit.conformation.operator.name; }),
    hkl: element_1.default.property(function (l) { return l.unit.conformation.operator.hkl; }),
    spgrOp: element_1.default.property(function (l) { return l.unit.conformation.operator.spgrOp; }),
    model_num: element_1.default.property(function (l) { return l.unit.model.modelNum; }),
    pdbx_struct_assembly_id: element_1.default.property(function (l) { return l.unit.conformation.operator.assembly.id; }),
    pdbx_struct_oper_list_ids: element_1.default.property(function (l) { return l.unit.conformation.operator.assembly.operList; }),
    struct_ncs_oper_id: element_1.default.property(function (l) { return l.unit.conformation.operator.ncsId; }),
};
var StructureProperties = {
    constant: constant,
    atom: atom,
    residue: residue,
    chain: chain,
    entity: entity,
    unit: unit,
    coarse: coarse
};
exports.default = StructureProperties;
