"use strict";
/**
 * Copyright (c) 2017-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var int_1 = require("../../../mol-data/int");
var util_1 = require("../../../mol-data/util");
var geometry_1 = require("../../../mol-math/geometry");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var mol_task_1 = require("../../../mol-task");
var model_1 = require("../model");
var query_1 = require("../query");
var structure_1 = require("./structure");
var unit_1 = require("./unit");
var StructureSymmetry;
(function (StructureSymmetry) {
    function buildAssembly(structure, asmName) {
        var _this = this;
        return mol_task_1.Task.create('Build Assembly', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var models, assembly, assembler, queryCtx, _i, _a, g, selection, units, _b, _c, oper, _d, units_1, unit;
            return __generator(this, function (_e) {
                models = structure.models;
                if (models.length !== 1)
                    throw new Error('Can only build assemblies from structures based on 1 model.');
                assembly = model_1.ModelSymmetry.findAssembly(models[0], asmName);
                if (!assembly)
                    throw new Error("Assembly '" + asmName + "' is not defined.");
                assembler = structure_1.default.Builder(void 0, geometry_1.SymmetryOperator.create(assembly.id, linear_algebra_1.Mat4.identity(), { id: assembly.id, operList: [] }));
                queryCtx = new query_1.QueryContext(structure);
                for (_i = 0, _a = assembly.operatorGroups; _i < _a.length; _i++) {
                    g = _a[_i];
                    selection = g.selector(queryCtx);
                    if (query_1.StructureSelection.structureCount(selection) === 0) {
                        continue;
                    }
                    units = query_1.StructureSelection.unionStructure(selection).units;
                    for (_b = 0, _c = g.operators; _b < _c.length; _b++) {
                        oper = _c[_b];
                        for (_d = 0, units_1 = units; _d < units_1.length; _d++) {
                            unit = units_1[_d];
                            assembler.addWithOperator(unit, oper);
                        }
                    }
                }
                return [2 /*return*/, assembler.getStructure()];
            });
        }); });
    }
    StructureSymmetry.buildAssembly = buildAssembly;
    function builderSymmetryMates(structure, radius) {
        return mol_task_1.Task.create('Find Symmetry Mates', function (ctx) { return findMatesRadius(ctx, structure, radius); });
    }
    StructureSymmetry.builderSymmetryMates = builderSymmetryMates;
    function buildSymmetryRange(structure, ijkMin, ijkMax) {
        return mol_task_1.Task.create('Build Symmetry', function (ctx) { return findSymmetryRange(ctx, structure, ijkMin, ijkMax); });
    }
    StructureSymmetry.buildSymmetryRange = buildSymmetryRange;
    /** Builds NCS structure, returns the original if NCS operators are not present. */
    function buildNcs(structure) {
        return mol_task_1.Task.create('Build NCS', function (ctx) { return _buildNCS(ctx, structure); });
    }
    StructureSymmetry.buildNcs = buildNcs;
    function areUnitsEquivalent(a, b) {
        return a.invariantId === b.invariantId && a.model.id === b.model.id && int_1.SortedArray.areEqual(a.elements, b.elements);
    }
    StructureSymmetry.areUnitsEquivalent = areUnitsEquivalent;
    function UnitEquivalenceBuilder() {
        return util_1.EquivalenceClasses(unit_1.default.hashUnit, areUnitsEquivalent);
    }
    StructureSymmetry.UnitEquivalenceBuilder = UnitEquivalenceBuilder;
    function computeTransformGroups(s) {
        var groups = UnitEquivalenceBuilder();
        for (var _i = 0, _a = s.units; _i < _a.length; _i++) {
            var u = _a[_i];
            groups.add(u.id, u);
        }
        var ret = [];
        for (var _b = 0, _c = groups.groups; _b < _c.length; _b++) {
            var eqUnits = _c[_b];
            ret.push(unit_1.default.SymmetryGroup(eqUnits.map(function (id) { return s.unitMap.get(id); })));
        }
        return ret;
    }
    StructureSymmetry.computeTransformGroups = computeTransformGroups;
    /** Checks if transform groups are equal up to their unit's transformations */
    function areTransformGroupsEquivalent(a, b) {
        if (a.length !== b.length)
            return false;
        for (var i = 0, il = a.length; i < il; ++i) {
            var au = a[i].units, bu = b[i].units;
            if (au.length !== bu.length)
                return false;
            if (a[i].hashCode !== b[i].hashCode)
                return false;
            for (var j = 0, _j = au.length; j < _j; j++) {
                if (au[j].conformation !== bu[j].conformation)
                    return false;
            }
        }
        return true;
    }
    StructureSymmetry.areTransformGroupsEquivalent = areTransformGroupsEquivalent;
})(StructureSymmetry || (StructureSymmetry = {}));
function getOperators(symmetry, ijkMin, ijkMax) {
    var operators = [];
    var spacegroup = symmetry.spacegroup;
    if (ijkMin[0] <= 0 && ijkMax[0] >= 0 &&
        ijkMin[1] <= 0 && ijkMax[1] >= 0 &&
        ijkMin[2] <= 0 && ijkMax[2] >= 0) {
        operators[0] = geometry_1.Spacegroup.getSymmetryOperator(spacegroup, 0, 0, 0, 0);
    }
    var ncsOperators = symmetry.ncsOperators;
    var ncsCount = (ncsOperators && ncsOperators.length) || 0;
    for (var op = 0; op < spacegroup.operators.length; op++) {
        for (var i = ijkMin[0]; i <= ijkMax[0]; i++) {
            for (var j = ijkMin[1]; j <= ijkMax[1]; j++) {
                for (var k = ijkMin[2]; k <= ijkMax[2]; k++) {
                    // we have added identity as the 1st operator.
                    if (op === 0 && i === 0 && j === 0 && k === 0)
                        continue;
                    var symOp = geometry_1.Spacegroup.getSymmetryOperator(spacegroup, op, i, j, k);
                    if (ncsCount) {
                        for (var u = 0; u < ncsCount; ++u) {
                            var ncsOp = ncsOperators[u];
                            var matrix = linear_algebra_1.Mat4.mul(linear_algebra_1.Mat4.zero(), symOp.matrix, ncsOp.matrix);
                            var operator = geometry_1.SymmetryOperator.create(symOp.name + " " + ncsOp.name, matrix, symOp.assembly, ncsOp.ncsId, symOp.hkl, symOp.spgrOp);
                            operators[operators.length] = operator;
                        }
                    }
                    else {
                        operators[operators.length] = symOp;
                    }
                }
            }
        }
    }
    return operators;
}
function getOperatorsCached333(symmetry) {
    if (typeof symmetry._operators_333 !== 'undefined')
        return symmetry._operators_333;
    symmetry._operators_333 = getOperators(symmetry, linear_algebra_1.Vec3.create(-3, -3, -3), linear_algebra_1.Vec3.create(3, 3, 3));
    return symmetry._operators_333;
}
function assembleOperators(structure, operators) {
    var assembler = structure_1.default.Builder(void 0, void 0);
    var units = structure.units;
    for (var _i = 0, operators_1 = operators; _i < operators_1.length; _i++) {
        var oper = operators_1[_i];
        for (var _a = 0, units_2 = units; _a < units_2.length; _a++) {
            var unit = units_2[_a];
            assembler.addWithOperator(unit, oper);
        }
    }
    return assembler.getStructure();
}
function _buildNCS(ctx, structure) {
    return __awaiter(this, void 0, void 0, function () {
        var models, operators;
        return __generator(this, function (_a) {
            models = structure.models;
            if (models.length !== 1)
                throw new Error('Can only build NCS from structures based on 1 model.');
            operators = models[0].symmetry.ncsOperators;
            if (!operators || !operators.length)
                return [2 /*return*/, structure];
            return [2 /*return*/, assembleOperators(structure, operators)];
        });
    });
}
function findSymmetryRange(ctx, structure, ijkMin, ijkMax) {
    return __awaiter(this, void 0, void 0, function () {
        var models, spacegroup, operators;
        return __generator(this, function (_a) {
            models = structure.models;
            if (models.length !== 1)
                throw new Error('Can only build symmetries from structures based on 1 model.');
            spacegroup = models[0].symmetry.spacegroup;
            if (geometry_1.SpacegroupCell.isZero(spacegroup.cell))
                return [2 /*return*/, structure];
            operators = getOperators(models[0].symmetry, ijkMin, ijkMax);
            return [2 /*return*/, assembleOperators(structure, operators)];
        });
    });
}
function findMatesRadius(ctx, structure, radius) {
    return __awaiter(this, void 0, void 0, function () {
        var models, symmetry, spacegroup, operators, lookup, assembler, units, center, _i, operators_2, oper, _a, units_3, unit, boundingSphere, closeUnits, uI, _uI, closeUnit;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    models = structure.models;
                    if (models.length !== 1)
                        throw new Error('Can only build symmetries from structures based on 1 model.');
                    symmetry = models[0].symmetry;
                    spacegroup = symmetry.spacegroup;
                    if (geometry_1.SpacegroupCell.isZero(spacegroup.cell))
                        return [2 /*return*/, structure];
                    if (!ctx.shouldUpdate) return [3 /*break*/, 2];
                    return [4 /*yield*/, ctx.update('Initialing...')];
                case 1:
                    _b.sent();
                    _b.label = 2;
                case 2:
                    operators = getOperatorsCached333(symmetry);
                    lookup = structure.lookup3d;
                    assembler = structure_1.default.Builder(void 0, void 0);
                    units = structure.units;
                    center = linear_algebra_1.Vec3.zero();
                    _i = 0, operators_2 = operators;
                    _b.label = 3;
                case 3:
                    if (!(_i < operators_2.length)) return [3 /*break*/, 6];
                    oper = operators_2[_i];
                    for (_a = 0, units_3 = units; _a < units_3.length; _a++) {
                        unit = units_3[_a];
                        boundingSphere = unit.lookup3d.boundary.sphere;
                        linear_algebra_1.Vec3.transformMat4(center, boundingSphere.center, oper.matrix);
                        closeUnits = lookup.findUnitIndices(center[0], center[1], center[2], boundingSphere.radius + radius);
                        for (uI = 0, _uI = closeUnits.count; uI < _uI; uI++) {
                            closeUnit = units[closeUnits.indices[uI]];
                            if (!closeUnit.lookup3d.check(center[0], center[1], center[2], boundingSphere.radius + radius))
                                continue;
                            assembler.addWithOperator(unit, oper);
                        }
                    }
                    if (!ctx.shouldUpdate) return [3 /*break*/, 5];
                    return [4 /*yield*/, ctx.update('Building symmetry...')];
                case 4:
                    _b.sent();
                    _b.label = 5;
                case 5:
                    _i++;
                    return [3 /*break*/, 3];
                case 6: return [2 /*return*/, assembler.getStructure()];
            }
        });
    });
}
exports.default = StructureSymmetry;
