"use strict";
/**
 * Copyright (c) 2017-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../structure");
var types_1 = require("../../model/types");
var int_1 = require("../../../../mol-data/int");
__export(require("./links/data"));
__export(require("./links/intra-compute"));
__export(require("./links/inter-compute"));
var Link;
(function (Link) {
    function Location(aUnit, aIndex, bUnit, bIndex) {
        return { kind: 'link-location', aUnit: aUnit, aIndex: aIndex, bUnit: bUnit, bIndex: bIndex };
    }
    Link.Location = Location;
    function isLocation(x) {
        return !!x && x.kind === 'link-location';
    }
    Link.isLocation = isLocation;
    function areLocationsEqual(locA, locB) {
        return (locA.aIndex === locB.aIndex && locA.bIndex === locB.bIndex &&
            locA.aUnit.id === locB.aUnit.id && locA.bUnit.id === locB.bUnit.id);
    }
    Link.areLocationsEqual = areLocationsEqual;
    function Loci(structure, links) {
        return { kind: 'link-loci', structure: structure, links: links };
    }
    Link.Loci = Loci;
    function isLoci(x) {
        return !!x && x.kind === 'link-loci';
    }
    Link.isLoci = isLoci;
    function areLociEqual(a, b) {
        if (a.links.length !== b.links.length)
            return false;
        for (var i = 0, il = a.links.length; i < il; ++i) {
            if (!areLocationsEqual(a.links[i], b.links[i]))
                return false;
        }
        return true;
    }
    Link.areLociEqual = areLociEqual;
    function toStructureElementLoci(loci) {
        var elements = [];
        var map = new Map();
        for (var _i = 0, _a = loci.links; _i < _a.length; _i++) {
            var lociLink = _a[_i];
            var aIndex = lociLink.aIndex, aUnit = lociLink.aUnit, bIndex = lociLink.bIndex, bUnit = lociLink.bUnit;
            if (aUnit === bUnit) {
                if (map.has(aUnit.id))
                    map.get(aUnit.id).push(aIndex, bIndex);
                else
                    map.set(aUnit.id, [aIndex, bIndex]);
            }
            else {
                if (map.has(aUnit.id))
                    map.get(aUnit.id).push(aIndex);
                else
                    map.set(aUnit.id, [aIndex]);
                if (map.has(bUnit.id))
                    map.get(bUnit.id).push(bIndex);
                else
                    map.set(bUnit.id, [bIndex]);
            }
        }
        map.forEach(function (indices, id) {
            elements.push({
                unit: loci.structure.unitMap.get(id),
                indices: int_1.SortedArray.deduplicate(int_1.SortedArray.ofUnsortedArray(indices))
            });
        });
        return structure_1.StructureElement.Loci(loci.structure, elements);
    }
    Link.toStructureElementLoci = toStructureElementLoci;
    function getType(structure, link) {
        if (link.aUnit === link.bUnit) {
            var links = link.aUnit.links;
            var idx = links.getEdgeIndex(link.aIndex, link.bIndex);
            if (idx < 0)
                return types_1.LinkType.create(0 /* None */);
            return types_1.LinkType.create(links.edgeProps.flags[idx]);
        }
        else {
            var bond = structure.links.getBondFromLocation(link);
            if (bond)
                return types_1.LinkType.create(bond.flag);
            return types_1.LinkType.create(0 /* None */);
        }
    }
    Link.getType = getType;
    function getOrder(structure, link) {
        if (link.aUnit === link.bUnit) {
            var links = link.aUnit.links;
            var idx = links.getEdgeIndex(link.aIndex, link.bIndex);
            if (idx < 0)
                return 0;
            return links.edgeProps.order[idx];
        }
        else {
            var bond = structure.links.getBondFromLocation(link);
            if (bond)
                return bond.order;
            return 0;
        }
    }
    Link.getOrder = getOrder;
})(Link || (Link = {}));
exports.Link = Link;
