/**
 * Copyright (c) 2017-2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { LinkType } from '../../../model/types';
import { IntAdjacencyGraph } from '../../../../../mol-math/graph';
import Unit from '../../unit';
import StructureElement from '../../element';
import { Link } from '../links';
declare type IntraUnitLinks = IntAdjacencyGraph<{
    readonly order: ArrayLike<number>;
    readonly flags: ArrayLike<LinkType.Flag>;
}>;
declare namespace IntraUnitLinks {
    const Empty: IntraUnitLinks;
}
declare class InterUnitBonds {
    private map;
    /** Number of inter-unit bonds */
    readonly bondCount: number;
    /** Array of inter-unit bonds */
    readonly bonds: ReadonlyArray<InterUnitBonds.Bond>;
    private readonly bondKeyIndex;
    private readonly elementKeyIndex;
    /** Get an array of unit-pair-bonds that are linked to the given unit */
    getLinkedUnits(unit: Unit): ReadonlyArray<InterUnitBonds.UnitPairBonds>;
    /** Index into this.bonds */
    getBondIndex(indexA: StructureElement.UnitIndex, unitA: Unit, indexB: StructureElement.UnitIndex, unitB: Unit): number;
    /** Get inter-unit bond given a pair of indices and units */
    getBond(indexA: StructureElement.UnitIndex, unitA: Unit, indexB: StructureElement.UnitIndex, unitB: Unit): InterUnitBonds.Bond | undefined;
    /** Get inter-unit bond given a link-location */
    getBondFromLocation(l: Link.Location): InterUnitBonds.Bond | undefined;
    /** Indices into this.bonds */
    getBondIndices(index: StructureElement.UnitIndex, unit: Unit): ReadonlyArray<number>;
    constructor(map: Map<number, InterUnitBonds.UnitPairBonds[]>);
}
declare namespace InterUnitBonds {
    class UnitPairBonds {
        unitA: Unit.Atomic;
        unitB: Unit.Atomic;
        bondCount: number;
        linkedElementIndices: ReadonlyArray<StructureElement.UnitIndex>;
        private linkMap;
        hasBonds(indexA: StructureElement.UnitIndex): boolean;
        getBonds(indexA: StructureElement.UnitIndex): ReadonlyArray<InterUnitBonds.BondInfo>;
        readonly areUnitsOrdered: boolean;
        constructor(unitA: Unit.Atomic, unitB: Unit.Atomic, bondCount: number, linkedElementIndices: ReadonlyArray<StructureElement.UnitIndex>, linkMap: Map<number, BondInfo[]>);
    }
    interface BondInfo {
        /** indexInto */
        readonly indexB: StructureElement.UnitIndex;
        readonly order: number;
        readonly flag: LinkType.Flag;
    }
    interface Bond {
        readonly unitA: Unit.Atomic;
        readonly unitB: Unit.Atomic;
        readonly indexA: StructureElement.UnitIndex;
        readonly indexB: StructureElement.UnitIndex;
        readonly order: number;
        readonly flag: LinkType.Flag;
    }
    function getBondKey(indexA: StructureElement.UnitIndex, unitA: Unit, indexB: StructureElement.UnitIndex, unitB: Unit): string;
    function getElementKey(index: StructureElement.UnitIndex, unit: Unit): string;
}
export { IntraUnitLinks, InterUnitBonds };
