"use strict";
/**
 * Copyright (c) 2017-2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var graph_1 = require("../../../../../mol-math/graph");
var IntraUnitLinks;
(function (IntraUnitLinks) {
    IntraUnitLinks.Empty = graph_1.IntAdjacencyGraph.create([], [], [], 0, { flags: [], order: [] });
})(IntraUnitLinks || (IntraUnitLinks = {}));
exports.IntraUnitLinks = IntraUnitLinks;
var InterUnitBonds = /** @class */ (function () {
    function InterUnitBonds(map) {
        this.map = map;
        var count = 0;
        var bonds = [];
        var bondKeyIndex = new Map();
        var elementKeyIndex = new Map();
        this.map.forEach(function (pairBondsArray) {
            pairBondsArray.forEach(function (pairBonds) {
                count += pairBonds.bondCount;
                pairBonds.linkedElementIndices.forEach(function (indexA) {
                    pairBonds.getBonds(indexA).forEach(function (bondInfo) {
                        var unitA = pairBonds.unitA, unitB = pairBonds.unitB;
                        var bondKey = InterUnitBonds.getBondKey(indexA, unitA, bondInfo.indexB, unitB);
                        bondKeyIndex.set(bondKey, bonds.length);
                        var elementKey = InterUnitBonds.getElementKey(indexA, unitA);
                        var e = elementKeyIndex.get(elementKey);
                        if (e === undefined)
                            elementKeyIndex.set(elementKey, [bonds.length]);
                        else
                            e.push(bonds.length);
                        bonds.push(__assign({}, bondInfo, { indexA: indexA, unitA: unitA, unitB: unitB }));
                    });
                });
            });
        });
        this.bondCount = count;
        this.bonds = bonds;
        this.bondKeyIndex = bondKeyIndex;
        this.elementKeyIndex = elementKeyIndex;
    }
    /** Get an array of unit-pair-bonds that are linked to the given unit */
    InterUnitBonds.prototype.getLinkedUnits = function (unit) {
        if (!this.map.has(unit.id))
            return emptyArray;
        return this.map.get(unit.id);
    };
    /** Index into this.bonds */
    InterUnitBonds.prototype.getBondIndex = function (indexA, unitA, indexB, unitB) {
        var bondKey = InterUnitBonds.getBondKey(indexA, unitA, indexB, unitB);
        var index = this.bondKeyIndex.get(bondKey);
        return index !== undefined ? index : -1;
    };
    /** Get inter-unit bond given a pair of indices and units */
    InterUnitBonds.prototype.getBond = function (indexA, unitA, indexB, unitB) {
        var index = this.getBondIndex(indexA, unitA, indexB, unitB);
        return index !== -1 ? this.bonds[index] : undefined;
    };
    /** Get inter-unit bond given a link-location */
    InterUnitBonds.prototype.getBondFromLocation = function (l) {
        return this.getBond(l.aIndex, l.aUnit, l.bIndex, l.bUnit);
    };
    /** Indices into this.bonds */
    InterUnitBonds.prototype.getBondIndices = function (index, unit) {
        return this.elementKeyIndex.get(InterUnitBonds.getElementKey(index, unit)) || [];
    };
    return InterUnitBonds;
}());
exports.InterUnitBonds = InterUnitBonds;
(function (InterUnitBonds) {
    var UnitPairBonds = /** @class */ (function () {
        function UnitPairBonds(unitA, unitB, bondCount, linkedElementIndices, linkMap) {
            this.unitA = unitA;
            this.unitB = unitB;
            this.bondCount = bondCount;
            this.linkedElementIndices = linkedElementIndices;
            this.linkMap = linkMap;
        }
        UnitPairBonds.prototype.hasBonds = function (indexA) {
            return this.linkMap.has(indexA);
        };
        UnitPairBonds.prototype.getBonds = function (indexA) {
            if (!this.linkMap.has(indexA))
                return emptyArray;
            return this.linkMap.get(indexA);
        };
        Object.defineProperty(UnitPairBonds.prototype, "areUnitsOrdered", {
            get: function () {
                return this.unitA.id < this.unitB.id;
            },
            enumerable: true,
            configurable: true
        });
        return UnitPairBonds;
    }());
    InterUnitBonds.UnitPairBonds = UnitPairBonds;
    function getBondKey(indexA, unitA, indexB, unitB) {
        return indexA + "|" + unitA.id + "|" + indexB + "|" + unitB.id;
    }
    InterUnitBonds.getBondKey = getBondKey;
    function getElementKey(index, unit) {
        return index + "|" + unit.id;
    }
    InterUnitBonds.getElementKey = getElementKey;
})(InterUnitBonds || (InterUnitBonds = {}));
exports.InterUnitBonds = InterUnitBonds;
var emptyArray = [];
