"use strict";
/**
 * Copyright (c) 2017 Mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var unit_1 = require("../../unit");
var common_1 = require("./common");
var data_1 = require("./data");
var generic_1 = require("../../../../../mol-data/generic");
var int_1 = require("../../../../../mol-data/int");
var linear_algebra_1 = require("../../../../../mol-math/linear-algebra");
var bonds_1 = require("../../../../../mol-model-formats/structure/mmcif/bonds");
var MAX_RADIUS = 4;
function addMapEntry(map, a, b) {
    if (map.has(a))
        map.get(a).push(b);
    else
        map.set(a, [b]);
}
function addLink(indexA, indexB, order, flag, state) {
    addMapEntry(state.mapAB, indexA, { indexB: indexB, order: order, flag: flag });
    addMapEntry(state.mapBA, indexB, { indexB: indexA, order: order, flag: flag });
    generic_1.UniqueArray.add(state.bondedA, indexA, indexA);
    generic_1.UniqueArray.add(state.bondedB, indexB, indexB);
}
var _imageTransform = linear_algebra_1.Mat4.zero();
function findPairLinks(unitA, unitB, params, map) {
    var state = { mapAB: new Map(), mapBA: new Map(), bondedA: generic_1.UniqueArray.create(), bondedB: generic_1.UniqueArray.create() };
    var bondCount = 0;
    var atomsA = unitA.elements;
    var _a = unitA.model.atomicConformation, xA = _a.x, yA = _a.y, zA = _a.z;
    var atomsB = unitB.elements;
    var atomCount = unitA.elements.length;
    var _b = unitA.model.atomicHierarchy.atoms, type_symbolA = _b.type_symbol, label_alt_idA = _b.label_alt_id;
    var _c = unitB.model.atomicHierarchy.atoms, type_symbolB = _c.type_symbol, label_alt_idB = _c.label_alt_id;
    var lookup3d = unitB.lookup3d;
    var structConn = unitA.model === unitB.model && unitA.model.sourceData.kind === 'mmCIF' ? bonds_1.StructConn.get(unitA.model) : void 0;
    // the lookup queries need to happen in the "unitB space".
    // that means imageA = inverseOperB(operA(aI))
    var imageTransform = linear_algebra_1.Mat4.mul(_imageTransform, unitB.conformation.operator.inverse, unitA.conformation.operator.matrix);
    var isNotIdentity = !linear_algebra_1.Mat4.isIdentity(imageTransform);
    var imageA = linear_algebra_1.Vec3.zero();
    var _d = lookup3d.boundary.sphere, bCenter = _d.center, bRadius = _d.radius;
    var testDistanceSq = (bRadius + MAX_RADIUS) * (bRadius + MAX_RADIUS);
    for (var _aI = 0; _aI < atomCount; _aI++) {
        var aI = atomsA[_aI];
        linear_algebra_1.Vec3.set(imageA, xA[aI], yA[aI], zA[aI]);
        if (isNotIdentity)
            linear_algebra_1.Vec3.transformMat4(imageA, imageA, imageTransform);
        if (linear_algebra_1.Vec3.squaredDistance(imageA, bCenter) > testDistanceSq)
            continue;
        var structConnEntries = params.forceCompute ? void 0 : structConn && structConn.getAtomEntries(aI);
        if (structConnEntries && structConnEntries.length) {
            for (var _i = 0, structConnEntries_1 = structConnEntries; _i < structConnEntries_1.length; _i++) {
                var se = structConnEntries_1[_i];
                if (se.distance < MAX_RADIUS)
                    continue;
                for (var _e = 0, _f = se.partners; _e < _f.length; _e++) {
                    var p = _f[_e];
                    var _bI = int_1.SortedArray.indexOf(unitB.elements, p.atomIndex);
                    if (_bI < 0)
                        continue;
                    addLink(_aI, _bI, se.order, se.flags, state);
                    bondCount++;
                }
            }
        }
        var _g = lookup3d.find(imageA[0], imageA[1], imageA[2], MAX_RADIUS), indices = _g.indices, count = _g.count, squaredDistances = _g.squaredDistances;
        if (count === 0)
            continue;
        var aeI = common_1.getElementIdx(type_symbolA.value(aI));
        var isHa = common_1.isHydrogen(aeI);
        var thresholdA = common_1.getElementThreshold(aeI);
        var altA = label_alt_idA.value(aI);
        var metalA = common_1.MetalsSet.has(aeI);
        for (var ni = 0; ni < count; ni++) {
            var _bI = indices[ni];
            var bI = atomsB[_bI];
            var altB = label_alt_idB.value(bI);
            // TODO: check if they have the same model?
            if (altA && altB && altA !== altB)
                continue;
            var beI = common_1.getElementIdx(type_symbolB.value(bI));
            var isMetal = metalA || common_1.MetalsSet.has(beI);
            var isHb = common_1.isHydrogen(beI);
            if (isHa && isHb)
                continue;
            var dist = Math.sqrt(squaredDistances[ni]);
            if (dist === 0)
                continue;
            // handle "struct conn" bonds.
            if (structConnEntries && structConnEntries.length) {
                var added = false;
                for (var _h = 0, structConnEntries_2 = structConnEntries; _h < structConnEntries_2.length; _h++) {
                    var se = structConnEntries_2[_h];
                    for (var _j = 0, _k = se.partners; _j < _k.length; _j++) {
                        var p = _k[_j];
                        if (p.atomIndex === bI) {
                            addLink(_aI, _bI, se.order, se.flags, state);
                            bondCount++;
                            added = true;
                            break;
                        }
                    }
                    if (added)
                        break;
                }
                if (added)
                    continue;
            }
            if (isHa || isHb) {
                if (dist < params.maxHbondLength) {
                    addLink(_aI, _bI, 1, 1 /* Covalent */ | 64 /* Computed */, state); // TODO: check if correct
                    bondCount++;
                }
                continue;
            }
            var thresholdAB = common_1.getElementPairThreshold(aeI, beI);
            var pairingThreshold = thresholdAB > 0
                ? thresholdAB
                : beI < 0 ? thresholdA : Math.max(thresholdA, common_1.getElementThreshold(beI));
            if (dist <= pairingThreshold) {
                addLink(_aI, _bI, 1, (isMetal ? 2 /* MetallicCoordination */ : 1 /* Covalent */) | 64 /* Computed */, state);
                bondCount++;
            }
        }
    }
    if (bondCount) {
        addMapEntry(map, unitA.id, new data_1.InterUnitBonds.UnitPairBonds(unitA, unitB, bondCount, state.bondedA.array, state.mapAB));
        addMapEntry(map, unitB.id, new data_1.InterUnitBonds.UnitPairBonds(unitB, unitA, bondCount, state.bondedB.array, state.mapBA));
    }
    return bondCount;
}
function findLinks(structure, params) {
    var map = new Map();
    if (!structure.units.some(function (u) { return unit_1.default.isAtomic(u); }))
        return new data_1.InterUnitBonds(map);
    var lookup = structure.lookup3d;
    var imageCenter = linear_algebra_1.Vec3.zero();
    for (var _i = 0, _a = structure.units; _i < _a.length; _i++) {
        var unit = _a[_i];
        if (!unit_1.default.isAtomic(unit))
            continue;
        var bs = unit.lookup3d.boundary.sphere;
        linear_algebra_1.Vec3.transformMat4(imageCenter, bs.center, unit.conformation.operator.matrix);
        var closeUnits = lookup.findUnitIndices(imageCenter[0], imageCenter[1], imageCenter[2], bs.radius + MAX_RADIUS);
        for (var i = 0; i < closeUnits.count; i++) {
            var other = structure.units[closeUnits.indices[i]];
            if (!unit_1.default.isAtomic(other) || unit.id >= other.id)
                continue;
            if (other.elements.length >= unit.elements.length)
                findPairLinks(unit, other, params, map);
            else
                findPairLinks(other, unit, params, map);
        }
    }
    return new data_1.InterUnitBonds(map);
}
function computeInterUnitBonds(structure, params) {
    return findLinks(structure, {
        maxHbondLength: (params && params.maxHbondLength) || 1.15,
        forceCompute: !!(params && params.forceCompute),
    });
}
exports.computeInterUnitBonds = computeInterUnitBonds;
