/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import Unit from '../../unit';
import { StructureElement } from '../../../structure';
interface PairRestraint {
    readonly unitA: Unit;
    readonly unitB: Unit;
    readonly indexA: StructureElement.UnitIndex;
    readonly indexB: StructureElement.UnitIndex;
}
export declare class PairRestraints<T extends PairRestraint> {
    pairs: ReadonlyArray<T>;
    readonly count: number;
    private readonly pairKeyIndices;
    /** Indices into this.pairs */
    getPairIndices(indexA: StructureElement.UnitIndex, unitA: Unit, indexB: StructureElement.UnitIndex, unitB: Unit): ReadonlyArray<number>;
    getPairs(indexA: StructureElement.UnitIndex, unitA: Unit, indexB: StructureElement.UnitIndex, unitB: Unit): T[] | undefined;
    constructor(pairs: ReadonlyArray<T>);
}
export interface CrossLinkRestraint extends PairRestraint {
    readonly restraintType: 'harmonic' | 'upper bound' | 'lower bound';
    readonly distanceThreshold: number;
    readonly psi: number;
    readonly sigma1: number;
    readonly sigma2: number;
}
export interface PredictedContactRestraint extends PairRestraint {
    readonly distance_lower_limit: number;
    readonly distance_upper_limit: number;
    readonly probability: number;
    readonly restraint_type: 'lower bound' | 'upper bound' | 'lower and upper bound';
    readonly model_granularity: 'by-residue' | 'by-feature' | 'by-atom';
}
export interface DistanceRestraint extends PairRestraint {
    readonly upper_limit: number;
    readonly upper_limit_esd: number;
    readonly lower_limit: number;
    readonly lower_limit_esd: number;
    readonly probability: number;
    readonly restraint_type: 'lower bound' | 'upper bound' | 'lower and upper bound';
    readonly granularity: 'by-residue' | 'by-atom';
}
export {};
