"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var geometry_1 = require("../../../../mol-math/geometry");
var boundary_helper_1 = require("../../../../mol-math/geometry/boundary-helper");
var linear_algebra_1 = require("../../../../mol-math/linear-algebra");
var tmpBox = geometry_1.Box3D.empty();
var tmpSphere = geometry_1.Sphere3D.zero();
var boundaryHelper = new boundary_helper_1.BoundaryHelper();
function computeStructureBoundary(s) {
    var min = linear_algebra_1.Vec3.create(Number.MAX_VALUE, Number.MAX_VALUE, Number.MAX_VALUE);
    var max = linear_algebra_1.Vec3.create(-Number.MAX_VALUE, -Number.MAX_VALUE, -Number.MAX_VALUE);
    var units = s.units;
    boundaryHelper.reset(0);
    for (var i = 0, _i = units.length; i < _i; i++) {
        var u = units[i];
        var invariantBoundary = u.lookup3d.boundary;
        var o = u.conformation.operator;
        if (o.isIdentity) {
            linear_algebra_1.Vec3.min(min, min, invariantBoundary.box.min);
            linear_algebra_1.Vec3.max(max, max, invariantBoundary.box.max);
            boundaryHelper.boundaryStep(invariantBoundary.sphere.center, invariantBoundary.sphere.radius);
        }
        else {
            geometry_1.Box3D.transform(tmpBox, invariantBoundary.box, o.matrix);
            linear_algebra_1.Vec3.min(min, min, tmpBox.min);
            linear_algebra_1.Vec3.max(max, max, tmpBox.max);
            geometry_1.Sphere3D.transform(tmpSphere, invariantBoundary.sphere, o.matrix);
            boundaryHelper.boundaryStep(tmpSphere.center, tmpSphere.radius);
        }
    }
    boundaryHelper.finishBoundaryStep();
    for (var i = 0, _i = units.length; i < _i; i++) {
        var u = units[i];
        var invariantBoundary = u.lookup3d.boundary;
        var o = u.conformation.operator;
        if (o.isIdentity) {
            boundaryHelper.extendStep(invariantBoundary.sphere.center, invariantBoundary.sphere.radius);
        }
        else {
            geometry_1.Sphere3D.transform(tmpSphere, invariantBoundary.sphere, o.matrix);
            boundaryHelper.extendStep(tmpSphere.center, tmpSphere.radius);
        }
    }
    return { box: { min: min, max: max }, sphere: boundaryHelper.getSphere() };
}
exports.computeStructureBoundary = computeStructureBoundary;
