"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var minimize_rmsd_1 = require("../../../../mol-math/linear-algebra/3d/minimize-rmsd");
var element_1 = require("../element");
var int_1 = require("../../../../mol-data/int");
function superposeStructures(xs) {
    var ret = [];
    if (xs.length <= 0)
        return ret;
    var n = getMinSize(xs);
    var input = { a: getPositionTable(xs[0], n), b: getPositionTable(xs[1], n) };
    ret[0] = minimize_rmsd_1.MinimizeRmsd.compute(input);
    for (var i = 2; i < xs.length; i++) {
        input.b = getPositionTable(xs[i], n);
        input.centerB = void 0;
        ret.push(minimize_rmsd_1.MinimizeRmsd.compute(input));
    }
    return ret;
}
exports.superposeStructures = superposeStructures;
function getPositionTable(xs, n) {
    var ret = minimize_rmsd_1.MinimizeRmsd.Positions.empty(n);
    var o = 0;
    for (var _a = 0, _b = xs.elements; _a < _b.length; _a++) {
        var u = _b[_a];
        var unit = u.unit, indices = u.indices;
        var elements = unit.elements;
        var _c = unit.conformation, x = _c.x, y = _c.y, z = _c.z;
        for (var i = 0, _i = int_1.OrderedSet.size(indices); i < _i; i++) {
            var e = elements[int_1.OrderedSet.getAt(indices, i)];
            ret.x[o] = x(e);
            ret.y[o] = y(e);
            ret.z[o] = z(e);
            o++;
            if (o >= n)
                break;
        }
        if (o >= n)
            break;
    }
    return ret;
}
function getMinSize(xs) {
    if (xs.length === 0)
        return 0;
    var s = element_1.default.Loci.size(xs[0]);
    for (var i = 1; i < xs.length; i++) {
        var t = element_1.default.Loci.size(xs[i]);
        if (t < s)
            s = t;
    }
    return s;
}
