"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../../../mol-math/linear-algebra");
var int_1 = require("../../../../mol-data/int");
var transform_data_1 = require("../../../../mol-geo/geometry/transform-data");
var StructureUnitTransforms = /** @class */ (function () {
    function StructureUnitTransforms(structure) {
        this.structure = structure;
        this.groupUnitTransforms = [];
        /** maps unit.id to offset of transform in unitTransforms */
        this.unitOffsetMap = int_1.IntMap.Mutable();
        this.groupIndexMap = int_1.IntMap.Mutable();
        this.unitTransforms = new Float32Array(structure.units.length * 16);
        this.size = structure.units.length;
        transform_data_1.fillIdentityTransform(this.unitTransforms, structure.units.length);
        var groupOffset = 0;
        for (var i = 0, il = structure.unitSymmetryGroups.length; i < il; ++i) {
            var g = structure.unitSymmetryGroups[i];
            this.groupIndexMap.set(g.hashCode, i);
            var groupTransforms = this.unitTransforms.subarray(groupOffset, groupOffset + g.units.length * 16);
            this.groupUnitTransforms.push(groupTransforms);
            for (var j = 0, jl = g.units.length; j < jl; ++j) {
                this.unitOffsetMap.set(g.units[j].id, groupOffset + j * 16);
            }
            groupOffset += g.units.length * 16;
        }
    }
    StructureUnitTransforms.prototype.reset = function () {
        transform_data_1.fillIdentityTransform(this.unitTransforms, this.size);
    };
    StructureUnitTransforms.prototype.setTransform = function (matrix, unit) {
        linear_algebra_1.Mat4.toArray(matrix, this.unitTransforms, this.unitOffsetMap.get(unit.id));
    };
    StructureUnitTransforms.prototype.getTransform = function (out, unit) {
        return linear_algebra_1.Mat4.fromArray(out, this.unitTransforms, this.unitOffsetMap.get(unit.id));
    };
    StructureUnitTransforms.prototype.getSymmetryGroupTransforms = function (group) {
        return this.groupUnitTransforms[this.groupIndexMap.get(group.hashCode)];
    };
    return StructureUnitTransforms;
}());
exports.StructureUnitTransforms = StructureUnitTransforms;
