"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("./model/types");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var matrix_1 = require("../../mol-math/linear-algebra/matrix/matrix");
function getCoarseBegCompId(unit, element) {
    var entityKey = unit.coarseElements.entityKey[element];
    var seq = unit.model.sequence.byEntityKey[entityKey];
    var seq_id_begin = unit.coarseElements.seq_id_begin.value(element);
    return seq.compId.value(seq_id_begin - 1); // 1-indexed
}
exports.getCoarseBegCompId = getCoarseBegCompId;
function getElementMoleculeType(unit, element) {
    switch (unit.kind) {
        case 0 /* Atomic */:
            return unit.model.atomicHierarchy.derived.residue.moleculeType[unit.residueIndex[element]];
        case 1 /* Spheres */:
        case 2 /* Gaussians */:
            // TODO add unit.model.coarseHierarchy.derived.residue.moleculeType
            var compId = getCoarseBegCompId(unit, element);
            var cc = unit.model.properties.chemicalComponentMap.get(compId);
            if (cc)
                return types_1.getMoleculeType(cc.type, compId);
    }
    return 0 /* unknown */;
}
exports.getElementMoleculeType = getElementMoleculeType;
function getAtomicMoleculeType(model, rI) {
    return model.atomicHierarchy.derived.residue.moleculeType[rI];
}
exports.getAtomicMoleculeType = getAtomicMoleculeType;
var EmptyAtomIds = new Set();
function getAtomIdForAtomRole(moleculeType, atomRole) {
    var m = types_1.MoleculeTypeAtomRoleId[moleculeType];
    if (m !== undefined) {
        var a = m[atomRole];
        if (a !== undefined)
            return a;
    }
    return EmptyAtomIds;
}
exports.getAtomIdForAtomRole = getAtomIdForAtomRole;
function residueLabel(model, rI) {
    var _a = model.atomicHierarchy, residues = _a.residues, chains = _a.chains, residueAtomSegments = _a.residueAtomSegments, chainAtomSegments = _a.chainAtomSegments;
    var label_comp_id = residues.label_comp_id, label_seq_id = residues.label_seq_id;
    var label_asym_id = chains.label_asym_id;
    var cI = chainAtomSegments.index[residueAtomSegments.offsets[rI]];
    return label_asym_id.value(cI) + " " + label_comp_id.value(rI) + " " + label_seq_id.value(rI);
}
exports.residueLabel = residueLabel;
function elementLabel(model, index) {
    var _a = model.atomicHierarchy, atoms = _a.atoms, residues = _a.residues, chains = _a.chains, residueAtomSegments = _a.residueAtomSegments, chainAtomSegments = _a.chainAtomSegments;
    var label_atom_id = atoms.label_atom_id;
    var auth_seq_id = residues.auth_seq_id, label_comp_id = residues.label_comp_id;
    var auth_asym_id = chains.auth_asym_id;
    var residueIndex = residueAtomSegments.index[index];
    var chainIndex = chainAtomSegments.index[residueIndex];
    return "[" + label_comp_id.value(residueIndex) + "]" + auth_seq_id.value(residueIndex) + ":" + auth_asym_id.value(chainIndex) + "." + label_atom_id.value(index);
}
exports.elementLabel = elementLabel;
// const centerPos = Vec3.zero()
// const centerMin = Vec3.zero()
// export function getCenterAndRadius(centroid: Vec3, unit: Unit, indices: ArrayLike<number>) {
//     const pos = unit.conformation.position
//     const { elements } = unit
//     Vec3.set(centroid, 0, 0, 0)
//     for (let i = 0, il = indices.length; i < il; ++i) {
//         pos(elements[indices[i]], centerPos)
//         Vec3.add(centroid, centroid, centerPos)
//         Vec3.min(centerMin, centerMin, centerPos)
//     }
//     Vec3.scale(centroid, centroid, 1/indices.length)
//     return Vec3.distance(centerMin, centroid)
// }
var matrixPos = linear_algebra_1.Vec3.zero();
function getPositionMatrix(unit, indices) {
    var pos = unit.conformation.position;
    var mat = matrix_1.default.create(3, indices.length);
    var elements = unit.elements;
    for (var i = 0, il = indices.length; i < il; ++i) {
        pos(elements[indices[i]], matrixPos);
        linear_algebra_1.Vec3.toArray(matrixPos, mat.data, i * 3);
    }
    return mat;
}
exports.getPositionMatrix = getPositionMatrix;
