/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { SpacegroupCell, Box3D } from '../../mol-math/geometry';
import { Tensor, Mat4 } from '../../mol-math/linear-algebra';
/** The basic unit cell that contains the data. */
interface VolumeData {
    readonly cell: SpacegroupCell;
    readonly fractionalBox: Box3D;
    readonly data: Tensor;
    readonly dataStats: Readonly<{
        min: number;
        max: number;
        mean: number;
        sigma: number;
    }>;
}
declare namespace VolumeData {
    const One: VolumeData;
    function getGridToCartesianTransform(volume: VolumeData): Mat4;
    function areEquivalent(volA: VolumeData, volB: VolumeData): boolean;
}
declare type VolumeIsoValue = VolumeIsoValue.Absolute | VolumeIsoValue.Relative;
declare namespace VolumeIsoValue {
    type Relative = Readonly<{
        kind: 'relative';
        relativeValue: number;
    }>;
    type Absolute = Readonly<{
        kind: 'absolute';
        absoluteValue: number;
    }>;
    function areSame(a: VolumeIsoValue, b: VolumeIsoValue, stats: VolumeData['dataStats']): boolean;
    function absolute(value: number): Absolute;
    function relative(value: number): Relative;
    function calcAbsolute(stats: VolumeData['dataStats'], relativeValue: number): number;
    function calcRelative(stats: VolumeData['dataStats'], absoluteValue: number): number;
    function toAbsolute(value: VolumeIsoValue, stats: VolumeData['dataStats']): Absolute;
    function toRelative(value: VolumeIsoValue, stats: VolumeData['dataStats']): Relative;
    function toString(value: VolumeIsoValue): string;
}
export { VolumeData, VolumeIsoValue };
