"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var geometry_1 = require("../../mol-math/geometry");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var common_1 = require("../../mol-math/linear-algebra/3d/common");
var VolumeData;
(function (VolumeData) {
    VolumeData.One = {
        cell: geometry_1.SpacegroupCell.Zero,
        fractionalBox: geometry_1.Box3D.empty(),
        data: linear_algebra_1.Tensor.create(linear_algebra_1.Tensor.Space([1, 1, 1], [0, 1, 2]), linear_algebra_1.Tensor.Data1([0])),
        dataStats: { min: 0, max: 0, mean: 0, sigma: 0 }
    };
    var _scale = linear_algebra_1.Mat4.zero(), _translate = linear_algebra_1.Mat4.zero();
    function getGridToCartesianTransform(volume) {
        var space = volume.data.space;
        var scale = linear_algebra_1.Mat4.fromScaling(_scale, linear_algebra_1.Vec3.div(linear_algebra_1.Vec3.zero(), geometry_1.Box3D.size(linear_algebra_1.Vec3.zero(), volume.fractionalBox), linear_algebra_1.Vec3.ofArray(space.dimensions)));
        var translate = linear_algebra_1.Mat4.fromTranslation(_translate, volume.fractionalBox.min);
        return linear_algebra_1.Mat4.mul3(linear_algebra_1.Mat4.zero(), volume.cell.fromFractional, translate, scale);
    }
    VolumeData.getGridToCartesianTransform = getGridToCartesianTransform;
    function areEquivalent(volA, volB) {
        return volA === volB;
    }
    VolumeData.areEquivalent = areEquivalent;
})(VolumeData || (VolumeData = {}));
exports.VolumeData = VolumeData;
var VolumeIsoValue;
(function (VolumeIsoValue) {
    function areSame(a, b, stats) {
        return common_1.equalEps(toAbsolute(a, stats).absoluteValue, toAbsolute(b, stats).absoluteValue, stats.sigma / 100);
    }
    VolumeIsoValue.areSame = areSame;
    function absolute(value) { return { kind: 'absolute', absoluteValue: value }; }
    VolumeIsoValue.absolute = absolute;
    function relative(value) { return { kind: 'relative', relativeValue: value }; }
    VolumeIsoValue.relative = relative;
    function calcAbsolute(stats, relativeValue) {
        return relativeValue * stats.sigma + stats.mean;
    }
    VolumeIsoValue.calcAbsolute = calcAbsolute;
    function calcRelative(stats, absoluteValue) {
        return stats.sigma === 0 ? 0 : ((absoluteValue - stats.mean) / stats.sigma);
    }
    VolumeIsoValue.calcRelative = calcRelative;
    function toAbsolute(value, stats) {
        return value.kind === 'absolute' ? value : { kind: 'absolute', absoluteValue: VolumeIsoValue.calcAbsolute(stats, value.relativeValue) };
    }
    VolumeIsoValue.toAbsolute = toAbsolute;
    function toRelative(value, stats) {
        return value.kind === 'relative' ? value : { kind: 'relative', relativeValue: VolumeIsoValue.calcRelative(stats, value.absoluteValue) };
    }
    VolumeIsoValue.toRelative = toRelative;
    function toString(value) {
        return value.kind === 'relative'
            ? value.relativeValue + " \u03C3"
            : "" + value.absoluteValue;
    }
    VolumeIsoValue.toString = toString;
})(VolumeIsoValue || (VolumeIsoValue = {}));
exports.VolumeIsoValue = VolumeIsoValue;
