"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var int_1 = require("../../../../../mol-data/int");
var structure_quality_report_1 = require("../../../../../mol-model-props/pdbe/structure-quality-report");
var structure_quality_report_2 = require("../../../../../mol-model-props/pdbe/themes/structure-quality-report");
var structure_1 = require("../../../../../mol-model/structure");
var param_definition_1 = require("../../../../../mol-util/param-definition");
var behavior_1 = require("../../../behavior");
exports.PDBeStructureQualityReport = behavior_1.PluginBehavior.create({
    name: 'pdbe-structure-quality-report-prop',
    category: 'custom-props',
    display: { name: 'PDBe Structure Quality Report' },
    ctor: /** @class */ (function (_super) {
        __extends(class_1, _super);
        function class_1() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.attach = structure_quality_report_1.StructureQualityReport.createAttachTask(function (m) { return "https://www.ebi.ac.uk/pdbe/api/validation/residuewise_outlier_summary/entry/" + m.label.toLowerCase(); }, _this.ctx.fetch);
            _this.provider = {
                option: [structure_quality_report_1.StructureQualityReport.Descriptor.name, 'PDBe Structure Quality Report'],
                descriptor: structure_quality_report_1.StructureQualityReport.Descriptor,
                defaultSelected: _this.params.autoAttach,
                attachableTo: function () { return true; },
                attach: _this.attach
            };
            return _this;
        }
        class_1.prototype.register = function () {
            this.ctx.customModelProperties.register(this.provider);
            this.ctx.lociLabels.addProvider(labelPDBeValidation);
            this.ctx.structureRepresentation.themeCtx.colorThemeRegistry.add('pdbe-structure-quality-report', {
                label: 'PDBe Structure Quality Report',
                factory: structure_quality_report_2.StructureQualityReportColorTheme,
                getParams: function () { return ({}); },
                defaultValues: {},
                isApplicable: function (ctx) { return !!ctx.structure && !ctx.structure.isEmpty && ctx.structure.models[0].customProperties.has(structure_quality_report_1.StructureQualityReport.Descriptor); }
            });
        };
        class_1.prototype.update = function (p) {
            var updated = this.params.autoAttach !== p.autoAttach;
            this.params.autoAttach = p.autoAttach;
            this.provider.defaultSelected = p.autoAttach;
            return updated;
        };
        class_1.prototype.unregister = function () {
            console.log('unregister');
            this.ctx.customModelProperties.unregister(structure_quality_report_1.StructureQualityReport.Descriptor.name);
            this.ctx.lociLabels.removeProvider(labelPDBeValidation);
            this.ctx.structureRepresentation.themeCtx.colorThemeRegistry.remove('pdbe-structure-quality-report');
        };
        return class_1;
    }(behavior_1.PluginBehavior.Handler)),
    params: function () { return ({
        autoAttach: param_definition_1.ParamDefinition.Boolean(false)
    }); }
});
function labelPDBeValidation(loci) {
    switch (loci.kind) {
        case 'element-loci':
            var e = loci.elements[0];
            var u = e.unit;
            if (!u.model.customProperties.has(structure_quality_report_1.StructureQualityReport.Descriptor))
                return void 0;
            var se = structure_1.StructureElement.create(u, u.elements[int_1.OrderedSet.getAt(e.indices, 0)]);
            var issues = structure_quality_report_1.StructureQualityReport.getIssues(se);
            if (issues.length === 0)
                return 'PDBe Validation: No Issues';
            return "PDBe Validation: " + issues.join(', ');
        default: return void 0;
    }
}
