"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../../../mol-util/param-definition");
var assembly_symmetry_1 = require("../../../../../mol-model-props/rcsb/assembly-symmetry");
var assembly_symmetry_cluster_1 = require("../../../../../mol-model-props/rcsb/themes/assembly-symmetry-cluster");
var assembly_symmetry_axes_1 = require("../../../../../mol-model-props/rcsb/representations/assembly-symmetry-axes");
var loci_1 = require("../../../../../mol-model/loci");
var int_1 = require("../../../../../mol-data/int");
var db_1 = require("../../../../../mol-data/db");
var behavior_1 = require("../../../behavior");
exports.RCSBAssemblySymmetry = behavior_1.PluginBehavior.create({
    name: 'rcsb-assembly-symmetry-prop',
    category: 'custom-props',
    display: { name: 'RCSB Assembly Symmetry' },
    ctor: /** @class */ (function (_super) {
        __extends(class_1, _super);
        function class_1() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.attach = assembly_symmetry_1.AssemblySymmetry.createAttachTask(_this.ctx.fetch);
            _this.provider = {
                option: [assembly_symmetry_1.AssemblySymmetry.Descriptor.name, 'RCSB Assembly Symmetry'],
                descriptor: assembly_symmetry_1.AssemblySymmetry.Descriptor,
                defaultSelected: _this.params.autoAttach,
                attachableTo: function () { return true; },
                attach: _this.attach
            };
            return _this;
        }
        class_1.prototype.register = function () {
            this.ctx.customModelProperties.register(this.provider);
            this.ctx.lociLabels.addProvider(labelAssemblySymmetryAxes);
            this.ctx.structureRepresentation.themeCtx.colorThemeRegistry.add('rcsb-assembly-symmetry-cluster', assembly_symmetry_cluster_1.AssemblySymmetryClusterColorThemeProvider);
            this.ctx.structureRepresentation.registry.add('rcsb-assembly-symmetry-axes', assembly_symmetry_axes_1.AssemblySymmetryAxesRepresentationProvider);
        };
        class_1.prototype.update = function (p) {
            var updated = this.params.autoAttach !== p.autoAttach;
            this.params.autoAttach = p.autoAttach;
            this.provider.defaultSelected = p.autoAttach;
            return updated;
        };
        class_1.prototype.unregister = function () {
            this.ctx.customModelProperties.unregister(assembly_symmetry_1.AssemblySymmetry.Descriptor.name);
            this.ctx.lociLabels.removeProvider(labelAssemblySymmetryAxes);
            this.ctx.structureRepresentation.themeCtx.colorThemeRegistry.remove('rcsb-assembly-symmetry-cluster');
            this.ctx.structureRepresentation.registry.remove('rcsb-assembly-symmetry-axes');
        };
        return class_1;
    }(behavior_1.PluginBehavior.Handler)),
    params: function () { return ({
        autoAttach: param_definition_1.ParamDefinition.Boolean(false)
    }); }
});
function labelAssemblySymmetryAxes(loci) {
    if (loci_1.isDataLoci(loci) && assembly_symmetry_1.AssemblySymmetry.is(loci.data) && loci.tag === 'axes') {
        var _a = loci.data.db, axis_1 = _a.rcsb_assembly_symmetry_axis, sym_1 = _a.rcsb_assembly_symmetry;
        var labels_1 = [];
        int_1.OrderedSet.forEach(loci.indices, function (v) {
            var symmetryId = axis_1.symmetry_id.value(v);
            var symmetry = db_1.Table.pickRow(sym_1, function (i) { return sym_1.id.value(i) === symmetryId; });
            if (symmetry) {
                labels_1.push("Axis of order " + axis_1.order.value(v) + " for " + symmetry.kind + " " + symmetry.type.toLowerCase() + " symmetry");
            }
        });
        return labels_1.length ? labels_1.join(', ') : undefined;
    }
    return undefined;
}
