"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var marker_action_1 = require("../../../mol-util/marker-action");
var objects_1 = require("../../state/objects");
var label_1 = require("../../../mol-theme/label");
var behavior_1 = require("../behavior");
var spine_1 = require("../../../mol-state/tree/spine");
exports.HighlightLoci = behavior_1.PluginBehavior.create({
    name: 'representation-highlight-loci',
    category: 'interaction',
    ctor: /** @class */ (function (_super) {
        __extends(class_1, _super);
        function class_1() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.lociMarkProvider = function (interactionLoci, action) {
                if (!_this.ctx.canvas3d)
                    return;
                _this.ctx.canvas3d.mark({ loci: interactionLoci.loci }, action);
            };
            return _this;
        }
        class_1.prototype.register = function () {
            this.ctx.interactivity.lociHighlights.addProvider(this.lociMarkProvider);
        };
        class_1.prototype.unregister = function () {
            this.ctx.interactivity.lociHighlights.removeProvider(this.lociMarkProvider);
        };
        return class_1;
    }(behavior_1.PluginBehavior.Handler)),
    display: { name: 'Highlight Loci on Canvas' }
});
exports.SelectLoci = behavior_1.PluginBehavior.create({
    name: 'representation-select-loci',
    category: 'interaction',
    ctor: /** @class */ (function (_super) {
        __extends(class_2, _super);
        function class_2(ctx, params) {
            var _this = _super.call(this, ctx, params) || this;
            _this.lociMarkProvider = function (interactionLoci, action) {
                if (!_this.ctx.canvas3d)
                    return;
                _this.ctx.canvas3d.mark({ loci: interactionLoci.loci }, action);
            };
            _this.spine = new spine_1.StateTreeSpine.Impl(ctx.state.dataState.cells);
            return _this;
        }
        class_2.prototype.register = function () {
            var _this = this;
            this.ctx.interactivity.lociSelections.addProvider(this.lociMarkProvider);
            this.subscribeObservable(this.ctx.events.state.object.created, function (_a) {
                var ref = _a.ref;
                var cell = _this.ctx.state.dataState.cells.get(ref);
                if (cell && objects_1.PluginStateObject.isRepresentation3D(cell.obj)) {
                    _this.spine.current = cell;
                    var so = _this.spine.getRootOfType(objects_1.PluginStateObject.Molecule.Structure);
                    if (so) {
                        var loci = _this.ctx.helpers.structureSelection.get(so.data);
                        _this.lociMarkProvider({ loci: loci }, marker_action_1.MarkerAction.Select);
                    }
                }
            });
        };
        class_2.prototype.unregister = function () {
            this.ctx.interactivity.lociSelections.removeProvider(this.lociMarkProvider);
        };
        return class_2;
    }(behavior_1.PluginBehavior.Handler)),
    display: { name: 'Select Loci on Canvas' }
});
exports.DefaultLociLabelProvider = behavior_1.PluginBehavior.create({
    name: 'default-loci-label-provider',
    category: 'interaction',
    ctor: /** @class */ (function () {
        function class_3(ctx) {
            this.ctx = ctx;
            this.f = label_1.labelFirst;
        }
        class_3.prototype.register = function () { this.ctx.lociLabels.addProvider(this.f); };
        class_3.prototype.unregister = function () { this.ctx.lociLabels.removeProvider(this.f); };
        return class_3;
    }()),
    display: { name: 'Provide Default Loci Label' }
});
