/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { ParamDefinition as PD } from '../../../../mol-util/param-definition';
import { PluginStateObject } from '../../../state/objects';
import { VolumeIsoValue, VolumeData } from '../../../../mol-model/volume';
import { VolumeServerInfo } from './model';
import { Vec3 } from '../../../../mol-math/linear-algebra';
import { Color } from '../../../../mol-util/color';
import { PluginBehavior } from '../../behavior';
import { PluginContext } from '../../../context';
declare const VolumeStreaming_base: {
    new (data: VolumeStreaming.Behavior, props?: {
        label: string;
        description?: string | undefined;
    } | undefined): {
        id: import("../../../../mol-util/uuid").default;
        type: PluginStateObject.TypeInfo;
        label: string;
        description?: string | undefined;
        data: VolumeStreaming.Behavior;
    };
    type: PluginStateObject.TypeInfo;
    is(obj?: import("../../../../mol-state/object").StateObject<any, import("../../../../mol-state/object").StateObject.Type<any>> | undefined): obj is {
        id: import("../../../../mol-util/uuid").default;
        type: PluginStateObject.TypeInfo;
        label: string;
        description?: string | undefined;
        data: VolumeStreaming.Behavior;
    };
};
export declare class VolumeStreaming extends VolumeStreaming_base {
}
export declare namespace VolumeStreaming {
    function channelParam(label: string, color: Color, defaultValue: VolumeIsoValue, stats: VolumeData['dataStats']): PD.Group<PD.Normalize<{
        isoValue: VolumeIsoValue;
        color: Color;
        wireframe: boolean;
        opacity: number;
    }>>;
    function createParams(data?: VolumeServerInfo.Data, defaultView?: ViewTypes): {
        view: PD.Mapped<PD.NamedParams<PD.Normalize<unknown>, "cell"> | PD.NamedParams<PD.Normalize<{
            bottomLeft: Vec3;
            topRight: Vec3;
        }>, "box"> | PD.NamedParams<PD.Normalize<{
            radius: number;
            bottomLeft: Vec3;
            topRight: Vec3;
        }>, "selection-box">>;
        detailLevel: PD.Select<number>;
        channels: PD.Group<PD.Normalize<{
            'em': PD.Normalize<{
                isoValue: any;
                color: any;
                wireframe: any;
                opacity: any;
            }>;
        }>> | PD.Group<PD.Normalize<{
            '2fo-fc': PD.Normalize<{
                isoValue: any;
                color: any;
                wireframe: any;
                opacity: any;
            }>;
            'fo-fc(+ve)': PD.Normalize<{
                isoValue: any;
                color: any;
                wireframe: any;
                opacity: any;
            }>;
            'fo-fc(-ve)': PD.Normalize<{
                isoValue: any;
                color: any;
                wireframe: any;
                opacity: any;
            }>;
        }>>;
    };
    type ViewTypes = 'box' | 'selection-box' | 'cell';
    type ParamDefinition = typeof createParams extends (...args: any[]) => (infer T) ? T : never;
    type Params = ParamDefinition extends PD.Params ? PD.Values<ParamDefinition> : {};
    type CT = typeof channelParam extends (...args: any[]) => (infer T) ? T : never;
    type ChannelParams = CT extends PD.Group<infer T> ? T : {};
    type ChannelType = 'em' | '2fo-fc' | 'fo-fc(+ve)' | 'fo-fc(-ve)';
    const ChannelTypeOptions: [ChannelType, string][];
    interface ChannelInfo {
        data: VolumeData;
        color: Color;
        wireframe: boolean;
        isoValue: VolumeIsoValue.Relative;
        opacity: number;
    }
    type Channels = {
        [name in ChannelType]?: ChannelInfo;
    };
    class Behavior extends PluginBehavior.WithSubscribers<Params> {
        plugin: PluginContext;
        info: VolumeServerInfo.Data;
        private cache;
        params: Params;
        channels: Channels;
        private queryData;
        private parseCif;
        private updateDynamicBox;
        private getStructureRoot;
        register(ref: string): void;
        update(params: Params): Promise<boolean>;
        private createChannel;
        getDescription(): "" | "Selection" | "Static Box" | "Cell";
        constructor(plugin: PluginContext, info: VolumeServerInfo.Data);
    }
}
export {};
