"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../../mol-util/param-definition");
var objects_1 = require("../../../state/objects");
var volume_1 = require("../../../../mol-model/volume");
var isosurface_1 = require("../../../../mol-repr/volume/isosurface");
var geometry_1 = require("../../../../mol-math/geometry");
var linear_algebra_1 = require("../../../../mol-math/linear-algebra");
var color_1 = require("../../../../mol-util/color");
var behavior_1 = require("../../behavior");
var lru_cache_1 = require("../../../../mol-util/lru-cache");
var url_1 = require("../../../../mol-util/url");
var cif_1 = require("../../../../mol-io/reader/cif");
var density_server_1 = require("../../../../mol-model-formats/volume/density-server");
var command_1 = require("../../../command");
var mol_state_1 = require("../../../../mol-state");
var representation_1 = require("../../../../mol-repr/representation");
var structure_1 = require("../../../../mol-model/structure");
var VolumeStreaming = /** @class */ (function (_super) {
    __extends(VolumeStreaming, _super);
    function VolumeStreaming() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return VolumeStreaming;
}(objects_1.PluginStateObject.CreateBehavior({ name: 'Volume Streaming' })));
exports.VolumeStreaming = VolumeStreaming;
(function (VolumeStreaming) {
    function channelParam(label, color, defaultValue, stats) {
        return param_definition_1.ParamDefinition.Group({
            isoValue: isosurface_1.createIsoValueParam(defaultValue, stats),
            color: param_definition_1.ParamDefinition.Color(color),
            wireframe: param_definition_1.ParamDefinition.Boolean(false),
            opacity: param_definition_1.ParamDefinition.Numeric(0.3, { min: 0, max: 1, step: 0.01 })
        }, { label: label, isExpanded: true });
    }
    var fakeSampling = {
        byteOffset: 0,
        rate: 1,
        sampleCount: [1, 1, 1],
        valuesInfo: [{ mean: 0, min: -1, max: 1, sigma: 0.1 }, { mean: 0, min: -1, max: 1, sigma: 0.1 }]
    };
    function createParams(data, defaultView) {
        // fake the info
        var info = data || { kind: 'em', header: { sampling: [fakeSampling], availablePrecisions: [{ precision: 0, maxVoxels: 0 }] }, emDefaultContourLevel: volume_1.VolumeIsoValue.relative(0) };
        var box = (data && data.structure.boundary.box) || geometry_1.Box3D.empty();
        return {
            view: param_definition_1.ParamDefinition.MappedStatic(defaultView || (info.kind === 'em' ? 'cell' : 'selection-box'), {
                'box': param_definition_1.ParamDefinition.Group({
                    bottomLeft: param_definition_1.ParamDefinition.Vec3(box.min),
                    topRight: param_definition_1.ParamDefinition.Vec3(box.max),
                }, { description: 'Static box defined by cartesian coords.', isFlat: true }),
                'selection-box': param_definition_1.ParamDefinition.Group({
                    radius: param_definition_1.ParamDefinition.Numeric(5, { min: 0, max: 50, step: 0.5 }),
                    bottomLeft: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.create(0, 0, 0), { isHidden: true }),
                    topRight: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.create(0, 0, 0), { isHidden: true }),
                }, { description: 'Box around last-interacted element.', isFlat: true }),
                'cell': param_definition_1.ParamDefinition.Group({}),
            }, { options: [['box', 'Bounded Box'], ['selection-box', 'Selection'], ['cell', 'Whole Structure']] }),
            detailLevel: param_definition_1.ParamDefinition.Select(Math.min(3, info.header.availablePrecisions.length - 1), info.header.availablePrecisions.map(function (p, i) { return [i, i + 1 + " [ " + (Math.pow(p.maxVoxels, 1 / 3) | 0) + "^3 cells ]"]; })),
            channels: info.kind === 'em'
                ? param_definition_1.ParamDefinition.Group({
                    'em': channelParam('EM', color_1.Color(0x638F8F), info.emDefaultContourLevel || volume_1.VolumeIsoValue.relative(1), info.header.sampling[0].valuesInfo[0])
                }, { isFlat: true })
                : param_definition_1.ParamDefinition.Group({
                    '2fo-fc': channelParam('2Fo-Fc', color_1.Color(0x3362B2), volume_1.VolumeIsoValue.relative(1.5), info.header.sampling[0].valuesInfo[0]),
                    'fo-fc(+ve)': channelParam('Fo-Fc(+ve)', color_1.Color(0x33BB33), volume_1.VolumeIsoValue.relative(3), info.header.sampling[0].valuesInfo[1]),
                    'fo-fc(-ve)': channelParam('Fo-Fc(-ve)', color_1.Color(0xBB3333), volume_1.VolumeIsoValue.relative(-3), info.header.sampling[0].valuesInfo[1]),
                }, { isFlat: true })
        };
    }
    VolumeStreaming.createParams = createParams;
    VolumeStreaming.ChannelTypeOptions = [['em', 'em'], ['2fo-fc', '2fo-fc'], ['fo-fc(+ve)', 'fo-fc(+ve)'], ['fo-fc(-ve)', 'fo-fc(-ve)']];
    var Behavior = /** @class */ (function (_super) {
        __extends(Behavior, _super);
        function Behavior(plugin, info) {
            var _this = _super.call(this, plugin) || this;
            _this.plugin = plugin;
            _this.info = info;
            _this.cache = lru_cache_1.LRUCache.create(25);
            _this.params = {};
            _this.channels = {};
            return _this;
        }
        Behavior.prototype.queryData = function (box) {
            return __awaiter(this, void 0, void 0, function () {
                var url, a, b, data, cif;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            url = url_1.urlCombine(this.info.serverUrl, this.info.kind + "/" + this.info.dataId.toLowerCase());
                            if (box) {
                                a = box.min, b = box.max;
                                url += "/box"
                                    + ("/" + a.map(function (v) { return Math.round(1000 * v) / 1000; }).join(','))
                                    + ("/" + b.map(function (v) { return Math.round(1000 * v) / 1000; }).join(','));
                            }
                            else {
                                url += "/cell";
                            }
                            url += "?detail=" + this.params.detailLevel;
                            data = lru_cache_1.LRUCache.get(this.cache, url);
                            if (data) {
                                return [2 /*return*/, data];
                            }
                            return [4 /*yield*/, this.plugin.runTask(this.plugin.fetch({ url: url, type: 'binary' }))];
                        case 1:
                            cif = _a.sent();
                            return [4 /*yield*/, this.parseCif(cif)];
                        case 2:
                            data = _a.sent();
                            if (!data) {
                                return [2 /*return*/];
                            }
                            lru_cache_1.LRUCache.set(this.cache, url, data);
                            return [2 /*return*/, data];
                    }
                });
            });
        };
        Behavior.prototype.parseCif = function (data) {
            return __awaiter(this, void 0, void 0, function () {
                var parsed, ret, i, block, densityServerCif, volume, _a, _b;
                return __generator(this, function (_c) {
                    switch (_c.label) {
                        case 0: return [4 /*yield*/, this.plugin.runTask(cif_1.CIF.parseBinary(data))];
                        case 1:
                            parsed = _c.sent();
                            if (parsed.isError) {
                                this.plugin.log.error('VolumeStreaming, parsing CIF: ' + parsed.toString());
                                return [2 /*return*/];
                            }
                            if (parsed.result.blocks.length < 2) {
                                this.plugin.log.error('VolumeStreaming: Invalid data.');
                                return [2 /*return*/];
                            }
                            ret = {};
                            i = 1;
                            _c.label = 2;
                        case 2:
                            if (!(i < parsed.result.blocks.length)) return [3 /*break*/, 6];
                            block = parsed.result.blocks[i];
                            densityServerCif = cif_1.CIF.schema.densityServer(block);
                            _b = (_a = this.plugin).runTask;
                            return [4 /*yield*/, density_server_1.volumeFromDensityServerData(densityServerCif)];
                        case 3: return [4 /*yield*/, _b.apply(_a, [_c.sent()])];
                        case 4:
                            volume = _c.sent();
                            ret[block.header] = volume;
                            _c.label = 5;
                        case 5:
                            i++;
                            return [3 /*break*/, 2];
                        case 6: return [2 /*return*/, ret];
                    }
                });
            });
        };
        Behavior.prototype.updateDynamicBox = function (ref, box) {
            if (this.params.view.name !== 'selection-box')
                return;
            var state = this.plugin.state.dataState;
            var newParams = __assign({}, this.params, { view: {
                    name: 'selection-box',
                    params: {
                        radius: this.params.view.params.radius,
                        bottomLeft: box.min,
                        topRight: box.max
                    }
                } });
            var update = state.build().to(ref).update(newParams);
            command_1.PluginCommands.State.Update.dispatch(this.plugin, { state: state, tree: update, options: { doNotUpdateCurrent: true } });
        };
        Behavior.prototype.getStructureRoot = function (ref) {
            return this.plugin.state.dataState.select(mol_state_1.StateSelection.Generators.byRef(ref).rootOfType([objects_1.PluginStateObject.Molecule.Structure]))[0];
        };
        Behavior.prototype.register = function (ref) {
            var _this = this;
            var lastLoci = representation_1.Representation.Loci.Empty;
            this.subscribeObservable(this.plugin.events.state.object.removed, function (o) {
                if (!objects_1.PluginStateObject.Molecule.Structure.is(o.obj) || lastLoci.loci.kind !== 'element-loci')
                    return;
                if (lastLoci.loci.structure === o.obj.data) {
                    lastLoci = representation_1.Representation.Loci.Empty;
                }
            });
            this.subscribeObservable(this.plugin.events.state.object.updated, function (o) {
                if (!objects_1.PluginStateObject.Molecule.Structure.is(o.oldObj) || lastLoci.loci.kind !== 'element-loci')
                    return;
                if (lastLoci.loci.structure === o.oldObj.data) {
                    lastLoci = representation_1.Representation.Loci.Empty;
                }
            });
            this.subscribeObservable(this.plugin.behaviors.interaction.click, function (_a) {
                var current = _a.current, buttons = _a.buttons, modifiers = _a.modifiers;
                if (buttons !== 2 /* Secondary */ || _this.params.view.name !== 'selection-box')
                    return;
                if (current.loci.kind === 'empty-loci') {
                    if (modifiers.control && buttons === 2 /* Secondary */) {
                        _this.updateDynamicBox(ref, geometry_1.Box3D.empty());
                        return;
                    }
                }
                // TODO: support link loci as well?
                // Perhaps structure loci too?
                if (!structure_1.StructureElement.isLoci(current.loci))
                    return;
                var parent = _this.plugin.helpers.substructureParent.get(current.loci.structure);
                if (!parent)
                    return;
                var root = _this.getStructureRoot(ref);
                if (!root || !root.obj || root.obj !== parent.obj)
                    return;
                if (representation_1.Representation.Loci.areEqual(lastLoci, current)) {
                    lastLoci = representation_1.Representation.Loci.Empty;
                    _this.updateDynamicBox(ref, geometry_1.Box3D.empty());
                    return;
                }
                lastLoci = current;
                var loci = structure_1.StructureElement.Loci.extendToWholeResidues(current.loci);
                var box = structure_1.StructureElement.Loci.getBoundary(loci).box;
                _this.updateDynamicBox(ref, box);
            });
        };
        Behavior.prototype.update = function (params) {
            return __awaiter(this, void 0, void 0, function () {
                var box, emptyData, r, data, _a, info;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            this.params = params;
                            box = void 0, emptyData = false;
                            switch (params.view.name) {
                                case 'box':
                                    box = geometry_1.Box3D.create(params.view.params.bottomLeft, params.view.params.topRight);
                                    emptyData = geometry_1.Box3D.volume(box) < 0.0001;
                                    break;
                                case 'selection-box': {
                                    box = geometry_1.Box3D.create(linear_algebra_1.Vec3.clone(params.view.params.bottomLeft), linear_algebra_1.Vec3.clone(params.view.params.topRight));
                                    r = params.view.params.radius;
                                    emptyData = geometry_1.Box3D.volume(box) < 0.0001;
                                    geometry_1.Box3D.expand(box, box, linear_algebra_1.Vec3.create(r, r, r));
                                    break;
                                }
                                case 'cell':
                                    box = this.info.kind === 'x-ray'
                                        ? this.info.structure.boundary.box
                                        : void 0;
                                    break;
                            }
                            if (!emptyData) return [3 /*break*/, 1];
                            _a = {};
                            return [3 /*break*/, 3];
                        case 1: return [4 /*yield*/, this.queryData(box)];
                        case 2:
                            _a = _b.sent();
                            _b.label = 3;
                        case 3:
                            data = _a;
                            if (!data)
                                return [2 /*return*/, false];
                            info = params.channels;
                            if (this.info.kind === 'x-ray') {
                                this.channels['2fo-fc'] = this.createChannel(data['2FO-FC'] || volume_1.VolumeData.One, info['2fo-fc'], this.info.header.sampling[0].valuesInfo[0]);
                                this.channels['fo-fc(+ve)'] = this.createChannel(data['FO-FC'] || volume_1.VolumeData.One, info['fo-fc(+ve)'], this.info.header.sampling[0].valuesInfo[1]);
                                this.channels['fo-fc(-ve)'] = this.createChannel(data['FO-FC'] || volume_1.VolumeData.One, info['fo-fc(-ve)'], this.info.header.sampling[0].valuesInfo[1]);
                            }
                            else {
                                this.channels['em'] = this.createChannel(data['EM'] || volume_1.VolumeData.One, info['em'], this.info.header.sampling[0].valuesInfo[0]);
                            }
                            return [2 /*return*/, true];
                    }
                });
            });
        };
        Behavior.prototype.createChannel = function (data, info, stats) {
            var i = info;
            return {
                data: data,
                color: i.color,
                wireframe: i.wireframe,
                opacity: i.opacity,
                isoValue: i.isoValue.kind === 'relative' ? i.isoValue : volume_1.VolumeIsoValue.toRelative(i.isoValue, stats)
            };
        };
        Behavior.prototype.getDescription = function () {
            if (this.params.view.name === 'selection-box')
                return 'Selection';
            if (this.params.view.name === 'box')
                return 'Static Box';
            if (this.params.view.name === 'cell')
                return 'Cell';
            return '';
        };
        return Behavior;
    }(behavior_1.PluginBehavior.WithSubscribers));
    VolumeStreaming.Behavior = Behavior;
})(VolumeStreaming = exports.VolumeStreaming || (exports.VolumeStreaming = {}));
exports.VolumeStreaming = VolumeStreaming;
