"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var objects_1 = require("../../../state/objects");
var model_1 = require("./model");
var param_definition_1 = require("../../../../mol-util/param-definition");
var mol_task_1 = require("../../../../mol-task");
var url_1 = require("../../../../mol-util/url");
var isosurface_1 = require("../../../../mol-repr/volume/isosurface");
var volume_1 = require("../../../../mol-model/volume");
var mol_state_1 = require("../../../../mol-state");
var util_1 = require("./util");
var behavior_1 = require("./behavior");
var representation_1 = require("../../../../mol-plugin/state/transforms/representation");
var registry_1 = require("../../../../mol-repr/volume/registry");
var theme_1 = require("../../../../mol-theme/theme");
var geometry_1 = require("../../../../mol-math/geometry");
var linear_algebra_1 = require("../../../../mol-math/linear-algebra");
// import { PluginContext } from '../../../../mol-plugin/context';
exports.InitVolumeStreaming = mol_state_1.StateAction.build({
    display: { name: 'Volume Streaming' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    params: function (a) {
        var method = util_1.getStreamingMethod(a && a.data);
        return {
            method: param_definition_1.ParamDefinition.Select(method, [['em', 'EM'], ['x-ray', 'X-Ray']]),
            id: param_definition_1.ParamDefinition.Text((a && a.data.models.length > 0 && a.data.models[0].label) || ''),
            serverUrl: param_definition_1.ParamDefinition.Text('https://ds.litemol.org'),
            defaultView: param_definition_1.ParamDefinition.Text(method === 'em' ? 'cell' : 'selection-box'),
            behaviorRef: param_definition_1.ParamDefinition.Text('', { isHidden: true })
        };
    },
    isApplicable: function (a) { return a.data.models.length === 1; }
})(function (_a, plugin) {
    var ref = _a.ref, state = _a.state, params = _a.params;
    return mol_task_1.Task.create('Volume Streaming', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
        var dataId, emDefaultContourLevel, emInfo, infoTree, infoObj, behTree;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    dataId = params.id.toLowerCase();
                    if (!(params.method === 'em')) return [3 /*break*/, 3];
                    return [4 /*yield*/, taskCtx.update('Getting EMDB info...')];
                case 1:
                    _a.sent();
                    return [4 /*yield*/, util_1.getEmdbIdAndContourLevel(plugin, taskCtx, dataId)];
                case 2:
                    emInfo = _a.sent();
                    dataId = emInfo.emdbId;
                    emDefaultContourLevel = emInfo.contour;
                    _a.label = 3;
                case 3:
                    infoTree = state.build().to(ref)
                        .apply(CreateVolumeStreamingInfo, {
                        serverUrl: params.serverUrl,
                        source: params.method === 'em'
                            ? { name: 'em', params: { isoValue: volume_1.VolumeIsoValue.absolute(emDefaultContourLevel || 0) } }
                            : { name: 'x-ray', params: {} },
                        dataId: dataId
                    });
                    return [4 /*yield*/, state.updateTree(infoTree).runInContext(taskCtx)];
                case 4:
                    infoObj = _a.sent();
                    behTree = state.build().to(infoTree.ref).apply(CreateVolumeStreamingBehavior, param_definition_1.ParamDefinition.getDefaultValues(behavior_1.VolumeStreaming.createParams(infoObj.data, params.defaultView)), { ref: params.behaviorRef ? params.behaviorRef : void 0 });
                    if (params.method === 'em') {
                        behTree.apply(VolumeStreamingVisual, { channel: 'em' }, { state: { isGhost: true } });
                    }
                    else {
                        behTree.apply(VolumeStreamingVisual, { channel: '2fo-fc' }, { state: { isGhost: true } });
                        behTree.apply(VolumeStreamingVisual, { channel: 'fo-fc(+ve)' }, { state: { isGhost: true } });
                        behTree.apply(VolumeStreamingVisual, { channel: 'fo-fc(-ve)' }, { state: { isGhost: true } });
                    }
                    return [4 /*yield*/, state.updateTree(behTree).runInContext(taskCtx)];
                case 5:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
});
exports.BoxifyVolumeStreaming = mol_state_1.StateAction.build({
    display: { name: 'Boxify Volume Streaming', description: 'Make the current box permanent.' },
    from: behavior_1.VolumeStreaming,
    isApplicable: function (a) { return a.data.params.view.name === 'selection-box'; }
})(function (_a, plugin) {
    var a = _a.a, ref = _a.ref, state = _a.state;
    var params = a.data.params;
    if (params.view.name !== 'selection-box')
        return;
    var box = geometry_1.Box3D.create(linear_algebra_1.Vec3.clone(params.view.params.bottomLeft), linear_algebra_1.Vec3.clone(params.view.params.topRight));
    var r = params.view.params.radius;
    geometry_1.Box3D.expand(box, box, linear_algebra_1.Vec3.create(r, r, r));
    var newParams = __assign({}, params, { view: {
            name: 'box',
            params: {
                bottomLeft: box.min,
                topRight: box.max
            }
        } });
    return state.updateTree(state.build().to(ref).update(newParams));
});
var CreateVolumeStreamingInfo = objects_1.PluginStateTransform.BuiltIn({
    name: 'create-volume-streaming-info',
    display: { name: 'Volume Streaming Info' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    to: model_1.VolumeServerInfo,
    params: function (a) {
        return {
            serverUrl: param_definition_1.ParamDefinition.Text('https://ds.litemol.org'),
            source: param_definition_1.ParamDefinition.MappedStatic('x-ray', {
                'em': param_definition_1.ParamDefinition.Group({
                    isoValue: isosurface_1.createIsoValueParam(volume_1.VolumeIsoValue.relative(1))
                }),
                'x-ray': param_definition_1.ParamDefinition.Group({})
            }),
            dataId: param_definition_1.ParamDefinition.Text('')
        };
    }
})({
    apply: function (_a, plugin) {
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
            var dataId, emDefaultContourLevel, header, data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dataId = params.dataId;
                        emDefaultContourLevel = params.source.name === 'em' ? params.source.params.isoValue : volume_1.VolumeIsoValue.relative(1);
                        return [4 /*yield*/, taskCtx.update('Getting server header...')];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, plugin.fetch({ url: url_1.urlCombine(params.serverUrl, params.source.name + "/" + dataId.toLocaleLowerCase()), type: 'json' }).runInContext(taskCtx)];
                    case 2:
                        header = _a.sent();
                        data = {
                            serverUrl: params.serverUrl,
                            dataId: dataId,
                            kind: params.source.name,
                            header: header,
                            emDefaultContourLevel: emDefaultContourLevel,
                            structure: a.data
                        };
                        return [2 /*return*/, new model_1.VolumeServerInfo(data, { label: "Volume Server: " + dataId })];
                }
            });
        }); });
    }
});
exports.CreateVolumeStreamingInfo = CreateVolumeStreamingInfo;
var CreateVolumeStreamingBehavior = objects_1.PluginStateTransform.BuiltIn({
    name: 'create-volume-streaming-behavior',
    display: { name: 'Volume Streaming Behavior' },
    from: model_1.VolumeServerInfo,
    to: behavior_1.VolumeStreaming,
    params: function (a) {
        return behavior_1.VolumeStreaming.createParams(a && a.data);
    }
})({
    canAutoUpdate: function (_a) {
        var oldParams = _a.oldParams, newParams = _a.newParams;
        return oldParams.view === newParams.view
            || (oldParams.view.name === newParams.view.name && oldParams.view.name === 'selection-box');
    },
    apply: function (_a, plugin) {
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Volume streaming', function (_) { return __awaiter(_this, void 0, void 0, function () {
            var behavior;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        behavior = new behavior_1.VolumeStreaming.Behavior(plugin, a.data);
                        return [4 /*yield*/, behavior.update(params)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, new behavior_1.VolumeStreaming(behavior, { label: 'Volume Streaming', description: behavior.getDescription() })];
                }
            });
        }); });
    },
    update: function (_a) {
        var _this = this;
        var b = _a.b, newParams = _a.newParams;
        return mol_task_1.Task.create('Update Volume Streaming', function (_) { return __awaiter(_this, void 0, void 0, function () {
            var ret;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, b.data.update(newParams)];
                    case 1:
                        ret = (_a.sent()) ? mol_state_1.StateTransformer.UpdateResult.Updated : mol_state_1.StateTransformer.UpdateResult.Unchanged;
                        b.description = b.data.getDescription();
                        return [2 /*return*/, ret];
                }
            });
        }); });
    }
});
exports.CreateVolumeStreamingBehavior = CreateVolumeStreamingBehavior;
var VolumeStreamingVisual = objects_1.PluginStateTransform.BuiltIn({
    name: 'create-volume-streaming-visual',
    display: { name: 'Volume Streaming Visual' },
    from: behavior_1.VolumeStreaming,
    to: objects_1.PluginStateObject.Volume.Representation3D,
    params: {
        channel: param_definition_1.ParamDefinition.Select('em', behavior_1.VolumeStreaming.ChannelTypeOptions, { isHidden: true })
    }
})({
    apply: function (_a, plugin) {
        var a = _a.a, srcParams = _a.params;
        return mol_task_1.Task.create('Volume Representation', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var channel, params, provider, props, repr;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        channel = a.data.channels[srcParams.channel];
                        if (!channel)
                            return [2 /*return*/, mol_state_1.StateObject.Null];
                        params = createVolumeProps(a.data, srcParams.channel);
                        provider = registry_1.BuiltInVolumeRepresentations.isosurface;
                        props = params.type.params || {};
                        repr = provider.factory(__assign({ webgl: plugin.canvas3d.webgl }, plugin.volumeRepresentation.themeCtx), provider.getParams);
                        repr.setTheme(theme_1.createTheme(plugin.volumeRepresentation.themeCtx, { volume: channel.data }, params));
                        return [4 /*yield*/, repr.createOrUpdate(props, channel.data).runInContext(ctx)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, new objects_1.PluginStateObject.Volume.Representation3D({ repr: repr, source: a }, { label: Math.round(channel.isoValue.relativeValue * 100) / 100 + " \u03C3 [" + srcParams.channel + "]" })];
                }
            });
        }); });
    },
    update: function (_a, plugin) {
        var a = _a.a, b = _a.b, oldParams = _a.oldParams, newParams = _a.newParams;
        return mol_task_1.Task.create('Volume Representation', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var channel, params, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        channel = a.data.channels[newParams.channel];
                        // TODO: is this correct behavior?
                        if (!channel)
                            return [2 /*return*/, mol_state_1.StateTransformer.UpdateResult.Unchanged];
                        params = createVolumeProps(a.data, newParams.channel);
                        props = __assign({}, b.data.repr.props, params.type.params);
                        b.data.repr.setTheme(theme_1.createTheme(plugin.volumeRepresentation.themeCtx, { volume: channel.data }, params));
                        return [4 /*yield*/, b.data.repr.createOrUpdate(props, channel.data).runInContext(ctx)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, mol_state_1.StateTransformer.UpdateResult.Updated];
                }
            });
        }); });
    }
});
exports.VolumeStreamingVisual = VolumeStreamingVisual;
function createVolumeProps(streaming, channelName) {
    var channel = streaming.channels[channelName];
    return representation_1.VolumeRepresentation3DHelpers.getDefaultParamsStatic(streaming.plugin, 'isosurface', { isoValue: channel.isoValue, alpha: channel.opacity, visuals: channel.wireframe ? ['wireframe'] : ['solid'] }, 'uniform', { value: channel.color });
}
