"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
function getStreamingMethod(s, defaultKind) {
    if (defaultKind === void 0) { defaultKind = 'x-ray'; }
    if (!s)
        return defaultKind;
    var model = s.models[0];
    if (model.sourceData.kind !== 'mmCIF')
        return defaultKind;
    var data = model.sourceData.data.exptl.method;
    for (var i = 0; i < data.rowCount; i++) {
        var v = data.value(i).toUpperCase();
        if (v.indexOf('MICROSCOPY') >= 0)
            return 'em';
    }
    return 'x-ray';
}
exports.getStreamingMethod = getStreamingMethod;
function getEmdbIdAndContourLevel(plugin, taskCtx, pdbId) {
    return __awaiter(this, void 0, void 0, function () {
        var summary, summaryEntry, emdbId, emdb_1, emdb, emdbEntry, contour;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, plugin.fetch({ url: "https://www.ebi.ac.uk/pdbe/api/pdb/entry/summary/" + pdbId, type: 'json' }).runInContext(taskCtx)];
                case 1:
                    summary = _a.sent();
                    summaryEntry = summary && summary[pdbId];
                    if (summaryEntry && summaryEntry[0] && summaryEntry[0].related_structures) {
                        emdb_1 = summaryEntry[0].related_structures.filter(function (s) { return s.resource === 'EMDB'; });
                        if (!emdb_1.length) {
                            throw new Error("No related EMDB entry found for '" + pdbId + "'.");
                        }
                        emdbId = emdb_1[0].accession;
                    }
                    else {
                        throw new Error("No related EMDB entry found for '" + pdbId + "'.");
                    }
                    return [4 /*yield*/, plugin.fetch({ url: "https://www.ebi.ac.uk/pdbe/api/emdb/entry/map/" + emdbId, type: 'json' }).runInContext(taskCtx)];
                case 2:
                    emdb = _a.sent();
                    emdbEntry = emdb && emdb[emdbId];
                    contour = void 0;
                    if (emdbEntry && emdbEntry[0] && emdbEntry[0].map && emdbEntry[0].map.contour_level && emdbEntry[0].map.contour_level.value !== void 0) {
                        contour = +emdbEntry[0].map.contour_level.value;
                    }
                    return [2 /*return*/, { emdbId: emdbId, contour: contour }];
            }
        });
    });
}
exports.getEmdbIdAndContourLevel = getEmdbIdAndContourLevel;
