"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var objects_1 = require("../../state/objects");
function registerDefault(ctx) {
    SyncRepresentationToCanvas(ctx);
    SyncStructureRepresentation3DState(ctx); // should be AFTER SyncRepresentationToCanvas
    UpdateRepresentationVisibility(ctx);
}
exports.registerDefault = registerDefault;
function SyncRepresentationToCanvas(ctx) {
    var reprCount = 0;
    var events = ctx.state.dataState.events;
    events.object.created.subscribe(function (e) {
        if (!objects_1.PluginStateObject.isRepresentation3D(e.obj))
            return;
        updateVisibility(e.state.cells.get(e.ref), e.obj.data.repr);
        e.obj.data.repr.setState({ syncManually: true });
        ctx.canvas3d.add(e.obj.data.repr);
        if (reprCount === 0)
            ctx.canvas3d.resetCamera();
        reprCount++;
    });
    events.object.updated.subscribe(function (e) {
        if (e.oldObj && objects_1.PluginStateObject.isRepresentation3D(e.oldObj)) {
            ctx.canvas3d.remove(e.oldObj.data.repr);
            ctx.canvas3d.requestDraw(true);
            e.oldObj.data.repr.destroy();
        }
        if (!objects_1.PluginStateObject.isRepresentation3D(e.obj)) {
            return;
        }
        updateVisibility(e.state.cells.get(e.ref), e.obj.data.repr);
        if (e.action === 'recreate') {
            e.obj.data.repr.setState({ syncManually: true });
        }
        ctx.canvas3d.add(e.obj.data.repr);
    });
    events.object.removed.subscribe(function (e) {
        if (!objects_1.PluginStateObject.isRepresentation3D(e.obj))
            return;
        ctx.canvas3d.remove(e.obj.data.repr);
        ctx.canvas3d.requestDraw(true);
        e.obj.data.repr.destroy();
        reprCount--;
    });
}
exports.SyncRepresentationToCanvas = SyncRepresentationToCanvas;
function SyncStructureRepresentation3DState(ctx) {
    // TODO: figure out how to do transform composition here?
    var events = ctx.state.dataState.events;
    events.object.created.subscribe(function (e) {
        if (!objects_1.PluginStateObject.Molecule.Structure.Representation3DState.is(e.obj))
            return;
        var data = e.obj.data;
        data.source.data.repr.setState(data.state);
        ctx.canvas3d.update(data.source.data.repr);
        ctx.canvas3d.requestDraw(true);
    });
    events.object.updated.subscribe(function (e) {
        if (!objects_1.PluginStateObject.Molecule.Structure.Representation3DState.is(e.obj))
            return;
        var data = e.obj.data;
        data.source.data.repr.setState(data.state);
        ctx.canvas3d.update(data.source.data.repr);
        ctx.canvas3d.requestDraw(true);
    });
    events.object.removed.subscribe(function (e) {
        if (!objects_1.PluginStateObject.Molecule.Structure.Representation3DState.is(e.obj))
            return;
        var data = e.obj.data;
        data.source.data.repr.setState(data.initialState);
        ctx.canvas3d.update(data.source.data.repr);
        ctx.canvas3d.requestDraw(true);
    });
}
exports.SyncStructureRepresentation3DState = SyncStructureRepresentation3DState;
function UpdateRepresentationVisibility(ctx) {
    ctx.state.dataState.events.cell.stateUpdated.subscribe(function (e) {
        var cell = e.state.cells.get(e.ref);
        if (!objects_1.PluginStateObject.isRepresentation3D(cell.obj))
            return;
        updateVisibility(cell, cell.obj.data.repr);
        ctx.canvas3d.requestDraw(true);
    });
}
exports.UpdateRepresentationVisibility = UpdateRepresentationVisibility;
function updateVisibility(cell, r) {
    r.setState({ visible: !cell.state.isHidden });
}
