"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var command_1 = require("../../command");
var mol_state_1 = require("../../../mol-state");
var snapshots_1 = require("../../../mol-plugin/state/snapshots");
var objects_1 = require("../../state/objects");
var date_1 = require("../../../mol-util/date");
var data_source_1 = require("../../../mol-util/data-source");
function registerDefault(ctx) {
    SyncBehaviors(ctx);
    SetCurrentObject(ctx);
    Update(ctx);
    ApplyAction(ctx);
    RemoveObject(ctx);
    ToggleExpanded(ctx);
    ToggleVisibility(ctx);
    Highlight(ctx);
    ClearHighlight(ctx);
    Snapshots(ctx);
}
exports.registerDefault = registerDefault;
function SyncBehaviors(ctx) {
    ctx.events.state.object.created.subscribe(function (o) {
        if (!objects_1.PluginStateObject.isBehavior(o.obj))
            return;
        o.obj.data.register(o.ref);
    });
    ctx.events.state.object.removed.subscribe(function (o) {
        if (!objects_1.PluginStateObject.isBehavior(o.obj))
            return;
        o.obj.data.unregister();
    });
    ctx.events.state.object.updated.subscribe(function (o) {
        if (o.action === 'recreate') {
            if (o.oldObj && objects_1.PluginStateObject.isBehavior(o.oldObj))
                o.oldObj.data.unregister();
            if (o.obj && objects_1.PluginStateObject.isBehavior(o.obj))
                o.obj.data.register(o.ref);
        }
    });
}
exports.SyncBehaviors = SyncBehaviors;
function SetCurrentObject(ctx) {
    command_1.PluginCommands.State.SetCurrentObject.subscribe(ctx, function (_a) {
        var state = _a.state, ref = _a.ref;
        return state.setCurrent(ref);
    });
}
exports.SetCurrentObject = SetCurrentObject;
function Update(ctx) {
    command_1.PluginCommands.State.Update.subscribe(ctx, function (_a) {
        var state = _a.state, tree = _a.tree, options = _a.options;
        return ctx.runTask(state.updateTree(tree, options));
    });
}
exports.Update = Update;
function ApplyAction(ctx) {
    command_1.PluginCommands.State.ApplyAction.subscribe(ctx, function (_a) {
        var state = _a.state, action = _a.action, ref = _a.ref;
        return ctx.runTask(state.applyAction(action.action, action.params, ref));
    });
}
exports.ApplyAction = ApplyAction;
function RemoveObject(ctx) {
    function remove(state, ref) {
        var tree = state.build().delete(ref).getTree();
        return ctx.runTask(state.updateTree(tree));
    }
    command_1.PluginCommands.State.RemoveObject.subscribe(ctx, function (_a) {
        var state = _a.state, ref = _a.ref, removeParentGhosts = _a.removeParentGhosts;
        if (removeParentGhosts) {
            var tree = state.tree;
            var curr = tree.transforms.get(ref);
            if (curr.parent === ref)
                return remove(state, ref);
            while (true) {
                var children = tree.children.get(curr.parent);
                if (curr.parent === curr.ref || children.size > 1)
                    return remove(state, curr.ref);
                var parent_1 = tree.transforms.get(curr.parent);
                // TODO: should this use "cell state" instead?
                if (!parent_1.state.isGhost)
                    return remove(state, curr.ref);
                curr = parent_1;
            }
        }
        else {
            return remove(state, ref);
        }
    });
}
exports.RemoveObject = RemoveObject;
function ToggleExpanded(ctx) {
    command_1.PluginCommands.State.ToggleExpanded.subscribe(ctx, function (_a) {
        var state = _a.state, ref = _a.ref;
        return state.updateCellState(ref, function (_a) {
            var isCollapsed = _a.isCollapsed;
            return ({ isCollapsed: !isCollapsed });
        });
    });
}
exports.ToggleExpanded = ToggleExpanded;
function ToggleVisibility(ctx) {
    command_1.PluginCommands.State.ToggleVisibility.subscribe(ctx, function (_a) {
        var state = _a.state, ref = _a.ref;
        return setVisibility(state, ref, !state.cells.get(ref).state.isHidden);
    });
}
exports.ToggleVisibility = ToggleVisibility;
function setVisibility(state, root, value) {
    mol_state_1.StateTree.doPreOrder(state.tree, state.transforms.get(root), { state: state, value: value }, setVisibilityVisitor);
}
function setVisibilityVisitor(t, tree, ctx) {
    ctx.state.updateCellState(t.ref, { isHidden: ctx.value });
}
// TODO make isHighlighted and isSelect part of StateObjectCell.State and subscribe from there???
// TODO select structures of subtree
// TODO should also work for volumes and shapes
function Highlight(ctx) {
    command_1.PluginCommands.State.Highlight.subscribe(ctx, function (_a) {
        var state = _a.state, ref = _a.ref;
        // const cell = state.select(ref)[0]
        // const repr = cell && SO.isRepresentation3D(cell.obj) ? cell.obj.data : undefined
        // if (cell && cell.obj && cell.obj.type === PluginStateObject.Molecule.Structure.type) {
        //     ctx.behaviors.canvas3d.highlight.next({ current: { loci: Structure.Loci(cell.obj.data) } });
        // } else if (repr) {
        //     ctx.behaviors.canvas3d.highlight.next({ current: { loci: EveryLoci, repr } });
        // }
    });
}
exports.Highlight = Highlight;
function ClearHighlight(ctx) {
    command_1.PluginCommands.State.ClearHighlight.subscribe(ctx, function (_a) {
        var state = _a.state, ref = _a.ref;
        // ctx.behaviors.canvas3d.highlight.next({ current: { loci: EmptyLoci } });
    });
}
exports.ClearHighlight = ClearHighlight;
function Snapshots(ctx) {
    var _this = this;
    command_1.PluginCommands.State.Snapshots.Clear.subscribe(ctx, function () {
        ctx.state.snapshots.clear();
    });
    command_1.PluginCommands.State.Snapshots.Remove.subscribe(ctx, function (_a) {
        var id = _a.id;
        ctx.state.snapshots.remove(id);
    });
    command_1.PluginCommands.State.Snapshots.Add.subscribe(ctx, function (_a) {
        var name = _a.name, description = _a.description, params = _a.params;
        var entry = snapshots_1.PluginStateSnapshotManager.Entry(ctx.state.getSnapshot(params), { name: name, description: description });
        ctx.state.snapshots.add(entry);
    });
    command_1.PluginCommands.State.Snapshots.Replace.subscribe(ctx, function (_a) {
        var id = _a.id, params = _a.params;
        ctx.state.snapshots.replace(id, ctx.state.getSnapshot(params));
    });
    command_1.PluginCommands.State.Snapshots.Move.subscribe(ctx, function (_a) {
        var id = _a.id, dir = _a.dir;
        ctx.state.snapshots.move(id, dir);
    });
    command_1.PluginCommands.State.Snapshots.Apply.subscribe(ctx, function (_a) {
        var id = _a.id;
        var snapshot = ctx.state.snapshots.setCurrent(id);
        if (!snapshot)
            return;
        return ctx.state.setSnapshot(snapshot);
    });
    command_1.PluginCommands.State.Snapshots.Upload.subscribe(ctx, function (_a) {
        var name = _a.name, description = _a.description, playOnLoad = _a.playOnLoad, serverUrl = _a.serverUrl;
        return fetch(serverUrl + "/set?name=" + encodeURIComponent(name || '') + "&description=" + encodeURIComponent(description || ''), {
            method: 'POST',
            mode: 'cors',
            referrer: 'no-referrer',
            headers: { 'Content-Type': 'application/json; charset=utf-8' },
            body: JSON.stringify(ctx.state.snapshots.getRemoteSnapshot({ name: name, description: description, playOnLoad: playOnLoad }))
        });
    });
    command_1.PluginCommands.State.Snapshots.Fetch.subscribe(ctx, function (_a) {
        var url = _a.url;
        return __awaiter(_this, void 0, void 0, function () {
            var json;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, ctx.runTask(ctx.fetch({ url: url, type: 'json' }))];
                    case 1:
                        json = _b.sent();
                        return [4 /*yield*/, ctx.state.snapshots.setRemoteSnapshot(json.data)];
                    case 2:
                        _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    });
    command_1.PluginCommands.State.Snapshots.DownloadToFile.subscribe(ctx, function (_a) {
        var name = _a.name;
        var element = document.createElement('a');
        var json = JSON.stringify(ctx.state.getSnapshot(), null, 2);
        element.setAttribute('href', 'data:application/json;charset=utf-8,' + encodeURIComponent(json));
        element.setAttribute('download', "mol-star_state_" + (name || date_1.getFormattedTime()) + ".json");
        element.style.display = 'none';
        document.body.appendChild(element);
        element.click();
        document.body.removeChild(element);
    });
    command_1.PluginCommands.State.Snapshots.OpenFile.subscribe(ctx, function (_a) {
        var file = _a.file;
        return __awaiter(_this, void 0, void 0, function () {
            var data, snapshot, e_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _b.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, data_source_1.readFromFile(file, 'string').run()];
                    case 1:
                        data = _b.sent();
                        snapshot = JSON.parse(data);
                        return [2 /*return*/, ctx.state.setSnapshot(snapshot)];
                    case 2:
                        e_1 = _b.sent();
                        ctx.log.error("Reading JSON state: " + e_1);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        });
    });
}
exports.Snapshots = Snapshots;
