/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { Camera } from '../mol-canvas3d/camera';
import { PluginCommand } from './command/base';
import { State, StateAction } from '../mol-state';
import { StructureElement } from '../mol-model/structure';
export * from './command/base';
export declare const PluginCommands: {
    State: {
        SetCurrentObject: PluginCommand<{
            state: State;
            ref: string;
        }>;
        ApplyAction: PluginCommand<{
            state: State;
            action: StateAction.Instance;
            ref?: string | undefined;
        }>;
        Update: PluginCommand<{
            state: State;
            tree: import("../mol-state/tree/immutable").StateTree | import("../mol-state/state/builder").StateBuilder;
            options?: Partial<State.UpdateOptions> | undefined;
        }>;
        RemoveObject: PluginCommand<{
            state: State;
            ref: string;
            removeParentGhosts?: boolean | undefined;
        }>;
        ToggleExpanded: PluginCommand<{
            state: State;
            ref: string;
        }>;
        ToggleVisibility: PluginCommand<{
            state: State;
            ref: string;
        }>;
        Highlight: PluginCommand<{
            state: State;
            ref: string;
        }>;
        ClearHighlight: PluginCommand<{
            state: State;
            ref: string;
        }>;
        Snapshots: {
            Add: PluginCommand<{
                name?: string | undefined;
                description?: string | undefined;
                params?: Partial<import("../mol-util/param-definition").ParamDefinition.Values<{
                    durationInMs: import("../mol-util/param-definition").ParamDefinition.Numeric;
                    data: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    behavior: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    animation: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    startAnimation: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    canvas3d: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    interactivity: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    camera: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    cameraSnapshots: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    cameraTranstion: import("../mol-util/param-definition").ParamDefinition.Mapped<import("../mol-util/param-definition").ParamDefinition.NamedParams<import("../mol-util/param-definition").ParamDefinition.Normalize<{
                        durationInMs: number;
                    }>, "animate"> | import("../mol-util/param-definition").ParamDefinition.NamedParams<import("../mol-util/param-definition").ParamDefinition.Normalize<unknown>, "instant">>;
                }>> | undefined;
            }>;
            Replace: PluginCommand<{
                id: string;
                params?: Partial<import("../mol-util/param-definition").ParamDefinition.Values<{
                    durationInMs: import("../mol-util/param-definition").ParamDefinition.Numeric;
                    data: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    behavior: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    animation: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    startAnimation: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    canvas3d: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    interactivity: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    camera: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    cameraSnapshots: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                    cameraTranstion: import("../mol-util/param-definition").ParamDefinition.Mapped<import("../mol-util/param-definition").ParamDefinition.NamedParams<import("../mol-util/param-definition").ParamDefinition.Normalize<{
                        durationInMs: number;
                    }>, "animate"> | import("../mol-util/param-definition").ParamDefinition.NamedParams<import("../mol-util/param-definition").ParamDefinition.Normalize<unknown>, "instant">>;
                }>> | undefined;
            }>;
            Move: PluginCommand<{
                id: string;
                dir: 1 | -1;
            }>;
            Remove: PluginCommand<{
                id: string;
            }>;
            Apply: PluginCommand<{
                id: string;
            }>;
            Clear: PluginCommand<{}>;
            Upload: PluginCommand<{
                name?: string | undefined;
                description?: string | undefined;
                playOnLoad?: boolean | undefined;
                serverUrl: string;
            }>;
            Fetch: PluginCommand<{
                url: string;
            }>;
            DownloadToFile: PluginCommand<{
                name?: string | undefined;
            }>;
            OpenFile: PluginCommand<{
                file: File;
            }>;
        };
    };
    Interactivity: {
        SetProps: PluginCommand<{
            props: Partial<import("../mol-util/param-definition").ParamDefinition.Values<{
                granularity: import("../mol-util/param-definition").ParamDefinition.Select<"residue" | "chain" | "element" | "structure">;
            }>>;
        }>;
        Structure: {
            Highlight: PluginCommand<{
                loci: StructureElement.Loci;
                isOff?: boolean | undefined;
            }>;
            Select: PluginCommand<{
                loci: StructureElement.Loci;
                isOff?: boolean | undefined;
            }>;
        };
    };
    Layout: {
        Update: PluginCommand<{
            state: Partial<import("../mol-util/param-definition").ParamDefinition.Values<{
                isExpanded: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
                showControls: import("../mol-util/param-definition").ParamDefinition.BooleanParam;
            }>>;
        }>;
    };
    Camera: {
        Reset: PluginCommand<{}>;
        SetSnapshot: PluginCommand<{
            snapshot: Partial<Camera.Snapshot>;
            durationMs?: number | undefined;
        }>;
        Snapshots: {
            Add: PluginCommand<{
                name?: string | undefined;
                description?: string | undefined;
            }>;
            Remove: PluginCommand<{
                id: string;
            }>;
            Apply: PluginCommand<{
                id: string;
            }>;
            Clear: PluginCommand<{}>;
        };
    };
    Canvas3D: {
        SetSettings: PluginCommand<{
            settings: Partial<import("../mol-util/param-definition").ParamDefinition.Values<{
                cameraMode: import("../mol-util/param-definition").ParamDefinition.Select<Camera.Mode>;
                cameraClipDistance: import("../mol-util/param-definition").ParamDefinition.Numeric;
                clip: import("../mol-util/param-definition").ParamDefinition.Interval;
                fog: import("../mol-util/param-definition").ParamDefinition.Interval;
                multiSample: import("../mol-util/param-definition").ParamDefinition.Group<import("../mol-util/param-definition").ParamDefinition.Normalize<{
                    mode: "on" | "off" | "temporal";
                    sampleLevel: number;
                }>>;
                postprocessing: import("../mol-util/param-definition").ParamDefinition.Group<import("../mol-util/param-definition").ParamDefinition.Normalize<{
                    occlusionEnable: boolean;
                    occlusionKernelSize: number;
                    occlusionBias: number;
                    occlusionRadius: number;
                    outlineEnable: boolean;
                    outlineScale: number;
                    outlineThreshold: number;
                    useFog: boolean;
                }>>;
                renderer: import("../mol-util/param-definition").ParamDefinition.Group<import("../mol-util/param-definition").ParamDefinition.Normalize<{
                    backgroundColor: import("../mol-util/color").Color;
                    pickingAlphaThreshold: number;
                    lightIntensity: number;
                    ambientIntensity: number;
                    metalness: number;
                    roughness: number;
                    reflectivity: number;
                }>>;
                trackball: import("../mol-util/param-definition").ParamDefinition.Group<import("../mol-util/param-definition").ParamDefinition.Normalize<{
                    noScroll: boolean;
                    rotateSpeed: number;
                    zoomSpeed: number;
                    panSpeed: number;
                    spin: boolean;
                    spinSpeed: number;
                    staticMoving: boolean;
                    dynamicDampingFactor: number;
                    minDistance: number;
                    maxDistance: number;
                }>>;
                debug: import("../mol-util/param-definition").ParamDefinition.Group<import("../mol-util/param-definition").ParamDefinition.Normalize<{
                    sceneBoundingSpheres: boolean;
                    objectBoundingSpheres: boolean;
                    instanceBoundingSpheres: boolean;
                }>>;
            }>>;
        }>;
    };
};
