/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { PluginContext } from '../context';
import { UUID } from '../../mol-util';
export { PluginCommand };
interface PluginCommand<T = unknown> {
    readonly id: UUID;
    dispatch(ctx: PluginContext, params: T): Promise<void>;
    subscribe(ctx: PluginContext, action: PluginCommand.Action<T>): PluginCommand.Subscription;
}
/** namespace.id must a globally unique identifier */
declare function PluginCommand<T>(): PluginCommand<T>;
declare namespace PluginCommand {
    type Id = string & {
        '@type': 'plugin-command-id';
    };
    interface Subscription {
        unsubscribe(): void;
    }
    type Action<T> = (params: T) => unknown | Promise<unknown>;
    class Manager {
        private subs;
        private disposing;
        subscribe<T>(cmd: PluginCommand<T>, action: Action<T>): Subscription;
        /** Resolves after all actions have completed */
        dispatch<T>(cmd: PluginCommand<T>, params: T): Promise<void>;
        dispose(): void;
        private resolve;
    }
}
