"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../mol-util");
/** namespace.id must a globally unique identifier */
function PluginCommand() {
    return new Impl();
}
exports.PluginCommand = PluginCommand;
var Impl = /** @class */ (function () {
    function Impl() {
        this.id = mol_util_1.UUID.create22();
    }
    Impl.prototype.dispatch = function (ctx, params) {
        return ctx.commands.dispatch(this, params);
    };
    Impl.prototype.subscribe = function (ctx, action) {
        return ctx.commands.subscribe(this, action);
    };
    return Impl;
}());
(function (PluginCommand) {
    var Manager = /** @class */ (function () {
        function Manager() {
            this.subs = new Map();
            this.disposing = false;
        }
        Manager.prototype.subscribe = function (cmd, action) {
            var _this = this;
            var actions = this.subs.get(cmd.id);
            if (!actions) {
                actions = [];
                this.subs.set(cmd.id, actions);
            }
            actions.push(action);
            return {
                unsubscribe: function () {
                    var actions = _this.subs.get(cmd.id);
                    if (!actions)
                        return;
                    var idx = actions.indexOf(action);
                    if (idx < 0)
                        return;
                    for (var i = idx + 1; i < actions.length; i++) {
                        actions[i - 1] = actions[i];
                    }
                    actions.pop();
                }
            };
        };
        /** Resolves after all actions have completed */
        Manager.prototype.dispatch = function (cmd, params) {
            var _this = this;
            return new Promise(function (resolve, reject) {
                if (_this.disposing) {
                    reject('disposed');
                    return;
                }
                var actions = _this.subs.get(cmd.id);
                if (!actions) {
                    resolve();
                    return;
                }
                _this.resolve({ cmd: cmd, params: params, resolve: resolve, reject: reject });
            });
        };
        Manager.prototype.dispose = function () {
            this.subs.clear();
        };
        Manager.prototype.resolve = function (instance) {
            return __awaiter(this, void 0, void 0, function () {
                var actions, _i, actions_1, a, e_1;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            actions = this.subs.get(instance.cmd.id);
                            if (!actions) {
                                instance.resolve();
                                return [2 /*return*/];
                            }
                            _a.label = 1;
                        case 1:
                            _a.trys.push([1, 6, , 7]);
                            _i = 0, actions_1 = actions;
                            _a.label = 2;
                        case 2:
                            if (!(_i < actions_1.length)) return [3 /*break*/, 5];
                            a = actions_1[_i];
                            return [4 /*yield*/, a(instance.params)];
                        case 3:
                            _a.sent();
                            _a.label = 4;
                        case 4:
                            _i++;
                            return [3 /*break*/, 2];
                        case 5:
                            instance.resolve();
                            return [3 /*break*/, 7];
                        case 6:
                            e_1 = _a.sent();
                            instance.reject(e_1);
                            return [3 /*break*/, 7];
                        case 7: return [2 /*return*/];
                    }
                });
            });
        };
        return Manager;
    }());
    PluginCommand.Manager = Manager;
})(PluginCommand || (PluginCommand = {}));
exports.PluginCommand = PluginCommand;
