"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var canvas3d_1 = require("../mol-canvas3d/canvas3d");
var custom_property_registry_1 = require("../mol-model-props/common/custom-property-registry");
var registry_1 = require("../mol-repr/structure/registry");
var registry_2 = require("../mol-repr/volume/registry");
var color_1 = require("../mol-theme/color");
var size_1 = require("../mol-theme/size");
var color_2 = require("../mol-util/color");
var data_source_1 = require("../mol-util/data-source");
var log_entry_1 = require("../mol-util/log-entry");
var rx_event_helper_1 = require("../mol-util/rx-event-helper");
var rxjs_1 = require("rxjs");
var behavior_1 = require("./behavior");
var behavior_2 = require("./behavior/behavior");
var command_1 = require("./command");
var layout_1 = require("./layout");
var state_1 = require("./state");
var data_format_1 = require("./state/actions/data-format");
var loci_label_manager_1 = require("./util/loci-label-manager");
var task_manager_1 = require("./util/task-manager");
var version_1 = require("./version");
var structure_element_selection_1 = require("./util/structure-element-selection");
var substructure_parent_helper_1 = require("./util/substructure-parent-helper");
var input_observer_1 = require("../mol-util/input/input-observer");
var debug_1 = require("../mol-util/debug");
var interactivity_1 = require("./util/interactivity");
var PluginContext = /** @class */ (function () {
    function PluginContext(spec) {
        var _this = this;
        this.spec = spec;
        this.disposed = false;
        this.ev = rx_event_helper_1.RxEventHelper.create();
        this.tasks = new task_manager_1.TaskManager();
        this.state = new state_1.PluginState(this);
        this.commands = new command_1.PluginCommand.Manager();
        this.events = {
            state: {
                cell: {
                    stateUpdated: rxjs_1.merge(this.state.dataState.events.cell.stateUpdated, this.state.behaviorState.events.cell.stateUpdated),
                    created: rxjs_1.merge(this.state.dataState.events.cell.created, this.state.behaviorState.events.cell.created),
                    removed: rxjs_1.merge(this.state.dataState.events.cell.removed, this.state.behaviorState.events.cell.removed),
                },
                object: {
                    created: rxjs_1.merge(this.state.dataState.events.object.created, this.state.behaviorState.events.object.created),
                    removed: rxjs_1.merge(this.state.dataState.events.object.removed, this.state.behaviorState.events.object.removed),
                    updated: rxjs_1.merge(this.state.dataState.events.object.updated, this.state.behaviorState.events.object.updated)
                },
                cameraSnapshots: this.state.cameraSnapshots.events,
                snapshots: this.state.snapshots.events,
            },
            log: this.ev(),
            task: this.tasks.events,
            canvas3d: {
                settingsUpdated: this.ev()
            },
            interactivity: {
                propsUpdated: this.ev()
            }
        };
        this.behaviors = {
            state: {
                isAnimating: this.ev.behavior(false),
                isUpdating: this.ev.behavior(false)
            },
            interaction: {
                highlight: this.ev.behavior({ current: interactivity_1.Interactivity.Loci.Empty }),
                click: this.ev.behavior({ current: interactivity_1.Interactivity.Loci.Empty, modifiers: input_observer_1.ModifiersKeys.None, buttons: 0 })
            },
            labels: {
                highlight: this.ev.behavior({ entries: [] })
            }
        };
        this.layout = new layout_1.PluginLayout(this);
        this.structureRepresentation = {
            registry: new registry_1.StructureRepresentationRegistry(),
            themeCtx: { colorThemeRegistry: color_1.ColorTheme.createRegistry(), sizeThemeRegistry: size_1.SizeTheme.createRegistry() }
        };
        this.volumeRepresentation = {
            registry: new registry_2.VolumeRepresentationRegistry(),
            themeCtx: { colorThemeRegistry: color_1.ColorTheme.createRegistry(), sizeThemeRegistry: size_1.SizeTheme.createRegistry() }
        };
        this.dataFormat = {
            registry: new data_format_1.DataFormatRegistry()
        };
        this.customModelProperties = new custom_property_registry_1.CustomPropertyRegistry();
        this.customStructureProperties = new custom_property_registry_1.CustomPropertyRegistry();
        this.customParamEditors = new Map();
        this.helpers = {
            structureSelection: new structure_element_selection_1.StructureElementSelectionManager(this),
            substructureParent: new substructure_parent_helper_1.SubstructureParentHelper(this)
        };
        this.log = {
            entries: immutable_1.List(),
            entry: function (e) { return _this.events.log.next(e); },
            error: function (msg) { return _this.events.log.next(log_entry_1.LogEntry.error(msg)); },
            message: function (msg) { return _this.events.log.next(log_entry_1.LogEntry.message(msg)); },
            info: function (msg) { return _this.events.log.next(log_entry_1.LogEntry.info(msg)); },
            warn: function (msg) { return _this.events.log.next(log_entry_1.LogEntry.warning(msg)); },
        };
        /**
         * This should be used in all transform related request so that it could be "spoofed" to allow
         * "static" access to resources.
         */
        this.fetch = data_source_1.ajaxGet;
        this.events.log.subscribe(function (e) { return _this.log.entries = _this.log.entries.push(e); });
        this.initBehaviorEvents();
        this.initBuiltInBehavior();
        this.initBehaviors();
        this.initDataActions();
        this.initAnimations();
        this.initCustomParamEditors();
        this.interactivity = new interactivity_1.Interactivity(this);
        this.lociLabels = new loci_label_manager_1.LociLabelManager(this);
        this.log.message("Mol* Plugin " + version_1.PLUGIN_VERSION + " [" + version_1.PLUGIN_VERSION_DATE.toLocaleString() + "]");
        if (!debug_1.isProductionMode)
            this.log.message("Development mode enabled");
        if (debug_1.isDebugMode)
            this.log.message("Debug mode enabled");
    }
    PluginContext.prototype.initViewer = function (canvas, container) {
        try {
            this.layout.setRoot(container);
            if (this.spec.layout && this.spec.layout.initial)
                this.layout.setProps(this.spec.layout.initial);
            this.canvas3d = canvas3d_1.Canvas3D.fromCanvas(canvas);
            var renderer = this.canvas3d.props.renderer;
            command_1.PluginCommands.Canvas3D.SetSettings.dispatch(this, { settings: { renderer: __assign({}, renderer, { backgroundColor: color_2.Color(0xFCFBF9) }) } });
            this.canvas3d.animate();
            return true;
        }
        catch (e) {
            this.log.error('' + e);
            console.error(e);
            return false;
        }
    };
    PluginContext.prototype.runTask = function (task) {
        return this.tasks.run(task);
    };
    PluginContext.prototype.dispose = function () {
        if (this.disposed)
            return;
        this.commands.dispose();
        this.canvas3d.dispose();
        this.ev.dispose();
        this.state.dispose();
        this.tasks.dispose();
        this.layout.dispose();
        this.disposed = true;
    };
    PluginContext.prototype.applyTransform = function (state, a, transformer, params) {
        var tree = state.build().to(a).apply(transformer, params);
        return command_1.PluginCommands.State.Update.dispatch(this, { state: state, tree: tree });
    };
    PluginContext.prototype.updateTransform = function (state, a, params) {
        var tree = state.build().to(a).update(params);
        return command_1.PluginCommands.State.Update.dispatch(this, { state: state, tree: tree });
    };
    PluginContext.prototype.initBehaviorEvents = function () {
        var _this = this;
        rxjs_1.merge(this.state.dataState.events.isUpdating, this.state.behaviorState.events.isUpdating).subscribe(function (u) {
            _this.behaviors.state.isUpdating.next(u);
        });
    };
    PluginContext.prototype.initBuiltInBehavior = function () {
        var _this = this;
        behavior_1.BuiltInPluginBehaviors.State.registerDefault(this);
        behavior_1.BuiltInPluginBehaviors.Representation.registerDefault(this);
        behavior_1.BuiltInPluginBehaviors.Camera.registerDefault(this);
        behavior_1.BuiltInPluginBehaviors.Misc.registerDefault(this);
        rxjs_1.merge(this.state.dataState.events.log, this.state.behaviorState.events.log).subscribe(function (e) { return _this.events.log.next(e); });
    };
    PluginContext.prototype.initBehaviors = function () {
        return __awaiter(this, void 0, void 0, function () {
            var tree, _i, _a, cat, _b, _c, b;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        tree = this.state.behaviorState.build();
                        for (_i = 0, _a = Object.keys(behavior_2.PluginBehavior.Categories); _i < _a.length; _i++) {
                            cat = _a[_i];
                            tree.toRoot().apply(behavior_2.PluginBehavior.CreateCategory, { label: behavior_2.PluginBehavior.Categories[cat] }, { ref: cat, state: { isLocked: true } });
                        }
                        for (_b = 0, _c = this.spec.behaviors; _b < _c.length; _b++) {
                            b = _c[_b];
                            tree.to(behavior_2.PluginBehavior.getCategoryId(b.transformer)).apply(b.transformer, b.defaultParams, { ref: b.transformer.id });
                        }
                        return [4 /*yield*/, this.runTask(this.state.behaviorState.updateTree(tree, { doNotUpdateCurrent: true, doNotLogTiming: true }))];
                    case 1:
                        _d.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    PluginContext.prototype.initDataActions = function () {
        for (var _i = 0, _a = this.spec.actions; _i < _a.length; _i++) {
            var a = _a[_i];
            this.state.dataState.actions.add(a.action);
        }
    };
    PluginContext.prototype.initAnimations = function () {
        if (!this.spec.animations)
            return;
        for (var _i = 0, _a = this.spec.animations; _i < _a.length; _i++) {
            var anim = _a[_i];
            this.state.animation.register(anim);
        }
    };
    PluginContext.prototype.initCustomParamEditors = function () {
        if (!this.spec.customParamEditors)
            return;
        for (var _i = 0, _a = this.spec.customParamEditors; _i < _a.length; _i++) {
            var _b = _a[_i], t = _b[0], e = _b[1];
            this.customParamEditors.set(t.id, e);
        }
    };
    return PluginContext;
}());
exports.PluginContext = PluginContext;
