"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var context_1 = require("./context");
var plugin_1 = require("./ui/plugin");
var React = require("react");
var ReactDOM = require("react-dom");
var spec_1 = require("./spec");
var transforms_1 = require("./state/transforms");
var behavior_1 = require("./behavior");
var built_in_1 = require("./state/animation/built-in");
var actions_1 = require("./state/actions");
var transformers_1 = require("./behavior/dynamic/volume-streaming/transformers");
var structure_representation_interaction_1 = require("./behavior/dynamic/selection/structure-representation-interaction");
var structure_1 = require("./state/actions/structure");
var volume_1 = require("./ui/custom/volume");
exports.DefaultPluginSpec = {
    actions: [
        spec_1.PluginSpec.Action(actions_1.StateActions.Structure.DownloadStructure),
        spec_1.PluginSpec.Action(actions_1.StateActions.Volume.DownloadDensity),
        spec_1.PluginSpec.Action(actions_1.StateActions.DataFormat.OpenFile),
        spec_1.PluginSpec.Action(actions_1.StateActions.Structure.CreateComplexRepresentation),
        spec_1.PluginSpec.Action(actions_1.StateActions.Structure.EnableModelCustomProps),
        spec_1.PluginSpec.Action(actions_1.StateActions.Structure.EnableStructureCustomProps),
        // Volume streaming
        spec_1.PluginSpec.Action(transformers_1.InitVolumeStreaming),
        spec_1.PluginSpec.Action(transformers_1.BoxifyVolumeStreaming),
        spec_1.PluginSpec.Action(transformers_1.CreateVolumeStreamingBehavior),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Data.Download),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Data.ParseCif),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Data.ParseCcp4),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Data.ParseDsn6),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.TrajectoryFromMmCif),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.TrajectoryFromPDB),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.StructureAssemblyFromModel),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.StructureSymmetryFromModel),
        spec_1.PluginSpec.Action(structure_1.TransformStructureConformation),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.StructureFromModel),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.ModelFromTrajectory),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Model.UserStructureSelection),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.StructureRepresentation3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.StructureLabels3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.ExplodeStructureRepresentation3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.UnwindStructureAssemblyRepresentation3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.OverpaintStructureRepresentation3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.TransparencyStructureRepresentation3D),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Volume.VolumeFromCcp4),
        spec_1.PluginSpec.Action(transforms_1.StateTransforms.Representation.VolumeRepresentation3D),
        spec_1.PluginSpec.Action(actions_1.StateActions.Structure.StructureFromSelection),
    ],
    behaviors: [
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.Representation.HighlightLoci),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.Representation.SelectLoci),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.Representation.DefaultLociLabelProvider),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.Camera.FocusLociOnSelect, { minRadius: 8, extraRadius: 4 }),
        // PluginSpec.Behavior(PluginBehaviors.Labels.SceneLabels),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.CustomProps.MolstarSecondaryStructure, { autoAttach: true }),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.CustomProps.PDBeStructureQualityReport, { autoAttach: true }),
        spec_1.PluginSpec.Behavior(behavior_1.PluginBehaviors.CustomProps.RCSBAssemblySymmetry, { autoAttach: true }),
        spec_1.PluginSpec.Behavior(structure_representation_interaction_1.StructureRepresentationInteraction)
    ],
    customParamEditors: [
        [transformers_1.CreateVolumeStreamingBehavior, volume_1.VolumeStreamingCustomControls]
    ],
    animations: [
        built_in_1.AnimateModelIndex,
        built_in_1.AnimateAssemblyUnwind,
        built_in_1.AnimateUnitsExplode,
        built_in_1.AnimateStateInterpolation
    ]
};
function createPlugin(target, spec) {
    var ctx = new context_1.PluginContext(spec || exports.DefaultPluginSpec);
    ReactDOM.render(React.createElement(plugin_1.Plugin, { plugin: ctx }), target);
    return ctx;
}
exports.createPlugin = createPlugin;
