/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
/// <reference types="react" />
import { StateTransformer, StateAction } from '../mol-state';
import { StateTransformParameters } from './ui/state/common';
import { PluginLayoutStateProps } from './layout';
import { PluginStateAnimation } from './state/animation/model';
export { PluginSpec };
interface PluginSpec {
    actions: PluginSpec.Action[];
    behaviors: PluginSpec.Behavior[];
    animations?: PluginStateAnimation[];
    customParamEditors?: [StateAction | StateTransformer, StateTransformParameters.Class][];
    layout?: {
        initial?: Partial<PluginLayoutStateProps>;
        controls?: PluginSpec.LayoutControls;
    };
}
declare namespace PluginSpec {
    interface Action {
        action: StateAction | StateTransformer;
        customControl?: StateTransformParameters.Class;
        autoUpdate?: boolean;
    }
    function Action(action: StateAction | StateTransformer, params?: {
        customControl?: StateTransformParameters.Class;
        autoUpdate?: boolean;
    }): Action;
    interface Behavior {
        transformer: StateTransformer;
        defaultParams?: any;
    }
    function Behavior<T extends StateTransformer>(transformer: T, defaultParams?: StateTransformer.Params<T>): Behavior;
    interface LayoutControls {
        top?: React.ComponentClass | 'none';
        left?: React.ComponentClass | 'none';
        right?: React.ComponentClass | 'none';
        bottom?: React.ComponentClass | 'none';
    }
}
