/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { State } from '../mol-state';
import { Camera } from '../mol-canvas3d/camera';
import { CameraSnapshotManager } from './state/camera';
import { PluginStateSnapshotManager } from './state/snapshots';
import { Canvas3DProps } from '../mol-canvas3d/canvas3d';
import { PluginAnimationManager } from './state/animation/manager';
import { ParamDefinition as PD } from '../mol-util/param-definition';
import { UUID } from '../mol-util';
import { Interactivity } from './util/interactivity';
export { PluginState };
declare class PluginState {
    private plugin;
    private ev;
    readonly dataState: State;
    readonly behaviorState: State;
    readonly animation: PluginAnimationManager;
    readonly cameraSnapshots: CameraSnapshotManager;
    readonly snapshots: PluginStateSnapshotManager;
    readonly behavior: {
        kind: import("rxjs").BehaviorSubject<PluginState.Kind>;
        currentObject: import("rxjs").BehaviorSubject<State.ObjectEvent>;
    };
    setKind(kind: PluginState.Kind): void;
    getSnapshot(params?: PluginState.GetSnapshotParams): PluginState.Snapshot;
    setSnapshot(snapshot: PluginState.Snapshot): Promise<void>;
    dispose(): void;
    constructor(plugin: import('./context').PluginContext);
}
declare namespace PluginState {
    type Kind = 'data' | 'behavior';
    type CameraTransitionStyle = 'instant' | 'animate';
    const GetSnapshotParams: {
        durationInMs: PD.Numeric;
        data: PD.BooleanParam;
        behavior: PD.BooleanParam;
        animation: PD.BooleanParam;
        startAnimation: PD.BooleanParam;
        canvas3d: PD.BooleanParam;
        interactivity: PD.BooleanParam;
        camera: PD.BooleanParam;
        cameraSnapshots: PD.BooleanParam;
        cameraTranstion: PD.Mapped<PD.NamedParams<PD.Normalize<{
            durationInMs: number;
        }>, "animate"> | PD.NamedParams<PD.Normalize<unknown>, "instant">>;
    };
    type GetSnapshotParams = Partial<PD.Values<typeof GetSnapshotParams>>;
    const DefaultGetSnapshotParams: PD.Values<{
        durationInMs: PD.Numeric;
        data: PD.BooleanParam;
        behavior: PD.BooleanParam;
        animation: PD.BooleanParam;
        startAnimation: PD.BooleanParam;
        canvas3d: PD.BooleanParam;
        interactivity: PD.BooleanParam;
        camera: PD.BooleanParam;
        cameraSnapshots: PD.BooleanParam;
        cameraTranstion: PD.Mapped<PD.NamedParams<PD.Normalize<{
            durationInMs: number;
        }>, "animate"> | PD.NamedParams<PD.Normalize<unknown>, "instant">>;
    }>;
    interface Snapshot {
        id: UUID;
        data?: State.Snapshot;
        behaviour?: State.Snapshot;
        animation?: PluginAnimationManager.Snapshot;
        startAnimation?: boolean;
        camera?: {
            current: Camera.Snapshot;
            transitionStyle: CameraTransitionStyle;
            transitionDurationInMs?: number;
        };
        cameraSnapshots?: CameraSnapshotManager.StateSnapshot;
        canvas3d?: {
            props?: Canvas3DProps;
        };
        interactivity?: {
            props?: Interactivity.Props;
        };
        durationInMs?: number;
    }
}
