"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var mol_state_1 = require("../mol-state");
var objects_1 = require("./state/objects");
var behavior_1 = require("./behavior");
var camera_1 = require("./state/camera");
var snapshots_1 = require("./state/snapshots");
var rx_event_helper_1 = require("../mol-util/rx-event-helper");
var command_1 = require("./command");
var manager_1 = require("./state/animation/manager");
var param_definition_1 = require("../mol-util/param-definition");
var mol_util_1 = require("../mol-util");
var PluginState = /** @class */ (function () {
    function PluginState(plugin) {
        var _this = this;
        this.plugin = plugin;
        this.ev = rx_event_helper_1.RxEventHelper.create();
        this.cameraSnapshots = new camera_1.CameraSnapshotManager();
        this.behavior = {
            kind: this.ev.behavior('data'),
            currentObject: this.ev.behavior({})
        };
        this.snapshots = new snapshots_1.PluginStateSnapshotManager(plugin);
        this.dataState = mol_state_1.State.create(new objects_1.PluginStateObject.Root({}), { globalContext: plugin });
        this.behaviorState = mol_state_1.State.create(new behavior_1.PluginBehavior.Root({}), { globalContext: plugin, rootState: { isLocked: true } });
        this.dataState.behaviors.currentObject.subscribe(function (o) {
            if (_this.behavior.kind.value === 'data')
                _this.behavior.currentObject.next(o);
        });
        this.behaviorState.behaviors.currentObject.subscribe(function (o) {
            if (_this.behavior.kind.value === 'behavior')
                _this.behavior.currentObject.next(o);
        });
        this.behavior.currentObject.next(this.dataState.behaviors.currentObject.value);
        this.animation = new manager_1.PluginAnimationManager(plugin);
    }
    PluginState.prototype.setKind = function (kind) {
        var current = this.behavior.kind.value;
        if (kind !== current) {
            this.behavior.kind.next(kind);
            this.behavior.currentObject.next(kind === 'data'
                ? this.dataState.behaviors.currentObject.value
                : this.behaviorState.behaviors.currentObject.value);
        }
    };
    PluginState.prototype.getSnapshot = function (params) {
        var p = __assign({}, PluginState.DefaultGetSnapshotParams, params);
        return {
            id: mol_util_1.UUID.create22(),
            data: p.data ? this.dataState.getSnapshot() : void 0,
            behaviour: p.behavior ? this.behaviorState.getSnapshot() : void 0,
            animation: p.animation ? this.animation.getSnapshot() : void 0,
            startAnimation: p.startAnimation ? !!p.startAnimation : void 0,
            camera: p.camera ? {
                current: this.plugin.canvas3d.camera.getSnapshot(),
                transitionStyle: p.cameraTranstion.name,
                transitionDurationInMs: (params && params.cameraTranstion && params.cameraTranstion.name === 'animate') ? params.cameraTranstion.params.durationInMs : undefined
            } : void 0,
            cameraSnapshots: p.cameraSnapshots ? this.cameraSnapshots.getStateSnapshot() : void 0,
            canvas3d: p.canvas3d ? { props: this.plugin.canvas3d.props } : void 0,
            interactivity: p.interactivity ? { props: this.plugin.interactivity.props } : void 0,
            durationInMs: params && params.durationInMs
        };
    };
    PluginState.prototype.setSnapshot = function (snapshot) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.animation.stop()];
                    case 1:
                        _a.sent();
                        if (!snapshot.behaviour) return [3 /*break*/, 3];
                        return [4 /*yield*/, this.plugin.runTask(this.behaviorState.setSnapshot(snapshot.behaviour))];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        if (!snapshot.data) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.plugin.runTask(this.dataState.setSnapshot(snapshot.data))];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5:
                        if (!snapshot.canvas3d) return [3 /*break*/, 7];
                        if (!snapshot.canvas3d.props) return [3 /*break*/, 7];
                        return [4 /*yield*/, command_1.PluginCommands.Canvas3D.SetSettings.dispatch(this.plugin, { settings: snapshot.canvas3d.props })];
                    case 6:
                        _a.sent();
                        _a.label = 7;
                    case 7:
                        if (!snapshot.interactivity) return [3 /*break*/, 9];
                        if (!snapshot.interactivity.props) return [3 /*break*/, 9];
                        return [4 /*yield*/, command_1.PluginCommands.Interactivity.SetProps.dispatch(this.plugin, { props: snapshot.interactivity.props })];
                    case 8:
                        _a.sent();
                        _a.label = 9;
                    case 9:
                        if (snapshot.cameraSnapshots)
                            this.cameraSnapshots.setStateSnapshot(snapshot.cameraSnapshots);
                        if (snapshot.animation) {
                            this.animation.setSnapshot(snapshot.animation);
                        }
                        if (snapshot.camera) {
                            command_1.PluginCommands.Camera.SetSnapshot.dispatch(this.plugin, {
                                snapshot: snapshot.camera.current,
                                durationMs: snapshot.camera.transitionStyle === 'animate'
                                    ? snapshot.camera.transitionDurationInMs
                                    : void 0
                            });
                        }
                        if (snapshot.startAnimation) {
                            this.animation.start();
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    PluginState.prototype.dispose = function () {
        this.ev.dispose();
        this.dataState.dispose();
        this.behaviorState.dispose();
        this.cameraSnapshots.dispose();
        this.animation.dispose();
    };
    return PluginState;
}());
exports.PluginState = PluginState;
(function (PluginState) {
    PluginState.GetSnapshotParams = {
        durationInMs: param_definition_1.ParamDefinition.Numeric(1500, { min: 100, max: 15000, step: 100 }, { label: 'Duration in ms' }),
        data: param_definition_1.ParamDefinition.Boolean(true),
        behavior: param_definition_1.ParamDefinition.Boolean(false),
        animation: param_definition_1.ParamDefinition.Boolean(true),
        startAnimation: param_definition_1.ParamDefinition.Boolean(false),
        canvas3d: param_definition_1.ParamDefinition.Boolean(true),
        interactivity: param_definition_1.ParamDefinition.Boolean(true),
        camera: param_definition_1.ParamDefinition.Boolean(true),
        // TODO: make camera snapshots same as the StateSnapshots with "child states?"
        cameraSnapshots: param_definition_1.ParamDefinition.Boolean(false),
        cameraTranstion: param_definition_1.ParamDefinition.MappedStatic('animate', {
            animate: param_definition_1.ParamDefinition.Group({
                durationInMs: param_definition_1.ParamDefinition.Numeric(250, { min: 100, max: 5000, step: 500 }, { label: 'Duration in ms' }),
            }),
            instant: param_definition_1.ParamDefinition.Group({})
        }, { options: [['animate', 'Animate'], ['instant', 'Instant']] })
    };
    PluginState.DefaultGetSnapshotParams = param_definition_1.ParamDefinition.getDefaultValues(PluginState.GetSnapshotParams);
})(PluginState || (PluginState = {}));
exports.PluginState = PluginState;
