/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { PluginContext } from '../../../mol-plugin/context';
import { State, StateBuilder, StateAction } from '../../../mol-state';
import { Task } from '../../../mol-task';
import { FileInfo } from '../../../mol-util/file-info';
import { PluginStateObject } from '../objects';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
export declare class DataFormatRegistry<D extends PluginStateObject.Data.Binary | PluginStateObject.Data.String> {
    private _list;
    private _map;
    private _extensions;
    private _binaryExtensions;
    private _options;
    readonly types: [string, string][];
    readonly extensions: Set<string>;
    readonly binaryExtensions: Set<string>;
    readonly options: [string, string][];
    constructor();
    private _clear;
    add(name: string, provider: DataFormatProvider<D>): void;
    remove(name: string): void;
    auto(info: FileInfo, dataStateObject: D): DataFormatProvider<D>;
    get(name: string): DataFormatProvider<D>;
    readonly list: {
        name: string;
        provider: DataFormatProvider<D>;
    }[];
}
export interface DataFormatProvider<D extends PluginStateObject.Data.Binary | PluginStateObject.Data.String> {
    label: string;
    description: string;
    stringExtensions: string[];
    binaryExtensions: string[];
    isApplicable(info: FileInfo, data: string | Uint8Array): boolean;
    getDefaultBuilder(ctx: PluginContext, data: StateBuilder.To<D>, state?: State): Task<void>;
}
export declare const OpenFile: StateAction<PluginStateObject.Root, void, PD.Normalize<{
    file: File;
    format: string;
}>>;
declare type cifVariants = 'dscif' | -1;
export declare function guessCifVariant(info: FileInfo, data: Uint8Array | string): cifVariants;
export {};
