"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var mol_state_1 = require("../../../mol-state");
var mol_task_1 = require("../../../mol-task");
var file_info_1 = require("../../../mol-util/file-info");
var objects_1 = require("../objects");
var param_definition_1 = require("../../../mol-util/param-definition");
var volume_1 = require("./volume");
var transforms_1 = require("../transforms");
var structure_1 = require("./structure");
var decode_1 = require("../../../mol-io/common/msgpack/decode");
var shape_1 = require("./shape");
var DataFormatRegistry = /** @class */ (function () {
    function DataFormatRegistry() {
        this._list = [];
        this._map = new Map();
        this._extensions = undefined;
        this._binaryExtensions = undefined;
        this._options = undefined;
        this.add('ccp4', volume_1.Ccp4Provider);
        this.add('dscif', volume_1.DscifProvider);
        this.add('dsn6', volume_1.Dsn6Provider);
        this.add('gro', structure_1.GroProvider);
        this.add('mmcif', structure_1.MmcifProvider);
        this.add('pdb', structure_1.PdbProvider);
        this.add('ply', shape_1.PlyProvider);
    }
    Object.defineProperty(DataFormatRegistry.prototype, "types", {
        get: function () {
            return this._list.map(function (e) { return [e.name, e.provider.label]; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataFormatRegistry.prototype, "extensions", {
        get: function () {
            if (this._extensions)
                return this._extensions;
            var extensions = new Set();
            this._list.forEach(function (_a) {
                var provider = _a.provider;
                provider.stringExtensions.forEach(function (ext) { return extensions.add(ext); });
                provider.binaryExtensions.forEach(function (ext) { return extensions.add(ext); });
            });
            this._extensions = extensions;
            return extensions;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataFormatRegistry.prototype, "binaryExtensions", {
        get: function () {
            if (this._binaryExtensions)
                return this._binaryExtensions;
            var binaryExtensions = new Set();
            this._list.forEach(function (_a) {
                var provider = _a.provider;
                return provider.binaryExtensions.forEach(function (ext) { return binaryExtensions.add(ext); });
            });
            this._binaryExtensions = binaryExtensions;
            return binaryExtensions;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DataFormatRegistry.prototype, "options", {
        get: function () {
            if (this._options)
                return this._options;
            var options = [['auto', 'Automatic']];
            this._list.forEach(function (_a) {
                var name = _a.name, provider = _a.provider;
                return options.push([name, provider.label]);
            });
            this._options = options;
            return options;
        },
        enumerable: true,
        configurable: true
    });
    ;
    DataFormatRegistry.prototype._clear = function () {
        this._extensions = undefined;
        this._binaryExtensions = undefined;
        this._options = undefined;
    };
    DataFormatRegistry.prototype.add = function (name, provider) {
        this._clear();
        this._list.push({ name: name, provider: provider });
        this._map.set(name, provider);
    };
    DataFormatRegistry.prototype.remove = function (name) {
        this._clear();
        this._list.splice(this._list.findIndex(function (e) { return e.name === name; }), 1);
        this._map.delete(name);
    };
    DataFormatRegistry.prototype.auto = function (info, dataStateObject) {
        for (var i = 0, il = this.list.length; i < il; ++i) {
            var provider = this._list[i].provider;
            if (provider.isApplicable(info, dataStateObject.data))
                return provider;
        }
        throw new Error('no compatible data format provider available');
    };
    DataFormatRegistry.prototype.get = function (name) {
        if (this._map.has(name)) {
            return this._map.get(name);
        }
        else {
            throw new Error("unknown data format name '" + name + "'");
        }
    };
    Object.defineProperty(DataFormatRegistry.prototype, "list", {
        get: function () {
            return this._list;
        },
        enumerable: true,
        configurable: true
    });
    return DataFormatRegistry;
}());
exports.DataFormatRegistry = DataFormatRegistry;
//
exports.OpenFile = mol_state_1.StateAction.build({
    display: { name: 'Open File', description: 'Load a file and create its default visual' },
    from: objects_1.PluginStateObject.Root,
    params: function (a, ctx) {
        var _a = ctx.dataFormat.registry, extensions = _a.extensions, options = _a.options;
        return {
            file: param_definition_1.ParamDefinition.File({ accept: Array.from(extensions).map(function (e) { return "." + e; }).join(',') }),
            format: param_definition_1.ParamDefinition.Select('auto', options),
        };
    }
})(function (_a, ctx) {
    var params = _a.params, state = _a.state;
    return mol_task_1.Task.create('Open File', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
        var info, data, dataStateObject, provider, b;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    info = file_info_1.getFileInfo(params.file);
                    data = state.build().toRoot().apply(transforms_1.StateTransforms.Data.ReadFile, { file: params.file, isBinary: ctx.dataFormat.registry.binaryExtensions.has(info.ext) });
                    return [4 /*yield*/, state.updateTree(data).runInContext(taskCtx)];
                case 1:
                    dataStateObject = _a.sent();
                    provider = params.format === 'auto' ? ctx.dataFormat.registry.auto(info, dataStateObject) : ctx.dataFormat.registry.get(params.format);
                    b = state.build().to(data.ref);
                    // need to await the 2nd update the so that the enclosing Task finishes after the update is done.
                    return [4 /*yield*/, provider.getDefaultBuilder(ctx, b, state).runInContext(taskCtx)];
                case 2:
                    // need to await the 2nd update the so that the enclosing Task finishes after the update is done.
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
});
function guessCifVariant(info, data) {
    if (info.ext === 'bcif') {
        try {
            // TODO find a way to run msgpackDecode only once
            //      now it is run twice, here and during file parsing
            if (decode_1.default(data).encoder.startsWith('VolumeServer'))
                return 'dscif';
        }
        catch (_a) { }
    }
    else if (info.ext === 'cif') {
        if (data.startsWith('data_SERVER\n#\n_density_server_result'))
            return 'dscif';
    }
    return -1;
}
exports.guessCifVariant = guessCifVariant;
