/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { PluginContext } from '../../../mol-plugin/context';
import { StateAction, StateBuilder, StateTransformer } from '../../../mol-state';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { PluginStateObject } from '../objects';
import { DataFormatProvider } from './data-format';
export declare const MmcifProvider: DataFormatProvider<any>;
export declare const PdbProvider: DataFormatProvider<any>;
export declare const GroProvider: DataFormatProvider<any>;
declare type StructureFormat = 'pdb' | 'cif' | 'gro';
export { DownloadStructure };
declare type DownloadStructure = typeof DownloadStructure;
declare const DownloadStructure: StateAction<PluginStateObject.Root, void, PD.Normalize<{
    source: PD.NamedParams<PD.Normalize<{
        url: any;
        format: any;
        isBinary: any;
        options: any;
    }>, "url"> | PD.NamedParams<PD.Normalize<{
        id: any;
        options: any;
    }>, "rcsb"> | PD.NamedParams<PD.Normalize<{
        id: any;
        options: any;
    }>, "bcif-static"> | PD.NamedParams<PD.Normalize<{
        id: any;
        options: any;
    }>, "pdbe-updated"> | PD.NamedParams<PD.Normalize<{
        id: any;
        options: any;
    }>, "pdb-dev"> | PD.NamedParams<PD.Normalize<{
        id: any;
        options: any;
    }>, "swissmodel">;
}>>;
export declare function createModelTree(b: StateBuilder.To<PluginStateObject.Data.Binary | PluginStateObject.Data.String>, format?: StructureFormat): StateBuilder.To<PluginStateObject.Molecule.Model, StateTransformer<import("../../../mol-state/object").StateObject<any, import("../../../mol-state/object").StateObject.Type<any>>, import("../../../mol-state/object").StateObject<any, import("../../../mol-state/object").StateObject.Type<any>>, any>>;
export declare function complexRepresentation(ctx: PluginContext, root: StateBuilder.To<PluginStateObject.Molecule.Structure>, params?: {
    hideSequence?: boolean;
    hideHET?: boolean;
    hideWater?: boolean;
    hideCoarse?: boolean;
}): void;
export declare const CreateComplexRepresentation: StateAction<PluginStateObject.Molecule.Structure, import("../../../mol-state/object").StateObject<any, import("../../../mol-state/object").StateObject.Type<any>>, PD.Normalize<{}>>;
export declare const UpdateTrajectory: StateAction<import("../../../mol-state/object").StateObject<any, import("../../../mol-state/object").StateObject.Type<any>>, void, PD.Normalize<{
    action: "reset" | "advance";
    by: number | undefined;
}>>;
export declare const EnableModelCustomProps: StateAction<PluginStateObject.Molecule.Model, PluginStateObject.Molecule.Model, PD.Normalize<{
    properties: string[];
}>>;
export declare const EnableStructureCustomProps: StateAction<PluginStateObject.Molecule.Structure, PluginStateObject.Molecule.Structure, PD.Normalize<{
    properties: string[];
}>>;
export declare const TransformStructureConformation: StateAction<PluginStateObject.Molecule.Structure, PluginStateObject.Molecule.Structure, PD.Normalize<{
    axis: any;
    angle: any;
    translation: any;
}>>;
export declare const StructureFromSelection: StateAction<PluginStateObject.Molecule.Structure, PluginStateObject.Molecule.Structure | undefined, PD.Normalize<{
    label: string;
}>>;
