"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var mol_state_1 = require("../../../mol-state");
var param_definition_1 = require("../../../mol-util/param-definition");
var objects_1 = require("../objects");
var transforms_1 = require("../transforms");
var representation_1 = require("../transforms/representation");
var model_1 = require("../transforms/model");
var data_format_1 = require("./data-format");
var mol_task_1 = require("../../../mol-task");
var structure_1 = require("../../../mol-model/structure");
exports.MmcifProvider = {
    label: 'mmCIF',
    description: 'mmCIF',
    stringExtensions: ['cif', 'mmcif', 'mcif'],
    binaryExtensions: ['bcif'],
    isApplicable: function (info, data) {
        if (info.ext === 'mmcif' || info.ext === 'mcif')
            return true;
        // assume cif/bcif files that are not DensityServer CIF are mmCIF
        if (info.ext === 'cif' || info.ext === 'bcif')
            return data_format_1.guessCifVariant(info, data) !== 'dscif';
        return false;
    },
    getDefaultBuilder: function (ctx, data, state) {
        return mol_task_1.Task.create('mmCIF default builder', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
            var traj;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        traj = createModelTree(data, 'cif');
                        return [4 /*yield*/, state.updateTree(createStructureTree(ctx, traj, false)).runInContext(taskCtx)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
    }
};
exports.PdbProvider = {
    label: 'PDB',
    description: 'PDB',
    stringExtensions: ['pdb', 'ent'],
    binaryExtensions: [],
    isApplicable: function (info, data) {
        return info.ext === 'pdb' || info.ext === 'ent';
    },
    getDefaultBuilder: function (ctx, data, state) {
        return mol_task_1.Task.create('PDB default builder', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
            var traj;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        traj = createModelTree(data, 'pdb');
                        return [4 /*yield*/, state.updateTree(createStructureTree(ctx, traj, false)).runInContext(taskCtx)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
    }
};
exports.GroProvider = {
    label: 'GRO',
    description: 'GRO',
    stringExtensions: ['gro'],
    binaryExtensions: [],
    isApplicable: function (info, data) {
        return info.ext === 'gro';
    },
    getDefaultBuilder: function (ctx, data, state) {
        return mol_task_1.Task.create('GRO default builder', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
            var traj;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        traj = createModelTree(data, 'gro');
                        return [4 /*yield*/, state.updateTree(createStructureTree(ctx, traj, false)).runInContext(taskCtx)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
    }
};
//
var DownloadStructurePdbIdSourceOptions = param_definition_1.ParamDefinition.Group({
    supportProps: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Boolean(false)),
    asTrajectory: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Boolean(false, { description: 'Load all entries into a single trajectory.' }))
});
var DownloadStructure = mol_state_1.StateAction.build({
    from: objects_1.PluginStateObject.Root,
    display: { name: 'Download Structure', description: 'Load a structure from the provided source and create its default Assembly and visual.' },
    params: {
        source: param_definition_1.ParamDefinition.MappedStatic('bcif-static', {
            'pdbe-updated': param_definition_1.ParamDefinition.Group({
                id: param_definition_1.ParamDefinition.Text('1cbs', { label: 'Id' }),
                options: DownloadStructurePdbIdSourceOptions
            }, { isFlat: true }),
            'rcsb': param_definition_1.ParamDefinition.Group({
                id: param_definition_1.ParamDefinition.Text('1tqn', { label: 'Id' }),
                options: DownloadStructurePdbIdSourceOptions
            }, { isFlat: true }),
            'pdb-dev': param_definition_1.ParamDefinition.Group({
                id: param_definition_1.ParamDefinition.Text('PDBDEV_00000001', { label: 'Id' }),
                options: DownloadStructurePdbIdSourceOptions
            }, { isFlat: true }),
            'bcif-static': param_definition_1.ParamDefinition.Group({
                id: param_definition_1.ParamDefinition.Text('1tqn', { label: 'Id' }),
                options: DownloadStructurePdbIdSourceOptions
            }, { isFlat: true }),
            'swissmodel': param_definition_1.ParamDefinition.Group({
                id: param_definition_1.ParamDefinition.Text('Q9Y2I8', { label: 'UniProtKB AC' }),
                options: DownloadStructurePdbIdSourceOptions
            }, { isFlat: true, description: 'Loads the best homology model or experimental structure' }),
            'url': param_definition_1.ParamDefinition.Group({
                url: param_definition_1.ParamDefinition.Text(''),
                format: param_definition_1.ParamDefinition.Select('cif', [['cif', 'CIF'], ['pdb', 'PDB']]),
                isBinary: param_definition_1.ParamDefinition.Boolean(false),
                options: param_definition_1.ParamDefinition.Group({
                    supportProps: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Boolean(false))
                })
            }, { isFlat: true })
        }, {
            options: [
                ['pdbe-updated', 'PDBe Updated'],
                ['rcsb', 'RCSB'],
                ['pdb-dev', 'PDBDEV'],
                ['bcif-static', 'BinaryCIF (static PDBe Updated)'],
                ['swissmodel', 'SWISS-MODEL'],
                ['url', 'URL']
            ]
        })
    }
})(function (_a, ctx) {
    var params = _a.params, state = _a.state;
    var b = state.build();
    var src = params.source;
    var downloadParams;
    var supportProps = false, asTrajectory = false, format = 'cif';
    switch (src.name) {
        case 'url':
            downloadParams = [{ url: src.params.url, isBinary: src.params.isBinary }];
            supportProps = !!src.params.options.supportProps;
            format = src.params.format;
            break;
        case 'pdbe-updated':
            downloadParams = getDownloadParams(src.params.id, function (id) { return "https://www.ebi.ac.uk/pdbe/static/entry/" + id.toLowerCase() + "_updated.cif"; }, function (id) { return "PDBe: " + id; }, false);
            supportProps = !!src.params.options.supportProps;
            asTrajectory = !!src.params.options.asTrajectory;
            break;
        case 'rcsb':
            downloadParams = getDownloadParams(src.params.id, function (id) { return "https://files.rcsb.org/download/" + id.toUpperCase() + ".cif"; }, function (id) { return "RCSB: " + id; }, false);
            supportProps = !!src.params.options.supportProps;
            asTrajectory = !!src.params.options.asTrajectory;
            break;
        case 'pdb-dev':
            downloadParams = getDownloadParams(src.params.id, function (id) {
                var nId = id.toUpperCase().startsWith('PDBDEV_') ? id : "PDBDEV_" + id.padStart(8, '0');
                return "https://pdb-dev.wwpdb.org/static/cif/" + nId.toUpperCase() + ".cif";
            }, function (id) { return id.toUpperCase().startsWith('PDBDEV_') ? id : "PDBDEV_" + id.padStart(8, '0'); }, false);
            supportProps = !!src.params.options.supportProps;
            asTrajectory = !!src.params.options.asTrajectory;
            break;
        case 'bcif-static':
            downloadParams = getDownloadParams(src.params.id, function (id) { return "https://webchem.ncbr.muni.cz/ModelServer/static/bcif/" + id.toLowerCase(); }, function (id) { return "BinaryCIF: " + id; }, true);
            supportProps = !!src.params.options.supportProps;
            asTrajectory = !!src.params.options.asTrajectory;
            break;
        case 'swissmodel':
            downloadParams = getDownloadParams(src.params.id, function (id) { return "https://swissmodel.expasy.org/repository/uniprot/" + id.toUpperCase() + ".pdb"; }, function (id) { return "SWISS-MODEL: " + id; }, false);
            supportProps = !!src.params.options.supportProps;
            asTrajectory = !!src.params.options.asTrajectory;
            format = 'pdb';
            break;
        default: throw new Error(src.name + " not supported.");
    }
    if (downloadParams.length > 0 && asTrajectory) {
        var traj = createSingleTrajectoryModel(downloadParams, b);
        createStructureTree(ctx, traj, supportProps);
    }
    else {
        for (var _i = 0, downloadParams_1 = downloadParams; _i < downloadParams_1.length; _i++) {
            var download = downloadParams_1[_i];
            var data = b.toRoot().apply(transforms_1.StateTransforms.Data.Download, download, { state: { isGhost: true } });
            var traj = createModelTree(data, format);
            createStructureTree(ctx, traj, supportProps);
        }
    }
    return state.updateTree(b);
});
exports.DownloadStructure = DownloadStructure;
function getDownloadParams(src, url, label, isBinary) {
    var ids = src.split(',').map(function (id) { return id.trim(); }).filter(function (id) { return !!id && (id.length >= 4 || /^[1-9][0-9]*$/.test(id)); });
    var ret = [];
    for (var _i = 0, ids_1 = ids; _i < ids_1.length; _i++) {
        var id = ids_1[_i];
        ret.push({ url: url(id), isBinary: isBinary, label: label(id) });
    }
    return ret;
}
function createSingleTrajectoryModel(sources, b) {
    return b.toRoot()
        .apply(transforms_1.StateTransforms.Data.DownloadBlob, {
        sources: sources.map(function (src, i) { return ({ id: '' + i, url: src.url, isBinary: src.isBinary }); }),
        maxConcurrency: 6
    }).apply(transforms_1.StateTransforms.Data.ParseBlob, {
        formats: sources.map(function (_, i) { return ({ id: '' + i, format: 'cif' }); })
    })
        .apply(transforms_1.StateTransforms.Model.TrajectoryFromBlob)
        .apply(transforms_1.StateTransforms.Model.ModelFromTrajectory, { modelIndex: 0 });
}
function createModelTree(b, format) {
    if (format === void 0) { format = 'cif'; }
    var parsed;
    switch (format) {
        case 'cif':
            parsed = b.apply(transforms_1.StateTransforms.Data.ParseCif, void 0, { state: { isGhost: true } })
                .apply(transforms_1.StateTransforms.Model.TrajectoryFromMmCif, void 0, { state: { isGhost: true } });
            break;
        case 'pdb':
            parsed = b.apply(transforms_1.StateTransforms.Model.TrajectoryFromPDB, void 0, { state: { isGhost: true } });
            break;
        case 'gro':
            parsed = b.apply(transforms_1.StateTransforms.Model.TrajectoryFromGRO, void 0, { state: { isGhost: true } });
            break;
        default:
            throw new Error('unsupported format');
    }
    return parsed.apply(transforms_1.StateTransforms.Model.ModelFromTrajectory, { modelIndex: 0 });
}
exports.createModelTree = createModelTree;
function createStructureTree(ctx, b, supportProps) {
    var root = b;
    if (supportProps) {
        root = root.apply(transforms_1.StateTransforms.Model.CustomModelProperties);
    }
    var structure = root.apply(transforms_1.StateTransforms.Model.StructureAssemblyFromModel);
    complexRepresentation(ctx, structure);
    return root;
}
function complexRepresentation(ctx, root, params) {
    if (!params || !params.hideSequence) {
        root.apply(transforms_1.StateTransforms.Model.StructureComplexElement, { type: 'atomic-sequence' })
            .apply(transforms_1.StateTransforms.Representation.StructureRepresentation3D, representation_1.StructureRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'cartoon'));
    }
    if (!params || !params.hideHET) {
        root.apply(transforms_1.StateTransforms.Model.StructureComplexElement, { type: 'atomic-het' })
            .apply(transforms_1.StateTransforms.Representation.StructureRepresentation3D, representation_1.StructureRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'ball-and-stick'));
    }
    if (!params || !params.hideWater) {
        root.apply(transforms_1.StateTransforms.Model.StructureComplexElement, { type: 'water' })
            .apply(transforms_1.StateTransforms.Representation.StructureRepresentation3D, representation_1.StructureRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'ball-and-stick', { alpha: 0.51 }));
    }
    if (!params || !params.hideCoarse) {
        root.apply(transforms_1.StateTransforms.Model.StructureComplexElement, { type: 'spheres' })
            .apply(transforms_1.StateTransforms.Representation.StructureRepresentation3D, representation_1.StructureRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'spacefill', {}, 'polymer-id'));
    }
}
exports.complexRepresentation = complexRepresentation;
exports.CreateComplexRepresentation = mol_state_1.StateAction.build({
    display: { name: 'Create Complex', description: 'Split the structure into Sequence/Water/Ligands/... ' },
    from: objects_1.PluginStateObject.Molecule.Structure
})(function (_a, ctx) {
    var ref = _a.ref, state = _a.state;
    var root = state.build().to(ref);
    complexRepresentation(ctx, root);
    return state.updateTree(root);
});
exports.UpdateTrajectory = mol_state_1.StateAction.build({
    display: { name: 'Update Trajectory' },
    params: {
        action: param_definition_1.ParamDefinition.Select('advance', [['advance', 'Advance'], ['reset', 'Reset']]),
        by: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Numeric(1, { min: -1, max: 1, step: 1 }))
    }
})(function (_a) {
    var params = _a.params, state = _a.state;
    var models = state.selectQ(function (q) { return q.ofTransformer(transforms_1.StateTransforms.Model.ModelFromTrajectory); });
    var update = state.build();
    if (params.action === 'reset') {
        for (var _i = 0, models_1 = models; _i < models_1.length; _i++) {
            var m = models_1[_i];
            update.to(m).update({ modelIndex: 0 });
        }
    }
    else {
        var _loop_1 = function (m) {
            var parent_1 = mol_state_1.StateSelection.findAncestorOfType(state.tree, state.cells, m.transform.ref, [objects_1.PluginStateObject.Molecule.Trajectory]);
            if (!parent_1 || !parent_1.obj)
                return "continue";
            var traj = parent_1.obj;
            update.to(m).update(function (old) {
                var modelIndex = (old.modelIndex + params.by) % traj.data.length;
                if (modelIndex < 0)
                    modelIndex += traj.data.length;
                return { modelIndex: modelIndex };
            });
        };
        for (var _b = 0, models_2 = models; _b < models_2.length; _b++) {
            var m = models_2[_b];
            _loop_1(m);
        }
    }
    return state.updateTree(update);
});
exports.EnableModelCustomProps = mol_state_1.StateAction.build({
    display: { name: 'Custom Model Properties', description: 'Enable the addition of custom properties to the model.' },
    from: objects_1.PluginStateObject.Molecule.Model,
    params: function (a, ctx) {
        if (!a)
            return { properties: param_definition_1.ParamDefinition.MultiSelect([], [], { description: 'A list of model property descriptor ids.' }) };
        return { properties: ctx.customModelProperties.getSelect(a.data) };
    },
    isApplicable: function (a, t, ctx) {
        return t.transformer !== model_1.CustomModelProperties;
    }
})(function (_a, ctx) {
    var ref = _a.ref, params = _a.params, state = _a.state;
    var root = state.build().to(ref).insert(model_1.CustomModelProperties, params);
    return state.updateTree(root);
});
exports.EnableStructureCustomProps = mol_state_1.StateAction.build({
    display: { name: 'Custom Structure Properties', description: 'Enable the addition of custom properties to the structure.' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    params: function (a, ctx) {
        if (!a)
            return { properties: param_definition_1.ParamDefinition.MultiSelect([], [], { description: 'A list of structure property descriptor ids.' }) };
        return { properties: ctx.customStructureProperties.getSelect(a.data) };
    },
    isApplicable: function (a, t, ctx) {
        return t.transformer !== model_1.CustomStructureProperties;
    }
})(function (_a, ctx) {
    var ref = _a.ref, params = _a.params, state = _a.state;
    var root = state.build().to(ref).insert(model_1.CustomStructureProperties, params);
    return state.updateTree(root);
});
exports.TransformStructureConformation = mol_state_1.StateAction.build({
    display: { name: 'Transform Conformation' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    params: transforms_1.StateTransforms.Model.TransformStructureConformation.definition.params,
})(function (_a) {
    var ref = _a.ref, params = _a.params, state = _a.state;
    var root = state.build().to(ref).insert(transforms_1.StateTransforms.Model.TransformStructureConformation, params);
    return state.updateTree(root);
});
exports.StructureFromSelection = mol_state_1.StateAction.build({
    display: { name: 'Selection Structure', description: 'Create a new Structure from the current selection.' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    params: {
        label: param_definition_1.ParamDefinition.Text()
    }
    // isApplicable(a, t, ctx: PluginContext) {
    //     return t.transformer !== CustomModelProperties;
    // }
})(function (_a, plugin) {
    var a = _a.a, ref = _a.ref, params = _a.params, state = _a.state;
    var sel = plugin.helpers.structureSelection.get(a.data);
    if (sel.kind === 'empty-loci')
        return mol_task_1.Task.constant('', void 0);
    var query = structure_1.StructureElement.Loci.toScriptExpression(sel);
    var root = state.build().to(ref).apply(model_1.StructureSelection, { query: query, label: params.label });
    return state.updateTree(root);
});
