"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var volume_1 = require("../../../mol-model/volume");
var mol_state_1 = require("../../../mol-state");
var mol_task_1 = require("../../../mol-task");
var tables_1 = require("../../../mol-util/color/tables");
var file_info_1 = require("../../../mol-util/file-info");
var param_definition_1 = require("../../../mol-util/param-definition");
var objects_1 = require("../objects");
var transforms_1 = require("../transforms");
var representation_1 = require("../transforms/representation");
var data_format_1 = require("./data-format");
exports.Ccp4Provider = {
    label: 'CCP4/MRC/BRIX',
    description: 'CCP4/MRC/BRIX',
    stringExtensions: [],
    binaryExtensions: ['ccp4', 'mrc', 'map'],
    isApplicable: function (info, data) {
        return info.ext === 'ccp4' || info.ext === 'mrc' || info.ext === 'map';
    },
    getDefaultBuilder: function (ctx, data, state) {
        return mol_task_1.Task.create('CCP4/MRC/BRIX default builder', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
            var tree;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tree = data.apply(transforms_1.StateTransforms.Data.ParseCcp4)
                            .apply(transforms_1.StateTransforms.Volume.VolumeFromCcp4)
                            .apply(transforms_1.StateTransforms.Representation.VolumeRepresentation3D);
                        return [4 /*yield*/, state.updateTree(tree).runInContext(taskCtx)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
    }
};
exports.Dsn6Provider = {
    label: 'DSN6/BRIX',
    description: 'DSN6/BRIX',
    stringExtensions: [],
    binaryExtensions: ['dsn6', 'brix'],
    isApplicable: function (info, data) {
        return info.ext === 'dsn6' || info.ext === 'brix';
    },
    getDefaultBuilder: function (ctx, data, state) {
        return mol_task_1.Task.create('DSN6/BRIX default builder', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
            var tree;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tree = data.apply(transforms_1.StateTransforms.Data.ParseDsn6)
                            .apply(transforms_1.StateTransforms.Volume.VolumeFromDsn6)
                            .apply(transforms_1.StateTransforms.Representation.VolumeRepresentation3D);
                        return [4 /*yield*/, state.updateTree(tree).runInContext(taskCtx)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
    }
};
exports.DscifProvider = {
    label: 'DensityServer CIF',
    description: 'DensityServer CIF',
    stringExtensions: ['cif'],
    binaryExtensions: ['bcif'],
    isApplicable: function (info, data) {
        return data_format_1.guessCifVariant(info, data) === 'dscif' ? true : false;
    },
    getDefaultBuilder: function (ctx, data, state) {
        return mol_task_1.Task.create('DensityServer CIF default builder', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
            var cifBuilder, cifStateObject, b, blocks, tree, vol, posParams, negParams;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        cifBuilder = data.apply(transforms_1.StateTransforms.Data.ParseCif);
                        return [4 /*yield*/, state.updateTree(cifBuilder).runInContext(taskCtx)];
                    case 1:
                        cifStateObject = _a.sent();
                        b = state.build().to(cifBuilder.ref);
                        blocks = cifStateObject.data.blocks.slice(1);
                        if (blocks.length === 1) {
                            tree = b
                                .apply(transforms_1.StateTransforms.Volume.VolumeFromDensityServerCif, { blockHeader: blocks[0].header })
                                .apply(transforms_1.StateTransforms.Representation.VolumeRepresentation3D, representation_1.VolumeRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'isosurface', { isoValue: volume_1.VolumeIsoValue.relative(1.5), alpha: 0.3 }, 'uniform', { value: tables_1.ColorNames.teal }));
                        }
                        else if (blocks.length === 2) {
                            tree = b
                                .apply(transforms_1.StateTransforms.Volume.VolumeFromDensityServerCif, { blockHeader: blocks[0].header })
                                .apply(transforms_1.StateTransforms.Representation.VolumeRepresentation3D, representation_1.VolumeRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'isosurface', { isoValue: volume_1.VolumeIsoValue.relative(1.5), alpha: 0.3 }, 'uniform', { value: tables_1.ColorNames.blue }));
                            vol = tree.to(cifBuilder.ref)
                                .apply(transforms_1.StateTransforms.Volume.VolumeFromDensityServerCif, { blockHeader: blocks[1].header });
                            posParams = representation_1.VolumeRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'isosurface', { isoValue: volume_1.VolumeIsoValue.relative(3), alpha: 0.3 }, 'uniform', { value: tables_1.ColorNames.green });
                            tree = vol.apply(transforms_1.StateTransforms.Representation.VolumeRepresentation3D, posParams);
                            negParams = representation_1.VolumeRepresentation3DHelpers.getDefaultParamsStatic(ctx, 'isosurface', { isoValue: volume_1.VolumeIsoValue.relative(-3), alpha: 0.3 }, 'uniform', { value: tables_1.ColorNames.red });
                            tree = tree.to(vol.ref).apply(transforms_1.StateTransforms.Representation.VolumeRepresentation3D, negParams);
                        }
                        else {
                            throw new Error('unknown number of blocks');
                        }
                        return [4 /*yield*/, state.updateTree(tree).runInContext(taskCtx)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
    }
};
var DownloadDensity = mol_state_1.StateAction.build({
    from: objects_1.PluginStateObject.Root,
    display: { name: 'Download Density', description: 'Load a density from the provided source and create its default visual.' },
    params: function (a, ctx) {
        var options = ctx.dataFormat.registry.options;
        return {
            source: param_definition_1.ParamDefinition.MappedStatic('rcsb', {
                'pdbe': param_definition_1.ParamDefinition.Group({
                    id: param_definition_1.ParamDefinition.Text('1tqn', { label: 'Id' }),
                    type: param_definition_1.ParamDefinition.Select('2fofc', [['2fofc', '2Fo-Fc'], ['fofc', 'Fo-Fc']]),
                }, { isFlat: true }),
                'pdbe-emd-ds': param_definition_1.ParamDefinition.Group({
                    id: param_definition_1.ParamDefinition.Text('emd-8004', { label: 'Id' }),
                    detail: param_definition_1.ParamDefinition.Numeric(3, { min: 0, max: 10, step: 1 }, { label: 'Detail' }),
                }, { isFlat: true }),
                'pdbe-xray-ds': param_definition_1.ParamDefinition.Group({
                    id: param_definition_1.ParamDefinition.Text('1tqn', { label: 'Id' }),
                    detail: param_definition_1.ParamDefinition.Numeric(3, { min: 0, max: 10, step: 1 }, { label: 'Detail' }),
                }, { isFlat: true }),
                'rcsb': param_definition_1.ParamDefinition.Group({
                    id: param_definition_1.ParamDefinition.Text('1tqn', { label: 'Id' }),
                    type: param_definition_1.ParamDefinition.Select('2fofc', [['2fofc', '2Fo-Fc'], ['fofc', 'Fo-Fc']]),
                }, { isFlat: true }),
                'url': param_definition_1.ParamDefinition.Group({
                    url: param_definition_1.ParamDefinition.Text(''),
                    isBinary: param_definition_1.ParamDefinition.Boolean(false),
                    format: param_definition_1.ParamDefinition.Select('auto', options),
                }, { isFlat: true })
            }, {
                options: [
                    ['pdbe', 'PDBe X-ray maps'],
                    ['pdbe-emd-ds', 'PDBe EMD Density Server'],
                    ['pdbe-xray-ds', 'PDBe X-ray Density Server'],
                    ['rcsb', 'RCSB X-ray maps'],
                    ['url', 'URL']
                ]
            })
        };
    }
})(function (_a, ctx) {
    var params = _a.params, state = _a.state;
    return mol_task_1.Task.create('Download Density', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
        var src, downloadParams, provider, data, dataStateObject, b;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    src = params.source;
                    switch (src.name) {
                        case 'url':
                            downloadParams = src.params;
                            break;
                        case 'pdbe':
                            downloadParams = {
                                url: src.params.type === '2fofc'
                                    ? "http://www.ebi.ac.uk/pdbe/coordinates/files/" + src.params.id.toLowerCase() + ".ccp4"
                                    : "http://www.ebi.ac.uk/pdbe/coordinates/files/" + src.params.id.toLowerCase() + "_diff.ccp4",
                                isBinary: true,
                                label: "PDBe X-ray map: " + src.params.id
                            };
                            break;
                        case 'pdbe-emd-ds':
                            downloadParams = {
                                url: "https://www.ebi.ac.uk/pdbe/densities/emd/" + src.params.id.toLowerCase() + "/cell?detail=" + src.params.detail,
                                isBinary: true,
                                label: "PDBe EMD Density Server: " + src.params.id
                            };
                            break;
                        case 'pdbe-xray-ds':
                            downloadParams = {
                                url: "https://www.ebi.ac.uk/pdbe/densities/x-ray/" + src.params.id.toLowerCase() + "/cell?detail=" + src.params.detail,
                                isBinary: true,
                                label: "PDBe X-ray Density Server: " + src.params.id
                            };
                            break;
                        case 'rcsb':
                            downloadParams = {
                                url: src.params.type === '2fofc'
                                    ? "https://edmaps.rcsb.org/maps/" + src.params.id.toLowerCase() + "_2fofc.dsn6"
                                    : "https://edmaps.rcsb.org/maps/" + src.params.id.toLowerCase() + "_fofc.dsn6",
                                isBinary: true,
                                label: "RCSB X-ray map: " + src.params.id
                            };
                            break;
                        default: throw new Error(src.name + " not supported.");
                    }
                    data = state.build().toRoot().apply(transforms_1.StateTransforms.Data.Download, downloadParams);
                    return [4 /*yield*/, state.updateTree(data).runInContext(taskCtx)];
                case 1:
                    dataStateObject = _a.sent();
                    switch (src.name) {
                        case 'url':
                            downloadParams = src.params;
                            provider = src.params.format === 'auto' ? ctx.dataFormat.registry.auto(file_info_1.getFileInfo(downloadParams.url), dataStateObject) : ctx.dataFormat.registry.get(src.params.format);
                            break;
                        case 'pdbe':
                            provider = ctx.dataFormat.registry.get('ccp4');
                            break;
                        case 'pdbe-emd-ds':
                        case 'pdbe-xray-ds':
                            provider = ctx.dataFormat.registry.get('dscif');
                            break;
                        case 'rcsb':
                            provider = ctx.dataFormat.registry.get('dsn6');
                            break;
                        default: throw new Error(src.name + " not supported.");
                    }
                    b = state.build().to(data.ref);
                    return [4 /*yield*/, provider.getDefaultBuilder(ctx, b, state).runInContext(taskCtx)];
                case 2:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    }); });
});
exports.DownloadDensity = DownloadDensity;
