"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var model_1 = require("./model");
var objects_1 = require("../objects");
var transforms_1 = require("../transforms");
var mol_state_1 = require("../../../mol-state");
var command_1 = require("../../../mol-plugin/command");
var param_definition_1 = require("../../../mol-util/param-definition");
exports.AnimateModelIndex = model_1.PluginStateAnimation.create({
    name: 'built-in.animate-model-index',
    display: { name: 'Animate Trajectory' },
    params: function () { return ({
        mode: param_definition_1.ParamDefinition.MappedStatic('palindrome', {
            palindrome: param_definition_1.ParamDefinition.Group({}),
            loop: param_definition_1.ParamDefinition.Group({}),
            once: param_definition_1.ParamDefinition.Group({ direction: param_definition_1.ParamDefinition.Select('forward', [['forward', 'Forward'], ['backward', 'Backward']]) }, { isFlat: true })
        }, { options: [['palindrome', 'Palindrome'], ['loop', 'Loop'], ['once', 'Once']] }),
        maxFPS: param_definition_1.ParamDefinition.Numeric(15, { min: 1, max: 30, step: 1 })
    }); },
    initialState: function () { return ({}); },
    apply: function (animState, t, ctx) {
        return __awaiter(this, void 0, void 0, function () {
            var state, models, update, params, palindromeDirections, isEnd, allSingles, _loop_1, _i, models_1, m;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        // limit fps
                        if (t.current > 0 && t.current - t.lastApplied < 1000 / ctx.params.maxFPS) {
                            return [2 /*return*/, { kind: 'skip' }];
                        }
                        state = ctx.plugin.state.dataState;
                        models = state.select(mol_state_1.StateSelection.Generators.ofTransformer(transforms_1.StateTransforms.Model.ModelFromTrajectory));
                        if (models.length === 0) {
                            // nothing more to do here
                            return [2 /*return*/, { kind: 'finished' }];
                        }
                        update = state.build();
                        params = ctx.params;
                        palindromeDirections = animState.palindromeDirections || {};
                        isEnd = false, allSingles = true;
                        _loop_1 = function (m) {
                            var parent_1 = mol_state_1.StateSelection.findAncestorOfType(state.tree, state.cells, m.transform.ref, [objects_1.PluginStateObject.Molecule.Trajectory]);
                            if (!parent_1 || !parent_1.obj)
                                return "continue";
                            var traj = parent_1.obj;
                            update.to(m).update(function (old) {
                                var len = traj.data.length;
                                if (len !== 1) {
                                    allSingles = false;
                                }
                                else {
                                    return old;
                                }
                                var dir = 1;
                                if (params.mode.name === 'once') {
                                    dir = params.mode.params.direction === 'backward' ? -1 : 1;
                                    // if we are at start or end already, do nothing.
                                    if ((dir === -1 && old.modelIndex === 0) || (dir === 1 && old.modelIndex === len - 1)) {
                                        isEnd = true;
                                        return old;
                                    }
                                }
                                else if (params.mode.name === 'palindrome') {
                                    if (old.modelIndex === 0)
                                        dir = 1;
                                    else if (old.modelIndex === len - 1)
                                        dir = -1;
                                    else
                                        dir = palindromeDirections[m.transform.ref] || 1;
                                }
                                palindromeDirections[m.transform.ref] = dir;
                                var modelIndex = (old.modelIndex + dir) % len;
                                if (modelIndex < 0)
                                    modelIndex += len;
                                isEnd = isEnd || (dir === -1 && modelIndex === 0) || (dir === 1 && modelIndex === len - 1);
                                return { modelIndex: modelIndex };
                            });
                        };
                        for (_i = 0, models_1 = models; _i < models_1.length; _i++) {
                            m = models_1[_i];
                            _loop_1(m);
                        }
                        return [4 /*yield*/, command_1.PluginCommands.State.Update.dispatch(ctx.plugin, { state: state, tree: update, options: { doNotLogTiming: true } })];
                    case 1:
                        _a.sent();
                        if (allSingles || (params.mode.name === 'once' && isEnd))
                            return [2 /*return*/, { kind: 'finished' }];
                        if (params.mode.name === 'palindrome')
                            return [2 /*return*/, { kind: 'next', state: { palindromeDirections: palindromeDirections } }];
                        return [2 /*return*/, { kind: 'next', state: {} }];
                }
            });
        });
    }
});
exports.AnimateAssemblyUnwind = model_1.PluginStateAnimation.create({
    name: 'built-in.animate-assembly-unwind',
    display: { name: 'Unwind Assembly' },
    params: function (plugin) {
        var targets = [['all', 'All']];
        var structures = plugin.state.dataState.select(mol_state_1.StateSelection.Generators.rootsOfType(objects_1.PluginStateObject.Molecule.Structure));
        for (var _i = 0, structures_1 = structures; _i < structures_1.length; _i++) {
            var s = structures_1[_i];
            targets.push([s.transform.ref, s.obj.data.models[0].label]);
        }
        return {
            durationInMs: param_definition_1.ParamDefinition.Numeric(3000, { min: 100, max: 10000, step: 100 }),
            playOnce: param_definition_1.ParamDefinition.Boolean(false),
            target: param_definition_1.ParamDefinition.Select(targets[0][0], targets)
        };
    },
    initialState: function () { return ({ t: 0 }); },
    setup: function (params, plugin) {
        return __awaiter(this, void 0, void 0, function () {
            var state, root, reprs, update, changed, _i, reprs_1, r, unwinds;
            return __generator(this, function (_a) {
                state = plugin.state.dataState;
                root = !params.target || params.target === 'all' ? mol_state_1.StateTransform.RootRef : params.target;
                reprs = state.select(mol_state_1.StateSelection.Generators.ofType(objects_1.PluginStateObject.Molecule.Structure.Representation3D, root));
                update = state.build();
                changed = false;
                for (_i = 0, reprs_1 = reprs; _i < reprs_1.length; _i++) {
                    r = reprs_1[_i];
                    // TODO: find a better way to handle this, perhaps add a different state object??
                    if (r.transform.transformer === transforms_1.StateTransforms.Representation.StructureLabels3D)
                        continue;
                    unwinds = state.select(mol_state_1.StateSelection.Generators.ofTransformer(transforms_1.StateTransforms.Representation.UnwindStructureAssemblyRepresentation3D, r.transform.ref));
                    if (unwinds.length > 0)
                        continue;
                    changed = true;
                    update.to(r)
                        .apply(transforms_1.StateTransforms.Representation.UnwindStructureAssemblyRepresentation3D, { t: 0 }, { tags: 'animate-assembly-unwind' });
                }
                if (!changed)
                    return [2 /*return*/];
                return [2 /*return*/, plugin.runTask(state.updateTree(update, { doNotUpdateCurrent: true }))];
            });
        });
    },
    teardown: function (_, plugin) {
        return __awaiter(this, void 0, void 0, function () {
            var state, reprs, update, _i, reprs_2, r;
            return __generator(this, function (_a) {
                state = plugin.state.dataState;
                reprs = state.select(mol_state_1.StateSelection.Generators.ofType(objects_1.PluginStateObject.Molecule.Structure.Representation3DState)
                    .withTag('animate-assembly-unwind'));
                if (reprs.length === 0)
                    return [2 /*return*/];
                update = state.build();
                for (_i = 0, reprs_2 = reprs; _i < reprs_2.length; _i++) {
                    r = reprs_2[_i];
                    update.delete(r.transform.ref);
                }
                return [2 /*return*/, plugin.runTask(state.updateTree(update))];
            });
        });
    },
    apply: function (animState, t, ctx) {
        return __awaiter(this, void 0, void 0, function () {
            var state, root, anims, update, d, newTime, finished, _i, anims_1, m;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        state = ctx.plugin.state.dataState;
                        root = !ctx.params.target || ctx.params.target === 'all' ? mol_state_1.StateTransform.RootRef : ctx.params.target;
                        anims = state.select(mol_state_1.StateSelection.Generators.ofTransformer(transforms_1.StateTransforms.Representation.UnwindStructureAssemblyRepresentation3D, root));
                        if (anims.length === 0) {
                            return [2 /*return*/, { kind: 'finished' }];
                        }
                        update = state.build();
                        d = (t.current - t.lastApplied) / ctx.params.durationInMs;
                        newTime = (animState.t + d), finished = false;
                        if (ctx.params.playOnce && newTime >= 1) {
                            finished = true;
                            newTime = 1;
                        }
                        else {
                            newTime = newTime % 1;
                        }
                        for (_i = 0, anims_1 = anims; _i < anims_1.length; _i++) {
                            m = anims_1[_i];
                            update.to(m).update({ t: newTime });
                        }
                        return [4 /*yield*/, command_1.PluginCommands.State.Update.dispatch(ctx.plugin, { state: state, tree: update, options: { doNotLogTiming: true } })];
                    case 1:
                        _a.sent();
                        if (finished)
                            return [2 /*return*/, { kind: 'finished' }];
                        return [2 /*return*/, { kind: 'next', state: { t: newTime } }];
                }
            });
        });
    }
});
exports.AnimateUnitsExplode = model_1.PluginStateAnimation.create({
    name: 'built-in.animate-units-explode',
    display: { name: 'Explode Units' },
    params: function () { return ({
        durationInMs: param_definition_1.ParamDefinition.Numeric(3000, { min: 100, max: 10000, step: 100 })
    }); },
    initialState: function () { return ({ t: 0 }); },
    setup: function (_, plugin) {
        return __awaiter(this, void 0, void 0, function () {
            var state, reprs, update, changed, _i, reprs_3, r, explodes;
            return __generator(this, function (_a) {
                state = plugin.state.dataState;
                reprs = state.select(mol_state_1.StateSelection.Generators.ofType(objects_1.PluginStateObject.Molecule.Structure.Representation3D));
                update = state.build();
                changed = false;
                for (_i = 0, reprs_3 = reprs; _i < reprs_3.length; _i++) {
                    r = reprs_3[_i];
                    // TODO: find a better way to handle this, perhaps add a different state object??
                    if (r.transform.transformer === transforms_1.StateTransforms.Representation.StructureLabels3D)
                        continue;
                    explodes = state.select(mol_state_1.StateSelection.Generators.ofTransformer(transforms_1.StateTransforms.Representation.ExplodeStructureRepresentation3D, r.transform.ref));
                    if (explodes.length > 0)
                        continue;
                    changed = true;
                    update.to(r.transform.ref)
                        .apply(transforms_1.StateTransforms.Representation.ExplodeStructureRepresentation3D, { t: 0 }, { tags: 'animate-units-explode' });
                }
                if (!changed)
                    return [2 /*return*/];
                return [2 /*return*/, plugin.runTask(state.updateTree(update, { doNotUpdateCurrent: true }))];
            });
        });
    },
    teardown: function (_, plugin) {
        return __awaiter(this, void 0, void 0, function () {
            var state, reprs, update, _i, reprs_4, r;
            return __generator(this, function (_a) {
                state = plugin.state.dataState;
                reprs = state.select(mol_state_1.StateSelection.Generators.ofType(objects_1.PluginStateObject.Molecule.Structure.Representation3DState)
                    .withTag('animate-units-explode'));
                if (reprs.length === 0)
                    return [2 /*return*/];
                update = state.build();
                for (_i = 0, reprs_4 = reprs; _i < reprs_4.length; _i++) {
                    r = reprs_4[_i];
                    update.delete(r.transform.ref);
                }
                return [2 /*return*/, plugin.runTask(state.updateTree(update))];
            });
        });
    },
    apply: function (animState, t, ctx) {
        return __awaiter(this, void 0, void 0, function () {
            var state, anims, update, d, newTime, _i, anims_2, m;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        state = ctx.plugin.state.dataState;
                        anims = state.select(mol_state_1.StateSelection.Generators.ofTransformer(transforms_1.StateTransforms.Representation.ExplodeStructureRepresentation3D));
                        if (anims.length === 0) {
                            return [2 /*return*/, { kind: 'finished' }];
                        }
                        update = state.build();
                        d = (t.current - t.lastApplied) / ctx.params.durationInMs;
                        newTime = (animState.t + d) % 1;
                        for (_i = 0, anims_2 = anims; _i < anims_2.length; _i++) {
                            m = anims_2[_i];
                            update.to(m).update({ t: newTime });
                        }
                        return [4 /*yield*/, command_1.PluginCommands.State.Update.dispatch(ctx.plugin, { state: state, tree: update, options: { doNotLogTiming: true } })];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, { kind: 'next', state: { t: newTime } }];
                }
            });
        });
    }
});
exports.AnimateStateInterpolation = model_1.PluginStateAnimation.create({
    name: 'built-in.animate-state-interpolation',
    display: { name: 'Animate State Interpolation' },
    params: function () { return ({
        transtionDurationInMs: param_definition_1.ParamDefinition.Numeric(2000, { min: 100, max: 30000, step: 10 })
    }); },
    initialState: function () { return ({}); },
    apply: function (animState, t, ctx) {
        return __awaiter(this, void 0, void 0, function () {
            var snapshots, currentT, srcIndex, tarIndex, _src, _tar, src, tar, state, update, _i, src_1, s, _a, tar_1, t_1, e, f;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        snapshots = ctx.plugin.state.snapshots.state.entries;
                        if (snapshots.size < 2)
                            return [2 /*return*/, { kind: 'finished' }];
                        currentT = (t.current % ctx.params.transtionDurationInMs) / ctx.params.transtionDurationInMs;
                        srcIndex = Math.floor(t.current / ctx.params.transtionDurationInMs) % snapshots.size;
                        tarIndex = Math.ceil(t.current / ctx.params.transtionDurationInMs);
                        if (tarIndex === srcIndex)
                            tarIndex++;
                        tarIndex = tarIndex % snapshots.size;
                        _src = snapshots.get(srcIndex).snapshot, _tar = snapshots.get(tarIndex).snapshot;
                        if (!_src.data || !_tar.data)
                            return [2 /*return*/, { kind: 'skip' }];
                        src = _src.data.tree.transforms, tar = _tar.data.tree.transforms;
                        state = ctx.plugin.state.dataState;
                        update = state.build();
                        for (_i = 0, src_1 = src; _i < src_1.length; _i++) {
                            s = src_1[_i];
                            for (_a = 0, tar_1 = tar; _a < tar_1.length; _a++) {
                                t_1 = tar_1[_a];
                                if (t_1.ref !== s.ref)
                                    continue;
                                if (t_1.version === s.version)
                                    continue;
                                e = mol_state_1.StateTransform.fromJSON(s), f = mol_state_1.StateTransform.fromJSON(t_1);
                                if (!e.transformer.definition.interpolate) {
                                    update.to(s.ref).update(currentT <= 0.5 ? e.params : f.params);
                                }
                                else {
                                    update.to(s.ref).update(e.transformer.definition.interpolate(e.params, f.params, currentT, ctx.plugin));
                                }
                            }
                        }
                        return [4 /*yield*/, command_1.PluginCommands.State.Update.dispatch(ctx.plugin, { state: state, tree: update, options: { doNotLogTiming: true } })];
                    case 1:
                        _b.sent();
                        return [2 /*return*/, { kind: 'next', state: {} }];
                }
            });
        });
    }
});
