"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var immutable_1 = require("immutable");
var mol_util_1 = require("../../mol-util");
var component_1 = require("../../mol-plugin/component");
var CameraSnapshotManager = /** @class */ (function (_super) {
    __extends(CameraSnapshotManager, _super);
    function CameraSnapshotManager() {
        var _this = _super.call(this, { entries: immutable_1.OrderedMap() }) || this;
        _this.events = {
            changed: _this.ev()
        };
        return _this;
    }
    CameraSnapshotManager.prototype.getEntry = function (id) {
        return this.state.entries.get(id);
    };
    CameraSnapshotManager.prototype.remove = function (id) {
        if (!this.state.entries.has(id))
            return;
        this.updateState({ entries: this.state.entries.delete(id) });
        this.events.changed.next();
    };
    CameraSnapshotManager.prototype.add = function (e) {
        this.updateState({ entries: this.state.entries.set(e.id, e) });
        this.events.changed.next();
    };
    CameraSnapshotManager.prototype.clear = function () {
        if (this.state.entries.size === 0)
            return;
        this.updateState({ entries: immutable_1.OrderedMap() });
        this.events.changed.next();
    };
    CameraSnapshotManager.prototype.getStateSnapshot = function () {
        var entries = [];
        this.state.entries.forEach(function (e) { return entries.push(e); });
        return { entries: entries };
    };
    CameraSnapshotManager.prototype.setStateSnapshot = function (state) {
        var entries = immutable_1.OrderedMap().asMutable();
        for (var _i = 0, _a = state.entries; _i < _a.length; _i++) {
            var e = _a[_i];
            entries.set(e.id, e);
        }
        this.updateState({ entries: entries.asImmutable() });
        this.events.changed.next();
    };
    return CameraSnapshotManager;
}(component_1.PluginComponent));
exports.CameraSnapshotManager = CameraSnapshotManager;
(function (CameraSnapshotManager) {
    function Entry(snapshot, name, description) {
        return { id: mol_util_1.UUID.create22(), timestamp: new Date().toLocaleString(), name: name, snapshot: snapshot, description: description };
    }
    CameraSnapshotManager.Entry = Entry;
})(CameraSnapshotManager || (CameraSnapshotManager = {}));
exports.CameraSnapshotManager = CameraSnapshotManager;
