/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { CifFile } from '../../mol-io/reader/cif';
import { PlyFile } from '../../mol-io/reader/ply/schema';
import { Model as _Model, Structure as _Structure } from '../../mol-model/structure';
import { VolumeData } from '../../mol-model/volume';
import { PluginBehavior } from '../../mol-plugin/behavior/behavior';
import { Representation } from '../../mol-repr/representation';
import { StructureRepresentation, StructureRepresentationState } from '../../mol-repr/structure/representation';
import { VolumeRepresentation } from '../../mol-repr/volume/representation';
import { StateObject, StateTransformer } from '../../mol-state';
import { Ccp4File } from '../../mol-io/reader/ccp4/schema';
import { Dsn6File } from '../../mol-io/reader/dsn6/schema';
import { ShapeRepresentation } from '../../mol-repr/shape/representation';
import { ShapeProvider } from '../../mol-model/shape/provider';
export declare type TypeClass = 'root' | 'data' | 'prop';
export declare namespace PluginStateObject {
    type Any = StateObject<any, TypeInfo>;
    type TypeClass = 'Root' | 'Group' | 'Data' | 'Object' | 'Representation3D' | 'Behavior';
    interface TypeInfo {
        name: string;
        typeClass: TypeClass;
    }
    const Create: <D = {}>(type: TypeInfo) => {
        new (data: D, props?: {
            label: string;
            description?: string | undefined;
        } | undefined): {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: D;
        };
        type: TypeInfo;
        is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: D;
        };
    };
    function isRepresentation3D(o?: Any): o is StateObject<Representation3DData<Representation.Any>, TypeInfo>;
    function isBehavior(o?: Any): o is StateObject<PluginBehavior, TypeInfo>;
    interface Representation3DData<T extends Representation.Any, S extends StateObject = StateObject> {
        repr: T;
        source: S;
    }
    function CreateRepresentation3D<T extends Representation.Any, S extends StateObject = StateObject>(type: {
        name: string;
    }): {
        new (data: Representation3DData<T, S>, props?: {
            label: string;
            description?: string | undefined;
        } | undefined): {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: Representation3DData<T, S>;
        };
        type: TypeInfo;
        is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: Representation3DData<T, S>;
        };
    };
    function CreateBehavior<T extends PluginBehavior>(type: {
        name: string;
    }): {
        new (data: T, props?: {
            label: string;
            description?: string | undefined;
        } | undefined): {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: T;
        };
        type: TypeInfo;
        is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: T;
        };
    };
    const Root_base: {
        new (data: {}, props?: {
            label: string;
            description?: string | undefined;
        } | undefined): {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: {};
        };
        type: TypeInfo;
        is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: {};
        };
    };
    class Root extends Root_base {
    }
    const Group_base: {
        new (data: {}, props?: {
            label: string;
            description?: string | undefined;
        } | undefined): {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: {};
        };
        type: TypeInfo;
        is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
            id: import("../../mol-util/uuid").default;
            type: TypeInfo;
            label: string;
            description?: string | undefined;
            data: {};
        };
    };
    class Group extends Group_base {
    }
    namespace Data {
        const String_base: {
            new (data: string, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: string;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: string;
            };
        };
        class String extends String_base {
        }
        const Binary_base: {
            new (data: Uint8Array, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Uint8Array;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Uint8Array;
            };
        };
        class Binary extends Binary_base {
        }
        type BlobEntry = {
            id: string;
        } & ({
            kind: 'string';
            data: string;
        } | {
            kind: 'binary';
            data: Uint8Array;
        });
        type BlobData = BlobEntry[];
        const Blob_base: {
            new (data: BlobEntry[], props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: BlobEntry[];
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: BlobEntry[];
            };
        };
        class Blob extends Blob_base {
        }
    }
    namespace Format {
        const Json_base: {
            new (data: any, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: any;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: any;
            };
        };
        class Json extends Json_base {
        }
        const Cif_base: {
            new (data: CifFile, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: CifFile;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: CifFile;
            };
        };
        class Cif extends Cif_base {
        }
        const Ply_base: {
            new (data: PlyFile, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: PlyFile;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: PlyFile;
            };
        };
        class Ply extends Ply_base {
        }
        const Ccp4_base: {
            new (data: Ccp4File, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Ccp4File;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Ccp4File;
            };
        };
        class Ccp4 extends Ccp4_base {
        }
        const Dsn6_base: {
            new (data: Dsn6File, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Dsn6File;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Dsn6File;
            };
        };
        class Dsn6 extends Dsn6_base {
        }
        type BlobEntry = {
            id: string;
        } & ({
            kind: 'json';
            data: unknown;
        } | {
            kind: 'string';
            data: string;
        } | {
            kind: 'binary';
            data: Uint8Array;
        } | {
            kind: 'cif';
            data: CifFile;
        } | {
            kind: 'ccp4';
            data: Ccp4File;
        } | {
            kind: 'dsn6';
            data: Dsn6File;
        } | {
            kind: 'ply';
            data: PlyFile;
        } | {
            kind: 'custom';
            data: unknown;
            tag: string;
        });
        type BlobData = BlobEntry[];
        const Blob_base_1: {
            new (data: BlobEntry[], props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: BlobEntry[];
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: BlobEntry[];
            };
        };
        class Blob extends Blob_base_1 {
        }
    }
    namespace Molecule {
        const Trajectory_base: {
            new (data: readonly _Model[], props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: readonly _Model[];
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: readonly _Model[];
            };
        };
        class Trajectory extends Trajectory_base {
        }
        const Model_base: {
            new (data: _Model, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: _Model;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: _Model;
            };
        };
        class Model extends Model_base {
        }
        const Structure_base: {
            new (data: _Structure, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: _Structure;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: _Structure;
            };
        };
        class Structure extends Structure_base {
        }
        namespace Structure {
            const Representation3D_base: {
                new (data: Representation3DData<StructureRepresentation<any> | ShapeRepresentation<any, any, any>, Structure>, props?: {
                    label: string;
                    description?: string | undefined;
                } | undefined): {
                    id: import("../../mol-util/uuid").default;
                    type: TypeInfo;
                    label: string;
                    description?: string | undefined;
                    data: Representation3DData<StructureRepresentation<any> | ShapeRepresentation<any, any, any>, Structure>;
                };
                type: TypeInfo;
                is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                    id: import("../../mol-util/uuid").default;
                    type: TypeInfo;
                    label: string;
                    description?: string | undefined;
                    data: Representation3DData<StructureRepresentation<any> | ShapeRepresentation<any, any, any>, Structure>;
                };
            };
            class Representation3D extends Representation3D_base {
            }
            interface Representation3DStateData {
                source: Representation3D;
                /** used to restore state when the obj is removed */
                initialState: Partial<StructureRepresentationState>;
                state: Partial<StructureRepresentationState>;
                info?: unknown;
            }
            const Representation3DState_base: {
                new (data: Representation3DStateData, props?: {
                    label: string;
                    description?: string | undefined;
                } | undefined): {
                    id: import("../../mol-util/uuid").default;
                    type: TypeInfo;
                    label: string;
                    description?: string | undefined;
                    data: Representation3DStateData;
                };
                type: TypeInfo;
                is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                    id: import("../../mol-util/uuid").default;
                    type: TypeInfo;
                    label: string;
                    description?: string | undefined;
                    data: Representation3DStateData;
                };
            };
            class Representation3DState extends Representation3DState_base {
            }
        }
    }
    namespace Volume {
        const Data_base: {
            new (data: VolumeData, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: VolumeData;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: VolumeData;
            };
        };
        class Data extends Data_base {
        }
        const Representation3D_base_1: {
            new (data: Representation3DData<VolumeRepresentation<any>, StateObject<any, StateObject.Type<any>>>, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Representation3DData<VolumeRepresentation<any>, StateObject<any, StateObject.Type<any>>>;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Representation3DData<VolumeRepresentation<any>, StateObject<any, StateObject.Type<any>>>;
            };
        };
        class Representation3D extends Representation3D_base_1 {
        }
    }
    namespace Shape {
        const Provider_base: {
            new (data: ShapeProvider<any, any, any>, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: ShapeProvider<any, any, any>;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: ShapeProvider<any, any, any>;
            };
        };
        class Provider extends Provider_base {
        }
        const Representation3D_base_2: {
            new (data: Representation3DData<ShapeRepresentation<any, any, any>, StateObject<any, StateObject.Type<any>>>, props?: {
                label: string;
                description?: string | undefined;
            } | undefined): {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Representation3DData<ShapeRepresentation<any, any, any>, StateObject<any, StateObject.Type<any>>>;
            };
            type: TypeInfo;
            is(obj?: StateObject<any, StateObject.Type<any>> | undefined): obj is {
                id: import("../../mol-util/uuid").default;
                type: TypeInfo;
                label: string;
                description?: string | undefined;
                data: Representation3DData<ShapeRepresentation<any, any, any>, StateObject<any, StateObject.Type<any>>>;
            };
        };
        class Representation3D extends Representation3D_base_2 {
        }
    }
}
export declare namespace PluginStateTransform {
    const CreateBuiltIn: <A extends StateObject<any, StateObject.Type<any>>, B extends StateObject<any, StateObject.Type<any>>, P extends {} = {}>(definition: StateTransformer.Definition<A, B, P>) => StateTransformer<A, B, P>;
    const BuiltIn: StateTransformer.Builder.Root;
}
