/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { List } from 'immutable';
import { UUID } from '../../mol-util';
import { PluginState } from '../state';
import { PluginComponent } from '../../mol-plugin/component';
import { PluginContext } from '../../mol-plugin/context';
export { PluginStateSnapshotManager };
declare class PluginStateSnapshotManager extends PluginComponent<{
    current?: UUID | undefined;
    entries: List<PluginStateSnapshotManager.Entry>;
    isPlaying: boolean;
    nextSnapshotDelayInMs: number;
}> {
    private plugin;
    static DefaultNextSnapshotDelayInMs: number;
    private entryMap;
    readonly events: {
        changed: import("rxjs").Subject<unknown>;
    };
    currentGetSnapshotParams: PluginState.GetSnapshotParams;
    getIndex(e: PluginStateSnapshotManager.Entry): number;
    getEntry(id: string | undefined): PluginStateSnapshotManager.Entry | undefined;
    remove(id: string): void;
    add(e: PluginStateSnapshotManager.Entry): void;
    replace(id: string, snapshot: PluginState.Snapshot): void;
    move(id: string, dir: -1 | 1): void;
    clear(): void;
    setCurrent(id: string): PluginState.Snapshot | undefined;
    getNextId(id: string | undefined, dir: -1 | 1): UUID | undefined;
    setRemoteSnapshot(snapshot: PluginStateSnapshotManager.RemoteSnapshot): Promise<PluginState.Snapshot | undefined>;
    getRemoteSnapshot(options?: {
        name?: string;
        description?: string;
        playOnLoad?: boolean;
    }): PluginStateSnapshotManager.RemoteSnapshot;
    private timeoutHandle;
    private next;
    play(delayFirst?: boolean): void;
    stop(): void;
    togglePlay(): void;
    constructor(plugin: PluginContext);
}
declare namespace PluginStateSnapshotManager {
    interface Entry {
        timestamp: number;
        name?: string;
        description?: string;
        snapshot: PluginState.Snapshot;
    }
    function Entry(snapshot: PluginState.Snapshot, params: {
        name?: string;
        description?: string;
    }): Entry;
    interface RemoteSnapshot {
        timestamp: number;
        name?: string;
        description?: string;
        current: UUID | undefined;
        playback: {
            isPlaying: boolean;
            nextSnapshotDelayInMs: number;
        };
        entries: Entry[];
    }
}
