"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var objects_1 = require("../objects");
var objects_2 = require("../objects");
var mol_task_1 = require("../../../mol-task");
var cif_1 = require("../../../mol-io/reader/cif");
var param_definition_1 = require("../../../mol-util/param-definition");
var mol_state_1 = require("../../../mol-state");
var data_source_1 = require("../../../mol-util/data-source");
var CCP4 = require("../../../mol-io/reader/ccp4/parser");
var DSN6 = require("../../../mol-io/reader/dsn6/parser");
var PLY = require("../../../mol-io/reader/ply/parser");
var Download = objects_1.PluginStateTransform.BuiltIn({
    name: 'download',
    display: { name: 'Download', description: 'Download string or binary data from the specified URL' },
    from: [objects_2.PluginStateObject.Root],
    to: [objects_2.PluginStateObject.Data.String, objects_2.PluginStateObject.Data.Binary],
    params: {
        url: param_definition_1.ParamDefinition.Text('https://www.ebi.ac.uk/pdbe/static/entry/1cbs_updated.cif', { description: 'Resource URL. Must be the same domain or support CORS.' }),
        label: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Text('')),
        isBinary: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Boolean(false, { description: 'If true, download data as binary (string otherwise)' }))
    }
})({
    apply: function (_a, globalCtx) {
        var _this = this;
        var p = _a.params;
        return mol_task_1.Task.create('Download', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, globalCtx.fetch({ url: p.url, type: p.isBinary ? 'binary' : 'string' }).runInContext(ctx)];
                    case 1:
                        data = _a.sent();
                        return [2 /*return*/, p.isBinary
                                ? new objects_2.PluginStateObject.Data.Binary(data, { label: p.label ? p.label : p.url })
                                : new objects_2.PluginStateObject.Data.String(data, { label: p.label ? p.label : p.url })];
                }
            });
        }); });
    },
    update: function (_a) {
        var oldParams = _a.oldParams, newParams = _a.newParams, b = _a.b;
        if (oldParams.url !== newParams.url || oldParams.isBinary !== newParams.isBinary)
            return mol_state_1.StateTransformer.UpdateResult.Recreate;
        if (oldParams.label !== newParams.label) {
            b.label = newParams.label || newParams.url;
            return mol_state_1.StateTransformer.UpdateResult.Updated;
        }
        return mol_state_1.StateTransformer.UpdateResult.Unchanged;
    }
});
exports.Download = Download;
var DownloadBlob = objects_1.PluginStateTransform.BuiltIn({
    name: 'download-blob',
    display: { name: 'Download Blob', description: 'Download multiple string or binary data from the specified URLs.' },
    from: objects_2.PluginStateObject.Root,
    to: objects_2.PluginStateObject.Data.Blob,
    params: {
        sources: param_definition_1.ParamDefinition.ObjectList({
            id: param_definition_1.ParamDefinition.Text('', { label: 'Unique ID' }),
            url: param_definition_1.ParamDefinition.Text('https://www.ebi.ac.uk/pdbe/static/entry/1cbs_updated.cif', { description: 'Resource URL. Must be the same domain or support CORS.' }),
            isBinary: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Boolean(false, { description: 'If true, download data as binary (string otherwise)' })),
            canFail: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Boolean(false, { description: 'Indicate whether the download can fail and not be included in the blob as a result.' }))
        }, function (e) { return e.id + ": " + e.url; }),
        maxConcurrency: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Numeric(4, { min: 1, max: 12, step: 1 }, { description: 'The maximum number of concurrent downloads.' }))
    }
})({
    apply: function (_a, plugin) {
        var _this = this;
        var params = _a.params;
        return mol_task_1.Task.create('Download Blob', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var entries, data, i, r, src;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        entries = [];
                        return [4 /*yield*/, data_source_1.ajaxGetMany(ctx, params.sources, params.maxConcurrency || 4)];
                    case 1:
                        data = _a.sent();
                        for (i = 0; i < data.length; i++) {
                            r = data[i], src = params.sources[i];
                            if (r.kind === 'error')
                                plugin.log.warn("Download " + r.id + " (" + src.url + ") failed: " + r.error);
                            else {
                                entries.push(src.isBinary
                                    ? { id: r.id, kind: 'binary', data: r.result }
                                    : { id: r.id, kind: 'string', data: r.result });
                            }
                        }
                        return [2 /*return*/, new objects_2.PluginStateObject.Data.Blob(entries, { label: 'Data Blob', description: entries.length + " " + (entries.length === 1 ? 'entry' : 'entries') })];
                }
            });
        }); });
    },
});
exports.DownloadBlob = DownloadBlob;
var ReadFile = objects_1.PluginStateTransform.BuiltIn({
    name: 'read-file',
    display: { name: 'Read File', description: 'Read string or binary data from the specified file' },
    from: objects_2.PluginStateObject.Root,
    to: [objects_2.PluginStateObject.Data.String, objects_2.PluginStateObject.Data.Binary],
    params: {
        file: param_definition_1.ParamDefinition.File(),
        label: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Text('')),
        isBinary: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Boolean(false, { description: 'If true, open file as as binary (string otherwise)' }))
    }
})({
    apply: function (_a) {
        var _this = this;
        var p = _a.params;
        return mol_task_1.Task.create('Open File', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var data;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, data_source_1.readFromFile(p.file, p.isBinary ? 'binary' : 'string').runInContext(ctx)];
                    case 1:
                        data = _a.sent();
                        return [2 /*return*/, p.isBinary
                                ? new objects_2.PluginStateObject.Data.Binary(data, { label: p.label ? p.label : p.file.name })
                                : new objects_2.PluginStateObject.Data.String(data, { label: p.label ? p.label : p.file.name })];
                }
            });
        }); });
    },
    update: function (_a) {
        var oldParams = _a.oldParams, newParams = _a.newParams, b = _a.b;
        if (oldParams.label !== newParams.label) {
            b.label = newParams.label || oldParams.file.name;
            return mol_state_1.StateTransformer.UpdateResult.Updated;
        }
        return mol_state_1.StateTransformer.UpdateResult.Unchanged;
    },
    isSerializable: function () { return ({ isSerializable: false, reason: 'Cannot serialize user loaded files.' }); }
});
exports.ReadFile = ReadFile;
var ParseBlob = objects_1.PluginStateTransform.BuiltIn({
    name: 'parse-blob',
    display: { name: 'Parse Blob', description: 'Parse multiple data enties' },
    from: objects_2.PluginStateObject.Data.Blob,
    to: objects_2.PluginStateObject.Format.Blob,
    params: {
        formats: param_definition_1.ParamDefinition.ObjectList({
            id: param_definition_1.ParamDefinition.Text('', { label: 'Unique ID' }),
            format: param_definition_1.ParamDefinition.Select('cif', [['cif', 'cif']])
        }, function (e) { return e.id + ": " + e.format; })
    }
})({
    apply: function (_a, plugin) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Parse Blob', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var map, _i, _a, f, entries, _b, _c, e, parsed;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        map = new Map();
                        for (_i = 0, _a = params.formats; _i < _a.length; _i++) {
                            f = _a[_i];
                            map.set(f.id, f.format);
                        }
                        entries = [];
                        _b = 0, _c = a.data;
                        _d.label = 1;
                    case 1:
                        if (!(_b < _c.length)) return [3 /*break*/, 4];
                        e = _c[_b];
                        if (!map.has(e.id))
                            return [3 /*break*/, 3];
                        return [4 /*yield*/, (e.kind === 'string' ? cif_1.CIF.parse(e.data) : cif_1.CIF.parseBinary(e.data)).runInContext(ctx)];
                    case 2:
                        parsed = _d.sent();
                        if (parsed.isError)
                            throw new Error(e.id + ": " + parsed.message);
                        entries.push({ id: e.id, kind: 'cif', data: parsed.result });
                        _d.label = 3;
                    case 3:
                        _b++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/, new objects_2.PluginStateObject.Format.Blob(entries, { label: 'Format Blob', description: entries.length + " " + (entries.length === 1 ? 'entry' : 'entries') })];
                }
            });
        }); });
    },
});
exports.ParseBlob = ParseBlob;
var ParseCif = objects_1.PluginStateTransform.BuiltIn({
    name: 'parse-cif',
    display: { name: 'Parse CIF', description: 'Parse CIF from String or Binary data' },
    from: [objects_2.PluginStateObject.Data.String, objects_2.PluginStateObject.Data.Binary],
    to: objects_2.PluginStateObject.Format.Cif
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a;
        return mol_task_1.Task.create('Parse CIF', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var parsed;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, (objects_2.PluginStateObject.Data.String.is(a) ? cif_1.CIF.parse(a.data) : cif_1.CIF.parseBinary(a.data)).runInContext(ctx)];
                    case 1:
                        parsed = _a.sent();
                        if (parsed.isError)
                            throw new Error(parsed.message);
                        return [2 /*return*/, new objects_2.PluginStateObject.Format.Cif(parsed.result)];
                }
            });
        }); });
    }
});
exports.ParseCif = ParseCif;
var ParsePly = objects_1.PluginStateTransform.BuiltIn({
    name: 'parse-ply',
    display: { name: 'Parse PLY', description: 'Parse PLY from String data' },
    from: [objects_2.PluginStateObject.Data.String],
    to: objects_2.PluginStateObject.Format.Ply
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a;
        return mol_task_1.Task.create('Parse PLY', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var parsed;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, PLY.parse(a.data).runInContext(ctx)];
                    case 1:
                        parsed = _a.sent();
                        if (parsed.isError)
                            throw new Error(parsed.message);
                        return [2 /*return*/, new objects_2.PluginStateObject.Format.Ply(parsed.result, { label: parsed.result.comments[0] || 'PLY Data' })];
                }
            });
        }); });
    }
});
exports.ParsePly = ParsePly;
var ParseCcp4 = objects_1.PluginStateTransform.BuiltIn({
    name: 'parse-ccp4',
    display: { name: 'Parse CCP4/MRC/MAP', description: 'Parse CCP4/MRC/MAP from Binary data' },
    from: [objects_2.PluginStateObject.Data.Binary],
    to: objects_2.PluginStateObject.Format.Ccp4
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a;
        return mol_task_1.Task.create('Parse CCP4/MRC/MAP', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var parsed;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, CCP4.parse(a.data).runInContext(ctx)];
                    case 1:
                        parsed = _a.sent();
                        if (parsed.isError)
                            throw new Error(parsed.message);
                        return [2 /*return*/, new objects_2.PluginStateObject.Format.Ccp4(parsed.result)];
                }
            });
        }); });
    }
});
exports.ParseCcp4 = ParseCcp4;
var ParseDsn6 = objects_1.PluginStateTransform.BuiltIn({
    name: 'parse-dsn6',
    display: { name: 'Parse DSN6/BRIX', description: 'Parse CCP4/BRIX from Binary data' },
    from: [objects_2.PluginStateObject.Data.Binary],
    to: objects_2.PluginStateObject.Format.Dsn6
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a;
        return mol_task_1.Task.create('Parse DSN6/BRIX', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var parsed;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, DSN6.parse(a.data).runInContext(ctx)];
                    case 1:
                        parsed = _a.sent();
                        if (parsed.isError)
                            throw new Error(parsed.message);
                        return [2 /*return*/, new objects_2.PluginStateObject.Format.Dsn6(parsed.result)];
                }
            });
        }); });
    }
});
exports.ParseDsn6 = ParseDsn6;
