/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Vec3, Mat4 } from '../../../mol-math/linear-algebra';
import Expression from '../../../mol-script/language/expression';
import { StateTransformer } from '../../../mol-state';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { PluginStateObject as SO } from '../objects';
export { TrajectoryFromBlob };
export { TrajectoryFromMmCif };
export { TrajectoryFromPDB };
export { TrajectoryFromGRO };
export { ModelFromTrajectory };
export { StructureFromModel };
export { StructureAssemblyFromModel };
export { StructureSymmetryFromModel };
export { TransformStructureConformation };
export { TransformStructureConformationByMatrix };
export { StructureSelection };
export { UserStructureSelection };
export { StructureComplexElement };
export { CustomModelProperties };
export { CustomStructureProperties };
declare type TrajectoryFromBlob = typeof TrajectoryFromBlob;
declare const TrajectoryFromBlob: StateTransformer<SO.Format.Blob, SO.Molecule.Trajectory, PD.Normalize<{}>>;
declare type TrajectoryFromMmCif = typeof TrajectoryFromMmCif;
declare const TrajectoryFromMmCif: StateTransformer<SO.Format.Cif, SO.Molecule.Trajectory, PD.Normalize<{
    blockHeader: string | undefined;
}>>;
declare type TrajectoryFromPDB = typeof TrajectoryFromPDB;
declare const TrajectoryFromPDB: StateTransformer<SO.Data.String, SO.Molecule.Trajectory, PD.Normalize<{}>>;
declare type TrajectoryFromGRO = typeof TrajectoryFromGRO;
declare const TrajectoryFromGRO: StateTransformer<SO.Data.String, SO.Molecule.Trajectory, PD.Normalize<{}>>;
declare type ModelFromTrajectory = typeof ModelFromTrajectory;
declare const ModelFromTrajectory: StateTransformer<SO.Molecule.Trajectory, SO.Molecule.Model, PD.Normalize<{
    modelIndex: number;
}>>;
declare type StructureFromModel = typeof StructureFromModel;
declare const StructureFromModel: StateTransformer<SO.Molecule.Model, SO.Molecule.Structure, PD.Normalize<{}>>;
declare type StructureAssemblyFromModel = typeof StructureAssemblyFromModel;
declare const StructureAssemblyFromModel: StateTransformer<SO.Molecule.Model, SO.Molecule.Structure, PD.Normalize<{
    id: string | undefined;
}>>;
declare type StructureSymmetryFromModel = typeof StructureSymmetryFromModel;
declare const StructureSymmetryFromModel: StateTransformer<SO.Molecule.Model, SO.Molecule.Structure, PD.Normalize<{
    ijkMin: Vec3;
    ijkMax: Vec3;
}>>;
declare type TransformStructureConformation = typeof TransformStructureConformation;
declare const TransformStructureConformation: StateTransformer<SO.Molecule.Structure, SO.Molecule.Structure, PD.Normalize<{
    axis: Vec3;
    angle: number;
    translation: Vec3;
}>>;
declare type TransformStructureConformationByMatrix = typeof TransformStructureConformation;
declare const TransformStructureConformationByMatrix: StateTransformer<SO.Molecule.Structure, SO.Molecule.Structure, PD.Normalize<{
    matrix: Mat4;
}>>;
declare type StructureSelection = typeof StructureSelection;
declare const StructureSelection: StateTransformer<SO.Molecule.Structure, SO.Molecule.Structure, PD.Normalize<{
    query: Expression;
    label: string | undefined;
}>>;
declare type UserStructureSelection = typeof UserStructureSelection;
declare const UserStructureSelection: StateTransformer<SO.Molecule.Structure, SO.Molecule.Structure, PD.Normalize<{
    query: {
        language: "mol-script";
        expression: string;
    };
    label: string | undefined;
}>>;
declare namespace StructureComplexElement {
    type Types = 'atomic-sequence' | 'water' | 'atomic-het' | 'spheres';
}
declare type StructureComplexElement = typeof StructureComplexElement;
declare const StructureComplexElement: StateTransformer<SO.Molecule.Structure, SO.Molecule.Structure, PD.Normalize<{
    type: StructureComplexElement.Types;
}>>;
declare type CustomModelProperties = typeof CustomModelProperties;
declare const CustomModelProperties: StateTransformer<SO.Molecule.Model, SO.Molecule.Model, PD.Normalize<{
    properties: string[];
}>>;
declare type CustomStructureProperties = typeof CustomStructureProperties;
declare const CustomStructureProperties: StateTransformer<SO.Molecule.Structure, SO.Molecule.Structure, PD.Normalize<{
    properties: string[];
}>>;
export { ShapeFromPly };
declare type ShapeFromPly = typeof ShapeFromPly;
declare const ShapeFromPly: StateTransformer<SO.Format.Ply, SO.Shape.Provider, PD.Normalize<{}>>;
