"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var parser_1 = require("../../../mol-io/reader/pdb/parser");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var mmcif_1 = require("../../../mol-model-formats/structure/mmcif");
var pdb_1 = require("../../../mol-model-formats/structure/pdb");
var structure_1 = require("../../../mol-model/structure");
var builder_1 = require("../../../mol-script/language/builder");
var compiler_1 = require("../../../mol-script/runtime/query/compiler");
var mol_state_1 = require("../../../mol-state");
var mol_task_1 = require("../../../mol-task");
var param_definition_1 = require("../../../mol-util/param-definition");
var string_1 = require("../../../mol-util/string");
var objects_1 = require("../objects");
var gro_1 = require("../../../mol-model-formats/structure/gro");
var parser_2 = require("../../../mol-io/reader/gro/parser");
var parser_3 = require("../../../mol-script/language/parser");
var symbols_1 = require("../../../mol-script/script/mol-script/symbols");
var ply_1 = require("../../../mol-model-formats/shape/ply");
var geometry_1 = require("../../../mol-math/geometry");
var helpers_1 = require("./helpers");
var TrajectoryFromBlob = objects_1.PluginStateTransform.BuiltIn({
    name: 'trajectory-from-blob',
    display: { name: 'Parse Blob', description: 'Parse format blob into a single trajectory.' },
    from: objects_1.PluginStateObject.Format.Blob,
    to: objects_1.PluginStateObject.Molecule.Trajectory
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a;
        return mol_task_1.Task.create('Parse Format Blob', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var models, _i, _a, e, block, xs, _b, xs_1, x, props;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        models = [];
                        _i = 0, _a = a.data;
                        _c.label = 1;
                    case 1:
                        if (!(_i < _a.length)) return [3 /*break*/, 4];
                        e = _a[_i];
                        if (e.kind !== 'cif')
                            return [3 /*break*/, 3];
                        block = e.data.blocks[0];
                        return [4 /*yield*/, mmcif_1.trajectoryFromMmCIF(block).runInContext(ctx)];
                    case 2:
                        xs = _c.sent();
                        if (xs.length === 0)
                            throw new Error('No models found.');
                        for (_b = 0, xs_1 = xs; _b < xs_1.length; _b++) {
                            x = xs_1[_b];
                            models.push(x);
                        }
                        _c.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 1];
                    case 4:
                        props = { label: "Trajectory", description: models.length + " model" + (models.length === 1 ? '' : 's') };
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Trajectory(models, props)];
                }
            });
        }); });
    }
});
exports.TrajectoryFromBlob = TrajectoryFromBlob;
var TrajectoryFromMmCif = objects_1.PluginStateTransform.BuiltIn({
    name: 'trajectory-from-mmcif',
    display: { name: 'Trajectory from mmCIF', description: 'Identify and create all separate models in the specified CIF data block' },
    from: objects_1.PluginStateObject.Format.Cif,
    to: objects_1.PluginStateObject.Molecule.Trajectory,
    params: function (a) {
        if (!a) {
            return {
                blockHeader: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Text(void 0, { description: 'Header of the block to parse. If none is specifed, the 1st data block in the file is used.' }))
            };
        }
        var blocks = a.data.blocks;
        return {
            blockHeader: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Select(blocks[0] && blocks[0].header, blocks.map(function (b) { return [b.header, b.header]; }), { description: 'Header of the block to parse' }))
        };
    }
})({
    isApplicable: function (a) { return a.data.blocks.length > 0; },
    apply: function (_a) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Parse mmCIF', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var header, block, models, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        header = params.blockHeader || a.data.blocks[0].header;
                        block = a.data.blocks.find(function (b) { return b.header === header; });
                        if (!block)
                            throw new Error("Data block '" + [header] + "' not found.");
                        return [4 /*yield*/, mmcif_1.trajectoryFromMmCIF(block).runInContext(ctx)];
                    case 1:
                        models = _a.sent();
                        if (models.length === 0)
                            throw new Error('No models found.');
                        props = { label: models[0].label, description: models.length + " model" + (models.length === 1 ? '' : 's') };
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Trajectory(models, props)];
                }
            });
        }); });
    }
});
exports.TrajectoryFromMmCif = TrajectoryFromMmCif;
var TrajectoryFromPDB = objects_1.PluginStateTransform.BuiltIn({
    name: 'trajectory-from-pdb',
    display: { name: 'Parse PDB', description: 'Parse PDB string and create trajectory.' },
    from: [objects_1.PluginStateObject.Data.String],
    to: objects_1.PluginStateObject.Molecule.Trajectory
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a;
        return mol_task_1.Task.create('Parse PDB', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var parsed, models, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, parser_1.parsePDB(a.data, a.label).runInContext(ctx)];
                    case 1:
                        parsed = _a.sent();
                        if (parsed.isError)
                            throw new Error(parsed.message);
                        return [4 /*yield*/, pdb_1.trajectoryFromPDB(parsed.result).runInContext(ctx)];
                    case 2:
                        models = _a.sent();
                        props = { label: models[0].label, description: models.length + " model" + (models.length === 1 ? '' : 's') };
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Trajectory(models, props)];
                }
            });
        }); });
    }
});
exports.TrajectoryFromPDB = TrajectoryFromPDB;
var TrajectoryFromGRO = objects_1.PluginStateTransform.BuiltIn({
    name: 'trajectory-from-gro',
    display: { name: 'Parse GRO', description: 'Parse GRO string and create trajectory.' },
    from: [objects_1.PluginStateObject.Data.String],
    to: objects_1.PluginStateObject.Molecule.Trajectory
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a;
        return mol_task_1.Task.create('Parse GRO', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var parsed, models, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, parser_2.parseGRO(a.data).runInContext(ctx)];
                    case 1:
                        parsed = _a.sent();
                        if (parsed.isError)
                            throw new Error(parsed.message);
                        return [4 /*yield*/, gro_1.trajectoryFromGRO(parsed.result).runInContext(ctx)];
                    case 2:
                        models = _a.sent();
                        props = { label: models[0].label, description: models.length + " model" + (models.length === 1 ? '' : 's') };
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Trajectory(models, props)];
                }
            });
        }); });
    }
});
exports.TrajectoryFromGRO = TrajectoryFromGRO;
var plus1 = function (v) { return v + 1; }, minus1 = function (v) { return v - 1; };
var ModelFromTrajectory = objects_1.PluginStateTransform.BuiltIn({
    name: 'model-from-trajectory',
    display: { name: 'Molecular Model', description: 'Create a molecular model from specified index in a trajectory.' },
    from: objects_1.PluginStateObject.Molecule.Trajectory,
    to: objects_1.PluginStateObject.Molecule.Model,
    params: function (a) {
        if (!a) {
            return { modelIndex: param_definition_1.ParamDefinition.Numeric(0, {}, { description: 'Zero-based index of the model' }) };
        }
        return { modelIndex: param_definition_1.ParamDefinition.Converted(plus1, minus1, param_definition_1.ParamDefinition.Numeric(1, { min: 1, max: a.data.length, step: 1 }, { description: 'Model Index' })) };
    }
})({
    isApplicable: function (a) { return a.data.length > 0; },
    apply: function (_a) {
        var a = _a.a, params = _a.params;
        if (params.modelIndex < 0 || params.modelIndex >= a.data.length)
            throw new Error("Invalid modelIndex " + params.modelIndex);
        var model = a.data[params.modelIndex];
        var label = a.data.length === 1 ? model.entry : model.entry + ": " + model.modelNum;
        var description = a.data.length === 1 ? undefined : "Model " + (params.modelIndex + 1) + " of " + a.data.length;
        return new objects_1.PluginStateObject.Molecule.Model(model, { label: label, description: description });
    }
});
exports.ModelFromTrajectory = ModelFromTrajectory;
var StructureFromModel = objects_1.PluginStateTransform.BuiltIn({
    name: 'structure-from-model',
    display: { name: 'Structure from Model', description: 'Create a molecular structure from the specified model.' },
    from: objects_1.PluginStateObject.Molecule.Model,
    to: objects_1.PluginStateObject.Molecule.Structure
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a;
        return mol_task_1.Task.create('Build Structure', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var s, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        s = structure_1.Structure.ofModel(a.data);
                        return [4 /*yield*/, helpers_1.ensureSecondaryStructure(s)];
                    case 1:
                        _a.sent();
                        props = { label: a.data.label, description: s.elementCount === 1 ? '1 element' : s.elementCount + " elements" };
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Structure(s, props)];
                }
            });
        }); });
    }
});
exports.StructureFromModel = StructureFromModel;
function structureDesc(s) {
    return s.elementCount === 1 ? '1 element' : s.elementCount + " elements";
}
var StructureAssemblyFromModel = objects_1.PluginStateTransform.BuiltIn({
    name: 'structure-assembly-from-model',
    display: { name: 'Structure Assembly', description: 'Create a molecular structure assembly.' },
    from: objects_1.PluginStateObject.Molecule.Model,
    to: objects_1.PluginStateObject.Molecule.Structure,
    params: function (a) {
        if (!a) {
            return { id: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Text('', { label: 'Assembly Id', description: 'Assembly Id. Value \'deposited\' can be used to specify deposited asymmetric unit.' })) };
        }
        var model = a.data;
        var ids = model.symmetry.assemblies.map(function (a) { return [a.id, a.id + ": " + string_1.stringToWords(a.details)]; });
        ids.push(['deposited', 'Deposited']);
        return {
            id: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Select(ids[0][0], ids, { label: 'Asm Id', description: 'Assembly Id' }))
        };
    }
})({
    apply: function (_a, plugin) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Build Assembly', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var model, id, asm, base, label, s, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        model = a.data;
                        id = params.id;
                        asm = void 0;
                        // if no id is specified, use the 1st assembly.
                        if (!id && model.symmetry.assemblies.length !== 0) {
                            id = model.symmetry.assemblies[0].id;
                        }
                        if (model.symmetry.assemblies.length === 0) {
                            if (id !== 'deposited') {
                                plugin.log.warn("Model '" + a.data.label + "' has no assembly, returning deposited structure.");
                            }
                        }
                        else {
                            asm = structure_1.ModelSymmetry.findAssembly(model, id || '');
                            if (!asm) {
                                plugin.log.warn("Model '" + a.data.label + "' has no assembly called '" + id + "', returning deposited structure.");
                            }
                        }
                        base = structure_1.Structure.ofModel(model);
                        if (!!asm) return [3 /*break*/, 2];
                        return [4 /*yield*/, helpers_1.ensureSecondaryStructure(base)];
                    case 1:
                        _a.sent();
                        label = { label: 'Deposited', description: structureDesc(base) };
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Structure(base, label)];
                    case 2:
                        id = asm.id;
                        return [4 /*yield*/, structure_1.StructureSymmetry.buildAssembly(base, id).runInContext(ctx)];
                    case 3:
                        s = _a.sent();
                        return [4 /*yield*/, helpers_1.ensureSecondaryStructure(s)];
                    case 4:
                        _a.sent();
                        props = { label: "Assembly " + id, description: structureDesc(s) };
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Structure(s, props)];
                }
            });
        }); });
    }
});
exports.StructureAssemblyFromModel = StructureAssemblyFromModel;
var StructureSymmetryFromModel = objects_1.PluginStateTransform.BuiltIn({
    name: 'structure-symmetry-from-model',
    display: { name: 'Structure Symmetry', description: 'Create a molecular structure symmetry.' },
    from: objects_1.PluginStateObject.Molecule.Model,
    to: objects_1.PluginStateObject.Molecule.Structure,
    params: function (a) {
        return {
            ijkMin: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.create(-1, -1, -1), { label: 'Min IJK', fieldLabels: { x: 'I', y: 'J', z: 'K' } }),
            ijkMax: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.create(1, 1, 1), { label: 'Max IJK', fieldLabels: { x: 'I', y: 'J', z: 'K' } })
        };
    }
})({
    apply: function (_a, plugin) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Build Symmetry', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var ijkMin, ijkMax, model, base, s, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        ijkMin = params.ijkMin, ijkMax = params.ijkMax;
                        model = a.data;
                        base = structure_1.Structure.ofModel(model);
                        return [4 /*yield*/, structure_1.StructureSymmetry.buildSymmetryRange(base, ijkMin, ijkMax).runInContext(ctx)];
                    case 1:
                        s = _a.sent();
                        return [4 /*yield*/, helpers_1.ensureSecondaryStructure(s)];
                    case 2:
                        _a.sent();
                        props = { label: "Symmetry [" + ijkMin + "] to [" + ijkMax + "]", description: structureDesc(s) };
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Structure(s, props)];
                }
            });
        }); });
    }
});
exports.StructureSymmetryFromModel = StructureSymmetryFromModel;
var _translation = linear_algebra_1.Vec3.zero(), _m = linear_algebra_1.Mat4.zero(), _n = linear_algebra_1.Mat4.zero();
var TransformStructureConformation = objects_1.PluginStateTransform.BuiltIn({
    name: 'transform-structure-conformation',
    display: { name: 'Transform Conformation' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    to: objects_1.PluginStateObject.Molecule.Structure,
    params: {
        axis: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.create(1, 0, 0)),
        angle: param_definition_1.ParamDefinition.Numeric(0, { min: -180, max: 180, step: 0.1 }),
        translation: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.create(0, 0, 0)),
    }
})({
    canAutoUpdate: function () {
        return true;
    },
    apply: function (_a) {
        // TODO: optimze
        var a = _a.a, params = _a.params;
        var center = a.data.boundary.sphere.center;
        linear_algebra_1.Mat4.fromTranslation(_m, linear_algebra_1.Vec3.negate(_translation, center));
        linear_algebra_1.Mat4.fromTranslation(_n, linear_algebra_1.Vec3.add(_translation, center, params.translation));
        var rot = linear_algebra_1.Mat4.fromRotation(linear_algebra_1.Mat4.zero(), Math.PI / 180 * params.angle, linear_algebra_1.Vec3.normalize(linear_algebra_1.Vec3.zero(), params.axis));
        var m = linear_algebra_1.Mat4.zero();
        linear_algebra_1.Mat4.mul3(m, _n, rot, _m);
        var s = structure_1.Structure.transform(a.data, m);
        var props = { label: "" + a.label, description: "Transformed" };
        return new objects_1.PluginStateObject.Molecule.Structure(s, props);
    },
    interpolate: function (src, tar, t) {
        // TODO: optimize
        var u = linear_algebra_1.Mat4.fromRotation(linear_algebra_1.Mat4.zero(), Math.PI / 180 * src.angle, linear_algebra_1.Vec3.normalize(linear_algebra_1.Vec3.zero(), src.axis));
        linear_algebra_1.Mat4.setTranslation(u, src.translation);
        var v = linear_algebra_1.Mat4.fromRotation(linear_algebra_1.Mat4.zero(), Math.PI / 180 * tar.angle, linear_algebra_1.Vec3.normalize(linear_algebra_1.Vec3.zero(), tar.axis));
        linear_algebra_1.Mat4.setTranslation(v, tar.translation);
        var m = geometry_1.SymmetryOperator.slerp(linear_algebra_1.Mat4.zero(), u, v, t);
        var rot = linear_algebra_1.Mat4.getRotation(linear_algebra_1.Quat.zero(), m);
        var axis = linear_algebra_1.Vec3.zero();
        var angle = linear_algebra_1.Quat.getAxisAngle(axis, rot);
        var translation = linear_algebra_1.Mat4.getTranslation(linear_algebra_1.Vec3.zero(), m);
        return { axis: axis, angle: angle, translation: translation };
    }
});
exports.TransformStructureConformation = TransformStructureConformation;
var TransformStructureConformationByMatrix = objects_1.PluginStateTransform.BuiltIn({
    name: 'transform-structure-conformation-by-matrix',
    display: { name: 'Transform Conformation' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    to: objects_1.PluginStateObject.Molecule.Structure,
    params: {
        matrix: param_definition_1.ParamDefinition.Value(linear_algebra_1.Mat4.identity(), { isHidden: true })
    }
})({
    canAutoUpdate: function () {
        return true;
    },
    apply: function (_a) {
        var a = _a.a, params = _a.params;
        var s = structure_1.Structure.transform(a.data, params.matrix);
        var props = { label: "" + a.label, description: "Transformed" };
        return new objects_1.PluginStateObject.Molecule.Structure(s, props);
    }
});
exports.TransformStructureConformationByMatrix = TransformStructureConformationByMatrix;
var StructureSelection = objects_1.PluginStateTransform.BuiltIn({
    name: 'structure-selection',
    display: { name: 'Structure Selection', description: 'Create a molecular structure from the specified query expression.' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    to: objects_1.PluginStateObject.Molecule.Structure,
    params: {
        query: param_definition_1.ParamDefinition.Value(builder_1.MolScriptBuilder.struct.generator.all, { isHidden: true }),
        label: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Text('', { isHidden: true }))
    }
})({
    apply: function (_a) {
        var a = _a.a, params = _a.params, cache = _a.cache;
        var compiled = compiler_1.compile(params.query);
        cache.compiled = compiled;
        cache.source = a.data;
        var result = compiled(new structure_1.QueryContext(a.data));
        var s = structure_1.StructureSelection.unionStructure(result);
        if (s.elementCount === 0)
            return mol_state_1.StateObject.Null;
        var props = { label: "" + (params.label || 'Selection'), description: structureDesc(s) };
        return new objects_1.PluginStateObject.Molecule.Structure(s, props);
    },
    update: function (_a) {
        var a = _a.a, b = _a.b, oldParams = _a.oldParams, newParams = _a.newParams, cache = _a.cache;
        if (oldParams.query !== newParams.query)
            return mol_state_1.StateTransformer.UpdateResult.Recreate;
        if (cache.source === a.data) {
            return mol_state_1.StateTransformer.UpdateResult.Unchanged;
        }
        cache.source = a.data;
        if (updateStructureFromQuery(cache.compiled, a.data, b, newParams.label)) {
            return mol_state_1.StateTransformer.UpdateResult.Updated;
        }
        return mol_state_1.StateTransformer.UpdateResult.Null;
    }
});
exports.StructureSelection = StructureSelection;
var UserStructureSelection = objects_1.PluginStateTransform.BuiltIn({
    name: 'user-structure-selection',
    display: { name: 'Structure Selection', description: 'Create a molecular structure from the specified query expression.' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    to: objects_1.PluginStateObject.Molecule.Structure,
    params: {
        query: param_definition_1.ParamDefinition.ScriptExpression({ language: 'mol-script', expression: '(sel.atom.atom-groups :residue-test (= atom.resname ALA))' }),
        label: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Text(''))
    }
})({
    apply: function (_a) {
        var a = _a.a, params = _a.params, cache = _a.cache;
        var parsed = parser_3.parseMolScript(params.query.expression);
        if (parsed.length === 0)
            throw new Error('No query');
        var query = symbols_1.transpileMolScript(parsed[0]);
        var compiled = compiler_1.compile(query);
        cache.compiled = compiled;
        cache.source = a.data;
        var result = compiled(new structure_1.QueryContext(a.data));
        var s = structure_1.StructureSelection.unionStructure(result);
        var props = { label: "" + (params.label || 'Selection'), description: structureDesc(s) };
        return new objects_1.PluginStateObject.Molecule.Structure(s, props);
    },
    update: function (_a) {
        var a = _a.a, b = _a.b, oldParams = _a.oldParams, newParams = _a.newParams, cache = _a.cache;
        if (oldParams.query.language !== newParams.query.language || oldParams.query.expression !== newParams.query.expression) {
            return mol_state_1.StateTransformer.UpdateResult.Recreate;
        }
        if (cache.source === a.data) {
            return mol_state_1.StateTransformer.UpdateResult.Unchanged;
        }
        cache.source = a.data;
        updateStructureFromQuery(cache.compiled, a.data, b, newParams.label);
        return mol_state_1.StateTransformer.UpdateResult.Updated;
    }
});
exports.UserStructureSelection = UserStructureSelection;
function updateStructureFromQuery(query, src, obj, label) {
    var result = query(new structure_1.QueryContext(src));
    var s = structure_1.StructureSelection.unionStructure(result);
    if (s.elementCount === 0) {
        return false;
    }
    obj.label = "" + (label || 'Selection');
    obj.description = structureDesc(s);
    obj.data = s;
    return true;
}
var StructureComplexElementTypes = ['atomic-sequence', 'water', 'atomic-het', 'spheres'].map(function (t) { return [t, t]; });
var StructureComplexElement = objects_1.PluginStateTransform.BuiltIn({
    name: 'structure-complex-element',
    display: { name: 'Complex Element', description: 'Create a molecular structure from the specified model.' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    to: objects_1.PluginStateObject.Molecule.Structure,
    params: { type: param_definition_1.ParamDefinition.Select('atomic-sequence', StructureComplexElementTypes, { isHidden: true }) }
})({
    apply: function (_a) {
        // TODO: update function.
        var a = _a.a, params = _a.params;
        var query, label;
        switch (params.type) {
            case 'atomic-sequence':
                query = structure_1.Queries.internal.atomicSequence();
                label = 'Sequence';
                break;
            case 'water':
                query = structure_1.Queries.internal.water();
                label = 'Water';
                break;
            case 'atomic-het':
                query = structure_1.Queries.internal.atomicHet();
                label = 'HET Groups/Ligands';
                break;
            case 'spheres':
                query = structure_1.Queries.internal.spheres();
                label = 'Coarse Spheres';
                break;
            default: throw new Error(params.type + " is a not valid complex element.");
        }
        var result = query(new structure_1.QueryContext(a.data));
        var s = structure_1.StructureSelection.unionStructure(result);
        if (s.elementCount === 0)
            return mol_state_1.StateObject.Null;
        return new objects_1.PluginStateObject.Molecule.Structure(s, { label: label, description: structureDesc(s) });
    }
});
exports.StructureComplexElement = StructureComplexElement;
var CustomModelProperties = objects_1.PluginStateTransform.BuiltIn({
    name: 'custom-model-properties',
    display: { name: 'Custom Model Properties' },
    from: objects_1.PluginStateObject.Molecule.Model,
    to: objects_1.PluginStateObject.Molecule.Model,
    params: function (a, ctx) {
        if (!a)
            return { properties: param_definition_1.ParamDefinition.MultiSelect([], [], { description: 'A list of property descriptor ids.' }) };
        return { properties: ctx.customModelProperties.getSelect(a.data) };
    }
})({
    apply: function (_a, ctx) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Custom Props', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, attachModelProps(a.data, ctx, taskCtx, params.properties)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Model(a.data, { label: 'Model Props', description: params.properties.length + " Selected" })];
                }
            });
        }); });
    }
});
exports.CustomModelProperties = CustomModelProperties;
function attachModelProps(model, ctx, taskCtx, names) {
    return __awaiter(this, void 0, void 0, function () {
        var _i, names_1, name_1, p;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _i = 0, names_1 = names;
                    _a.label = 1;
                case 1:
                    if (!(_i < names_1.length)) return [3 /*break*/, 4];
                    name_1 = names_1[_i];
                    p = ctx.customModelProperties.get(name_1);
                    return [4 /*yield*/, p.attach(model).runInContext(taskCtx)];
                case 2:
                    _a.sent();
                    _a.label = 3;
                case 3:
                    _i++;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/];
            }
        });
    });
}
var CustomStructureProperties = objects_1.PluginStateTransform.BuiltIn({
    name: 'custom-structure-properties',
    display: { name: 'Custom Structure Properties' },
    from: objects_1.PluginStateObject.Molecule.Structure,
    to: objects_1.PluginStateObject.Molecule.Structure,
    params: function (a, ctx) {
        if (!a)
            return { properties: param_definition_1.ParamDefinition.MultiSelect([], [], { description: 'A list of property descriptor ids.' }) };
        return { properties: ctx.customStructureProperties.getSelect(a.data) };
    }
})({
    apply: function (_a, ctx) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Custom Props', function (taskCtx) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, attachStructureProps(a.data, ctx, taskCtx, params.properties)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Structure(a.data, { label: 'Structure Props', description: params.properties.length + " Selected" })];
                }
            });
        }); });
    }
});
exports.CustomStructureProperties = CustomStructureProperties;
function attachStructureProps(structure, ctx, taskCtx, names) {
    return __awaiter(this, void 0, void 0, function () {
        var _i, names_2, name_2, p;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _i = 0, names_2 = names;
                    _a.label = 1;
                case 1:
                    if (!(_i < names_2.length)) return [3 /*break*/, 4];
                    name_2 = names_2[_i];
                    p = ctx.customStructureProperties.get(name_2);
                    return [4 /*yield*/, p.attach(structure).runInContext(taskCtx)];
                case 2:
                    _a.sent();
                    _a.label = 3;
                case 3:
                    _i++;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/];
            }
        });
    });
}
var ShapeFromPly = objects_1.PluginStateTransform.BuiltIn({
    name: 'shape-from-ply',
    display: { name: 'Shape from PLY', description: 'Create Shape from PLY data' },
    from: objects_1.PluginStateObject.Format.Ply,
    to: objects_1.PluginStateObject.Shape.Provider,
    params: function (a) {
        return {};
    }
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Create shape from PLY', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var shape, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, ply_1.shapeFromPly(a.data, params).runInContext(ctx)];
                    case 1:
                        shape = _a.sent();
                        props = { label: 'Shape' };
                        return [2 /*return*/, new objects_1.PluginStateObject.Shape.Provider(shape, props)];
                }
            });
        }); });
    }
});
exports.ShapeFromPly = ShapeFromPly;
