/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Structure } from '../../../mol-model/structure';
import { VolumeData } from '../../../mol-model/volume';
import { PluginContext } from '../../../mol-plugin/context';
import { RepresentationProvider } from '../../../mol-repr/representation';
import { BuiltInStructureRepresentationsName } from '../../../mol-repr/structure/registry';
import { StructureParams } from '../../../mol-repr/structure/representation';
import { BuiltInVolumeRepresentationsName } from '../../../mol-repr/volume/registry';
import { VolumeParams } from '../../../mol-repr/volume/representation';
import { StateTransformer } from '../../../mol-state';
import { BuiltInColorThemeName, ColorTheme } from '../../../mol-theme/color';
import { BuiltInSizeThemeName, SizeTheme } from '../../../mol-theme/size';
import { ThemeRegistryContext } from '../../../mol-theme/theme';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { PluginStateObject as SO } from '../objects';
import { Color } from '../../../mol-util/color';
import { Transparency } from '../../../mol-theme/transparency';
export { StructureRepresentation3D };
export { StructureRepresentation3DHelpers };
export { StructureLabels3D };
export { ExplodeStructureRepresentation3D };
export { UnwindStructureAssemblyRepresentation3D };
export { OverpaintStructureRepresentation3D };
export { TransparencyStructureRepresentation3D };
export { VolumeRepresentation3D };
declare namespace StructureRepresentation3DHelpers {
    function getDefaultParams(ctx: PluginContext, name: BuiltInStructureRepresentationsName, structure: Structure, structureParams?: Partial<PD.Values<StructureParams>>): StateTransformer.Params<StructureRepresentation3D>;
    function createParams<R extends RepresentationProvider<Structure, any, any>, C extends ColorTheme.Provider<any>, S extends SizeTheme.Provider<any>>(ctx: PluginContext, structure: Structure, params: {
        repr?: R | [R, (r: R, ctx: ThemeRegistryContext, s: Structure) => RepresentationProvider.ParamValues<R>];
        color?: C | [C, (c: C, ctx: ThemeRegistryContext) => ColorTheme.ParamValues<C>];
        size?: S | [S, (c: S, ctx: ThemeRegistryContext) => SizeTheme.ParamValues<S>];
    }): StateTransformer.Params<StructureRepresentation3D>;
    function getDefaultParamsWithTheme(ctx: PluginContext, reprName: BuiltInStructureRepresentationsName, colorName: BuiltInColorThemeName | undefined, structure: Structure, structureParams?: Partial<PD.Values<StructureParams>>): StateTransformer.Params<StructureRepresentation3D>;
    function getDefaultParamsStatic(ctx: PluginContext, name: BuiltInStructureRepresentationsName, structureParams?: Partial<PD.Values<StructureParams>>, colorName?: BuiltInColorThemeName): StateTransformer.Params<StructureRepresentation3D>;
}
declare type StructureRepresentation3D = typeof StructureRepresentation3D;
declare const StructureRepresentation3D: StateTransformer<SO.Molecule.Structure, SO.Molecule.Structure.Representation3D, PD.Normalize<{
    type: PD.NamedParams<any, string>;
    colorTheme: PD.NamedParams<any, string>;
    sizeTheme: PD.NamedParams<any, string>;
}>>;
declare type StructureLabels3D = typeof StructureLabels3D;
declare const StructureLabels3D: StateTransformer<SO.Molecule.Structure, SO.Molecule.Structure.Representation3D, PD.Normalize<{
    target: PD.NamedParams<PD.Normalize<unknown>, "elements"> | PD.NamedParams<PD.Normalize<unknown>, "residues"> | PD.NamedParams<PD.Normalize<{
        value: any;
        size: any;
        position: any;
    }>, "static-text">;
    options: PD.Normalize<{
        background: any;
        backgroundMargin: any;
        backgroundColor: any;
        backgroundOpacity: any;
        sizeFactor: any;
        borderWidth: any;
        borderColor: any;
        offsetX: any;
        offsetY: any;
        offsetZ: any;
        tether: any;
        tetherLength: any;
        tetherBaseWidth: any;
        attachment: any;
        fontFamily: any;
        fontQuality: any;
        fontStyle: any;
        fontVariant: any;
        fontWeight: any;
        alpha: any;
        useFog: any;
        highlightColor: any;
        selectColor: any;
        quality: any;
    }>;
}>>;
declare type UnwindStructureAssemblyRepresentation3D = typeof UnwindStructureAssemblyRepresentation3D;
declare const UnwindStructureAssemblyRepresentation3D: StateTransformer<SO.Molecule.Structure.Representation3D, SO.Molecule.Structure.Representation3DState, PD.Normalize<{
    t: number;
}>>;
declare type ExplodeStructureRepresentation3D = typeof ExplodeStructureRepresentation3D;
declare const ExplodeStructureRepresentation3D: StateTransformer<SO.Molecule.Structure.Representation3D, SO.Molecule.Structure.Representation3DState, PD.Normalize<{
    t: number;
}>>;
declare type OverpaintStructureRepresentation3D = typeof OverpaintStructureRepresentation3D;
declare const OverpaintStructureRepresentation3D: StateTransformer<SO.Molecule.Structure.Representation3D, SO.Molecule.Structure.Representation3DState, PD.Normalize<{
    layers: PD.Normalize<{
        script: {
            language: string;
            expression: string;
        };
        color: Color;
    }>[];
    alpha: number;
}>>;
declare type TransparencyStructureRepresentation3D = typeof TransparencyStructureRepresentation3D;
declare const TransparencyStructureRepresentation3D: StateTransformer<SO.Molecule.Structure.Representation3D, SO.Molecule.Structure.Representation3DState, PD.Normalize<{
    script: {
        language: "mol-script";
        expression: string;
    };
    value: number;
    variant: Transparency.Variant;
}>>;
export declare namespace VolumeRepresentation3DHelpers {
    function getDefaultParams(ctx: PluginContext, name: BuiltInVolumeRepresentationsName, volume: VolumeData, volumeParams?: Partial<PD.Values<VolumeParams>>): StateTransformer.Params<VolumeRepresentation3D>;
    function getDefaultParamsStatic(ctx: PluginContext, name: BuiltInVolumeRepresentationsName, volumeParams?: Partial<PD.Values<PD.Params>>, colorName?: BuiltInColorThemeName, colorParams?: Partial<ColorTheme.Props>, sizeName?: BuiltInSizeThemeName, sizeParams?: Partial<SizeTheme.Props>): StateTransformer.Params<VolumeRepresentation3D>;
    function getDescription(props: any): string;
}
declare type VolumeRepresentation3D = typeof VolumeRepresentation3D;
declare const VolumeRepresentation3D: StateTransformer<SO.Volume.Data, SO.Volume.Representation3D, PD.Normalize<{
    type: PD.NamedParams<any, string>;
    colorTheme: PD.NamedParams<any, string>;
    sizeTheme: PD.NamedParams<any, string>;
}>>;
export { ShapeRepresentation3D };
declare type ShapeRepresentation3D = typeof ShapeRepresentation3D;
declare const ShapeRepresentation3D: StateTransformer<SO.Shape.Provider, SO.Shape.Representation3D, PD.Normalize<{}>>;
