"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../../mol-model/structure");
var volume_1 = require("../../../mol-model/volume");
var mol_state_1 = require("../../../mol-state");
var mol_task_1 = require("../../../mol-task");
var color_1 = require("../../../mol-theme/color");
var theme_1 = require("../../../mol-theme/theme");
var param_definition_1 = require("../../../mol-util/param-definition");
var objects_1 = require("../objects");
var text_1 = require("../../../mol-geo/geometry/text/text");
var tables_1 = require("../../../mol-util/color/tables");
var structure_labels_1 = require("../../../mol-plugin/util/structure-labels");
var representation_1 = require("../../../mol-repr/shape/representation");
var unit_transforms_1 = require("../../../mol-model/structure/structure/util/unit-transforms");
var helpers_1 = require("../animation/helpers");
var color_2 = require("../../../mol-util/color");
var overpaint_1 = require("../../../mol-theme/overpaint");
var transparency_1 = require("../../../mol-theme/transparency");
var helpers_2 = require("./helpers");
var base_1 = require("../../../mol-geo/geometry/base");
var StructureRepresentation3DHelpers;
(function (StructureRepresentation3DHelpers) {
    function getDefaultParams(ctx, name, structure, structureParams) {
        var type = ctx.structureRepresentation.registry.get(name);
        var themeDataCtx = { structure: structure };
        var colorParams = ctx.structureRepresentation.themeCtx.colorThemeRegistry.get(type.defaultColorTheme).getParams(themeDataCtx);
        var sizeParams = ctx.structureRepresentation.themeCtx.sizeThemeRegistry.get(type.defaultSizeTheme).getParams(themeDataCtx);
        var structureDefaultParams = param_definition_1.ParamDefinition.getDefaultValues(type.getParams(ctx.structureRepresentation.themeCtx, structure));
        return ({
            type: { name: name, params: structureParams ? __assign({}, structureDefaultParams, structureParams) : structureDefaultParams },
            colorTheme: { name: type.defaultColorTheme, params: param_definition_1.ParamDefinition.getDefaultValues(colorParams) },
            sizeTheme: { name: type.defaultSizeTheme, params: param_definition_1.ParamDefinition.getDefaultValues(sizeParams) }
        });
    }
    StructureRepresentation3DHelpers.getDefaultParams = getDefaultParams;
    function createParams(ctx, structure, params) {
        var themeCtx = ctx.structureRepresentation.themeCtx;
        var repr = params.repr
            ? params.repr instanceof Array ? params.repr[0] : params.repr
            : ctx.structureRepresentation.registry.default.provider;
        var reprParams = params.repr instanceof Array
            ? params.repr[1](repr, themeCtx, structure)
            : param_definition_1.ParamDefinition.getDefaultValues(repr.getParams(themeCtx, structure));
        var color = params.color
            ? params.color instanceof Array ? params.color[0] : params.color
            : themeCtx.colorThemeRegistry.get(repr.defaultColorTheme);
        var colorParams = params.color instanceof Array
            ? params.color[1](color, themeCtx)
            : param_definition_1.ParamDefinition.getDefaultValues(color.getParams(themeCtx));
        var size = params.size
            ? params.size instanceof Array ? params.size[0] : params.size
            : themeCtx.sizeThemeRegistry.get(repr.defaultSizeTheme);
        var sizeParams = params.size instanceof Array
            ? params.size[1](size, themeCtx)
            : param_definition_1.ParamDefinition.getDefaultValues(size.getParams(themeCtx));
        return ({
            type: { name: ctx.structureRepresentation.registry.getName(repr), params: reprParams },
            colorTheme: { name: themeCtx.colorThemeRegistry.getName(color), params: colorParams },
            sizeTheme: { name: themeCtx.sizeThemeRegistry.getName(size), params: sizeParams }
        });
    }
    StructureRepresentation3DHelpers.createParams = createParams;
    function getDefaultParamsWithTheme(ctx, reprName, colorName, structure, structureParams) {
        var type = ctx.structureRepresentation.registry.get(reprName);
        var themeDataCtx = { structure: structure };
        var color = colorName || type.defaultColorTheme;
        var colorParams = ctx.structureRepresentation.themeCtx.colorThemeRegistry.get(color).getParams(themeDataCtx);
        var sizeParams = ctx.structureRepresentation.themeCtx.sizeThemeRegistry.get(type.defaultSizeTheme).getParams(themeDataCtx);
        var structureDefaultParams = param_definition_1.ParamDefinition.getDefaultValues(type.getParams(ctx.structureRepresentation.themeCtx, structure));
        return ({
            type: { name: reprName, params: structureParams ? __assign({}, structureDefaultParams, structureParams) : structureDefaultParams },
            colorTheme: { name: color, params: param_definition_1.ParamDefinition.getDefaultValues(colorParams) },
            sizeTheme: { name: type.defaultSizeTheme, params: param_definition_1.ParamDefinition.getDefaultValues(sizeParams) }
        });
    }
    StructureRepresentation3DHelpers.getDefaultParamsWithTheme = getDefaultParamsWithTheme;
    function getDefaultParamsStatic(ctx, name, structureParams, colorName) {
        var type = ctx.structureRepresentation.registry.get(name);
        var color = colorName || type.defaultColorTheme;
        var colorParams = ctx.structureRepresentation.themeCtx.colorThemeRegistry.get(color).defaultValues;
        var sizeParams = ctx.structureRepresentation.themeCtx.sizeThemeRegistry.get(type.defaultSizeTheme).defaultValues;
        return ({
            type: { name: name, params: structureParams ? __assign({}, type.defaultValues, structureParams) : type.defaultValues },
            colorTheme: { name: color, params: colorParams },
            sizeTheme: { name: type.defaultSizeTheme, params: sizeParams }
        });
    }
    StructureRepresentation3DHelpers.getDefaultParamsStatic = getDefaultParamsStatic;
})(StructureRepresentation3DHelpers || (StructureRepresentation3DHelpers = {}));
exports.StructureRepresentation3DHelpers = StructureRepresentation3DHelpers;
var StructureRepresentation3D = objects_1.PluginStateTransform.BuiltIn({
    name: 'structure-representation-3d',
    display: '3D Representation',
    from: objects_1.PluginStateObject.Molecule.Structure,
    to: objects_1.PluginStateObject.Molecule.Structure.Representation3D,
    params: function (a, ctx) {
        var _a = ctx.structureRepresentation, registry = _a.registry, themeCtx = _a.themeCtx;
        var type = registry.get(registry.default.name);
        if (!a) {
            return {
                type: param_definition_1.ParamDefinition.Mapped(registry.default.name, registry.types, function (name) { return param_definition_1.ParamDefinition.Group(registry.get(name).getParams(themeCtx, structure_1.Structure.Empty)); }),
                colorTheme: param_definition_1.ParamDefinition.Mapped(type.defaultColorTheme, themeCtx.colorThemeRegistry.types, function (name) { return param_definition_1.ParamDefinition.Group(themeCtx.colorThemeRegistry.get(name).getParams({ structure: structure_1.Structure.Empty })); }),
                sizeTheme: param_definition_1.ParamDefinition.Mapped(type.defaultSizeTheme, themeCtx.sizeThemeRegistry.types, function (name) { return param_definition_1.ParamDefinition.Group(themeCtx.sizeThemeRegistry.get(name).getParams({ structure: structure_1.Structure.Empty })); })
            };
        }
        var dataCtx = { structure: a.data };
        return ({
            type: param_definition_1.ParamDefinition.Mapped(registry.default.name, registry.types, function (name) { return param_definition_1.ParamDefinition.Group(registry.get(name).getParams(themeCtx, a.data)); }),
            colorTheme: param_definition_1.ParamDefinition.Mapped(type.defaultColorTheme, themeCtx.colorThemeRegistry.getApplicableTypes(dataCtx), function (name) { return param_definition_1.ParamDefinition.Group(themeCtx.colorThemeRegistry.get(name).getParams(dataCtx)); }),
            sizeTheme: param_definition_1.ParamDefinition.Mapped(type.defaultSizeTheme, themeCtx.sizeThemeRegistry.types, function (name) { return param_definition_1.ParamDefinition.Group(themeCtx.sizeThemeRegistry.get(name).getParams(dataCtx)); })
        });
    }
})({
    canAutoUpdate: function (_a) {
        var a = _a.a, oldParams = _a.oldParams, newParams = _a.newParams;
        // TODO: other criteria as well?
        return a.data.elementCount < 10000 || oldParams.type.name === newParams.type.name;
    },
    apply: function (_a, plugin) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Structure Representation', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var provider, props, repr;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        provider = plugin.structureRepresentation.registry.get(params.type.name);
                        props = params.type.params || {};
                        repr = provider.factory(__assign({ webgl: plugin.canvas3d.webgl }, plugin.structureRepresentation.themeCtx), provider.getParams);
                        repr.setTheme(theme_1.createTheme(plugin.structureRepresentation.themeCtx, { structure: a.data }, params));
                        // TODO set initial state, repr.setState({})
                        return [4 /*yield*/, repr.createOrUpdate(props, a.data).runInContext(ctx)];
                    case 1:
                        // TODO set initial state, repr.setState({})
                        _a.sent();
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Structure.Representation3D({ repr: repr, source: a }, { label: provider.label })];
                }
            });
        }); });
    },
    update: function (_a, plugin) {
        var _this = this;
        var a = _a.a, b = _a.b, oldParams = _a.oldParams, newParams = _a.newParams;
        return mol_task_1.Task.create('Structure Representation', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (newParams.type.name !== oldParams.type.name)
                            return [2 /*return*/, mol_state_1.StateTransformer.UpdateResult.Recreate];
                        props = __assign({}, b.data.repr.props, newParams.type.params);
                        b.data.repr.setTheme(theme_1.createTheme(plugin.structureRepresentation.themeCtx, { structure: a.data }, newParams));
                        return [4 /*yield*/, b.data.repr.createOrUpdate(props, a.data).runInContext(ctx)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, mol_state_1.StateTransformer.UpdateResult.Updated];
                }
            });
        }); });
    },
    interpolate: function (src, tar, t) {
        if (src.colorTheme.name !== 'uniform' || tar.colorTheme.name !== 'uniform') {
            return t <= 0.5 ? src : tar;
        }
        color_1.BuiltInColorThemes;
        var from = src.colorTheme.params.value, to = tar.colorTheme.params.value;
        var value = color_2.Color.interpolate(from, to, t);
        return {
            type: t <= 0.5 ? src.type : tar.type,
            colorTheme: { name: 'uniform', params: { value: value } },
            sizeTheme: t <= 0.5 ? src.sizeTheme : tar.sizeTheme,
        };
    }
});
exports.StructureRepresentation3D = StructureRepresentation3D;
var StructureLabels3D = objects_1.PluginStateTransform.BuiltIn({
    name: 'structure-labels-3d',
    display: '3D Labels',
    from: objects_1.PluginStateObject.Molecule.Structure,
    to: objects_1.PluginStateObject.Molecule.Structure.Representation3D,
    params: {
        // TODO: other targets
        target: param_definition_1.ParamDefinition.MappedStatic('residues', {
            'elements': param_definition_1.ParamDefinition.Group({}),
            'residues': param_definition_1.ParamDefinition.Group({}),
            'static-text': param_definition_1.ParamDefinition.Group({
                value: param_definition_1.ParamDefinition.Text(''),
                size: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Numeric(1, { min: 1, max: 1000, step: 0.1 })),
                // TODO: this changes the position while rotated etc... fix
                position: param_definition_1.ParamDefinition.Optional(text_1.Text.Params.attachment)
            }, { isFlat: true })
        }),
        options: param_definition_1.ParamDefinition.Group(__assign({}, text_1.Text.Params, { background: param_definition_1.ParamDefinition.Boolean(true), backgroundMargin: param_definition_1.ParamDefinition.Numeric(0.2, { min: 0, max: 1, step: 0.01 }), backgroundColor: param_definition_1.ParamDefinition.Color(tables_1.ColorNames.snow), backgroundOpacity: param_definition_1.ParamDefinition.Numeric(0.9, { min: 0, max: 1, step: 0.01 }) }))
    }
})({
    canAutoUpdate: function (_a) {
        var oldParams = _a.oldParams, newParams = _a.newParams;
        return (oldParams.target.name === 'static-text' && newParams.target.name === 'static-text' && oldParams.target.params.value === newParams.target.params.value)
            || newParams.target.name === oldParams.target.name;
    },
    apply: function (_a) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Structure Labels', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var repr;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, structure_labels_1.getLabelRepresentation(ctx, a.data, params)];
                    case 1:
                        repr = _a.sent();
                        return [2 /*return*/, new objects_1.PluginStateObject.Molecule.Structure.Representation3D({ repr: repr, source: a }, { label: "Labels", description: params.target.name })];
                }
            });
        }); });
    },
    update: function (_a) {
        var _this = this;
        var a = _a.a, b = _a.b, newParams = _a.newParams;
        return mol_task_1.Task.create('Structure Labels', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, structure_labels_1.getLabelRepresentation(ctx, a.data, newParams, b.data.repr)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, mol_state_1.StateTransformer.UpdateResult.Updated];
                }
            });
        }); });
    }
});
exports.StructureLabels3D = StructureLabels3D;
var UnwindStructureAssemblyRepresentation3D = objects_1.PluginStateTransform.BuiltIn({
    name: 'unwind-structure-assembly-representation-3d',
    display: 'Unwind Assembly 3D Representation',
    from: objects_1.PluginStateObject.Molecule.Structure.Representation3D,
    to: objects_1.PluginStateObject.Molecule.Structure.Representation3DState,
    params: { t: param_definition_1.ParamDefinition.Numeric(0, { min: 0, max: 1, step: 0.01 }) }
})({
    canAutoUpdate: function () {
        return true;
    },
    apply: function (_a) {
        var a = _a.a, params = _a.params;
        var structure = a.data.source.data;
        var unitTransforms = new unit_transforms_1.StructureUnitTransforms(structure);
        helpers_1.unwindStructureAssembly(structure, unitTransforms, params.t);
        return new objects_1.PluginStateObject.Molecule.Structure.Representation3DState({
            state: { unitTransforms: unitTransforms },
            initialState: { unitTransforms: new unit_transforms_1.StructureUnitTransforms(structure) },
            info: structure,
            source: a
        }, { label: "Unwind T = " + params.t.toFixed(2) });
    },
    update: function (_a) {
        var a = _a.a, b = _a.b, newParams = _a.newParams, oldParams = _a.oldParams;
        var structure = b.data.info;
        if (a.data.source.data !== structure)
            return mol_state_1.StateTransformer.UpdateResult.Recreate;
        if (oldParams.t === newParams.t)
            return mol_state_1.StateTransformer.UpdateResult.Unchanged;
        var unitTransforms = b.data.state.unitTransforms;
        helpers_1.unwindStructureAssembly(structure, unitTransforms, newParams.t);
        b.label = "Unwind T = " + newParams.t.toFixed(2);
        b.data.source = a;
        return mol_state_1.StateTransformer.UpdateResult.Updated;
    }
});
exports.UnwindStructureAssemblyRepresentation3D = UnwindStructureAssemblyRepresentation3D;
var ExplodeStructureRepresentation3D = objects_1.PluginStateTransform.BuiltIn({
    name: 'explode-structure-representation-3d',
    display: 'Explode 3D Representation',
    from: objects_1.PluginStateObject.Molecule.Structure.Representation3D,
    to: objects_1.PluginStateObject.Molecule.Structure.Representation3DState,
    params: { t: param_definition_1.ParamDefinition.Numeric(0, { min: 0, max: 1, step: 0.01 }) }
})({
    canAutoUpdate: function () {
        return true;
    },
    apply: function (_a) {
        var a = _a.a, params = _a.params, spine = _a.spine;
        var rootStructure = spine.getRootOfType(objects_1.PluginStateObject.Molecule.Structure).data;
        var structure = a.data.source.data;
        var unitTransforms = new unit_transforms_1.StructureUnitTransforms(rootStructure);
        helpers_1.explodeStructure(structure, unitTransforms, params.t);
        return new objects_1.PluginStateObject.Molecule.Structure.Representation3DState({
            state: { unitTransforms: unitTransforms },
            initialState: { unitTransforms: new unit_transforms_1.StructureUnitTransforms(rootStructure) },
            info: rootStructure,
            source: a
        }, { label: "Explode T = " + params.t.toFixed(2) });
    },
    update: function (_a) {
        var a = _a.a, b = _a.b, newParams = _a.newParams, oldParams = _a.oldParams, spine = _a.spine;
        var rootStructure = spine.getRootOfType(objects_1.PluginStateObject.Molecule.Structure).data;
        if (b.data.info !== rootStructure)
            return mol_state_1.StateTransformer.UpdateResult.Recreate;
        if (oldParams.t === newParams.t)
            return mol_state_1.StateTransformer.UpdateResult.Unchanged;
        var unitTransforms = b.data.state.unitTransforms;
        helpers_1.explodeStructure(rootStructure, unitTransforms, newParams.t);
        b.label = "Explode T = " + newParams.t.toFixed(2);
        b.data.source = a;
        return mol_state_1.StateTransformer.UpdateResult.Updated;
    }
});
exports.ExplodeStructureRepresentation3D = ExplodeStructureRepresentation3D;
var OverpaintStructureRepresentation3D = objects_1.PluginStateTransform.BuiltIn({
    name: 'overpaint-structure-representation-3d',
    display: 'Overpaint 3D Representation',
    from: objects_1.PluginStateObject.Molecule.Structure.Representation3D,
    to: objects_1.PluginStateObject.Molecule.Structure.Representation3DState,
    params: {
        layers: param_definition_1.ParamDefinition.ObjectList({
            script: param_definition_1.ParamDefinition.ScriptExpression({ language: 'mol-script', expression: '(sel.atom.atom-groups :residue-test (= atom.resname LYS))' }),
            color: param_definition_1.ParamDefinition.Color(tables_1.ColorNames.blueviolet)
        }, function (e) { return "" + color_2.Color.toRgbString(e.color); }, {
            defaultValue: [
                {
                    script: {
                        language: 'mol-script',
                        expression: '(sel.atom.atom-groups :residue-test (= atom.resname LYS))'
                    },
                    color: tables_1.ColorNames.blueviolet
                },
                {
                    script: {
                        language: 'mol-script',
                        expression: '(sel.atom.atom-groups :residue-test (= atom.resname ALA))'
                    },
                    color: tables_1.ColorNames.chartreuse
                }
            ]
        }),
        alpha: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 1, step: 0.01 }, { label: 'Opacity' }),
    }
})({
    canAutoUpdate: function () {
        return true;
    },
    apply: function (_a) {
        var a = _a.a, params = _a.params;
        var structure = a.data.source.data;
        var overpaint = helpers_2.getStructureOverpaint(structure, params.layers, params.alpha);
        return new objects_1.PluginStateObject.Molecule.Structure.Representation3DState({
            state: { overpaint: overpaint },
            initialState: { overpaint: overpaint_1.Overpaint.Empty },
            info: structure,
            source: a
        }, { label: "Overpaint (" + overpaint.layers.length + " Layers)" });
    },
    update: function (_a) {
        var a = _a.a, b = _a.b, newParams = _a.newParams, oldParams = _a.oldParams;
        var structure = b.data.info;
        if (a.data.source.data !== structure)
            return mol_state_1.StateTransformer.UpdateResult.Recreate;
        var oldOverpaint = b.data.state.overpaint;
        var newOverpaint = helpers_2.getStructureOverpaint(structure, newParams.layers, newParams.alpha);
        if (oldParams.alpha === newParams.alpha && overpaint_1.Overpaint.areEqual(oldOverpaint, newOverpaint))
            return mol_state_1.StateTransformer.UpdateResult.Unchanged;
        b.data.state.overpaint = newOverpaint;
        b.data.source = a;
        b.label = "Overpaint (" + newOverpaint.layers.length + " Layers)";
        return mol_state_1.StateTransformer.UpdateResult.Updated;
    }
});
exports.OverpaintStructureRepresentation3D = OverpaintStructureRepresentation3D;
var TransparencyStructureRepresentation3D = objects_1.PluginStateTransform.BuiltIn({
    name: 'transparency-structure-representation-3d',
    display: 'Transparency 3D Representation',
    from: objects_1.PluginStateObject.Molecule.Structure.Representation3D,
    to: objects_1.PluginStateObject.Molecule.Structure.Representation3DState,
    params: {
        script: param_definition_1.ParamDefinition.ScriptExpression({ language: 'mol-script', expression: '(sel.atom.atom-groups :chain-test (= atom.label_asym_id A))' }),
        value: param_definition_1.ParamDefinition.Numeric(0.75, { min: 0, max: 1, step: 0.01 }, { label: 'Transparency' }),
        variant: param_definition_1.ParamDefinition.Select('single', [['single', 'Single-layer'], ['multi', 'Multi-layer']])
    }
})({
    canAutoUpdate: function () {
        return true;
    },
    apply: function (_a) {
        var a = _a.a, params = _a.params;
        var structure = a.data.source.data;
        var transparency = helpers_2.getStructureTransparency(structure, params.script, params.value, params.variant);
        return new objects_1.PluginStateObject.Molecule.Structure.Representation3DState({
            state: { transparency: transparency },
            initialState: { transparency: transparency_1.Transparency.Empty },
            info: structure,
            source: a
        }, { label: "Transparency (" + transparency.value + ")" });
    },
    update: function (_a) {
        var a = _a.a, b = _a.b, newParams = _a.newParams, oldParams = _a.oldParams;
        var structure = b.data.info;
        if (a.data.source.data !== structure)
            return mol_state_1.StateTransformer.UpdateResult.Recreate;
        var oldTransparency = b.data.state.transparency;
        var newTransparency = helpers_2.getStructureTransparency(structure, newParams.script, newParams.value, newParams.variant);
        if (transparency_1.Transparency.areEqual(oldTransparency, newTransparency))
            return mol_state_1.StateTransformer.UpdateResult.Unchanged;
        b.data.state.transparency = newTransparency;
        b.data.source = a;
        b.label = "Transparency (" + newTransparency.value + ")";
        return mol_state_1.StateTransformer.UpdateResult.Updated;
    }
});
exports.TransparencyStructureRepresentation3D = TransparencyStructureRepresentation3D;
//
var VolumeRepresentation3DHelpers;
(function (VolumeRepresentation3DHelpers) {
    function getDefaultParams(ctx, name, volume, volumeParams) {
        var type = ctx.volumeRepresentation.registry.get(name);
        var themeDataCtx = { volume: volume };
        var colorParams = ctx.volumeRepresentation.themeCtx.colorThemeRegistry.get(type.defaultColorTheme).getParams(themeDataCtx);
        var sizeParams = ctx.volumeRepresentation.themeCtx.sizeThemeRegistry.get(type.defaultSizeTheme).getParams(themeDataCtx);
        var volumeDefaultParams = param_definition_1.ParamDefinition.getDefaultValues(type.getParams(ctx.volumeRepresentation.themeCtx, volume));
        return ({
            type: { name: name, params: volumeParams ? __assign({}, volumeDefaultParams, volumeParams) : volumeDefaultParams },
            colorTheme: { name: type.defaultColorTheme, params: param_definition_1.ParamDefinition.getDefaultValues(colorParams) },
            sizeTheme: { name: type.defaultSizeTheme, params: param_definition_1.ParamDefinition.getDefaultValues(sizeParams) }
        });
    }
    VolumeRepresentation3DHelpers.getDefaultParams = getDefaultParams;
    function getDefaultParamsStatic(ctx, name, volumeParams, colorName, colorParams, sizeName, sizeParams) {
        var type = ctx.volumeRepresentation.registry.get(name);
        var colorType = ctx.volumeRepresentation.themeCtx.colorThemeRegistry.get(colorName || type.defaultColorTheme);
        var sizeType = ctx.volumeRepresentation.themeCtx.sizeThemeRegistry.get(sizeName || type.defaultSizeTheme);
        return ({
            type: { name: name, params: volumeParams ? __assign({}, type.defaultValues, volumeParams) : type.defaultValues },
            colorTheme: { name: type.defaultColorTheme, params: colorParams ? __assign({}, colorType.defaultValues, colorParams) : colorType.defaultValues },
            sizeTheme: { name: type.defaultSizeTheme, params: sizeParams ? __assign({}, sizeType.defaultValues, sizeParams) : sizeType.defaultValues }
        });
    }
    VolumeRepresentation3DHelpers.getDefaultParamsStatic = getDefaultParamsStatic;
    function getDescription(props) {
        return props.isoValue && volume_1.VolumeIsoValue.toString(props.isoValue);
    }
    VolumeRepresentation3DHelpers.getDescription = getDescription;
})(VolumeRepresentation3DHelpers = exports.VolumeRepresentation3DHelpers || (exports.VolumeRepresentation3DHelpers = {}));
var VolumeRepresentation3D = objects_1.PluginStateTransform.BuiltIn({
    name: 'volume-representation-3d',
    display: '3D Representation',
    from: objects_1.PluginStateObject.Volume.Data,
    to: objects_1.PluginStateObject.Volume.Representation3D,
    params: function (a, ctx) {
        var _a = ctx.volumeRepresentation, registry = _a.registry, themeCtx = _a.themeCtx;
        var type = registry.get(registry.default.name);
        if (!a) {
            return {
                type: param_definition_1.ParamDefinition.Mapped(registry.default.name, registry.types, function (name) { return param_definition_1.ParamDefinition.Group(registry.get(name).getParams(themeCtx, volume_1.VolumeData.One)); }),
                colorTheme: param_definition_1.ParamDefinition.Mapped(type.defaultColorTheme, themeCtx.colorThemeRegistry.types, function (name) { return param_definition_1.ParamDefinition.Group(themeCtx.colorThemeRegistry.get(name).getParams({ volume: volume_1.VolumeData.One })); }),
                sizeTheme: param_definition_1.ParamDefinition.Mapped(type.defaultSizeTheme, themeCtx.sizeThemeRegistry.types, function (name) { return param_definition_1.ParamDefinition.Group(themeCtx.sizeThemeRegistry.get(name).getParams({ volume: volume_1.VolumeData.One })); })
            };
        }
        var dataCtx = { volume: a.data };
        return ({
            type: param_definition_1.ParamDefinition.Mapped(registry.default.name, registry.types, function (name) { return param_definition_1.ParamDefinition.Group(registry.get(name).getParams(themeCtx, a.data)); }),
            colorTheme: param_definition_1.ParamDefinition.Mapped(type.defaultColorTheme, themeCtx.colorThemeRegistry.getApplicableTypes(dataCtx), function (name) { return param_definition_1.ParamDefinition.Group(themeCtx.colorThemeRegistry.get(name).getParams(dataCtx)); }),
            sizeTheme: param_definition_1.ParamDefinition.Mapped(type.defaultSizeTheme, themeCtx.sizeThemeRegistry.types, function (name) { return param_definition_1.ParamDefinition.Group(themeCtx.sizeThemeRegistry.get(name).getParams(dataCtx)); })
        });
    }
})({
    canAutoUpdate: function (_a) {
        var oldParams = _a.oldParams, newParams = _a.newParams;
        // TODO: allow for small molecules
        return oldParams.type.name === newParams.type.name;
    },
    apply: function (_a, plugin) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Volume Representation', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var provider, props, repr;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        provider = plugin.volumeRepresentation.registry.get(params.type.name);
                        props = params.type.params || {};
                        repr = provider.factory(__assign({ webgl: plugin.canvas3d.webgl }, plugin.volumeRepresentation.themeCtx), provider.getParams);
                        repr.setTheme(theme_1.createTheme(plugin.volumeRepresentation.themeCtx, { volume: a.data }, params));
                        // TODO set initial state, repr.setState({})
                        return [4 /*yield*/, repr.createOrUpdate(props, a.data).runInContext(ctx)];
                    case 1:
                        // TODO set initial state, repr.setState({})
                        _a.sent();
                        return [2 /*return*/, new objects_1.PluginStateObject.Volume.Representation3D({ repr: repr, source: a }, { label: provider.label, description: VolumeRepresentation3DHelpers.getDescription(props) })];
                }
            });
        }); });
    },
    update: function (_a, plugin) {
        var _this = this;
        var a = _a.a, b = _a.b, oldParams = _a.oldParams, newParams = _a.newParams;
        return mol_task_1.Task.create('Volume Representation', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (newParams.type.name !== oldParams.type.name)
                            return [2 /*return*/, mol_state_1.StateTransformer.UpdateResult.Recreate];
                        props = __assign({}, b.data.repr.props, newParams.type.params);
                        b.data.repr.setTheme(theme_1.createTheme(plugin.volumeRepresentation.themeCtx, { volume: a.data }, newParams));
                        return [4 /*yield*/, b.data.repr.createOrUpdate(props, a.data).runInContext(ctx)];
                    case 1:
                        _a.sent();
                        b.description = VolumeRepresentation3DHelpers.getDescription(props);
                        return [2 /*return*/, mol_state_1.StateTransformer.UpdateResult.Updated];
                }
            });
        }); });
    }
});
exports.VolumeRepresentation3D = VolumeRepresentation3D;
var ShapeRepresentation3D = objects_1.PluginStateTransform.BuiltIn({
    name: 'shape-representation-3d',
    display: '3D Representation',
    from: objects_1.PluginStateObject.Shape.Provider,
    to: objects_1.PluginStateObject.Shape.Representation3D,
    params: function (a, ctx) {
        return a ? a.data.params : base_1.BaseGeometry.Params;
    }
})({
    canAutoUpdate: function () {
        return true;
    },
    apply: function (_a, plugin) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Shape Representation', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var props, repr;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        props = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(a.data.params), { params: params });
                        repr = representation_1.ShapeRepresentation(a.data.getShape, a.data.geometryUtils);
                        // TODO set initial state, repr.setState({})
                        return [4 /*yield*/, repr.createOrUpdate(props, a.data.data).runInContext(ctx)];
                    case 1:
                        // TODO set initial state, repr.setState({})
                        _a.sent();
                        return [2 /*return*/, new objects_1.PluginStateObject.Shape.Representation3D({ repr: repr, source: a }, { label: a.data.label })];
                }
            });
        }); });
    },
    update: function (_a, plugin) {
        var _this = this;
        var a = _a.a, b = _a.b, oldParams = _a.oldParams, newParams = _a.newParams;
        return mol_task_1.Task.create('Shape Representation', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        props = __assign({}, b.data.repr.props, newParams);
                        return [4 /*yield*/, b.data.repr.createOrUpdate(props, a.data.data).runInContext(ctx)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, mol_state_1.StateTransformer.UpdateResult.Updated];
                }
            });
        }); });
    }
});
exports.ShapeRepresentation3D = ShapeRepresentation3D;
