"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var cif_1 = require("../../../mol-io/reader/cif");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var ccp4_1 = require("../../../mol-model-formats/volume/ccp4");
var density_server_1 = require("../../../mol-model-formats/volume/density-server");
var dsn6_1 = require("../../../mol-model-formats/volume/dsn6");
var mol_task_1 = require("../../../mol-task");
var param_definition_1 = require("../../../mol-util/param-definition");
var objects_1 = require("../objects");
var VolumeFromCcp4 = objects_1.PluginStateTransform.BuiltIn({
    name: 'volume-from-ccp4',
    display: { name: 'Volume from CCP4/MRC/MAP', description: 'Create Volume from CCP4/MRC/MAP data' },
    from: objects_1.PluginStateObject.Format.Ccp4,
    to: objects_1.PluginStateObject.Volume.Data,
    params: function (a) {
        return {
            voxelSize: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.create(1, 1, 1))
        };
    }
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Create volume from CCP4/MRC/MAP', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var volume, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, ccp4_1.volumeFromCcp4(a.data, params).runInContext(ctx)];
                    case 1:
                        volume = _a.sent();
                        props = { label: 'Volume' };
                        return [2 /*return*/, new objects_1.PluginStateObject.Volume.Data(volume, props)];
                }
            });
        }); });
    }
});
exports.VolumeFromCcp4 = VolumeFromCcp4;
var VolumeFromDsn6 = objects_1.PluginStateTransform.BuiltIn({
    name: 'volume-from-dsn6',
    display: { name: 'Volume from DSN6/BRIX', description: 'Create Volume from DSN6/BRIX data' },
    from: objects_1.PluginStateObject.Format.Dsn6,
    to: objects_1.PluginStateObject.Volume.Data,
    params: function (a) {
        return {
            voxelSize: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.create(1, 1, 1))
        };
    }
})({
    apply: function (_a) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Create volume from DSN6/BRIX', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var volume, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, dsn6_1.volumeFromDsn6(a.data, params).runInContext(ctx)];
                    case 1:
                        volume = _a.sent();
                        props = { label: 'Volume' };
                        return [2 /*return*/, new objects_1.PluginStateObject.Volume.Data(volume, props)];
                }
            });
        }); });
    }
});
exports.VolumeFromDsn6 = VolumeFromDsn6;
var VolumeFromDensityServerCif = objects_1.PluginStateTransform.BuiltIn({
    name: 'volume-from-density-server-cif',
    display: { name: 'Volume from density-server CIF', description: 'Identify and create all separate models in the specified CIF data block' },
    from: objects_1.PluginStateObject.Format.Cif,
    to: objects_1.PluginStateObject.Volume.Data,
    params: function (a) {
        if (!a) {
            return {
                blockHeader: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Text(void 0, { description: 'Header of the block to parse. If none is specifed, the 1st data block in the file is used.' }))
            };
        }
        var blocks = a.data.blocks.slice(1); // zero block contains query meta-data
        return {
            blockHeader: param_definition_1.ParamDefinition.Optional(param_definition_1.ParamDefinition.Select(blocks[0] && blocks[0].header, blocks.map(function (b) { return [b.header, b.header]; }), { description: 'Header of the block to parse' }))
        };
    }
})({
    isApplicable: function (a) { return a.data.blocks.length > 0; },
    apply: function (_a) {
        var _this = this;
        var a = _a.a, params = _a.params;
        return mol_task_1.Task.create('Parse density-server CIF', function (ctx) { return __awaiter(_this, void 0, void 0, function () {
            var header, block, densityServerCif, volume, props;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        header = params.blockHeader || a.data.blocks[1].header;
                        block = a.data.blocks.find(function (b) { return b.header === header; });
                        if (!block)
                            throw new Error("Data block '" + [header] + "' not found.");
                        densityServerCif = cif_1.CIF.schema.densityServer(block);
                        return [4 /*yield*/, density_server_1.volumeFromDensityServerData(densityServerCif).runInContext(ctx)];
                    case 1:
                        volume = _a.sent();
                        props = { label: densityServerCif.volume_data_3d_info.name.value(0), description: "" + densityServerCif.volume_data_3d_info.name.value(0) };
                        return [2 /*return*/, new objects_1.PluginStateObject.Volume.Data(volume, props)];
                }
            });
        }); });
    }
});
exports.VolumeFromDensityServerCif = VolumeFromDensityServerCif;
