"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var color_1 = require("../../../mol-util/color");
var ControlGroup = /** @class */ (function (_super) {
    __extends(ControlGroup, _super);
    function ControlGroup() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { isExpanded: !!_this.props.initialExpanded };
        _this.toggleExpanded = function () { return _this.setState({ isExpanded: !_this.state.isExpanded }); };
        return _this;
    }
    ControlGroup.prototype.render = function () {
        return React.createElement("div", { className: 'msp-control-group-wrapper' },
            React.createElement("div", { className: 'msp-control-group-header' },
                React.createElement("button", { className: 'msp-btn msp-btn-block', onClick: this.toggleExpanded },
                    React.createElement("span", { className: "msp-icon msp-icon-" + (this.state.isExpanded ? 'collapse' : 'expand') }),
                    this.props.header)),
            this.state.isExpanded && React.createElement("div", { className: 'msp-control-offset', style: { display: this.state.isExpanded ? 'block' : 'none' } }, this.props.children));
    };
    return ControlGroup;
}(React.Component));
exports.ControlGroup = ControlGroup;
var NumericInput = /** @class */ (function (_super) {
    __extends(NumericInput, _super);
    function NumericInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { value: '0' };
        _this.input = React.createRef();
        _this.onChange = function (e) {
            var value = +e.target.value;
            _this.setState({ value: e.target.value }, function () {
                if (!Number.isNaN(value) && value !== _this.props.value) {
                    _this.props.onChange(value);
                }
            });
        };
        _this.onKeyPress = function (e) {
            if ((e.keyCode === 13 || e.charCode === 13)) {
                if (_this.props.blurOnEnter && _this.input.current) {
                    _this.input.current.blur();
                }
                if (_this.props.onEnter)
                    _this.props.onEnter();
            }
        };
        _this.onBlur = function () {
            _this.setState({ value: '' + _this.props.value });
            if (_this.props.onBlur)
                _this.props.onBlur();
        };
        return _this;
    }
    NumericInput.getDerivedStateFromProps = function (props, state) {
        var value = +state.value;
        if (Number.isNaN(value) || value === props.value)
            return null;
        return { value: '' + props.value };
    };
    NumericInput.prototype.render = function () {
        return React.createElement("input", { type: 'text', ref: this.input, onBlur: this.onBlur, value: this.state.value, placeholder: this.props.placeholder, onChange: this.onChange, onKeyPress: this.props.onEnter || this.props.blurOnEnter ? this.onKeyPress : void 0, disabled: !!this.props.isDisabled });
    };
    return NumericInput;
}(React.PureComponent));
exports.NumericInput = NumericInput;
function Icon(props) {
    return React.createElement("span", { className: "msp-icon msp-icon-" + props.name });
}
exports.Icon = Icon;
function IconButton(props) {
    var className = "msp-btn-link msp-btn-icon" + (props.isSmall ? '-small' : '') + (props.customClass ? ' ' + props.customClass : '');
    if (typeof props.toggleState !== 'undefined')
        className += " msp-btn-link-toggle-" + (props.toggleState ? 'on' : 'off');
    return React.createElement("button", { className: className, onClick: props.onClick, title: props.title, disabled: props.disabled, "data-id": props['data-id'] },
        React.createElement("span", { className: "msp-icon msp-icon-" + props.icon }));
}
exports.IconButton = IconButton;
var ExpandableGroup = /** @class */ (function (_super) {
    __extends(ExpandableGroup, _super);
    function ExpandableGroup() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { isExpanded: false };
        _this.toggleExpanded = function () { return _this.setState({ isExpanded: !_this.state.isExpanded }); };
        return _this;
    }
    ExpandableGroup.prototype.render = function () {
        var _a = this.props, label = _a.label, pivot = _a.pivot, controls = _a.controls;
        // TODO: fix the inline CSS
        return React.createElement(React.Fragment, null,
            React.createElement("div", { className: 'msp-control-row' },
                React.createElement("span", null,
                    label,
                    React.createElement("button", { className: 'msp-btn-link msp-btn-icon msp-conrol-group-expander', onClick: this.toggleExpanded, title: (this.state.isExpanded ? 'Less' : 'More') + " options", style: { background: 'transparent', textAlign: 'left', padding: '0' } },
                        React.createElement("span", { className: "msp-icon msp-icon-" + (this.state.isExpanded ? 'minus' : 'plus'), style: { display: 'inline-block' } }))),
                React.createElement("div", null, pivot),
                this.props.colorStripe && React.createElement("div", { className: 'msp-expandable-group-color-stripe', style: { backgroundColor: color_1.Color.toStyle(this.props.colorStripe) } })),
            this.state.isExpanded && React.createElement("div", { className: 'msp-control-offset' }, controls));
    };
    return ExpandableGroup;
}(React.Component));
exports.ExpandableGroup = ExpandableGroup;
// export const ToggleButton = (props: {
//     onChange: (v: boolean) => void,
//     value: boolean,
//     label: string,
//     title?: string
// }) => <div className='lm-control-row lm-toggle-button' title={props.title}>
//         <span>{props.label}</span>
//         <div>
//             <button onClick={e => { props.onChange.call(null, !props.value); (e.target as HTMLElement).blur(); }}>
//                     <span className={ `lm-icon lm-icon-${props.value ? 'ok' : 'off'}` }></span> {props.value ? 'On' : 'Off'}
//             </button>
//         </div>
//     </div>
