/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Vec2 } from '../../../mol-math/linear-algebra';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import * as React from 'react';
export interface ParameterControlsProps<P extends PD.Params = PD.Params> {
    params: P;
    values: any;
    onChange: ParamOnChange;
    isDisabled?: boolean;
    onEnter?: () => void;
}
export declare class ParameterControls<P extends PD.Params> extends React.PureComponent<ParameterControlsProps<P>, {}> {
    render(): JSX.Element | null;
}
export declare type ParamOnChange = (params: {
    param: PD.Base<any>;
    name: string;
    value: any;
}) => void;
export interface ParamProps<P extends PD.Base<any> = PD.Base<any>> {
    name: string;
    value: P['defaultValue'];
    param: P;
    isDisabled?: boolean;
    onChange: ParamOnChange;
    onEnter?: () => void;
}
export declare type ParamControl = React.ComponentClass<ParamProps<any>>;
export declare abstract class SimpleParam<P extends PD.Any> extends React.PureComponent<ParamProps<P>> {
    protected update(value: P['defaultValue']): void;
    abstract renderControl(): JSX.Element;
    render(): JSX.Element;
}
export declare class BoolControl extends SimpleParam<PD.BooleanParam> {
    onClick: (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    renderControl(): JSX.Element;
}
export declare class LineGraphControl extends React.PureComponent<ParamProps<PD.LineGraph>, {
    isExpanded: boolean;
    isOverPoint: boolean;
    message: string;
}> {
    state: {
        isExpanded: boolean;
        isOverPoint: boolean;
        message: string;
    };
    onHover: (point?: Vec2 | undefined) => void;
    onDrag: (point: Vec2) => void;
    onChange: (value: Vec2[]) => void;
    toggleExpanded: (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    render(): JSX.Element;
}
export declare class NumberInputControl extends React.PureComponent<ParamProps<PD.Numeric>> {
    state: {
        value: string;
    };
    update: (value: number) => void;
    render(): JSX.Element;
}
export declare class NumberRangeControl extends SimpleParam<PD.Numeric> {
    onChange: (v: number) => void;
    renderControl(): JSX.Element;
}
export declare class TextControl extends SimpleParam<PD.Text> {
    onChange: (e: React.ChangeEvent<HTMLInputElement>) => void;
    onKeyPress: (e: React.KeyboardEvent<HTMLInputElement>) => void;
    renderControl(): JSX.Element;
}
export declare class SelectControl extends SimpleParam<PD.Select<string | number>> {
    onChange: (e: React.ChangeEvent<HTMLSelectElement>) => void;
    renderControl(): JSX.Element;
}
export declare class IntervalControl extends SimpleParam<PD.Interval> {
    onChange: (v: [number, number]) => void;
    renderControl(): JSX.Element;
}
export declare class BoundedIntervalControl extends SimpleParam<PD.Interval> {
    onChange: (v: [number, number]) => void;
    renderControl(): JSX.Element;
}
export declare class ColorControl extends SimpleParam<PD.Color> {
    onChange: (e: React.ChangeEvent<HTMLSelectElement>) => void;
    stripStyle(): React.CSSProperties;
    renderControl(): JSX.Element;
}
export declare class ColorScaleControl extends SimpleParam<PD.ColorScale<any>> {
    onChange: (e: React.ChangeEvent<HTMLSelectElement>) => void;
    stripStyle(): React.CSSProperties;
    renderControl(): JSX.Element;
}
export declare class Vec3Control extends React.PureComponent<ParamProps<PD.Vec3>, {
    isExpanded: boolean;
}> {
    state: {
        isExpanded: boolean;
    };
    components: {
        0: PD.Numeric;
        1: PD.Numeric;
        2: PD.Numeric;
    };
    change(value: PD.MultiSelect<any>['defaultValue']): void;
    componentChange: ParamOnChange;
    toggleExpanded: (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    render(): JSX.Element;
}
export declare class FileControl extends React.PureComponent<ParamProps<PD.FileParam>> {
    change(value: File): void;
    onChangeFile: (e: React.ChangeEvent<HTMLInputElement>) => void;
    render(): JSX.Element;
}
export declare class MultiSelectControl extends React.PureComponent<ParamProps<PD.MultiSelect<any>>, {
    isExpanded: boolean;
}> {
    state: {
        isExpanded: boolean;
    };
    change(value: PD.MultiSelect<any>['defaultValue']): void;
    toggle(key: string): (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    toggleExpanded: (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    render(): JSX.Element;
}
export declare class GroupControl extends React.PureComponent<ParamProps<PD.Group<any>>, {
    isExpanded: boolean;
}> {
    state: {
        isExpanded: boolean;
    };
    change(value: any): void;
    onChangeParam: ParamOnChange;
    toggleExpanded: () => void;
    render(): JSX.Element | null;
}
export declare class MappedControl extends React.PureComponent<ParamProps<PD.Mapped<any>>> {
    private valuesCache;
    private setValues;
    private getValues;
    change(value: PD.Mapped<any>['defaultValue']): void;
    onChangeName: ParamOnChange;
    onChangeParam: ParamOnChange;
    render(): JSX.Element;
}
export declare class ObjectListControl extends React.PureComponent<ParamProps<PD.ObjectList>, {
    isExpanded: boolean;
}> {
    state: {
        isExpanded: boolean;
    };
    change(value: any): void;
    add: (v: object) => void;
    actions: {
        update: (v: object, i: number) => void;
        move: (i: number, dir: 1 | -1) => void;
        remove: (i: number) => void;
    };
    toggleExpanded: (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;
    render(): JSX.Element;
}
export declare class ConditionedControl extends React.PureComponent<ParamProps<PD.Conditioned<any, any, any>>> {
    change(value: PD.Conditioned<any, any, any>['defaultValue']): void;
    onChangeCondition: ParamOnChange;
    onChangeParam: ParamOnChange;
    render(): JSX.Element;
}
export declare class ConvertedControl extends React.PureComponent<ParamProps<PD.Converted<any, any>>> {
    onChange: ParamOnChange;
    render(): JSX.Element | null;
}
export declare class ScriptExpressionControl extends SimpleParam<PD.ScriptExpression> {
    onChange: (e: React.ChangeEvent<HTMLInputElement>) => void;
    onKeyPress: (e: React.KeyboardEvent<HTMLInputElement>) => void;
    renderControl(): JSX.Element;
}
