"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var color_1 = require("../../../mol-util/color");
var scale_1 = require("../../../mol-util/color/scale");
var tables_1 = require("../../../mol-util/color/tables");
var memoize_1 = require("../../../mol-util/memoize");
var param_definition_1 = require("../../../mol-util/param-definition");
var string_1 = require("../../../mol-util/string");
var React = require("react");
var line_graph_component_1 = require("./line-graph/line-graph-component");
var slider_1 = require("./slider");
var common_1 = require("./common");
var ParameterControls = /** @class */ (function (_super) {
    __extends(ParameterControls, _super);
    function ParameterControls() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ParameterControls.prototype.render = function () {
        var _this = this;
        var params = this.props.params;
        var values = this.props.values;
        var keys = Object.keys(params);
        if (keys.length === 0)
            return null;
        return React.createElement(React.Fragment, null, keys.map(function (key) {
            var param = params[key];
            if (param.isHidden)
                return null;
            var Control = controlFor(param);
            if (!Control)
                return null;
            return React.createElement(Control, { param: param, key: key, onChange: _this.props.onChange, onEnter: _this.props.onEnter, isDisabled: _this.props.isDisabled, name: key, value: values[key] });
        }));
    };
    return ParameterControls;
}(React.PureComponent));
exports.ParameterControls = ParameterControls;
function controlFor(param) {
    switch (param.type) {
        case 'value': return void 0;
        case 'boolean': return BoolControl;
        case 'number': return typeof param.min !== 'undefined' && typeof param.max !== 'undefined'
            ? NumberRangeControl : NumberInputControl;
        case 'converted': return ConvertedControl;
        case 'conditioned': return ConditionedControl;
        case 'multi-select': return MultiSelectControl;
        case 'color': return ColorControl;
        case 'color-scale': return ColorScaleControl;
        case 'vec3': return Vec3Control;
        case 'file': return FileControl;
        case 'select': return SelectControl;
        case 'text': return TextControl;
        case 'interval': return typeof param.min !== 'undefined' && typeof param.max !== 'undefined'
            ? BoundedIntervalControl : IntervalControl;
        case 'group': return GroupControl;
        case 'mapped': return MappedControl;
        case 'line-graph': return LineGraphControl;
        case 'script-expression': return ScriptExpressionControl;
        case 'object-list': return ObjectListControl;
        default:
            var _ = param;
            console.warn(_ + " has no associated UI component");
            return void 0;
    }
}
var SimpleParam = /** @class */ (function (_super) {
    __extends(SimpleParam, _super);
    function SimpleParam() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SimpleParam.prototype.update = function (value) {
        this.props.onChange({ param: this.props.param, name: this.props.name, value: value });
    };
    SimpleParam.prototype.render = function () {
        var label = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        return React.createElement("div", { className: 'msp-control-row' },
            React.createElement("span", { title: this.props.param.description }, label),
            React.createElement("div", null, this.renderControl()));
    };
    return SimpleParam;
}(React.PureComponent));
exports.SimpleParam = SimpleParam;
var BoolControl = /** @class */ (function (_super) {
    __extends(BoolControl, _super);
    function BoolControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onClick = function (e) { _this.update(!_this.props.value); e.currentTarget.blur(); };
        return _this;
    }
    BoolControl.prototype.renderControl = function () {
        return React.createElement("button", { onClick: this.onClick, disabled: this.props.isDisabled },
            React.createElement("span", { className: "msp-icon msp-icon-" + (this.props.value ? 'ok' : 'off') }),
            this.props.value ? 'On' : 'Off');
    };
    return BoolControl;
}(SimpleParam));
exports.BoolControl = BoolControl;
var LineGraphControl = /** @class */ (function (_super) {
    __extends(LineGraphControl, _super);
    function LineGraphControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isExpanded: false,
            isOverPoint: false,
            message: _this.props.param.defaultValue.length + " points",
        };
        _this.onHover = function (point) {
            _this.setState({ isOverPoint: !_this.state.isOverPoint });
            if (point) {
                _this.setState({ message: "(" + point[0].toFixed(2) + ", " + point[1].toFixed(2) + ")" });
                return;
            }
            _this.setState({ message: _this.props.value.length + " points" });
        };
        _this.onDrag = function (point) {
            _this.setState({ message: "(" + point[0].toFixed(2) + ", " + point[1].toFixed(2) + ")" });
        };
        _this.onChange = function (value) {
            _this.props.onChange({ name: _this.props.name, param: _this.props.param, value: value });
        };
        _this.toggleExpanded = function (e) {
            _this.setState({ isExpanded: !_this.state.isExpanded });
            e.currentTarget.blur();
        };
        return _this;
    }
    LineGraphControl.prototype.render = function () {
        var label = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        return React.createElement(React.Fragment, null,
            React.createElement("div", { className: 'msp-control-row' },
                React.createElement("span", null, label),
                React.createElement("div", null,
                    React.createElement("button", { onClick: this.toggleExpanded }, "" + this.state.message))),
            React.createElement("div", { className: 'msp-control-offset', style: { display: this.state.isExpanded ? 'block' : 'none' } },
                React.createElement(line_graph_component_1.default, { data: this.props.param.defaultValue, onChange: this.onChange, onHover: this.onHover, onDrag: this.onDrag })));
    };
    return LineGraphControl;
}(React.PureComponent));
exports.LineGraphControl = LineGraphControl;
var NumberInputControl = /** @class */ (function (_super) {
    __extends(NumberInputControl, _super);
    function NumberInputControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { value: '0' };
        _this.update = function (value) {
            _this.props.onChange({ param: _this.props.param, name: _this.props.name, value: value });
        };
        return _this;
    }
    NumberInputControl.prototype.render = function () {
        var placeholder = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        var label = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        return React.createElement("div", { className: 'msp-control-row' },
            React.createElement("span", { title: this.props.param.description }, label),
            React.createElement("div", null,
                React.createElement(common_1.NumericInput, { value: this.props.value, onEnter: this.props.onEnter, placeholder: placeholder, isDisabled: this.props.isDisabled, onChange: this.update })));
    };
    return NumberInputControl;
}(React.PureComponent));
exports.NumberInputControl = NumberInputControl;
var NumberRangeControl = /** @class */ (function (_super) {
    __extends(NumberRangeControl, _super);
    function NumberRangeControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (v) { _this.update(v); };
        return _this;
    }
    NumberRangeControl.prototype.renderControl = function () {
        var value = typeof this.props.value === 'undefined' ? this.props.param.defaultValue : this.props.value;
        return React.createElement(slider_1.Slider, { value: value, min: this.props.param.min, max: this.props.param.max, step: this.props.param.step, onChange: this.onChange, disabled: this.props.isDisabled, onEnter: this.props.onEnter });
    };
    return NumberRangeControl;
}(SimpleParam));
exports.NumberRangeControl = NumberRangeControl;
var TextControl = /** @class */ (function (_super) {
    __extends(TextControl, _super);
    function TextControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (e) {
            var value = e.target.value;
            if (value !== _this.props.value) {
                _this.update(value);
            }
        };
        _this.onKeyPress = function (e) {
            if (!_this.props.onEnter)
                return;
            if ((e.keyCode === 13 || e.charCode === 13)) {
                _this.props.onEnter();
            }
        };
        return _this;
    }
    TextControl.prototype.renderControl = function () {
        var placeholder = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        return React.createElement("input", { type: 'text', value: this.props.value || '', placeholder: placeholder, onChange: this.onChange, onKeyPress: this.props.onEnter ? this.onKeyPress : void 0, disabled: this.props.isDisabled });
    };
    return TextControl;
}(SimpleParam));
exports.TextControl = TextControl;
var SelectControl = /** @class */ (function (_super) {
    __extends(SelectControl, _super);
    function SelectControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (e) {
            if (typeof _this.props.param.defaultValue === 'number') {
                _this.update(parseInt(e.target.value, 10));
            }
            else {
                _this.update(e.target.value);
            }
        };
        return _this;
    }
    SelectControl.prototype.renderControl = function () {
        var _this = this;
        var isInvalid = this.props.value !== void 0 && !this.props.param.options.some(function (e) { return e[0] === _this.props.value; });
        return React.createElement("select", { value: this.props.value !== void 0 ? this.props.value : this.props.param.defaultValue, onChange: this.onChange, disabled: this.props.isDisabled },
            isInvalid && React.createElement("option", { key: this.props.value, value: this.props.value }, "[Invalid] " + this.props.value),
            this.props.param.options.map(function (_a) {
                var value = _a[0], label = _a[1];
                return React.createElement("option", { key: value, value: value }, label);
            }));
    };
    return SelectControl;
}(SimpleParam));
exports.SelectControl = SelectControl;
var IntervalControl = /** @class */ (function (_super) {
    __extends(IntervalControl, _super);
    function IntervalControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (v) { _this.update(v); };
        return _this;
    }
    IntervalControl.prototype.renderControl = function () {
        return React.createElement("span", null, "interval TODO");
    };
    return IntervalControl;
}(SimpleParam));
exports.IntervalControl = IntervalControl;
var BoundedIntervalControl = /** @class */ (function (_super) {
    __extends(BoundedIntervalControl, _super);
    function BoundedIntervalControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (v) { _this.update(v); };
        return _this;
    }
    BoundedIntervalControl.prototype.renderControl = function () {
        return React.createElement(slider_1.Slider2, { value: this.props.value, min: this.props.param.min, max: this.props.param.max, step: this.props.param.step, onChange: this.onChange, disabled: this.props.isDisabled, onEnter: this.props.onEnter });
    };
    return BoundedIntervalControl;
}(SimpleParam));
exports.BoundedIntervalControl = BoundedIntervalControl;
var _colors = void 0;
function ColorOptions() {
    if (_colors)
        return _colors;
    _colors = React.createElement(React.Fragment, null, Object.keys(tables_1.ColorNames).map(function (name) {
        return React.createElement("option", { key: name, value: tables_1.ColorNames[name], style: { background: "" + color_1.Color.toStyle(tables_1.ColorNames[name]) } }, name);
    }));
    return _colors;
}
function ColorValueOption(color) {
    return !tables_1.ColorNamesValueMap.has(color) ? React.createElement("option", { key: color_1.Color.toHexString(color), value: color, style: { background: "" + color_1.Color.toStyle(color) } }, color_1.Color.toRgbString(color)) : null;
}
var ColorControl = /** @class */ (function (_super) {
    __extends(ColorControl, _super);
    function ColorControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (e) {
            _this.update(color_1.Color(parseInt(e.target.value)));
        };
        return _this;
    }
    ColorControl.prototype.stripStyle = function () {
        return {
            background: color_1.Color.toStyle(this.props.value),
            position: 'absolute',
            bottom: '0',
            height: '4px',
            right: '0',
            left: '0'
        };
    };
    ColorControl.prototype.renderControl = function () {
        return React.createElement("div", { style: { position: 'relative' } },
            React.createElement("select", { value: this.props.value, onChange: this.onChange },
                ColorValueOption(this.props.value),
                ColorOptions()),
            React.createElement("div", { style: this.stripStyle() }));
    };
    return ColorControl;
}(SimpleParam));
exports.ColorControl = ColorControl;
var colorScaleGradient = memoize_1.memoize1(function (n) { return "linear-gradient(to right, " + scale_1.getColorListFromName(n).map(function (c) { return color_1.Color.toStyle(c); }).join(', ') + ")"; });
var ColorScaleControl = /** @class */ (function (_super) {
    __extends(ColorScaleControl, _super);
    function ColorScaleControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (e) { _this.update(e.target.value); };
        return _this;
    }
    ColorScaleControl.prototype.stripStyle = function () {
        return {
            background: colorScaleGradient(this.props.value),
            position: 'absolute',
            bottom: '0',
            height: '4px',
            right: '0',
            left: '0'
        };
    };
    ColorScaleControl.prototype.renderControl = function () {
        return React.createElement("div", { style: { position: 'relative' } },
            React.createElement("select", { value: this.props.value || '', onChange: this.onChange, disabled: this.props.isDisabled }, this.props.param.options.map(function (_a) {
                var value = _a[0], label = _a[1];
                return React.createElement("option", { key: value, value: value }, label);
            })),
            React.createElement("div", { style: this.stripStyle() }));
    };
    return ColorScaleControl;
}(SimpleParam));
exports.ColorScaleControl = ColorScaleControl;
var Vec3Control = /** @class */ (function (_super) {
    __extends(Vec3Control, _super);
    function Vec3Control() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { isExpanded: false };
        _this.components = {
            0: param_definition_1.ParamDefinition.Numeric(0, void 0, { label: (_this.props.param.fieldLabels && _this.props.param.fieldLabels.x) || 'X' }),
            1: param_definition_1.ParamDefinition.Numeric(0, void 0, { label: (_this.props.param.fieldLabels && _this.props.param.fieldLabels.y) || 'Y' }),
            2: param_definition_1.ParamDefinition.Numeric(0, void 0, { label: (_this.props.param.fieldLabels && _this.props.param.fieldLabels.z) || 'Z' })
        };
        _this.componentChange = function (_a) {
            var name = _a.name, value = _a.value;
            var v = linear_algebra_1.Vec3.copy(linear_algebra_1.Vec3.zero(), _this.props.value);
            v[+name] = value;
            _this.change(v);
        };
        _this.toggleExpanded = function (e) {
            _this.setState({ isExpanded: !_this.state.isExpanded });
            e.currentTarget.blur();
        };
        return _this;
    }
    Vec3Control.prototype.change = function (value) {
        this.props.onChange({ name: this.props.name, param: this.props.param, value: value });
    };
    Vec3Control.prototype.render = function () {
        var v = this.props.value;
        var label = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        var value = "[" + v[0].toFixed(2) + ", " + v[1].toFixed(2) + ", " + v[2].toFixed(2) + "]";
        return React.createElement(React.Fragment, null,
            React.createElement("div", { className: 'msp-control-row' },
                React.createElement("span", null, label),
                React.createElement("div", null,
                    React.createElement("button", { onClick: this.toggleExpanded }, value))),
            React.createElement("div", { className: 'msp-control-offset', style: { display: this.state.isExpanded ? 'block' : 'none' } },
                React.createElement(ParameterControls, { params: this.components, values: v, onChange: this.componentChange, onEnter: this.props.onEnter })));
    };
    return Vec3Control;
}(React.PureComponent));
exports.Vec3Control = Vec3Control;
var FileControl = /** @class */ (function (_super) {
    __extends(FileControl, _super);
    function FileControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChangeFile = function (e) {
            _this.change(e.target.files[0]);
        };
        return _this;
    }
    FileControl.prototype.change = function (value) {
        this.props.onChange({ name: this.props.name, param: this.props.param, value: value });
    };
    FileControl.prototype.render = function () {
        var value = this.props.value;
        // return <input disabled={this.props.isDisabled} value={void 0} type='file' multiple={false} />
        return React.createElement("div", { className: 'msp-btn msp-btn-block msp-btn-action msp-loader-msp-btn-file', style: { marginTop: '1px' } },
            value ? value.name : 'Select a file...',
            " ",
            React.createElement("input", { disabled: this.props.isDisabled, onChange: this.onChangeFile, type: 'file', multiple: false, accept: this.props.param.accept }));
    };
    return FileControl;
}(React.PureComponent));
exports.FileControl = FileControl;
var MultiSelectControl = /** @class */ (function (_super) {
    __extends(MultiSelectControl, _super);
    function MultiSelectControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { isExpanded: false };
        _this.toggleExpanded = function (e) {
            _this.setState({ isExpanded: !_this.state.isExpanded });
            e.currentTarget.blur();
        };
        return _this;
    }
    MultiSelectControl.prototype.change = function (value) {
        this.props.onChange({ name: this.props.name, param: this.props.param, value: value });
    };
    MultiSelectControl.prototype.toggle = function (key) {
        var _this = this;
        return function (e) {
            if (_this.props.value.indexOf(key) < 0)
                _this.change(_this.props.value.concat(key));
            else
                _this.change(_this.props.value.filter(function (v) { return v !== key; }));
            e.currentTarget.blur();
        };
    };
    MultiSelectControl.prototype.render = function () {
        var _this = this;
        var current = this.props.value;
        var label = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        return React.createElement(React.Fragment, null,
            React.createElement("div", { className: 'msp-control-row' },
                React.createElement("span", null, label),
                React.createElement("div", null,
                    React.createElement("button", { onClick: this.toggleExpanded }, current.length + " of " + this.props.param.options.length))),
            React.createElement("div", { className: 'msp-control-offset', style: { display: this.state.isExpanded ? 'block' : 'none' } }, this.props.param.options.map(function (_a) {
                var value = _a[0], label = _a[1];
                var sel = current.indexOf(value) >= 0;
                return React.createElement("div", { key: value, className: 'msp-row' },
                    React.createElement("button", { onClick: _this.toggle(value), disabled: _this.props.isDisabled },
                        React.createElement("span", { style: { float: sel ? 'left' : 'right' } }, sel ? "\u2713 " + label : label + " \u2717")));
            })));
    };
    return MultiSelectControl;
}(React.PureComponent));
exports.MultiSelectControl = MultiSelectControl;
var GroupControl = /** @class */ (function (_super) {
    __extends(GroupControl, _super);
    function GroupControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { isExpanded: !!_this.props.param.isExpanded };
        _this.onChangeParam = function (e) {
            var _a;
            _this.change(__assign({}, _this.props.value, (_a = {}, _a[e.name] = e.value, _a)));
        };
        _this.toggleExpanded = function () { return _this.setState({ isExpanded: !_this.state.isExpanded }); };
        return _this;
    }
    GroupControl.prototype.change = function (value) {
        this.props.onChange({ name: this.props.name, param: this.props.param, value: value });
    };
    GroupControl.prototype.render = function () {
        var params = this.props.param.params;
        // Do not show if there are no params.
        if (Object.keys(params).length === 0)
            return null;
        var label = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        var controls = React.createElement(ParameterControls, { params: params, onChange: this.onChangeParam, values: this.props.value, onEnter: this.props.onEnter, isDisabled: this.props.isDisabled });
        if (this.props.param.isFlat) {
            return controls;
        }
        return React.createElement("div", { className: 'msp-control-group-wrapper' },
            React.createElement("div", { className: 'msp-control-group-header' },
                React.createElement("button", { className: 'msp-btn msp-btn-block', onClick: this.toggleExpanded },
                    React.createElement("span", { className: "msp-icon msp-icon-" + (this.state.isExpanded ? 'collapse' : 'expand') }),
                    label)),
            this.state.isExpanded && React.createElement("div", { className: 'msp-control-offset', style: { display: this.state.isExpanded ? 'block' : 'none' } }, controls));
    };
    return GroupControl;
}(React.PureComponent));
exports.GroupControl = GroupControl;
var MappedControl = /** @class */ (function (_super) {
    __extends(MappedControl, _super);
    function MappedControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.valuesCache = {};
        _this.onChangeName = function (e) {
            _this.change({ name: e.value, params: _this.getValues(e.value) });
        };
        _this.onChangeParam = function (e) {
            _this.setValues(_this.props.value.name, e.value);
            _this.change({ name: _this.props.value.name, params: e.value });
        };
        return _this;
    }
    MappedControl.prototype.setValues = function (name, values) {
        this.valuesCache[name] = values;
    };
    MappedControl.prototype.getValues = function (name) {
        if (name in this.valuesCache) {
            return this.valuesCache[name];
        }
        else {
            return this.props.param.map(name).defaultValue;
        }
    };
    MappedControl.prototype.change = function (value) {
        this.props.onChange({ name: this.props.name, param: this.props.param, value: value });
    };
    MappedControl.prototype.render = function () {
        var value = this.props.value;
        var param = this.props.param.map(value.name);
        var label = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        var Mapped = controlFor(param);
        var select = React.createElement(SelectControl, { param: this.props.param.select, isDisabled: this.props.isDisabled, onChange: this.onChangeName, onEnter: this.props.onEnter, name: label, value: value.name });
        if (!Mapped) {
            return select;
        }
        return React.createElement(React.Fragment, null,
            select,
            React.createElement(Mapped, { param: param, value: value.params, name: label + " Properties", onChange: this.onChangeParam, onEnter: this.props.onEnter, isDisabled: this.props.isDisabled }));
    };
    return MappedControl;
}(React.PureComponent));
exports.MappedControl = MappedControl;
var ObjectListEditor = /** @class */ (function (_super) {
    __extends(ObjectListEditor, _super);
    function ObjectListEditor() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { params: {}, value: void 0, current: void 0 };
        _this.onChangeParam = function (e) {
            var _a;
            _this.setState({ current: __assign({}, _this.state.current, (_a = {}, _a[e.name] = e.value, _a)) });
        };
        _this.apply = function () {
            _this.props.apply(_this.state.current);
        };
        return _this;
    }
    ObjectListEditor.getDerivedStateFromProps = function (props, state) {
        if (props.params === state.params && props.value === state.value)
            return null;
        return {
            params: props.params,
            value: props.value,
            current: props.value
        };
    };
    ObjectListEditor.prototype.render = function () {
        return React.createElement(React.Fragment, null,
            React.createElement(ParameterControls, { params: this.props.params, onChange: this.onChangeParam, values: this.state.current, onEnter: this.apply, isDisabled: this.props.isDisabled }),
            React.createElement("button", { className: "msp-btn msp-btn-block msp-form-control msp-control-top-offset", onClick: this.apply, disabled: this.props.isDisabled }, this.props.isUpdate ? 'Update' : 'Add'));
    };
    return ObjectListEditor;
}(React.PureComponent));
var ObjectListItem = /** @class */ (function (_super) {
    __extends(ObjectListItem, _super);
    function ObjectListItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { isExpanded: false };
        _this.update = function (v) {
            _this.setState({ isExpanded: false });
            _this.props.actions.update(v, _this.props.index);
        };
        _this.moveUp = function () {
            _this.props.actions.move(_this.props.index, -1);
        };
        _this.moveDown = function () {
            _this.props.actions.move(_this.props.index, 1);
        };
        _this.remove = function () {
            _this.setState({ isExpanded: false });
            _this.props.actions.remove(_this.props.index);
        };
        _this.toggleExpanded = function (e) {
            _this.setState({ isExpanded: !_this.state.isExpanded });
            e.currentTarget.blur();
        };
        return _this;
    }
    ObjectListItem.getDerivedStateFromProps = function (props, state) {
        if (props.params === state.params && props.value === state.value)
            return null;
        return {
            params: props.params,
            value: props.value,
            current: props.value
        };
    };
    ObjectListItem.prototype.render = function () {
        return React.createElement(React.Fragment, null,
            React.createElement("div", { className: 'msp-param-object-list-item' },
                React.createElement("button", { className: 'msp-btn msp-btn-block msp-form-control', onClick: this.toggleExpanded },
                    React.createElement("span", null, this.props.index + 1 + ": "),
                    this.props.param.getLabel(this.props.value)),
                React.createElement("div", null,
                    React.createElement(common_1.IconButton, { icon: 'up-thin', title: 'Move Up', onClick: this.moveUp, isSmall: true }),
                    React.createElement(common_1.IconButton, { icon: 'down-thin', title: 'Move Down', onClick: this.moveDown, isSmall: true }),
                    React.createElement(common_1.IconButton, { icon: 'remove', title: 'Remove', onClick: this.remove, isSmall: true }))),
            this.state.isExpanded && React.createElement("div", { className: 'msp-control-offset' },
                React.createElement(ObjectListEditor, { params: this.props.param.element, apply: this.update, value: this.props.value, isUpdate: true, isDisabled: this.props.isDisabled })));
    };
    return ObjectListItem;
}(React.PureComponent));
var ObjectListControl = /** @class */ (function (_super) {
    __extends(ObjectListControl, _super);
    function ObjectListControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { isExpanded: false };
        _this.add = function (v) {
            _this.change(_this.props.value.concat([v]));
        };
        _this.actions = {
            update: function (v, i) {
                var value = _this.props.value.slice(0);
                value[i] = v;
                _this.change(value);
            },
            move: function (i, dir) {
                var xs = _this.props.value;
                if (xs.length === 1)
                    return;
                var j = (i + dir) % xs.length;
                if (j < 0)
                    j += xs.length;
                xs = xs.slice(0);
                var t = xs[i];
                xs[i] = xs[j];
                xs[j] = t;
                _this.change(xs);
            },
            remove: function (i) {
                var xs = _this.props.value;
                var update = [];
                for (var j = 0; j < xs.length; j++) {
                    if (i !== j)
                        update.push(xs[j]);
                }
                _this.change(update);
            }
        };
        _this.toggleExpanded = function (e) {
            _this.setState({ isExpanded: !_this.state.isExpanded });
            e.currentTarget.blur();
        };
        return _this;
    }
    ObjectListControl.prototype.change = function (value) {
        this.props.onChange({ name: this.props.name, param: this.props.param, value: value });
    };
    ObjectListControl.prototype.render = function () {
        var _this = this;
        var v = this.props.value;
        var label = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        var value = v.length + " item" + (v.length !== 1 ? 's' : '');
        return React.createElement(React.Fragment, null,
            React.createElement("div", { className: 'msp-control-row' },
                React.createElement("span", null, label),
                React.createElement("div", null,
                    React.createElement("button", { onClick: this.toggleExpanded }, value))),
            this.state.isExpanded && React.createElement("div", { className: 'msp-control-offset' },
                this.props.value.map(function (v, i) { return React.createElement(ObjectListItem, { key: i, param: _this.props.param, value: v, index: i, actions: _this.actions }); }),
                React.createElement(common_1.ControlGroup, { header: 'New Item' },
                    React.createElement(ObjectListEditor, { params: this.props.param.element, apply: this.add, value: this.props.param.ctor(), isDisabled: this.props.isDisabled }))));
    };
    return ObjectListControl;
}(React.PureComponent));
exports.ObjectListControl = ObjectListControl;
var ConditionedControl = /** @class */ (function (_super) {
    __extends(ConditionedControl, _super);
    function ConditionedControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChangeCondition = function (e) {
            _this.change(_this.props.param.conditionedValue(_this.props.value, e.value));
        };
        _this.onChangeParam = function (e) {
            _this.change(e.value);
        };
        return _this;
    }
    ConditionedControl.prototype.change = function (value) {
        this.props.onChange({ name: this.props.name, param: this.props.param, value: value });
    };
    ConditionedControl.prototype.render = function () {
        var value = this.props.value;
        var condition = this.props.param.conditionForValue(value);
        var param = this.props.param.conditionParams[condition];
        var label = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        var Conditioned = controlFor(param);
        var select = React.createElement(SelectControl, { param: this.props.param.select, isDisabled: this.props.isDisabled, onChange: this.onChangeCondition, onEnter: this.props.onEnter, name: label + " Kind", value: condition });
        if (!Conditioned) {
            return select;
        }
        return React.createElement(React.Fragment, null,
            select,
            React.createElement(Conditioned, { param: param, value: value, name: label, onChange: this.onChangeParam, onEnter: this.props.onEnter, isDisabled: this.props.isDisabled }));
    };
    return ConditionedControl;
}(React.PureComponent));
exports.ConditionedControl = ConditionedControl;
var ConvertedControl = /** @class */ (function (_super) {
    __extends(ConvertedControl, _super);
    function ConvertedControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (e) {
            _this.props.onChange({
                name: _this.props.name,
                param: _this.props.param,
                value: _this.props.param.toValue(e.value)
            });
        };
        return _this;
    }
    ConvertedControl.prototype.render = function () {
        var value = this.props.param.fromValue(this.props.value);
        var Converted = controlFor(this.props.param.converted);
        if (!Converted)
            return null;
        return React.createElement(Converted, { param: this.props.param.converted, value: value, name: this.props.name, onChange: this.onChange, onEnter: this.props.onEnter, isDisabled: this.props.isDisabled });
    };
    return ConvertedControl;
}(React.PureComponent));
exports.ConvertedControl = ConvertedControl;
var ScriptExpressionControl = /** @class */ (function (_super) {
    __extends(ScriptExpressionControl, _super);
    function ScriptExpressionControl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (e) {
            var value = e.target.value;
            if (value !== _this.props.value.expression) {
                _this.update({ language: _this.props.value.language, expression: value });
            }
        };
        _this.onKeyPress = function (e) {
            if (!_this.props.onEnter)
                return;
            if ((e.keyCode === 13 || e.charCode === 13)) {
                _this.props.onEnter();
            }
        };
        return _this;
    }
    ScriptExpressionControl.prototype.renderControl = function () {
        // TODO: improve!
        var placeholder = this.props.param.label || string_1.camelCaseToWords(this.props.name);
        return React.createElement("input", { type: 'text', value: this.props.value.expression || '', placeholder: placeholder, onChange: this.onChange, onKeyPress: this.props.onEnter ? this.onKeyPress : void 0, disabled: this.props.isDisabled });
    };
    return ScriptExpressionControl;
}(SimpleParam));
exports.ScriptExpressionControl = ScriptExpressionControl;
