"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("../base");
var React = require("react");
var common_1 = require("../controls/common");
var param_definition_1 = require("../../../mol-util/param-definition");
var parameters_1 = require("../controls/parameters");
var slider_1 = require("../controls/slider");
var volume_1 = require("../../../mol-model/volume");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var ChannelParams = {
    color: param_definition_1.ParamDefinition.Color(0),
    wireframe: param_definition_1.ParamDefinition.Boolean(false),
    opacity: param_definition_1.ParamDefinition.Numeric(0.3, { min: 0, max: 1, step: 0.01 })
};
function Channel(props) {
    var isRelative = props.isRelative, stats = props.stats;
    var channel = props.channels[props.name];
    var min = stats.min, max = stats.max, mean = stats.mean, sigma = stats.sigma;
    var value = channel.isoValue.kind === 'relative' ? channel.isoValue.relativeValue : channel.isoValue.absoluteValue;
    var relMin = (min - mean) / sigma;
    var relMax = (max - mean) / sigma;
    return React.createElement(common_1.ExpandableGroup, { label: props.label + (props.isRelative ? ' \u03C3' : ''), colorStripe: channel.color, pivot: React.createElement(slider_1.Slider, { value: value, min: isRelative ? relMin : min, max: isRelative ? relMax : max, step: isRelative ? sigma / 100 : Math.round(((max - min) / sigma)) / 100, onChange: function (v) { return props.changeIso(props.name, v, isRelative); }, disabled: props.params.isDisabled, onEnter: props.params.events.onEnter }), controls: React.createElement(parameters_1.ParameterControls, { onChange: function (_a) {
                var name = _a.name, value = _a.value;
                return props.changeParams(props.name, name, value);
            }, params: ChannelParams, values: channel, onEnter: props.params.events.onEnter }) });
}
var VolumeStreamingCustomControls = /** @class */ (function (_super) {
    __extends(VolumeStreamingCustomControls, _super);
    function VolumeStreamingCustomControls() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.changeIso = function (name, value, isRelative) {
            var _a;
            var old = _this.props.params;
            _this.newParams(__assign({}, old, { channels: __assign({}, old.channels, (_a = {}, _a[name] = __assign({}, old.channels[name], { isoValue: isRelative ? volume_1.VolumeIsoValue.relative(value) : volume_1.VolumeIsoValue.absolute(value) }), _a)) }));
        };
        _this.changeParams = function (name, param, value) {
            var _a, _b;
            var old = _this.props.params;
            _this.newParams(__assign({}, old, { channels: __assign({}, old.channels, (_a = {}, _a[name] = __assign({}, old.channels[name], (_b = {}, _b[param] = value, _b)), _a)) }));
        };
        _this.changeOption = function (_a) {
            var value = _a.value;
            var b = _this.props.b.data;
            var isEM = b.info.kind === 'em';
            var isRelative = value.params.isRelative;
            var sampling = b.info.header.sampling[0];
            var old = _this.props.params, oldChannels = old.channels;
            var oldView = old.view.name === value.name
                ? old.view.params
                : _this.props.info.params.view.map(value.name).defaultValue;
            var viewParams = __assign({}, oldView);
            if (value.name === 'selection-box') {
                viewParams.radius = value.params.radius;
            }
            else if (value.name === 'box') {
                viewParams.bottomLeft = value.params.bottomLeft;
                viewParams.topRight = value.params.topRight;
            }
            _this.newParams(__assign({}, old, { view: {
                    name: value.name,
                    params: viewParams
                }, detailLevel: value.params.detailLevel, channels: isEM
                    ? { em: _this.convert(oldChannels.em, sampling.valuesInfo[0], isRelative) }
                    : {
                        '2fo-fc': _this.convert(oldChannels['2fo-fc'], sampling.valuesInfo[0], isRelative),
                        'fo-fc(+ve)': _this.convert(oldChannels['fo-fc(+ve)'], sampling.valuesInfo[1], isRelative),
                        'fo-fc(-ve)': _this.convert(oldChannels['fo-fc(-ve)'], sampling.valuesInfo[1], isRelative)
                    } }));
        };
        return _this;
    }
    VolumeStreamingCustomControls.prototype.areInitial = function (params) {
        return param_definition_1.ParamDefinition.areEqual(this.props.info.params, params, this.props.info.initialValues);
    };
    VolumeStreamingCustomControls.prototype.newParams = function (params) {
        this.props.events.onChange(params, this.areInitial(params));
    };
    VolumeStreamingCustomControls.prototype.convert = function (channel, stats, isRelative) {
        return __assign({}, channel, { isoValue: isRelative
                ? volume_1.VolumeIsoValue.toRelative(channel.isoValue, stats)
                : volume_1.VolumeIsoValue.toAbsolute(channel.isoValue, stats) });
    };
    VolumeStreamingCustomControls.prototype.render = function () {
        if (!this.props.b)
            return null;
        var b = this.props.b.data;
        var isEM = b.info.kind === 'em';
        var pivot = isEM ? 'em' : '2fo-fc';
        var params = this.props.params;
        var isRelative = params.channels[pivot].isoValue.kind === 'relative';
        var sampling = b.info.header.sampling[0];
        // TODO: factor common things out
        var OptionsParams = {
            view: param_definition_1.ParamDefinition.MappedStatic(params.view.name, {
                'box': param_definition_1.ParamDefinition.Group({
                    bottomLeft: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.zero()),
                    topRight: param_definition_1.ParamDefinition.Vec3(linear_algebra_1.Vec3.zero()),
                    detailLevel: this.props.info.params.detailLevel,
                    isRelative: param_definition_1.ParamDefinition.Boolean(isRelative, { description: 'Use relative or absolute iso values.' })
                }, { description: 'Static box defined by cartesian coords.' }),
                'selection-box': param_definition_1.ParamDefinition.Group({
                    radius: param_definition_1.ParamDefinition.Numeric(5, { min: 0, max: 50, step: 0.5 }),
                    detailLevel: this.props.info.params.detailLevel,
                    isRelative: param_definition_1.ParamDefinition.Boolean(isRelative, { description: 'Use relative or absolute iso values.' })
                }, { description: 'Box around last-interacted element.' }),
                'cell': param_definition_1.ParamDefinition.Group({
                    detailLevel: this.props.info.params.detailLevel,
                    isRelative: param_definition_1.ParamDefinition.Boolean(isRelative, { description: 'Use relative or absolute iso values.' })
                }, { description: 'Box around the structure\'s bounding box.' }),
            }, { options: [['box', 'Bounded Box'], ['selection-box', 'Selection'], ['cell', 'Whole Structure']] })
        };
        var options = {
            view: {
                name: params.view.name,
                params: {
                    detailLevel: params.detailLevel,
                    radius: params.view.params.radius,
                    bottomLeft: params.view.params.bottomLeft,
                    topRight: params.view.params.topRight,
                    isRelative: isRelative
                }
            }
        };
        return React.createElement(React.Fragment, null,
            !isEM && React.createElement(Channel, { label: '2Fo-Fc', name: '2fo-fc', channels: params.channels, changeIso: this.changeIso, changeParams: this.changeParams, isRelative: isRelative, params: this.props, stats: sampling.valuesInfo[0] }),
            !isEM && React.createElement(Channel, { label: 'Fo-Fc(+ve)', name: 'fo-fc(+ve)', channels: params.channels, changeIso: this.changeIso, changeParams: this.changeParams, isRelative: isRelative, params: this.props, stats: sampling.valuesInfo[1] }),
            !isEM && React.createElement(Channel, { label: 'Fo-Fc(-ve)', name: 'fo-fc(-ve)', channels: params.channels, changeIso: this.changeIso, changeParams: this.changeParams, isRelative: isRelative, params: this.props, stats: sampling.valuesInfo[1] }),
            isEM && React.createElement(Channel, { label: 'EM', name: 'em', channels: params.channels, changeIso: this.changeIso, changeParams: this.changeParams, isRelative: isRelative, params: this.props, stats: sampling.valuesInfo[0] }),
            React.createElement(parameters_1.ParameterControls, { onChange: this.changeOption, params: OptionsParams, values: options, onEnter: this.props.events.onEnter }));
    };
    return VolumeStreamingCustomControls;
}(base_1.PluginUIComponent));
exports.VolumeStreamingCustomControls = VolumeStreamingCustomControls;
