"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../mol-util");
var React = require("react");
var base_1 = require("./base");
var controls_1 = require("./controls");
var state_1 = require("./state");
var actions_1 = require("./state/actions");
var tree_1 = require("./state/tree");
var task_1 = require("./task");
var viewport_1 = require("./viewport");
var mol_state_1 = require("../../mol-state");
var update_transform_1 = require("./state/update-transform");
var sequence_1 = require("./sequence");
var Plugin = /** @class */ (function (_super) {
    __extends(Plugin, _super);
    function Plugin() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Plugin.prototype.region = function (kind, element) {
        return React.createElement("div", { className: "msp-layout-region msp-layout-" + kind },
            React.createElement("div", { className: 'msp-layout-static' }, element));
    };
    Plugin.prototype.render = function () {
        return React.createElement(base_1.PluginReactContext.Provider, { value: this.props.plugin },
            React.createElement(Layout, null));
    };
    return Plugin;
}(React.Component));
exports.Plugin = Plugin;
var PluginContextContainer = /** @class */ (function (_super) {
    __extends(PluginContextContainer, _super);
    function PluginContextContainer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PluginContextContainer.prototype.render = function () {
        return React.createElement(base_1.PluginReactContext.Provider, { value: this.props.plugin },
            React.createElement("div", { className: 'msp-plugin' }, this.props.children));
    };
    return PluginContextContainer;
}(React.Component));
exports.PluginContextContainer = PluginContextContainer;
var Layout = /** @class */ (function (_super) {
    __extends(Layout, _super);
    function Layout() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Layout.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.layout.events.updated, function () { return _this.forceUpdate(); });
    };
    Layout.prototype.region = function (kind, Element) {
        return React.createElement("div", { className: "msp-layout-region msp-layout-" + kind },
            React.createElement("div", { className: 'msp-layout-static' }, Element ? React.createElement(Element, null) : null));
    };
    Object.defineProperty(Layout.prototype, "layoutVisibilityClassName", {
        get: function () {
            var layout = this.plugin.layout.state;
            var controls = (this.plugin.spec.layout && this.plugin.spec.layout.controls) || {};
            var classList = [];
            if (controls.top === 'none' || !layout.showControls) {
                classList.push('msp-layout-hide-top');
            }
            if (controls.left === 'none' || !layout.showControls) {
                classList.push('msp-layout-hide-left');
            }
            if (controls.right === 'none' || !layout.showControls) {
                classList.push('msp-layout-hide-right');
            }
            if (controls.bottom === 'none' || !layout.showControls) {
                classList.push('msp-layout-hide-bottom');
            }
            return classList.join(' ');
        },
        enumerable: true,
        configurable: true
    });
    Layout.prototype.render = function () {
        var layout = this.plugin.layout.state;
        var controls = (this.plugin.spec.layout && this.plugin.spec.layout.controls) || {};
        return React.createElement("div", { className: 'msp-plugin' },
            React.createElement("div", { className: "msp-plugin-content " + (layout.isExpanded ? 'msp-layout-expanded' : 'msp-layout-standard msp-layout-standard-outside') },
                React.createElement("div", { className: this.layoutVisibilityClassName },
                    this.region('main', ViewportWrapper),
                    layout.showControls && controls.top !== 'none' && this.region('top', controls.top || sequence_1.SequenceView),
                    layout.showControls && controls.left !== 'none' && this.region('left', controls.left || State),
                    layout.showControls && controls.right !== 'none' && this.region('right', controls.right || ControlsWrapper),
                    layout.showControls && controls.bottom !== 'none' && this.region('bottom', controls.bottom || Log))));
    };
    return Layout;
}(base_1.PluginUIComponent));
var ControlsWrapper = /** @class */ (function (_super) {
    __extends(ControlsWrapper, _super);
    function ControlsWrapper() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ControlsWrapper.prototype.render = function () {
        return React.createElement("div", { className: 'msp-scrollable-container msp-right-controls' },
            React.createElement(CurrentObject, null),
            React.createElement(state_1.StateSnapshots, null));
    };
    return ControlsWrapper;
}(base_1.PluginUIComponent));
exports.ControlsWrapper = ControlsWrapper;
var ViewportWrapper = /** @class */ (function (_super) {
    __extends(ViewportWrapper, _super);
    function ViewportWrapper() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ViewportWrapper.prototype.render = function () {
        return React.createElement(React.Fragment, null,
            React.createElement(viewport_1.Viewport, null),
            React.createElement("div", { className: 'msp-viewport-top-left-controls' },
                React.createElement(controls_1.AnimationViewportControls, null),
                React.createElement(controls_1.TrajectoryViewportControls, null),
                React.createElement(controls_1.StateSnapshotViewportControls, null)),
            React.createElement(viewport_1.ViewportControls, null),
            React.createElement("div", { style: { position: 'absolute', left: '10px', bottom: '10px' } },
                React.createElement(task_1.BackgroundTaskProgress, null)),
            React.createElement(controls_1.LociLabelControl, null));
    };
    return ViewportWrapper;
}(base_1.PluginUIComponent));
exports.ViewportWrapper = ViewportWrapper;
var State = /** @class */ (function (_super) {
    __extends(State, _super);
    function State() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    State.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.state.behavior.kind, function () { return _this.forceUpdate(); });
    };
    State.prototype.set = function (kind) {
        // TODO: do command for this?
        this.plugin.state.setKind(kind);
    };
    State.prototype.render = function () {
        var _this = this;
        var kind = this.plugin.state.behavior.kind.value;
        return React.createElement("div", { className: 'msp-scrollable-container' },
            React.createElement("div", { className: 'msp-btn-row-group msp-data-beh' },
                React.createElement("button", { className: 'msp-btn msp-btn-block msp-form-control', onClick: function () { return _this.set('data'); }, style: { fontWeight: kind === 'data' ? 'bold' : 'normal' } },
                    React.createElement("span", { className: 'msp-icon msp-icon-database' }),
                    " Data"),
                React.createElement("button", { className: 'msp-btn msp-btn-block msp-form-control', onClick: function () { return _this.set('behavior'); }, style: { fontWeight: kind === 'behavior' ? 'bold' : 'normal' } },
                    React.createElement("span", { className: 'msp-icon msp-icon-tools' }),
                    " Behavior")),
            React.createElement(tree_1.StateTree, { state: kind === 'data' ? this.plugin.state.dataState : this.plugin.state.behaviorState }));
    };
    return State;
}(base_1.PluginUIComponent));
exports.State = State;
var Log = /** @class */ (function (_super) {
    __extends(Log, _super);
    function Log() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.wrapper = React.createRef();
        _this.state = { entries: _this.plugin.log.entries };
        return _this;
    }
    Log.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.events.log, function () { return _this.setState({ entries: _this.plugin.log.entries }); });
    };
    Log.prototype.componentDidUpdate = function () {
        this.scrollToBottom();
    };
    Log.prototype.scrollToBottom = function () {
        var log = this.wrapper.current;
        if (log)
            log.scrollTop = log.scrollHeight - log.clientHeight - 1;
    };
    Log.prototype.render = function () {
        // TODO: ability to show full log
        // showing more entries dramatically slows animations.
        var maxEntries = 10;
        var xs = this.state.entries, l = xs.size;
        var entries = [];
        for (var i = Math.max(0, l - maxEntries), o = 0; i < l; i++) {
            var e = xs.get(i);
            entries.push(React.createElement("li", { key: o++ },
                React.createElement("div", { className: 'msp-log-entry-badge msp-log-entry-' + e.type }),
                React.createElement("div", { className: 'msp-log-timestamp' }, mol_util_1.formatTime(e.timestamp)),
                React.createElement("div", { className: 'msp-log-entry' }, e.message)));
        }
        return React.createElement("div", { ref: this.wrapper, className: 'msp-log', style: { position: 'absolute', top: '0', right: '0', bottom: '0', left: '0', overflowY: 'auto' } },
            React.createElement("ul", { className: 'msp-list-unstyled' }, entries));
    };
    return Log;
}(base_1.PluginUIComponent));
exports.Log = Log;
var CurrentObject = /** @class */ (function (_super) {
    __extends(CurrentObject, _super);
    function CurrentObject() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(CurrentObject.prototype, "current", {
        get: function () {
            return this.plugin.state.behavior.currentObject.value;
        },
        enumerable: true,
        configurable: true
    });
    CurrentObject.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.state.behavior.currentObject, function (o) {
            _this.forceUpdate();
        });
        this.subscribe(this.plugin.events.state.object.updated, function (_a) {
            var ref = _a.ref, state = _a.state;
            var current = _this.current;
            if (current.ref !== ref || current.state !== state)
                return;
            _this.forceUpdate();
        });
    };
    CurrentObject.prototype.render = function () {
        var current = this.current;
        var ref = current.ref;
        var cell = current.state.cells.get(ref);
        var transform = cell.transform;
        var showActions = true;
        if (ref === mol_state_1.StateTransform.RootRef) {
            var children = current.state.tree.children.get(ref);
            showActions = children.size !== 0;
        }
        if (!showActions)
            return null;
        return React.createElement(React.Fragment, null,
            (cell.status === 'ok' || cell.status === 'error') && React.createElement(update_transform_1.UpdateTransformControl, { state: current.state, transform: transform }),
            cell.status === 'ok' && React.createElement(actions_1.StateObjectActions, { state: current.state, nodeRef: ref, initiallyColapsed: true }));
    };
    return CurrentObject;
}(base_1.PluginUIComponent));
exports.CurrentObject = CurrentObject;
