"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var base_1 = require("./base");
var spine_1 = require("../../mol-state/tree/spine");
var objects_1 = require("../state/objects");
var sequence_1 = require("./sequence/sequence");
var structure_1 = require("../../mol-model/structure");
var polymer_1 = require("./sequence/polymer");
var marker_action_1 = require("../../mol-util/marker-action");
var parameters_1 = require("./controls/parameters");
var param_definition_1 = require("../../mol-util/param-definition");
function opKey(l) {
    var ids = structure_1.StructureProperties.unit.pdbx_struct_oper_list_ids(l);
    var ncs = structure_1.StructureProperties.unit.struct_ncs_oper_id(l);
    var hkl = structure_1.StructureProperties.unit.hkl(l);
    var spgrOp = structure_1.StructureProperties.unit.spgrOp(l);
    return ids.sort().join(',') + "|" + ncs + "|" + hkl + "|" + spgrOp;
}
function getSequenceWrapper(state, structureSelection) {
    var structure = state.structure, entity = state.entity, chain = state.chain, operator = state.operator;
    var l = structure_1.StructureElement.create();
    for (var i = 0, il = structure.units.length; i < il; ++i) {
        var unit = structure.units[i];
        if (unit.polymerElements.length === 0)
            continue;
        structure_1.StructureElement.set(l, unit, unit.elements[0]);
        if (structure_1.StructureProperties.entity.id(l) !== entity)
            continue;
        if (structure_1.StructureProperties.chain.label_asym_id(l) !== chain)
            continue;
        if (opKey(l) !== operator)
            continue;
        // console.log('new PolymerSequenceWrapper', structureSelection.get(structure))
        var sw = new polymer_1.PolymerSequenceWrapper({ structure: structure, unit: unit });
        sw.markResidue(structureSelection.get(structure), marker_action_1.MarkerAction.Select);
        return sw;
    }
}
function getEntityOptions(structure) {
    var options = [];
    var l = structure_1.StructureElement.create();
    var seen = new Set();
    structure.units.forEach(function (unit) {
        if (unit.polymerElements.length === 0)
            return;
        structure_1.StructureElement.set(l, unit, unit.elements[0]);
        var id = structure_1.StructureProperties.entity.id(l);
        if (seen.has(id))
            return;
        var label = id + ": " + structure_1.StructureProperties.entity.pdbx_description(l).join(', ');
        options.push([id, label]);
        seen.add(id);
    });
    if (options.length === 0)
        options.push(['', 'No entities']);
    return options;
}
function getChainOptions(structure, entityId) {
    var options = [];
    var l = structure_1.StructureElement.create();
    var seen = new Set();
    structure.units.forEach(function (unit) {
        if (unit.polymerElements.length === 0)
            return;
        structure_1.StructureElement.set(l, unit, unit.elements[0]);
        if (structure_1.StructureProperties.entity.id(l) !== entityId)
            return;
        var id = structure_1.StructureProperties.chain.label_asym_id(l);
        if (seen.has(id))
            return;
        var label = id + ": " + structure_1.StructureProperties.chain.auth_asym_id(l);
        options.push([id, label]);
        seen.add(id);
    });
    if (options.length === 0)
        options.push(['', 'No chains']);
    return options;
}
function getOperatorOptions(structure, entityId, label_asym_id) {
    var options = [];
    var l = structure_1.StructureElement.create();
    var seen = new Set();
    structure.units.forEach(function (unit) {
        if (unit.polymerElements.length === 0)
            return;
        structure_1.StructureElement.set(l, unit, unit.elements[0]);
        if (structure_1.StructureProperties.entity.id(l) !== entityId)
            return;
        if (structure_1.StructureProperties.chain.label_asym_id(l) !== label_asym_id)
            return;
        var id = opKey(l);
        if (seen.has(id))
            return;
        var label = unit.conformation.operator.name;
        options.push([id, label]);
        seen.add(id);
    });
    if (options.length === 0)
        options.push(['', 'No operators']);
    return options;
}
var SequenceView = /** @class */ (function (_super) {
    __extends(SequenceView, _super);
    function SequenceView(props, context) {
        var _this = _super.call(this, props, context) || this;
        _this.state = { structure: structure_1.Structure.Empty, entity: '', chain: '', operator: '' };
        _this.setParamProps = function (p) {
            var state = __assign({}, _this.state);
            switch (p.name) {
                case 'entity':
                    state.entity = p.value;
                    state.chain = getChainOptions(state.structure, state.entity)[0][0];
                    state.operator = getOperatorOptions(state.structure, state.entity, state.chain)[0][0];
                    break;
                case 'chain':
                    state.chain = p.value;
                    state.operator = getOperatorOptions(state.structure, state.entity, state.chain)[0][0];
                    break;
                case 'operator':
                    state.operator = p.value;
                    break;
            }
            _this.setState(state);
        };
        _this.spine = new spine_1.StateTreeSpine.Impl(_this.plugin.state.dataState.cells);
        return _this;
    }
    SequenceView.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.state.behavior.currentObject, function (o) {
            var current = _this.plugin.state.dataState.cells.get(o.ref);
            _this.spine.current = current;
            if (!structure_1.Structure.areParentsEqual(_this.state.structure, _this.getStructure())) {
                _this.setState(_this.getInitialState());
            }
        });
        this.subscribe(this.plugin.events.state.object.updated, function (_a) {
            var ref = _a.ref, state = _a.state;
            var current = _this.spine.current;
            if (!current || current.sourceRef !== ref || current.state !== state)
                return;
            _this.setState(_this.getInitialState());
        });
    };
    SequenceView.prototype.getStructure = function () {
        var so = this.spine.getRootOfType(objects_1.PluginStateObject.Molecule.Structure);
        return (so && so.data) || structure_1.Structure.Empty;
    };
    SequenceView.prototype.getSequenceWrapper = function () {
        return getSequenceWrapper(this.state, this.plugin.helpers.structureSelection);
    };
    SequenceView.prototype.getInitialState = function () {
        var structure = this.getStructure();
        var entity = getEntityOptions(structure)[0][0];
        var chain = getChainOptions(structure, entity)[0][0];
        var operator = getOperatorOptions(structure, entity, chain)[0][0];
        return { structure: structure, entity: entity, chain: chain, operator: operator };
    };
    Object.defineProperty(SequenceView.prototype, "params", {
        get: function () {
            var _a = this.state, structure = _a.structure, entity = _a.entity, chain = _a.chain;
            var entityOptions = getEntityOptions(structure);
            var chainOptions = getChainOptions(structure, entity);
            var operatorOptions = getOperatorOptions(structure, entity, chain);
            return {
                entity: param_definition_1.ParamDefinition.Select(entityOptions[0][0], entityOptions),
                chain: param_definition_1.ParamDefinition.Select(chainOptions[0][0], chainOptions),
                operator: param_definition_1.ParamDefinition.Select(operatorOptions[0][0], operatorOptions)
            };
        },
        enumerable: true,
        configurable: true
    });
    SequenceView.prototype.render = function () {
        if (this.state.structure === structure_1.Structure.Empty)
            return React.createElement("div", { className: 'msp-sequence' },
                React.createElement("div", { className: 'msp-sequence-wrapper' }, "No structure available"));
        var sequenceWrapper = this.getSequenceWrapper();
        return React.createElement("div", { className: 'msp-sequence' },
            React.createElement("div", { className: 'msp-sequence-select' },
                React.createElement(parameters_1.ParameterControls, { params: this.params, values: this.state, onChange: this.setParamProps })),
            sequenceWrapper !== undefined
                ? React.createElement(sequence_1.Sequence, { sequenceWrapper: sequenceWrapper })
                : React.createElement("div", { className: 'msp-sequence-wrapper' }, "No sequence available"));
    };
    return SequenceView;
}(base_1.PluginUIComponent));
exports.SequenceView = SequenceView;
