"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var structure_1 = require("../../../mol-model/structure");
var base_1 = require("../base");
var interactivity_1 = require("../../util/interactivity");
var input_observer_1 = require("../../../mol-util/input/input-observer");
var mol_util_1 = require("../../../mol-util");
var util_1 = require("./util");
var residue_1 = require("./residue");
// TODO: this is really inefficient and should be done using a canvas.
var BaseSequence = /** @class */ (function (_super) {
    __extends(BaseSequence, _super);
    function BaseSequence() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            markerData: mol_util_1.ValueBox.create(new Uint8Array(_this.props.markerArray))
        };
        _this.lociHighlightProvider = function (loci, action) {
            var markerData = _this.state.markerData;
            var changed = util_1.markResidue(loci.loci, _this.props.structureSeq, markerData.value, action);
            if (changed)
                _this.setState({ markerData: mol_util_1.ValueBox.withValue(markerData, markerData.value) });
        };
        _this.lociSelectionProvider = function (loci, action) {
            var markerData = _this.state.markerData;
            var changed = util_1.markResidue(loci.loci, _this.props.structureSeq, markerData.value, action);
            if (changed)
                _this.setState({ markerData: mol_util_1.ValueBox.withValue(markerData, markerData.value) });
        };
        _this.contextMenu = function (e) {
            e.preventDefault();
        };
        _this.mouseDown = function (e) {
            var buttons = input_observer_1.getButtons(e.nativeEvent);
            var modifiers = input_observer_1.getModifiers(e.nativeEvent);
            _this.click(undefined, buttons, modifiers);
        };
        return _this;
    }
    BaseSequence.getDerivedStateFromProps = function (nextProps, prevState) {
        if (prevState.markerData.value !== nextProps.markerArray) {
            return { markerData: mol_util_1.ValueBox.create(nextProps.markerArray) };
        }
        return null;
    };
    BaseSequence.prototype.componentDidMount = function () {
        this.plugin.interactivity.lociHighlights.addProvider(this.lociHighlightProvider);
        this.plugin.interactivity.lociSelections.addProvider(this.lociSelectionProvider);
    };
    BaseSequence.prototype.componentWillUnmount = function () {
        this.plugin.interactivity.lociHighlights.removeProvider(this.lociHighlightProvider);
        this.plugin.interactivity.lociSelections.removeProvider(this.lociSelectionProvider);
    };
    BaseSequence.prototype.getLoci = function (seqId) {
        var _a = this.props.structureSeq, structure = _a.structure, seq = _a.seq;
        var query = util_1.createResidueQuery(seq.entityId, seqId);
        return structure_1.StructureSelection.toLoci2(structure_1.StructureQuery.run(query, structure));
    };
    BaseSequence.prototype.highlight = function (seqId, modifiers) {
        var ev = { current: interactivity_1.Interactivity.Loci.Empty, modifiers: modifiers };
        if (seqId !== undefined) {
            var loci = this.getLoci(seqId);
            if (loci.elements.length > 0)
                ev.current = { loci: loci };
        }
        this.plugin.behaviors.interaction.highlight.next(ev);
    };
    BaseSequence.prototype.click = function (seqId, buttons, modifiers) {
        var ev = { current: interactivity_1.Interactivity.Loci.Empty, buttons: buttons, modifiers: modifiers };
        if (seqId !== undefined) {
            var loci = this.getLoci(seqId);
            if (loci.elements.length > 0)
                ev.current = { loci: loci };
        }
        this.plugin.behaviors.interaction.click.next(ev);
    };
    BaseSequence.prototype.render = function () {
        var markerData = this.state.markerData;
        var seq = this.props.structureSeq.seq;
        var _a = seq.sequence, offset = _a.offset, sequence = _a.sequence;
        var elems = [];
        for (var i = 0, _i = sequence.length; i < _i; i++) {
            elems[elems.length] = React.createElement(residue_1.Residue, { seqId: offset + i + 1, letter: sequence[i], parent: this, marker: markerData.value[i], key: i });
        }
        return React.createElement("div", { className: 'msp-sequence-entity', onContextMenu: this.contextMenu, onMouseDown: this.mouseDown },
            React.createElement("span", { style: { fontWeight: 'bold' } },
                seq.entityId,
                ":",
                offset,
                "\u00A0"),
            elems);
    };
    return BaseSequence;
}(base_1.PluginUIComponent));
exports.BaseSequence = BaseSequence;
