"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../../mol-model/structure");
var util_1 = require("./util");
var int_1 = require("../../../mol-data/int");
var tables_1 = require("../../../mol-util/color/tables");
var PolymerSequenceWrapper = /** @class */ (function (_super) {
    __extends(PolymerSequenceWrapper, _super);
    function PolymerSequenceWrapper(data) {
        var _this = this;
        var l = structure_1.StructureElement.create(data.unit, data.unit.elements[0]);
        var sequence = data.unit.model.sequence.byEntityKey[structure_1.StructureProperties.entity.key(l)].sequence;
        var markerArray = new Uint8Array(sequence.sequence.length);
        _this = _super.call(this, data, markerArray, sequence.sequence.length) || this;
        _this.sequence = sequence;
        _this.location = structure_1.StructureElement.create();
        _this.missing = data.unit.model.properties.missingResidues;
        _this.modelNum = data.unit.model.modelNum;
        _this.asymId = structure_1.StructureProperties.chain.label_asym_id(l);
        return _this;
    }
    PolymerSequenceWrapper.prototype.seqId = function (i) {
        return this.sequence.offset + i + 1;
    };
    PolymerSequenceWrapper.prototype.residueLabel = function (i) {
        return this.sequence.sequence[i];
    };
    PolymerSequenceWrapper.prototype.residueColor = function (i) {
        return this.missing.has(this.modelNum, this.asymId, this.seqId(i))
            ? tables_1.ColorNames.grey
            : tables_1.ColorNames.black;
    };
    PolymerSequenceWrapper.prototype.eachResidue = function (loci, apply) {
        var changed = false;
        var _a = this.data, structure = _a.structure, unit = _a.unit;
        if (!structure_1.StructureElement.isLoci(loci))
            return false;
        if (!structure_1.Structure.areParentsEquivalent(loci.structure, structure))
            return false;
        var location = this.location;
        var _loop_1 = function (e) {
            var rIprev = -1;
            location.unit = e.unit;
            var residueIndex = e.unit.model.atomicHierarchy.residueAtomSegments.index;
            int_1.OrderedSet.forEach(e.indices, function (v) {
                location.element = e.unit.elements[v];
                var rI = residueIndex[location.element];
                // avoid checking for the same residue multiple times
                if (rI !== rIprev) {
                    if (structure_1.StructureProperties.unit.id(location) !== unit.id)
                        return;
                    if (apply(getSeqIdInterval(location)))
                        changed = true;
                    rIprev = rI;
                }
            });
        };
        for (var _i = 0, _b = loci.elements; _i < _b.length; _i++) {
            var e = _b[_i];
            _loop_1(e);
        }
        return changed;
    };
    PolymerSequenceWrapper.prototype.getLoci = function (seqId) {
        var query = createResidueQuery(this.data.unit.id, seqId);
        return structure_1.StructureSelection.toLoci2(structure_1.StructureQuery.run(query, this.data.structure));
    };
    return PolymerSequenceWrapper;
}(util_1.SequenceWrapper));
exports.PolymerSequenceWrapper = PolymerSequenceWrapper;
function createResidueQuery(unitId, label_seq_id) {
    return structure_1.Queries.generators.atoms({
        unitTest: function (ctx) {
            return structure_1.StructureProperties.unit.id(ctx.element) === unitId;
        },
        residueTest: function (ctx) {
            if (ctx.element.unit.kind === 0 /* Atomic */) {
                return structure_1.StructureProperties.residue.label_seq_id(ctx.element) === label_seq_id;
            }
            else {
                return (structure_1.StructureProperties.coarse.seq_id_begin(ctx.element) <= label_seq_id &&
                    structure_1.StructureProperties.coarse.seq_id_end(ctx.element) >= label_seq_id);
            }
        }
    });
}
/** Zero-indexed */
function getSeqIdInterval(location) {
    var unit = location.unit, element = location.element;
    var model = unit.model;
    switch (unit.kind) {
        case 0 /* Atomic */:
            var residueIndex = model.atomicHierarchy.residueAtomSegments.index[element];
            var seqId = model.atomicHierarchy.residues.label_seq_id.value(residueIndex);
            return int_1.Interval.ofSingleton(seqId - 1);
        case 1 /* Spheres */:
            return int_1.Interval.ofRange(model.coarseHierarchy.spheres.seq_id_begin.value(element) - 1, model.coarseHierarchy.spheres.seq_id_end.value(element) - 1);
        case 2 /* Gaussians */:
            return int_1.Interval.ofRange(model.coarseHierarchy.gaussians.seq_id_begin.value(element) - 1, model.coarseHierarchy.gaussians.seq_id_end.value(element) - 1);
    }
}
