"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var base_1 = require("../base");
var interactivity_1 = require("../../util/interactivity");
var input_observer_1 = require("../../../mol-util/input/input-observer");
var mol_util_1 = require("../../../mol-util");
var residue_1 = require("./residue");
function getState(markerData) {
    return { markerData: mol_util_1.ValueBox.withValue(markerData, markerData.value) };
}
// TODO: this is really inefficient and should be done using a canvas.
var Sequence = /** @class */ (function (_super) {
    __extends(Sequence, _super);
    function Sequence() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            markerData: mol_util_1.ValueBox.create(_this.props.sequenceWrapper.markerArray)
        };
        _this.lociHighlightProvider = function (loci, action) {
            var changed = _this.props.sequenceWrapper.markResidue(loci.loci, action);
            if (changed)
                _this.setMarkerData(_this.state.markerData);
        };
        _this.lociSelectionProvider = function (loci, action) {
            var changed = _this.props.sequenceWrapper.markResidue(loci.loci, action);
            if (changed)
                _this.setMarkerData(_this.state.markerData);
        };
        _this.contextMenu = function (e) {
            e.preventDefault();
        };
        _this.mouseDown = function (e) {
            var buttons = input_observer_1.getButtons(e.nativeEvent);
            var modifiers = input_observer_1.getModifiers(e.nativeEvent);
            _this.click(undefined, buttons, modifiers);
        };
        return _this;
    }
    Sequence.prototype.setMarkerData = function (markerData) {
        this.setState(getState(markerData));
    };
    Sequence.getDerivedStateFromProps = function (nextProps, prevState) {
        if (prevState.markerData.value !== nextProps.sequenceWrapper.markerArray) {
            return getState(mol_util_1.ValueBox.create(nextProps.sequenceWrapper.markerArray));
        }
        return null;
    };
    Sequence.prototype.componentDidMount = function () {
        this.plugin.interactivity.lociHighlights.addProvider(this.lociHighlightProvider);
        this.plugin.interactivity.lociSelections.addProvider(this.lociSelectionProvider);
    };
    Sequence.prototype.componentWillUnmount = function () {
        this.plugin.interactivity.lociHighlights.removeProvider(this.lociHighlightProvider);
        this.plugin.interactivity.lociSelections.removeProvider(this.lociSelectionProvider);
    };
    Sequence.prototype.highlight = function (seqId, modifiers) {
        var ev = { current: interactivity_1.Interactivity.Loci.Empty, modifiers: modifiers };
        if (seqId !== undefined) {
            var loci = this.props.sequenceWrapper.getLoci(seqId);
            if (loci.elements.length > 0)
                ev.current = { loci: loci };
        }
        this.plugin.behaviors.interaction.highlight.next(ev);
    };
    Sequence.prototype.click = function (seqId, buttons, modifiers) {
        var ev = { current: interactivity_1.Interactivity.Loci.Empty, buttons: buttons, modifiers: modifiers };
        if (seqId !== undefined) {
            var loci = this.props.sequenceWrapper.getLoci(seqId);
            if (loci.elements.length > 0)
                ev.current = { loci: loci };
        }
        this.plugin.behaviors.interaction.click.next(ev);
    };
    Sequence.prototype.render = function () {
        var markerData = this.state.markerData;
        var sw = this.props.sequenceWrapper;
        var elems = [];
        for (var i = 0, il = sw.length; i < il; ++i) {
            elems[elems.length] = React.createElement(residue_1.Residue, { seqId: sw.seqId(i), label: sw.residueLabel(i), parent: this, marker: markerData.value[i], color: sw.residueColor(i), key: i });
        }
        return React.createElement("div", { className: 'msp-sequence-wrapper', onContextMenu: this.contextMenu, onMouseDown: this.mouseDown }, elems);
    };
    return Sequence;
}(base_1.PluginUIComponent));
exports.Sequence = Sequence;
