"use strict";
/**
 * Copyright (c) 2018 - 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var mol_state_1 = require("../../../mol-state");
var command_1 = require("../../../mol-plugin/command");
var base_1 = require("../base");
var actions_1 = require("./actions");
var StateTree = /** @class */ (function (_super) {
    __extends(StateTree, _super);
    function StateTree() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = { showActions: true };
        return _this;
    }
    StateTree.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.events.state.cell.created, function (e) {
            if (e.cell.transform.parent === mol_state_1.StateTransform.RootRef)
                _this.forceUpdate();
        });
        this.subscribe(this.plugin.events.state.cell.removed, function (e) {
            if (e.parent === mol_state_1.StateTransform.RootRef)
                _this.forceUpdate();
        });
    };
    StateTree.getDerivedStateFromProps = function (props, state) {
        var n = props.state.tree.root.ref;
        var children = props.state.tree.children.get(n);
        var showActions = children.size === 0;
        if (state.showActions === showActions)
            return null;
        return { showActions: showActions };
    };
    StateTree.prototype.render = function () {
        var ref = this.props.state.tree.root.ref;
        if (this.state.showActions) {
            return React.createElement(actions_1.StateObjectActions, { state: this.props.state, nodeRef: ref, hideHeader: true });
        }
        return React.createElement(StateTreeNode, { cell: this.props.state.cells.get(ref), depth: 0 });
    };
    return StateTree;
}(base_1.PluginUIComponent));
exports.StateTree = StateTree;
var StateTreeNode = /** @class */ (function (_super) {
    __extends(StateTreeNode, _super);
    function StateTreeNode() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isCollapsed: !!_this.props.cell.state.isCollapsed
        };
        return _this;
    }
    StateTreeNode.prototype.is = function (e) {
        return e.ref === this.ref && e.state === this.props.cell.parent;
    };
    Object.defineProperty(StateTreeNode.prototype, "ref", {
        get: function () {
            return this.props.cell.transform.ref;
        },
        enumerable: true,
        configurable: true
    });
    StateTreeNode.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.events.state.cell.stateUpdated, function (e) {
            if (_this.props.cell === e.cell && _this.is(e) && e.state.cells.has(_this.ref)) {
                _this.forceUpdate();
                // if (!!this.props.cell.state.isCollapsed !== this.state.isCollapsed) {
                //     this.setState({ isCollapsed: !!e.cell.state.isCollapsed });
                // }
            }
        });
        this.subscribe(this.plugin.events.state.cell.created, function (e) {
            if (_this.props.cell.parent === e.state && _this.ref === e.cell.transform.parent) {
                _this.forceUpdate();
            }
        });
        this.subscribe(this.plugin.events.state.cell.removed, function (e) {
            if (_this.props.cell.parent === e.state && _this.ref === e.parent) {
                _this.forceUpdate();
            }
        });
    };
    StateTreeNode.getDerivedStateFromProps = function (props, state) {
        if (!!props.cell.state.isCollapsed === state.isCollapsed)
            return null;
        return { isCollapsed: !!props.cell.state.isCollapsed };
    };
    StateTreeNode.prototype.render = function () {
        var cell = this.props.cell;
        if (!cell || cell.obj === mol_state_1.StateObject.Null || !cell.parent.tree.transforms.has(cell.transform.ref)) {
            return null;
        }
        var cellState = cell.state;
        var showLabel = cell.status !== 'ok' || !cell.state.isGhost;
        var children = cell.parent.tree.children.get(this.ref);
        var newDepth = showLabel ? this.props.depth + 1 : this.props.depth;
        if (!showLabel) {
            if (children.size === 0)
                return null;
            return React.createElement("div", { style: { display: cellState.isCollapsed ? 'none' : 'block' } }, children.map(function (c) { return React.createElement(StateTreeNode, { cell: cell.parent.cells.get(c), key: c, depth: newDepth }); }));
        }
        return React.createElement(React.Fragment, null,
            React.createElement(StateTreeNodeLabel, { cell: cell, depth: this.props.depth }),
            children.size === 0
                ? void 0
                : React.createElement("div", { style: { display: cellState.isCollapsed ? 'none' : 'block' } }, children.map(function (c) { return React.createElement(StateTreeNode, { cell: cell.parent.cells.get(c), key: c, depth: newDepth }); })));
    };
    return StateTreeNode;
}(base_1.PluginUIComponent));
var StateTreeNodeLabel = /** @class */ (function (_super) {
    __extends(StateTreeNodeLabel, _super);
    function StateTreeNodeLabel() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isCurrent: _this.props.cell.parent.current === _this.ref,
            isCollapsed: !!_this.props.cell.state.isCollapsed
        };
        _this.setCurrent = function (e) {
            e.preventDefault();
            e.currentTarget.blur();
            command_1.PluginCommands.State.SetCurrentObject.dispatch(_this.plugin, { state: _this.props.cell.parent, ref: _this.ref });
        };
        _this.remove = function (e) {
            e.preventDefault();
            command_1.PluginCommands.State.RemoveObject.dispatch(_this.plugin, { state: _this.props.cell.parent, ref: _this.ref, removeParentGhosts: true });
        };
        _this.toggleVisible = function (e) {
            e.preventDefault();
            command_1.PluginCommands.State.ToggleVisibility.dispatch(_this.plugin, { state: _this.props.cell.parent, ref: _this.ref });
            e.currentTarget.blur();
        };
        _this.toggleExpanded = function (e) {
            e.preventDefault();
            command_1.PluginCommands.State.ToggleExpanded.dispatch(_this.plugin, { state: _this.props.cell.parent, ref: _this.ref });
            e.currentTarget.blur();
        };
        _this.highlight = function (e) {
            e.preventDefault();
            command_1.PluginCommands.State.Highlight.dispatch(_this.plugin, { state: _this.props.cell.parent, ref: _this.ref });
            e.currentTarget.blur();
        };
        _this.clearHighlight = function (e) {
            e.preventDefault();
            command_1.PluginCommands.State.ClearHighlight.dispatch(_this.plugin, { state: _this.props.cell.parent, ref: _this.ref });
            e.currentTarget.blur();
        };
        return _this;
    }
    StateTreeNodeLabel.prototype.is = function (e) {
        return e.ref === this.ref && e.state === this.props.cell.parent;
    };
    Object.defineProperty(StateTreeNodeLabel.prototype, "ref", {
        get: function () {
            return this.props.cell.transform.ref;
        },
        enumerable: true,
        configurable: true
    });
    StateTreeNodeLabel.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.events.state.cell.stateUpdated, function (e) {
            if (_this.is(e))
                _this.forceUpdate();
        });
        this.subscribe(this.plugin.state.behavior.currentObject, function (e) {
            if (!_this.is(e)) {
                if (_this.state.isCurrent && e.state.transforms.has(_this.ref)) {
                    _this.setState({ isCurrent: _this.props.cell.parent.current === _this.ref });
                }
                return;
            }
            if (e.state.transforms.has(_this.ref)) {
                _this.setState({
                    isCurrent: _this.props.cell.parent.current === _this.ref,
                    isCollapsed: !!_this.props.cell.state.isCollapsed
                });
            }
        });
    };
    StateTreeNodeLabel.getDerivedStateFromProps = function (props, state) {
        var isCurrent = props.cell.parent.current === props.cell.transform.ref;
        var isCollapsed = !!props.cell.state.isCollapsed;
        if (state.isCollapsed === isCollapsed && state.isCurrent === isCurrent)
            return null;
        return { isCurrent: isCurrent, isCollapsed: isCollapsed };
    };
    // private toggleUpdaterObs = new Subject();
    // toggleUpdater = (e: React.MouseEvent<HTMLAnchorElement>) => {
    //     e.preventDefault();
    //     e.currentTarget.blur();
    //     this.toggleUpdaterObs.next();
    // }
    StateTreeNodeLabel.prototype.render = function () {
        var cell = this.props.cell;
        var n = cell.transform;
        if (!cell)
            return null;
        var isCurrent = this.state.isCurrent; // this.is(cell.parent.behaviors.currentObject.value);
        var label;
        if (cell.status === 'pending' || cell.status === 'processing') {
            var name_1 = n.transformer.definition.display.name;
            label = React.createElement(React.Fragment, null,
                React.createElement("b", null,
                    "[",
                    cell.status,
                    "]"),
                " ",
                React.createElement("span", { title: name_1 }, name_1));
        }
        else if (cell.status !== 'ok' || !cell.obj) {
            var name_2 = n.transformer.definition.display.name;
            var title = "" + cell.errorText;
            label = React.createElement(React.Fragment, null,
                React.createElement("a", { title: title, href: '#', onClick: this.setCurrent },
                    React.createElement("b", null,
                        "[",
                        cell.status,
                        "]"),
                    " ",
                    name_2,
                    ": ",
                    React.createElement("i", null, cell.errorText),
                    " "));
        }
        else {
            var obj = cell.obj;
            var title = obj.label + " " + (obj.description ? obj.description : '');
            if (this.state.isCurrent) {
                label = React.createElement(React.Fragment, null,
                    React.createElement("a", { title: title, href: '#' },
                        React.createElement("b", null, obj.label),
                        " ",
                        obj.description ? React.createElement("small", null, obj.description) : void 0));
            }
            else {
                label = React.createElement(React.Fragment, null,
                    React.createElement("a", { title: title, href: '#', onClick: this.setCurrent },
                        obj.label,
                        " ",
                        obj.description ? React.createElement("small", null, obj.description) : void 0));
            }
        }
        var children = cell.parent.tree.children.get(this.ref);
        var cellState = cell.state;
        var visibility = React.createElement("button", { onClick: this.toggleVisible, className: "msp-btn msp-btn-link msp-tree-visibility" + (cellState.isHidden ? ' msp-tree-visibility-hidden' : '') },
            React.createElement("span", { className: 'msp-icon msp-icon-visual-visibility' }));
        var style = {
            marginLeft: /* this.state.isCurrent ? void 0 :*/ this.props.depth * 10 + "px",
            // paddingLeft: !this.state.isCurrent ? void 0 : `${this.props.depth * 10}px`,
            borderLeft: /* isCurrent || */ this.props.depth === 0 ? 'none' : void 0
        };
        var row = React.createElement("div", { className: "msp-tree-row" + (isCurrent ? ' msp-tree-row-current' : ''), onMouseEnter: this.highlight, onMouseLeave: this.clearHighlight, style: style },
            label,
            children.size > 0 && React.createElement("button", { onClick: this.toggleExpanded, className: 'msp-btn msp-btn-link msp-tree-toggle-exp-button' },
                React.createElement("span", { className: "msp-icon msp-icon-" + (cellState.isCollapsed ? 'expand' : 'collapse') })),
            !cell.state.isLocked && React.createElement("button", { onClick: this.remove, className: 'msp-btn msp-btn-link msp-tree-remove-button' },
                React.createElement("span", { className: 'msp-icon msp-icon-remove' })),
            visibility);
        // if (this.state.isCurrent) {
        //     return <>
        //         {row}
        //         <StateTreeNodeTransform {...this.props} toggleCollapsed={this.toggleUpdaterObs} />
        //     </>
        // }
        return row;
    };
    return StateTreeNodeLabel;
}(base_1.PluginUIComponent));
// class StateTreeNodeTransform extends PluginUIComponent<{ nodeRef: string, state: State, depth: number, toggleCollapsed?: Observable<any> }> {
//     componentDidMount() {
//         // this.subscribe(this.plugin.events.state.object.updated, ({ ref, state }) => {
//         //     if (this.props.nodeRef !== ref || this.props.state !== state) return;
//         //     this.forceUpdate();
//         // });
//     }
//     render() {
//         const ref = this.props.nodeRef;
//         const cell = this.props.state.cells.get(ref)!;
//         const parent: StateObjectCell | undefined = (cell.sourceRef && this.props.state.cells.get(cell.sourceRef)!) || void 0;
//         if (!parent || parent.status !== 'ok') return null;
//         const transform = cell.transform;
//         return <UpdateTransformContol state={this.props.state} transform={transform} initiallyCollapsed={true} toggleCollapsed={this.props.toggleCollapsed} />;
//     }
// }
