"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var base_1 = require("./base");
var command_1 = require("../../mol-plugin/command");
var parameters_1 = require("./controls/parameters");
var canvas3d_1 = require("../../mol-canvas3d/canvas3d");
var layout_1 = require("../../mol-plugin/layout");
var common_1 = require("./controls/common");
var util_1 = require("../../mol-canvas3d/util");
var interactivity_1 = require("../util/interactivity");
var ViewportControls = /** @class */ (function (_super) {
    __extends(ViewportControls, _super);
    function ViewportControls() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isSettingsExpanded: false
        };
        _this.resetCamera = function () {
            command_1.PluginCommands.Camera.Reset.dispatch(_this.plugin, {});
        };
        _this.toggleSettingsExpanded = function (e) {
            _this.setState({ isSettingsExpanded: !_this.state.isSettingsExpanded });
            e.currentTarget.blur();
        };
        _this.toggleControls = function () {
            command_1.PluginCommands.Layout.Update.dispatch(_this.plugin, { state: { showControls: !_this.plugin.layout.state.showControls } });
        };
        _this.toggleExpanded = function () {
            command_1.PluginCommands.Layout.Update.dispatch(_this.plugin, { state: { isExpanded: !_this.plugin.layout.state.isExpanded } });
        };
        _this.setSettings = function (p) {
            var _a;
            command_1.PluginCommands.Canvas3D.SetSettings.dispatch(_this.plugin, { settings: (_a = {}, _a[p.name] = p.value, _a) });
        };
        _this.setLayout = function (p) {
            var _a;
            command_1.PluginCommands.Layout.Update.dispatch(_this.plugin, { state: (_a = {}, _a[p.name] = p.value, _a) });
        };
        _this.setInteractivityProps = function (p) {
            var _a;
            command_1.PluginCommands.Interactivity.SetProps.dispatch(_this.plugin, { props: (_a = {}, _a[p.name] = p.value, _a) });
        };
        return _this;
    }
    ViewportControls.prototype.componentDidMount = function () {
        var _this = this;
        this.subscribe(this.plugin.events.canvas3d.settingsUpdated, function () { return _this.forceUpdate(); });
        this.subscribe(this.plugin.layout.events.updated, function () { return _this.forceUpdate(); });
        this.subscribe(this.plugin.events.interactivity.propsUpdated, function () { return _this.forceUpdate(); });
    };
    ViewportControls.prototype.icon = function (name, onClick, title, isOn) {
        if (isOn === void 0) { isOn = true; }
        return React.createElement(common_1.IconButton, { icon: name, toggleState: isOn, onClick: onClick, title: title });
    };
    ViewportControls.prototype.render = function () {
        return React.createElement("div", { className: 'msp-viewport-controls' },
            React.createElement("div", { className: 'msp-viewport-controls-buttons' },
                this.icon('reset-scene', this.resetCamera, 'Reset Camera'),
                React.createElement("br", null),
                this.icon('tools', this.toggleControls, 'Toggle Controls', this.plugin.layout.state.showControls),
                React.createElement("br", null),
                this.icon('expand-layout', this.toggleExpanded, 'Toggle Expanded', this.plugin.layout.state.isExpanded),
                React.createElement("br", null),
                this.icon('settings', this.toggleSettingsExpanded, 'Settings', this.state.isSettingsExpanded),
                React.createElement("br", null)),
            this.state.isSettingsExpanded && React.createElement("div", { className: 'msp-viewport-controls-scene-options' },
                React.createElement(common_1.ControlGroup, { header: 'Layout', initialExpanded: true },
                    React.createElement(parameters_1.ParameterControls, { params: layout_1.PluginLayoutStateParams, values: this.plugin.layout.state, onChange: this.setLayout })),
                React.createElement(common_1.ControlGroup, { header: 'Interactivity', initialExpanded: true },
                    React.createElement(parameters_1.ParameterControls, { params: interactivity_1.Interactivity.Params, values: this.plugin.interactivity.props, onChange: this.setInteractivityProps })),
                React.createElement(common_1.ControlGroup, { header: 'Viewport', initialExpanded: true },
                    React.createElement(parameters_1.ParameterControls, { params: canvas3d_1.Canvas3DParams, values: this.plugin.canvas3d.props, onChange: this.setSettings }))));
    };
    return ViewportControls;
}(base_1.PluginUIComponent));
exports.ViewportControls = ViewportControls;
var Viewport = /** @class */ (function (_super) {
    __extends(Viewport, _super);
    function Viewport() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.container = React.createRef();
        _this.canvas = React.createRef();
        _this.state = {
            noWebGl: false
        };
        _this.handleResize = function () {
            var container = _this.container.current;
            var canvas = _this.canvas.current;
            if (container && canvas) {
                util_1.resizeCanvas(canvas, container);
                _this.plugin.canvas3d.handleResize();
            }
        };
        return _this;
    }
    Viewport.prototype.componentDidMount = function () {
        var _this = this;
        if (!this.canvas.current || !this.container.current || !this.plugin.initViewer(this.canvas.current, this.container.current)) {
            this.setState({ noWebGl: true });
        }
        this.handleResize();
        var canvas3d = this.plugin.canvas3d;
        this.subscribe(canvas3d.input.resize, this.handleResize);
        this.subscribe(canvas3d.interaction.click, function (e) { return _this.plugin.behaviors.interaction.click.next(e); });
        this.subscribe(canvas3d.interaction.highlight, function (e) { return _this.plugin.behaviors.interaction.highlight.next(e); });
        this.subscribe(this.plugin.layout.events.updated, function () {
            setTimeout(_this.handleResize, 50);
        });
    };
    Viewport.prototype.componentWillUnmount = function () {
        if (_super.prototype.componentWillUnmount)
            _super.prototype.componentWillUnmount.call(this);
        // TODO viewer cleanup
    };
    Viewport.prototype.renderMissing = function () {
        return React.createElement("div", null,
            React.createElement("div", null,
                React.createElement("p", null,
                    React.createElement("b", null, "WebGL does not seem to be available.")),
                React.createElement("p", null, "This can be caused by an outdated browser, graphics card driver issue, or bad weather. Sometimes, just restarting the browser helps."),
                React.createElement("p", null,
                    "For a list of supported browsers, refer to ",
                    React.createElement("a", { href: 'http://caniuse.com/#feat=webgl', target: '_blank' }, "http://caniuse.com/#feat=webgl"),
                    ".")));
    };
    Viewport.prototype.render = function () {
        if (this.state.noWebGl)
            return this.renderMissing();
        return React.createElement("div", { className: 'msp-viewport' },
            React.createElement("div", { className: 'msp-viewport-host3d', ref: this.container },
                React.createElement("canvas", { ref: this.canvas })));
    };
    return Viewport;
}(base_1.PluginUIComponent));
exports.Viewport = Viewport;
