/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { Loci as ModelLoci } from '../../mol-model/loci';
import { ModifiersKeys, ButtonsType } from '../../mol-util/input/input-observer';
import { Representation } from '../../mol-repr/representation';
import { MarkerAction } from '../../mol-util/marker-action';
import { StructureElementSelectionManager } from './structure-element-selection';
import { PluginContext } from '../context';
import { ParamDefinition as PD } from '../../mol-util/param-definition';
export declare namespace Interaction {
    interface Loci<T extends ModelLoci = ModelLoci> {
        loci: T;
        repr?: Representation.Any;
    }
    namespace Loci {
        function areEqual(a: Loci, b: Loci): boolean;
        const Empty: Loci;
    }
    const Params: {
        lociExpansion: PD.Select<"element" | "residue" | "chain" | "structure">;
    };
    type Props = PD.Values<typeof Params>;
    interface HighlightEvent {
        current: Loci;
        modifiers?: ModifiersKeys;
    }
    interface ClickEvent {
        current: Loci;
        buttons: ButtonsType;
        modifiers: ModifiersKeys;
    }
    type LociMarkProvider = (loci: Loci, action: MarkerAction) => void;
    abstract class LociMarkManager<MarkEvent extends any> {
        readonly ctx: PluginContext;
        protected providers: LociMarkProvider[];
        protected sel: StructureElementSelectionManager;
        readonly props: Readonly<Props>;
        setProps(props: Partial<Props>): void;
        addProvider(provider: LociMarkProvider): void;
        removeProvider(provider: LociMarkProvider): void;
        expandLoci(loci: ModelLoci): ModelLoci;
        toggleSel(current: Loci<ModelLoci>): void;
        protected mark(current: Loci<ModelLoci>, action: MarkerAction): void;
        abstract apply(e: MarkEvent): void;
        constructor(ctx: PluginContext, props?: Partial<Props>);
    }
    class LociHighlightManager extends LociMarkManager<HighlightEvent> {
        private prev;
        apply(e: HighlightEvent): void;
        constructor(ctx: PluginContext, props?: Partial<Props>);
    }
    class LociSelectionManager extends LociMarkManager<ClickEvent> {
        toggleSel(current: Loci<ModelLoci>): void;
        apply(e: ClickEvent): void;
        constructor(ctx: PluginContext, props?: Partial<Props>);
    }
}
