"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var loci_1 = require("../../mol-model/loci");
var input_observer_1 = require("../../mol-util/input/input-observer");
var structure_1 = require("../../mol-model/structure");
var marker_action_1 = require("../../mol-util/marker-action");
var structure_2 = require("../../mol-model/structure");
var param_definition_1 = require("../../mol-util/param-definition");
var command_1 = require("../command");
var Interaction;
(function (Interaction) {
    var Loci;
    (function (Loci) {
        function areEqual(a, b) {
            return a.repr === b.repr && loci_1.Loci.areEqual(a.loci, b.loci);
        }
        Loci.areEqual = areEqual;
        Loci.Empty = { loci: loci_1.EmptyLoci };
    })(Loci = Interaction.Loci || (Interaction.Loci = {}));
    var LociExpansion = {
        'element': function (loci) { return loci; },
        'residue': function (loci) { return structure_2.StructureElement.isLoci(loci) ? structure_2.StructureElement.Loci.extendToWholeResidues(loci) : loci; },
        'chain': function (loci) { return structure_2.StructureElement.isLoci(loci) ? structure_2.StructureElement.Loci.extendToWholeChains(loci) : loci; },
        'structure': function (loci) { return structure_2.StructureElement.isLoci(loci) ? structure_2.Structure.Loci(loci.structure) : loci; }
    };
    var LociExpansionOptions = Object.keys(LociExpansion).map(function (n) { return [n, n]; });
    Interaction.Params = {
        lociExpansion: param_definition_1.ParamDefinition.Select('chain', LociExpansionOptions),
    };
    var LociMarkManager = /** @class */ (function () {
        function LociMarkManager(ctx, props) {
            var _this = this;
            if (props === void 0) { props = {}; }
            this.ctx = ctx;
            this.providers = [];
            this.sel = ctx.helpers.structureSelection;
            this.props = __assign({}, param_definition_1.ParamDefinition.getDefaultValues(Interaction.Params), props);
            command_1.PluginCommands.Interactivity.SetProps.subscribe(ctx, function (p) { return _this.setProps(p); });
        }
        LociMarkManager.prototype.setProps = function (props) {
            Object.assign(this.props, props);
        };
        LociMarkManager.prototype.addProvider = function (provider) {
            this.providers.push(provider);
        };
        LociMarkManager.prototype.removeProvider = function (provider) {
            this.providers = this.providers.filter(function (p) { return p !== provider; });
            // TODO clear, then re-apply remaining providers
        };
        LociMarkManager.prototype.expandLoci = function (loci) {
            return LociExpansion[this.props.lociExpansion](loci);
        };
        LociMarkManager.prototype.toggleSel = function (current) {
            if (this.sel.has(current.loci)) {
                this.sel.remove(current.loci);
                this.mark(current, marker_action_1.MarkerAction.Deselect);
            }
            else {
                this.sel.add(current.loci);
                this.mark(current, marker_action_1.MarkerAction.Select);
            }
        };
        LociMarkManager.prototype.mark = function (current, action) {
            for (var _i = 0, _a = this.providers; _i < _a.length; _i++) {
                var p = _a[_i];
                p(current, action);
            }
        };
        return LociMarkManager;
    }());
    Interaction.LociMarkManager = LociMarkManager;
    var LociHighlightManager = /** @class */ (function (_super) {
        __extends(LociHighlightManager, _super);
        function LociHighlightManager(ctx, props) {
            if (props === void 0) { props = {}; }
            var _this = _super.call(this, ctx, props) || this;
            _this.prev = { loci: loci_1.EmptyLoci, repr: void 0 };
            ctx.behaviors.interaction.highlight.subscribe(function (e) { return _this.apply(e); });
            return _this;
        }
        LociHighlightManager.prototype.apply = function (e) {
            var current = e.current, modifiers = e.modifiers;
            var expanded = { loci: this.expandLoci(current.loci), repr: current.repr };
            if (structure_1.StructureElement.isLoci(expanded)) {
                var loci = expanded;
                if (modifiers && modifiers.shift) {
                    loci = this.sel.tryGetRange(loci) || loci;
                }
                this.mark(this.prev, marker_action_1.MarkerAction.RemoveHighlight);
                var toHighlight = { loci: loci, repr: expanded.repr };
                this.mark(toHighlight, marker_action_1.MarkerAction.Highlight);
                this.prev = toHighlight;
            }
            else {
                if (!Loci.areEqual(this.prev, expanded)) {
                    this.mark(this.prev, marker_action_1.MarkerAction.RemoveHighlight);
                    this.mark(expanded, marker_action_1.MarkerAction.Highlight);
                    this.prev = expanded;
                }
            }
        };
        return LociHighlightManager;
    }(LociMarkManager));
    Interaction.LociHighlightManager = LociHighlightManager;
    var LociSelectionManager = /** @class */ (function (_super) {
        __extends(LociSelectionManager, _super);
        function LociSelectionManager(ctx, props) {
            if (props === void 0) { props = {}; }
            var _this = _super.call(this, ctx, props) || this;
            ctx.behaviors.interaction.click.subscribe(function (e) { return _this.apply(e); });
            return _this;
        }
        LociSelectionManager.prototype.toggleSel = function (current) {
            if (this.sel.has(current.loci)) {
                this.sel.remove(current.loci);
                this.mark(current, marker_action_1.MarkerAction.Deselect);
            }
            else {
                this.sel.add(current.loci);
                this.mark(current, marker_action_1.MarkerAction.Select);
            }
        };
        LociSelectionManager.prototype.apply = function (e) {
            var current = e.current, buttons = e.buttons, modifiers = e.modifiers;
            var expanded = { loci: this.expandLoci(current.loci), repr: current.repr };
            if (expanded.loci.kind === 'empty-loci') {
                if (modifiers.control && buttons === 2 /* Secondary */) {
                    // clear the selection on Ctrl + Right-Click on empty
                    var sels = this.sel.clear();
                    for (var _i = 0, sels_1 = sels; _i < sels_1.length; _i++) {
                        var s = sels_1[_i];
                        this.mark({ loci: s }, marker_action_1.MarkerAction.Deselect);
                    }
                }
            }
            else if (structure_1.StructureElement.isLoci(expanded.loci)) {
                if (modifiers.control && buttons === 2 /* Secondary */) {
                    // select only the current element on Ctrl + Right-Click
                    var old = this.sel.get(expanded.loci.structure);
                    this.mark({ loci: old }, marker_action_1.MarkerAction.Deselect);
                    this.sel.set(expanded.loci);
                    this.mark(expanded, marker_action_1.MarkerAction.Select);
                }
                else if (modifiers.control && buttons === 1 /* Primary */) {
                    // toggle current element on Ctrl + Left-Click
                    this.toggleSel(expanded);
                }
                else if (modifiers.shift && buttons === 1 /* Primary */) {
                    // try to extend sequence on Shift + Left-Click
                    var loci = expanded.loci;
                    if (modifiers && modifiers.shift) {
                        loci = this.sel.tryGetRange(loci) || loci;
                    }
                    this.toggleSel({ loci: loci, repr: expanded.repr });
                }
            }
            else {
                if (!input_observer_1.ButtonsType.has(buttons, 2 /* Secondary */))
                    return;
                for (var _a = 0, _b = this.providers; _a < _b.length; _a++) {
                    var p = _b[_a];
                    p(expanded, marker_action_1.MarkerAction.Toggle);
                }
            }
        };
        return LociSelectionManager;
    }(LociMarkManager));
    Interaction.LociSelectionManager = LociSelectionManager;
})(Interaction = exports.Interaction || (exports.Interaction = {}));
