"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var loci_1 = require("../../mol-model/loci");
var input_observer_1 = require("../../mol-util/input/input-observer");
var structure_1 = require("../../mol-model/structure");
var marker_action_1 = require("../../mol-util/marker-action");
var structure_2 = require("../../mol-model/structure");
var param_definition_1 = require("../../mol-util/param-definition");
var command_1 = require("../command");
var string_1 = require("../../mol-util/string");
var Interactivity = /** @class */ (function () {
    function Interactivity(ctx, props) {
        var _this = this;
        if (props === void 0) { props = {}; }
        this.ctx = ctx;
        this._props = param_definition_1.ParamDefinition.getDefaultValues(Interactivity.Params);
        Object.assign(this._props, props);
        this.lociSelections = new Interactivity.LociSelectionManager(ctx, this._props);
        this.lociHighlights = new Interactivity.LociHighlightManager(ctx, this._props);
        command_1.PluginCommands.Interactivity.SetProps.subscribe(ctx, function (e) { return _this.setProps(e.props); });
    }
    Object.defineProperty(Interactivity.prototype, "props", {
        get: function () { return __assign({}, this._props); },
        enumerable: true,
        configurable: true
    });
    Interactivity.prototype.setProps = function (props) {
        Object.assign(this._props, props);
        this.lociSelections.setProps(this._props);
        this.lociHighlights.setProps(this._props);
    };
    return Interactivity;
}());
exports.Interactivity = Interactivity;
(function (Interactivity) {
    var Loci;
    (function (Loci) {
        function areEqual(a, b) {
            return a.repr === b.repr && loci_1.Loci.areEqual(a.loci, b.loci);
        }
        Loci.areEqual = areEqual;
        Loci.Empty = { loci: loci_1.EmptyLoci };
    })(Loci = Interactivity.Loci || (Interactivity.Loci = {}));
    var Granularity = {
        'element': function (loci) { return loci; },
        'residue': function (loci) { return structure_2.StructureElement.isLoci(loci) ? structure_2.StructureElement.Loci.extendToWholeResidues(loci) : loci; },
        'chain': function (loci) { return structure_2.StructureElement.isLoci(loci) ? structure_2.StructureElement.Loci.extendToWholeChains(loci) : loci; },
        'structure': function (loci) { return structure_2.StructureElement.isLoci(loci) ? structure_2.Structure.Loci(loci.structure) : loci; }
    };
    var GranularityOptions = Object.keys(Granularity).map(function (n) { return [n, string_1.capitalize(n)]; });
    Interactivity.Params = {
        granularity: param_definition_1.ParamDefinition.Select('residue', GranularityOptions),
    };
    var LociMarkManager = /** @class */ (function () {
        function LociMarkManager(ctx, props) {
            if (props === void 0) { props = {}; }
            this.ctx = ctx;
            this.providers = [];
            this.props = param_definition_1.ParamDefinition.getDefaultValues(Interactivity.Params);
            this.sel = ctx.helpers.structureSelection;
            this.setProps(props);
        }
        LociMarkManager.prototype.setProps = function (props) {
            Object.assign(this.props, props);
        };
        LociMarkManager.prototype.addProvider = function (provider) {
            this.providers.push(provider);
        };
        LociMarkManager.prototype.removeProvider = function (provider) {
            this.providers = this.providers.filter(function (p) { return p !== provider; });
            // TODO clear, then re-apply remaining providers
        };
        LociMarkManager.prototype.normalizedLoci = function (interactivityLoci) {
            var loci = interactivityLoci.loci, repr = interactivityLoci.repr;
            if (this.props.granularity !== 'element' && structure_1.Link.isLoci(loci)) {
                // convert Link.Loci to a StructureElement.Loci so granularity can be applied
                loci = structure_1.Link.toStructureElementLoci(loci);
            }
            loci = Granularity[this.props.granularity](loci);
            if (structure_1.StructureElement.isLoci(loci) && loci.structure.parent) {
                loci = structure_1.StructureElement.Loci.remap(loci, loci.structure.parent);
            }
            return { loci: loci, repr: repr };
        };
        LociMarkManager.prototype.mark = function (current, action) {
            for (var _i = 0, _a = this.providers; _i < _a.length; _i++) {
                var p = _a[_i];
                p(current, action);
            }
        };
        return LociMarkManager;
    }());
    Interactivity.LociMarkManager = LociMarkManager;
    var LociHighlightManager = /** @class */ (function (_super) {
        __extends(LociHighlightManager, _super);
        function LociHighlightManager(ctx, props) {
            if (props === void 0) { props = {}; }
            var _this = _super.call(this, ctx, props) || this;
            _this.prev = { loci: loci_1.EmptyLoci, repr: void 0 };
            ctx.behaviors.interaction.highlight.subscribe(function (e) { return _this.apply(e); });
            return _this;
        }
        LociHighlightManager.prototype.apply = function (e) {
            var current = e.current, modifiers = e.modifiers;
            var normalized = this.normalizedLoci(current);
            if (structure_1.StructureElement.isLoci(normalized.loci)) {
                var loci = normalized.loci;
                if (modifiers && modifiers.shift) {
                    loci = this.sel.tryGetRange(loci) || loci;
                }
                this.mark(this.prev, marker_action_1.MarkerAction.RemoveHighlight);
                var toHighlight = { loci: loci, repr: normalized.repr };
                this.mark(toHighlight, marker_action_1.MarkerAction.Highlight);
                this.prev = toHighlight;
            }
            else {
                if (!Loci.areEqual(this.prev, normalized)) {
                    this.mark(this.prev, marker_action_1.MarkerAction.RemoveHighlight);
                    this.mark(normalized, marker_action_1.MarkerAction.Highlight);
                    this.prev = normalized;
                }
            }
        };
        return LociHighlightManager;
    }(LociMarkManager));
    Interactivity.LociHighlightManager = LociHighlightManager;
    var LociSelectionManager = /** @class */ (function (_super) {
        __extends(LociSelectionManager, _super);
        function LociSelectionManager(ctx, props) {
            if (props === void 0) { props = {}; }
            var _this = _super.call(this, ctx, props) || this;
            ctx.behaviors.interaction.click.subscribe(function (e) { return _this.apply(e); });
            return _this;
        }
        LociSelectionManager.prototype.toggleSel = function (current) {
            if (this.sel.has(current.loci)) {
                this.sel.remove(current.loci);
                this.mark(current, marker_action_1.MarkerAction.Deselect);
            }
            else {
                this.sel.add(current.loci);
                this.mark(current, marker_action_1.MarkerAction.Select);
            }
        };
        LociSelectionManager.prototype.apply = function (e) {
            var current = e.current, buttons = e.buttons, modifiers = e.modifiers;
            var normalized = this.normalizedLoci(current);
            if (normalized.loci.kind === 'empty-loci') {
                if (modifiers.control && buttons === 2 /* Secondary */) {
                    // clear the selection on Ctrl + Right-Click on empty
                    var sels = this.sel.clear();
                    for (var _i = 0, sels_1 = sels; _i < sels_1.length; _i++) {
                        var s = sels_1[_i];
                        this.mark({ loci: s }, marker_action_1.MarkerAction.Deselect);
                    }
                }
            }
            else if (structure_1.StructureElement.isLoci(normalized.loci)) {
                if (modifiers.control && buttons === 2 /* Secondary */) {
                    // select only the current element on Ctrl + Right-Click
                    var old = this.sel.get(normalized.loci.structure);
                    this.mark({ loci: old }, marker_action_1.MarkerAction.Deselect);
                    this.sel.set(normalized.loci);
                    this.mark(normalized, marker_action_1.MarkerAction.Select);
                }
                else if (modifiers.control && buttons === 1 /* Primary */) {
                    // toggle current element on Ctrl + Left-Click
                    this.toggleSel(normalized);
                }
                else if (modifiers.shift && buttons === 1 /* Primary */) {
                    // try to extend sequence on Shift + Left-Click
                    var loci = normalized.loci;
                    if (modifiers.shift) {
                        loci = this.sel.tryGetRange(loci) || loci;
                    }
                    this.toggleSel({ loci: loci, repr: normalized.repr });
                }
            }
            else {
                if (!input_observer_1.ButtonsType.has(buttons, 2 /* Secondary */))
                    return;
                for (var _a = 0, _b = this.providers; _a < _b.length; _a++) {
                    var p = _b[_a];
                    p(normalized, marker_action_1.MarkerAction.Toggle);
                }
            }
        };
        return LociSelectionManager;
    }(LociMarkManager));
    Interactivity.LociSelectionManager = LociSelectionManager;
})(Interactivity || (Interactivity = {}));
exports.Interactivity = Interactivity;
