"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../mol-model/structure");
var representation_1 = require("../../mol-repr/shape/representation");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var text_1 = require("../../mol-geo/geometry/text/text");
var text_builder_1 = require("../../mol-geo/geometry/text/text-builder");
var shape_1 = require("../../mol-model/shape");
var tables_1 = require("../../mol-util/color/tables");
var boundary_helper_1 = require("../../mol-math/geometry/boundary-helper");
function getLabelsText(data, props, text) {
    var texts = data.texts, positions = data.positions, depths = data.depths;
    var textBuilder = text_builder_1.TextBuilder.create(props, texts.length * 10, texts.length * 10 / 2, text);
    for (var i = 0, il = texts.length; i < il; ++i) {
        var p = positions[i];
        textBuilder.add(texts[i], p[0], p[1], p[2], depths[i], i);
    }
    return textBuilder.getText();
}
function getLabelRepresentation(ctx, structure, params, prev) {
    return __awaiter(this, void 0, void 0, function () {
        var repr, data;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    repr = prev || representation_1.ShapeRepresentation(getLabelsShape, text_1.Text.Utils);
                    data = getLabelData(structure, params);
                    return [4 /*yield*/, repr.createOrUpdate(params.options, data).runInContext(ctx)];
                case 1:
                    _a.sent();
                    repr.setState({ pickable: false });
                    return [2 /*return*/, repr];
            }
        });
    });
}
exports.getLabelRepresentation = getLabelRepresentation;
function getLabelsShape(ctx, data, props, shape) {
    var geo = getLabelsText(data, props, shape && shape.geometry);
    return shape_1.Shape.create('Scene Labels', data, geo, function () { return tables_1.ColorNames.dimgrey; }, function (g) { return data.sizes[g]; }, function () { return ''; });
}
var boundaryHelper = new boundary_helper_1.BoundaryHelper();
function getLabelData(structure, params) {
    if (params.target.name === 'static-text') {
        return getLabelDataStatic(structure, params.target.params.value, params.target.params.size || 1, params.target.params.position || 'middle-center');
    }
    else {
        return getLabelDataComputed(structure, params.target.name);
    }
}
function getLabelDataStatic(structure, text, size, position) {
    var boundary = structure.boundary.sphere;
    var oX = 0, oY = 0;
    if (position.indexOf('left') >= 0)
        oX = -boundary.radius;
    if (position.indexOf('right') >= 0)
        oX = boundary.radius;
    if (position.indexOf('top') >= 0)
        oY = boundary.radius;
    if (position.indexOf('bottom') >= 0)
        oY = -boundary.radius;
    return {
        texts: [text],
        positions: [linear_algebra_1.Vec3.add(linear_algebra_1.Vec3.zero(), boundary.center, linear_algebra_1.Vec3.create(oX, oY, 0))],
        sizes: [size],
        depths: [boundary.radius + Math.sqrt(oX * oX + oY * oY)]
    };
}
function getLabelDataComputed(structure, level) {
    var data = { texts: [], positions: [], sizes: [], depths: [] };
    var l = structure_1.StructureElement.create();
    var units = structure.units;
    var label_atom_id = structure_1.StructureProperties.atom.label_atom_id;
    var _a = structure_1.StructureProperties.residue, auth_seq_id = _a.auth_seq_id, label_comp_id = _a.label_comp_id;
    var auth_asym_id = structure_1.StructureProperties.chain.auth_asym_id;
    var p = linear_algebra_1.Vec3.zero();
    for (var _i = 0, units_1 = units; _i < units_1.length; _i++) {
        var unit = units_1[_i];
        // TODO: support coarse models
        if (unit.kind !== 0 /* Atomic */)
            continue;
        l.unit = unit;
        var elements = unit.elements;
        var pos = unit.conformation.position;
        if (level === 'elements') {
            for (var j = 0, _j = elements.length; j < _j; j++) {
                l.element = elements[j];
                pos(l.element, p);
                data.texts.push(label_atom_id(l));
                data.positions.push(linear_algebra_1.Vec3.clone(p));
                data.sizes.push(1);
                data.depths.push(2);
            }
        }
        else {
            var residueIndex = unit.model.atomicHierarchy.residueAtomSegments.index;
            var i = 0, len = elements.length;
            while (i < len) {
                var start = i, rI = residueIndex[elements[i]];
                i++;
                while (i < len && residueIndex[elements[i]] === rI)
                    i++;
                boundaryHelper.reset(0);
                for (var eI = start; eI < i; eI++) {
                    pos(elements[eI], p);
                    boundaryHelper.boundaryStep(p, 0);
                }
                boundaryHelper.finishBoundaryStep();
                for (var eI = start; eI < i; eI++) {
                    pos(elements[eI], p);
                    boundaryHelper.extendStep(p, 0);
                }
                l.element = elements[start];
                data.texts.push(label_comp_id(l) + " " + auth_seq_id(l) + ":" + auth_asym_id(l));
                data.positions.push(linear_algebra_1.Vec3.clone(boundaryHelper.center));
                data.sizes.push(Math.max(1, boundaryHelper.radius / 5));
                data.depths.push(boundaryHelper.radius);
            }
        }
    }
    return data;
}
