"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var mol_state_1 = require("../../mol-state");
var objects_1 = require("../state/objects");
var SubstructureParentHelper = /** @class */ (function () {
    function SubstructureParentHelper(plugin) {
        var _this = this;
        this.plugin = plugin;
        this.root = new Map();
        this.tracked = new Map();
        plugin.state.dataState.events.object.created.subscribe(function (e) {
            _this.addMapping(e.state, e.ref, e.obj);
        });
        plugin.state.dataState.events.object.removed.subscribe(function (e) {
            _this.removeMapping(e.ref);
        });
        plugin.state.dataState.events.object.updated.subscribe(function (e) {
            _this.updateMapping(e.state, e.ref, e.oldObj, e.obj);
        });
    }
    SubstructureParentHelper.prototype.get = function (s) {
        var r = this.root.get(s);
        if (!r)
            return;
        return this.plugin.state.dataState.cells.get(r);
    };
    SubstructureParentHelper.prototype.addMapping = function (state, ref, obj) {
        if (!objects_1.PluginStateObject.Molecule.Structure.is(obj))
            return;
        var parent = state.select(mol_state_1.StateSelection.Generators.byRef(ref).rootOfType([objects_1.PluginStateObject.Molecule.Structure]))[0];
        this.tracked.set(ref, obj.data);
        if (!parent) {
            this.root.set(obj.data, ref);
        }
        else {
            this.root.set(obj.data, parent.transform.ref);
        }
    };
    SubstructureParentHelper.prototype.removeMapping = function (ref) {
        if (!this.tracked.has(ref))
            return;
        var s = this.tracked.get(ref);
        this.tracked.delete(ref);
        this.root.get(s);
        this.root.delete(s);
    };
    SubstructureParentHelper.prototype.updateMapping = function (state, ref, oldObj, obj) {
        if (!objects_1.PluginStateObject.Molecule.Structure.is(obj))
            return;
        this.removeMapping(ref);
        this.addMapping(state, ref, obj);
    };
    return SubstructureParentHelper;
}());
exports.SubstructureParentHelper = SubstructureParentHelper;
