"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var geometry_1 = require("../../mol-geo/geometry/geometry");
var representation_1 = require("../representation");
var shape_1 = require("../../mol-model/shape");
var rxjs_1 = require("rxjs");
var render_object_1 = require("../../mol-gl/render-object");
var theme_1 = require("../../mol-theme/theme");
var location_iterator_1 = require("../../mol-geo/util/location-iterator");
var util_1 = require("../util");
var shape_group_1 = require("../../mol-theme/color/shape-group");
var shape_group_2 = require("../../mol-theme/size/shape-group");
var marker_data_1 = require("../../mol-geo/geometry/marker-data");
var mol_util_1 = require("../../mol-util");
var color_data_1 = require("../../mol-geo/geometry/color-data");
var size_data_1 = require("../../mol-geo/geometry/size-data");
var loci_1 = require("../../mol-model/loci");
var int_1 = require("../../mol-data/int");
var visual_1 = require("../visual");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var transform_data_1 = require("../../mol-geo/geometry/transform-data");
var mol_task_1 = require("../../mol-task");
var param_definition_1 = require("../../mol-util/param-definition");
function ShapeRepresentation(getShape, geometryUtils) {
    var version = 0;
    var updated = new rxjs_1.Subject();
    var _state = representation_1.Representation.createState();
    var materialId = render_object_1.getNextMaterialId();
    var renderObjects = [];
    var _renderObject;
    var _shape;
    var _theme = theme_1.createEmptyTheme();
    var currentProps = param_definition_1.ParamDefinition.getDefaultValues(geometryUtils.Params); // TODO avoid casting
    var currentParams;
    var locationIt;
    var updateState = util_1.VisualUpdateState.create();
    function prepareUpdate(props, shape) {
        if (props === void 0) { props = {}; }
        util_1.VisualUpdateState.reset(updateState);
        if (!shape && !_shape) {
            // console.error('no shape given')
            return;
        }
        else if (shape && !_shape) {
            // console.log('first shape')
            updateState.createNew = true;
        }
        else if (shape && _shape && shape.id === _shape.id) {
            // console.log('same shape')
            // nothing to set
        }
        else if (shape && _shape && shape.id !== _shape.id) {
            // console.log('new shape')
            updateState.updateTransform = true;
            updateState.createGeometry = true;
        }
        else if (!shape) {
            // console.log('only props')
            // nothing to set
        }
        else {
            console.warn('unexpected state');
        }
        if (updateState.updateTransform) {
            updateState.updateColor = true;
            updateState.updateSize = true;
        }
        if (updateState.createGeometry) {
            updateState.updateColor = true;
            updateState.updateSize = true;
        }
    }
    function createOrUpdate(props, data) {
        var _this = this;
        if (props === void 0) { props = {}; }
        return mol_task_1.Task.create('ShapeRepresentation.create', function (runtime) { return __awaiter(_this, void 0, void 0, function () {
            var newProps, shape, _a, transform, values, state, instanceCount, groupCount;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        newProps = Object.assign(currentProps, props);
                        if (!data) return [3 /*break*/, 2];
                        return [4 /*yield*/, getShape(runtime, data, newProps, _shape)];
                    case 1:
                        _a = _b.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        _a = undefined;
                        _b.label = 3;
                    case 3:
                        shape = _a;
                        prepareUpdate(props, shape);
                        if (shape) {
                            _shape = shape;
                            _theme.color = shape_group_1.ShapeGroupColorTheme({ shape: _shape }, {});
                            _theme.size = shape_group_2.ShapeGroupSizeTheme({ shape: _shape }, {});
                        }
                        if (updateState.createNew) {
                            renderObjects.length = 0; // clear list o renderObjects
                            locationIt = ShapeGroupIterator.fromShape(_shape);
                            transform = createShapeTransform(_shape.transforms);
                            values = geometryUtils.createValues(_shape.geometry, transform, locationIt, _theme, newProps);
                            state = geometryUtils.createRenderableState(newProps);
                            _renderObject = render_object_1.createRenderObject(_shape.geometry.kind, values, state, materialId);
                            if (_renderObject)
                                renderObjects.push(_renderObject); // add new renderObject to list
                        }
                        else {
                            if (!_renderObject) {
                                throw new Error('expected renderObject to be available');
                            }
                            if (updateState.updateTransform) {
                                // console.log('update transform')
                                createShapeTransform(_shape.transforms, _renderObject.values);
                                locationIt = ShapeGroupIterator.fromShape(_shape);
                                instanceCount = locationIt.instanceCount, groupCount = locationIt.groupCount;
                                marker_data_1.createMarkers(instanceCount * groupCount, _renderObject.values);
                            }
                            if (updateState.createGeometry) {
                                // console.log('update geometry')
                                mol_util_1.ValueCell.update(_renderObject.values.drawCount, geometry_1.Geometry.getDrawCount(_shape.geometry));
                            }
                            if (updateState.updateTransform || updateState.createGeometry) {
                                // console.log('updateBoundingSphere')
                                geometryUtils.updateBoundingSphere(_renderObject.values, _shape.geometry);
                            }
                            if (updateState.updateColor) {
                                // console.log('update color')
                                color_data_1.createColors(locationIt, _theme.color, _renderObject.values);
                            }
                            if (updateState.updateSize) {
                                // not all geometries have size data, so check here
                                if ('uSize' in _renderObject.values) {
                                    // console.log('update size')
                                    size_data_1.createSizes(locationIt, _theme.size, _renderObject.values);
                                }
                            }
                            geometryUtils.updateValues(_renderObject.values, newProps);
                            geometryUtils.updateRenderableState(_renderObject.state, newProps);
                        }
                        currentProps = newProps;
                        // increment version
                        updated.next(version++);
                        return [2 /*return*/];
                }
            });
        }); });
    }
    function lociApply(loci, apply) {
        if (loci_1.isEveryLoci(loci) || (shape_1.Shape.isLoci(loci) && loci.shape === _shape)) {
            return apply(int_1.Interval.ofBounds(0, _shape.groupCount * _shape.transforms.length));
        }
        else {
            return eachShapeGroup(loci, _shape, apply);
        }
    }
    return {
        label: 'Shape geometry',
        get groupCount() { return locationIt ? locationIt.count : 0; },
        get props() { return currentProps; },
        get params() { return currentParams; },
        get state() { return _state; },
        get theme() { return _theme; },
        renderObjects: renderObjects,
        updated: updated,
        createOrUpdate: createOrUpdate,
        getLoci: function (pickingId) {
            var objectId = pickingId.objectId, groupId = pickingId.groupId, instanceId = pickingId.instanceId;
            if (_renderObject && _renderObject.id === objectId) {
                return shape_1.ShapeGroup.Loci(_shape, [{ ids: int_1.OrderedSet.ofSingleton(groupId) }], instanceId);
            }
            return loci_1.EmptyLoci;
        },
        mark: function (loci, action) {
            return visual_1.Visual.mark(_renderObject, loci, action, lociApply);
        },
        setState: function (state) {
            if (_renderObject) {
                if (state.visible !== undefined)
                    visual_1.Visual.setVisibility(_renderObject, state.visible);
                if (state.alphaFactor !== undefined)
                    visual_1.Visual.setAlphaFactor(_renderObject, state.alphaFactor);
                if (state.pickable !== undefined)
                    visual_1.Visual.setPickable(_renderObject, state.pickable);
                if (state.overpaint !== undefined) {
                    visual_1.Visual.setOverpaint(_renderObject, state.overpaint, lociApply, true);
                }
                if (state.transparency !== undefined) {
                    visual_1.Visual.setTransparency(_renderObject, state.transparency, lociApply, true);
                }
                if (state.transform !== undefined)
                    visual_1.Visual.setTransform(_renderObject, state.transform);
            }
            representation_1.Representation.updateState(_state, state);
        },
        setTheme: function (theme) {
            console.warn('The `ShapeRepresentation` theme is fixed to `ShapeGroupColorTheme` and `ShapeGroupSizeTheme`. Colors are taken from `Shape.getColor` and sizes from `Shape.getSize`');
        },
        destroy: function () {
            // TODO
            renderObjects.length = 0;
            _renderObject = undefined;
        }
    };
}
exports.ShapeRepresentation = ShapeRepresentation;
function createShapeTransform(transforms, transformData) {
    var transformArray = transformData && transformData.aTransform.ref.value.length >= transforms.length * 16 ? transformData.aTransform.ref.value : new Float32Array(transforms.length * 16);
    for (var i = 0, il = transforms.length; i < il; ++i) {
        linear_algebra_1.Mat4.toArray(transforms[i], transformArray, i * 16);
    }
    return transform_data_1.createTransform(transformArray, transforms.length, transformData);
}
function eachShapeGroup(loci, shape, apply) {
    if (!shape_1.ShapeGroup.isLoci(loci))
        return false;
    if (loci.shape !== shape)
        return false;
    var changed = false;
    var groupCount = shape.groupCount;
    var instance = loci.instance, groups = loci.groups;
    for (var _a = 0, groups_1 = groups; _a < groups_1.length; _a++) {
        var g = groups_1[_a];
        if (int_1.Interval.is(g.ids)) {
            var start = instance * groupCount + int_1.Interval.start(g.ids);
            var end = instance * groupCount + int_1.Interval.end(g.ids);
            if (apply(int_1.Interval.ofBounds(start, end)))
                changed = true;
        }
        else {
            for (var i = 0, _i = g.ids.length; i < _i; i++) {
                var idx = instance * groupCount + g.ids[i];
                if (apply(int_1.Interval.ofSingleton(idx)))
                    changed = true;
            }
        }
    }
    return changed;
}
var ShapeGroupIterator;
(function (ShapeGroupIterator) {
    function fromShape(shape) {
        var instanceCount = shape.transforms.length;
        var location = shape_1.ShapeGroup.Location(shape);
        var getLocation = function (groupIndex, instanceIndex) {
            location.group = groupIndex;
            location.instance = instanceIndex;
            return location;
        };
        return location_iterator_1.LocationIterator(shape.groupCount, instanceCount, getLocation);
    }
    ShapeGroupIterator.fromShape = fromShape;
})(ShapeGroupIterator = exports.ShapeGroupIterator || (exports.ShapeGroupIterator = {}));
