/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { StructureParams } from './representation';
import { Visual, VisualContext } from '../visual';
import { Structure } from '../../mol-model/structure';
import { Geometry, GeometryUtils } from '../../mol-geo/geometry/geometry';
import { LocationIterator } from '../../mol-geo/util/location-iterator';
import { Theme } from '../../mol-theme/theme';
import { PickingId } from '../../mol-geo/geometry/picking';
import { Loci } from '../../mol-model/loci';
import { Interval } from '../../mol-data/int';
import { VisualUpdateState } from '../util';
import { UnitsParams } from './units-representation';
import { Mesh } from '../../mol-geo/geometry/mesh/mesh';
import { DirectVolume } from '../../mol-geo/geometry/direct-volume/direct-volume';
export interface ComplexVisual<P extends StructureParams> extends Visual<Structure, P> {
}
declare const ComplexParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
};
declare type ComplexParams = typeof ComplexParams;
interface ComplexVisualBuilder<P extends ComplexParams, G extends Geometry> {
    defaultProps: PD.Values<P>;
    createGeometry(ctx: VisualContext, structure: Structure, theme: Theme, props: PD.Values<P>, geometry?: G): Promise<G> | G;
    createLocationIterator(structure: Structure): LocationIterator;
    getLoci(pickingId: PickingId, structure: Structure, id: number): Loci;
    eachLocation(loci: Loci, structure: Structure, apply: (interval: Interval) => boolean): boolean;
    setUpdateState(state: VisualUpdateState, newProps: PD.Values<P>, currentProps: PD.Values<P>, newTheme: Theme, currentTheme: Theme): void;
}
interface ComplexVisualGeometryBuilder<P extends UnitsParams, G extends Geometry> extends ComplexVisualBuilder<P, G> {
    geometryUtils: GeometryUtils<G>;
}
export declare function ComplexVisual<G extends Geometry, P extends ComplexParams & Geometry.Params<G>>(builder: ComplexVisualGeometryBuilder<P, G>, materialId: number): ComplexVisual<P>;
export declare const ComplexMeshParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type ComplexMeshParams = typeof ComplexMeshParams;
export interface ComplexMeshVisualBuilder<P extends ComplexMeshParams> extends ComplexVisualBuilder<P, Mesh> {
}
export declare function ComplexMeshVisual<P extends ComplexMeshParams>(builder: ComplexMeshVisualBuilder<P>, materialId: number): ComplexVisual<P>;
export declare const ComplexDirectVolumeParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    isoValueNorm: PD.Numeric;
    renderMode: PD.Select<string>;
    controlPoints: PD.LineGraph;
    list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
};
export declare type ComplexDirectVolumeParams = typeof ComplexDirectVolumeParams;
export interface ComplexDirectVolumeVisualBuilder<P extends ComplexDirectVolumeParams> extends ComplexVisualBuilder<P, DirectVolume> {
}
export declare function ComplexDirectVolumeVisual<P extends ComplexDirectVolumeParams>(builder: ComplexDirectVolumeVisualBuilder<P>, materialId: number): ComplexVisual<P>;
export {};
