"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../mol-util/param-definition");
var representation_1 = require("./representation");
var visual_1 = require("../visual");
var structure_1 = require("../../mol-model/structure");
var geometry_1 = require("../../mol-geo/geometry/geometry");
var theme_1 = require("../../mol-theme/theme");
var transform_data_1 = require("../../mol-geo/geometry/transform-data");
var render_object_1 = require("../../mol-gl/render-object");
var common_1 = require("./visual/util/common");
var loci_1 = require("../../mol-model/loci");
var int_1 = require("../../mol-data/int");
var util_1 = require("../util");
var color_1 = require("../../mol-theme/color");
var mol_util_1 = require("../../mol-util");
var size_data_1 = require("../../mol-geo/geometry/size-data");
var color_data_1 = require("../../mol-geo/geometry/color-data");
var mesh_1 = require("../../mol-geo/geometry/mesh/mesh");
var size_1 = require("../../mol-theme/size");
var direct_volume_1 = require("../../mol-geo/geometry/direct-volume/direct-volume");
function createComplexRenderObject(structure, geometry, locationIt, theme, props, materialId) {
    var _a = geometry_1.Geometry.getUtils(geometry), createValues = _a.createValues, createRenderableState = _a.createRenderableState;
    var transform = transform_data_1.createIdentityTransform();
    var values = createValues(geometry, transform, locationIt, theme, props);
    var state = createRenderableState(props);
    return render_object_1.createRenderObject(geometry.kind, values, state, materialId);
}
var ComplexParams = __assign({}, representation_1.StructureParams, { unitKinds: param_definition_1.ParamDefinition.MultiSelect(['atomic', 'spheres'], common_1.UnitKindOptions) });
function ComplexVisual(builder, materialId) {
    var defaultProps = builder.defaultProps, createGeometry = builder.createGeometry, createLocationIterator = builder.createLocationIterator, getLoci = builder.getLoci, eachLocation = builder.eachLocation, setUpdateState = builder.setUpdateState;
    var _a = builder.geometryUtils, updateValues = _a.updateValues, updateBoundingSphere = _a.updateBoundingSphere, updateRenderableState = _a.updateRenderableState;
    var updateState = util_1.VisualUpdateState.create();
    var renderObject;
    var newProps;
    var newTheme;
    var newStructure;
    var currentProps = Object.assign({}, defaultProps);
    var currentTheme = theme_1.createEmptyTheme();
    var currentStructure;
    var geometry;
    var locationIt;
    function prepareUpdate(theme, props, structure) {
        if (!structure && !currentStructure) {
            throw new Error('missing structure');
        }
        newProps = Object.assign({}, currentProps, props);
        newTheme = theme;
        newStructure = structure;
        util_1.VisualUpdateState.reset(updateState);
        if (!renderObject) {
            updateState.createNew = true;
        }
        else if (!currentStructure || !structure_1.Structure.areEquivalent(newStructure, currentStructure)) {
            updateState.createNew = true;
        }
        if (updateState.createNew) {
            updateState.createGeometry = true;
            return;
        }
        setUpdateState(updateState, newProps, currentProps, newTheme, currentTheme);
        if (structure_1.Structure.conformationHash(newStructure) !== structure_1.Structure.conformationHash(currentStructure)) {
            updateState.createGeometry = true;
        }
        if (!color_1.ColorTheme.areEqual(theme.color, currentTheme.color)) {
            updateState.updateColor = true;
        }
        if (!mol_util_1.deepEqual(newProps.unitKinds, currentProps.unitKinds)) {
            updateState.createGeometry = true;
        }
        if (updateState.createGeometry) {
            updateState.updateColor = true;
        }
    }
    function update(newGeometry) {
        if (updateState.createNew) {
            locationIt = createLocationIterator(newStructure);
            if (newGeometry) {
                renderObject = createComplexRenderObject(newStructure, newGeometry, locationIt, newTheme, newProps, materialId);
            }
            else {
                throw new Error('expected geometry to be given');
            }
        }
        else {
            if (!renderObject) {
                throw new Error('expected renderObject to be available');
            }
            locationIt.reset();
            if (updateState.createGeometry) {
                if (newGeometry) {
                    mol_util_1.ValueCell.update(renderObject.values.drawCount, geometry_1.Geometry.getDrawCount(newGeometry));
                    updateBoundingSphere(renderObject.values, newGeometry);
                }
                else {
                    throw new Error('expected geometry to be given');
                }
            }
            if (updateState.updateSize) {
                // not all geometries have size data, so check here
                if ('uSize' in renderObject.values) {
                    size_data_1.createSizes(locationIt, newTheme.size, renderObject.values);
                }
            }
            if (updateState.updateColor) {
                color_data_1.createColors(locationIt, newTheme.color, renderObject.values);
            }
            updateValues(renderObject.values, newProps);
            updateRenderableState(renderObject.state, newProps);
        }
        currentProps = newProps;
        currentTheme = newTheme;
        currentStructure = newStructure;
        if (newGeometry)
            geometry = newGeometry;
    }
    function lociApply(loci, apply) {
        if (loci_1.isEveryLoci(loci) || (structure_1.Structure.isLoci(loci) && structure_1.Structure.areParentsEquivalent(loci.structure, currentStructure))) {
            return apply(int_1.Interval.ofBounds(0, locationIt.groupCount * locationIt.instanceCount));
        }
        else {
            return eachLocation(loci, currentStructure, apply);
        }
    }
    return {
        get groupCount() { return locationIt ? locationIt.count : 0; },
        get renderObject() { return locationIt && locationIt.count ? renderObject : undefined; },
        createOrUpdate: function (ctx, theme, props, structure) {
            if (props === void 0) { props = {}; }
            prepareUpdate(theme, props, structure || currentStructure);
            if (updateState.createGeometry) {
                var newGeometry = createGeometry(ctx, newStructure, newTheme, newProps, geometry);
                return newGeometry instanceof Promise ? newGeometry.then(update) : update(newGeometry);
            }
            else {
                update();
            }
        },
        getLoci: function (pickingId) {
            return renderObject ? getLoci(pickingId, currentStructure, renderObject.id) : loci_1.EmptyLoci;
        },
        mark: function (loci, action) {
            return visual_1.Visual.mark(renderObject, loci, action, lociApply);
        },
        setVisibility: function (visible) {
            visual_1.Visual.setVisibility(renderObject, visible);
        },
        setAlphaFactor: function (alphaFactor) {
            visual_1.Visual.setAlphaFactor(renderObject, alphaFactor);
        },
        setPickable: function (pickable) {
            visual_1.Visual.setPickable(renderObject, pickable);
        },
        setTransform: function (matrix, instanceMatrices) {
            visual_1.Visual.setTransform(renderObject, matrix, instanceMatrices);
        },
        setOverpaint: function (overpaint) {
            return visual_1.Visual.setOverpaint(renderObject, overpaint, lociApply, true);
        },
        setTransparency: function (transparency) {
            return visual_1.Visual.setTransparency(renderObject, transparency, lociApply, true);
        },
        destroy: function () {
            // TODO
            renderObject = undefined;
        }
    };
}
exports.ComplexVisual = ComplexVisual;
// mesh
exports.ComplexMeshParams = __assign({}, representation_1.StructureMeshParams, { unitKinds: param_definition_1.ParamDefinition.MultiSelect(['atomic', 'spheres'], common_1.UnitKindOptions) });
function ComplexMeshVisual(builder, materialId) {
    return ComplexVisual(__assign({}, builder, { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.createGeometry = true;
        }, geometryUtils: mesh_1.Mesh.Utils }), materialId);
}
exports.ComplexMeshVisual = ComplexMeshVisual;
// direct-volume
exports.ComplexDirectVolumeParams = __assign({}, representation_1.StructureDirectVolumeParams, { unitKinds: param_definition_1.ParamDefinition.MultiSelect(['atomic', 'spheres', 'gaussians'], common_1.UnitKindOptions) });
function ComplexDirectVolumeVisual(builder, materialId) {
    return ComplexVisual(__assign({}, builder, { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.createGeometry = true;
        }, geometryUtils: direct_volume_1.DirectVolume.Utils }), materialId);
}
exports.ComplexDirectVolumeVisual = ComplexDirectVolumeVisual;
