/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Structure } from '../../mol-model/structure';
import { RepresentationProvider, RepresentationRegistry } from '../representation';
import { StructureRepresentationState } from './representation';
export declare class StructureRepresentationRegistry extends RepresentationRegistry<Structure, StructureRepresentationState> {
    constructor();
}
export declare const BuiltInStructureRepresentations: {
    'cartoon': RepresentationProvider<Structure, {
        sizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        visuals: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"polymer-trace" | "polymer-gap" | "nucleotide-block" | "nucleotide-ring" | "direction-wedge", ("polymer-trace" | "polymer-gap" | "nucleotide-block" | "nucleotide-ring" | "direction-wedge")[]>;
        unitKinds: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
        alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useFog: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        highlightColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        selectColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        quality: import("../../mol-util/param-definition").ParamDefinition.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
        doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        radialSegments: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        detail: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        linearSegments: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        aspectRatio: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        arrowFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
    }, StructureRepresentationState>;
    'ball-and-stick': RepresentationProvider<Structure, {
        unitKinds: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
        sizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        sizeAspectRatio: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        visuals: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"element-sphere" | "intra-link" | "inter-link", ("element-sphere" | "intra-link" | "inter-link")[]>;
        linkScale: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        linkSpacing: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        radialSegments: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useFog: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        highlightColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        selectColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        quality: import("../../mol-util/param-definition").ParamDefinition.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
        doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        detail: import("../../mol-util/param-definition").ParamDefinition.Numeric;
    }, StructureRepresentationState>;
    'carbohydrate': RepresentationProvider<Structure, {
        visuals: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"carbohydrate-symbol" | "carbohydrate-link" | "carbohydrate-terminal-link", ("carbohydrate-symbol" | "carbohydrate-link" | "carbohydrate-terminal-link")[]>;
        linkSizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        linkScale: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        linkSpacing: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        radialSegments: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        unitKinds: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
        alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useFog: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        highlightColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        selectColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        quality: import("../../mol-util/param-definition").ParamDefinition.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
        doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        detail: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        sizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
    }, StructureRepresentationState>;
    'distance-restraint': RepresentationProvider<Structure, {
        unitKinds: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
        sizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        linkScale: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        linkSpacing: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        radialSegments: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useFog: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        highlightColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        selectColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        quality: import("../../mol-util/param-definition").ParamDefinition.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
        doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
    }, StructureRepresentationState>;
    'gaussian-surface': RepresentationProvider<Structure, {
        visuals: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"gaussian-surface-mesh" | "gaussian-surface-texture-mesh" | "gaussian-wireframe", ("gaussian-surface-mesh" | "gaussian-surface-texture-mesh" | "gaussian-wireframe")[]>;
        lineSizeAttenuation: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        resolution: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        radiusOffset: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        smoothness: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useGpu: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        unitKinds: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
        alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useFog: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        highlightColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        selectColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        quality: import("../../mol-util/param-definition").ParamDefinition.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
        sizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
    }, StructureRepresentationState>;
    'molecular-surface': RepresentationProvider<Structure, {
        visuals: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"molecular-surface-mesh", "molecular-surface-mesh"[]>;
        resolution: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        probeRadius: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        probePositions: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        unitKinds: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
        alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useFog: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        highlightColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        selectColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        quality: import("../../mol-util/param-definition").ParamDefinition.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
        doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
    }, StructureRepresentationState>;
    'point': RepresentationProvider<Structure, {
        unitKinds: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
        pointSizeAttenuation: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useFog: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        highlightColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        selectColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        quality: import("../../mol-util/param-definition").ParamDefinition.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
        sizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        pointFilledCircle: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        pointEdgeBleach: import("../../mol-util/param-definition").ParamDefinition.Numeric;
    }, StructureRepresentationState>;
    'putty': RepresentationProvider<Structure, {
        sizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        visuals: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"polymer-gap" | "polymer-tube", ("polymer-gap" | "polymer-tube")[]>;
        radialSegments: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        unitKinds: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
        alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useFog: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        highlightColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        selectColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        quality: import("../../mol-util/param-definition").ParamDefinition.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
        doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        linearSegments: import("../../mol-util/param-definition").ParamDefinition.Numeric;
    }, StructureRepresentationState>;
    'spacefill': RepresentationProvider<Structure, {
        unitKinds: import("../../mol-util/param-definition").ParamDefinition.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
        sizeFactor: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        detail: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        alpha: import("../../mol-util/param-definition").ParamDefinition.Numeric;
        useFog: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        highlightColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        selectColor: import("../../mol-util/param-definition").ParamDefinition.Color;
        quality: import("../../mol-util/param-definition").ParamDefinition.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
        doubleSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flipSided: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
        flatShaded: import("../../mol-util/param-definition").ParamDefinition.BooleanParam;
    }, StructureRepresentationState>;
};
export declare type BuiltInStructureRepresentationsName = keyof typeof BuiltInStructureRepresentations;
export declare const BuiltInStructureRepresentationsNames: string[];
export declare const BuiltInStructureRepresentationsOptions: ["carbohydrate" | "cartoon" | "ball-and-stick" | "distance-restraint" | "gaussian-surface" | "molecular-surface" | "point" | "putty" | "spacefill", string][];
