"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var representation_1 = require("../representation");
var cartoon_1 = require("./representation/cartoon");
var ball_and_stick_1 = require("./representation/ball-and-stick");
var gaussian_surface_1 = require("./representation/gaussian-surface");
var carbohydrate_1 = require("./representation/carbohydrate");
var spacefill_1 = require("./representation/spacefill");
var distance_restraint_1 = require("./representation/distance-restraint");
var point_1 = require("./representation/point");
var putty_1 = require("./representation/putty");
var molecular_surface_1 = require("./representation/molecular-surface");
var StructureRepresentationRegistry = /** @class */ (function (_super) {
    __extends(StructureRepresentationRegistry, _super);
    function StructureRepresentationRegistry() {
        var _this = _super.call(this) || this;
        Object.keys(exports.BuiltInStructureRepresentations).forEach(function (name) {
            var p = exports.BuiltInStructureRepresentations[name];
            _this.add(name, p);
        });
        return _this;
    }
    return StructureRepresentationRegistry;
}(representation_1.RepresentationRegistry));
exports.StructureRepresentationRegistry = StructureRepresentationRegistry;
exports.BuiltInStructureRepresentations = {
    'cartoon': cartoon_1.CartoonRepresentationProvider,
    'ball-and-stick': ball_and_stick_1.BallAndStickRepresentationProvider,
    'carbohydrate': carbohydrate_1.CarbohydrateRepresentationProvider,
    'distance-restraint': distance_restraint_1.DistanceRestraintRepresentationProvider,
    'gaussian-surface': gaussian_surface_1.GaussianSurfaceRepresentationProvider,
    // 'gaussian-volume': GaussianVolumeRepresentationProvider, // TODO disabled for now, needs more work
    'molecular-surface': molecular_surface_1.MolecularSurfaceRepresentationProvider,
    'point': point_1.PointRepresentationProvider,
    'putty': putty_1.PuttyRepresentationProvider,
    'spacefill': spacefill_1.SpacefillRepresentationProvider,
};
exports.BuiltInStructureRepresentationsNames = Object.keys(exports.BuiltInStructureRepresentations);
exports.BuiltInStructureRepresentationsOptions = exports.BuiltInStructureRepresentationsNames.map(function (n) { return [n, n]; });
