/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { Representation, RepresentationProps, RepresentationProvider } from '../representation';
import { StructureUnitTransforms } from '../../mol-model/structure/structure/util/unit-transforms';
import { Structure } from '../../mol-model/structure';
export interface StructureRepresentationState extends Representation.State {
    unitTransforms: StructureUnitTransforms | null;
}
export declare const StructureRepresentationStateBuilder: Representation.StateBuilder<StructureRepresentationState>;
export interface StructureRepresentation<P extends RepresentationProps = {}> extends Representation<Structure, P, StructureRepresentationState> {
}
export declare type StructureRepresentationProvider<P extends PD.Params> = RepresentationProvider<Structure, P, StructureRepresentationState>;
export declare const StructureParams: {
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
};
export declare type StructureParams = typeof StructureParams;
export declare const StructureMeshParams: {
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type StructureMeshParams = typeof StructureMeshParams;
export declare const StructureSpheresParams: {
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    sizeFactor: PD.Numeric;
    doubleSided: PD.BooleanParam;
};
export declare type StructureSpheresParams = typeof StructureSpheresParams;
export declare const StructurePointsParams: {
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    sizeFactor: PD.Numeric;
    pointSizeAttenuation: PD.BooleanParam;
    pointFilledCircle: PD.BooleanParam;
    pointEdgeBleach: PD.Numeric;
};
export declare type StructurePointsParams = typeof StructurePointsParams;
export declare const StructureLinesParams: {
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    sizeFactor: PD.Numeric;
    lineSizeAttenuation: PD.BooleanParam;
};
export declare type StructureLinesParams = typeof StructureLinesParams;
export declare const StructureDirectVolumeParams: {
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    isoValueNorm: PD.Numeric;
    renderMode: PD.Select<string>;
    controlPoints: PD.LineGraph;
    list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
};
export declare type StructureDirectVolumeParams = typeof StructureDirectVolumeParams;
export declare const StructureTextureMeshParams: {
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type StructureTextureMeshParams = typeof StructureTextureMeshParams;
export { ComplexRepresentation } from './complex-representation';
export { UnitsRepresentation } from './units-representation';
export { ComplexVisual } from './complex-visual';
export { UnitsVisual } from './units-visual';
