"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var element_sphere_1 = require("../visual/element-sphere");
var intra_unit_link_cylinder_1 = require("../visual/intra-unit-link-cylinder");
var inter_unit_link_cylinder_1 = require("../visual/inter-unit-link-cylinder");
var param_definition_1 = require("../../../mol-util/param-definition");
var units_representation_1 = require("../units-representation");
var complex_representation_1 = require("../complex-representation");
var representation_1 = require("../representation");
var representation_2 = require("../../../mol-repr/representation");
var common_1 = require("../visual/util/common");
var BallAndStickVisuals = {
    'element-sphere': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Element sphere mesh', ctx, getParams, element_sphere_1.getElementSphereVisual(ctx.webgl)); },
    'intra-link': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Intra-unit link cylinder', ctx, getParams, intra_unit_link_cylinder_1.IntraUnitLinkVisual); },
    'inter-link': function (ctx, getParams) { return complex_representation_1.ComplexRepresentation('Inter-unit link cylinder', ctx, getParams, inter_unit_link_cylinder_1.InterUnitLinkVisual); },
};
var BallAndStickVisualOptions = Object.keys(BallAndStickVisuals).map(function (name) { return [name, name]; });
exports.BallAndStickParams = __assign({}, element_sphere_1.ElementSphereParams, intra_unit_link_cylinder_1.IntraUnitLinkParams, inter_unit_link_cylinder_1.InterUnitLinkParams, { unitKinds: param_definition_1.ParamDefinition.MultiSelect(['atomic'], common_1.UnitKindOptions), sizeFactor: param_definition_1.ParamDefinition.Numeric(0.3, { min: 0.01, max: 10, step: 0.01 }), sizeAspectRatio: param_definition_1.ParamDefinition.Numeric(2 / 3, { min: 0.01, max: 3, step: 0.01 }), visuals: param_definition_1.ParamDefinition.MultiSelect(['element-sphere', 'intra-link', 'inter-link'], BallAndStickVisualOptions) });
function getBallAndStickParams(ctx, structure) {
    return param_definition_1.ParamDefinition.clone(exports.BallAndStickParams);
}
exports.getBallAndStickParams = getBallAndStickParams;
function BallAndStickRepresentation(ctx, getParams) {
    return representation_2.Representation.createMulti('Ball & Stick', ctx, getParams, representation_1.StructureRepresentationStateBuilder, BallAndStickVisuals);
}
exports.BallAndStickRepresentation = BallAndStickRepresentation;
exports.BallAndStickRepresentationProvider = {
    label: 'Ball & Stick',
    description: 'Displays atoms as spheres and bonds as cylinders.',
    factory: BallAndStickRepresentation,
    getParams: getBallAndStickParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.BallAndStickParams),
    defaultColorTheme: 'element-symbol',
    defaultSizeTheme: 'uniform'
};
