"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var representation_1 = require("../../../mol-repr/representation");
var param_definition_1 = require("../../../mol-util/param-definition");
var complex_representation_1 = require("../complex-representation");
var representation_2 = require("../representation");
var carbohydrate_link_cylinder_1 = require("../visual/carbohydrate-link-cylinder");
var carbohydrate_symbol_mesh_1 = require("../visual/carbohydrate-symbol-mesh");
var carbohydrate_terminal_link_cylinder_1 = require("../visual/carbohydrate-terminal-link-cylinder");
var CarbohydrateVisuals = {
    'carbohydrate-symbol': function (ctx, getParams) { return complex_representation_1.ComplexRepresentation('Carbohydrate symbol mesh', ctx, getParams, carbohydrate_symbol_mesh_1.CarbohydrateSymbolVisual); },
    'carbohydrate-link': function (ctx, getParams) { return complex_representation_1.ComplexRepresentation('Carbohydrate link cylinder', ctx, getParams, carbohydrate_link_cylinder_1.CarbohydrateLinkVisual); },
    'carbohydrate-terminal-link': function (ctx, getParams) { return complex_representation_1.ComplexRepresentation('Carbohydrate terminal link cylinder', ctx, getParams, carbohydrate_terminal_link_cylinder_1.CarbohydrateTerminalLinkVisual); },
};
var CarbohydrateVisualOptions = Object.keys(CarbohydrateVisuals).map(function (name) { return [name, name]; });
exports.CarbohydrateParams = __assign({}, carbohydrate_symbol_mesh_1.CarbohydrateSymbolParams, carbohydrate_link_cylinder_1.CarbohydrateLinkParams, carbohydrate_terminal_link_cylinder_1.CarbohydrateTerminalLinkParams, { visuals: param_definition_1.ParamDefinition.MultiSelect(['carbohydrate-symbol', 'carbohydrate-link', 'carbohydrate-terminal-link'], CarbohydrateVisualOptions) });
function getCarbohydrateParams(ctx, structure) {
    return param_definition_1.ParamDefinition.clone(exports.CarbohydrateParams);
}
exports.getCarbohydrateParams = getCarbohydrateParams;
function CarbohydrateRepresentation(ctx, getParams) {
    return representation_1.Representation.createMulti('Carbohydrate', ctx, getParams, representation_2.StructureRepresentationStateBuilder, CarbohydrateVisuals);
}
exports.CarbohydrateRepresentation = CarbohydrateRepresentation;
exports.CarbohydrateRepresentationProvider = {
    label: 'Carbohydrate',
    description: 'Displays carbohydrate symbols (3D SNFG).',
    factory: CarbohydrateRepresentation,
    getParams: getCarbohydrateParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.CarbohydrateParams),
    defaultColorTheme: 'carbohydrate-symbol',
    defaultSizeTheme: 'uniform'
};
