/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { StructureRepresentation, StructureRepresentationProvider } from '../representation';
import { RepresentationParamsGetter, RepresentationContext } from '../../../mol-repr/representation';
import { ThemeRegistryContext } from '../../../mol-theme/theme';
import { Structure } from '../../../mol-model/structure';
export declare const GaussianSurfaceParams: {
    visuals: PD.MultiSelect<"gaussian-surface-mesh" | "gaussian-surface-texture-mesh" | "gaussian-wireframe", ("gaussian-surface-mesh" | "gaussian-surface-texture-mesh" | "gaussian-wireframe")[]>;
    lineSizeAttenuation: PD.BooleanParam;
    resolution: PD.Numeric;
    radiusOffset: PD.Numeric;
    smoothness: PD.Numeric;
    useGpu: PD.BooleanParam;
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    sizeFactor: PD.Numeric;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type GaussianSurfaceParams = typeof GaussianSurfaceParams;
export declare function getGaussianSurfaceParams(ctx: ThemeRegistryContext, structure: Structure): {
    visuals: PD.MultiSelect<"gaussian-surface-mesh" | "gaussian-surface-texture-mesh" | "gaussian-wireframe", ("gaussian-surface-mesh" | "gaussian-surface-texture-mesh" | "gaussian-wireframe")[]>;
    lineSizeAttenuation: PD.BooleanParam;
    resolution: PD.Numeric;
    radiusOffset: PD.Numeric;
    smoothness: PD.Numeric;
    useGpu: PD.BooleanParam;
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    sizeFactor: PD.Numeric;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type GaussianSurfaceRepresentation = StructureRepresentation<GaussianSurfaceParams>;
export declare function GaussianSurfaceRepresentation(ctx: RepresentationContext, getParams: RepresentationParamsGetter<Structure, GaussianSurfaceParams>): GaussianSurfaceRepresentation;
export declare const GaussianSurfaceRepresentationProvider: StructureRepresentationProvider<GaussianSurfaceParams>;
