"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var gaussian_surface_mesh_1 = require("../visual/gaussian-surface-mesh");
var units_representation_1 = require("../units-representation");
var gaussian_surface_wireframe_1 = require("../visual/gaussian-surface-wireframe");
var param_definition_1 = require("../../../mol-util/param-definition");
var representation_1 = require("../representation");
var representation_2 = require("../../../mol-repr/representation");
var GaussianSurfaceVisuals = {
    'gaussian-surface-mesh': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Gaussian surface', ctx, getParams, gaussian_surface_mesh_1.GaussianSurfaceMeshVisual); },
    'gaussian-surface-texture-mesh': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Gaussian surface', ctx, getParams, gaussian_surface_mesh_1.GaussianSurfaceTextureMeshVisual); },
    'gaussian-wireframe': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Gaussian wireframe', ctx, getParams, gaussian_surface_wireframe_1.GaussianWireframeVisual); },
};
var GaussianSurfaceVisualOptions = Object.keys(GaussianSurfaceVisuals).map(function (name) { return [name, name]; });
exports.GaussianSurfaceParams = __assign({}, gaussian_surface_mesh_1.GaussianSurfaceMeshParams, gaussian_surface_wireframe_1.GaussianWireframeParams, { visuals: param_definition_1.ParamDefinition.MultiSelect(['gaussian-surface-mesh'], GaussianSurfaceVisualOptions) });
function getGaussianSurfaceParams(ctx, structure) {
    return param_definition_1.ParamDefinition.clone(exports.GaussianSurfaceParams);
}
exports.getGaussianSurfaceParams = getGaussianSurfaceParams;
function GaussianSurfaceRepresentation(ctx, getParams) {
    return representation_2.Representation.createMulti('Gaussian Surface', ctx, getParams, representation_1.StructureRepresentationStateBuilder, GaussianSurfaceVisuals);
}
exports.GaussianSurfaceRepresentation = GaussianSurfaceRepresentation;
exports.GaussianSurfaceRepresentationProvider = {
    label: 'Gaussian Surface',
    description: 'Displays a gaussian molecular surface.',
    factory: GaussianSurfaceRepresentation,
    getParams: getGaussianSurfaceParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.GaussianSurfaceParams),
    defaultColorTheme: 'polymer-id',
    defaultSizeTheme: 'uniform'
};
