"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var molecular_surface_mesh_1 = require("../visual/molecular-surface-mesh");
var units_representation_1 = require("../units-representation");
var param_definition_1 = require("../../../mol-util/param-definition");
var representation_1 = require("../representation");
var representation_2 = require("../../../mol-repr/representation");
var MolecularSurfaceVisuals = {
    'molecular-surface-mesh': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Molecular surface', ctx, getParams, molecular_surface_mesh_1.MolecularSurfaceMeshVisual); },
};
var MolecularSurfaceVisualOptions = Object.keys(MolecularSurfaceVisuals).map(function (name) { return [name, name]; });
exports.MolecularSurfaceParams = __assign({}, molecular_surface_mesh_1.MolecularSurfaceMeshParams, { visuals: param_definition_1.ParamDefinition.MultiSelect(['molecular-surface-mesh'], MolecularSurfaceVisualOptions) });
function getMolecularSurfaceParams(ctx, structure) {
    return param_definition_1.ParamDefinition.clone(exports.MolecularSurfaceParams);
}
exports.getMolecularSurfaceParams = getMolecularSurfaceParams;
function MolecularSurfaceRepresentation(ctx, getParams) {
    return representation_2.Representation.createMulti('Molecular Surface', ctx, getParams, representation_1.StructureRepresentationStateBuilder, MolecularSurfaceVisuals);
}
exports.MolecularSurfaceRepresentation = MolecularSurfaceRepresentation;
exports.MolecularSurfaceRepresentationProvider = {
    label: 'Molecular Surface',
    description: 'Displays a molecular surface.',
    factory: MolecularSurfaceRepresentation,
    getParams: getMolecularSurfaceParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.MolecularSurfaceParams),
    defaultColorTheme: 'polymer-id',
    defaultSizeTheme: 'uniform'
};
