"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var polymer_tube_mesh_1 = require("../visual/polymer-tube-mesh");
var polymer_gap_cylinder_1 = require("../visual/polymer-gap-cylinder");
var param_definition_1 = require("../../../mol-util/param-definition");
var units_representation_1 = require("../units-representation");
var representation_1 = require("../representation");
var representation_2 = require("../../../mol-repr/representation");
var structure_1 = require("../../../mol-model/structure");
var PuttyVisuals = {
    'polymer-tube': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Polymer tube mesh', ctx, getParams, polymer_tube_mesh_1.PolymerTubeVisual); },
    'polymer-gap': function (ctx, getParams) { return units_representation_1.UnitsRepresentation('Polymer gap cylinder', ctx, getParams, polymer_gap_cylinder_1.PolymerGapVisual); },
};
var PuttyVisualOptions = Object.keys(PuttyVisuals).map(function (name) { return [name, name]; });
exports.PuttyParams = __assign({}, polymer_tube_mesh_1.PolymerTubeParams, polymer_gap_cylinder_1.PolymerGapParams, { sizeFactor: param_definition_1.ParamDefinition.Numeric(0.2, { min: 0, max: 10, step: 0.01 }), visuals: param_definition_1.ParamDefinition.MultiSelect(['polymer-tube', 'polymer-gap'], PuttyVisualOptions) });
function getPuttyParams(ctx, structure) {
    var params = param_definition_1.ParamDefinition.clone(exports.PuttyParams);
    var hasNucleotides = false;
    var hasGaps = false;
    structure.units.forEach(function (u) {
        if (!hasNucleotides && structure_1.Unit.isAtomic(u) && u.nucleotideElements.length)
            hasNucleotides = true;
        if (!hasGaps && u.gapElements.length)
            hasGaps = true;
    });
    params.visuals.defaultValue = ['polymer-tube'];
    if (hasGaps)
        params.visuals.defaultValue.push('polymer-gap');
    return params;
}
exports.getPuttyParams = getPuttyParams;
function PuttyRepresentation(ctx, getParams) {
    return representation_2.Representation.createMulti('Putty', ctx, getParams, representation_1.StructureRepresentationStateBuilder, PuttyVisuals);
}
exports.PuttyRepresentation = PuttyRepresentation;
exports.PuttyRepresentationProvider = {
    label: 'Putty',
    description: 'Displays a tube smoothly following the trace atoms of polymers.',
    factory: PuttyRepresentation,
    getParams: getPuttyParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.PuttyParams),
    defaultColorTheme: 'polymer-id',
    defaultSizeTheme: 'uncertainty'
};
