/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { Structure, Unit } from '../../mol-model/structure';
import { RepresentationProps } from '../representation';
import { Visual, VisualContext } from '../visual';
import { Geometry, GeometryUtils } from '../../mol-geo/geometry/geometry';
import { LocationIterator } from '../../mol-geo/util/location-iterator';
import { Theme } from '../../mol-theme/theme';
import { UnitsParams } from './units-representation';
import { PickingId } from '../../mol-geo/geometry/picking';
import { Loci } from '../../mol-model/loci';
import { Interval } from '../../mol-data/int';
import { VisualUpdateState } from '../util';
import { Mesh } from '../../mol-geo/geometry/mesh/mesh';
import { Spheres } from '../../mol-geo/geometry/spheres/spheres';
import { Points } from '../../mol-geo/geometry/points/points';
import { Lines } from '../../mol-geo/geometry/lines/lines';
import { DirectVolume } from '../../mol-geo/geometry/direct-volume/direct-volume';
import { TextureMesh } from '../../mol-geo/geometry/texture-mesh/texture-mesh';
export declare type StructureGroup = {
    structure: Structure;
    group: Unit.SymmetryGroup;
};
export interface UnitsVisual<P extends RepresentationProps = {}> extends Visual<StructureGroup, P> {
}
interface UnitsVisualBuilder<P extends UnitsParams, G extends Geometry> {
    defaultProps: PD.Values<P>;
    createGeometry(ctx: VisualContext, unit: Unit, structure: Structure, theme: Theme, props: PD.Values<P>, geometry?: G): Promise<G> | G;
    createLocationIterator(group: Unit.SymmetryGroup): LocationIterator;
    getLoci(pickingId: PickingId, structureGroup: StructureGroup, id: number): Loci;
    eachLocation(loci: Loci, structureGroup: StructureGroup, apply: (interval: Interval) => boolean): boolean;
    setUpdateState(state: VisualUpdateState, newProps: PD.Values<P>, currentProps: PD.Values<P>, newTheme: Theme, currentTheme: Theme, newStructureGroup: StructureGroup, currentStructureGroup: StructureGroup): void;
}
interface UnitsVisualGeometryBuilder<P extends UnitsParams, G extends Geometry> extends UnitsVisualBuilder<P, G> {
    geometryUtils: GeometryUtils<G>;
}
export declare function UnitsVisual<G extends Geometry, P extends UnitsParams & Geometry.Params<G>>(builder: UnitsVisualGeometryBuilder<P, G>, materialId: number): UnitsVisual<P>;
export declare const UnitsMeshParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type UnitsMeshParams = typeof UnitsMeshParams;
export interface UnitsMeshVisualBuilder<P extends UnitsMeshParams> extends UnitsVisualBuilder<P, Mesh> {
}
export declare function UnitsMeshVisual<P extends UnitsMeshParams>(builder: UnitsMeshVisualBuilder<P>, materialId: number): UnitsVisual<P>;
export declare const UnitsSpheresParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    sizeFactor: PD.Numeric;
    doubleSided: PD.BooleanParam;
};
export declare type UnitsSpheresParams = typeof UnitsSpheresParams;
export interface UnitsSpheresVisualBuilder<P extends UnitsSpheresParams> extends UnitsVisualBuilder<P, Spheres> {
}
export declare function UnitsSpheresVisual<P extends UnitsSpheresParams>(builder: UnitsSpheresVisualBuilder<P>, materialId: number): UnitsVisual<P>;
export declare const UnitsPointsParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    sizeFactor: PD.Numeric;
    pointSizeAttenuation: PD.BooleanParam;
    pointFilledCircle: PD.BooleanParam;
    pointEdgeBleach: PD.Numeric;
};
export declare type UnitsPointsParams = typeof UnitsPointsParams;
export interface UnitsPointVisualBuilder<P extends UnitsPointsParams> extends UnitsVisualBuilder<P, Points> {
}
export declare function UnitsPointsVisual<P extends UnitsPointsParams>(builder: UnitsPointVisualBuilder<P>, materialId: number): UnitsVisual<P>;
export declare const UnitsLinesParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    sizeFactor: PD.Numeric;
    lineSizeAttenuation: PD.BooleanParam;
};
export declare type UnitsLinesParams = typeof UnitsLinesParams;
export interface UnitsLinesVisualBuilder<P extends UnitsLinesParams> extends UnitsVisualBuilder<P, Lines> {
}
export declare function UnitsLinesVisual<P extends UnitsLinesParams>(builder: UnitsLinesVisualBuilder<P>, materialId: number): UnitsVisual<P>;
export declare const UnitsDirectVolumeParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    isoValueNorm: PD.Numeric;
    renderMode: PD.Select<string>;
    controlPoints: PD.LineGraph;
    list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
};
export declare type UnitsDirectVolumeParams = typeof UnitsDirectVolumeParams;
export interface UnitsDirectVolumeVisualBuilder<P extends UnitsDirectVolumeParams> extends UnitsVisualBuilder<P, DirectVolume> {
}
export declare function UnitsDirectVolumeVisual<P extends UnitsDirectVolumeParams>(builder: UnitsDirectVolumeVisualBuilder<P>, materialId: number): UnitsVisual<P>;
export declare const UnitsTextureMeshParams: {
    unitKinds: PD.MultiSelect<"spheres" | "gaussians" | "atomic", ("spheres" | "gaussians" | "atomic")[]>;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type UnitsTextureMeshParams = typeof UnitsTextureMeshParams;
export interface UnitsTextureMeshVisualBuilder<P extends UnitsTextureMeshParams> extends UnitsVisualBuilder<P, TextureMesh> {
}
export declare function UnitsTextureMeshVisual<P extends UnitsTextureMeshParams>(builder: UnitsTextureMeshVisualBuilder<P>, materialId: number): UnitsVisual<P>;
export {};
