"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../mol-model/structure");
var visual_1 = require("../visual");
var geometry_1 = require("../../mol-geo/geometry/geometry");
var theme_1 = require("../../mol-theme/theme");
var common_1 = require("./visual/util/common");
var render_object_1 = require("../../mol-gl/render-object");
var units_representation_1 = require("./units-representation");
var loci_1 = require("../../mol-model/loci");
var int_1 = require("../../mol-data/int");
var util_1 = require("../util");
var color_1 = require("../../mol-theme/color");
var marker_data_1 = require("../../mol-geo/geometry/marker-data");
var mol_util_1 = require("../../mol-util");
var size_data_1 = require("../../mol-geo/geometry/size-data");
var color_data_1 = require("../../mol-geo/geometry/color-data");
var representation_1 = require("./representation");
var mesh_1 = require("../../mol-geo/geometry/mesh/mesh");
var size_1 = require("../../mol-theme/size");
var spheres_1 = require("../../mol-geo/geometry/spheres/spheres");
var points_1 = require("../../mol-geo/geometry/points/points");
var lines_1 = require("../../mol-geo/geometry/lines/lines");
var direct_volume_1 = require("../../mol-geo/geometry/direct-volume/direct-volume");
var texture_mesh_1 = require("../../mol-geo/geometry/texture-mesh/texture-mesh");
function createUnitsRenderObject(group, geometry, locationIt, theme, props, materialId) {
    var _a = geometry_1.Geometry.getUtils(geometry), createValues = _a.createValues, createRenderableState = _a.createRenderableState;
    var transform = common_1.createUnitsTransform(group);
    var values = createValues(geometry, transform, locationIt, theme, props);
    var state = createRenderableState(props);
    return render_object_1.createRenderObject(geometry.kind, values, state, materialId);
}
function UnitsVisual(builder, materialId) {
    var defaultProps = builder.defaultProps, createGeometry = builder.createGeometry, createLocationIterator = builder.createLocationIterator, getLoci = builder.getLoci, eachLocation = builder.eachLocation, setUpdateState = builder.setUpdateState;
    var _a = builder.geometryUtils, createEmptyGeometry = _a.createEmpty, updateValues = _a.updateValues, updateBoundingSphere = _a.updateBoundingSphere, updateRenderableState = _a.updateRenderableState;
    var updateState = util_1.VisualUpdateState.create();
    var renderObject;
    var newProps = Object.assign({}, defaultProps);
    var newTheme = theme_1.createEmptyTheme();
    var newStructureGroup;
    var currentProps;
    var currentTheme;
    var currentStructureGroup;
    var geometry;
    var locationIt;
    function prepareUpdate(theme, props, structureGroup) {
        if (props === void 0) { props = {}; }
        if (!structureGroup && !currentStructureGroup) {
            throw new Error('missing structureGroup');
        }
        newProps = Object.assign({}, currentProps, props);
        newTheme = theme;
        newStructureGroup = structureGroup;
        util_1.VisualUpdateState.reset(updateState);
        if (!renderObject) {
            updateState.createNew = true;
        }
        else if (!currentStructureGroup || !structure_1.Unit.SymmetryGroup.areInvariantElementsEqual(newStructureGroup.group, currentStructureGroup.group)) {
            updateState.createNew = true;
        }
        if (updateState.createNew) {
            updateState.createGeometry = true;
            return;
        }
        setUpdateState(updateState, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
        if (!color_1.ColorTheme.areEqual(newTheme.color, currentTheme.color)) {
            // console.log('new colorTheme')
            updateState.updateColor = true;
        }
        if (!mol_util_1.deepEqual(newProps.unitKinds, currentProps.unitKinds)) {
            // console.log('new unitKinds')
            updateState.createGeometry = true;
        }
        if (newStructureGroup.group.transformHash !== currentStructureGroup.group.transformHash) {
            // console.log('new transformHash')
            if (newStructureGroup.group.units.length !== currentStructureGroup.group.units.length || updateState.updateColor) {
                updateState.updateTransform = true;
            }
            else {
                updateState.updateMatrix = true;
            }
        }
        // check if the conformation of unit.model has changed
        var newUnit = newStructureGroup.group.units[0];
        var currentUnit = currentStructureGroup.group.units[0];
        // if (Unit.conformationId(newUnit) !== Unit.conformationId(currentUnit)) {
        if (structure_1.Unit.conformationId(newUnit) !== structure_1.Unit.conformationId(currentUnit)
            // TODO: this needs more attention
            || newUnit.conformation !== currentUnit.conformation) {
            // console.log('new conformation')
            updateState.updateTransform = true;
            updateState.createGeometry = true;
        }
        if (updateState.updateTransform) {
            updateState.updateColor = true;
            updateState.updateSize = true;
            updateState.updateMatrix = true;
        }
        if (updateState.createGeometry) {
            updateState.updateColor = true;
            updateState.updateSize = true;
        }
    }
    function update(newGeometry) {
        if (updateState.createNew) {
            locationIt = createLocationIterator(newStructureGroup.group);
            if (newGeometry) {
                renderObject = createUnitsRenderObject(newStructureGroup.group, newGeometry, locationIt, newTheme, newProps, materialId);
            }
            else {
                throw new Error('expected geometry to be given');
            }
        }
        else {
            if (!renderObject) {
                throw new Error('expected renderObject to be available');
            }
            if (updateState.updateTransform) {
                // console.log('update transform')
                locationIt = createLocationIterator(newStructureGroup.group);
                var instanceCount = locationIt.instanceCount, groupCount = locationIt.groupCount;
                marker_data_1.createMarkers(instanceCount * groupCount, renderObject.values);
            }
            if (updateState.updateMatrix) {
                // console.log('update matrix')
                common_1.createUnitsTransform(newStructureGroup.group, renderObject.values);
            }
            if (updateState.createGeometry) {
                // console.log('update geometry')
                if (newGeometry) {
                    mol_util_1.ValueCell.update(renderObject.values.drawCount, geometry_1.Geometry.getDrawCount(newGeometry));
                }
                else {
                    throw new Error('expected geometry to be given');
                }
            }
            if (updateState.updateTransform || updateState.createGeometry) {
                // console.log('UnitsVisual.updateBoundingSphere')
                updateBoundingSphere(renderObject.values, newGeometry || geometry);
            }
            if (updateState.updateSize) {
                // not all geometries have size data, so check here
                if ('uSize' in renderObject.values) {
                    // console.log('update size')
                    size_data_1.createSizes(locationIt, newTheme.size, renderObject.values);
                }
            }
            if (updateState.updateColor) {
                // console.log('update color')
                color_data_1.createColors(locationIt, newTheme.color, renderObject.values);
            }
            updateValues(renderObject.values, newProps);
            updateRenderableState(renderObject.state, newProps);
        }
        currentProps = newProps;
        currentTheme = newTheme;
        currentStructureGroup = newStructureGroup;
        if (newGeometry)
            geometry = newGeometry;
    }
    function _createGeometry(ctx, unit, structure, theme, props, geometry) {
        return common_1.includesUnitKind(props.unitKinds, unit)
            ? createGeometry(ctx, unit, structure, theme, props, geometry)
            : createEmptyGeometry(geometry);
    }
    function lociApply(loci, apply) {
        if (loci_1.isEveryLoci(loci) || (structure_1.Structure.isLoci(loci) && structure_1.Structure.areParentsEquivalent(loci.structure, currentStructureGroup.structure))) {
            return apply(int_1.Interval.ofBounds(0, locationIt.groupCount * locationIt.instanceCount));
        }
        else {
            return eachLocation(loci, currentStructureGroup, apply);
        }
    }
    return {
        get groupCount() { return locationIt ? locationIt.count : 0; },
        get renderObject() { return locationIt && locationIt.count ? renderObject : undefined; },
        createOrUpdate: function (ctx, theme, props, structureGroup) {
            if (props === void 0) { props = {}; }
            prepareUpdate(theme, props, structureGroup || currentStructureGroup);
            if (updateState.createGeometry) {
                var newGeometry = _createGeometry(ctx, newStructureGroup.group.units[0], newStructureGroup.structure, newTheme, newProps, geometry);
                return newGeometry instanceof Promise ? newGeometry.then(update) : update(newGeometry);
            }
            else {
                update();
            }
        },
        getLoci: function (pickingId) {
            return renderObject ? getLoci(pickingId, currentStructureGroup, renderObject.id) : loci_1.EmptyLoci;
        },
        mark: function (loci, action) {
            return visual_1.Visual.mark(renderObject, loci, action, lociApply);
        },
        setVisibility: function (visible) {
            visual_1.Visual.setVisibility(renderObject, visible);
        },
        setAlphaFactor: function (alphaFactor) {
            visual_1.Visual.setAlphaFactor(renderObject, alphaFactor);
        },
        setPickable: function (pickable) {
            visual_1.Visual.setPickable(renderObject, pickable);
        },
        setTransform: function (matrix, instanceMatrices) {
            visual_1.Visual.setTransform(renderObject, matrix, instanceMatrices);
        },
        setOverpaint: function (overpaint) {
            return visual_1.Visual.setOverpaint(renderObject, overpaint, lociApply, true);
        },
        setTransparency: function (transparency) {
            return visual_1.Visual.setTransparency(renderObject, transparency, lociApply, true);
        },
        destroy: function () {
            // TODO
            renderObject = undefined;
        }
    };
}
exports.UnitsVisual = UnitsVisual;
// mesh
exports.UnitsMeshParams = __assign({}, representation_1.StructureMeshParams, units_representation_1.UnitsParams);
function UnitsMeshVisual(builder, materialId) {
    return UnitsVisual(__assign({}, builder, { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.createGeometry = true;
        }, geometryUtils: mesh_1.Mesh.Utils }), materialId);
}
exports.UnitsMeshVisual = UnitsMeshVisual;
// spheres
exports.UnitsSpheresParams = __assign({}, representation_1.StructureSpheresParams, units_representation_1.UnitsParams);
function UnitsSpheresVisual(builder, materialId) {
    return UnitsVisual(__assign({}, builder, { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.updateSize = true;
        }, geometryUtils: spheres_1.Spheres.Utils }), materialId);
}
exports.UnitsSpheresVisual = UnitsSpheresVisual;
// points
exports.UnitsPointsParams = __assign({}, representation_1.StructurePointsParams, units_representation_1.UnitsParams);
function UnitsPointsVisual(builder, materialId) {
    return UnitsVisual(__assign({}, builder, { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.updateSize = true;
        }, geometryUtils: points_1.Points.Utils }), materialId);
}
exports.UnitsPointsVisual = UnitsPointsVisual;
// lines
exports.UnitsLinesParams = __assign({}, representation_1.StructureLinesParams, units_representation_1.UnitsParams);
function UnitsLinesVisual(builder, materialId) {
    return UnitsVisual(__assign({}, builder, { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.updateSize = true;
        }, geometryUtils: lines_1.Lines.Utils }), materialId);
}
exports.UnitsLinesVisual = UnitsLinesVisual;
// direct-volume
exports.UnitsDirectVolumeParams = __assign({}, representation_1.StructureDirectVolumeParams, units_representation_1.UnitsParams);
function UnitsDirectVolumeVisual(builder, materialId) {
    return UnitsVisual(__assign({}, builder, { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.createGeometry = true;
        }, geometryUtils: direct_volume_1.DirectVolume.Utils }), materialId);
}
exports.UnitsDirectVolumeVisual = UnitsDirectVolumeVisual;
// texture-mesh
exports.UnitsTextureMeshParams = __assign({}, representation_1.StructureTextureMeshParams, units_representation_1.UnitsParams);
function UnitsTextureMeshVisual(builder, materialId) {
    return UnitsVisual(__assign({}, builder, { setUpdateState: function (state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup) {
            builder.setUpdateState(state, newProps, currentProps, newTheme, currentTheme, newStructureGroup, currentStructureGroup);
            if (!size_1.SizeTheme.areEqual(newTheme.size, currentTheme.size))
                state.createGeometry = true;
        }, geometryUtils: texture_mesh_1.TextureMesh.Utils }), materialId);
}
exports.UnitsTextureMeshVisual = UnitsTextureMeshVisual;
