"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var box_1 = require("../../../mol-geo/primitive/box");
var pyramid_1 = require("../../../mol-geo/primitive/pyramid");
var star_1 = require("../../../mol-geo/primitive/star");
var octahedron_1 = require("../../../mol-geo/primitive/octahedron");
var prism_1 = require("../../../mol-geo/primitive/prism");
var structure_1 = require("../../../mol-model/structure");
var mesh_builder_1 = require("../../../mol-geo/geometry/mesh/mesh-builder");
var constants_1 = require("../../../mol-model/structure/structure/carbohydrates/constants");
var sphere_1 = require("../../../mol-geo/geometry/mesh/builder/sphere");
var complex_visual_1 = require("../complex-visual");
var param_definition_1 = require("../../../mol-util/param-definition");
var location_iterator_1 = require("../../../mol-geo/util/location-iterator");
var int_1 = require("../../../mol-data/int");
var loci_1 = require("../../../mol-model/loci");
var common_1 = require("./util/common");
var t = linear_algebra_1.Mat4.identity();
var sVec = linear_algebra_1.Vec3.zero();
var pd = linear_algebra_1.Vec3.zero();
var SideFactor = 2 * 0.806; // 0.806 == Math.cos(Math.PI / 4)
var box = box_1.Box();
var perforatedBox = box_1.PerforatedBox();
var octagonalPyramid = pyramid_1.OctagonalPyramid();
var perforatedOctagonalPyramid = pyramid_1.PerforatedOctagonalPyramid();
var star = star_1.Star({ outerRadius: 1, innerRadius: 0.5, thickness: 0.5, pointCount: 5 });
var octahedron = octahedron_1.Octahedron();
var perforatedOctahedron = octahedron_1.PerforatedOctahedron();
var diamondPrism = prism_1.DiamondPrism();
var pentagonalPrism = prism_1.PentagonalPrism();
var hexagonalPrism = prism_1.HexagonalPrism();
function createCarbohydrateSymbolMesh(ctx, structure, theme, props, mesh) {
    var builderState = mesh_builder_1.MeshBuilder.createState(256, 128, mesh);
    var detail = props.detail, sizeFactor = props.sizeFactor;
    var carbohydrates = structure.carbohydrates;
    var n = carbohydrates.elements.length;
    var l = structure_1.StructureElement.create();
    for (var i = 0; i < n; ++i) {
        var c = carbohydrates.elements[i];
        var shapeType = constants_1.getSaccharideShape(c.component.type);
        l.unit = c.unit;
        l.element = c.unit.elements[c.anomericCarbon];
        var size = theme.size.size(l);
        var radius = size * sizeFactor;
        var side = size * sizeFactor * SideFactor;
        var _a = c.geometry, center = _a.center, normal = _a.normal, direction = _a.direction;
        linear_algebra_1.Vec3.add(pd, center, direction);
        linear_algebra_1.Mat4.targetTo(t, center, pd, normal);
        linear_algebra_1.Mat4.setTranslation(t, center);
        builderState.currentGroup = i * 2;
        switch (shapeType) {
            case 0 /* FilledSphere */:
                sphere_1.addSphere(builderState, center, radius, detail);
                break;
            case 1 /* FilledCube */:
                linear_algebra_1.Mat4.scaleUniformly(t, t, side);
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, box);
                break;
            case 2 /* CrossedCube */:
                linear_algebra_1.Mat4.scaleUniformly(t, t, side);
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, perforatedBox);
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotZ90X180);
                builderState.currentGroup += 1;
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, perforatedBox);
                break;
            case 4 /* FilledCone */:
                linear_algebra_1.Mat4.scaleUniformly(t, t, side * 1.2);
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, octagonalPyramid);
                break;
            case 5 /* DevidedCone */:
                linear_algebra_1.Mat4.scaleUniformly(t, t, side * 1.2);
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, perforatedOctagonalPyramid);
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotZ90);
                builderState.currentGroup += 1;
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, perforatedOctagonalPyramid);
                break;
            case 6 /* FlatBox */:
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotZY90);
                linear_algebra_1.Mat4.scale(t, t, linear_algebra_1.Vec3.set(sVec, side, side, side / 2));
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, box);
                break;
            case 7 /* FilledStar */:
                linear_algebra_1.Mat4.scaleUniformly(t, t, side);
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotZY90);
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, star);
                break;
            case 8 /* FilledDiamond */:
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotZY90);
                linear_algebra_1.Mat4.scale(t, t, linear_algebra_1.Vec3.set(sVec, side * 1.4, side * 1.4, side * 1.4));
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, octahedron);
                break;
            case 3 /* DividedDiamond */:
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotZY90);
                linear_algebra_1.Mat4.scale(t, t, linear_algebra_1.Vec3.set(sVec, side * 1.4, side * 1.4, side * 1.4));
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, perforatedOctahedron);
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotY90);
                builderState.currentGroup += 1;
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, perforatedOctahedron);
                break;
            case 9 /* FlatDiamond */:
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotZY90);
                linear_algebra_1.Mat4.scale(t, t, linear_algebra_1.Vec3.set(sVec, side, side / 2, side / 2));
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, diamondPrism);
                break;
            case 11 /* Pentagon */:
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotZY90);
                linear_algebra_1.Mat4.scale(t, t, linear_algebra_1.Vec3.set(sVec, side, side, side / 2));
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, pentagonalPrism);
                break;
            case 10 /* FlatHexagon */:
            default:
                linear_algebra_1.Mat4.mul(t, t, linear_algebra_1.Mat4.rotZYZ90);
                linear_algebra_1.Mat4.scale(t, t, linear_algebra_1.Vec3.set(sVec, side / 1.5, side, side / 2));
                mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, hexagonalPrism);
                break;
        }
    }
    return mesh_builder_1.MeshBuilder.getMesh(builderState);
}
exports.CarbohydrateSymbolParams = __assign({}, complex_visual_1.ComplexMeshParams, { detail: param_definition_1.ParamDefinition.Numeric(0, { min: 0, max: 3, step: 1 }), sizeFactor: param_definition_1.ParamDefinition.Numeric(1.75, { min: 0, max: 10, step: 0.01 }) });
function CarbohydrateSymbolVisual(materialId) {
    return complex_visual_1.ComplexMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.CarbohydrateSymbolParams),
        createGeometry: createCarbohydrateSymbolMesh,
        createLocationIterator: CarbohydrateElementIterator,
        getLoci: getCarbohydrateLoci,
        eachLocation: eachCarbohydrate,
        setUpdateState: function (state, newProps, currentProps) {
            state.createGeometry = (newProps.sizeFactor !== currentProps.sizeFactor ||
                newProps.detail !== currentProps.detail);
        }
    }, materialId);
}
exports.CarbohydrateSymbolVisual = CarbohydrateSymbolVisual;
function CarbohydrateElementIterator(structure) {
    var carbElements = structure.carbohydrates.elements;
    var groupCount = carbElements.length * 2;
    var instanceCount = 1;
    var location = structure_1.StructureElement.create();
    function getLocation(groupIndex, instanceIndex) {
        var carb = carbElements[Math.floor(groupIndex / 2)];
        location.unit = carb.unit;
        location.element = carb.anomericCarbon;
        return location;
    }
    function isSecondary(elementIndex, instanceIndex) {
        return (elementIndex % 2) === 1;
    }
    return location_iterator_1.LocationIterator(groupCount, instanceCount, getLocation, true, isSecondary);
}
/** Return a Loci for the elements of the whole residue of a carbohydrate. */
function getCarbohydrateLoci(pickingId, structure, id) {
    var objectId = pickingId.objectId, groupId = pickingId.groupId;
    if (id === objectId) {
        var carb = structure.carbohydrates.elements[Math.floor(groupId / 2)];
        return common_1.getAltResidueLoci(structure, carb.unit, carb.anomericCarbon);
    }
    return loci_1.EmptyLoci;
}
/** For each carbohydrate (usually a monosaccharide) when all its residue's elements are in a loci. */
function eachCarbohydrate(loci, structure, apply) {
    var _a = structure.carbohydrates, getElementIndex = _a.getElementIndex, getAnomericCarbons = _a.getAnomericCarbons;
    var changed = false;
    if (!structure_1.StructureElement.isLoci(loci))
        return false;
    if (!structure_1.Structure.areParentsEquivalent(loci.structure, structure))
        return false;
    var _loop_1 = function (e) {
        // TODO make more efficient by handling/grouping `e.indices` by residue index
        // TODO only call apply when the full alt-residue of the unit is part of `e`
        int_1.OrderedSet.forEach(e.indices, function (v) {
            var _a = e.unit, model = _a.model, elements = _a.elements;
            var index = model.atomicHierarchy.residueAtomSegments.index;
            var rI = index[elements[v]];
            var eIndices = getAnomericCarbons(e.unit, rI);
            for (var i = 0, il = eIndices.length; i < il; ++i) {
                var eI = eIndices[i];
                if (!int_1.OrderedSet.has(e.indices, int_1.OrderedSet.indexOf(elements, eI)))
                    continue;
                var idx = getElementIndex(e.unit, eI);
                if (idx !== undefined) {
                    if (apply(int_1.Interval.ofBounds(idx * 2, idx * 2 + 2)))
                        changed = true;
                }
            }
        });
    };
    for (var _i = 0, _b = loci.elements; _i < _b.length; _i++) {
        var e = _b[_i];
        _loop_1(e);
    }
    return changed;
}
