"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var structure_1 = require("../../../mol-model/structure");
var mesh_1 = require("../../../mol-geo/geometry/mesh/mesh");
var mol_util_1 = require("../../../mol-util");
var link_1 = require("./util/link");
var complex_visual_1 = require("../complex-visual");
var location_iterator_1 = require("../../../mol-geo/util/location-iterator");
var loci_1 = require("../../../mol-model/loci");
var int_1 = require("../../../mol-data/int");
function createCrossLinkRestraintCylinderMesh(ctx, structure, theme, props, mesh) {
    var crossLinks = structure.crossLinkRestraints;
    if (!crossLinks.count)
        return mesh_1.Mesh.createEmpty(mesh);
    var sizeFactor = props.sizeFactor;
    var location = structure_1.StructureElement.create();
    var builderProps = {
        linkCount: crossLinks.count,
        referencePosition: function () { return null; },
        position: function (posA, posB, edgeIndex) {
            var b = crossLinks.pairs[edgeIndex];
            var uA = b.unitA, uB = b.unitB;
            uA.conformation.position(uA.elements[b.indexA], posA);
            uB.conformation.position(uB.elements[b.indexB], posB);
        },
        order: function () { return 1; },
        flags: function () { return mol_util_1.BitFlags.create(0 /* None */); },
        radius: function (edgeIndex) {
            var b = crossLinks.pairs[edgeIndex];
            location.unit = b.unitA;
            location.element = b.unitA.elements[b.indexA];
            return theme.size.size(location) * sizeFactor;
        }
    };
    return link_1.createLinkCylinderMesh(ctx, builderProps, props, mesh);
}
exports.CrossLinkRestraintParams = __assign({}, complex_visual_1.ComplexMeshParams, link_1.LinkCylinderParams, { sizeFactor: param_definition_1.ParamDefinition.Numeric(1, { min: 0, max: 10, step: 0.1 }) });
function CrossLinkRestraintVisual(materialId) {
    return complex_visual_1.ComplexMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.CrossLinkRestraintParams),
        createGeometry: createCrossLinkRestraintCylinderMesh,
        createLocationIterator: CrossLinkRestraintIterator,
        getLoci: getLinkLoci,
        eachLocation: eachCrossLink,
        setUpdateState: function (state, newProps, currentProps) {
            state.createGeometry = (newProps.sizeFactor !== currentProps.sizeFactor ||
                newProps.radialSegments !== currentProps.radialSegments);
        }
    }, materialId);
}
exports.CrossLinkRestraintVisual = CrossLinkRestraintVisual;
function CrossLinkRestraintIterator(structure) {
    var pairs = structure.crossLinkRestraints.pairs;
    var groupCount = pairs.length;
    var instanceCount = 1;
    var location = structure_1.Link.Location();
    var getLocation = function (groupIndex) {
        var pair = pairs[groupIndex];
        location.aUnit = pair.unitA;
        location.aIndex = pair.indexA;
        location.bUnit = pair.unitB;
        location.bIndex = pair.indexB;
        return location;
    };
    return location_iterator_1.LocationIterator(groupCount, instanceCount, getLocation, true);
}
function getLinkLoci(pickingId, structure, id) {
    var objectId = pickingId.objectId, groupId = pickingId.groupId;
    if (id === objectId) {
        var pair = structure.crossLinkRestraints.pairs[groupId];
        if (pair) {
            return structure_1.Link.Loci(structure, [
                structure_1.Link.Location(pair.unitA, pair.indexA, pair.unitB, pair.indexB),
                structure_1.Link.Location(pair.unitB, pair.indexB, pair.unitA, pair.indexA)
            ]);
        }
    }
    return loci_1.EmptyLoci;
}
function eachCrossLink(loci, structure, apply) {
    var crossLinks = structure.crossLinkRestraints;
    var changed = false;
    if (structure_1.Link.isLoci(loci)) {
        if (!structure_1.Structure.areParentsEquivalent(loci.structure, structure))
            return false;
        for (var _i = 0, _a = loci.links; _i < _a.length; _i++) {
            var b = _a[_i];
            var indices = crossLinks.getPairIndices(b.aIndex, b.aUnit, b.bIndex, b.bUnit);
            if (indices) {
                for (var i = 0, il = indices.length; i < il; ++i) {
                    if (apply(int_1.Interval.ofSingleton(indices[i])))
                        changed = true;
                }
            }
        }
    }
    return changed;
}
