"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var units_visual_1 = require("../units-visual");
var gaussian_1 = require("./util/gaussian");
var mesh_1 = require("../../../mol-geo/geometry/mesh/mesh");
var algorithm_1 = require("../../../mol-geo/util/marching-cubes/algorithm");
var element_1 = require("./util/element");
var texture_mesh_1 = require("../../../mol-geo/geometry/texture-mesh/texture-mesh");
var active_voxels_1 = require("../../../mol-gl/compute/marching-cubes/active-voxels");
var reduction_1 = require("../../../mol-gl/compute/histogram-pyramid/reduction");
var isosurface_1 = require("../../../mol-gl/compute/marching-cubes/isosurface");
var geometry_1 = require("../../../mol-math/geometry");
exports.GaussianSurfaceMeshParams = __assign({}, units_visual_1.UnitsMeshParams, units_visual_1.UnitsTextureMeshParams, gaussian_1.GaussianDensityParams);
function getGaussianSurfaceVisual(webgl) {
    return webgl && webgl.extensions.drawBuffers ? GaussianSurfaceTextureMeshVisual : GaussianSurfaceMeshVisual;
}
exports.getGaussianSurfaceVisual = getGaussianSurfaceVisual;
//
function createGaussianSurfaceMesh(ctx, unit, structure, theme, props, mesh) {
    return __awaiter(this, void 0, void 0, function () {
        var smoothness, _a, transform, field, idField, params, surface;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    smoothness = props.smoothness;
                    return [4 /*yield*/, gaussian_1.computeUnitGaussianDensity(unit, props, ctx.webgl).runInContext(ctx.runtime)];
                case 1:
                    _a = _b.sent(), transform = _a.transform, field = _a.field, idField = _a.idField;
                    params = {
                        isoLevel: Math.exp(-smoothness),
                        scalarField: field,
                        idField: idField
                    };
                    return [4 /*yield*/, algorithm_1.computeMarchingCubesMesh(params, mesh).runAsChild(ctx.runtime)];
                case 2:
                    surface = _b.sent();
                    mesh_1.Mesh.transformImmediate(surface, transform);
                    mesh_1.Mesh.uniformTriangleGroup(surface);
                    return [2 /*return*/, surface];
            }
        });
    });
}
function GaussianSurfaceMeshVisual(materialId) {
    return units_visual_1.UnitsMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.GaussianSurfaceMeshParams),
        createGeometry: createGaussianSurfaceMesh,
        createLocationIterator: element_1.StructureElementIterator.fromGroup,
        getLoci: element_1.getElementLoci,
        eachLocation: element_1.eachElement,
        setUpdateState: function (state, newProps, currentProps) {
            if (newProps.resolution !== currentProps.resolution)
                state.createGeometry = true;
            if (newProps.radiusOffset !== currentProps.radiusOffset)
                state.createGeometry = true;
            if (newProps.smoothness !== currentProps.smoothness)
                state.createGeometry = true;
            if (newProps.useGpu !== currentProps.useGpu)
                state.createGeometry = true;
        }
    }, materialId);
}
exports.GaussianSurfaceMeshVisual = GaussianSurfaceMeshVisual;
//
function createGaussianSurfaceTextureMesh(ctx, unit, structure, theme, props, textureMesh) {
    return __awaiter(this, void 0, void 0, function () {
        var isoLevel, densityTextureData, activeVoxelsTex, compacted, gv, boundingSphere, surface;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!ctx.webgl)
                        throw new Error('webgl context required to create gaussian surface texture-mesh');
                    isoLevel = Math.exp(-props.smoothness);
                    return [4 /*yield*/, gaussian_1.computeUnitGaussianDensityTexture2d(unit, props, ctx.webgl).runInContext(ctx.runtime)
                        // console.log(densityTextureData)
                        // console.log('vertexGroupTexture', readTexture(ctx.webgl, densityTextureData.texture))
                        // ctx.webgl.waitForGpuCommandsCompleteSync()
                    ];
                case 1:
                    densityTextureData = _a.sent();
                    activeVoxelsTex = active_voxels_1.calcActiveVoxels(ctx.webgl, densityTextureData.texture, densityTextureData.gridDim, densityTextureData.gridTexDim, isoLevel, densityTextureData.gridTexScale);
                    compacted = reduction_1.createHistogramPyramid(ctx.webgl, activeVoxelsTex, densityTextureData.gridTexScale);
                    gv = isosurface_1.createIsosurfaceBuffers(ctx.webgl, activeVoxelsTex, densityTextureData.texture, compacted, densityTextureData.gridDim, densityTextureData.gridTexDim, densityTextureData.transform, isoLevel, textureMesh ? textureMesh.vertexGroupTexture.ref.value : undefined, textureMesh ? textureMesh.normalTexture.ref.value : undefined);
                    boundingSphere = geometry_1.Sphere3D.fromBox3D(geometry_1.Sphere3D(), densityTextureData.bbox);
                    surface = texture_mesh_1.TextureMesh.create(gv.vertexCount, 1, gv.vertexGroupTexture, gv.normalTexture, boundingSphere, textureMesh);
                    // ctx.webgl.waitForGpuCommandsCompleteSync()
                    return [2 /*return*/, surface];
            }
        });
    });
}
function GaussianSurfaceTextureMeshVisual(materialId) {
    return units_visual_1.UnitsTextureMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.GaussianSurfaceMeshParams),
        createGeometry: createGaussianSurfaceTextureMesh,
        createLocationIterator: element_1.StructureElementIterator.fromGroup,
        getLoci: element_1.getElementLoci,
        eachLocation: element_1.eachElement,
        setUpdateState: function (state, newProps, currentProps) {
            if (newProps.resolution !== currentProps.resolution)
                state.createGeometry = true;
            if (newProps.radiusOffset !== currentProps.radiusOffset)
                state.createGeometry = true;
            if (newProps.smoothness !== currentProps.smoothness)
                state.createGeometry = true;
        }
    }, materialId);
}
exports.GaussianSurfaceTextureMeshVisual = GaussianSurfaceTextureMeshVisual;
