"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var structure_1 = require("../../../mol-model/structure");
var mesh_1 = require("../../../mol-geo/geometry/mesh/mesh");
var mol_util_1 = require("../../../mol-util");
var link_1 = require("./util/link");
var complex_visual_1 = require("../complex-visual");
var loci_1 = require("../../../mol-model/loci");
var int_1 = require("../../../mol-data/int");
function createInterUnitLinkCylinderMesh(ctx, structure, theme, props, mesh) {
    var links = structure.links;
    var bondCount = links.bondCount, bonds = links.bonds;
    var sizeFactor = props.sizeFactor, sizeAspectRatio = props.sizeAspectRatio;
    if (!bondCount)
        return mesh_1.Mesh.createEmpty(mesh);
    var location = structure_1.StructureElement.create();
    var builderProps = {
        linkCount: bondCount,
        referencePosition: function (edgeIndex) { return null; },
        position: function (posA, posB, edgeIndex) {
            var b = bonds[edgeIndex];
            var uA = b.unitA, uB = b.unitB;
            uA.conformation.position(uA.elements[b.indexA], posA);
            uB.conformation.position(uB.elements[b.indexB], posB);
        },
        order: function (edgeIndex) { return bonds[edgeIndex].order; },
        flags: function (edgeIndex) { return mol_util_1.BitFlags.create(bonds[edgeIndex].flag); },
        radius: function (edgeIndex) {
            var b = bonds[edgeIndex];
            location.unit = b.unitA;
            location.element = b.unitA.elements[b.indexA];
            return theme.size.size(location) * sizeFactor * sizeAspectRatio;
        }
    };
    return link_1.createLinkCylinderMesh(ctx, builderProps, props, mesh);
}
exports.InterUnitLinkParams = __assign({}, complex_visual_1.ComplexMeshParams, link_1.LinkCylinderParams, { sizeFactor: param_definition_1.ParamDefinition.Numeric(0.3, { min: 0, max: 10, step: 0.01 }), sizeAspectRatio: param_definition_1.ParamDefinition.Numeric(2 / 3, { min: 0, max: 3, step: 0.01 }) });
function InterUnitLinkVisual(materialId) {
    return complex_visual_1.ComplexMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.InterUnitLinkParams),
        createGeometry: createInterUnitLinkCylinderMesh,
        createLocationIterator: link_1.LinkIterator.fromStructure,
        getLoci: getLinkLoci,
        eachLocation: eachLink,
        setUpdateState: function (state, newProps, currentProps) {
            state.createGeometry = (newProps.sizeFactor !== currentProps.sizeFactor ||
                newProps.sizeAspectRatio !== currentProps.sizeAspectRatio ||
                newProps.radialSegments !== currentProps.radialSegments ||
                newProps.linkScale !== currentProps.linkScale ||
                newProps.linkSpacing !== currentProps.linkSpacing);
        }
    }, materialId);
}
exports.InterUnitLinkVisual = InterUnitLinkVisual;
function getLinkLoci(pickingId, structure, id) {
    var objectId = pickingId.objectId, groupId = pickingId.groupId;
    if (id === objectId) {
        var bond = structure.links.bonds[groupId];
        return structure_1.Link.Loci(structure, [
            structure_1.Link.Location(bond.unitA, bond.indexA, bond.unitB, bond.indexB),
            structure_1.Link.Location(bond.unitB, bond.indexB, bond.unitA, bond.indexA)
        ]);
    }
    return loci_1.EmptyLoci;
}
function eachLink(loci, structure, apply) {
    var changed = false;
    if (structure_1.Link.isLoci(loci)) {
        if (!structure_1.Structure.areParentsEquivalent(loci.structure, structure))
            return false;
        for (var _i = 0, _a = loci.links; _i < _a.length; _i++) {
            var b = _a[_i];
            var idx = structure.links.getBondIndex(b.aIndex, b.aUnit, b.bIndex, b.bUnit);
            if (idx !== -1) {
                if (apply(int_1.Interval.ofSingleton(idx)))
                    changed = true;
            }
        }
    }
    else if (structure_1.StructureElement.isLoci(loci)) {
        if (!structure_1.Structure.areParentsEquivalent(loci.structure, structure))
            return false;
        var _loop_1 = function (e) {
            int_1.OrderedSet.forEach(e.indices, function (v) {
                var indices = structure.links.getBondIndices(v, e.unit);
                for (var i = 0, il = indices.length; i < il; ++i) {
                    if (apply(int_1.Interval.ofSingleton(indices[i])))
                        changed = true;
                }
            });
        };
        // TODO mark link only when both of the link elements are in a StructureElement.Loci
        for (var _b = 0, _c = loci.elements; _b < _c.length; _b++) {
            var e = _c[_b];
            _loop_1(e);
        }
    }
    return changed;
}
