"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var structure_1 = require("../../../mol-model/structure");
var mesh_1 = require("../../../mol-geo/geometry/mesh/mesh");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var mol_util_1 = require("../../../mol-util");
var link_1 = require("./util/link");
var units_visual_1 = require("../units-visual");
var loci_1 = require("../../../mol-model/loci");
var int_1 = require("../../../mol-data/int");
function createIntraUnitLinkCylinderMesh(ctx, unit, structure, theme, props, mesh) {
    if (!structure_1.Unit.isAtomic(unit))
        return mesh_1.Mesh.createEmpty(mesh);
    var location = structure_1.StructureElement.create(unit);
    var elements = unit.elements;
    var links = unit.links;
    var edgeCount = links.edgeCount, a = links.a, b = links.b, edgeProps = links.edgeProps, offset = links.offset;
    var _order = edgeProps.order, _flags = edgeProps.flags;
    var sizeFactor = props.sizeFactor, sizeAspectRatio = props.sizeAspectRatio;
    if (!edgeCount)
        return mesh_1.Mesh.createEmpty(mesh);
    var vRef = linear_algebra_1.Vec3.zero();
    var pos = unit.conformation.invariantPosition;
    var builderProps = {
        linkCount: edgeCount * 2,
        referencePosition: function (edgeIndex) {
            var _a, _b;
            var aI = a[edgeIndex], bI = b[edgeIndex];
            if (aI > bI)
                _a = [bI, aI], aI = _a[0], bI = _a[1];
            if (offset[aI + 1] - offset[aI] === 1)
                _b = [bI, aI], aI = _b[0], bI = _b[1];
            // TODO prefer reference atoms in rings
            for (var i = offset[aI], il = offset[aI + 1]; i < il; ++i) {
                var _bI = b[i];
                if (_bI !== bI && _bI !== aI)
                    return pos(elements[_bI], vRef);
            }
            for (var i = offset[bI], il = offset[bI + 1]; i < il; ++i) {
                var _aI = a[i];
                if (_aI !== aI && _aI !== bI)
                    return pos(elements[_aI], vRef);
            }
            return null;
        },
        position: function (posA, posB, edgeIndex) {
            pos(elements[a[edgeIndex]], posA);
            pos(elements[b[edgeIndex]], posB);
        },
        order: function (edgeIndex) { return _order[edgeIndex]; },
        flags: function (edgeIndex) { return mol_util_1.BitFlags.create(_flags[edgeIndex]); },
        radius: function (edgeIndex) {
            location.element = elements[a[edgeIndex]];
            return theme.size.size(location) * sizeFactor * sizeAspectRatio;
        }
    };
    return link_1.createLinkCylinderMesh(ctx, builderProps, props, mesh);
}
exports.IntraUnitLinkParams = __assign({}, units_visual_1.UnitsMeshParams, link_1.LinkCylinderParams, { sizeFactor: param_definition_1.ParamDefinition.Numeric(0.3, { min: 0, max: 10, step: 0.01 }), sizeAspectRatio: param_definition_1.ParamDefinition.Numeric(2 / 3, { min: 0, max: 3, step: 0.01 }) });
function IntraUnitLinkVisual(materialId) {
    return units_visual_1.UnitsMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.IntraUnitLinkParams),
        createGeometry: createIntraUnitLinkCylinderMesh,
        createLocationIterator: link_1.LinkIterator.fromGroup,
        getLoci: getLinkLoci,
        eachLocation: eachLink,
        setUpdateState: function (state, newProps, currentProps) {
            state.createGeometry = (newProps.sizeFactor !== currentProps.sizeFactor ||
                newProps.sizeAspectRatio !== currentProps.sizeAspectRatio ||
                newProps.radialSegments !== currentProps.radialSegments ||
                newProps.linkScale !== currentProps.linkScale ||
                newProps.linkSpacing !== currentProps.linkSpacing);
        }
    }, materialId);
}
exports.IntraUnitLinkVisual = IntraUnitLinkVisual;
function getLinkLoci(pickingId, structureGroup, id) {
    var objectId = pickingId.objectId, instanceId = pickingId.instanceId, groupId = pickingId.groupId;
    if (id === objectId) {
        var structure = structureGroup.structure, group = structureGroup.group;
        var unit = group.units[instanceId];
        if (structure_1.Unit.isAtomic(unit)) {
            return structure_1.Link.Loci(structure, [
                structure_1.Link.Location(unit, unit.links.a[groupId], unit, unit.links.b[groupId]),
                structure_1.Link.Location(unit, unit.links.b[groupId], unit, unit.links.a[groupId])
            ]);
        }
    }
    return loci_1.EmptyLoci;
}
function eachLink(loci, structureGroup, apply) {
    var changed = false;
    if (structure_1.Link.isLoci(loci)) {
        var structure = structureGroup.structure, group = structureGroup.group;
        if (!structure_1.Structure.areParentsEquivalent(loci.structure, structure))
            return false;
        var unit = group.units[0];
        if (!structure_1.Unit.isAtomic(unit))
            return false;
        var groupCount = unit.links.edgeCount * 2;
        for (var _i = 0, _a = loci.links; _i < _a.length; _i++) {
            var b = _a[_i];
            var unitIdx = group.unitIndexMap.get(b.aUnit.id);
            if (unitIdx !== undefined) {
                var idx = unit.links.getDirectedEdgeIndex(b.aIndex, b.bIndex);
                if (idx !== -1) {
                    if (apply(int_1.Interval.ofSingleton(unitIdx * groupCount + idx)))
                        changed = true;
                }
            }
        }
    }
    else if (structure_1.StructureElement.isLoci(loci)) {
        var structure = structureGroup.structure, group = structureGroup.group;
        if (!structure_1.Structure.areParentsEquivalent(loci.structure, structure))
            return false;
        var unit = group.units[0];
        if (!structure_1.Unit.isAtomic(unit))
            return false;
        var groupCount_1 = unit.links.edgeCount * 2;
        var _loop_1 = function (e) {
            var unitIdx = group.unitIndexMap.get(e.unit.id);
            if (unitIdx !== undefined) {
                var _a = unit.links, offset_1 = _a.offset, b_1 = _a.b;
                int_1.OrderedSet.forEach(e.indices, function (v) {
                    for (var t = offset_1[v], _t = offset_1[v + 1]; t < _t; t++) {
                        if (int_1.OrderedSet.has(e.indices, b_1[t])) {
                            if (apply(int_1.Interval.ofSingleton(unitIdx * groupCount_1 + t)))
                                changed = true;
                        }
                    }
                });
            }
        };
        for (var _b = 0, _c = loci.elements; _b < _c.length; _b++) {
            var e = _c[_b];
            _loop_1(e);
        }
    }
    return changed;
}
