"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var box_1 = require("../../../mol-geo/primitive/box");
var structure_1 = require("../../../mol-model/structure");
var mesh_1 = require("../../../mol-geo/geometry/mesh/mesh");
var mesh_builder_1 = require("../../../mol-geo/geometry/mesh/mesh-builder");
var int_1 = require("../../../mol-data/int");
var types_1 = require("../../../mol-model/structure/model/types");
var cylinder_1 = require("../../../mol-geo/geometry/mesh/builder/cylinder");
var units_visual_1 = require("../units-visual");
var nucleotide_1 = require("./util/nucleotide");
var p1 = linear_algebra_1.Vec3.zero();
var p2 = linear_algebra_1.Vec3.zero();
var p3 = linear_algebra_1.Vec3.zero();
var p4 = linear_algebra_1.Vec3.zero();
var p5 = linear_algebra_1.Vec3.zero();
var p6 = linear_algebra_1.Vec3.zero();
var v12 = linear_algebra_1.Vec3.zero();
var v34 = linear_algebra_1.Vec3.zero();
var vC = linear_algebra_1.Vec3.zero();
var center = linear_algebra_1.Vec3.zero();
var t = linear_algebra_1.Mat4.identity();
var sVec = linear_algebra_1.Vec3.zero();
var box = box_1.Box();
exports.NucleotideBlockMeshParams = {
    sizeFactor: param_definition_1.ParamDefinition.Numeric(0.2, { min: 0, max: 10, step: 0.01 }),
    radialSegments: param_definition_1.ParamDefinition.Numeric(16, { min: 3, max: 56, step: 1 }),
};
exports.DefaultNucleotideBlockMeshProps = param_definition_1.ParamDefinition.getDefaultValues(exports.NucleotideBlockMeshParams);
function createNucleotideBlockMesh(ctx, unit, structure, theme, props, mesh) {
    if (!structure_1.Unit.isAtomic(unit))
        return mesh_1.Mesh.createEmpty(mesh);
    var nucleotideElementCount = unit.nucleotideElements.length;
    if (!nucleotideElementCount)
        return mesh_1.Mesh.createEmpty(mesh);
    var sizeFactor = props.sizeFactor, radialSegments = props.radialSegments;
    var vertexCount = nucleotideElementCount * (box.vertices.length / 3 + radialSegments * 2);
    var builderState = mesh_builder_1.MeshBuilder.createState(vertexCount, vertexCount / 4, mesh);
    var elements = unit.elements, model = unit.model;
    var modifiedResidues = model.properties.modifiedResidues;
    var _a = model.atomicHierarchy, chainAtomSegments = _a.chainAtomSegments, residueAtomSegments = _a.residueAtomSegments, residues = _a.residues, atomicIndex = _a.index;
    var _b = model.atomicHierarchy.derived.residue, moleculeType = _b.moleculeType, traceElementIndex = _b.traceElementIndex;
    var label_comp_id = residues.label_comp_id;
    var pos = unit.conformation.invariantPosition;
    var chainIt = int_1.Segmentation.transientSegments(chainAtomSegments, elements);
    var residueIt = int_1.Segmentation.transientSegments(residueAtomSegments, elements);
    var cylinderProps = { radiusTop: 1 * sizeFactor, radiusBottom: 1 * sizeFactor, radialSegments: radialSegments };
    var i = 0;
    while (chainIt.hasNext) {
        residueIt.setSegment(chainIt.move());
        while (residueIt.hasNext) {
            var residueIndex = residueIt.move().index;
            if (types_1.isNucleic(moleculeType[residueIndex])) {
                var compId = label_comp_id.value(residueIndex);
                var parentId = modifiedResidues.parentId.get(compId);
                if (parentId !== undefined)
                    compId = parentId;
                var idx1 = -1, idx2 = -1, idx3 = -1, idx4 = -1, idx5 = -1, idx6 = -1;
                var width = 4.5, height = 4.5, depth = 2.5 * sizeFactor;
                if (types_1.isPurinBase(compId)) {
                    height = 4.5;
                    idx1 = atomicIndex.findAtomOnResidue(residueIndex, 'N1');
                    idx2 = atomicIndex.findAtomOnResidue(residueIndex, 'C4');
                    idx3 = atomicIndex.findAtomOnResidue(residueIndex, 'C6');
                    idx4 = atomicIndex.findAtomOnResidue(residueIndex, 'C2');
                    idx5 = atomicIndex.findAtomOnResidue(residueIndex, 'N9');
                    idx6 = traceElementIndex[residueIndex];
                }
                else if (types_1.isPyrimidineBase(compId)) {
                    height = 3.0;
                    idx1 = atomicIndex.findAtomOnResidue(residueIndex, 'N3');
                    idx2 = atomicIndex.findAtomOnResidue(residueIndex, 'C6');
                    idx3 = atomicIndex.findAtomOnResidue(residueIndex, 'C4');
                    idx4 = atomicIndex.findAtomOnResidue(residueIndex, 'C2');
                    idx5 = atomicIndex.findAtomOnResidue(residueIndex, 'N1');
                    idx6 = traceElementIndex[residueIndex];
                }
                if (idx5 !== -1 && idx6 !== -1) {
                    pos(idx5, p5);
                    pos(idx6, p6);
                    builderState.currentGroup = i;
                    cylinder_1.addCylinder(builderState, p5, p6, 1, cylinderProps);
                    if (idx1 !== -1 && idx2 !== -1 && idx3 !== -1 && idx4 !== -1) {
                        pos(idx1, p1);
                        pos(idx2, p2);
                        pos(idx3, p3);
                        pos(idx4, p4);
                        linear_algebra_1.Vec3.normalize(v12, linear_algebra_1.Vec3.sub(v12, p2, p1));
                        linear_algebra_1.Vec3.normalize(v34, linear_algebra_1.Vec3.sub(v34, p4, p3));
                        linear_algebra_1.Vec3.normalize(vC, linear_algebra_1.Vec3.cross(vC, v12, v34));
                        linear_algebra_1.Mat4.targetTo(t, p1, p2, vC);
                        linear_algebra_1.Vec3.scaleAndAdd(center, p1, v12, height / 2 - 0.2);
                        linear_algebra_1.Mat4.scale(t, t, linear_algebra_1.Vec3.set(sVec, width, depth, height));
                        linear_algebra_1.Mat4.setTranslation(t, center);
                        mesh_builder_1.MeshBuilder.addPrimitive(builderState, t, box);
                    }
                }
                ++i;
            }
        }
    }
    return mesh_builder_1.MeshBuilder.getMesh(builderState);
}
exports.NucleotideBlockParams = __assign({}, units_visual_1.UnitsMeshParams, exports.NucleotideBlockMeshParams);
function NucleotideBlockVisual(materialId) {
    return units_visual_1.UnitsMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.NucleotideBlockParams),
        createGeometry: createNucleotideBlockMesh,
        createLocationIterator: nucleotide_1.NucleotideLocationIterator.fromGroup,
        getLoci: nucleotide_1.getNucleotideElementLoci,
        eachLocation: nucleotide_1.eachNucleotideElement,
        setUpdateState: function (state, newProps, currentProps) {
            state.createGeometry = (newProps.sizeFactor !== currentProps.sizeFactor ||
                newProps.radialSegments !== currentProps.radialSegments);
        }
    }, materialId);
}
exports.NucleotideBlockVisual = NucleotideBlockVisual;
