"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var mesh_1 = require("../../../mol-geo/geometry/mesh/mesh");
var mesh_builder_1 = require("../../../mol-geo/geometry/mesh/mesh-builder");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var polymer_1 = require("./util/polymer");
var int_1 = require("../../../mol-data/int");
var cylinder_1 = require("../../../mol-geo/geometry/mesh/builder/cylinder");
var units_visual_1 = require("../units-visual");
var element_1 = require("./util/element");
exports.PolymerBackboneCylinderParams = {
    sizeFactor: param_definition_1.ParamDefinition.Numeric(0.3, { min: 0, max: 10, step: 0.01 }),
    radialSegments: param_definition_1.ParamDefinition.Numeric(16, { min: 3, max: 56, step: 1 }),
};
exports.DefaultPolymerBackboneCylinderProps = param_definition_1.ParamDefinition.getDefaultValues(exports.PolymerBackboneCylinderParams);
// TODO do group id based on polymer index not element index
function createPolymerBackboneCylinderMesh(ctx, unit, structure, theme, props, mesh) {
    var polymerElementCount = unit.polymerElements.length;
    if (!polymerElementCount)
        return mesh_1.Mesh.createEmpty(mesh);
    var radialSegments = props.radialSegments, sizeFactor = props.sizeFactor;
    var vertexCountEstimate = radialSegments * 2 * polymerElementCount * 2;
    var builderState = mesh_builder_1.MeshBuilder.createState(vertexCountEstimate, vertexCountEstimate / 10, mesh);
    var elements = unit.elements;
    var pos = unit.conformation.invariantPosition;
    var pA = linear_algebra_1.Vec3.zero();
    var pB = linear_algebra_1.Vec3.zero();
    var cylinderProps = { radiusTop: 1, radiusBottom: 1, radialSegments: radialSegments };
    var polymerBackboneIt = polymer_1.PolymerBackboneIterator(unit);
    while (polymerBackboneIt.hasNext) {
        var _a = polymerBackboneIt.move(), centerA = _a.centerA, centerB = _a.centerB;
        pos(centerA.element, pA);
        pos(centerB.element, pB);
        cylinderProps.radiusTop = cylinderProps.radiusBottom = theme.size.size(centerA) * sizeFactor;
        builderState.currentGroup = int_1.OrderedSet.indexOf(elements, centerA.element);
        cylinder_1.addCylinder(builderState, pA, pB, 0.5, cylinderProps);
        cylinderProps.radiusTop = cylinderProps.radiusBottom = theme.size.size(centerB) * sizeFactor;
        builderState.currentGroup = int_1.OrderedSet.indexOf(elements, centerB.element);
        cylinder_1.addCylinder(builderState, pB, pA, 0.5, cylinderProps);
    }
    return mesh_builder_1.MeshBuilder.getMesh(builderState);
}
exports.PolymerBackboneParams = __assign({}, units_visual_1.UnitsMeshParams, exports.PolymerBackboneCylinderParams);
function PolymerBackboneVisual(materialId) {
    return units_visual_1.UnitsMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.PolymerBackboneParams),
        createGeometry: createPolymerBackboneCylinderMesh,
        // TODO create a specialized location iterator
        createLocationIterator: element_1.StructureElementIterator.fromGroup,
        getLoci: element_1.getElementLoci,
        eachLocation: element_1.eachElement,
        setUpdateState: function (state, newProps, currentProps) {
            state.createGeometry = (newProps.sizeFactor !== currentProps.sizeFactor ||
                newProps.radialSegments !== currentProps.radialSegments);
        }
    }, materialId);
}
exports.PolymerBackboneVisual = PolymerBackboneVisual;
