"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../../mol-util/param-definition");
var mesh_1 = require("../../../mol-geo/geometry/mesh/mesh");
var mesh_builder_1 = require("../../../mol-geo/geometry/mesh/mesh-builder");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var polymer_1 = require("./util/polymer");
var sphere_1 = require("../../../mol-geo/geometry/mesh/builder/sphere");
var cylinder_1 = require("../../../mol-geo/geometry/mesh/builder/cylinder");
var units_visual_1 = require("../units-visual");
var link_1 = require("./util/link");
var segmentCount = 10;
exports.PolymerGapCylinderParams = {
    sizeFactor: param_definition_1.ParamDefinition.Numeric(0.2, { min: 0, max: 10, step: 0.01 }),
    radialSegments: param_definition_1.ParamDefinition.Numeric(16, { min: 3, max: 56, step: 1 }),
};
exports.DefaultPolymerGapCylinderProps = param_definition_1.ParamDefinition.getDefaultValues(exports.PolymerGapCylinderParams);
function createPolymerGapCylinderMesh(ctx, unit, structure, theme, props, mesh) {
    var polymerGapCount = unit.gapElements.length;
    if (!polymerGapCount)
        return mesh_1.Mesh.createEmpty(mesh);
    var sizeFactor = props.sizeFactor, radialSegments = props.radialSegments;
    var vertexCountEstimate = segmentCount * radialSegments * 2 * polymerGapCount * 2;
    var builderState = mesh_builder_1.MeshBuilder.createState(vertexCountEstimate, vertexCountEstimate / 10, mesh);
    var pos = unit.conformation.invariantPosition;
    var pA = linear_algebra_1.Vec3.zero();
    var pB = linear_algebra_1.Vec3.zero();
    var cylinderProps = {
        radiusTop: 1, radiusBottom: 1, topCap: true, bottomCap: true, radialSegments: radialSegments
    };
    var i = 0;
    var polymerGapIt = polymer_1.PolymerGapIterator(unit);
    while (polymerGapIt.hasNext) {
        var _a = polymerGapIt.move(), centerA = _a.centerA, centerB = _a.centerB;
        if (centerA.element === centerB.element) {
            builderState.currentGroup = i;
            pos(centerA.element, pA);
            sphere_1.addSphere(builderState, pA, 0.6, 0);
        }
        else {
            pos(centerA.element, pA);
            pos(centerB.element, pB);
            cylinderProps.radiusTop = cylinderProps.radiusBottom = theme.size.size(centerA) * sizeFactor;
            builderState.currentGroup = i;
            cylinder_1.addFixedCountDashedCylinder(builderState, pA, pB, 0.5, segmentCount, cylinderProps);
            cylinderProps.radiusTop = cylinderProps.radiusBottom = theme.size.size(centerB) * sizeFactor;
            builderState.currentGroup = i + 1;
            cylinder_1.addFixedCountDashedCylinder(builderState, pB, pA, 0.5, segmentCount, cylinderProps);
        }
        i += 2;
    }
    return mesh_builder_1.MeshBuilder.getMesh(builderState);
}
exports.InterUnitLinkParams = __assign({}, units_visual_1.UnitsMeshParams, link_1.LinkCylinderParams);
exports.DefaultIntraUnitLinkProps = param_definition_1.ParamDefinition.getDefaultValues(exports.InterUnitLinkParams);
exports.PolymerGapParams = __assign({}, units_visual_1.UnitsMeshParams, exports.PolymerGapCylinderParams);
function PolymerGapVisual(materialId) {
    return units_visual_1.UnitsMeshVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.PolymerGapParams),
        createGeometry: createPolymerGapCylinderMesh,
        createLocationIterator: polymer_1.PolymerGapLocationIterator.fromGroup,
        getLoci: polymer_1.getPolymerGapElementLoci,
        eachLocation: polymer_1.eachPolymerGapElement,
        setUpdateState: function (state, newProps, currentProps) {
            state.createGeometry = (newProps.sizeFactor !== currentProps.sizeFactor ||
                newProps.radialSegments !== currentProps.radialSegments);
        }
    }, materialId);
}
exports.PolymerGapVisual = PolymerGapVisual;
