/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Unit, Structure, ElementIndex } from '../../../../mol-model/structure';
import { TransformData } from '../../../../mol-geo/geometry/transform-data';
import { SortedArray } from '../../../../mol-data/int';
import { Loci } from '../../../../mol-model/loci';
/** Return a Loci for the elements of a whole residue the elementIndex belongs to. */
export declare function getResidueLoci(structure: Structure, unit: Unit.Atomic, elementIndex: ElementIndex): Loci;
/**
 * Return a Loci for the elements of a whole residue the elementIndex belongs to but
 * restrict to elements that have the same label_alt_id or none
 */
export declare function getAltResidueLoci(structure: Structure, unit: Unit.Atomic, elementIndex: ElementIndex): Loci;
export declare function createUnitsTransform({ units }: Unit.SymmetryGroup, transformData?: TransformData): TransformData;
export declare const UnitKindInfo: {
    'atomic': {};
    'spheres': {};
    'gaussians': {};
};
export declare type UnitKind = keyof typeof UnitKindInfo;
export declare const UnitKindNames: string[];
export declare const UnitKindOptions: ["spheres" | "gaussians" | "atomic", string][];
export declare function includesUnitKind(unitKinds: UnitKind[], unit: Unit): boolean;
export declare function getConformation(unit: Unit): import("../../../../mol-model/structure/model/properties/atomic").AtomicConformation | import("../../../../mol-model/structure/model/properties/coarse").CoarseSphereConformation | import("../../../../mol-model/structure/model/properties/coarse").CoarseGaussianConformation;
export declare function getUnitConformationAndRadius(unit: Unit): {
    position: {
        indices: SortedArray<ElementIndex>;
        x: ArrayLike<number>;
        y: ArrayLike<number>;
        z: ArrayLike<number>;
    };
    radius: (index: number) => number;
};
export declare function getStructureConformationAndRadius(structure: Structure): {
    position: {
        indices: import("../../../../mol-data/int/interval").default<number> | SortedArray<number>;
        x: Float32Array;
        y: Float32Array;
        z: Float32Array;
    };
    radius: (index: number) => number;
};
