"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var linear_algebra_1 = require("../../../../mol-math/linear-algebra");
var structure_1 = require("../../../../mol-model/structure");
var loci_1 = require("../../../../mol-model/loci");
var int_1 = require("../../../../mol-data/int");
var sphere_1 = require("../../../../mol-geo/primitive/sphere");
var mesh_builder_1 = require("../../../../mol-geo/geometry/mesh/mesh-builder");
var sphere_2 = require("../../../../mol-geo/geometry/mesh/builder/sphere");
var location_iterator_1 = require("../../../../mol-geo/util/location-iterator");
var spheres_builder_1 = require("../../../../mol-geo/geometry/spheres/spheres-builder");
function createElementSphereMesh(ctx, unit, structure, theme, props, mesh) {
    var detail = props.detail, sizeFactor = props.sizeFactor;
    var elements = unit.elements;
    var elementCount = elements.length;
    var vertexCount = elementCount * sphere_1.sphereVertexCount(detail);
    var builderState = mesh_builder_1.MeshBuilder.createState(vertexCount, vertexCount / 2, mesh);
    var v = linear_algebra_1.Vec3.zero();
    var pos = unit.conformation.invariantPosition;
    var l = structure_1.StructureElement.create();
    l.unit = unit;
    for (var i = 0; i < elementCount; i++) {
        l.element = elements[i];
        pos(elements[i], v);
        builderState.currentGroup = i;
        sphere_2.addSphere(builderState, v, theme.size.size(l) * sizeFactor, detail);
    }
    return mesh_builder_1.MeshBuilder.getMesh(builderState);
}
exports.createElementSphereMesh = createElementSphereMesh;
function createElementSphereImpostor(ctx, unit, structure, theme, props, spheres) {
    var elements = unit.elements;
    var elementCount = elements.length;
    var builder = spheres_builder_1.SpheresBuilder.create(elementCount, elementCount / 2, spheres);
    var v = linear_algebra_1.Vec3.zero();
    var pos = unit.conformation.invariantPosition;
    for (var i = 0; i < elementCount; i++) {
        pos(elements[i], v);
        builder.add(v[0], v[1], v[2], i);
    }
    return builder.getSpheres();
}
exports.createElementSphereImpostor = createElementSphereImpostor;
function eachElement(loci, structureGroup, apply) {
    var changed = false;
    if (!structure_1.StructureElement.isLoci(loci))
        return false;
    var structure = structureGroup.structure, group = structureGroup.group;
    if (!structure_1.Structure.areParentsEquivalent(loci.structure, structure))
        return false;
    var elementCount = group.elements.length;
    for (var _a = 0, _b = loci.elements; _a < _b.length; _a++) {
        var e = _b[_a];
        var unitIdx = group.unitIndexMap.get(e.unit.id);
        if (unitIdx !== undefined) {
            if (int_1.Interval.is(e.indices)) {
                var start = unitIdx * elementCount + int_1.Interval.start(e.indices);
                var end = unitIdx * elementCount + int_1.Interval.end(e.indices);
                if (apply(int_1.Interval.ofBounds(start, end)))
                    changed = true;
            }
            else {
                for (var i = 0, _i = e.indices.length; i < _i; i++) {
                    var idx = unitIdx * elementCount + e.indices[i];
                    if (apply(int_1.Interval.ofSingleton(idx)))
                        changed = true;
                }
            }
        }
    }
    return changed;
}
exports.eachElement = eachElement;
function getElementLoci(pickingId, structureGroup, id) {
    var objectId = pickingId.objectId, instanceId = pickingId.instanceId, groupId = pickingId.groupId;
    if (id === objectId) {
        var structure = structureGroup.structure, group = structureGroup.group;
        var unit = group.units[instanceId];
        var indices = int_1.OrderedSet.ofSingleton(groupId);
        return structure_1.StructureElement.Loci(structure, [{ unit: unit, indices: indices }]);
    }
    return loci_1.EmptyLoci;
}
exports.getElementLoci = getElementLoci;
var StructureElementIterator;
(function (StructureElementIterator) {
    function fromGroup(group) {
        var groupCount = group.elements.length;
        var instanceCount = group.units.length;
        var location = structure_1.StructureElement.create();
        var getLocation = function (groupIndex, instanceIndex) {
            var unit = group.units[instanceIndex];
            location.unit = unit;
            location.element = unit.elements[groupIndex];
            return location;
        };
        return location_iterator_1.LocationIterator(groupCount, instanceCount, getLocation);
    }
    StructureElementIterator.fromGroup = fromGroup;
})(StructureElementIterator = exports.StructureElementIterator || (exports.StructureElementIterator = {}));
