/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Vec3 } from '../../../../mol-math/linear-algebra';
import { LinkType } from '../../../../mol-model/structure/model/types';
import { Unit, Structure } from '../../../../mol-model/structure';
import { ParamDefinition as PD } from '../../../../mol-util/param-definition';
import { Mesh } from '../../../../mol-geo/geometry/mesh/mesh';
import { LocationIterator } from '../../../../mol-geo/util/location-iterator';
import { VisualContext } from '../../../../mol-repr/visual';
export declare const LinkCylinderParams: {
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    radialSegments: PD.Numeric;
};
export declare const DefaultLinkCylinderProps: PD.Values<{
    linkScale: PD.Numeric;
    linkSpacing: PD.Numeric;
    radialSegments: PD.Numeric;
}>;
export declare type LinkCylinderProps = typeof DefaultLinkCylinderProps;
/** Calculate 'shift' direction that is perpendiculat to v1 - v2 and goes through v3 */
export declare function calculateShiftDir(out: Vec3, v1: Vec3, v2: Vec3, v3: Vec3 | null): Vec3;
export interface LinkCylinderMeshBuilderProps {
    linkCount: number;
    referencePosition(edgeIndex: number): Vec3 | null;
    position(posA: Vec3, posB: Vec3, edgeIndex: number): void;
    order(edgeIndex: number): number;
    flags(edgeIndex: number): LinkType;
    radius(edgeIndex: number): number;
}
/**
 * Each edge is included twice to allow for coloring/picking
 * the half closer to the first vertex, i.e. vertex a.
 */
export declare function createLinkCylinderMesh(ctx: VisualContext, linkBuilder: LinkCylinderMeshBuilderProps, props: LinkCylinderProps, mesh?: Mesh): Mesh;
export declare namespace LinkIterator {
    function fromGroup(group: Unit.SymmetryGroup): LocationIterator;
    function fromStructure(structure: Structure): LocationIterator;
}
