"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../../../mol-model/structure");
var loci_1 = require("../../../../mol-model/loci");
var int_1 = require("../../../../mol-data/int");
var location_iterator_1 = require("../../../../mol-geo/util/location-iterator");
var common_1 = require("./common");
var NucleotideLocationIterator;
(function (NucleotideLocationIterator) {
    function fromGroup(group) {
        var u = group.units[0];
        var nucleotideElementIndices = structure_1.Unit.isAtomic(u) ? u.nucleotideElements : [];
        var groupCount = nucleotideElementIndices.length;
        var instanceCount = group.units.length;
        var location = structure_1.StructureElement.create();
        var getLocation = function (groupIndex, instanceIndex) {
            var unit = group.units[instanceIndex];
            location.unit = unit;
            location.element = nucleotideElementIndices[groupIndex];
            return location;
        };
        return location_iterator_1.LocationIterator(groupCount, instanceCount, getLocation);
    }
    NucleotideLocationIterator.fromGroup = fromGroup;
})(NucleotideLocationIterator = exports.NucleotideLocationIterator || (exports.NucleotideLocationIterator = {}));
function getNucleotideElementLoci(pickingId, structureGroup, id) {
    var objectId = pickingId.objectId, instanceId = pickingId.instanceId, groupId = pickingId.groupId;
    if (id === objectId) {
        var structure = structureGroup.structure, group = structureGroup.group;
        var unit = group.units[instanceId];
        if (structure_1.Unit.isAtomic(unit)) {
            return common_1.getResidueLoci(structure, unit, unit.polymerElements[groupId]);
        }
    }
    return loci_1.EmptyLoci;
}
exports.getNucleotideElementLoci = getNucleotideElementLoci;
function eachNucleotideElement(loci, structureGroup, apply) {
    var changed = false;
    if (!structure_1.StructureElement.isLoci(loci))
        return false;
    var structure = structureGroup.structure, group = structureGroup.group;
    if (!structure_1.Structure.areParentsEquivalent(loci.structure, structure))
        return false;
    var unit = group.units[0];
    if (!structure_1.Unit.isAtomic(unit))
        return false;
    var nucleotideElements = unit.nucleotideElements, model = unit.model, elements = unit.elements;
    var _a = model.atomicHierarchy.residueAtomSegments, index = _a.index, offsets = _a.offsets;
    var traceElementIndex = model.atomicHierarchy.derived.residue.traceElementIndex;
    var groupCount = nucleotideElements.length;
    var _loop_1 = function (e) {
        var unitIdx = group.unitIndexMap.get(e.unit.id);
        var eUnit = e.unit;
        if (unitIdx !== undefined && structure_1.Unit.isAtomic(eUnit)) {
            // TODO optimized implementation for intervals
            int_1.OrderedSet.forEach(e.indices, function (v) {
                var rI = index[elements[v]];
                var unitIndexMin = int_1.OrderedSet.findPredecessorIndex(elements, offsets[rI]);
                var unitIndexMax = int_1.OrderedSet.findPredecessorIndex(elements, offsets[rI + 1] - 1);
                var unitIndexInterval = int_1.Interval.ofRange(unitIndexMin, unitIndexMax);
                if (!int_1.OrderedSet.isSubset(e.indices, unitIndexInterval))
                    return;
                var eI = traceElementIndex[rI];
                var idx = int_1.OrderedSet.indexOf(eUnit.nucleotideElements, eI);
                if (idx !== -1) {
                    if (apply(int_1.Interval.ofSingleton(unitIdx * groupCount + idx)))
                        changed = true;
                }
            });
        }
    };
    for (var _i = 0, _b = loci.elements; _i < _b.length; _i++) {
        var e = _b[_i];
        _loop_1(e);
    }
    return changed;
}
exports.eachNucleotideElement = eachNucleotideElement;
