/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Unit, ElementIndex, Link } from '../../../../mol-model/structure';
import SortedRanges from '../../../../mol-data/int/sorted-ranges';
import { Interval } from '../../../../mol-data/int';
import { Loci } from '../../../../mol-model/loci';
import { LocationIterator } from '../../../../mol-geo/util/location-iterator';
import { PickingId } from '../../../../mol-geo/geometry/picking';
import { StructureGroup } from '../../../structure/units-visual';
export * from './polymer/backbone-iterator';
export * from './polymer/gap-iterator';
export * from './polymer/trace-iterator';
export * from './polymer/curve-segment';
export declare function getPolymerRanges(unit: Unit): SortedRanges<ElementIndex>;
export declare function getGapRanges(unit: Unit): SortedRanges<ElementIndex>;
export declare namespace PolymerLocationIterator {
    function fromGroup(group: Unit.SymmetryGroup): LocationIterator;
}
export declare namespace PolymerGapLocationIterator {
    function fromGroup(group: Unit.SymmetryGroup): LocationIterator;
}
/** Return a Loci for the elements of the whole residue of a polymer element. */
export declare function getPolymerElementLoci(pickingId: PickingId, structureGroup: StructureGroup, id: number): Loci;
/**
 * Mark a polymer element (e.g. part of a cartoon trace)
 * - for atomic units mark only when all its residue's elements are in a loci
 */
export declare function eachPolymerElement(loci: Loci, structureGroup: StructureGroup, apply: (interval: Interval) => boolean): boolean;
/** Return a Loci for both directions of the polymer gap element. */
export declare function getPolymerGapElementLoci(pickingId: PickingId, structureGroup: StructureGroup, id: number): Link.Loci | {
    kind: "empty-loci";
};
export declare function eachPolymerGapElement(loci: Loci, structureGroup: StructureGroup, apply: (interval: Interval) => boolean): boolean;
