"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../mol-util");
var structure_1 = require("../mol-model/structure");
var VisualUpdateState;
(function (VisualUpdateState) {
    function create() {
        return {
            updateTransform: false,
            updateMatrix: false,
            updateColor: false,
            updateSize: false,
            createGeometry: false,
            createNew: false,
            info: {}
        };
    }
    VisualUpdateState.create = create;
    function reset(state) {
        state.updateTransform = false;
        state.updateMatrix = false;
        state.updateColor = false;
        state.updateSize = false;
        state.createGeometry = false;
        state.createNew = false;
    }
    VisualUpdateState.reset = reset;
})(VisualUpdateState = exports.VisualUpdateState || (exports.VisualUpdateState = {}));
function getQualityProps(props, data) {
    var quality = mol_util_1.defaults(props.quality, 'auto');
    var detail = mol_util_1.defaults(props.detail, 1);
    var radialSegments = mol_util_1.defaults(props.radialSegments, 12);
    var linearSegments = mol_util_1.defaults(props.linearSegments, 8);
    var resolution = mol_util_1.defaults(props.resolution, 2);
    if (quality === 'auto' && data instanceof structure_1.Structure) {
        var score = data.elementCount;
        if (data.isCoarse)
            score *= 10;
        if (score > 500000) {
            quality = 'lowest';
        }
        else if (score > 300000) {
            quality = 'lower';
        }
        else if (score > 100000) {
            quality = 'low';
        }
        else if (score > 30000) {
            quality = 'medium';
        }
        else if (score > 2000) {
            quality = 'high';
        }
        else {
            quality = 'higher';
        }
    }
    switch (quality) {
        case 'highest':
            detail = 3;
            radialSegments = 36;
            linearSegments = 18;
            resolution = 0.1;
            break;
        case 'higher':
            detail = 3;
            radialSegments = 28;
            linearSegments = 14;
            resolution = 0.3;
            break;
        case 'high':
            detail = 2;
            radialSegments = 20;
            linearSegments = 10;
            resolution = 0.5;
            break;
        case 'medium':
            detail = 1;
            radialSegments = 12;
            linearSegments = 8;
            resolution = 1;
            break;
        case 'low':
            detail = 0;
            radialSegments = 8;
            linearSegments = 3;
            resolution = 2;
            break;
        case 'lower':
            detail = 0;
            radialSegments = 4;
            linearSegments = 2;
            resolution = 4;
            break;
        case 'lowest':
            detail = 0;
            radialSegments = 2;
            linearSegments = 1;
            resolution = 8;
            break;
        case 'custom':
            // use defaults or given props as set above
            break;
    }
    return {
        detail: detail,
        radialSegments: radialSegments,
        linearSegments: linearSegments,
        resolution: resolution
    };
}
exports.getQualityProps = getQualityProps;
