/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { VolumeData } from '../../mol-model/volume';
import { RuntimeContext } from '../../mol-task';
import { WebGLContext } from '../../mol-gl/webgl/context';
import { DirectVolume } from '../../mol-geo/geometry/direct-volume/direct-volume';
import { VisualContext } from '../visual';
import { Theme, ThemeRegistryContext } from '../../mol-theme/theme';
import { VolumeVisual, VolumeRepresentation, VolumeRepresentationProvider } from './representation';
import { RepresentationContext, RepresentationParamsGetter } from '../representation';
export declare function createDirectVolume2d(ctx: RuntimeContext, webgl: WebGLContext, volume: VolumeData, directVolume?: DirectVolume): DirectVolume;
export declare function createDirectVolume3d(ctx: RuntimeContext, webgl: WebGLContext, volume: VolumeData, directVolume?: DirectVolume): DirectVolume;
export declare function createDirectVolume(ctx: VisualContext, volume: VolumeData, theme: Theme, props: PD.Values<DirectVolumeParams>, directVolume?: DirectVolume): Promise<DirectVolume>;
export declare const DirectVolumeParams: {
    isoValueNorm: PD.Numeric;
    renderMode: PD.Select<string>;
    controlPoints: PD.LineGraph;
    list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
};
export declare type DirectVolumeParams = typeof DirectVolumeParams;
export declare function getDirectVolumeParams(ctx: ThemeRegistryContext, volume: VolumeData): {
    isoValueNorm: PD.Numeric;
    renderMode: PD.Select<string>;
    controlPoints: PD.LineGraph;
    list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
};
export declare function DirectVolumeVisual(materialId: number): VolumeVisual<DirectVolumeParams>;
export declare function DirectVolumeRepresentation(ctx: RepresentationContext, getParams: RepresentationParamsGetter<VolumeData, DirectVolumeParams>): VolumeRepresentation<DirectVolumeParams>;
export declare const DirectVolumeRepresentationProvider: VolumeRepresentationProvider<DirectVolumeParams>;
