"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../mol-util/param-definition");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var geometry_1 = require("../../mol-math/geometry");
var volume_1 = require("../../mol-model/volume");
var direct_volume_1 = require("../../mol-geo/geometry/direct-volume/direct-volume");
var texture_1 = require("../../mol-gl/webgl/texture");
var base_1 = require("../../mol-geo/geometry/base");
var representation_1 = require("./representation");
var location_iterator_1 = require("../../mol-geo/util/location-iterator");
var location_1 = require("../../mol-model/location");
var loci_1 = require("../../mol-model/loci");
function getBoundingBox(gridDimension, transform) {
    var bbox = geometry_1.Box3D.empty();
    geometry_1.Box3D.add(bbox, gridDimension);
    geometry_1.Box3D.transform(bbox, bbox, transform);
    return bbox;
}
// 2d volume texture
function getVolumeTexture2dLayout(dim, maxTextureSize) {
    var width = 0;
    var height = dim[1];
    var rows = 1;
    var columns = dim[0];
    if (maxTextureSize < dim[0] * dim[2]) {
        columns = Math.floor(maxTextureSize / dim[0]);
        rows = Math.ceil(dim[2] / columns);
        width = columns * dim[0];
        height *= rows;
    }
    else {
        width = dim[0] * dim[2];
    }
    width += columns; // horizontal padding
    height += rows; // vertical padding
    return { width: width, height: height, columns: columns, rows: rows };
}
function createVolumeTexture2d(volume, maxTextureSize) {
    var tensor = volume.data, stats = volume.dataStats;
    var space = tensor.space, data = tensor.data;
    var dim = space.dimensions;
    var get = space.get;
    var _a = getVolumeTexture2dLayout(dim, maxTextureSize), width = _a.width, height = _a.height, columns = _a.columns, rows = _a.rows;
    var array = new Uint8Array(width * height * 4);
    var textureImage = { array: array, width: width, height: height };
    var xl = dim[0], yl = dim[1], zl = dim[2];
    var xlp = xl + 1; // horizontal padding
    var ylp = yl + 1; // vertical padding
    function setTex(value, x, y, z) {
        var column = Math.floor(((z * xlp) % width) / xlp);
        var row = Math.floor((z * xlp) / width);
        var px = column * xlp + x;
        var index = 4 * ((row * ylp * width) + (y * width) + px);
        array[index + 3] = ((value - stats.min) / (stats.max - stats.min)) * 255;
    }
    console.log('dim', dim);
    console.log('layout', { width: width, height: height, columns: columns, rows: rows });
    for (var z = 0; z < zl; ++z) {
        for (var y = 0; y < yl; ++y) {
            for (var x = 0; x < xl; ++x) {
                setTex(get(data, x, y, z), x, y, z);
            }
        }
    }
    return textureImage;
}
function createDirectVolume2d(ctx, webgl, volume, directVolume) {
    var gridDimension = volume.data.space.dimensions;
    var textureImage = createVolumeTexture2d(volume, webgl.maxTextureSize);
    // debugTexture(createImageData(textureImage.array, textureImage.width, textureImage.height), 1/3)
    var transform = volume_1.VolumeData.getGridToCartesianTransform(volume);
    var bbox = getBoundingBox(gridDimension, transform);
    var dim = linear_algebra_1.Vec3.create(gridDimension[0], gridDimension[1], gridDimension[2]);
    dim[0] += 1; // horizontal padding
    dim[0] += 1; // vertical padding
    var texture = directVolume ? directVolume.gridTexture.ref.value : texture_1.createTexture(webgl, 'image-uint8', 'rgba', 'ubyte', 'linear');
    texture.load(textureImage);
    return direct_volume_1.DirectVolume.create(bbox, dim, transform, texture, directVolume);
}
exports.createDirectVolume2d = createDirectVolume2d;
// 3d volume texture
function createVolumeTexture3d(volume) {
    var tensor = volume.data, stats = volume.dataStats;
    var space = tensor.space, data = tensor.data;
    var _a = space.dimensions, width = _a[0], height = _a[1], depth = _a[2];
    var get = space.get;
    var array = new Uint8Array(width * height * depth * 4);
    var textureVolume = { array: array, width: width, height: height, depth: depth };
    var i = 0;
    for (var z = 0; z < depth; ++z) {
        for (var y = 0; y < height; ++y) {
            for (var x = 0; x < width; ++x) {
                if (i < 100) {
                    console.log(get(data, x, y, z), ((get(data, x, y, z) - stats.min) / (stats.max - stats.min)) * 255);
                }
                array[i + 3] = ((get(data, x, y, z) - stats.min) / (stats.max - stats.min)) * 255;
                i += 4;
            }
        }
    }
    return textureVolume;
}
function createDirectVolume3d(ctx, webgl, volume, directVolume) {
    var gridDimension = volume.data.space.dimensions;
    var textureVolume = createVolumeTexture3d(volume);
    var transform = volume_1.VolumeData.getGridToCartesianTransform(volume);
    // Mat4.invert(transform, transform)
    var bbox = getBoundingBox(gridDimension, transform);
    var texture = directVolume ? directVolume.gridTexture.ref.value : texture_1.createTexture(webgl, 'volume-uint8', 'rgba', 'ubyte', 'linear');
    texture.load(textureVolume);
    return direct_volume_1.DirectVolume.create(bbox, gridDimension, transform, texture, directVolume);
}
exports.createDirectVolume3d = createDirectVolume3d;
//
function createDirectVolume(ctx, volume, theme, props, directVolume) {
    return __awaiter(this, void 0, void 0, function () {
        var runtime, webgl, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    runtime = ctx.runtime, webgl = ctx.webgl;
                    if (webgl === undefined)
                        throw new Error('DirectVolumeVisual requires `webgl` in props');
                    if (!webgl.isWebGL2) return [3 /*break*/, 2];
                    return [4 /*yield*/, createDirectVolume3d(runtime, webgl, volume, directVolume)];
                case 1:
                    _a = _b.sent();
                    return [3 /*break*/, 4];
                case 2: return [4 /*yield*/, createDirectVolume2d(runtime, webgl, volume, directVolume)];
                case 3:
                    _a = _b.sent();
                    _b.label = 4;
                case 4: return [2 /*return*/, _a];
            }
        });
    });
}
exports.createDirectVolume = createDirectVolume;
//
exports.DirectVolumeParams = __assign({}, base_1.BaseGeometry.Params, direct_volume_1.DirectVolume.Params);
function getDirectVolumeParams(ctx, volume) {
    return param_definition_1.ParamDefinition.clone(exports.DirectVolumeParams);
}
exports.getDirectVolumeParams = getDirectVolumeParams;
function DirectVolumeVisual(materialId) {
    return representation_1.VolumeVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.DirectVolumeParams),
        createGeometry: createDirectVolume,
        createLocationIterator: function (volume) { return location_iterator_1.LocationIterator(1, 1, function () { return location_1.NullLocation; }); },
        getLoci: function () { return loci_1.EmptyLoci; },
        eachLocation: function () { return false; },
        setUpdateState: function (state, volume, newProps, currentProps) {
        },
        geometryUtils: direct_volume_1.DirectVolume.Utils
    }, materialId);
}
exports.DirectVolumeVisual = DirectVolumeVisual;
function DirectVolumeRepresentation(ctx, getParams) {
    return representation_1.VolumeRepresentation('Direct Volume', ctx, getParams, DirectVolumeVisual);
}
exports.DirectVolumeRepresentation = DirectVolumeRepresentation;
exports.DirectVolumeRepresentationProvider = {
    label: 'Direct Volume',
    description: 'Direct volume rendering of volumetric data.',
    factory: DirectVolumeRepresentation,
    getParams: getDirectVolumeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.DirectVolumeParams),
    defaultColorTheme: 'uniform',
    defaultSizeTheme: 'uniform'
};
