/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { VolumeData, VolumeIsoValue } from '../../mol-model/volume';
import { VisualContext } from '../visual';
import { Theme, ThemeRegistryContext } from '../../mol-theme/theme';
import { Mesh } from '../../mol-geo/geometry/mesh/mesh';
import { VolumeVisual, VolumeRepresentation, VolumeRepresentationProvider } from './representation';
import { Lines } from '../../mol-geo/geometry/lines/lines';
import { RepresentationContext, RepresentationParamsGetter } from '../representation';
export declare function createIsoValueParam(defaultValue: VolumeIsoValue, stats?: VolumeData['dataStats']): PD.Conditioned<VolumeIsoValue, PD.Base<VolumeIsoValue>, {
    'absolute': PD.Converted<Readonly<{
        kind: "absolute";
        absoluteValue: number;
    }>, number>;
    'relative': PD.Converted<Readonly<{
        kind: "relative";
        relativeValue: number;
    }>, number>;
}>;
export declare const IsoValueParam: PD.Conditioned<VolumeIsoValue, PD.Base<VolumeIsoValue>, {
    'absolute': PD.Converted<Readonly<{
        kind: "absolute";
        absoluteValue: number;
    }>, number>;
    'relative': PD.Converted<Readonly<{
        kind: "relative";
        relativeValue: number;
    }>, number>;
}>;
export declare const VolumeIsosurfaceParams: {
    isoValue: PD.Conditioned<VolumeIsoValue, PD.Base<VolumeIsoValue>, {
        'absolute': PD.Converted<Readonly<{
            kind: "absolute";
            absoluteValue: number;
        }>, number>;
        'relative': PD.Converted<Readonly<{
            kind: "relative";
            relativeValue: number;
        }>, number>;
    }>;
};
export declare type VolumeIsosurfaceParams = typeof VolumeIsosurfaceParams;
export declare type VolumeIsosurfaceProps = PD.Values<VolumeIsosurfaceParams>;
export declare function createVolumeIsosurfaceMesh(ctx: VisualContext, volume: VolumeData, theme: Theme, props: VolumeIsosurfaceProps, mesh?: Mesh): Promise<Mesh>;
export declare const IsosurfaceMeshParams: {
    isoValue: PD.Conditioned<VolumeIsoValue, PD.Base<VolumeIsoValue>, {
        'absolute': PD.Converted<Readonly<{
            kind: "absolute";
            absoluteValue: number;
        }>, number>;
        'relative': PD.Converted<Readonly<{
            kind: "relative";
            relativeValue: number;
        }>, number>;
    }>;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
};
export declare type IsosurfaceMeshParams = typeof IsosurfaceMeshParams;
export declare function IsosurfaceMeshVisual(materialId: number): VolumeVisual<IsosurfaceMeshParams>;
export declare function createVolumeIsosurfaceWireframe(ctx: VisualContext, volume: VolumeData, theme: Theme, props: VolumeIsosurfaceProps, lines?: Lines): Promise<Lines>;
export declare const IsosurfaceWireframeParams: {
    isoValue: PD.Conditioned<VolumeIsoValue, PD.Base<VolumeIsoValue>, {
        'absolute': PD.Converted<Readonly<{
            kind: "absolute";
            absoluteValue: number;
        }>, number>;
        'relative': PD.Converted<Readonly<{
            kind: "relative";
            relativeValue: number;
        }>, number>;
    }>;
    sizeFactor: PD.Numeric;
    lineSizeAttenuation: PD.BooleanParam;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
};
export declare type IsosurfaceWireframeParams = typeof IsosurfaceWireframeParams;
export declare function IsosurfaceWireframeVisual(materialId: number): VolumeVisual<IsosurfaceWireframeParams>;
export declare const IsosurfaceParams: {
    visuals: PD.MultiSelect<"solid" | "wireframe", ("solid" | "wireframe")[]>;
    isoValue: PD.Conditioned<VolumeIsoValue, PD.Base<VolumeIsoValue>, {
        'absolute': PD.Converted<Readonly<{
            kind: "absolute";
            absoluteValue: number;
        }>, number>;
        'relative': PD.Converted<Readonly<{
            kind: "relative";
            relativeValue: number;
        }>, number>;
    }>;
    sizeFactor: PD.Numeric;
    lineSizeAttenuation: PD.BooleanParam;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type IsosurfaceParams = typeof IsosurfaceParams;
export declare function getIsosurfaceParams(ctx: ThemeRegistryContext, volume: VolumeData): {
    visuals: PD.MultiSelect<"solid" | "wireframe", ("solid" | "wireframe")[]>;
    isoValue: PD.Conditioned<VolumeIsoValue, PD.Base<VolumeIsoValue>, {
        'absolute': PD.Converted<Readonly<{
            kind: "absolute";
            absoluteValue: number;
        }>, number>;
        'relative': PD.Converted<Readonly<{
            kind: "relative";
            relativeValue: number;
        }>, number>;
    }>;
    sizeFactor: PD.Numeric;
    lineSizeAttenuation: PD.BooleanParam;
    alpha: PD.Numeric;
    useFog: PD.BooleanParam;
    highlightColor: PD.Color;
    selectColor: PD.Color;
    quality: PD.Select<"auto" | "custom" | "low" | "medium" | "high" | "highest" | "higher" | "lower" | "lowest">;
    doubleSided: PD.BooleanParam;
    flipSided: PD.BooleanParam;
    flatShaded: PD.BooleanParam;
};
export declare type IsosurfaceRepresentation = VolumeRepresentation<IsosurfaceParams>;
export declare function IsosurfaceRepresentation(ctx: RepresentationContext, getParams: RepresentationParamsGetter<VolumeData, IsosurfaceParams>): IsosurfaceRepresentation;
export declare const IsosurfaceRepresentationProvider: VolumeRepresentationProvider<IsosurfaceParams>;
