"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../mol-util/param-definition");
var volume_1 = require("../../mol-model/volume");
var mesh_1 = require("../../mol-geo/geometry/mesh/mesh");
var algorithm_1 = require("../../mol-geo/util/marching-cubes/algorithm");
var representation_1 = require("./representation");
var location_iterator_1 = require("../../mol-geo/util/location-iterator");
var location_1 = require("../../mol-model/location");
var loci_1 = require("../../mol-model/loci");
var lines_1 = require("../../mol-geo/geometry/lines/lines");
var representation_2 = require("../representation");
var defaultStats = { min: -1, max: 1, mean: 0, sigma: 0.1 };
function createIsoValueParam(defaultValue, stats) {
    var sts = stats || defaultStats;
    var min = sts.min, max = sts.max, mean = sts.mean, sigma = sts.sigma;
    // using ceil/floor could lead to "ouf of bounds" when converting
    var relMin = (min - mean) / sigma;
    var relMax = (max - mean) / sigma;
    var def = defaultValue;
    if (defaultValue.kind === 'absolute') {
        if (defaultValue.absoluteValue < min)
            def = volume_1.VolumeIsoValue.absolute(min);
        else if (defaultValue.absoluteValue > max)
            def = volume_1.VolumeIsoValue.absolute(max);
    }
    else {
        if (defaultValue.relativeValue < relMin)
            def = volume_1.VolumeIsoValue.relative(relMin);
        else if (defaultValue.relativeValue > relMax)
            def = volume_1.VolumeIsoValue.relative(relMax);
    }
    return param_definition_1.ParamDefinition.Conditioned(def, {
        'absolute': param_definition_1.ParamDefinition.Converted(function (v) { return volume_1.VolumeIsoValue.toAbsolute(v, volume_1.VolumeData.One.dataStats).absoluteValue; }, function (v) { return volume_1.VolumeIsoValue.absolute(v); }, param_definition_1.ParamDefinition.Numeric(mean, { min: min, max: max, step: sigma / 100 })),
        'relative': param_definition_1.ParamDefinition.Converted(function (v) { return volume_1.VolumeIsoValue.toRelative(v, volume_1.VolumeData.One.dataStats).relativeValue; }, function (v) { return volume_1.VolumeIsoValue.relative(v); }, param_definition_1.ParamDefinition.Numeric(Math.min(1, relMax), { min: relMin, max: relMax, step: Math.round(((max - min) / sigma)) / 100 }))
    }, function (v) { return v.kind === 'absolute' ? 'absolute' : 'relative'; }, function (v, c) { return c === 'absolute' ? volume_1.VolumeIsoValue.toAbsolute(v, sts) : volume_1.VolumeIsoValue.toRelative(v, sts); });
}
exports.createIsoValueParam = createIsoValueParam;
exports.IsoValueParam = createIsoValueParam(volume_1.VolumeIsoValue.relative(2));
exports.VolumeIsosurfaceParams = {
    isoValue: exports.IsoValueParam
};
//
function createVolumeIsosurfaceMesh(ctx, volume, theme, props, mesh) {
    return __awaiter(this, void 0, void 0, function () {
        var surface, transform;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    ctx.runtime.update({ message: 'Marching cubes...' });
                    return [4 /*yield*/, algorithm_1.computeMarchingCubesMesh({
                            isoLevel: volume_1.VolumeIsoValue.toAbsolute(props.isoValue, volume.dataStats).absoluteValue,
                            scalarField: volume.data
                        }, mesh).runAsChild(ctx.runtime)];
                case 1:
                    surface = _a.sent();
                    transform = volume_1.VolumeData.getGridToCartesianTransform(volume);
                    ctx.runtime.update({ message: 'Transforming mesh...' });
                    mesh_1.Mesh.transformImmediate(surface, transform);
                    return [2 /*return*/, surface];
            }
        });
    });
}
exports.createVolumeIsosurfaceMesh = createVolumeIsosurfaceMesh;
exports.IsosurfaceMeshParams = __assign({}, mesh_1.Mesh.Params, exports.VolumeIsosurfaceParams);
function IsosurfaceMeshVisual(materialId) {
    return representation_1.VolumeVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.IsosurfaceMeshParams),
        createGeometry: createVolumeIsosurfaceMesh,
        createLocationIterator: function (volume) { return location_iterator_1.LocationIterator(1, 1, function () { return location_1.NullLocation; }); },
        getLoci: function () { return loci_1.EmptyLoci; },
        eachLocation: function () { return false; },
        setUpdateState: function (state, volume, newProps, currentProps) {
            if (!volume_1.VolumeIsoValue.areSame(newProps.isoValue, currentProps.isoValue, volume.dataStats))
                state.createGeometry = true;
        },
        geometryUtils: mesh_1.Mesh.Utils
    }, materialId);
}
exports.IsosurfaceMeshVisual = IsosurfaceMeshVisual;
//
function createVolumeIsosurfaceWireframe(ctx, volume, theme, props, lines) {
    return __awaiter(this, void 0, void 0, function () {
        var wireframe, transform;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    ctx.runtime.update({ message: 'Marching cubes...' });
                    return [4 /*yield*/, algorithm_1.computeMarchingCubesLines({
                            isoLevel: volume_1.VolumeIsoValue.toAbsolute(props.isoValue, volume.dataStats).absoluteValue,
                            scalarField: volume.data
                        }, lines).runAsChild(ctx.runtime)];
                case 1:
                    wireframe = _a.sent();
                    transform = volume_1.VolumeData.getGridToCartesianTransform(volume);
                    lines_1.Lines.transformImmediate(wireframe, transform);
                    return [2 /*return*/, wireframe];
            }
        });
    });
}
exports.createVolumeIsosurfaceWireframe = createVolumeIsosurfaceWireframe;
exports.IsosurfaceWireframeParams = __assign({}, lines_1.Lines.Params, exports.VolumeIsosurfaceParams);
function IsosurfaceWireframeVisual(materialId) {
    return representation_1.VolumeVisual({
        defaultProps: param_definition_1.ParamDefinition.getDefaultValues(exports.IsosurfaceWireframeParams),
        createGeometry: createVolumeIsosurfaceWireframe,
        createLocationIterator: function (volume) { return location_iterator_1.LocationIterator(1, 1, function () { return location_1.NullLocation; }); },
        getLoci: function () { return loci_1.EmptyLoci; },
        eachLocation: function () { return false; },
        setUpdateState: function (state, volume, newProps, currentProps) {
            if (!volume_1.VolumeIsoValue.areSame(newProps.isoValue, currentProps.isoValue, volume.dataStats))
                state.createGeometry = true;
        },
        geometryUtils: lines_1.Lines.Utils
    }, materialId);
}
exports.IsosurfaceWireframeVisual = IsosurfaceWireframeVisual;
//
var IsosurfaceVisuals = {
    'solid': function (ctx, getParams) { return representation_1.VolumeRepresentation('Isosurface mesh', ctx, getParams, IsosurfaceMeshVisual); },
    'wireframe': function (ctx, getParams) { return representation_1.VolumeRepresentation('Isosurface wireframe', ctx, getParams, IsosurfaceWireframeVisual); },
};
var IsosurfaceVisualOptions = Object.keys(IsosurfaceVisuals).map(function (name) { return [name, name]; });
exports.IsosurfaceParams = __assign({}, exports.IsosurfaceMeshParams, exports.IsosurfaceWireframeParams, { visuals: param_definition_1.ParamDefinition.MultiSelect(['solid'], IsosurfaceVisualOptions) });
function getIsosurfaceParams(ctx, volume) {
    var p = param_definition_1.ParamDefinition.clone(exports.IsosurfaceParams);
    p.isoValue = createIsoValueParam(volume_1.VolumeIsoValue.relative(2), volume.dataStats);
    return p;
}
exports.getIsosurfaceParams = getIsosurfaceParams;
function IsosurfaceRepresentation(ctx, getParams) {
    return representation_2.Representation.createMulti('Isosurface', ctx, getParams, representation_2.Representation.StateBuilder, IsosurfaceVisuals);
}
exports.IsosurfaceRepresentation = IsosurfaceRepresentation;
exports.IsosurfaceRepresentationProvider = {
    label: 'Isosurface',
    description: 'Displays an isosurface of volumetric data.',
    factory: IsosurfaceRepresentation,
    getParams: getIsosurfaceParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.IsosurfaceParams),
    defaultColorTheme: 'uniform',
    defaultSizeTheme: 'uniform'
};
