"use strict";
/**
 * Copyright (c) 2018 Mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var symbol_table_1 = require("../../language/symbol-table");
var compiler_1 = require("./compiler");
var structure_1 = require("../../../mol-model/structure");
var types_1 = require("../../../mol-model/structure/model/types");
var set_1 = require("../../../mol-util/set");
var upper_case_1 = require("../../../mol-util/upper-case");
var atomic_1 = require("../../../mol-model/structure/model/properties/atomic");
var util_1 = require("../../../mol-data/util");
var C = compiler_1.QuerySymbolRuntime.Const;
var D = compiler_1.QuerySymbolRuntime.Dynamic;
var symbols = [
    // ============= TYPES =============
    C(symbol_table_1.MolScriptSymbolTable.core.type.bool, function (ctx, v) { return !!v[0](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.type.num, function (ctx, v) { return +v[0](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.type.str, function (ctx, v) { return '' + v[0](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.type.list, function (ctx, xs) { return compiler_1.QueryRuntimeArguments.forEachEval(xs, ctx, function (v, i, list) { return list[i] = v; }, []); }),
    C(symbol_table_1.MolScriptSymbolTable.core.type.set, function (ctx, xs) { return compiler_1.QueryRuntimeArguments.forEachEval(xs, ctx, function (v, i, set) { return set.add(v); }, new Set()); }),
    C(symbol_table_1.MolScriptSymbolTable.core.type.regex, function (ctx, v) { return new RegExp(v[0](ctx), (v[1] && v[1](ctx)) || ''); }),
    C(symbol_table_1.MolScriptSymbolTable.core.type.bitflags, function (ctx, v) { return +v[0](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.type.compositeKey, function (ctx, xs) { return compiler_1.QueryRuntimeArguments.forEachEval(xs, ctx, function (v, i, list) { return list[i] = '' + v; }, []).join('-'); }),
    // ============= LOGIC ================
    C(symbol_table_1.MolScriptSymbolTable.core.logic.not, function (ctx, v) { return !v[0](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.logic.and, function (ctx, xs) {
        if (typeof xs.length === 'number') {
            for (var i = 0, _i = xs.length; i < _i; i++)
                if (!xs[i](ctx))
                    return false;
        }
        else {
            for (var _a = 0, _b = Object.keys(xs); _a < _b.length; _a++) {
                var k = _b[_a];
                if (!xs[k](ctx))
                    return false;
            }
        }
        return true;
    }),
    C(symbol_table_1.MolScriptSymbolTable.core.logic.or, function (ctx, xs) {
        if (typeof xs.length === 'number') {
            for (var i = 0, _i = xs.length; i < _i; i++)
                if (xs[i](ctx))
                    return true;
        }
        else {
            for (var _a = 0, _b = Object.keys(xs); _a < _b.length; _a++) {
                var k = _b[_a];
                if (xs[k](ctx))
                    return true;
            }
        }
        return false;
    }),
    // ============= RELATIONAL ================
    C(symbol_table_1.MolScriptSymbolTable.core.rel.eq, function (ctx, v) { return v[0](ctx) === v[1](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.rel.neq, function (ctx, v) { return v[0](ctx) !== v[1](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.rel.lt, function (ctx, v) { return v[0](ctx) < v[1](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.rel.lte, function (ctx, v) { return v[0](ctx) <= v[1](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.rel.gr, function (ctx, v) { return v[0](ctx) > v[1](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.rel.gre, function (ctx, v) { return v[0](ctx) >= v[1](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.rel.inRange, function (ctx, v) {
        var x = v[0](ctx);
        return x >= v[1](ctx) && x <= v[2](ctx);
    }),
    // ============= ARITHMETIC ================
    C(symbol_table_1.MolScriptSymbolTable.core.math.add, function (ctx, xs) {
        var ret = 0;
        if (typeof xs.length === 'number') {
            for (var i = 0, _i = xs.length; i < _i; i++)
                ret += xs[i](ctx);
        }
        else {
            for (var _a = 0, _b = Object.keys(xs); _a < _b.length; _a++) {
                var k = _b[_a];
                ret += xs[k](ctx);
            }
        }
        return ret;
    }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.sub, function (ctx, xs) {
        var ret = 0;
        if (typeof xs.length === 'number') {
            if (xs.length === 1)
                return -xs[0](ctx);
            ret = xs[0](ctx) || 0;
            for (var i = 1, _i = xs.length; i < _i; i++)
                ret -= xs[i](ctx);
        }
        else {
            var keys = Object.keys(xs);
            if (keys.length === 1)
                ret = xs[keys[0]](ctx) || 0;
            for (var i = 1, _i = keys.length; i < _i; i++)
                ret -= xs[keys[i]](ctx);
        }
        return ret;
    }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.mult, function (ctx, xs) {
        var ret = 1;
        if (typeof xs.length === 'number') {
            for (var i = 0, _i = xs.length; i < _i; i++)
                ret *= xs[i](ctx);
        }
        else {
            for (var _a = 0, _b = Object.keys(xs); _a < _b.length; _a++) {
                var k = _b[_a];
                ret *= xs[k](ctx);
            }
        }
        return ret;
    }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.div, function (ctx, v) { return v[0](ctx) / v[1](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.pow, function (ctx, v) { return Math.pow(v[0](ctx), v[1](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.mod, function (ctx, v) { return v[0](ctx) % v[1](ctx); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.min, function (ctx, xs) {
        var ret = Number.POSITIVE_INFINITY;
        if (typeof xs.length === 'number') {
            for (var i = 0, _i = xs.length; i < _i; i++)
                ret = Math.min(xs[i](ctx), ret);
        }
        else {
            for (var _a = 0, _b = Object.keys(xs); _a < _b.length; _a++) {
                var k = _b[_a];
                ret = Math.min(xs[k](ctx), ret);
            }
        }
        return ret;
    }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.max, function (ctx, xs) {
        var ret = Number.NEGATIVE_INFINITY;
        if (typeof xs.length === 'number') {
            for (var i = 0, _i = xs.length; i < _i; i++)
                ret = Math.max(xs[i](ctx), ret);
        }
        else {
            for (var _a = 0, _b = Object.keys(xs); _a < _b.length; _a++) {
                var k = _b[_a];
                ret = Math.max(xs[k](ctx), ret);
            }
        }
        return ret;
    }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.floor, function (ctx, v) { return Math.floor(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.ceil, function (ctx, v) { return Math.ceil(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.roundInt, function (ctx, v) { return Math.round(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.abs, function (ctx, v) { return Math.abs(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.sqrt, function (ctx, v) { return Math.sqrt(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.sin, function (ctx, v) { return Math.sin(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.cos, function (ctx, v) { return Math.cos(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.tan, function (ctx, v) { return Math.tan(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.asin, function (ctx, v) { return Math.asin(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.acos, function (ctx, v) { return Math.acos(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.atan, function (ctx, v) { return Math.atan(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.sinh, function (ctx, v) { return Math.sinh(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.cosh, function (ctx, v) { return Math.cosh(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.tanh, function (ctx, v) { return Math.tanh(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.exp, function (ctx, v) { return Math.exp(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.log, function (ctx, v) { return Math.log(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.log10, function (ctx, v) { return Math.log10(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.math.atan2, function (ctx, v) { return Math.atan2(v[0](ctx), v[1](ctx)); }),
    // ============= STRING ================
    C(symbol_table_1.MolScriptSymbolTable.core.str.match, function (ctx, v) { return v[0](ctx).test(v[1](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.str.concat, function (ctx, xs) {
        var ret = [];
        if (typeof xs.length === 'number') {
            for (var i = 0, _i = xs.length; i < _i; i++)
                ret.push(xs[i](ctx).toString());
        }
        else {
            for (var _a = 0, _b = Object.keys(xs); _a < _b.length; _a++) {
                var k = _b[_a];
                ret.push(xs[k](ctx).toString());
            }
        }
        return ret.join('');
    }),
    // ============= LIST ================
    C(symbol_table_1.MolScriptSymbolTable.core.list.getAt, function (ctx, v) { return v[0](ctx)[v[1](ctx)]; }),
    // ============= SET ================
    C(symbol_table_1.MolScriptSymbolTable.core.set.has, function (ctx, v) { return v[0](ctx).has(v[1](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.core.set.isSubset, function (ctx, v) { return set_1.SetUtils.isSuperset(v[1](ctx), v[0](ctx)); }),
    // ============= FLAGS ================
    C(symbol_table_1.MolScriptSymbolTable.core.flags.hasAny, function (ctx, v) {
        var test = v[1](ctx);
        var tested = v[0](ctx);
        if (!test)
            return !!tested;
        return (tested & test) !== 0;
    }),
    C(symbol_table_1.MolScriptSymbolTable.core.flags.hasAll, function (ctx, v) {
        var test = v[1](ctx);
        var tested = v[0](ctx);
        if (!test)
            return !tested;
        return (tested & test) === test;
    }),
    ////////////////////////////////////
    // Structure
    // ============= TYPES ================
    C(symbol_table_1.MolScriptSymbolTable.structureQuery.type.elementSymbol, function (ctx, v) { return types_1.ElementSymbol(v[0](ctx)); }),
    C(symbol_table_1.MolScriptSymbolTable.structureQuery.type.atomName, function (ctx, v) { return upper_case_1.default(v[0](ctx)); }),
    // TODO:
    // C(MolScript.structureQuery.type.bondFlags, (ctx, v) => StructureRuntime.BondProperties.createFlags(env, v)),
    // C(MolScript.structureQuery.type.secondaryStructureFlags, (ctx, v) => StructureRuntime.AtomProperties.createSecondaryStructureFlags(env, v)),
    // C(MolScript.structureQuery.type.entityType, (ctx, v) => StructureRuntime.Common.entityType(v[0](ctx))),
    // C(MolScript.structureQuery.type.ringFingerprint, (ctx, v) => StructureRuntime.Common.ringFingerprint(env, v as any)),
    // C(MolScript.structureQuery.type.authResidueId, (ctx, v) => ResidueIdentifier.auth(v[0](ctx), v[1](ctx), v[2] && v[2](ctx))),
    // C(MolScript.structureQuery.type.labelResidueId, (ctx, v) => ResidueIdentifier.label(v[0](ctx), v[1](ctx), v[2](ctx), v[3] && v[3](ctx))),
    // ============= SLOTS ================
    // TODO: slots might not be needed after all: reducer simply pushes/pops current element
    C(symbol_table_1.MolScriptSymbolTable.structureQuery.slot.element, function (ctx, _) { return ctx.element; }),
    // C(MolScript.structureQuery.slot.elementSetReduce, (ctx, _) => ctx.element),
    // ============= FILTERS ================
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.filter.pick, function (ctx, xs) { return structure_1.Queries.filters.pick(xs[0], xs['test'])(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.filter.first, function (ctx, xs) { return structure_1.Queries.filters.first(xs[0])(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.filter.withSameAtomProperties, function (ctx, xs) { return structure_1.Queries.filters.withSameAtomProperties(xs[0], xs['source'], xs['property'])(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.filter.intersectedBy, function (ctx, xs) { return structure_1.Queries.filters.areIntersectedBy(xs[0], xs['by'])(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.filter.within, function (ctx, xs) { return structure_1.Queries.filters.within({
        query: xs[0],
        target: xs['target'],
        minRadius: xs['min-radius'],
        maxRadius: xs['max-radius'],
        elementRadius: xs['atom-radius'],
        invert: xs['invert']
    })(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.filter.isConnectedTo, function (ctx, xs) { return structure_1.Queries.filters.isConnectedTo({
        query: xs[0],
        target: xs['target'],
        disjunct: xs['disjunct'],
        invert: xs['invert'],
        bondTest: xs['bond-test']
    })(ctx); }),
    // ============= GENERATORS ================
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.generator.atomGroups, function (ctx, xs) { return structure_1.Queries.generators.atoms({
        entityTest: xs['entity-test'],
        chainTest: xs['chain-test'],
        residueTest: xs['residue-test'],
        atomTest: xs['atom-test'],
        groupBy: xs['group-by']
    })(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.generator.all, function (ctx) { return structure_1.Queries.generators.all(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.generator.empty, function (ctx) { return structure_1.Queries.generators.none(ctx); }),
    // ============= MODIFIERS ================
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.modifier.includeSurroundings, function (ctx, xs) { return structure_1.Queries.modifiers.includeSurroundings(xs[0], {
        radius: xs['radius'](ctx),
        wholeResidues: !!(xs['as-whole-residues'] && xs['as-whole-residues'](ctx)),
        elementRadius: xs['atom-radius']
    })(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.modifier.wholeResidues, function (ctx, xs) { return structure_1.Queries.modifiers.wholeResidues(xs[0])(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.modifier.union, function (ctx, xs) { return structure_1.Queries.modifiers.union(xs[0])(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.modifier.expandProperty, function (ctx, xs) { return structure_1.Queries.modifiers.expandProperty(xs[0], xs['property'])(ctx); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.modifier.exceptBy, function (ctx, xs) { return structure_1.Queries.modifiers.exceptBy(xs[0], xs['by'])(ctx); }),
    // ============= COMBINATORS ================
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.combinator.merge, function (ctx, xs) { return structure_1.Queries.combinators.merge(xs)(ctx); }),
    // ============= ATOM PROPERTIES ================
    // ~~~ CORE ~~~
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.elementSymbol, atomProp(structure_1.StructureProperties.atom.type_symbol)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.vdw, function (ctx, _) { return atomic_1.VdwRadius(structure_1.StructureProperties.atom.type_symbol(ctx.element)); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.mass, function (ctx, _) { return atomic_1.AtomWeight(structure_1.StructureProperties.atom.type_symbol(ctx.element)); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.atomicNumber, function (ctx, _) { return atomic_1.AtomNumber(structure_1.StructureProperties.atom.type_symbol(ctx.element)); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.x, atomProp(structure_1.StructureProperties.atom.x)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.y, atomProp(structure_1.StructureProperties.atom.y)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.z, atomProp(structure_1.StructureProperties.atom.z)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.sourceIndex, atomProp(structure_1.StructureProperties.atom.sourceIndex)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.operatorName, atomProp(structure_1.StructureProperties.unit.operator_name)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.core.atomKey, function (ctx, _) { return util_1.cantorPairing(ctx.element.unit.id, ctx.element.element); }),
    // TODO:
    // D(MolScript.structureQuery.atomProperty.core.bondCount, (ctx, _) => ),
    // ~~~ TOPOLOGY ~~~
    // TODO
    // ~~~ MACROMOLECULAR ~~~
    // TODO:
    // // identifiers
    // labelResidueId: prop((env, v) => ResidueIdentifier.labelOfResidueIndex(env.context.model, getAddress(env, v).residue)),
    // authResidueId: prop((env, v) => ResidueIdentifier.authOfResidueIndex(env.context.model, getAddress(env, v).residue)),
    // keys
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.residueKey, function (ctx, _) { return structure_1.StructureElement.residueIndex(ctx.element); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.chainKey, function (ctx, _) { return structure_1.StructureElement.chainIndex(ctx.element); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.entityKey, function (ctx, _) { return structure_1.StructureElement.entityIndex(ctx.element); }),
    // mmCIF
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.id, atomProp(structure_1.StructureProperties.atom.id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.isHet, function (ctx, _) { return structure_1.StructureProperties.residue.group_PDB(ctx.element) !== 'ATOM'; }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.label_atom_id, atomProp(structure_1.StructureProperties.atom.label_atom_id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.label_alt_id, atomProp(structure_1.StructureProperties.atom.label_alt_id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.label_asym_id, atomProp(structure_1.StructureProperties.chain.label_asym_id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.label_comp_id, atomProp(structure_1.StructureProperties.residue.label_comp_id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.label_seq_id, atomProp(structure_1.StructureProperties.residue.label_seq_id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.label_entity_id, atomProp(structure_1.StructureProperties.entity.id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.auth_atom_id, atomProp(structure_1.StructureProperties.atom.auth_atom_id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.auth_asym_id, atomProp(structure_1.StructureProperties.chain.auth_asym_id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.auth_comp_id, atomProp(structure_1.StructureProperties.residue.auth_comp_id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.auth_seq_id, atomProp(structure_1.StructureProperties.residue.auth_seq_id)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.pdbx_PDB_ins_code, atomProp(structure_1.StructureProperties.residue.pdbx_PDB_ins_code)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.pdbx_formal_charge, atomProp(structure_1.StructureProperties.atom.pdbx_formal_charge)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.occupancy, atomProp(structure_1.StructureProperties.atom.occupancy)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.B_iso_or_equiv, atomProp(structure_1.StructureProperties.atom.B_iso_or_equiv)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.entityType, atomProp(structure_1.StructureProperties.entity.type)),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.isModified, function (ctx, _) { return ctx.element.unit.model.properties.modifiedResidues.parentId.has(structure_1.StructureProperties.residue.label_comp_id(ctx.element)); }),
    D(symbol_table_1.MolScriptSymbolTable.structureQuery.atomProperty.macromolecular.modifiedParentName, function (ctx, _) {
        var id = structure_1.StructureProperties.residue.label_comp_id(ctx.element);
        return ctx.element.unit.model.properties.modifiedResidues.parentId.get(id) || id;
    })
    // TODO
    // MolScript.structureQuery.atomProperty.macromolecular.secondaryStructureKey
    // MolScript.structureQuery.atomProperty.macromolecular.secondaryStructureFlags
    // ============= BOND PROPERTIES ================
];
function atomProp(p) {
    return function (ctx, _) { return p(ctx.element); };
}
(function () {
    for (var _a = 0, symbols_1 = symbols; _a < symbols_1.length; _a++) {
        var s = symbols_1[_a];
        compiler_1.DefaultQueryRuntimeTable.addSymbol(s);
    }
})();
