/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { UUID } from '../mol-util';
import { StateTransform } from './transform';
import { ParamDefinition } from '../mol-util/param-definition';
import { State } from './state';
import { StateSelection, StateTransformer } from '../mol-state';
export { StateObject, StateObjectCell };
interface StateObject<D = any, T extends StateObject.Type = StateObject.Type<any>> {
    readonly id: UUID;
    readonly type: T;
    readonly data: D;
    readonly label: string;
    readonly description?: string;
    readonly tags?: string[];
}
declare namespace StateObject {
    function factory<T extends Type>(): <D = {}>(type: T) => {
        new (data: D, props?: {
            label: string;
            description?: string | undefined;
        } | undefined): {
            id: UUID;
            type: T;
            label: string;
            description?: string | undefined;
            data: D;
        };
        type: T;
        is(obj?: StateObject<any, Type<any>> | undefined): obj is {
            id: UUID;
            type: T;
            label: string;
            description?: string | undefined;
            data: D;
        };
    };
    type Type<Cls extends string = string> = {
        name: string;
        typeClass: Cls;
    };
    type Ctor<T extends StateObject = StateObject> = {
        new (...args: any[]): T;
        type: any;
    };
    type From<C extends Ctor> = C extends Ctor<infer T> ? T : never;
    function create<Data, T extends Type>(type: T): {
        new (data: Data, props?: {
            label: string;
            description?: string | undefined;
        } | undefined): {
            id: UUID;
            type: T;
            label: string;
            description?: string | undefined;
            data: Data;
        };
        type: T;
        is(obj?: StateObject<any, Type<any>> | undefined): obj is {
            id: UUID;
            type: T;
            label: string;
            description?: string | undefined;
            data: Data;
        };
    };
    /** A special object indicating a transformer result has no value. */
    const Null: StateObject<any, any>;
}
interface StateObjectCell<T extends StateObject = StateObject, F extends StateTransform<StateTransformer<any, T, any>> = StateTransform<StateTransformer<any, T, any>>> {
    parent: State;
    transform: F;
    sourceRef: StateTransform.Ref | undefined;
    status: StateObjectCell.Status;
    state: StateTransform.State;
    params: {
        definition: ParamDefinition.Params;
        values: any;
    } | undefined;
    errorText?: string;
    obj?: T;
    cache: unknown | undefined;
}
declare namespace StateObjectCell {
    type Status = 'ok' | 'error' | 'pending' | 'processing';
    type Obj<C extends StateObjectCell> = C extends StateObjectCell<infer T> ? T : never;
    type Transform<C extends StateObjectCell> = C extends StateObjectCell<any, infer T> ? T : never;
}
export declare class StateObjectTracker<T extends StateObject> {
    private state;
    private query;
    private version;
    cell: StateObjectCell | undefined;
    data: T['data'] | undefined;
    setQuery(sel: StateSelection.Selector): void;
    update(): boolean;
    constructor(state: State);
}
