/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { StateTree } from '../tree/immutable';
import { TransientTree } from '../tree/transient';
import { StateObject, StateObjectCell } from '../object';
import { StateTransform } from '../transform';
import { StateTransformer } from '../transformer';
import { State } from '../state';
export { StateBuilder };
interface StateBuilder {
    readonly editInfo: StateBuilder.EditInfo;
    getTree(): StateTree;
}
declare namespace StateBuilder {
    interface EditInfo {
        sourceTree: StateTree;
        count: number;
        lastUpdate?: StateTransform.Ref;
    }
    interface BuildState {
        state: State | undefined;
        tree: TransientTree;
        editInfo: EditInfo;
        actions: Action[];
    }
    type Action = {
        kind: 'add';
        transform: StateTransform;
    } | {
        kind: 'update';
        ref: string;
        params: any;
    } | {
        kind: 'delete';
        ref: string;
    } | {
        kind: 'insert';
        ref: string;
        transform: StateTransform;
    };
    function is(obj: any): obj is StateBuilder;
    function isTo(obj: any): obj is StateBuilder.To<any>;
    class Root implements StateBuilder {
        private state;
        readonly editInfo: EditInfo;
        to<A extends StateObject>(ref: StateTransform.Ref): To<A>;
        to<C extends StateObjectCell>(cell: C): To<StateObjectCell.Obj<C>, StateTransform.Transformer<StateObjectCell.Transform<C>>>;
        toRoot<A extends StateObject>(): To<A, StateTransformer<StateObject<any, StateObject.Type<any>>, StateObject<any, StateObject.Type<any>>, any>>;
        delete(ref: StateTransform.Ref): this;
        getTree(): StateTree;
        constructor(tree: StateTree, state?: State);
    }
    class To<A extends StateObject, T extends StateTransformer = StateTransformer> implements StateBuilder {
        private state;
        private root;
        readonly editInfo: EditInfo;
        readonly ref: StateTransform.Ref;
        /**
         * Apply the transformed to the parent node
         * If no params are specified (params <- undefined), default params are lazily resolved.
         */
        apply<T extends StateTransformer<A, any, any>>(tr: T, params?: StateTransformer.Params<T>, options?: Partial<StateTransform.Options>): To<StateTransformer.To<T>>;
        /**
         * If the ref is present, the transform is applied.
         * Otherwise a transform with the specifed ref is created.
         */
        applyOrUpdate<T extends StateTransformer<A, any, any>>(ref: StateTransform.Ref, tr: T, params?: StateTransformer.Params<T>, options?: Partial<StateTransform.Options>): To<StateTransformer.To<T>>;
        /**
         * A helper to greate a group-like state object and keep the current type.
         */
        group<T extends StateTransformer<A, any, any>>(tr: T, params?: StateTransformer.Params<T>, options?: Partial<StateTransform.Options>): To<A>;
        /**
         * Inserts a new transform that does not change the object type and move the original children to it.
         */
        insert<T extends StateTransformer<A, A, any>>(tr: T, params?: StateTransformer.Params<T>, options?: Partial<StateTransform.Options>): To<StateTransformer.To<T>>;
        update<T extends StateTransformer<any, A, any>>(transformer: T, params: (old: StateTransformer.Params<T>) => StateTransformer.Params<T>): Root;
        update(params: StateTransformer.Params<T> | ((old: StateTransformer.Params<T>) => StateTransformer.Params<T>)): Root;
        to<A extends StateObject>(ref: StateTransform.Ref): To<A, StateTransformer<StateObject<any, StateObject.Type<any>>, StateObject<any, StateObject.Type<any>>, any>>;
        toRoot<A extends StateObject>(): To<A, StateTransformer<StateObject<any, StateObject.Type<any>>, StateObject<any, StateObject.Type<any>>, any>>;
        delete(ref: StateTransform.Ref): Root;
        getTree(): StateTree;
        constructor(state: BuildState, ref: StateTransform.Ref, root: Root);
    }
}
