/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Color } from '../mol-util/color';
import { Location } from '../mol-model/location';
import { ColorType } from '../mol-geo/geometry/color-data';
import { ParamDefinition as PD } from '../mol-util/param-definition';
import { ThemeDataContext, ThemeRegistry, ThemeProvider } from './theme';
import { ScaleLegend } from '../mol-util/color/scale';
import { TableLegend } from '../mol-util/color/tables';
export declare type LocationColor = (location: Location, isSecondary: boolean) => Color;
export { ColorTheme };
interface ColorTheme<P extends PD.Params> {
    readonly factory: ColorTheme.Factory<P>;
    readonly granularity: ColorType;
    readonly color: LocationColor;
    readonly props: Readonly<PD.Values<P>>;
    readonly contextHash?: number;
    readonly description?: string;
    readonly legend?: Readonly<ScaleLegend | TableLegend>;
}
declare namespace ColorTheme {
    type Props = {
        [k: string]: any;
    };
    type Factory<P extends PD.Params> = (ctx: ThemeDataContext, props: PD.Values<P>) => ColorTheme<P>;
    const EmptyFactory: () => ColorTheme<{}>;
    const Empty: ColorTheme<{}>;
    function areEqual(themeA: ColorTheme<any>, themeB: ColorTheme<any>): boolean;
    interface Provider<P extends PD.Params> extends ThemeProvider<ColorTheme<P>, P> {
    }
    const EmptyProvider: Provider<{}>;
    type Registry = ThemeRegistry<ColorTheme<any>>;
    function createRegistry(): ThemeRegistry<ColorTheme<any>>;
    type ParamValues<C extends ColorTheme.Provider<any>> = C extends ColorTheme.Provider<infer P> ? PD.Values<P> : never;
}
export declare const BuiltInColorThemes: {
    'carbohydrate-symbol': ColorTheme.Provider<{}>;
    'chain-id': ColorTheme.Provider<{
        palette: PD.Mapped<PD.NamedParams<PD.Normalize<{
            hue: [number, number];
            chroma: [number, number];
            luminance: [number, number];
            clusteringStepCount: number;
            minSampleCount: number;
        }>, "generate"> | PD.NamedParams<PD.Normalize<{
            list: "OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue";
        }>, "scale">>;
    }>;
    'cross-link': ColorTheme.Provider<{
        domain: PD.Interval;
        list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
    }>;
    'element-index': ColorTheme.Provider<{
        list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
    }>;
    'element-symbol': ColorTheme.Provider<{
        saturation: PD.Numeric;
        lightness: PD.Numeric;
    }>;
    'entity-source': ColorTheme.Provider<{
        palette: PD.Mapped<PD.NamedParams<PD.Normalize<{
            hue: [number, number];
            chroma: [number, number];
            luminance: [number, number];
            clusteringStepCount: number;
            minSampleCount: number;
        }>, "generate"> | PD.NamedParams<PD.Normalize<{
            list: "OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue";
        }>, "scale">>;
    }>;
    'hydrophobicity': ColorTheme.Provider<{
        list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
        scale: PD.Select<"DGwif" | "DGwoct" | "Oct-IF">;
    }>;
    'illustrative': ColorTheme.Provider<{}>;
    'molecule-type': ColorTheme.Provider<{
        saturation: PD.Numeric;
        lightness: PD.Numeric;
    }>;
    'polymer-id': ColorTheme.Provider<{
        palette: PD.Mapped<PD.NamedParams<PD.Normalize<{
            hue: [number, number];
            chroma: [number, number];
            luminance: [number, number];
            clusteringStepCount: number;
            minSampleCount: number;
        }>, "generate"> | PD.NamedParams<PD.Normalize<{
            list: "OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue";
        }>, "scale">>;
    }>;
    'polymer-index': ColorTheme.Provider<{
        palette: PD.Mapped<PD.NamedParams<PD.Normalize<{
            hue: [number, number];
            chroma: [number, number];
            luminance: [number, number];
            clusteringStepCount: number;
            minSampleCount: number;
        }>, "generate"> | PD.NamedParams<PD.Normalize<{
            list: "OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue";
        }>, "scale">>;
    }>;
    'residue-name': ColorTheme.Provider<{
        saturation: PD.Numeric;
        lightness: PD.Numeric;
    }>;
    'secondary-structure': ColorTheme.Provider<{
        saturation: PD.Numeric;
        lightness: PD.Numeric;
    }>;
    'sequence-id': ColorTheme.Provider<{
        list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
    }>;
    'shape-group': ColorTheme.Provider<{}>;
    'uncertainty': ColorTheme.Provider<{
        domain: PD.Interval;
        list: PD.ColorScale<"OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue">;
    }>;
    'unit-index': ColorTheme.Provider<{
        palette: PD.Mapped<PD.NamedParams<PD.Normalize<{
            hue: [number, number];
            chroma: [number, number];
            luminance: [number, number];
            clusteringStepCount: number;
            minSampleCount: number;
        }>, "generate"> | PD.NamedParams<PD.Normalize<{
            list: "OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue";
        }>, "scale">>;
    }>;
    'uniform': ColorTheme.Provider<{
        value: PD.Color;
    }>;
};
export declare type BuiltInColorThemeName = keyof typeof BuiltInColorThemes;
