"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../mol-model/structure");
var constants_1 = require("../../mol-model/structure/structure/carbohydrates/constants");
var color_1 = require("../../mol-util/color");
var param_definition_1 = require("../../mol-util/param-definition");
var tables_1 = require("../../mol-util/color/tables");
var DefaultColor = color_1.Color(0xCCCCCC);
var Description = 'Assigns colors according to the Symbol Nomenclature for Glycans (SNFG).';
exports.CarbohydrateSymbolColorThemeParams = {};
function getCarbohydrateSymbolColorThemeParams(ctx) {
    return exports.CarbohydrateSymbolColorThemeParams; // TODO return copy
}
exports.getCarbohydrateSymbolColorThemeParams = getCarbohydrateSymbolColorThemeParams;
function CarbohydrateSymbolColorTheme(ctx, props) {
    var color;
    if (ctx.structure) {
        var _a = ctx.structure.carbohydrates, elements_1 = _a.elements, getElementIndex_1 = _a.getElementIndex, getAnomericCarbons_1 = _a.getAnomericCarbons;
        var getColor_1 = function (unit, index) {
            var residueIndex = unit.model.atomicHierarchy.residueAtomSegments.index[index];
            var anomericCarbons = getAnomericCarbons_1(unit, residueIndex);
            if (anomericCarbons.length > 0) {
                var idx = getElementIndex_1(unit, anomericCarbons[0]);
                if (idx !== undefined)
                    return elements_1[idx].component.color;
            }
            return DefaultColor;
        };
        color = function (location, isSecondary) {
            if (isSecondary) {
                return constants_1.SaccharideColors.Secondary;
            }
            else {
                if (structure_1.StructureElement.isLocation(location)) {
                    return getColor_1(location.unit, location.element);
                }
                else if (structure_1.Link.isLocation(location)) {
                    return getColor_1(location.aUnit, location.aUnit.elements[location.aIndex]);
                }
            }
            return DefaultColor;
        };
    }
    else {
        color = function () { return DefaultColor; };
    }
    return {
        factory: CarbohydrateSymbolColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: Description,
        legend: tables_1.TableLegend(constants_1.MonosaccharidesColorTable)
    };
}
exports.CarbohydrateSymbolColorTheme = CarbohydrateSymbolColorTheme;
exports.CarbohydrateSymbolColorThemeProvider = {
    label: 'Carbohydrate Symbol',
    factory: CarbohydrateSymbolColorTheme,
    getParams: getCarbohydrateSymbolColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.CarbohydrateSymbolColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
