"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../mol-model/structure");
var color_1 = require("../../mol-util/color");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var param_definition_1 = require("../../mol-util/param-definition");
var scale_1 = require("../../mol-util/color/scale");
var DefaultColor = color_1.Color(0xCCCCCC);
var Description = 'Colors cross-links by the deviation of the observed distance versus the modeled distance (e.g. `ihm_cross_link_restraint.distance_threshold`).';
exports.CrossLinkColorThemeParams = {
    domain: param_definition_1.ParamDefinition.Interval([-10, 10]),
    list: param_definition_1.ParamDefinition.ColorScale('RedGrey', scale_1.ColorListOptions),
};
function getCrossLinkColorThemeParams(ctx) {
    return exports.CrossLinkColorThemeParams; // TODO return copy
}
exports.getCrossLinkColorThemeParams = getCrossLinkColorThemeParams;
var distVecA = linear_algebra_1.Vec3.zero(), distVecB = linear_algebra_1.Vec3.zero();
function linkDistance(link) {
    link.aUnit.conformation.position(link.aUnit.elements[link.aIndex], distVecA);
    link.bUnit.conformation.position(link.bUnit.elements[link.bIndex], distVecB);
    return linear_algebra_1.Vec3.distance(distVecA, distVecB);
}
function CrossLinkColorTheme(ctx, props) {
    var color;
    var scale = undefined;
    if (ctx.structure) {
        var crosslinks_1 = ctx.structure.crossLinkRestraints;
        scale = color_1.ColorScale.create({
            domain: props.domain,
            listOrName: props.list
        });
        var scaleColor_1 = scale.color;
        color = function (location) {
            if (structure_1.Link.isLocation(location)) {
                var pairs = crosslinks_1.getPairs(location.aIndex, location.aUnit, location.bIndex, location.bUnit);
                if (pairs) {
                    return scaleColor_1(linkDistance(location) - pairs[0].distanceThreshold);
                }
            }
            return DefaultColor;
        };
    }
    else {
        color = function () { return DefaultColor; };
    }
    return {
        factory: CrossLinkColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: Description,
        legend: scale ? scale.legend : undefined
    };
}
exports.CrossLinkColorTheme = CrossLinkColorTheme;
exports.CrossLinkColorThemeProvider = {
    label: 'Cross Link',
    factory: CrossLinkColorTheme,
    getParams: getCrossLinkColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.CrossLinkColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
