"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = require("../../mol-util/color");
var structure_1 = require("../../mol-model/structure");
var int_1 = require("../../mol-data/int");
var param_definition_1 = require("../../mol-util/param-definition");
var scale_1 = require("../../mol-util/color/scale");
var DefaultColor = color_1.Color(0xCCCCCC);
var Description = 'Gives every element (atom or coarse sphere/gaussian) a unique color based on the position (index) of the element in the list of elements in the structure.';
exports.ElementIndexColorThemeParams = {
    list: param_definition_1.ParamDefinition.ColorScale('RedYellowBlue', scale_1.ColorListOptions),
};
function getElementIndexColorThemeParams(ctx) {
    return exports.ElementIndexColorThemeParams; // TODO return copy
}
exports.getElementIndexColorThemeParams = getElementIndexColorThemeParams;
function ElementIndexColorTheme(ctx, props) {
    var color;
    var scale = color_1.ColorScale.create({ listOrName: props.list, minLabel: 'Start', maxLabel: 'End' });
    if (ctx.structure) {
        var units = ctx.structure.units;
        var unitCount = units.length;
        var cummulativeElementCount_1 = new Map();
        var unitIdIndex_1 = new Map();
        var elementCount = 0;
        for (var i = 0; i < unitCount; ++i) {
            cummulativeElementCount_1.set(i, elementCount);
            elementCount += units[i].elements.length;
            unitIdIndex_1.set(units[i].id, i);
        }
        scale.setDomain(0, elementCount - 1);
        var scaleColor_1 = scale.color;
        color = function (location) {
            if (structure_1.StructureElement.isLocation(location)) {
                var unitIndex = unitIdIndex_1.get(location.unit.id);
                var unitElementIndex = int_1.OrderedSet.findPredecessorIndex(location.unit.elements, location.element);
                return scaleColor_1(cummulativeElementCount_1.get(unitIndex) + unitElementIndex);
            }
            else if (structure_1.Link.isLocation(location)) {
                var unitIndex = unitIdIndex_1.get(location.aUnit.id);
                return scaleColor_1(cummulativeElementCount_1.get(unitIndex) + location.aIndex);
            }
            return DefaultColor;
        };
    }
    else {
        color = function () { return DefaultColor; };
    }
    return {
        factory: ElementIndexColorTheme,
        granularity: 'groupInstance',
        color: color,
        props: props,
        description: Description,
        legend: scale ? scale.legend : undefined
    };
}
exports.ElementIndexColorTheme = ElementIndexColorTheme;
exports.ElementIndexColorThemeProvider = {
    label: 'Element Index',
    factory: ElementIndexColorTheme,
    getParams: getElementIndexColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.ElementIndexColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
