"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("../../mol-model/structure/model/types");
var color_1 = require("../../mol-util/color");
var structure_1 = require("../../mol-model/structure");
var param_definition_1 = require("../../mol-util/param-definition");
var element_symbol_1 = require("./element-symbol");
var color_2 = require("../../mol-util/color/color");
var DefaultIllustrativeColor = color_1.Color(0xFFFFFF);
var Description = "Assigns an illustrative color similar to David Goodsell's Molecule of the Month style.";
exports.IllustrativeColorThemeParams = {};
function getIllustrativeColorThemeParams(ctx) {
    return exports.IllustrativeColorThemeParams; // TODO return copy
}
exports.getIllustrativeColorThemeParams = getIllustrativeColorThemeParams;
function illustrativeColor(colorMap, typeSymbol, moleculeType) {
    if (types_1.isNucleic(moleculeType)) {
        if (typeSymbol === 'O') {
            return color_1.Color(0xFF8C8C);
        }
        else if (typeSymbol === 'P') {
            return color_1.Color(0xFF7D7D);
        }
        else {
            return color_1.Color(0xFFA6A6);
        }
    }
    else if (types_1.isProtein(moleculeType)) {
        if (typeSymbol === 'C') {
            return color_1.Color(0x7FB2FF);
        }
        else {
            return color_1.Color(0x6699FF);
        }
    }
    else {
        return element_symbol_1.elementSymbolColor(colorMap, typeSymbol);
    }
}
exports.illustrativeColor = illustrativeColor;
function IllustrativeColorTheme(ctx, props) {
    var colorMap = color_2.getAdjustedColorMap(element_symbol_1.ElementSymbolColors, 0, 0.7);
    function color(location) {
        if (structure_1.StructureElement.isLocation(location)) {
            if (structure_1.Unit.isAtomic(location.unit)) {
                var moleculeType = location.unit.model.atomicHierarchy.derived.residue.moleculeType[location.unit.residueIndex[location.element]];
                var typeSymbol = location.unit.model.atomicHierarchy.atoms.type_symbol.value(location.element);
                return illustrativeColor(colorMap, typeSymbol, moleculeType);
            }
        }
        else if (structure_1.Link.isLocation(location)) {
            if (structure_1.Unit.isAtomic(location.aUnit)) {
                var elementIndex = location.aUnit.elements[location.aIndex];
                var moleculeType = location.aUnit.model.atomicHierarchy.derived.residue.moleculeType[location.aUnit.residueIndex[elementIndex]];
                var typeSymbol = location.aUnit.model.atomicHierarchy.atoms.type_symbol.value(elementIndex);
                return illustrativeColor(colorMap, typeSymbol, moleculeType);
            }
        }
        return DefaultIllustrativeColor;
    }
    return {
        factory: IllustrativeColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: Description,
    };
}
exports.IllustrativeColorTheme = IllustrativeColorTheme;
exports.IllustrativeColorThemeProvider = {
    label: 'Illustrative',
    factory: IllustrativeColorTheme,
    getParams: getIllustrativeColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.IllustrativeColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
