"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = require("../../mol-util/color");
var structure_1 = require("../../mol-model/structure");
var util_1 = require("../../mol-model/structure/util");
var param_definition_1 = require("../../mol-util/param-definition");
var tables_1 = require("../../mol-util/color/tables");
var color_2 = require("../../mol-util/color/color");
exports.MoleculeTypeColors = color_1.ColorMap({
    water: 0x386cb0,
    ion: 0xf0027f,
    protein: 0xbeaed4,
    RNA: 0xfdc086,
    DNA: 0xbf5b17,
    PNA: 0x42A49A,
    saccharide: 0x7fc97f,
});
var DefaultMoleculeTypeColor = color_1.Color(0xffff99);
var Description = 'Assigns a color based on the molecule type of a residue.';
exports.MoleculeTypeColorThemeParams = {
    saturation: param_definition_1.ParamDefinition.Numeric(0, { min: -6, max: 6, step: 0.1 }),
    lightness: param_definition_1.ParamDefinition.Numeric(0, { min: -6, max: 6, step: 0.1 })
};
function getMoleculeTypeColorThemeParams(ctx) {
    return exports.MoleculeTypeColorThemeParams; // TODO return copy
}
exports.getMoleculeTypeColorThemeParams = getMoleculeTypeColorThemeParams;
function moleculeTypeColor(colorMap, unit, element) {
    var moleculeType = util_1.getElementMoleculeType(unit, element);
    switch (moleculeType) {
        case 2 /* water */: return colorMap.water;
        case 3 /* ion */: return colorMap.ion;
        case 4 /* protein */: return colorMap.protein;
        case 5 /* RNA */: return colorMap.RNA;
        case 6 /* DNA */: return colorMap.DNA;
        case 7 /* PNA */: return colorMap.PNA;
        case 8 /* saccharide */: return colorMap.saccharide;
    }
    return DefaultMoleculeTypeColor;
}
exports.moleculeTypeColor = moleculeTypeColor;
function MoleculeTypeColorTheme(ctx, props) {
    var colorMap = color_2.getAdjustedColorMap(exports.MoleculeTypeColors, props.saturation, props.lightness);
    function color(location) {
        if (structure_1.StructureElement.isLocation(location)) {
            return moleculeTypeColor(colorMap, location.unit, location.element);
        }
        else if (structure_1.Link.isLocation(location)) {
            return moleculeTypeColor(colorMap, location.aUnit, location.aUnit.elements[location.aIndex]);
        }
        return DefaultMoleculeTypeColor;
    }
    return {
        factory: MoleculeTypeColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: Description,
        legend: tables_1.TableLegend(Object.keys(exports.MoleculeTypeColors).map(function (name) {
            return [name, exports.MoleculeTypeColors[name]];
        }).concat([['Other/unknown', DefaultMoleculeTypeColor]]))
    };
}
exports.MoleculeTypeColorTheme = MoleculeTypeColorTheme;
exports.MoleculeTypeColorThemeProvider = {
    label: 'Molecule Type',
    factory: MoleculeTypeColorTheme,
    getParams: getMoleculeTypeColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.MoleculeTypeColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
