"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../mol-model/structure");
var color_1 = require("../../mol-util/color");
var param_definition_1 = require("../../mol-util/param-definition");
var util_1 = require("./util");
var DefaultColor = color_1.Color(0xCCCCCC);
var Description = 'Gives every polymer chain a color based on its `asym_id` value.';
exports.PolymerIdColorThemeParams = __assign({}, util_1.getPaletteParams({ scaleList: 'RedYellowBlue' }));
function getPolymerIdColorThemeParams(ctx) {
    return exports.PolymerIdColorThemeParams; // TODO return copy
}
exports.getPolymerIdColorThemeParams = getPolymerIdColorThemeParams;
function getAsymId(unit) {
    switch (unit.kind) {
        case 0 /* Atomic */:
            return structure_1.StructureProperties.chain.label_asym_id;
        case 1 /* Spheres */:
        case 2 /* Gaussians */:
            return structure_1.StructureProperties.coarse.asym_id;
    }
}
function addPolymerAsymIds(map, asymId, entityId, entities) {
    var j = map.size;
    for (var o = 0, ol = asymId.rowCount; o < ol; ++o) {
        var e = entityId.value(o);
        var eI = entities.getEntityIndex(e);
        if (entities.data.type.value(eI) === 'polymer') {
            var k = asymId.value(o);
            if (!map.has(k)) {
                map.set(k, j);
                j += 1;
            }
        }
    }
}
function PolymerIdColorTheme(ctx, props) {
    var color;
    var legend;
    if (ctx.structure) {
        // TODO same asym ids in different models should get different color
        var l_1 = structure_1.StructureElement.create();
        var models = ctx.structure.models;
        var polymerAsymIdSerialMap_1 = new Map();
        for (var i = 0, il = models.length; i < il; ++i) {
            var m = models[i];
            addPolymerAsymIds(polymerAsymIdSerialMap_1, m.atomicHierarchy.chains.label_asym_id, m.atomicHierarchy.chains.label_entity_id, m.entities);
            if (m.coarseHierarchy.isDefined) {
                addPolymerAsymIds(polymerAsymIdSerialMap_1, m.coarseHierarchy.spheres.asym_id, m.coarseHierarchy.spheres.entity_id, m.entities);
                addPolymerAsymIds(polymerAsymIdSerialMap_1, m.coarseHierarchy.gaussians.asym_id, m.coarseHierarchy.spheres.entity_id, m.entities);
            }
        }
        var palette_1 = util_1.getPalette(polymerAsymIdSerialMap_1.size, props);
        legend = palette_1.legend;
        color = function (location) {
            if (structure_1.StructureElement.isLocation(location)) {
                var asym_id = getAsymId(location.unit);
                return palette_1.color(polymerAsymIdSerialMap_1.get(asym_id(location)) || 0);
            }
            else if (structure_1.Link.isLocation(location)) {
                var asym_id = getAsymId(location.aUnit);
                l_1.unit = location.aUnit;
                l_1.element = location.aUnit.elements[location.aIndex];
                return palette_1.color(polymerAsymIdSerialMap_1.get(asym_id(l_1)) || 0);
            }
            return DefaultColor;
        };
    }
    else {
        color = function () { return DefaultColor; };
    }
    return {
        factory: PolymerIdColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: Description,
        legend: legend
    };
}
exports.PolymerIdColorTheme = PolymerIdColorTheme;
exports.PolymerIdColorThemeProvider = {
    label: 'Polymer Id',
    factory: PolymerIdColorTheme,
    getParams: getPolymerIdColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.PolymerIdColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
