"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = require("../../mol-util/color");
var structure_1 = require("../../mol-model/structure");
var param_definition_1 = require("../../mol-util/param-definition");
var util_1 = require("./util");
var DefaultColor = color_1.Color(0xCCCCCC);
var Description = 'Gives every polymer a unique color based on the position (index) of the polymer in the list of polymers in the structure.';
exports.PolymerIndexColorThemeParams = __assign({}, util_1.getPaletteParams({ scaleList: 'RedYellowBlue' }));
function getPolymerIndexColorThemeParams(ctx) {
    return exports.PolymerIndexColorThemeParams; // TODO return copy
}
exports.getPolymerIndexColorThemeParams = getPolymerIndexColorThemeParams;
function PolymerIndexColorTheme(ctx, props) {
    var color;
    var legend;
    if (ctx.structure) {
        var units = ctx.structure.units;
        var polymerCount = 0;
        for (var i = 0, il = units.length; i < il; ++i) {
            if (units[i].polymerElements.length > 0)
                ++polymerCount;
        }
        var palette = util_1.getPalette(polymerCount, props);
        legend = palette.legend;
        var unitIdColor_1 = new Map();
        for (var i = 0, j = 0, il = units.length; i < il; ++i) {
            if (units[i].polymerElements.length > 0) {
                unitIdColor_1.set(units[i].id, palette.color(j));
                ++j;
            }
        }
        color = function (location) {
            var color;
            if (structure_1.StructureElement.isLocation(location)) {
                color = unitIdColor_1.get(location.unit.id);
            }
            else if (structure_1.Link.isLocation(location)) {
                color = unitIdColor_1.get(location.aUnit.id);
            }
            return color !== undefined ? color : DefaultColor;
        };
    }
    else {
        color = function () { return DefaultColor; };
    }
    return {
        factory: PolymerIndexColorTheme,
        granularity: 'instance',
        color: color,
        props: props,
        description: Description,
        legend: legend
    };
}
exports.PolymerIndexColorTheme = PolymerIndexColorTheme;
exports.PolymerIndexColorThemeProvider = {
    label: 'Polymer Index',
    factory: PolymerIndexColorTheme,
    getParams: getPolymerIndexColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.PolymerIndexColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
