"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = require("../../mol-util/color");
var structure_1 = require("../../mol-model/structure");
var param_definition_1 = require("../../mol-util/param-definition");
var util_1 = require("./util");
var DefaultColor = color_1.Color(0xCCCCCC);
var Description = 'Gives every unit (single chain or collection of single elements) a unique color based on the position (index) of the unit in the list of units in the structure.';
exports.UnitIndexColorThemeParams = __assign({}, util_1.getPaletteParams({ scaleList: 'RedYellowBlue' }));
function getUnitIndexColorThemeParams(ctx) {
    return exports.UnitIndexColorThemeParams; // TODO return copy
}
exports.getUnitIndexColorThemeParams = getUnitIndexColorThemeParams;
function UnitIndexColorTheme(ctx, props) {
    var color;
    var legend;
    if (ctx.structure) {
        var units = ctx.structure.units;
        var palette = util_1.getPalette(units.length, props);
        legend = palette.legend;
        var unitIdColor_1 = new Map();
        for (var i = 0, il = units.length; i < il; ++i) {
            unitIdColor_1.set(units[i].id, palette.color(i));
        }
        color = function (location) {
            if (structure_1.StructureElement.isLocation(location)) {
                return unitIdColor_1.get(location.unit.id);
            }
            else if (structure_1.Link.isLocation(location)) {
                return unitIdColor_1.get(location.aUnit.id);
            }
            return DefaultColor;
        };
    }
    else {
        color = function () { return DefaultColor; };
    }
    return {
        factory: UnitIndexColorTheme,
        granularity: 'instance',
        color: color,
        props: props,
        description: Description,
        legend: legend
    };
}
exports.UnitIndexColorTheme = UnitIndexColorTheme;
exports.UnitIndexColorThemeProvider = {
    label: 'Unit Index',
    factory: UnitIndexColorTheme,
    getParams: getUnitIndexColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.UnitIndexColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
