"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var param_definition_1 = require("../../mol-util/param-definition");
var distinct_1 = require("../../mol-util/color/distinct");
var scale_1 = require("../../mol-util/color/scale");
var DefaultGetPaletteProps = {
    scaleList: 'RedYellowBlue'
};
function getPaletteParams(props) {
    if (props === void 0) { props = {}; }
    var p = __assign({}, DefaultGetPaletteProps, { props: props });
    return {
        palette: param_definition_1.ParamDefinition.MappedStatic('generate', {
            scale: param_definition_1.ParamDefinition.Group({
                list: param_definition_1.ParamDefinition.ColorScale(p.scaleList, scale_1.ColorListOptions),
            }, { isFlat: true }),
            generate: param_definition_1.ParamDefinition.Group(distinct_1.DistinctColorsParams, { isFlat: true })
        }, {
            options: [
                ['scale', 'From Scale'],
                ['generate', 'Generate Distinct']
            ]
        })
    };
}
exports.getPaletteParams = getPaletteParams;
var DefaultPaletteProps = param_definition_1.ParamDefinition.getDefaultValues(getPaletteParams());
function getPalette(count, props) {
    var color;
    var legend;
    if (props.palette.name === 'scale') {
        var listOrName = props.palette.params.list;
        var domain = [0, count - 1];
        var scale = scale_1.ColorScale.create({ listOrName: listOrName, domain: domain, minLabel: 'Start', maxLabel: 'End' });
        legend = scale.legend;
        color = scale.color;
    }
    else {
        var colors_1 = distinct_1.distinctColors(count, props.palette.params);
        color = function (i) { return colors_1[i]; };
    }
    return { color: color, legend: legend };
}
exports.getPalette = getPalette;
