"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../mol-model/structure");
var int_1 = require("../mol-data/int");
// for `labelFirst`, don't create right away to avoid problems with circular dependencies/imports
var elementLocA;
var elementLocB;
function setElementLocation(loc, unit, index) {
    loc.unit = unit;
    loc.element = unit.elements[index];
}
function labelFirst(loci) {
    switch (loci.kind) {
        case 'structure-loci':
            return loci.structure.models.map(function (m) { return m.label; }).join(', ');
        case 'element-loci':
            var e = loci.elements[0];
            if (e) {
                var el = e.unit.elements[int_1.OrderedSet.getAt(e.indices, 0)];
                return elementLabel(structure_1.StructureElement.create(e.unit, el));
            }
            else {
                return 'Unknown';
            }
        case 'link-loci':
            var link = loci.links[0];
            return link ? linkLabel(link) : 'Unknown';
        case 'shape-loci':
            return loci.shape.name;
        case 'group-loci':
            var g = loci.groups[0];
            if (g) {
                return loci.shape.getLabel(int_1.OrderedSet.getAt(g.ids, 0), loci.instance);
            }
            else {
                return 'Unknown';
            }
        case 'every-loci':
            return 'Everything';
        case 'empty-loci':
            return 'Nothing';
        case 'data-loci':
            return '';
    }
}
exports.labelFirst = labelFirst;
function linkLabel(link) {
    if (!elementLocA)
        elementLocA = structure_1.StructureElement.create();
    if (!elementLocB)
        elementLocB = structure_1.StructureElement.create();
    setElementLocation(elementLocA, link.aUnit, link.aIndex);
    setElementLocation(elementLocB, link.bUnit, link.bIndex);
    return elementLabel(elementLocA) + " - " + elementLabel(elementLocB);
}
exports.linkLabel = linkLabel;
function elementLabel(location) {
    var model = location.unit.model.label;
    var instance = location.unit.conformation.operator.name;
    var label = '';
    if (structure_1.Unit.isAtomic(location.unit)) {
        var asym_id = structure_1.StructureProperties.chain.auth_asym_id(location);
        var seq_id = location.unit.model.atomicHierarchy.residues.auth_seq_id.isDefined ? structure_1.StructureProperties.residue.auth_seq_id(location) : structure_1.StructureProperties.residue.label_seq_id(location);
        var comp_id = structure_1.StructureProperties.residue.label_comp_id(location);
        var atom_id = structure_1.StructureProperties.atom.label_atom_id(location);
        var alt_id = structure_1.StructureProperties.atom.label_alt_id(location);
        label = "[" + comp_id + "]" + seq_id + ":" + asym_id + "." + atom_id + (alt_id ? "%" + alt_id : '');
    }
    else if (structure_1.Unit.isCoarse(location.unit)) {
        var asym_id = structure_1.StructureProperties.coarse.asym_id(location);
        var seq_id_begin = structure_1.StructureProperties.coarse.seq_id_begin(location);
        var seq_id_end = structure_1.StructureProperties.coarse.seq_id_end(location);
        if (seq_id_begin === seq_id_end) {
            var entityIndex = structure_1.StructureProperties.coarse.entityKey(location);
            var seq = location.unit.model.sequence.byEntityKey[entityIndex];
            var comp_id = seq.compId.value(seq_id_begin - 1); // 1-indexed
            label = "[" + comp_id + "]" + seq_id_begin + ":" + asym_id;
        }
        else {
            label = seq_id_begin + "-" + seq_id_end + ":" + asym_id;
        }
    }
    else {
        label = 'unknown';
    }
    return model + " " + instance + " " + label;
}
exports.elementLabel = elementLabel;
