/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { SizeType, LocationSize } from '../mol-geo/geometry/size-data';
import { ParamDefinition as PD } from '../mol-util/param-definition';
import { ThemeDataContext, ThemeRegistry, ThemeProvider } from '../mol-theme/theme';
export { SizeTheme };
interface SizeTheme<P extends PD.Params> {
    readonly factory: SizeTheme.Factory<P>;
    readonly granularity: SizeType;
    readonly size: LocationSize;
    readonly props: Readonly<PD.Values<P>>;
    readonly description?: string;
}
declare namespace SizeTheme {
    type Props = {
        [k: string]: any;
    };
    type Factory<P extends PD.Params> = (ctx: ThemeDataContext, props: PD.Values<P>) => SizeTheme<P>;
    const EmptyFactory: () => SizeTheme<{}>;
    const Empty: SizeTheme<{}>;
    function areEqual(themeA: SizeTheme<any>, themeB: SizeTheme<any>): boolean;
    interface Provider<P extends PD.Params> extends ThemeProvider<SizeTheme<P>, P> {
    }
    const EmptyProvider: Provider<{}>;
    type Registry = ThemeRegistry<SizeTheme<any>>;
    function createRegistry(): ThemeRegistry<SizeTheme<any>>;
    type ParamValues<C extends SizeTheme.Provider<any>> = C extends SizeTheme.Provider<infer P> ? PD.Values<P> : never;
}
export declare const BuiltInSizeThemes: {
    'physical': SizeTheme.Provider<{}>;
    'shape-group': SizeTheme.Provider<{}>;
    'uncertainty': SizeTheme.Provider<{
        bfactorFactor: PD.Numeric;
        rmsfFactor: PD.Numeric;
        baseSize: PD.Numeric;
    }>;
    'uniform': SizeTheme.Provider<{
        value: PD.Numeric;
    }>;
};
export declare type BuiltInSizeThemeName = keyof typeof BuiltInSizeThemes;
