/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { ColorTheme } from './color';
import { SizeTheme } from './size';
import { Structure } from '../mol-model/structure';
import { VolumeData } from '../mol-model/volume';
import { ParamDefinition as PD } from '../mol-util/param-definition';
import { Shape } from '../mol-model/shape';
export interface ThemeRegistryContext {
    colorThemeRegistry: ColorTheme.Registry;
    sizeThemeRegistry: SizeTheme.Registry;
}
export interface ThemeDataContext {
    [k: string]: any;
    structure?: Structure;
    volume?: VolumeData;
    shape?: Shape;
}
export interface Theme {
    color: ColorTheme<any>;
    size: SizeTheme<any>;
}
declare type Props = {
    [k: string]: any;
};
export declare function createTheme(ctx: ThemeRegistryContext, data: ThemeDataContext, props: Props, theme?: Theme): Theme;
export declare function createEmptyTheme(): Theme;
export interface ThemeProvider<T extends ColorTheme<P> | SizeTheme<P>, P extends PD.Params> {
    readonly label: string;
    readonly factory: (ctx: ThemeDataContext, props: PD.Values<P>) => T;
    readonly getParams: (ctx: ThemeDataContext) => P;
    readonly defaultValues: PD.Values<P>;
    readonly isApplicable: (ctx: ThemeDataContext) => boolean;
}
export declare class ThemeRegistry<T extends ColorTheme<any> | SizeTheme<any>> {
    private emptyProvider;
    private _list;
    private _map;
    private _name;
    readonly default: {
        name: string;
        provider: ThemeProvider<T, any>;
    };
    readonly list: {
        name: string;
        provider: ThemeProvider<T, any>;
    }[];
    readonly types: [string, string][];
    constructor(builtInThemes: {
        [k: string]: ThemeProvider<T, any>;
    }, emptyProvider: ThemeProvider<T, any>);
    add<P extends PD.Params>(name: string, provider: ThemeProvider<T, P>): void;
    remove(name: string): void;
    get<P extends PD.Params>(name: string): ThemeProvider<T, P>;
    getName(provider: ThemeProvider<T, any>): string;
    create(name: string, ctx: ThemeDataContext, props?: {}): T;
    getApplicableList(ctx: ThemeDataContext): {
        name: string;
        provider: ThemeProvider<T, any>;
    }[];
    getApplicableTypes(ctx: ThemeDataContext): [string, string][];
}
export {};
