"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = require("./color");
var size_1 = require("./size");
var param_definition_1 = require("../mol-util/param-definition");
function createTheme(ctx, data, props, theme) {
    theme = theme || createEmptyTheme();
    var colorProps = props.colorTheme;
    var sizeProps = props.sizeTheme;
    theme.color = ctx.colorThemeRegistry.create(colorProps.name, data, colorProps.params);
    theme.size = ctx.sizeThemeRegistry.create(sizeProps.name, data, sizeProps.params);
    return theme;
}
exports.createTheme = createTheme;
function createEmptyTheme() {
    return { color: color_1.ColorTheme.Empty, size: size_1.SizeTheme.Empty };
}
exports.createEmptyTheme = createEmptyTheme;
function getTypes(list) {
    return list.map(function (e) { return [e.name, e.provider.label]; });
}
var ThemeRegistry = /** @class */ (function () {
    function ThemeRegistry(builtInThemes, emptyProvider) {
        var _this = this;
        this.emptyProvider = emptyProvider;
        this._list = [];
        this._map = new Map();
        this._name = new Map();
        Object.keys(builtInThemes).forEach(function (name) { return _this.add(name, builtInThemes[name]); });
    }
    Object.defineProperty(ThemeRegistry.prototype, "default", {
        get: function () { return this._list[0]; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ThemeRegistry.prototype, "list", {
        get: function () { return this._list; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ThemeRegistry.prototype, "types", {
        get: function () { return getTypes(this._list); },
        enumerable: true,
        configurable: true
    });
    ThemeRegistry.prototype.add = function (name, provider) {
        this._list.push({ name: name, provider: provider });
        this._map.set(name, provider);
        this._name.set(provider, name);
    };
    ThemeRegistry.prototype.remove = function (name) {
        this._list.splice(this._list.findIndex(function (e) { return e.name === name; }), 1);
        var p = this._map.get(name);
        if (p) {
            this._map.delete(name);
            this._name.delete(p);
        }
    };
    ThemeRegistry.prototype.get = function (name) {
        return this._map.get(name) || this.emptyProvider;
    };
    ThemeRegistry.prototype.getName = function (provider) {
        if (!this._name.has(provider))
            throw new Error("'" + provider.label + "' is not a registered represenatation provider.");
        return this._name.get(provider);
    };
    ThemeRegistry.prototype.create = function (name, ctx, props) {
        if (props === void 0) { props = {}; }
        var provider = this.get(name);
        return provider.factory(ctx, __assign({}, param_definition_1.ParamDefinition.getDefaultValues(provider.getParams(ctx)), props));
    };
    ThemeRegistry.prototype.getApplicableList = function (ctx) {
        return this._list.filter(function (e) { return e.provider.isApplicable(ctx); });
    };
    ThemeRegistry.prototype.getApplicableTypes = function (ctx) {
        return getTypes(this.getApplicableList(ctx));
    };
    return ThemeRegistry;
}());
exports.ThemeRegistry = ThemeRegistry;
