"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
// TODO move to mol-math as Vector???
/** Get the maximum value in an array */
function arrayMax(array) {
    var max = -Infinity;
    for (var i = 0, il = array.length; i < il; ++i) {
        if (array[i] > max)
            max = array[i];
    }
    return max;
}
exports.arrayMax = arrayMax;
/** Get the minimum value in an array */
function arrayMin(array) {
    var min = Infinity;
    for (var i = 0, il = array.length; i < il; ++i) {
        if (array[i] < min)
            min = array[i];
    }
    return min;
}
exports.arrayMin = arrayMin;
/** Get the sum of values in an array */
function arraySum(array, stride, offset) {
    if (stride === void 0) { stride = 1; }
    if (offset === void 0) { offset = 0; }
    var n = array.length;
    var sum = 0;
    for (var i = offset; i < n; i += stride) {
        sum += array[i];
    }
    return sum;
}
exports.arraySum = arraySum;
/** Get the mean of values in an array */
function arrayMean(array, stride, offset) {
    if (stride === void 0) { stride = 1; }
    if (offset === void 0) { offset = 0; }
    return arraySum(array, stride, offset) / (array.length / stride);
}
exports.arrayMean = arrayMean;
/** Get the root mean square of values in an array */
function arrayRms(array) {
    var n = array.length;
    var sumSq = 0;
    for (var i = 0; i < n; ++i) {
        var di = array[i];
        sumSq += di * di;
    }
    return Math.sqrt(sumSq / n);
}
exports.arrayRms = arrayRms;
/** Fill an array with serial numbers starting from 0 until n - 1 (defaults to array.length) */
function fillSerial(array, n) {
    for (var i = 0, il = n ? Math.min(n, array.length) : array.length; i < il; ++i)
        array[i] = i;
    return array;
}
exports.fillSerial = fillSerial;
/** Fill an array with value starting from index 0 until n - 1 (defaults to array.length) */
function fillUniform(array, v, n) {
    for (var i = 0, il = n ? Math.min(n, array.length) : array.length; i < il; ++i)
        array[i] = v;
    return array;
}
exports.fillUniform = fillUniform;
function arrayRemoveInPlace(xs, x) {
    var i = 0, l = xs.length, found = false;
    for (; i < l; i++) {
        if (xs[i] === x) {
            found = true;
            break;
        }
    }
    if (!found)
        return false;
    i++;
    for (; i < l; i++) {
        xs[i - 1] = xs[i];
    }
    xs.pop();
    return true;
}
exports.arrayRemoveInPlace = arrayRemoveInPlace;
