/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Color } from './color';
import { ColorBrewer, ColorMatplotlib, ColorOther } from './tables';
import { NumberArray } from '../../mol-util/type-helpers';
export declare type ColorListName = (keyof typeof ColorBrewer | keyof typeof ColorMatplotlib | keyof typeof ColorOther);
export declare const ColorListNames: string[];
export declare const ColorListOptions: ["OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue", string][];
export declare function getColorListFromName(name: ColorListName): Color[];
export interface ScaleLegend {
    kind: 'scale-legend';
    minLabel: string;
    maxLabel: string;
    colors: Color[];
}
export declare function ScaleLegend(minLabel: string, maxLabel: string, colors: Color[]): ScaleLegend;
export interface ColorScale {
    /** Returns hex color for given value */
    color: (value: number) => Color;
    /** Copies color to rgb int8 array */
    colorToArray: (value: number, array: NumberArray, offset: number) => void;
    /** Copies normalized (0 to 1) hex color to rgb array */
    normalizedColorToArray: (value: number, array: NumberArray, offset: number) => void;
    /**  */
    setDomain: (min: number, max: number) => void;
    /** Legend */
    readonly legend: ScaleLegend;
}
export declare const DefaultColorScaleProps: {
    domain: [number, number];
    reverse: boolean;
    listOrName: "OrangeRed" | "PurpleBlue" | "BluePurple" | "Oranges" | "BlueGreen" | "YellowOrangeBrown" | "YellowGreen" | "Reds" | "RedPurple" | "Greens" | "YellowGreenBlue" | "Purples" | "GreenBlue" | "Greys" | "YellowOrangeRed" | "PurpleRed" | "Blues" | "PurpleBlueGreen" | "Spectral" | "RedYellowGreen" | "RedBlue" | "PinkYellowGreen" | "PurpleGreen" | "RedYellowBlue" | "BrownWhiteGreen" | "RedGrey" | "PurpleOrange" | "Set2" | "Accent" | "Set1" | "Set3" | "Dark2" | "Paired" | "Pastel2" | "Pastel1" | Color[] | "Magma" | "Inferno" | "Plasma" | "Viridis" | "Cividis" | "Twilight" | "Rainbow" | "RedWhiteBlue";
    minLabel: string | undefined;
    maxLabel: string | undefined;
};
export declare type ColorScaleProps = Partial<typeof DefaultColorScaleProps>;
export declare namespace ColorScale {
    function create(props: ColorScaleProps): ColorScale;
}
