"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = require("./color");
var tables_1 = require("./tables");
var mol_util_1 = require("../../mol-util");
exports.ColorListNames = Object.keys(tables_1.ColorBrewer).concat(Object.keys(tables_1.ColorMatplotlib), Object.keys(tables_1.ColorOther));
exports.ColorListOptions = exports.ColorListNames.map(function (n) { return [n, n]; });
function getColorListFromName(name) {
    if (name in tables_1.ColorBrewer) {
        return tables_1.ColorBrewer[name];
    }
    else if (name in tables_1.ColorMatplotlib) {
        return tables_1.ColorMatplotlib[name];
    }
    else if (name in tables_1.ColorOther) {
        return tables_1.ColorOther[name];
    }
    console.warn("unknown color list named '" + name + "'");
    return tables_1.ColorBrewer.RedYellowBlue;
}
exports.getColorListFromName = getColorListFromName;
function ScaleLegend(minLabel, maxLabel, colors) {
    return { kind: 'scale-legend', minLabel: minLabel, maxLabel: maxLabel, colors: colors };
}
exports.ScaleLegend = ScaleLegend;
exports.DefaultColorScaleProps = {
    domain: [0, 1],
    reverse: false,
    listOrName: tables_1.ColorBrewer.RedYellowBlue,
    minLabel: '',
    maxLabel: '',
};
var ColorScale;
(function (ColorScale) {
    function create(props) {
        var _a = __assign({}, exports.DefaultColorScaleProps, props), domain = _a.domain, reverse = _a.reverse, listOrName = _a.listOrName;
        var list = typeof listOrName === 'string' ? getColorListFromName(listOrName) : listOrName;
        var colors = reverse ? list.slice().reverse() : list;
        var count1 = colors.length - 1;
        var diff = 0, min = 0, max = 0;
        function setDomain(_min, _max) {
            min = _min;
            max = _max;
            diff = (max - min) || 1;
        }
        setDomain(domain[0], domain[1]);
        var minLabel = mol_util_1.defaults(props.minLabel, min.toString());
        var maxLabel = mol_util_1.defaults(props.maxLabel, max.toString());
        function color(value) {
            var t = Math.min(colors.length - 1, Math.max(0, ((value - min) / diff) * count1));
            var tf = Math.floor(t);
            var c1 = colors[tf];
            var c2 = colors[Math.ceil(t)];
            return color_1.Color.interpolate(c1, c2, t - tf);
        }
        return {
            color: color,
            colorToArray: function (value, array, offset) {
                color_1.Color.toArray(color(value), array, offset);
            },
            normalizedColorToArray: function (value, array, offset) {
                color_1.Color.toArrayNormalized(color(value), array, offset);
            },
            setDomain: setDomain,
            get legend() { return ScaleLegend(minLabel, maxLabel, colors); }
        };
    }
    ColorScale.create = create;
})(ColorScale = exports.ColorScale || (exports.ColorScale = {}));
