/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 *
 * Adapted from LiteMol
 */
import { Task, RuntimeContext } from '../mol-task';
export interface AjaxGetParams<T extends 'string' | 'binary' | 'json' = 'string'> {
    url: string;
    type?: T;
    title?: string;
    body?: string;
}
export declare function readStringFromFile(file: File): Task<string>;
export declare function readUint8ArrayFromFile(file: File): Task<Uint8Array>;
export declare function readFromFile(file: File, type: 'string' | 'binary'): Task<string | Uint8Array>;
export declare function ajaxGet(url: string): Task<string>;
export declare function ajaxGet(params: AjaxGetParams<'string'>): Task<string>;
export declare function ajaxGet(params: AjaxGetParams<'binary'>): Task<Uint8Array>;
export declare function ajaxGet<T = any>(params: AjaxGetParams<'json'>): Task<T>;
export declare function ajaxGet(params: AjaxGetParams<'string' | 'binary'>): Task<string | Uint8Array>;
export declare function ajaxGet(params: AjaxGetParams<'string' | 'binary' | 'json'>): Task<string | Uint8Array | object>;
export declare type AjaxTask = typeof ajaxGet;
export declare type AjaxGetManyEntry<T> = {
    kind: 'ok';
    id: string;
    result: T;
} | {
    kind: 'error';
    id: string;
    error: any;
};
export declare function ajaxGetMany(ctx: RuntimeContext, sources: {
    id: string;
    url: string;
    isBinary?: boolean;
    canFail?: boolean;
}[], maxConcurrency: number): Promise<AjaxGetManyEntry<string | Uint8Array>[]>;
