"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var interpolate_1 = require("../mol-math/interpolate");
var maxFloat = 10000.0; // NOTE same constant is set in shaders
var floatLogFactor = Math.log(maxFloat + 1.0);
/** encode float logarithmically */
function encodeFloatLog(value) { return Math.log(value + 1.0) / floatLogFactor; }
exports.encodeFloatLog = encodeFloatLog;
/** decode logarithmically encoded float */
function decodeFloatLog(value) { return Math.exp(value * floatLogFactor) - 1.0; }
exports.decodeFloatLog = decodeFloatLog;
/** encode float as normalized rgb triplet */
function encodeFloatRGB(value) {
    value = interpolate_1.clamp(value, 0.0, 16777216.0 - 1.0) + 1.0;
    var b = (value % 256) / 255.0;
    value = Math.floor(value / 256.0);
    var g = (value % 256) / 255.0;
    value = Math.floor(value / 256.0);
    var r = (value % 256) / 255.0;
    return [r, g, b];
}
exports.encodeFloatRGB = encodeFloatRGB;
/** decode float encoded as rgb triplet */
function decodeFloatRGB(r, g, b) {
    return (Math.floor(r) * 256 * 256 + Math.floor(g) * 256 + Math.floor(b)) - 1;
}
exports.decodeFloatRGB = decodeFloatRGB;
