"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var linear_algebra_1 = require("../../mol-math/linear-algebra");
var mol_util_1 = require("../../mol-util");
function getButtons(event) {
    if (typeof event === 'object') {
        if ('buttons' in event) {
            return event.buttons;
        }
        else if ('which' in event) {
            var b = event.which; // 'any' to support older browsers
            if (b === 2) {
                return 4;
            }
            else if (b === 3) {
                return 2;
            }
            else if (b > 0) {
                return 1 << (b - 1);
            }
        }
        else if ('button' in event) {
            var b = event.button; // 'any' to support older browsers
            if (b === 1) {
                return 4;
            }
            else if (b === 2) {
                return 2;
            }
            else if (b >= 0) {
                return 1 << b;
            }
        }
    }
    return 0;
}
exports.getButtons = getButtons;
function getModifiers(event) {
    return {
        alt: 'altKey' in event ? event.altKey : false,
        shift: 'shiftKey' in event ? event.shiftKey : false,
        control: 'ctrlKey' in event ? event.ctrlKey : false,
        meta: 'metaKey' in event ? event.metaKey : false
    };
}
exports.getModifiers = getModifiers;
exports.DefaultInputObserverProps = {
    noScroll: true,
    noContextMenu: true,
    noPinchZoom: true
};
var ModifiersKeys;
(function (ModifiersKeys) {
    ModifiersKeys.None = { shift: false, alt: false, control: false, meta: false };
    function areEqual(a, b) {
        return a.shift === b.shift && a.alt === b.alt && a.control === b.control && a.meta === b.meta;
    }
    ModifiersKeys.areEqual = areEqual;
})(ModifiersKeys = exports.ModifiersKeys || (exports.ModifiersKeys = {}));
var ButtonsType;
(function (ButtonsType) {
    ButtonsType.has = mol_util_1.BitFlags.has;
    ButtonsType.create = mol_util_1.BitFlags.create;
})(ButtonsType = exports.ButtonsType || (exports.ButtonsType = {}));
function createEvents() {
    return {
        drag: new rxjs_1.Subject(),
        interactionEnd: new rxjs_1.Subject(),
        click: new rxjs_1.Subject(),
        move: new rxjs_1.Subject(),
        wheel: new rxjs_1.Subject(),
        pinch: new rxjs_1.Subject(),
        resize: new rxjs_1.Subject(),
        leave: new rxjs_1.Subject(),
        enter: new rxjs_1.Subject(),
        modifiers: new rxjs_1.Subject(),
    };
}
var InputObserver;
(function (InputObserver) {
    function create(props) {
        if (props === void 0) { props = {}; }
        var _a = __assign({}, exports.DefaultInputObserverProps, props), noScroll = _a.noScroll, noContextMenu = _a.noContextMenu;
        return __assign({ noScroll: noScroll,
            noContextMenu: noContextMenu }, createEvents(), { dispose: mol_util_1.noop });
    }
    InputObserver.create = create;
    function fromElement(element, props) {
        if (props === void 0) { props = {}; }
        var _a = __assign({}, exports.DefaultInputObserverProps, props), noScroll = _a.noScroll, noContextMenu = _a.noContextMenu, noPinchZoom = _a.noPinchZoom;
        var lastTouchDistance = 0;
        var pointerDown = linear_algebra_1.Vec2.zero();
        var pointerStart = linear_algebra_1.Vec2.zero();
        var pointerEnd = linear_algebra_1.Vec2.zero();
        var pointerDelta = linear_algebra_1.Vec2.zero();
        var rectSize = linear_algebra_1.Vec2.zero();
        var modifierKeys = {
            shift: false,
            alt: false,
            control: false,
            meta: false
        };
        function getModifierKeys() {
            return __assign({}, modifierKeys);
        }
        var dragging = 0 /* Stopped */;
        var disposed = false;
        var buttons = 0;
        var isInside = false;
        var events = createEvents();
        var drag = events.drag, interactionEnd = events.interactionEnd, wheel = events.wheel, pinch = events.pinch, click = events.click, move = events.move, leave = events.leave, enter = events.enter, resize = events.resize, modifiers = events.modifiers;
        attach();
        return __assign({ get noScroll() { return noScroll; },
            set noScroll(value) { noScroll = value; },
            get noContextMenu() { return noContextMenu; },
            set noContextMenu(value) { noContextMenu = value; } }, events, { dispose: dispose });
        function attach() {
            element.addEventListener('contextmenu', onContextMenu, false);
            element.addEventListener('wheel', onMouseWheel, false);
            element.addEventListener('mousedown', onMouseDown, false);
            // for dragging to work outside canvas bounds,
            // mouse move/up events have to be added to a parent, i.e. window
            window.addEventListener('mousemove', onMouseMove, false);
            window.addEventListener('mouseup', onMouseUp, false);
            element.addEventListener('mouseenter', onMouseEnter, false);
            element.addEventListener('mouseleave', onMouseLeave, false);
            element.addEventListener('touchstart', onTouchStart, false);
            element.addEventListener('touchmove', onTouchMove, false);
            element.addEventListener('touchend', onTouchEnd, false);
            element.addEventListener('blur', handleBlur);
            window.addEventListener('keyup', handleKeyUp, false);
            window.addEventListener('keydown', handleKeyDown, false);
            window.addEventListener('resize', onResize, false);
        }
        function dispose() {
            if (disposed)
                return;
            disposed = true;
            element.removeEventListener('contextmenu', onContextMenu, false);
            element.removeEventListener('wheel', onMouseWheel, false);
            element.removeEventListener('mousedown', onMouseDown, false);
            window.removeEventListener('mousemove', onMouseMove, false);
            window.removeEventListener('mouseup', onMouseUp, false);
            element.removeEventListener('mouseenter', onMouseEnter, false);
            element.removeEventListener('mouseleave', onMouseLeave, false);
            element.removeEventListener('touchstart', onTouchStart, false);
            element.removeEventListener('touchmove', onTouchMove, false);
            element.removeEventListener('touchend', onTouchEnd, false);
            element.removeEventListener('blur', handleBlur);
            window.removeEventListener('keyup', handleKeyUp, false);
            window.removeEventListener('keydown', handleKeyDown, false);
            window.removeEventListener('resize', onResize, false);
        }
        function onContextMenu(event) {
            if (noContextMenu) {
                event.preventDefault();
            }
        }
        function handleBlur() {
            if (buttons || modifierKeys.shift || modifierKeys.alt || modifierKeys.meta || modifierKeys.control) {
                buttons = 0;
                modifierKeys.shift = modifierKeys.alt = modifierKeys.control = modifierKeys.meta = false;
            }
        }
        function handleKeyDown(event) {
            var changed = false;
            if (!modifierKeys.alt && event.altKey) {
                changed = true;
                modifierKeys.alt = true;
            }
            if (!modifierKeys.shift && event.shiftKey) {
                changed = true;
                modifierKeys.shift = true;
            }
            if (!modifierKeys.control && event.ctrlKey) {
                changed = true;
                modifierKeys.control = true;
            }
            if (!modifierKeys.meta && event.metaKey) {
                changed = true;
                modifierKeys.meta = true;
            }
            if (changed && isInside)
                modifiers.next(getModifierKeys());
        }
        function handleKeyUp(event) {
            var changed = false;
            if (modifierKeys.alt && !event.altKey) {
                changed = true;
                modifierKeys.alt = false;
            }
            if (modifierKeys.shift && !event.shiftKey) {
                changed = true;
                modifierKeys.shift = false;
            }
            if (modifierKeys.control && !event.ctrlKey) {
                changed = true;
                modifierKeys.control = false;
            }
            if (modifierKeys.meta && !event.metaKey) {
                changed = true;
                modifierKeys.meta = false;
            }
            if (changed && isInside)
                modifiers.next(getModifierKeys());
        }
        function getCenterTouch(ev) {
            var t0 = ev.touches[0];
            var t1 = ev.touches[1];
            return {
                clientX: (t0.clientX + t1.clientX) / 2,
                clientY: (t0.clientY + t1.clientY) / 2,
                pageX: (t0.pageX + t1.pageX) / 2,
                pageY: (t0.pageY + t1.pageY) / 2
            };
        }
        function getTouchDistance(ev) {
            var dx = ev.touches[0].pageX - ev.touches[1].pageX;
            var dy = ev.touches[0].pageY - ev.touches[1].pageY;
            return Math.sqrt(dx * dx + dy * dy);
        }
        function onTouchStart(ev) {
            if (ev.touches.length === 1) {
                buttons = 1 /* Primary */;
                onPointerDown(ev.touches[0]);
            }
            else if (ev.touches.length >= 2) {
                buttons = 2 /* Secondary */;
                onPointerDown(getCenterTouch(ev));
                var touchDistance = getTouchDistance(ev);
                lastTouchDistance = touchDistance;
                pinch.next({ distance: touchDistance, fraction: 1, delta: 0, isStart: true });
            }
        }
        function onTouchEnd(ev) {
            endDrag();
        }
        function onTouchMove(ev) {
            if (noPinchZoom) {
                ev.preventDefault();
                ev.stopPropagation();
                if (ev.originalEvent) {
                    ev.originalEvent.preventDefault();
                    ev.originalEvent.stopPropagation();
                }
            }
            if (ev.touches.length === 1) {
                buttons = 1 /* Primary */;
                onPointerMove(ev.touches[0]);
            }
            else if (ev.touches.length >= 2) {
                var touchDistance = getTouchDistance(ev);
                var touchDelta = lastTouchDistance - touchDistance;
                if (Math.abs(touchDelta) < 4) {
                    buttons = 2 /* Secondary */;
                    onPointerMove(getCenterTouch(ev));
                }
                else {
                    pinch.next({
                        delta: touchDelta,
                        fraction: lastTouchDistance / touchDistance,
                        distance: touchDistance,
                        isStart: false
                    });
                }
                lastTouchDistance = touchDistance;
            }
        }
        function onMouseDown(ev) {
            buttons = getButtons(ev);
            onPointerDown(ev);
        }
        function onMouseMove(ev) {
            buttons = getButtons(ev);
            onPointerMove(ev);
        }
        function onMouseUp(ev) {
            onPointerUp(ev);
            endDrag();
        }
        function endDrag() {
            interactionEnd.next();
        }
        function onPointerDown(ev) {
            eventOffset(pointerStart, ev);
            linear_algebra_1.Vec2.copy(pointerDown, pointerStart);
            if (insideBounds(pointerStart)) {
                dragging = 1 /* Started */;
            }
        }
        function onPointerUp(ev) {
            dragging = 0 /* Stopped */;
            eventOffset(pointerEnd, ev);
            if (linear_algebra_1.Vec2.distance(pointerEnd, pointerDown) < 4) {
                var pageX = ev.pageX, pageY = ev.pageY;
                var x = pointerEnd[0], y = pointerEnd[1];
                click.next({ x: x, y: y, pageX: pageX, pageY: pageY, buttons: buttons, modifiers: getModifierKeys() });
            }
        }
        function onPointerMove(ev) {
            eventOffset(pointerEnd, ev);
            var pageX = ev.pageX, pageY = ev.pageY;
            var x = pointerEnd[0], y = pointerEnd[1];
            var inside = insideBounds(pointerEnd);
            move.next({ x: x, y: y, pageX: pageX, pageY: pageY, buttons: buttons, modifiers: getModifierKeys(), inside: inside });
            if (dragging === 0 /* Stopped */)
                return;
            linear_algebra_1.Vec2.div(pointerDelta, linear_algebra_1.Vec2.sub(pointerDelta, pointerEnd, pointerStart), getClientSize(rectSize));
            var isStart = dragging === 1 /* Started */;
            var dx = pointerDelta[0], dy = pointerDelta[1];
            drag.next({ x: x, y: y, dx: dx, dy: dy, pageX: pageX, pageY: pageY, buttons: buttons, modifiers: getModifierKeys(), isStart: isStart });
            linear_algebra_1.Vec2.copy(pointerStart, pointerEnd);
            dragging = 2 /* Moving */;
        }
        function onMouseWheel(ev) {
            if (noScroll) {
                ev.preventDefault();
            }
            var scale = 1;
            switch (ev.deltaMode) {
                case 0:
                    scale = 1;
                    break; // pixels
                case 1:
                    scale = 40;
                    break; // lines
                case 2:
                    scale = 800;
                    break; // pages
            }
            var dx = (ev.deltaX || 0) * scale;
            var dy = (ev.deltaY || 0) * scale;
            var dz = (ev.deltaZ || 0) * scale;
            if (dx || dy || dz) {
                wheel.next({ dx: dx, dy: dy, dz: dz, buttons: buttons, modifiers: getModifierKeys() });
            }
        }
        function onMouseEnter(ev) {
            isInside = true;
            enter.next();
        }
        function onMouseLeave(ev) {
            isInside = false;
            leave.next();
        }
        function onResize(ev) {
            resize.next();
        }
        function insideBounds(pos) {
            if (element instanceof Window || element instanceof Document || element === document.body) {
                return true;
            }
            else {
                var rect = element.getBoundingClientRect();
                return pos[0] >= 0 && pos[1] >= 0 && pos[0] < rect.width && pos[1] < rect.height;
            }
        }
        function getClientSize(out) {
            out[0] = element.clientWidth;
            out[1] = element.clientHeight;
            return out;
        }
        function eventOffset(out, ev) {
            var cx = ev.clientX || 0;
            var cy = ev.clientY || 0;
            var rect = element.getBoundingClientRect();
            out[0] = cx - rect.left;
            out[1] = cy - rect.top;
            return out;
        }
    }
    InputObserver.fromElement = fromElement;
})(InputObserver || (InputObserver = {}));
exports.default = InputObserver;
