"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * Adapted from LiteMol.
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var generic_1 = require("../mol-data/generic");
var LRUCache;
(function (LRUCache) {
    function entry(key, data) {
        return { key: key, data: data };
    }
    function create(capacity) {
        return {
            entries: generic_1.LinkedList(),
            capacity: Math.max(1, capacity)
        };
    }
    LRUCache.create = create;
    function get(cache, key) {
        for (var e = cache.entries.first; e; e = e.next) {
            if (e.value.key === key) {
                cache.entries.remove(e);
                cache.entries.addLast(e.value);
                return e.value.data;
            }
        }
        return void 0;
    }
    LRUCache.get = get;
    function set(cache, key, data) {
        if (cache.entries.count >= cache.capacity) {
            cache.entries.remove(cache.entries.first);
        }
        cache.entries.addLast(entry(key, data));
        return data;
    }
    LRUCache.set = set;
})(LRUCache || (LRUCache = {}));
exports.LRUCache = LRUCache;
