/**
 * Copyright (c) 2017 Mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
export declare function sortAsc<T extends ArrayLike<number>>(array: T): T;
interface Mask {
    '@type': 'mask';
    size: number;
    has(i: number): boolean;
    /** in-order iteration of all "masked elements". */
    forEach<Ctx>(f: (i: number, ctx?: Ctx) => void, ctx?: Ctx): Ctx | undefined;
}
declare namespace Mask {
    class EmptyMask implements Mask {
        '@type': 'mask';
        size: number;
        has(i: number): boolean;
        forEach<Ctx>(f: (i: number, ctx?: Ctx) => void, ctx: Ctx): Ctx;
        constructor();
    }
    class SingletonMask implements Mask {
        private idx;
        '@type': 'mask';
        size: number;
        has(i: number): boolean;
        forEach<Ctx>(f: (i: number, ctx?: Ctx) => void, ctx?: Ctx): Ctx | undefined;
        constructor(idx: number);
    }
    class AllMask implements Mask {
        size: number;
        '@type': 'mask';
        has(i: number): boolean;
        private _forEach;
        forEach<Ctx>(f: (i: number, ctx?: Ctx) => void, ctx?: Ctx): Ctx | undefined;
        constructor(size: number);
    }
    function always(size: number): AllMask;
    const never: EmptyMask;
    function ofSet(set: Set<number>): Mask;
    function singleton(i: number): SingletonMask;
    function ofUniqueIndices(indices: ArrayLike<number>): Mask;
    function ofMask(mask: boolean[], size: number): Mask;
    function hasAny(mask: Mask, xs: number[]): boolean;
    function complement(mask: Mask, against: Mask): Mask;
}
export default Mask;
