"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("./index");
var linear_algebra_1 = require("../mol-math/linear-algebra");
var object_1 = require("./object");
var ParamDefinition;
(function (ParamDefinition) {
    function setInfo(param, info) {
        if (!info)
            return param;
        if (info.label)
            param.label = info.label;
        if (info.description)
            param.description = info.description;
        if (info.fieldLabels)
            param.fieldLabels = info.fieldLabels;
        if (info.isHidden)
            param.isHidden = info.isHidden;
        return param;
    }
    function Optional(p) {
        var ret = __assign({}, p);
        ret.isOptional = true;
        return ret;
    }
    ParamDefinition.Optional = Optional;
    function Value(defaultValue, info) {
        return setInfo({ type: 'value', defaultValue: defaultValue }, info);
    }
    ParamDefinition.Value = Value;
    function Select(defaultValue, options, info) {
        return setInfo({ type: 'select', defaultValue: defaultValue, options: options }, info);
    }
    ParamDefinition.Select = Select;
    function ColorScale(defaultValue, options, info) {
        return setInfo({ type: 'color-scale', defaultValue: defaultValue, options: options }, info);
    }
    ParamDefinition.ColorScale = ColorScale;
    function MultiSelect(defaultValue, options, info) {
        return setInfo({ type: 'multi-select', defaultValue: defaultValue, options: options }, info);
    }
    ParamDefinition.MultiSelect = MultiSelect;
    function Boolean(defaultValue, info) {
        return setInfo({ type: 'boolean', defaultValue: defaultValue }, info);
    }
    ParamDefinition.Boolean = Boolean;
    function Text(defaultValue, info) {
        if (defaultValue === void 0) { defaultValue = ''; }
        return setInfo({ type: 'text', defaultValue: defaultValue }, info);
    }
    ParamDefinition.Text = Text;
    function Color(defaultValue, info) {
        return setInfo({ type: 'color', defaultValue: defaultValue }, info);
    }
    ParamDefinition.Color = Color;
    function Vec3(defaultValue, info) {
        return setInfo({ type: 'vec3', defaultValue: defaultValue }, info);
    }
    ParamDefinition.Vec3 = Vec3;
    function File(info) {
        var ret = setInfo({ type: 'file', defaultValue: void 0 }, info);
        if (info && info.accept)
            ret.accept = info.accept;
        return ret;
    }
    ParamDefinition.File = File;
    function setRange(p, range) {
        if (!range)
            return p;
        if (typeof range.min !== 'undefined')
            p.min = range.min;
        if (typeof range.max !== 'undefined')
            p.max = range.max;
        if (typeof range.step !== 'undefined')
            p.step = range.step;
        return p;
    }
    function Numeric(defaultValue, range, info) {
        return setInfo(setRange({ type: 'number', defaultValue: defaultValue }, range), info);
    }
    ParamDefinition.Numeric = Numeric;
    function Interval(defaultValue, range, info) {
        return setInfo(setRange({ type: 'interval', defaultValue: defaultValue }, range), info);
    }
    ParamDefinition.Interval = Interval;
    function LineGraph(defaultValue, info) {
        return setInfo({ type: 'line-graph', defaultValue: defaultValue }, info);
    }
    ParamDefinition.LineGraph = LineGraph;
    function Group(params, info) {
        var ret = setInfo({ type: 'group', defaultValue: getDefaultValues(params), params: params }, info);
        if (info && info.isExpanded)
            ret.isExpanded = info.isExpanded;
        if (info && info.isFlat)
            ret.isFlat = info.isFlat;
        return ret;
    }
    ParamDefinition.Group = Group;
    function Mapped(defaultKey, names, map, info) {
        return setInfo({
            type: 'mapped',
            defaultValue: { name: defaultKey, params: map(defaultKey).defaultValue },
            select: Select(defaultKey, names, info),
            map: map
        }, info);
    }
    ParamDefinition.Mapped = Mapped;
    function MappedStatic(defaultKey, map, info) {
        var options = info && info.options
            ? info.options
            : Object.keys(map).map(function (k) { return [k, k]; });
        return setInfo({
            type: 'mapped',
            defaultValue: { name: defaultKey, params: map[defaultKey].defaultValue },
            select: Select(defaultKey, options, info),
            map: function (key) { return map[key]; }
        }, info);
    }
    ParamDefinition.MappedStatic = MappedStatic;
    function ObjectList(element, getLabel, info) {
        return setInfo({ type: 'object-list', element: element, getLabel: getLabel, ctor: _defaultObjectListCtor, defaultValue: (info && info.defaultValue) || [] });
    }
    ParamDefinition.ObjectList = ObjectList;
    function _defaultObjectListCtor() { return getDefaultValues(this.element); }
    function Converted(fromValue, toValue, converted) {
        return { type: 'converted', defaultValue: toValue(converted.defaultValue), converted: converted, fromValue: fromValue, toValue: toValue };
    }
    ParamDefinition.Converted = Converted;
    function Conditioned(defaultValue, conditionParams, conditionForValue, conditionedValue) {
        var options = Object.keys(conditionParams).map(function (k) { return [k, k]; });
        return { type: 'conditioned', select: Select(conditionForValue(defaultValue), options), defaultValue: defaultValue, conditionParams: conditionParams, conditionForValue: conditionForValue, conditionedValue: conditionedValue };
    }
    ParamDefinition.Conditioned = Conditioned;
    function ScriptExpression(defaultValue, info) {
        return setInfo({ type: 'script-expression', defaultValue: defaultValue }, info);
    }
    ParamDefinition.ScriptExpression = ScriptExpression;
    function getDefaultValues(params) {
        var d = {};
        for (var _a = 0, _b = Object.keys(params); _a < _b.length; _a++) {
            var k = _b[_a];
            if (params[k].isOptional)
                continue;
            d[k] = params[k].defaultValue;
        }
        return d;
    }
    ParamDefinition.getDefaultValues = getDefaultValues;
    function clone(params) {
        return object_1.deepClone(params);
    }
    ParamDefinition.clone = clone;
    function validate(params, values) {
        // TODO
        return void 0;
    }
    ParamDefinition.validate = validate;
    function areEqual(params, a, b) {
        if (a === b)
            return true;
        if (!a)
            return !b;
        if (!b)
            return !a;
        if (typeof a !== 'object' || typeof b !== 'object')
            return false;
        for (var _a = 0, _b = Object.keys(params); _a < _b.length; _a++) {
            var k = _b[_a];
            if (!isParamEqual(params[k], a[k], b[k]))
                return false;
        }
        return true;
    }
    ParamDefinition.areEqual = areEqual;
    function isParamEqual(p, a, b) {
        if (a === b)
            return true;
        if (!a)
            return !b;
        if (!b)
            return !a;
        if (p.type === 'group') {
            return areEqual(p.params, a, b);
        }
        else if (p.type === 'mapped') {
            var u = a, v = b;
            if (u.name !== v.name)
                return false;
            var map = p.map(u.name);
            return isParamEqual(map, u.params, v.params);
        }
        else if (p.type === 'multi-select') {
            var u = a, v = b;
            if (u.length !== v.length)
                return false;
            if (u.length < 10) {
                for (var i = 0, _i = u.length; i < _i; i++) {
                    if (u[i] === v[i])
                        continue;
                    if (v.indexOf(u[i]) < 0)
                        return false;
                }
            }
            else {
                // TODO: should the value of multiselect be a set?
                var vSet = new Set(v);
                for (var i = 0, _i = u.length; i < _i; i++) {
                    if (u[i] === v[i])
                        continue;
                    if (!vSet.has(u[i]))
                        return false;
                }
            }
            return true;
        }
        else if (p.type === 'interval') {
            return a[0] === b[0] && a[1] === b[1];
        }
        else if (p.type === 'line-graph') {
            var u = a, v = b;
            if (u.length !== v.length)
                return false;
            for (var i = 0, _i = u.length; i < _i; i++) {
                if (!linear_algebra_1.Vec2.areEqual(u[i], v[i]))
                    return false;
            }
            return true;
        }
        else if (p.type === 'vec3') {
            return linear_algebra_1.Vec3.equals(a, b);
        }
        else if (p.type === 'script-expression') {
            var u = a, v = b;
            return u.language === v.language && u.expression === v.expression;
        }
        else if (p.type === 'object-list') {
            var u = a, v = b;
            var l = u.length;
            if (l !== v.length)
                return false;
            for (var i = 0; i < l; i++) {
                if (!areEqual(p.element, u[i], v[i]))
                    return false;
            }
            return true;
        }
        else if (typeof a === 'object' && typeof b === 'object') {
            return index_1.shallowEqual(a, b);
        }
        // a === b was checked at the top.
        return false;
    }
    ParamDefinition.isParamEqual = isParamEqual;
})(ParamDefinition = exports.ParamDefinition || (exports.ParamDefinition = {}));
