"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
// TODO remove Array.from workaround when targeting ES6
var SetUtils;
(function (SetUtils) {
    /** Test if set a contains all elements of set b. */
    function isSuperset(setA, setB) {
        for (var _i = 0, _a = Array.from(setB); _i < _a.length; _i++) {
            var elm = _a[_i];
            if (!setA.has(elm))
                return false;
        }
        return true;
    }
    SetUtils.isSuperset = isSuperset;
    /** Create set containing elements of both set a and set b. */
    function union(setA, setB) {
        var union = new Set(setA);
        for (var _i = 0, _a = Array.from(setB); _i < _a.length; _i++) {
            var elem = _a[_i];
            union.add(elem);
        }
        return union;
    }
    SetUtils.union = union;
    function unionMany() {
        var sets = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            sets[_i] = arguments[_i];
        }
        if (sets.length === 0)
            return new Set();
        if (sets.length === 1)
            return sets[0];
        var union = new Set(sets[0]);
        for (var i = 1; i < sets.length; i++) {
            for (var _a = 0, _b = Array.from(sets[i]); _a < _b.length; _a++) {
                var elem = _b[_a];
                union.add(elem);
            }
        }
        return union;
    }
    SetUtils.unionMany = unionMany;
    function unionManyArrays(arrays) {
        if (arrays.length === 0)
            return new Set();
        var union = new Set(arrays[0]);
        for (var i = 1; i < arrays.length; i++) {
            for (var _i = 0, _a = arrays[i]; _i < _a.length; _i++) {
                var elem = _a[_i];
                union.add(elem);
            }
        }
        return union;
    }
    SetUtils.unionManyArrays = unionManyArrays;
    /** Create set containing elements of set a that are also in set b. */
    function intersection(setA, setB) {
        var intersection = new Set();
        for (var _i = 0, _a = Array.from(setB); _i < _a.length; _i++) {
            var elem = _a[_i];
            if (setA.has(elem))
                intersection.add(elem);
        }
        return intersection;
    }
    SetUtils.intersection = intersection;
    /** Create set containing elements of set a that are not in set b. */
    function difference(setA, setB) {
        var difference = new Set(setA);
        for (var _i = 0, _a = Array.from(setB); _i < _a.length; _i++) {
            var elem = _a[_i];
            difference.delete(elem);
        }
        return difference;
    }
    SetUtils.difference = difference;
    /** Test if set a and b contain the same elements. */
    function areEqual(setA, setB) {
        if (setA.size !== setB.size)
            return false;
        for (var _i = 0, _a = Array.from(setB); _i < _a.length; _i++) {
            var elm = _a[_i];
            if (!setA.has(elm))
                return false;
        }
        return true;
    }
    SetUtils.areEqual = areEqual;
})(SetUtils = exports.SetUtils || (exports.SetUtils = {}));
