/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { ModelPropertyProviderConfig } from './property-provider';
declare const config: {
    /**
     * Determine if and how long to cache entries after a request.
     */
    cacheParams: {
        useCache: boolean;
        maxApproximateSizeInBytes: number;
        entryTimeoutInMs: number;
    };
    /**
     * Node (V8) sometimes exhibits GC related issues  that significantly slow down the execution
     * (https://github.com/nodejs/node/issues/8670).
     *
     * Therefore an option is provided that automatically shuts down the server.
     * For this to work, the server must be run using a deamon (i.e. forever.js on Linux
     * or IISnode on Windows) so that the server is automatically restarted when the shutdown happens.
     */
    shutdownParams: {
        timeoutMinutes: number;
        timeoutVarianceMinutes: number;
    };
    defaultPort: number;
    /**
     * Specify the prefix of the API, i.e.
     * <host>/<apiPrefix>/<API queries>
     */
    appPrefix: string;
    /**
     * The maximum time the server dedicates to executing a query.
     * Does not include the time it takes to read and export the data.
     */
    maxQueryTimeInMs: number;
    /** Maximum number of requests before "server busy" */
    maxQueueLength: number;
    /**
     * Provide a property config or a path a JSON file with the config.
     */
    customProperties: string | ModelPropertyProviderConfig;
    /**
     * Maps a request identifier to a filename.
     *
     * @param source
     *   Source of the data.
     * @param id
     *   Id provided in the request.
     */
    mapFile(source: string, id: string): string | undefined;
};
export default config;
