"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var cluster = require("cluster");
var now_1 = require("../../../mol-util/now");
var performance_monitor_1 = require("../../../mol-util/performance-monitor");
var preprocess_1 = require("./preprocess");
var property_provider_1 = require("../property-provider");
function runMaster(config, entries) {
    var started = now_1.now();
    var progress = 0;
    var onMessage = function (msg) {
        if (msg.type === 'tick') {
            progress++;
            var elapsed = now_1.now() - started;
            console.log("[" + progress + "/" + entries.length + "] in " + performance_monitor_1.PerformanceMonitor.format(elapsed) + " (avg " + performance_monitor_1.PerformanceMonitor.format(elapsed / progress) + ").");
        }
        else if (msg.type === 'error') {
            console.error(msg.id + ": " + msg.error);
        }
    };
    if (entries.length === 1) {
        runSingle(entries[0], config, onMessage);
    }
    else {
        var parts = partitionArray(entries, config.numProcesses || 1);
        for (var _i = 0, parts_1 = parts; _i < parts_1.length; _i++) {
            var _ = parts_1[_i];
            var worker = cluster.fork();
            worker.on('message', onMessage);
        }
        var i = 0;
        for (var id in cluster.workers) {
            cluster.workers[id].send({ entries: parts[i++], config: config });
        }
    }
}
exports.runMaster = runMaster;
function runChild() {
    var _this = this;
    process.on('message', function (_a) {
        var entries = _a.entries, config = _a.config;
        return __awaiter(_this, void 0, void 0, function () {
            var props, _i, entries_1, entry, e_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        props = property_provider_1.createModelPropertiesProvider(config.customProperties);
                        _i = 0, entries_1 = entries;
                        _b.label = 1;
                    case 1:
                        if (!(_i < entries_1.length)) return [3 /*break*/, 7];
                        entry = entries_1[_i];
                        _b.label = 2;
                    case 2:
                        _b.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, preprocess_1.preprocessFile(entry.source, props, entry.cif, entry.bcif)];
                    case 3:
                        _b.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        e_1 = _b.sent();
                        process.send({ type: 'error', id: path.parse(entry.source).name, error: '' + e_1 });
                        return [3 /*break*/, 5];
                    case 5:
                        process.send({ type: 'tick' });
                        _b.label = 6;
                    case 6:
                        _i++;
                        return [3 /*break*/, 1];
                    case 7:
                        process.exit();
                        return [2 /*return*/];
                }
            });
        });
    });
}
exports.runChild = runChild;
function runSingle(entry, config, onMessage) {
    return __awaiter(this, void 0, void 0, function () {
        var props, e_2;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    props = property_provider_1.createModelPropertiesProvider(config.customProperties);
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, preprocess_1.preprocessFile(entry.source, props, entry.cif, entry.bcif)];
                case 2:
                    _a.sent();
                    return [3 /*break*/, 4];
                case 3:
                    e_2 = _a.sent();
                    onMessage({ type: 'error', id: path.parse(entry.source).name, error: '' + e_2 });
                    return [3 /*break*/, 4];
                case 4:
                    onMessage({ type: 'tick' });
                    return [2 /*return*/];
            }
        });
    });
}
function partitionArray(xs, count) {
    var ret = [];
    var s = Math.ceil(xs.length / count);
    for (var i = 0; i < xs.length; i += s) {
        var bucket = [];
        for (var j = i, _j = Math.min(xs.length, i + s); j < _j; j++) {
            bucket.push(xs[j]);
        }
        ret.push(bucket);
    }
    return ret;
}
