"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var structure_quality_report_1 = require("../../../../mol-model-props/pdbe/structure-quality-report");
var fetch_retry_1 = require("../../utils/fetch-retry");
var mol_util_1 = require("../../../../mol-util");
var preferred_assembly_1 = require("../../../../mol-model-props/pdbe/preferred-assembly");
var struct_ref_domain_1 = require("../../../../mol-model-props/pdbe/struct-ref-domain");
var console_logger_1 = require("../../../../mol-util/console-logger");
var util_1 = require("../../../common/util");
exports.PDBe_structureQualityReport = function (_a) {
    var model = _a.model, params = _a.params, cache = _a.cache;
    var PDBe_apiSourceJson = useFileSource(params)
        ? residuewise_outlier_summary.getDataFromAggregateFile(getFilePrefix(params, 'residuewise_outlier_summary'))
        : apiQueryProvider(getApiUrl(params, 'residuewise_outlier_summary', 'https://www.ebi.ac.uk/pdbe/api/validation/residuewise_outlier_summary/entry'), cache);
    return structure_quality_report_1.StructureQualityReport.attachFromCifOrApi(model, { PDBe_apiSourceJson: PDBe_apiSourceJson });
};
exports.PDBe_preferredAssembly = function (_a) {
    var model = _a.model, params = _a.params, cache = _a.cache;
    var PDBe_apiSourceJson = apiQueryProvider(getApiUrl(params, 'preferred_assembly', 'https://www.ebi.ac.uk/pdbe/api/pdb/entry/summary'), cache);
    return preferred_assembly_1.PDBePreferredAssembly.attachFromCifOrApi(model, { PDBe_apiSourceJson: PDBe_apiSourceJson });
};
exports.PDBe_structRefDomain = function (_a) {
    var model = _a.model, params = _a.params, cache = _a.cache;
    var PDBe_apiSourceJson = apiQueryProvider(getApiUrl(params, 'struct_ref_domain', 'https://www.ebi.ac.uk/pdbe/api/mappings/sequence_domains'), cache);
    return struct_ref_domain_1.PDBeStructRefDomain.attachFromCifOrApi(model, { PDBe_apiSourceJson: PDBe_apiSourceJson });
};
var residuewise_outlier_summary;
(function (residuewise_outlier_summary) {
    var json = new Map();
    function getDataFromAggregateFile(pathPrefix) {
        var _this = this;
        // This is for "testing" purposes and should probably only read
        // a single file with the appropriate prop in the "production" version.
        return function (model) { return __awaiter(_this, void 0, void 0, function () {
            var key, fn;
            return __generator(this, function (_a) {
                key = "" + model.label[1] + model.label[2];
                if (!json.has(key)) {
                    fn = path.join(pathPrefix, key + ".json");
                    if (!fs.existsSync(fn))
                        json.set(key, {});
                    // TODO: use async readFile?
                    else
                        json.set(key, JSON.parse(fs.readFileSync(fn, 'utf8')));
                }
                return [2 /*return*/, json.get(key)[model.label.toLowerCase()] || {}];
            });
        }); };
    }
    residuewise_outlier_summary.getDataFromAggregateFile = getDataFromAggregateFile;
})(residuewise_outlier_summary || (residuewise_outlier_summary = {}));
function getApiUrl(params, name, fallback) {
    var url = util_1.getParam(params, 'PDBe', 'API', name);
    if (!url)
        return fallback;
    if (url[url.length - 1] === '/')
        return url.substring(0, url.length - 1);
    return url;
}
function getFilePrefix(params, name) {
    var ret = util_1.getParam(params, 'PDBe', 'File', name);
    if (!ret)
        throw new Error("PDBe file prefix '" + name + "' not set!");
    return ret;
}
function useFileSource(params) {
    return !!util_1.getParam(params, 'PDBe', 'UseFileSource');
}
function apiQueryProvider(urlPrefix, cache) {
    var _this = this;
    var cacheKey = mol_util_1.UUID.create22();
    return function (model) { return __awaiter(_this, void 0, void 0, function () {
        var rawData, json, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 3, , 4]);
                    if (cache[cacheKey])
                        return [2 /*return*/, cache[cacheKey]];
                    return [4 /*yield*/, fetch_retry_1.fetchRetry(urlPrefix + "/" + model.label.toLowerCase(), 1500, 5)];
                case 1:
                    rawData = _a.sent();
                    // TODO: is this ok?
                    if (rawData.status !== 200)
                        return [2 /*return*/, {}];
                    return [4 /*yield*/, rawData.json()];
                case 2:
                    json = (_a.sent())[model.label.toLowerCase()] || {};
                    cache[cacheKey] = json;
                    return [2 /*return*/, json];
                case 3:
                    e_1 = _a.sent();
                    // TODO: handle better
                    console_logger_1.ConsoleLogger.warn('Props', "Count not retrieve prop @" + (urlPrefix + "/" + model.label.toLowerCase()));
                    return [2 /*return*/, {}];
                case 4: return [2 /*return*/];
            }
        });
    }); };
}
