"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var express = require("express");
var compression = require("compression");
var config_1 = require("./config");
var console_logger_1 = require("../../mol-util/console-logger");
var performance_monitor_1 = require("../../mol-util/performance-monitor");
var api_web_1 = require("./server/api-web");
var version_1 = require("./version");
function setupShutdown() {
    if (config_1.default.shutdownParams.timeoutVarianceMinutes > config_1.default.shutdownParams.timeoutMinutes) {
        console_logger_1.ConsoleLogger.log('Server', 'Shutdown timeout variance is greater than the timer itself, ignoring.');
    }
    else {
        var tVar = 0;
        if (config_1.default.shutdownParams.timeoutVarianceMinutes > 0) {
            tVar = 2 * (Math.random() - 0.5) * config_1.default.shutdownParams.timeoutVarianceMinutes;
        }
        var tMs = (config_1.default.shutdownParams.timeoutMinutes + tVar) * 60 * 1000;
        console.log("----------------------------------------------------------------------------");
        console.log("  The server will shut down in " + performance_monitor_1.PerformanceMonitor.format(tMs) + " to prevent slow performance.");
        console.log("  Please make sure a daemon is running that will automatically restart it.");
        console.log("----------------------------------------------------------------------------");
        console.log();
        setTimeout(function () {
            /*if (WebApi.ApiState.pendingQueries > 0) {
                WebApi.ApiState.shutdownOnZeroPending = true;
            } else*/ {
                console_logger_1.ConsoleLogger.log('Server', "Shut down due to timeout.");
                process.exit(0);
            }
        }, tMs);
    }
}
var port = process.env.port || config_1.default.defaultPort;
function startServer() {
    var app = express();
    app.use(compression({ level: 6, memLevel: 9, chunkSize: 16 * 16384, filter: function () { return true; } }));
    // app.get(ServerConfig.appPrefix + '/documentation', (req, res) => {
    //     res.writeHead(200, { 'Content-Type': 'text/html' });
    //     res.write(Documentation.getHTMLDocs(ServerConfig.appPrefix));
    //     res.end();
    // });
    api_web_1.initWebApi(app);
    // app.get('*', (req, res) => {
    //     res.writeHead(200, { 'Content-Type': 'text/html' });
    //     res.write(Documentation.getHTMLDocs(ServerConfig.appPrefix));
    //     res.end();
    // });
    app.listen(port);
}
startServer();
console.log("Mol* ModelServer " + version_1.default);
console.log("");
console.log("The server is running on port " + port + ".");
console.log("");
if (config_1.default.shutdownParams && config_1.default.shutdownParams.timeoutMinutes > 0) {
    setupShutdown();
}
