"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var jobs_1 = require("./jobs");
var console_logger_1 = require("../../../mol-util/console-logger");
var query_1 = require("./query");
var structure_wrapper_1 = require("./structure-wrapper");
var now_1 = require("../../../mol-util/now");
var performance_monitor_1 = require("../../../mol-util/performance-monitor");
function runLocal(input) {
    return __awaiter(this, void 0, void 0, function () {
        var _i, input_1, job_1, binary, started, job, key, progress, encoder, writer, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!input.length) {
                        console_logger_1.ConsoleLogger.error('Local', 'No input');
                        return [2 /*return*/];
                    }
                    for (_i = 0, input_1 = input; _i < input_1.length; _i++) {
                        job_1 = input_1[_i];
                        binary = /\.bcif/.test(job_1.output);
                        jobs_1.JobManager.add({
                            entryId: job_1.input,
                            queryName: job_1.query,
                            queryParams: job_1.params || {},
                            options: {
                                modelNums: job_1.modelNums,
                                outputFilename: job_1.output,
                                binary: binary
                            }
                        });
                    }
                    jobs_1.JobManager.sort();
                    started = now_1.now();
                    job = jobs_1.JobManager.getNext();
                    key = job.key;
                    progress = 0;
                    _a.label = 1;
                case 1:
                    if (!job) return [3 /*break*/, 6];
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4 /*yield*/, query_1.resolveJob(job)];
                case 3:
                    encoder = _a.sent();
                    writer = wrapFileToWriter(job.outputFilename);
                    encoder.writeTo(writer);
                    writer.end();
                    console_logger_1.ConsoleLogger.logId(job.id, 'Query', 'Written.');
                    if (jobs_1.JobManager.hasNext()) {
                        job = jobs_1.JobManager.getNext();
                        if (key !== job.key)
                            structure_wrapper_1.StructureCache.expire(key);
                        key = job.key;
                    }
                    else {
                        return [3 /*break*/, 6];
                    }
                    return [3 /*break*/, 5];
                case 4:
                    e_1 = _a.sent();
                    console_logger_1.ConsoleLogger.errorId(job.id, e_1);
                    return [3 /*break*/, 5];
                case 5:
                    console_logger_1.ConsoleLogger.log('Progress', "[" + ++progress + "/" + input.length + "] after " + performance_monitor_1.PerformanceMonitor.format(now_1.now() - started) + ".");
                    return [3 /*break*/, 1];
                case 6:
                    console_logger_1.ConsoleLogger.log('Progress', "Done in " + performance_monitor_1.PerformanceMonitor.format(now_1.now() - started) + ".");
                    structure_wrapper_1.StructureCache.expireAll();
                    return [2 /*return*/];
            }
        });
    });
}
exports.runLocal = runLocal;
function wrapFileToWriter(fn) {
    var w = {
        open: function () {
            if (this.opened)
                return;
            makeDir(path.dirname(fn));
            this.file = fs.openSync(fn, 'w');
            this.opened = true;
        },
        writeBinary: function (data) {
            this.open();
            fs.writeSync(this.file, Buffer.from(data.buffer));
            return true;
        },
        writeString: function (data) {
            this.open();
            fs.writeSync(this.file, data);
            return true;
        },
        end: function () {
            if (!this.opened || this.ended)
                return;
            fs.close(this.file, function () { });
            this.ended = true;
        },
        file: 0,
        ended: false,
        opened: false
    };
    return w;
}
exports.wrapFileToWriter = wrapFileToWriter;
function makeDir(path, root) {
    var dirs = path.split(/\/|\\/g), dir = dirs.shift();
    root = (root || '') + dir + '/';
    try {
        fs.mkdirSync(root);
    }
    catch (e) {
        if (!fs.statSync(root).isDirectory())
            throw new Error(e);
    }
    return !dirs.length || makeDir(dirs.join('/'), root);
}
