"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var config_1 = require("../config");
var console_logger_1 = require("../../../mol-util/console-logger");
var query_1 = require("./query");
var jobs_1 = require("./jobs");
var landing_1 = require("./landing");
function makePath(p) {
    return config_1.default.appPrefix + '/' + p;
}
function wrapResponse(fn, res) {
    var w = {
        doError: function (code, message) {
            if (code === void 0) { code = 404; }
            if (message === void 0) { message = 'Not Found.'; }
            if (!this.headerWritten) {
                res.status(code).send(message);
                this.headerWritten = true;
            }
            this.end();
        },
        writeHeader: function (binary) {
            if (this.headerWritten)
                return;
            res.writeHead(200, {
                'Content-Type': binary ? 'application/octet-stream' : 'text/plain; charset=utf-8',
                'Access-Control-Allow-Origin': '*',
                'Access-Control-Allow-Headers': 'X-Requested-With',
                'Content-Disposition': "inline; filename=\"" + fn + "\""
            });
            this.headerWritten = true;
        },
        writeBinary: function (data) {
            if (!this.headerWritten)
                this.writeHeader(true);
            return res.write(Buffer.from(data.buffer));
        },
        writeString: function (data) {
            if (!this.headerWritten)
                this.writeHeader(false);
            return res.write(data);
        },
        end: function () {
            if (this.ended)
                return;
            res.end();
            this.ended = true;
        },
        ended: false,
        headerWritten: false
    };
    return w;
}
var responseMap = new Map();
function processNextJob() {
    return __awaiter(this, void 0, void 0, function () {
        var job, response, filenameBase, writer, encoder, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!jobs_1.JobManager.hasNext())
                        return [2 /*return*/];
                    job = jobs_1.JobManager.getNext();
                    response = responseMap.get(job.id);
                    responseMap.delete(job.id);
                    filenameBase = job.entryId + "_" + job.queryDefinition.name.replace(/\s/g, '_');
                    writer = wrapResponse(job.responseFormat.isBinary ? filenameBase + ".bcif" : filenameBase + ".cif", response);
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, 4, 5]);
                    return [4 /*yield*/, query_1.resolveJob(job)];
                case 2:
                    encoder = _a.sent();
                    writer.writeHeader(job.responseFormat.isBinary);
                    encoder.writeTo(writer);
                    return [3 /*break*/, 5];
                case 3:
                    e_1 = _a.sent();
                    console_logger_1.ConsoleLogger.errorId(job.id, '' + e_1);
                    writer.doError(404, '' + e_1);
                    return [3 /*break*/, 5];
                case 4:
                    writer.end();
                    console_logger_1.ConsoleLogger.logId(job.id, 'Query', 'Finished.');
                    setImmediate(processNextJob);
                    return [7 /*endfinally*/];
                case 5: return [2 /*return*/];
            }
        });
    });
}
// function mapQuery(app: express.Express, queryName: string, queryDefinition: QueryDefinition) {
//     app.get(makePath(':entryId/' + queryName), (req, res) => {
//         ConsoleLogger.log('Server', `Query '${req.params.entryId}/${queryName}'...`);
//         if (JobManager.size >= Config.maxQueueLength) {
//             res.status(503).send('Too many queries, please try again later.');
//             res.end();
//             return;
//         }
//         const jobId = JobManager.add('pdb', req.params.entryId, queryName, req.query);
//         responseMap.set(jobId, res);
//         if (JobManager.size === 1) processNextJob();
//     });
// }
function initWebApi(app) {
    var _this = this;
    app.get(makePath('static/:format/:id'), function (req, res) { return __awaiter(_this, void 0, void 0, function () {
        var binary, id, fn;
        return __generator(this, function (_a) {
            binary = req.params.format === 'bcif';
            id = req.params.id;
            fn = config_1.default.mapFile(binary ? 'pdb-bcif' : 'pdb-cif', id);
            if (!fn || !fs.existsSync(fn)) {
                res.status(404);
                res.end();
                return [2 /*return*/];
            }
            fs.readFile(fn, function (err, data) {
                if (err) {
                    res.status(404);
                    res.end();
                    return;
                }
                var f = path.parse(fn);
                res.writeHead(200, {
                    'Content-Type': binary ? 'application/octet-stream' : 'text/plain; charset=utf-8',
                    'Access-Control-Allow-Origin': '*',
                    'Access-Control-Allow-Headers': 'X-Requested-With',
                    'Content-Disposition': "inline; filename=\"" + f.name + f.ext + "\""
                });
                res.write(data);
                res.end();
            });
            return [2 /*return*/];
        });
    }); });
    app.get(makePath('api/v1'), function (req, res) {
        var query = /\?(.*)$/.exec(req.url)[1];
        var args = JSON.parse(decodeURIComponent(query));
        var name = args.name;
        var entryId = args.id;
        var queryParams = args.params || {};
        var jobId = jobs_1.JobManager.add({
            sourceId: 'pdb',
            entryId: entryId,
            queryName: name,
            queryParams: queryParams,
            options: { modelNums: args.modelNums, binary: args.binary }
        });
        responseMap.set(jobId, res);
        if (jobs_1.JobManager.size === 1)
            processNextJob();
    });
    app.get('*', function (req, res) {
        res.send(landing_1.LandingPage);
    });
    // for (const q of QueryList) {
    //     mapQuery(app, q.name, q.definition);
    // }
}
exports.initWebApi = initWebApi;
