/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
import { Structure, StructureQuery } from '../../../mol-model/structure';
export declare enum QueryParamType {
    JSON = 0,
    String = 1,
    Integer = 2,
    Float = 3
}
export interface QueryParamInfo {
    name: string;
    type: QueryParamType;
    description?: string;
    required?: boolean;
    defaultValue?: any;
    exampleValues?: any[];
    validation?: (v: any) => void;
}
export interface QueryDefinition<Params = any> {
    name: string;
    niceName: string;
    exampleId: string;
    query: (params: any, structure: Structure) => StructureQuery;
    description: string;
    params: QueryParamInfo[];
    structureTransform?: (params: any, s: Structure) => Promise<Structure>;
    '@params': Params;
}
export interface AtomSiteSchemaElement {
    label_entity_id?: string;
    label_asym_id?: string;
    auth_asym_id?: string;
    label_comp_id?: string;
    auth_comp_id?: string;
    label_seq_id?: string;
    auth_seq_id?: string;
    pdbx_PDB_ins_code?: string;
    label_atom_id?: string;
    auth_atom_id?: string;
    type_symbol?: string;
}
export declare type AtomSiteSchema = AtomSiteSchemaElement | AtomSiteSchemaElement[];
declare const QueryMap: {
    'full': Partial<QueryDefinition<{} | undefined>>;
    'atoms': Partial<QueryDefinition<{
        atom_site: AtomSiteSchema;
    }>>;
    'symmetryMates': Partial<QueryDefinition<{
        radius: number;
    }>>;
    'assembly': Partial<QueryDefinition<{
        name: string;
    }>>;
    'residueInteraction': Partial<QueryDefinition<{
        atom_site: AtomSiteSchema;
        radius: number;
    }>>;
    'residueSurroundings': Partial<QueryDefinition<{
        atom_site: AtomSiteSchema;
        radius: number;
    }>>;
};
export declare type QueryName = keyof typeof QueryMap;
export declare type QueryParams<Q extends QueryName> = Partial<(typeof QueryMap)[Q]['@params']>;
export declare function getQueryByName(name: QueryName): QueryDefinition;
export declare const QueryList: {
    name: string;
    definition: QueryDefinition<any>;
}[];
export declare function normalizeQueryParams(query: QueryDefinition, params: any): any;
export {};
