"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var mol_util_1 = require("../../../mol-util");
var api_1 = require("./api");
var generic_1 = require("../../../mol-data/generic");
function createJob(definition) {
    var queryDefinition = api_1.getQueryByName(definition.queryName);
    if (!queryDefinition)
        throw new Error("Query '" + definition.queryName + "' is not supported.");
    var normalizedParams = api_1.normalizeQueryParams(queryDefinition, definition.queryParams);
    var sourceId = definition.sourceId || '_local_';
    return {
        id: mol_util_1.UUID.create22(),
        datetime_utc: "" + new Date().toISOString().replace(/T/, ' ').replace(/\..+/, ''),
        key: sourceId + "/" + definition.entryId,
        sourceId: sourceId,
        entryId: definition.entryId,
        queryDefinition: queryDefinition,
        normalizedParams: normalizedParams,
        responseFormat: { isBinary: !!(definition.options && definition.options.binary) },
        modelNums: definition.options && definition.options.modelNums,
        outputFilename: definition.options && definition.options.outputFilename
    };
}
exports.createJob = createJob;
var _JobQueue = /** @class */ (function () {
    function _JobQueue() {
        this.list = generic_1.LinkedList();
    }
    Object.defineProperty(_JobQueue.prototype, "size", {
        get: function () {
            return this.list.count;
        },
        enumerable: true,
        configurable: true
    });
    _JobQueue.prototype.add = function (definition) {
        var job = createJob(definition);
        this.list.addLast(job);
        return job.id;
    };
    _JobQueue.prototype.hasNext = function () {
        return this.list.count > 0;
    };
    _JobQueue.prototype.getNext = function () {
        return this.list.removeFirst();
    };
    /** Sort the job list by key = sourceId/entryId */
    _JobQueue.prototype.sort = function () {
        if (this.list.count === 0)
            return;
        var jobs = [];
        for (var j = this.list.first; !!j; j = j.next) {
            jobs[jobs.length] = j.value;
        }
        jobs.sort(function (a, b) { return a.key < b.key ? -1 : 1; });
        this.list = generic_1.LinkedList();
        for (var _i = 0, jobs_1 = jobs; _i < jobs_1.length; _i++) {
            var j = jobs_1[_i];
            this.list.addLast(j);
        }
    };
    return _JobQueue;
}());
exports.JobManager = new _JobQueue();
