"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var cif_1 = require("../../../mol-io/writer/cif");
var structure_1 = require("../../../mol-model/structure");
var mmcif_1 = require("../../../mol-model/structure/export/mmcif");
var now_1 = require("../../../mol-util/now");
var console_logger_1 = require("../../../mol-util/console-logger");
var performance_monitor_1 = require("../../../mol-util/performance-monitor");
var config_1 = require("../config");
var version_1 = require("../version");
var structure_wrapper_1 = require("./structure-wrapper");
var CifField = cif_1.CifWriter.Field;
var property_provider_1 = require("../property-provider");
var perf = new performance_monitor_1.PerformanceMonitor();
var _propertyProvider;
function propertyProvider() {
    if (_propertyProvider)
        return _propertyProvider;
    _propertyProvider = property_provider_1.createModelPropertiesProviderFromConfig() || (function () { return []; });
    return _propertyProvider;
}
function resolveJob(job) {
    return __awaiter(this, void 0, void 0, function () {
        var wrappedStructure, sourceStructures, structures, _i, sourceStructures_1, s, _a, _b, queries, result, i, _c, _d, encoder, stats, e_1;
        return __generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    console_logger_1.ConsoleLogger.logId(job.id, 'Query', 'Starting.');
                    return [4 /*yield*/, structure_wrapper_1.createStructureWrapperFromJob(job, propertyProvider())];
                case 1:
                    wrappedStructure = _e.sent();
                    _e.label = 2;
                case 2:
                    _e.trys.push([2, 12, , 13]);
                    perf.start('query');
                    return [4 /*yield*/, structure_wrapper_1.resolveStructures(wrappedStructure, job.modelNums)];
                case 3:
                    sourceStructures = _e.sent();
                    if (!sourceStructures.length)
                        throw new Error('Model not available');
                    structures = sourceStructures;
                    if (!job.queryDefinition.structureTransform) return [3 /*break*/, 7];
                    structures = [];
                    _i = 0, sourceStructures_1 = sourceStructures;
                    _e.label = 4;
                case 4:
                    if (!(_i < sourceStructures_1.length)) return [3 /*break*/, 7];
                    s = sourceStructures_1[_i];
                    _b = (_a = structures).push;
                    return [4 /*yield*/, job.queryDefinition.structureTransform(job.normalizedParams, s)];
                case 5:
                    _b.apply(_a, [_e.sent()]);
                    _e.label = 6;
                case 6:
                    _i++;
                    return [3 /*break*/, 4];
                case 7:
                    queries = structures.map(function (s) { return job.queryDefinition.query(job.normalizedParams, s); });
                    result = [];
                    i = 0;
                    _e.label = 8;
                case 8:
                    if (!(i < structures.length)) return [3 /*break*/, 11];
                    _d = (_c = result).push;
                    return [4 /*yield*/, structure_1.StructureSelection.unionStructure(structure_1.StructureQuery.run(queries[i], structures[i], config_1.default.maxQueryTimeInMs))];
                case 9:
                    _d.apply(_c, [_e.sent()]);
                    _e.label = 10;
                case 10:
                    i++;
                    return [3 /*break*/, 8];
                case 11:
                    perf.end('query');
                    encoder = cif_1.CifWriter.createEncoder({
                        binary: job.responseFormat.isBinary,
                        encoderName: "ModelServer " + version_1.default,
                        binaryEncodingPovider: getEncodingProvider(wrappedStructure),
                        binaryAutoClassifyEncoding: true
                    });
                    console_logger_1.ConsoleLogger.logId(job.id, 'Query', 'Query finished.');
                    perf.start('encode');
                    encoder.startDataBlock(sourceStructures[0].models[0].label.toUpperCase());
                    encoder.writeCategory(_model_server_result, job);
                    encoder.writeCategory(_model_server_params, job);
                    // encoder.setFilter(mmCIF_Export_Filters.onlyPositions);
                    mmcif_1.encode_mmCIF_categories(encoder, result);
                    // encoder.setFilter();
                    perf.end('encode');
                    stats = {
                        structure: wrappedStructure,
                        queryTimeMs: perf.time('query'),
                        encodeTimeMs: perf.time('encode')
                    };
                    encoder.writeCategory(_model_server_stats, stats);
                    encoder.encode();
                    console_logger_1.ConsoleLogger.logId(job.id, 'Query', 'Encoded.');
                    return [2 /*return*/, encoder];
                case 12:
                    e_1 = _e.sent();
                    console_logger_1.ConsoleLogger.errorId(job.id, e_1);
                    return [2 /*return*/, doError(job, e_1)];
                case 13: return [2 /*return*/];
            }
        });
    });
}
exports.resolveJob = resolveJob;
function getEncodingProvider(structure) {
    if (!structure.isBinary)
        return void 0;
    return cif_1.CifWriter.createEncodingProviderFromCifFrame(structure.cifFrame);
}
function doError(job, e) {
    var encoder = cif_1.CifWriter.createEncoder({ binary: job.responseFormat.isBinary, encoderName: "ModelServer " + version_1.default });
    encoder.writeCategory(_model_server_result, job);
    encoder.writeCategory(_model_server_params, job);
    encoder.writeCategory(_model_server_error, '' + e);
    encoder.encode();
    return encoder;
}
var maxTime = config_1.default.maxQueryTimeInMs;
function abortingObserver(p) {
    if (now_1.now() - p.root.progress.startedTime > maxTime) {
        p.requestAbort("Exceeded maximum allowed time for a query (" + maxTime + "ms)");
    }
}
exports.abortingObserver = abortingObserver;
function string(name, str, isSpecified) {
    if (isSpecified) {
        return CifField.str(name, function (i, d) { return str(d, i); }, { valueKind: function (i, d) { return isSpecified(d) ? 0 /* Present */ : 1 /* NotPresent */; } });
    }
    return CifField.str(name, function (i, d) { return str(d, i); });
}
function int32(name, value) {
    return CifField.int(name, function (i, d) { return value(d); });
}
var _model_server_result_fields = [
    string('job_id', function (ctx) { return '' + ctx.id; }),
    string('datetime_utc', function (ctx) { return ctx.datetime_utc; }),
    string('server_version', function (ctx) { return version_1.default; }),
    string('query_name', function (ctx) { return ctx.queryDefinition.name; }),
    string('source_id', function (ctx) { return ctx.sourceId; }),
    string('entry_id', function (ctx) { return ctx.entryId; }),
];
var _model_server_params_fields = [
    string('name', function (ctx, i) { return ctx[i][0]; }),
    string('value', function (ctx, i) { return ctx[i][1]; })
];
var _model_server_error_fields = [
    string('message', function (ctx, i) { return ctx; })
];
var _model_server_stats_fields = [
    int32('io_time_ms', function (ctx) { return ctx.structure.info.readTime | 0; }),
    int32('parse_time_ms', function (ctx) { return ctx.structure.info.parseTime | 0; }),
    // int32<Stats>('attach_props_time_ms', ctx => ctx.structure.info.attachPropsTime | 0),
    int32('create_model_time_ms', function (ctx) { return ctx.structure.info.createModelTime | 0; }),
    int32('query_time_ms', function (ctx) { return ctx.queryTimeMs | 0; }),
    int32('encode_time_ms', function (ctx) { return ctx.encodeTimeMs | 0; })
];
var _model_server_result = {
    name: 'model_server_result',
    instance: function (job) { return cif_1.CifWriter.categoryInstance(_model_server_result_fields, { data: job, rowCount: 1 }); }
};
var _model_server_error = {
    name: 'model_server_error',
    instance: function (message) { return cif_1.CifWriter.categoryInstance(_model_server_error_fields, { data: message, rowCount: 1 }); }
};
var _model_server_params = {
    name: 'model_server_params',
    instance: function (job) {
        var params = [];
        for (var _i = 0, _a = Object.keys(job.normalizedParams); _i < _a.length; _i++) {
            var k = _a[_i];
            params.push([k, JSON.stringify(job.normalizedParams[k])]);
        }
        return cif_1.CifWriter.categoryInstance(_model_server_params_fields, { data: params, rowCount: params.length });
    }
};
var _model_server_stats = {
    name: 'model_server_stats',
    instance: function (stats) { return cif_1.CifWriter.categoryInstance(_model_server_stats_fields, { data: stats, rowCount: 1 }); }
};
