"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var structure_1 = require("../../../mol-model/structure");
var performance_monitor_1 = require("../../../mol-util/performance-monitor");
var cache_1 = require("./cache");
var config_1 = require("../config");
var cif_1 = require("../../../mol-io/reader/cif");
var util = require("util");
var fs = require("fs");
var zlib = require("zlib");
var console_logger_1 = require("../../../mol-util/console-logger");
var mmcif_1 = require("../../../mol-model-formats/structure/mmcif");
require('util.promisify').shim();
var StructureSourceType;
(function (StructureSourceType) {
    StructureSourceType[StructureSourceType["File"] = 0] = "File";
    StructureSourceType[StructureSourceType["Cache"] = 1] = "Cache";
})(StructureSourceType = exports.StructureSourceType || (exports.StructureSourceType = {}));
function createStructureWrapperFromJob(job, propertyProvider, allowCache) {
    if (allowCache === void 0) { allowCache = true; }
    return __awaiter(this, void 0, void 0, function () {
        var ret_1, ret;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (allowCache && config_1.default.cacheParams.useCache) {
                        ret_1 = exports.StructureCache.get(job.key);
                        if (ret_1)
                            return [2 /*return*/, ret_1];
                    }
                    return [4 /*yield*/, readStructureWrapper(job.key, job.sourceId, job.entryId, propertyProvider)];
                case 1:
                    ret = _a.sent();
                    if (allowCache && config_1.default.cacheParams.useCache) {
                        exports.StructureCache.add(ret);
                    }
                    return [2 /*return*/, ret];
            }
        });
    });
}
exports.createStructureWrapperFromJob = createStructureWrapperFromJob;
exports.StructureCache = new cache_1.Cache(function (s) { return s.key; }, function (s) { return s.approximateSize; });
var perf = new performance_monitor_1.PerformanceMonitor();
var readFileAsync = util.promisify(fs.readFile);
var unzipAsync = util.promisify(zlib.unzip);
function readFile(filename) {
    return __awaiter(this, void 0, void 0, function () {
        var isGz, input, data, i, data, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    isGz = /\.gz$/i.test(filename);
                    if (!filename.match(/\.bcif/)) return [3 /*break*/, 4];
                    return [4 /*yield*/, readFileAsync(filename)];
                case 1:
                    input = _b.sent();
                    if (!isGz) return [3 /*break*/, 3];
                    return [4 /*yield*/, unzipAsync(input)];
                case 2:
                    input = _b.sent();
                    _b.label = 3;
                case 3:
                    data = new Uint8Array(input.byteLength);
                    for (i = 0; i < input.byteLength; i++)
                        data[i] = input[i];
                    return [2 /*return*/, data];
                case 4:
                    if (!isGz) return [3 /*break*/, 7];
                    _a = unzipAsync;
                    return [4 /*yield*/, readFileAsync(filename)];
                case 5: return [4 /*yield*/, _a.apply(void 0, [_b.sent()])];
                case 6:
                    data = _b.sent();
                    return [2 /*return*/, data.toString('utf8')];
                case 7: return [2 /*return*/, readFileAsync(filename, 'utf8')];
            }
        });
    });
}
function parseCif(data) {
    return __awaiter(this, void 0, void 0, function () {
        var comp, parsed;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    comp = cif_1.CIF.parse(data);
                    return [4 /*yield*/, comp.run()];
                case 1:
                    parsed = _a.sent();
                    if (parsed.isError)
                        throw parsed;
                    return [2 /*return*/, parsed.result];
            }
        });
    });
}
function readDataAndFrame(filename, key) {
    return __awaiter(this, void 0, void 0, function () {
        var data, e_1, frame;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    perf.start('read');
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, readFile(filename)];
                case 2:
                    data = _a.sent();
                    return [3 /*break*/, 4];
                case 3:
                    e_1 = _a.sent();
                    console_logger_1.ConsoleLogger.error(key || filename, '' + e_1);
                    throw new Error("Could not read the file for '" + (key || filename) + "' from disk.");
                case 4:
                    perf.end('read');
                    perf.start('parse');
                    return [4 /*yield*/, parseCif(data)];
                case 5:
                    frame = (_a.sent()).blocks[0];
                    perf.end('parse');
                    return [2 /*return*/, { data: data, frame: frame }];
            }
        });
    });
}
exports.readDataAndFrame = readDataAndFrame;
function readStructureWrapper(key, sourceId, entryId, propertyProvider) {
    return __awaiter(this, void 0, void 0, function () {
        var filename, _a, data, frame, models, modelMap, _i, models_1, m, ret;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    filename = sourceId === '_local_' ? entryId : config_1.default.mapFile(sourceId, entryId);
                    if (!filename)
                        throw new Error("Cound not map '" + key + "' to a valid filename.");
                    if (!fs.existsSync(filename))
                        throw new Error("Could not find source file for '" + key + "'.");
                    return [4 /*yield*/, readDataAndFrame(filename, key)];
                case 1:
                    _a = _b.sent(), data = _a.data, frame = _a.frame;
                    perf.start('createModel');
                    return [4 /*yield*/, mmcif_1.trajectoryFromMmCIF(frame).run()];
                case 2:
                    models = _b.sent();
                    perf.end('createModel');
                    modelMap = new Map();
                    for (_i = 0, models_1 = models; _i < models_1.length; _i++) {
                        m = models_1[_i];
                        modelMap.set(m.modelNum, m);
                    }
                    ret = {
                        info: {
                            sourceType: StructureSourceType.File,
                            readTime: perf.time('read'),
                            parseTime: perf.time('parse'),
                            createModelTime: perf.time('createModel'),
                            attachPropsTime: 0,
                            sourceId: sourceId,
                            entryId: entryId
                        },
                        isBinary: /\.bcif/.test(filename),
                        key: key,
                        approximateSize: typeof data === 'string' ? 2 * data.length : data.length,
                        models: models,
                        modelMap: modelMap,
                        structureModelMap: new Map(),
                        cifFrame: frame,
                        propertyProvider: propertyProvider,
                        cache: Object.create(null)
                    };
                    return [2 /*return*/, ret];
            }
        });
    });
}
exports.readStructureWrapper = readStructureWrapper;
function resolveStructure(wrapper, modelNum) {
    return __awaiter(this, void 0, void 0, function () {
        var model, structure, modelProps, _i, modelProps_1, p;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (typeof modelNum === 'undefined')
                        modelNum = wrapper.models[0].modelNum;
                    if (wrapper.structureModelMap.has(modelNum))
                        return [2 /*return*/, wrapper.structureModelMap.get(modelNum)];
                    if (!wrapper.modelMap.has(modelNum)) {
                        return [2 /*return*/, void 0];
                    }
                    model = wrapper.modelMap.get(modelNum);
                    structure = structure_1.Structure.ofModel(model);
                    if (!wrapper.propertyProvider) return [3 /*break*/, 4];
                    modelProps = wrapper.propertyProvider(model, wrapper.cache);
                    _i = 0, modelProps_1 = modelProps;
                    _a.label = 1;
                case 1:
                    if (!(_i < modelProps_1.length)) return [3 /*break*/, 4];
                    p = modelProps_1[_i];
                    return [4 /*yield*/, tryAttach(wrapper.key, p)];
                case 2:
                    _a.sent();
                    _a.label = 3;
                case 3:
                    _i++;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/, structure];
            }
        });
    });
}
exports.resolveStructure = resolveStructure;
function resolveStructures(wrapper, modelNums) {
    return __awaiter(this, void 0, void 0, function () {
        var ret, _i, _a, n, s;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    ret = [];
                    _i = 0, _a = modelNums || wrapper.models.map(function (m) { return m.modelNum; });
                    _b.label = 1;
                case 1:
                    if (!(_i < _a.length)) return [3 /*break*/, 4];
                    n = _a[_i];
                    return [4 /*yield*/, resolveStructure(wrapper, n)];
                case 2:
                    s = _b.sent();
                    if (s)
                        ret.push(s);
                    _b.label = 3;
                case 3:
                    _i++;
                    return [3 /*break*/, 1];
                case 4: return [2 /*return*/, ret];
            }
        });
    });
}
exports.resolveStructures = resolveStructures;
function tryAttach(key, promise) {
    return __awaiter(this, void 0, void 0, function () {
        var e_2;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 2, , 3]);
                    return [4 /*yield*/, promise];
                case 1:
                    _a.sent();
                    return [3 /*break*/, 3];
                case 2:
                    e_2 = _a.sent();
                    console_logger_1.ConsoleLogger.errorId(key, 'Custom prop:' + e_2);
                    return [3 /*break*/, 3];
                case 3: return [2 /*return*/];
            }
        });
    });
}
