"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var node_fetch_1 = require("node-fetch");
var fs = require("fs");
var path = require("path");
var argparse = require("argparse");
var make_dir_1 = require("../../../mol-util/make-dir");
var now_1 = require("../../../mol-util/now");
var performance_monitor_1 = require("../../../mol-util/performance-monitor");
var cmdParser = new argparse.ArgumentParser({
    addHelp: true,
    description: 'Download JSON data from PDBe API'
});
cmdParser.addArgument(['--in'], { help: 'Input folder', required: true });
cmdParser.addArgument(['--out'], { help: 'Output folder', required: true });
var cmdArgs = cmdParser.parseArgs();
function getPDBid(name) {
    var idx = name.indexOf('_');
    if (idx < 0)
        idx = name.indexOf('.');
    return name.substr(0, idx).toLowerCase();
}
function findEntries() {
    var files = fs.readdirSync(cmdArgs.in);
    var cifTest = /\.cif$/;
    var groups = new Map();
    var keys = [];
    for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
        var f = files_1[_i];
        if (!cifTest.test(f))
            continue;
        var id = getPDBid(f);
        var groupId = "" + id[1] + id[2];
        if (groups.has(groupId))
            groups.get(groupId).push(id);
        else {
            keys.push(groupId);
            groups.set(groupId, [id]);
        }
    }
    var ret = [];
    for (var _a = 0, keys_1 = keys; _a < keys_1.length; _a++) {
        var key = keys_1[_a];
        ret.push({ key: key, entries: groups.get(key) });
    }
    return ret;
}
function process() {
    return __awaiter(this, void 0, void 0, function () {
        var entries, started, prog, _i, entries_1, e, ts, data, _a, _b, ee, query, _c, _d, e_1, time;
        return __generator(this, function (_e) {
            switch (_e.label) {
                case 0:
                    entries = findEntries();
                    make_dir_1.makeDir(cmdArgs.out);
                    started = now_1.now();
                    prog = 0;
                    _i = 0, entries_1 = entries;
                    _e.label = 1;
                case 1:
                    if (!(_i < entries_1.length)) return [3 /*break*/, 12];
                    e = entries_1[_i];
                    ts = now_1.now();
                    console.log(prog + "/" + entries.length + " " + e.entries.length + " entries.");
                    data = Object.create(null);
                    _a = 0, _b = e.entries;
                    _e.label = 2;
                case 2:
                    if (!(_a < _b.length)) return [3 /*break*/, 10];
                    ee = _b[_a];
                    return [4 /*yield*/, node_fetch_1.default("https://www.ebi.ac.uk/pdbe/api/validation/residuewise_outlier_summary/entry/" + ee)];
                case 3:
                    query = _e.sent();
                    _e.label = 4;
                case 4:
                    _e.trys.push([4, 8, , 9]);
                    if (!(query.status === 200)) return [3 /*break*/, 6];
                    _c = data;
                    _d = ee;
                    return [4 /*yield*/, query.json()];
                case 5:
                    _c[_d] = (_e.sent())[ee] || {};
                    return [3 /*break*/, 7];
                case 6:
                    console.error(ee, query.status);
                    _e.label = 7;
                case 7: return [3 /*break*/, 9];
                case 8:
                    e_1 = _e.sent();
                    console.error(ee, '' + e_1);
                    return [3 /*break*/, 9];
                case 9:
                    _a++;
                    return [3 /*break*/, 2];
                case 10:
                    // const query = await fetch(`https://www.ebi.ac.uk/pdbe/api/validation/residuewise_outlier_summary/entry`, { method: 'POST', body });
                    // console.log(query.status);
                    // const data = await query.text();
                    fs.writeFileSync(path.join(cmdArgs.out, e.key + '.json'), JSON.stringify(data));
                    time = now_1.now() - started;
                    console.log(++prog + "/" + entries.length + " in " + performance_monitor_1.PerformanceMonitor.format(time) + " (last " + performance_monitor_1.PerformanceMonitor.format(now_1.now() - ts) + ", avg " + performance_monitor_1.PerformanceMonitor.format(time / prog) + ")");
                    _e.label = 11;
                case 11:
                    _i++;
                    return [3 /*break*/, 1];
                case 12: return [2 /*return*/];
            }
        });
    });
}
process();
