"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
function addLimitsArgs(parser) {
    parser.addArgument(['--maxRequestBlockCount'], {
        defaultValue: DefaultLimitsConfig.maxRequestBlockCount,
        metavar: 'COUNT',
        help: "Maximum number of blocks that could be read in 1 query.\nThis is somewhat tied to the maxOutputSizeInVoxelCountByPrecisionLevel\nin that the <maximum number of voxel> = maxRequestBlockCount * <block size>^3.\nThe default block size is 96 which corresponds to 28,311,552 voxels with 32 max blocks."
    });
    parser.addArgument(['--maxFractionalBoxVolume'], {
        defaultValue: DefaultLimitsConfig.maxFractionalBoxVolume,
        metavar: 'VOLUME',
        help: "The maximum fractional volume of the query box (to prevent queries that are too big)."
    });
    parser.addArgument(['--maxOutputSizeInVoxelCountByPrecisionLevel'], {
        nargs: '+',
        defaultValue: DefaultLimitsConfig.maxOutputSizeInVoxelCountByPrecisionLevel,
        metavar: 'LEVEL',
        help: "What is the (approximate) maximum desired size in voxel count by precision level\nRule of thumb: <response gzipped size> in [<voxel count> / 8, <voxel count> / 4].\nThe maximum number of voxels is tied to maxRequestBlockCount."
    });
}
exports.addLimitsArgs = addLimitsArgs;
function addServerArgs(parser) {
    parser.addArgument(['--apiPrefix'], {
        defaultValue: DefaultServerConfig.apiPrefix,
        metavar: 'PREFIX',
        help: "Specify the prefix of the API, i.e. <host>/<apiPrefix>/<API queries>"
    });
    parser.addArgument(['--defaultPort'], {
        defaultValue: DefaultServerConfig.defaultPort,
        metavar: 'PORT',
        help: "Specify the prefix of the API, i.e. <host>/<apiPrefix>/<API queries>"
    });
    parser.addArgument(['--shutdownTimeoutMinutes'], {
        defaultValue: DefaultServerConfig.shutdownTimeoutMinutes,
        metavar: 'TIME',
        help: "0 for off, server will shut down after this amount of minutes."
    });
    parser.addArgument(['--shutdownTimeoutVarianceMinutes'], {
        defaultValue: DefaultServerConfig.shutdownTimeoutVarianceMinutes,
        metavar: 'VARIANCE',
        help: "modifies the shutdown timer by +/- timeoutVarianceMinutes (to avoid multiple instances shutting at the same time)"
    });
    parser.addArgument(['--idMap'], {
        nargs: 2,
        action: 'append',
        metavar: ['TYPE', 'PATH'],
        help: [
            'Map `id`s for a `type` to a file path.',
            'Example: x-ray \'../../data/mdb/xray/${id}-ccp4.mdb\'',
            'Note: Can be specified multiple times.'
        ].join('\n'),
    });
}
exports.addServerArgs = addServerArgs;
var DefaultServerConfig = {
    apiPrefix: '/VolumeServer',
    defaultPort: 1337,
    shutdownTimeoutMinutes: 24 * 60,
    shutdownTimeoutVarianceMinutes: 60,
    idMap: []
};
exports.ServerConfig = __assign({}, DefaultServerConfig);
function setServerConfig(config) {
    Object.assign(exports.ServerConfig, config);
}
exports.setServerConfig = setServerConfig;
var DefaultLimitsConfig = {
    maxRequestBlockCount: 32,
    maxFractionalBoxVolume: 1024,
    maxOutputSizeInVoxelCountByPrecisionLevel: [
        0.5 * 1024 * 1024,
        1 * 1024 * 1024,
        2 * 1024 * 1024,
        4 * 1024 * 1024,
        8 * 1024 * 1024,
        16 * 1024 * 1024,
        24 * 1024 * 1024
    ]
};
exports.LimitsConfig = __assign({}, DefaultLimitsConfig);
function setLimitsConfig(config) {
    Object.assign(exports.LimitsConfig, config);
}
exports.setLimitsConfig = setLimitsConfig;
function setConfig(config) {
    setServerConfig(config);
    setLimitsConfig(config);
}
exports.setConfig = setConfig;
