"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var File = require("../common/file");
var file_handle_1 = require("../../../mol-io/common/file-handle");
var ccp4_1 = require("./format/ccp4");
var typed_array_1 = require("../../../mol-io/common/typed-array");
var dsn6_1 = require("./format/dsn6");
function assignSliceBuffer(data, blockSizeInMB) {
    var _a = data.header, extent = _a.extent, valueType = _a.valueType;
    var maxBlockBytes = blockSizeInMB * 1024 * 1024;
    var sliceSize = extent[0] * extent[1] * typed_array_1.getElementByteSize(valueType);
    var sliceCapacity = Math.max(1, Math.floor(Math.min(maxBlockBytes, sliceSize * extent[2]) / sliceSize));
    var buffer = typed_array_1.createTypedArrayBufferContext(sliceCapacity * extent[0] * extent[1], valueType);
    data.slices = {
        buffer: buffer,
        maxBlockBytes: maxBlockBytes,
        sliceCapacity: sliceCapacity,
        slicesRead: 0,
        values: buffer.values,
        sliceCount: 0,
        isFinished: false
    };
}
exports.assignSliceBuffer = assignSliceBuffer;
function compareProp(a, b) {
    if (a instanceof Array && b instanceof Array) {
        if (a.length !== b.length)
            return false;
        for (var i = 0; i < a.length; i++) {
            if (a[i] !== b[i])
                return false;
        }
        return true;
    }
    return a === b;
}
function compareHeaders(a, b) {
    for (var _i = 0, _a = ['grid', 'axisOrder', 'extent', 'origin', 'spacegroupNumber', 'cellSize', 'cellAngles', 'mode']; _i < _a.length; _i++) {
        var p = _a[_i];
        if (!compareProp(a[p], b[p]))
            return false;
    }
    return true;
}
exports.compareHeaders = compareHeaders;
function getProviderFromType(type) {
    switch (type) {
        case 'ccp4': return ccp4_1.Ccp4Provider;
        case 'dsn6': return dsn6_1.Dsn6Provider;
    }
}
exports.getProviderFromType = getProviderFromType;
function open(name, filename, type) {
    return __awaiter(this, void 0, void 0, function () {
        var provider, descriptor, file, header, data;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    provider = getProviderFromType(type);
                    return [4 /*yield*/, File.openRead(filename)];
                case 1:
                    descriptor = _a.sent();
                    file = file_handle_1.FileHandle.fromDescriptor(descriptor);
                    return [4 /*yield*/, provider.readHeader(name, file)];
                case 2:
                    header = _a.sent();
                    data = { header: header, file: file, slices: void 0 };
                    return [2 /*return*/, { data: data, provider: provider }];
            }
        });
    });
}
exports.open = open;
