"use strict";
/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var parser_1 = require("../../../../mol-io/reader/dsn6/parser");
var typed_array_1 = require("../../../../mol-io/common/typed-array");
function readHeader(name, file) {
    return __awaiter(this, void 0, void 0, function () {
        var _a, dsn6Header, littleEndian, header;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, parser_1.readDsn6Header(file)];
                case 1:
                    _a = _b.sent(), dsn6Header = _a.header, littleEndian = _a.littleEndian;
                    header = {
                        name: name,
                        valueType: typed_array_1.TypedArrayValueType.Float32,
                        grid: [dsn6Header.xRate, dsn6Header.yRate, dsn6Header.zRate].reverse(),
                        axisOrder: [0, 1, 2].reverse(),
                        extent: [dsn6Header.xExtent, dsn6Header.yExtent, dsn6Header.zExtent].reverse(),
                        origin: [dsn6Header.xStart, dsn6Header.yStart, dsn6Header.zStart].reverse(),
                        spacegroupNumber: 1,
                        cellSize: [dsn6Header.xlen, dsn6Header.ylen, dsn6Header.zlen],
                        cellAngles: [dsn6Header.alpha, dsn6Header.beta, dsn6Header.gamma],
                        littleEndian: littleEndian,
                        dataOffset: parser_1.dsn6HeaderSize,
                        originalHeader: dsn6Header
                    };
                    return [2 /*return*/, header];
            }
        });
    });
}
function readSlices(data) {
    return __awaiter(this, void 0, void 0, function () {
        var slices, header, file, extent, dataOffset, originalHeader, sliceCount, byteCount, buffer;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    slices = data.slices, header = data.header, file = data.file;
                    if (slices.isFinished) {
                        return [2 /*return*/];
                    }
                    extent = header.extent, dataOffset = header.dataOffset, originalHeader = header.originalHeader;
                    sliceCount = extent[2];
                    byteCount = parser_1.getDsn6Counts(originalHeader).byteCount;
                    if (byteCount > slices.maxBlockBytes) {
                        throw new Error("dsn6 file to large, can't read " + byteCount + " bytes at once, increase block size or use another file format");
                    }
                    return [4 /*yield*/, file.readBuffer(dataOffset, byteCount)];
                case 1:
                    buffer = (_a.sent()).buffer;
                    if (!(slices.values instanceof Float32Array)) {
                        throw new Error("dsn6 reader only supports Float32Array for output values");
                    }
                    return [4 /*yield*/, parser_1.parseDsn6Values(originalHeader, buffer, slices.values, header.littleEndian)];
                case 2:
                    _a.sent();
                    slices.slicesRead += sliceCount;
                    slices.sliceCount = sliceCount;
                    if (slices.slicesRead >= extent[2]) {
                        slices.isFinished = true;
                    }
                    return [2 /*return*/];
            }
        });
    });
}
exports.readSlices = readSlices;
exports.Dsn6Provider = { readHeader: readHeader, readSlices: readSlices };
