"use strict";
/**
 * Copyright (c) 2018-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * Taken/adapted from DensityServer (https://github.com/dsehnal/DensityServer)
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
var express = require("express");
var compression = require("compression");
var web_api_1 = require("./server/web-api");
var version_1 = require("./server/version");
var console_logger_1 = require("../../mol-util/console-logger");
var state_1 = require("./server/state");
var config_1 = require("./config");
var argparse = require("argparse");
function setupShutdown() {
    if (config_1.ServerConfig.shutdownTimeoutVarianceMinutes > config_1.ServerConfig.shutdownTimeoutMinutes) {
        console_logger_1.ConsoleLogger.log('Server', 'Shutdown timeout variance is greater than the timer itself, ignoring.');
    }
    else {
        var tVar = 0;
        if (config_1.ServerConfig.shutdownTimeoutVarianceMinutes > 0) {
            tVar = 2 * (Math.random() - 0.5) * config_1.ServerConfig.shutdownTimeoutVarianceMinutes;
        }
        var tMs = (config_1.ServerConfig.shutdownTimeoutMinutes + tVar) * 60 * 1000;
        console.log("----------------------------------------------------------------------------");
        console.log("  The server will shut down in " + console_logger_1.ConsoleLogger.formatTime(tMs) + " to prevent slow performance.");
        console.log("  Please make sure a daemon is running that will automatically restart it.");
        console.log("----------------------------------------------------------------------------");
        console.log();
        setTimeout(function () {
            if (state_1.State.pendingQueries > 0) {
                state_1.State.shutdownOnZeroPending = true;
            }
            else {
                console_logger_1.ConsoleLogger.log('Server', "Shut down due to timeout.");
                process.exit(0);
            }
        }, tMs);
    }
}
var parser = new argparse.ArgumentParser({
    addHelp: true,
    description: "VolumeServer " + version_1.default + ", (c) 2018-2019, Mol* contributors"
});
config_1.addServerArgs(parser);
config_1.addLimitsArgs(parser);
var config = parser.parseArgs();
config_1.setConfig(config); // sets the config for global use
var port = process.env.port || config_1.ServerConfig.defaultPort;
var app = express();
app.use(compression({ level: 6, memLevel: 9, chunkSize: 16 * 16384, filter: function () { return true; } }));
web_api_1.default(app);
app.listen(port);
console.log("VolumeServer " + version_1.default + ", (c) 2018-2019, Mol* contributors");
console.log("");
console.log("The server is running on port " + port + ".");
console.log("");
if (config.shutdownTimeoutMinutes > 0) {
    setupShutdown();
}
