"use strict";
/**
 * Copyright (c) 2018 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * Taken/adapted from DensityServer (https://github.com/dsehnal/DensityServer)
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var File = require("../common/file");
var execute_1 = require("./query/execute");
var console_logger_1 = require("../../../mol-util/console-logger");
var DataFormat = require("../common/data-format");
var file_handle_1 = require("../../../mol-io/common/file-handle");
var config_1 = require("../config");
function getOutputFilename(source, id, _a) {
    var asBinary = _a.asBinary, box = _a.box, detail = _a.detail, forcedSamplingLevel = _a.forcedSamplingLevel;
    function n(s) { return (s || '').replace(/[ \n\t]/g, '').toLowerCase(); }
    function r(v) { return Math.round(10 * v) / 10; }
    var det = forcedSamplingLevel !== void 0
        ? "l" + forcedSamplingLevel
        : "d" + Math.min(Math.max(0, detail | 0), config_1.LimitsConfig.maxOutputSizeInVoxelCountByPrecisionLevel.length - 1);
    var boxInfo = box.kind === 'Cell'
        ? 'cell'
        : (box.kind === 'Cartesian' ? 'cartn' : 'frac') + "_" + r(box.a[0]) + "_" + r(box.a[1]) + "_" + r(box.a[2]) + "_" + r(box.b[0]) + "_" + r(box.b[1]) + "_" + r(box.b[2]);
    return n(source) + "_" + n(id) + "-" + boxInfo + "_" + det + "." + (asBinary ? 'bcif' : 'cif');
}
exports.getOutputFilename = getOutputFilename;
/** Reads the header and includes information about available detail levels */
function getExtendedHeaderJson(filename, sourceId) {
    return __awaiter(this, void 0, void 0, function () {
        var header, _a, sampleCount, maxVoxelCount, precisions, availablePrecisions, _i, precisions_1, p, e_1;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    console_logger_1.ConsoleLogger.log('Header', sourceId);
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    if (!filename || !File.exists(filename)) {
                        console_logger_1.ConsoleLogger.error("Header " + sourceId, 'File not found.');
                        return [2 /*return*/, void 0];
                    }
                    _a = [{}];
                    return [4 /*yield*/, readHeader(filename, sourceId)];
                case 2:
                    header = __assign.apply(void 0, _a.concat([_b.sent()]));
                    sampleCount = header.sampling[0].sampleCount;
                    maxVoxelCount = sampleCount[0] * sampleCount[1] * sampleCount[2];
                    precisions = config_1.LimitsConfig.maxOutputSizeInVoxelCountByPrecisionLevel
                        .map(function (maxVoxels, precision) { return ({ precision: precision, maxVoxels: maxVoxels }); });
                    availablePrecisions = [];
                    for (_i = 0, precisions_1 = precisions; _i < precisions_1.length; _i++) {
                        p = precisions_1[_i];
                        availablePrecisions.push(p);
                        if (p.maxVoxels > maxVoxelCount)
                            break;
                    }
                    header.availablePrecisions = availablePrecisions;
                    header.isAvailable = true;
                    return [2 /*return*/, JSON.stringify(header, null, 2)];
                case 3:
                    e_1 = _b.sent();
                    console_logger_1.ConsoleLogger.error("Header " + sourceId, e_1);
                    return [2 /*return*/, void 0];
                case 4: return [2 /*return*/];
            }
        });
    });
}
exports.getExtendedHeaderJson = getExtendedHeaderJson;
function queryBox(params, outputProvider) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, execute_1.default(params, outputProvider)];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
exports.queryBox = queryBox;
function readHeader(filename, sourceId) {
    return __awaiter(this, void 0, void 0, function () {
        var file, _a, _b, header, e_2;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    _c.trys.push([0, 3, 4, 5]);
                    if (!filename)
                        return [2 /*return*/, void 0];
                    _b = (_a = file_handle_1.FileHandle).fromDescriptor;
                    return [4 /*yield*/, File.openRead(filename)];
                case 1:
                    file = _b.apply(_a, [_c.sent()]);
                    return [4 /*yield*/, DataFormat.readHeader(file)];
                case 2:
                    header = _c.sent();
                    return [2 /*return*/, header.header];
                case 3:
                    e_2 = _c.sent();
                    console_logger_1.ConsoleLogger.error("Info " + sourceId, e_2);
                    return [2 /*return*/, void 0];
                case 4:
                    if (file)
                        file.close();
                    return [7 /*endfinally*/];
                case 5: return [2 /*return*/];
            }
        });
    });
}
