/**
 * Copyright (c) 2017-2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author Michael Krone <michael.krone@uni-tuebingen.de>
 */
declare const _default: "\nprecision highp float;\n\nvarying vec3 unitCoord;\nvarying vec3 origPos;\nvarying float instance;\n\nuniform mat4 uInvView;\nuniform float uIsoValue;\nuniform vec3 uGridDim;\nuniform sampler2D tTransferTex;\n\nuniform int uObjectId;\nuniform int uInstanceCount;\nuniform int uGroupCount;\n\nuniform vec3 uHighlightColor;\nuniform vec3 uSelectColor;\nuniform vec2 uMarkerTexDim;\nuniform sampler2D tMarker;\n\nuniform float uAlpha;\nuniform float uPickingAlphaThreshold;\nuniform int uPickable;\n\n#if defined(dGridTexType_2d)\n    precision highp sampler2D;\n    uniform sampler2D tGridTex;\n    uniform vec3 uGridTexDim;\n#elif defined(dGridTexType_3d)\n    precision highp sampler3D;\n    uniform sampler3D tGridTex;\n#endif\n\n#if defined(dColorType_uniform)\n    uniform vec3 uColor;\n#elif defined(dColorType_instance) || defined(dColorType_group) || defined(dColorType_groupInstance)\n    uniform vec2 uColorTexDim;\n    uniform sampler2D tColor;\n#endif\n\n#include common\n#include light_frag_params\n\n#include read_from_texture\n#include texture3d_from_2d_nearest\n#include texture3d_from_2d_linear\n\n#if defined(dGridTexType_2d)\n    vec4 textureVal(vec3 pos) {\n        return texture3dFrom2dLinear(tGridTex, pos, uGridDim, uGridTexDim.xy);\n    }\n    vec4 textureGroup(vec3 pos) {\n        vec3 nearestPos = floor(pos * uGridDim + 0.5) / uGridDim + 0.5 / uGridDim;\n        return texture3dFrom2dNearest(tGridTex, nearestPos, uGridDim, uGridTexDim.xy);\n    }\n#elif defined(dGridTexType_3d)\n    vec4 textureVal(vec3 pos) {\n        return texture(tGridTex, pos);\n    }\n    vec4 textureGroup(vec3 pos) {\n        return texelFetch(tGridTex, ivec3(pos * uGridDim), 0);\n    }\n#endif\n\nvec4 transferFunction(float value) {\n    return texture2D(tTransferTex, vec2(value, 0.0));\n}\n\nconst float gradOffset = 0.5;\n\nvec4 raymarch(vec3 startLoc, vec3 step, vec3 viewDir) {\n    vec3 scaleVol = vec3(1.0) / uGridDim;\n    vec3 pos = startLoc;\n    float prevValue = -1.0;\n    float value = 0.0;\n    vec4 src = vec4(0.0);\n    vec4 dst = vec4(0.0);\n\n    #if defined(dRenderMode_isosurface)\n        vec3 isoPos;\n        float tmp;\n\n        vec3 color = vec3(0.45, 0.55, 0.8);\n        vec3 gradient = vec3(1.0);\n        vec3 dx = vec3(gradOffset * scaleVol.x, 0.0, 0.0);\n        vec3 dy = vec3(0.0, gradOffset * scaleVol.y, 0.0);\n        vec3 dz = vec3(0.0, 0.0, gradOffset * scaleVol.z);\n    #endif\n\n    for(int i = 0; i < dMaxSteps; ++i){\n        value = textureVal(pos).a; // current voxel value\n        if(pos.x > 1.01 || pos.y > 1.01 || pos.z > 1.01 || pos.x < -0.01 || pos.y < -0.01 || pos.z < -0.01)\n            break;\n\n        #if defined(dRenderMode_volume)\n            src = transferFunction(value);\n            src.rgb *= src.a;\n            dst = (1.0 - dst.a) * src + dst; // standard blending\n        #endif\n\n        #if defined(dRenderMode_isosurface)\n            if(prevValue > 0.0 && ( // there was a prev Value\n                (prevValue < uIsoValue && value > uIsoValue) || // entering isosurface\n                (prevValue > uIsoValue && value < uIsoValue) // leaving isosurface\n            )) {\n                tmp = ((prevValue - uIsoValue) / ((prevValue - uIsoValue) - (value - uIsoValue)));\n                isoPos = mix(pos - step, pos, tmp);\n\n                #if defined(dColorType_objectPicking) || defined(dColorType_instancePicking) || defined(dColorType_groupPicking)\n                    if (uAlpha < uPickingAlphaThreshold)\n                        discard; // ignore so the element below can be picked\n                    if (uPickable == 0)\n                        return vec4(0.0, 0.0, 0.0, 1.0); // set to empty picking id\n                #endif\n\n                #if defined(dColorType_objectPicking)\n                    return vec4(encodeFloatRGB(float(uObjectId)), 1.0);\n                #elif defined(dColorType_instancePicking)\n                    return vec4(encodeFloatRGB(instance), 1.0);\n                #elif defined(dColorType_groupPicking)\n                    float group = floor(decodeFloatRGB(textureGroup(isoPos).rgb) + 0.5);\n                    return vec4(encodeFloatRGB(group), 1.0);\n                #elif defined(dColorType_depth)\n                    return packDepthToRGBA(gl_FragCoord.z); // TODO calculate isosurface depth\n                #else\n                    // compute gradient by central differences\n                    gradient.x = textureVal(isoPos - dx).a - textureVal(isoPos + dx).a;\n                    gradient.y = textureVal(isoPos - dy).a - textureVal(isoPos + dy).a;\n                    gradient.z = textureVal(isoPos - dz).a - textureVal(isoPos + dz).a;\n                    gradient = normalize(gradient);\n                    float d = float(dot(gradient, viewDir) > 0.0);\n                    gradient = (2.0 * d - 1.0) * gradient;\n\n                    float group = floor(decodeFloatRGB(textureGroup(isoPos).rgb) + 0.5);\n\n                    #if defined(dColorType_instance)\n                        color = readFromTexture(tColor, instance, uColorTexDim).rgb;\n                    #elif defined(dColorType_group)\n                        color = readFromTexture(tColor, group, uColorTexDim).rgb;\n                    #elif defined(dColorType_groupInstance)\n                        color = readFromTexture(tColor, instance * float(uGroupCount) + group, uColorTexDim).rgb;\n                    #endif\n\n                    vec3 normal = normalize(gradient);\n                    vec3 vViewPosition = normalize(viewDir);\n                    vec4 material = vec4(color, uAlpha);\n                    #include apply_light_color\n\n                    float vMarker = readFromTexture(tMarker, instance * float(uGroupCount) + group, uMarkerTexDim).a;\n                    #include apply_marker_color\n\n                    src.rgb = gl_FragColor.rgb;\n                    src.a = gl_FragColor.a;\n\n                    // draw interior darker\n                    if( (prevValue - uIsoValue) > 0.0 ) {\n                        src.rgb *= 0.5;\n                    }\n\n                    src.rgb *= src.a;\n                    dst = (1.0 - dst.a) * src + dst; // standard blending\n                    if(dst.a >= 1.0) {\n                        break;\n                    }\n                #endif\n            }\n            prevValue = value;\n        #endif\n\n        pos += step;\n    }\n    return dst;\n}\n\nvoid main () {\n    vec3 cameraPos = uInvView[3].xyz / uInvView[3].w;\n\n    vec3 rayDir = normalize(origPos - cameraPos);\n    vec3 startLoc = unitCoord;\n    vec3 step = rayDir * (1.0 / uGridDim) * 0.1;\n\n    gl_FragColor = raymarch(startLoc, step, normalize(cameraPos));\n    if (length(gl_FragColor.rgb) < 0.00001) discard;\n    #if defined(dRenderMode_volume)\n        gl_FragColor.a *= uAlpha;\n    #endif\n}\n";
export default _default;
