declare const _default: "\nprecision highp float;\nprecision highp int;\nprecision highp sampler2D;\n\nuniform sampler2D tColor;\nuniform sampler2D tDepth;\nuniform vec2 uTexSize;\n\nuniform float uNear;\nuniform float uFar;\nuniform float uFogNear;\nuniform float uFogFar;\nuniform vec3 uFogColor;\n\nuniform float uOcclusionBias;\nuniform float uOcclusionRadius;\n\nuniform float uOutlineScale;\nuniform float uOutlineThreshold;\n\nconst float noiseAmount = 0.0002;\nconst vec4 occlusionColor = vec4(0.0, 0.0, 0.0, 1.0);\n\n#include common\n\nfloat noise(const in vec2 coords) {\n\tfloat a = 12.9898;\n\tfloat b = 78.233;\n\tfloat c = 43758.5453;\n\tfloat dt = dot(coords, vec2(a,b));\n\tfloat sn = mod(dt, 3.14159);\n\n\treturn fract(sin(sn) * c);\n}\n\nfloat perspectiveDepthToViewZ(const in float invClipZ, const in float near, const in float far) {\n\treturn (near * far) / ((far - near) * invClipZ - far);\n}\n\nfloat orthographicDepthToViewZ(const in float linearClipZ, const in float near, const in float far) {\n\treturn linearClipZ * (near - far) - near;\n}\n\nfloat getViewZ(const in float depth) {\n\t#if dOrthographic == 1\n\t\treturn orthographicDepthToViewZ(depth, uNear, uFar);\n\t#else\n\t\treturn perspectiveDepthToViewZ(depth, uNear, uFar);\n\t#endif\n}\n\nfloat getDepth(const in vec2 coords) {\n\t#ifdef dPackedDepth\n\t\treturn unpackRGBAToDepth(texture2D(tDepth, coords));\n\t#else\n\t\treturn texture2D(tDepth, coords).r;\n\t#endif\n}\n\nfloat calcSSAO(const in vec2 coords, const in float depth) {\n\tfloat occlusionFactor = 0.0;\n\n\tfor (int i = -dOcclusionKernelSize; i <= dOcclusionKernelSize; i++) {\n\t\tfor (int j = -dOcclusionKernelSize; j <= dOcclusionKernelSize; j++) {\n\t\t\tvec2 coordsDelta = coords + uOcclusionRadius / float(dOcclusionKernelSize) * vec2(float(i) / uTexSize.x, float(j) / uTexSize.y);\n            coordsDelta += noiseAmount * (noise(coordsDelta) - 0.5) / uTexSize;\n            coordsDelta = clamp(coordsDelta, 0.5 / uTexSize, 1.0 - 1.0 / uTexSize);\n\t\t\tif (getDepth(coordsDelta) < depth) occlusionFactor += 1.0;\n\t\t}\n\t}\n\n\treturn occlusionFactor / float((2 * dOcclusionKernelSize + 1) * (2 * dOcclusionKernelSize + 1));\n}\n\nvec2 calcEdgeDepth(const in vec2 coords) {\n    vec2 invTexSize = 1.0 / uTexSize;\n    float halfScaleFloor = floor(uOutlineScale * 0.5);\n    float halfScaleCeil = ceil(uOutlineScale * 0.5);\n\n    vec2 bottomLeftUV = coords - invTexSize * halfScaleFloor;\n    vec2 topRightUV = coords + invTexSize * halfScaleCeil;\n    vec2 bottomRightUV = coords + vec2(invTexSize.x * halfScaleCeil, -invTexSize.y * halfScaleFloor);\n    vec2 topLeftUV = coords + vec2(-invTexSize.x * halfScaleFloor, invTexSize.y * halfScaleCeil);\n\n    float depth0 = getDepth(bottomLeftUV);\n    float depth1 = getDepth(topRightUV);\n    float depth2 = getDepth(bottomRightUV);\n    float depth3 = getDepth(topLeftUV);\n\n    float depthFiniteDifference0 = depth1 - depth0;\n    float depthFiniteDifference1 = depth3 - depth2;\n\n    return vec2(\n\t\tsqrt(pow(depthFiniteDifference0, 2.0) + pow(depthFiniteDifference1, 2.0)) * 100.0,\n\t\tmin(depth0, min(depth1, min(depth2, depth3)))\n\t);\n}\n\nvoid main(void) {\n\tvec2 coords = gl_FragCoord.xy / uTexSize;\n\tvec4 color = texture2D(tColor, coords);\n\n\t#ifdef dOutlineEnable\n\t\tvec2 edgeDepth = calcEdgeDepth(coords);\n\t\tfloat edgeFlag = step(edgeDepth.x, uOutlineThreshold);\n    \tcolor.rgb *= edgeFlag;\n\t\t#ifdef dUseFog\n\t\t\tfloat viewDist = abs(getViewZ(edgeDepth.y));\n\t\t\tfloat fogFactor = smoothstep(uFogNear, uFogFar, viewDist) * (1.0 - edgeFlag);\n\t\t\tcolor.rgb = mix(color.rgb, uFogColor, fogFactor);\n\t\t#endif\n\t#endif\n\n\t// occlusion needs to be handled after outline to darken them properly\n\t#ifdef dOcclusionEnable\n\t\tfloat depth = getDepth(coords);\n\t\tif (depth != 1.0) {\n\t\t\tfloat occlusionFactor = calcSSAO(coords, depth);\n\t\t\tcolor = mix(color, occlusionColor, uOcclusionBias * occlusionFactor);\n\t\t}\n\t#endif\n\n\tgl_FragColor = color;\n}\n";
export default _default;
