/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
declare const _default: "\nprecision highp float;\nprecision highp int;\n\n#include common\n#include common_frag_params\n#include color_frag_params\n#include light_frag_params\n\nuniform mat4 uProjection;\n// uniform vec3 uInteriorColor;\n// uniform float uInteriorDarkening;\nvec3 uInteriorColor = vec3(0.5, 0.5, 0.5);\nfloat uInteriorDarkening = 0.0;\n\nuniform float uClipNear;\nuniform float uIsOrtho;\n\nvarying float vRadius;\nvarying float vRadiusSq;\nvarying vec3 vPoint;\nvarying vec3 vPointViewPosition;\n\nbool flag2 = false;\nbool interior = false;\nvec3 cameraPos;\nvec3 cameraNormal;\n\n// Calculate depth based on the given camera position.\nfloat calcDepth(const in vec3 cameraPos){\n    vec2 clipZW = cameraPos.z * uProjection[2].zw + uProjection[3].zw;\n    return 0.5 + 0.5 * clipZW.x / clipZW.y;\n}\n\nfloat calcClip(const in vec3 cameraPos) {\n    return dot(vec4(cameraPos, 1.0), vec4(0.0, 0.0, 1.0, uClipNear - 0.5));\n}\n\nbool Impostor(out vec3 cameraPos, out vec3 cameraNormal){\n    vec3 cameraSpherePos = -vPointViewPosition;\n    cameraSpherePos.z += vRadius;\n\n    vec3 rayOrigin = mix(vec3(0.0, 0.0, 0.0), vPoint, uIsOrtho);\n    vec3 rayDirection = mix(normalize(vPoint), vec3(0.0, 0.0, 1.0), uIsOrtho);\n    vec3 cameraSphereDir = mix(cameraSpherePos, rayOrigin - cameraSpherePos, uIsOrtho);\n\n    float B = dot(rayDirection, cameraSphereDir);\n    float det = B * B + vRadiusSq - dot(cameraSphereDir, cameraSphereDir);\n\n    if(det < 0.0){\n        discard;\n        return false;\n    }\n\n    float sqrtDet = sqrt(det);\n    float posT = mix(B + sqrtDet, B + sqrtDet, uIsOrtho);\n    float negT = mix(B - sqrtDet, sqrtDet - B, uIsOrtho);\n\n    cameraPos = rayDirection * negT + rayOrigin;\n\n    #ifdef NEAR_CLIP\n        if(calcDepth(cameraPos) <= 0.0){\n            cameraPos = rayDirection * posT + rayOrigin;\n            interior = true;\n        }else if(calcClip(cameraPos) > 0.0){\n            cameraPos = rayDirection * posT + rayOrigin;\n            interior = true;\n            flag2 = true;\n        }\n    #else\n        if(calcDepth(cameraPos) <= 0.0){\n            cameraPos = rayDirection * posT + rayOrigin;\n            interior = true;\n        }\n    #endif\n\n    cameraNormal = normalize(cameraPos - cameraSpherePos);\n    cameraNormal *= float(!interior) * 2.0 - 1.0;\n\n    return !interior;\n}\n\nvoid main(void){\n    bool flag = Impostor(cameraPos, cameraNormal);\n\n    #ifdef NEAR_CLIP\n        if(calcClip(cameraPos) > 0.0)\n            discard;\n    #endif\n\n    // FIXME not compatible with custom clipping plane\n    // Set the depth based on the new cameraPos.\n    gl_FragDepthEXT = calcDepth(cameraPos);\n    if(!flag){\n        // clamp to near clipping plane and add a tiny value to\n        // make spheres with a greater radius occlude smaller ones\n        #ifdef NEAR_CLIP\n            if( flag2 ){\n                gl_FragDepthEXT = max(0.0, calcDepth(vec3(-(uClipNear - 0.5))) + (0.0000001 / vRadius));\n            }else if(gl_FragDepthEXT >= 0.0){\n                gl_FragDepthEXT = 0.0 + (0.0000001 / vRadius);\n            }\n        #else\n            if(gl_FragDepthEXT >= 0.0){\n                gl_FragDepthEXT = 0.0 + (0.0000001 / vRadius);\n            }\n        #endif\n    }\n\n    // bugfix (mac only?)\n    if (gl_FragDepthEXT < 0.0)\n        discard;\n    if (gl_FragDepthEXT > 1.0)\n        discard;\n\n    #include assign_material_color\n\n    #if defined(dColorType_objectPicking) || defined(dColorType_instancePicking) || defined(dColorType_groupPicking)\n        if (uAlpha < uPickingAlphaThreshold)\n            discard; // ignore so the element below can be picked\n        gl_FragColor = material;\n    #elif defined(dColorType_depth)\n        gl_FragColor = material;\n    #else\n        vec3 normal = cameraNormal;\n        vec3 vViewPosition = -cameraPos;\n        #include apply_light_color\n\n        if(interior){\n            #ifdef USE_INTERIOR_COLOR\n                gl_FragColor.rgb = uInteriorColor;\n            #endif\n            gl_FragColor.rgb *= 1.0 - uInteriorDarkening;\n        }\n\n        #include apply_marker_color\n        #include apply_fog\n    #endif\n}\n";
export default _default;
